/**
* Copyright (C) 2018 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.databaseAdmin.preferencePage;

import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;

import eu.etaxonomy.cdm.api.application.ICdmRepository;
import eu.etaxonomy.cdm.api.service.IPreferenceService;
import eu.etaxonomy.cdm.io.specimen.abcd206.in.Abcd206ImportConfigurator;
import eu.etaxonomy.cdm.model.metadata.CdmPreference;
import eu.etaxonomy.cdm.model.metadata.PreferencePredicate;
import eu.etaxonomy.cdm.model.metadata.PreferenceSubject;
import eu.etaxonomy.taxeditor.preference.AbcdImportPreference;
import eu.etaxonomy.taxeditor.preference.PreferencesUtil;
import eu.etaxonomy.taxeditor.store.CdmStore;

/**
 * @author k.luther
 * @since 23.03.2018
 *
 */
public class AbcdImportAdminPreference extends AbcdImportPreference implements IE4AdminPreferencePage, SelectionListener {

    @Override
    public boolean performOk() {
        if (!isApply()){
            return true;
        }
        ICdmRepository controller = CdmStore.getCurrentApplicationConfiguration();
        if (controller == null){
            return false;
        }
        IPreferenceService service = controller.getPreferenceService();
        if (configurator != null){
            String configString = configurator.toString();
            Abcd206ImportConfigurator defaultConfig = Abcd206ImportConfigurator.NewInstance(null,null);
            defaultConfig.setNomenclaturalCode(PreferencesUtil.getPreferredNomenclaturalCode());
            if (configString.equals(defaultConfig.toString())){
                configString = null;
            }
            CdmPreference pref = CdmPreference.NewTaxEditorInstance(PreferencePredicate.AbcdImportConfig, configString);
            pref.setAllowOverride(allowOverride);


            service.set(pref);
            PreferencesUtil.updateDBPreferences();

        }
        if (preference == null){
            service.remove(CdmPreference.NewKey(PreferenceSubject.NewTaxEditorInstance(), PreferencePredicate.AbcdImportConfig));
        }
        return true;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public void widgetDefaultSelected(SelectionEvent e) {
        // TODO Auto-generated method stub

    }

    @Override
    protected void performDefaults() {
        configurator = Abcd206ImportConfigurator.NewInstance(null,null);
        preference = null;
        super.performDefaults();
    }

    @Override
    public void getValues() {
        super.getValues();
        isAdminPreference = true;
        configurator = PreferencesUtil.getDBAbcdImportConfigurationPreference();

    }

}
