/**
* Copyright (C) 2018 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.databaseAdmin.preferencePage;

import org.apache.commons.lang3.StringUtils;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;

import eu.etaxonomy.cdm.api.application.ICdmRepository;
import eu.etaxonomy.cdm.api.service.IPreferenceService;
import eu.etaxonomy.cdm.model.metadata.CdmPreference;
import eu.etaxonomy.cdm.model.metadata.PreferencePredicate;
import eu.etaxonomy.taxeditor.preference.AbcdImportProvider;
import eu.etaxonomy.taxeditor.preference.ListComponent;
import eu.etaxonomy.taxeditor.preference.PreferencesUtil;
import eu.etaxonomy.taxeditor.store.CdmStore;

/**
 * @author k.luther
 * @since 23.03.2018
 */
public class AbcdImportProviderAdminPreferencePage extends AbcdImportProvider implements IE4AdminPreferencePage{



    @Override
    protected Control createContents(Composite parent) {
        isAdminPreference = true;
        final Composite composite = new Composite(parent, SWT.NULL);

        GridLayout gridLayout = new GridLayout();
        composite.setLayout(gridLayout);

        gridLayout.numColumns = 2;
        //gridLayout.horizontalSpacing= 5;
        //gridLayout.makeColumnsEqualWidth = true;

        composite.setLayout(gridLayout);
        biocaseProviderList = new ListComponent(composite, SWT.SCROLL_LINE,true, this);
        allowOverride = createAllowOverrideButton(composite);
        GridData layoutData = (GridData) allowOverride.getLayoutData();
        layoutData.horizontalAlignment = SWT.LEFT;
        //layoutData.horizontalSpan = 2;

        allowOverride.setSelection(biocaseProviderList.getProviderListPreference() != null? biocaseProviderList.getProviderListPreference().isAllowOverride(): true);
        allowOverride.addSelectionListener(new SelectionAdapter() {
            @Override
            public void widgetSelected(SelectionEvent e) {
                setApply(true);
            }
        });

        setApply(true);
        return composite;
    }

    @Override
    public boolean performOk() {
        String providerList = null;
        if (biocaseProviderList != null){
            providerList = biocaseProviderList.createAllProviderString();

            CdmPreference pref = CdmPreference.NewTaxEditorInstance(PreferencePredicate.BioCaseProvider, providerList);
            pref.setAllowOverride(allowOverride.getSelection());

            ICdmRepository controller = CdmStore.getCurrentApplicationConfiguration();
            if (controller == null){
                return false;
            }
            IPreferenceService service = controller.getPreferenceService();
            if (StringUtils.isBlank(providerList)){
                service.remove(pref.getKey());
            }else{
                service.set(pref);
            }
        }
        PreferencesUtil.updateDBPreferences();
        return true;
    }

}
