/**
* Copyright (C) 2020 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.databaseAdmin.preferencePage;

import eu.etaxonomy.cdm.api.application.ICdmRepository;
import eu.etaxonomy.cdm.api.service.IPreferenceService;
import eu.etaxonomy.cdm.ext.geo.CondensedDistributionRecipe;
import eu.etaxonomy.cdm.model.metadata.CdmPreference;
import eu.etaxonomy.cdm.model.metadata.PreferencePredicate;
import eu.etaxonomy.cdm.model.metadata.PreferenceSubject;
import eu.etaxonomy.taxeditor.preference.CdmLightExportPreference;
import eu.etaxonomy.taxeditor.store.CdmStore;

/**
 * @author k.luther
 * @since Nov 12, 2020
 */
public class CdmLightExportAdminPreference extends CdmLightExportPreference {

    @Override
    public void getValues() {
        super.getValues();
        isAdminPreference = true;
        if (preferenceIsExportCondensedDistribution.getValue() != null){
            isExportCondensedDistribution = Boolean.valueOf(preferenceIsExportCondensedDistribution.getValue());
        }else{
            isExportCondensedDistribution = null;
        }
        if (preferenceCondensedDistribtionRecipe.getValue() != null){
            recipe = CondensedDistributionRecipe.valueOf(preferenceCondensedDistribtionRecipe.getValue().toString());
        }else{
            recipe = null;
        }

    }

    @Override
    public boolean performOk() {
        if (!isApply()){
            return true;
        }
        ICdmRepository controller = CdmStore.getCurrentApplicationConfiguration();
        if (controller == null){
            return false;
        }
        IPreferenceService service = controller.getPreferenceService();
        CdmPreference pref;
        if (isExportCondensedDistribution == null && isAllowOverrideIsExportCondensedDistribution){
            service.remove(CdmPreference.NewKey(PreferenceSubject.NewTaxEditorInstance(), PreferencePredicate.CdmLightExportCondensedDistributionString));
        }else{
            pref = CdmPreference.NewTaxEditorInstance( PreferencePredicate.CdmLightExportCondensedDistributionString, isExportCondensedDistribution != null? Boolean.toString(isExportCondensedDistribution): null);
            pref.setAllowOverride(isAllowOverrideIsExportCondensedDistribution);
            service.set(pref);
        }

        if (recipe == null && preferenceCondensedDistribtionRecipe.isAllowOverride()){
            service.remove(CdmPreference.NewKey(PreferenceSubject.NewDatabaseInstance(), PreferencePredicate.CondensedDistributionRecipe));
        }else{
            pref = CdmPreference.NewDatabaseInstance( PreferencePredicate.CondensedDistributionRecipe, recipe != null? recipe.getKey(): null);
            pref.setAllowOverride(isAllowOverrideCondensedDistributionRecipe);
            service.set(pref);
        }
        return true;
    }

}
