/**
* Copyright (C) 2020 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.databaseAdmin.preferencePage;

import eu.etaxonomy.cdm.model.metadata.CdmPreference;
import eu.etaxonomy.cdm.model.metadata.PreferencePredicate;
import eu.etaxonomy.cdm.model.term.TermType;
import eu.etaxonomy.taxeditor.preference.LocalOrDefaultEnum;
import eu.etaxonomy.taxeditor.preference.PreferencesUtil;
import eu.etaxonomy.taxeditor.preference.menu.LanguageMenuPreferences;

/**
 * @author k.luther
 * @since 04.02.2020
 */
public class LanguageAdminMenuPreference extends LanguageMenuPreferences {

    public LanguageAdminMenuPreference() {
        super();
        isAdminPreference = true;
        setPredicate(PreferencePredicate.AvailableLanguages);
        type = TermType.Language;
    }

    @Override
    public boolean performOk() {
        if (!isApply()){
            return true;
        }

        boolean result = performOkAdmin();
        if (isMultilanguageTextEditingCapability == null &&  multiLangugeActivatedPref.isAllowOverride()){
            CdmPreference savePref = CdmPreference.NewTaxEditorInstance(PreferencePredicate.MultiLanguageTextEditing, null);
            PreferencesUtil.removeFromDB(savePref);
        }else{
            CdmPreference savePref = CdmPreference.NewTaxEditorInstance(PreferencePredicate.MultiLanguageTextEditing, isMultilanguageTextEditingCapability != null? Boolean.toString(isMultilanguageTextEditingCapability): null);
            String text = this.useLocalOrAdmin.getText();
            if (text.equals(LocalOrDefaultEnum.Database.getLabel()) ){
                savePref.setAllowOverride(false);
            }else if (text.equals(LocalOrDefaultEnum.AllowOverride.getLabel())){
                savePref.setAllowOverride(true);
            }

            PreferencesUtil.setPreferenceToDB(savePref);
        }
        PreferencesUtil.updateDBPreferences();
        PreferencesUtil.firePreferencesChanged(this.getClass());
//        CdmStore.getTermManager().clearTermMapForTermType(TermType.Language);
        return result;
    }

    @Override
    public void getValues() {
           isAdminPreference = true;
           setPreferenceStore(PreferencesUtil.getPreferenceStore());
           multiLangugeActivatedPref = PreferencesUtil.getPreferenceFromDB(PreferencePredicate.MultiLanguageTextEditing);

           isMultilanguageTextEditingCapability = multiLangugeActivatedPref == null? null: multiLangugeActivatedPref.getValue()!= null?Boolean.parseBoolean(multiLangugeActivatedPref.getValue()): null;
    }
}