/**
* Copyright (C) 2019 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.databaseAdmin.preferencePage;

import eu.etaxonomy.cdm.api.application.ICdmRepository;
import eu.etaxonomy.cdm.api.service.IPreferenceService;
import eu.etaxonomy.cdm.model.metadata.CdmPreference;
import eu.etaxonomy.cdm.model.metadata.CdmPreference.PrefKey;
import eu.etaxonomy.cdm.model.metadata.PreferencePredicate;
import eu.etaxonomy.cdm.model.metadata.PreferenceSubject;
import eu.etaxonomy.cdm.model.name.NomenclaturalCode;
import eu.etaxonomy.taxeditor.preference.NomenclaturalCodePreferences;
import eu.etaxonomy.taxeditor.preference.PreferencesUtil;
import eu.etaxonomy.taxeditor.store.CdmStore;

/**
 * @author k.luther
 * @since 19.08.2019
 */
public class NomenclaturalCodeAdminPreferencePage extends NomenclaturalCodePreferences {


    @Override
    public boolean performOk() {
       if (!isApply()){
           return true;
       }
       CdmPreference pref = null;
       if (actualCode != null){
           pref = CdmPreference.NewDatabaseInstance(PreferencePredicate.NomenclaturalCode, actualCode.toString());
           pref.setAllowOverride(allowOverride);
       }

       PreferencesUtil.setPreferredNomenclaturalCode(pref);

       return true;
   }

    @Override
    protected void getValues(){
        isAdminPreference = true;
        ICdmRepository controller = CdmStore.getCurrentApplicationConfiguration();
        if (controller != null){
            IPreferenceService service = controller.getPreferenceService();
            PrefKey key = CdmPreference.NewKey(PreferenceSubject.NewDatabaseInstance(), PreferencePredicate.NomenclaturalCode);
            pref = service.find(key);
            if (pref != null){
                allowOverride = pref.isAllowOverride();
                actualCode = NomenclaturalCode.fromString(pref.getValue());

            }else{
                actualCode = null;
                allowOverride = true;
            }

        }


    }
}
