/**
* Copyright (C) 2018 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.databaseAdmin.preferencePage;

import eu.etaxonomy.cdm.api.application.ICdmRepository;
import eu.etaxonomy.cdm.api.service.IPreferenceService;
import eu.etaxonomy.cdm.model.metadata.CdmPreference;
import eu.etaxonomy.cdm.model.metadata.PreferencePredicate;
import eu.etaxonomy.cdm.model.metadata.PublishEnum;
import eu.etaxonomy.taxeditor.l10n.Messages;
import eu.etaxonomy.taxeditor.preference.PreferencesUtil;
import eu.etaxonomy.taxeditor.preference.PublishFlagLocalPreference;
import eu.etaxonomy.taxeditor.store.CdmStore;

/**
 * @author k.luther
 * @since 23.03.2018
 *
 */
public class PublishFlagPreference extends PublishFlagLocalPreference implements IE4AdminPreferencePage{


   @Override
   public void init() {
       super.init();


   }



    @Override
    public boolean performOk() {
        if (!isApply()){
            return true;
        }
        ICdmRepository controller = CdmStore.getCurrentApplicationConfiguration();
        if (controller == null){
            return false;
        }
        IPreferenceService service = controller.getPreferenceService();
        String textCombo = publishFlagBehaviour.getText();
        String text = null;
        // default -> delete preference
        if(textCombo.startsWith(Messages.Preference_Use_Default) && allowOverride){
            service.remove(pref.getKey());
            PreferencesUtil.updateDBPreferences();
            return true;
        }else if (textCombo.startsWith(Messages.Preference_Use_Default)){
            text = null;
        }else{
            for (PublishEnum display: PublishEnum.values()){
                if (display.getLabel().equals(textCombo)){
                    text = display.getKey();
                    break;
                }
            }
        }
        CdmPreference pref = CdmPreference.NewTaxEditorInstance(PreferencePredicate.DefaultBehaviourForPublishFlag, text);
        pref.setAllowOverride(allowOverride);

        service.set(pref);
        PreferencesUtil.updateDBPreferences();

        return true;
    }



    @Override
    public void getValues() {
        isAdminPreference = true;
        pref = PreferencesUtil.getPreferenceFromDB(PreferencePredicate.DefaultBehaviourForPublishFlag);
        if (pref == null){
            pref = CdmPreference.NewTaxEditorInstance(PreferencePredicate.DefaultBehaviourForPublishFlag, ((PublishEnum)PreferencePredicate.DefaultBehaviourForPublishFlag.getDefaultValue()).getKey());
            publishBehaviour = null;
            allowOverride = pref.isAllowOverride();
        }
        else {
            allowOverride = pref.isAllowOverride();
            publishBehaviour = pref.getValue() != null? PublishEnum.valueOf(pref.getValue()): null;
        }
    }





}
