/**
* Copyright (C) 2019 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.databaseAdmin.preferencePage;

import eu.etaxonomy.cdm.api.application.ICdmRepository;
import eu.etaxonomy.cdm.api.service.IPreferenceService;
import eu.etaxonomy.cdm.model.metadata.CdmPreference;
import eu.etaxonomy.cdm.model.metadata.PreferencePredicate;
import eu.etaxonomy.cdm.model.metadata.TermSearchField;
import eu.etaxonomy.taxeditor.preference.PreferencesUtil;
import eu.etaxonomy.taxeditor.preference.SearchDialogPreferences;
import eu.etaxonomy.taxeditor.store.CdmStore;

/**
 * @author k.luther
 * @since 08.08.2019
 */
public class SearchDialogAdminPreferences extends SearchDialogPreferences implements IE4AdminPreferencePage {




    @Override
    public void getValues(){
        super.getValues();
        isAdminPreference = true;
        if(showIdInSelectionDialogPref != null){
            showIdInSelectionDialog = showIdInSelectionDialogPref.getValue() != null? Boolean.parseBoolean(showIdInSelectionDialogPref.getValue()): null;
        }else{
            showIdInSelectionDialog = null;
            showIdInSelectionDialogPref = CdmPreference.NewTaxEditorInstance(PreferencePredicate.ShowIdInSelectionDialog, null);
        }
        if (searchForIdentifierAsDefaultPref != null){
            searchForIdentifierAsDefault = searchForIdentifierAsDefaultPref.getValue() != null? Boolean.parseBoolean(searchForIdentifierAsDefaultPref.getValue()): null;
        }else{
            searchForIdentifierAsDefault = null;
            searchForIdentifierAsDefaultPref = CdmPreference.NewTaxEditorInstance(PreferencePredicate.SearchForIdentifierAsDefault, null);
        }
        if (searchForIdentifierAndTitleCachePref != null){
            searchForIdentifierAndTitleCache = searchForIdentifierAndTitleCachePref.getValue() != null? Boolean.parseBoolean(searchForIdentifierAndTitleCachePref.getValue()): null;
        }else{
            searchForIdentifierAndTitleCache = null;
            searchForIdentifierAndTitleCachePref =CdmPreference.NewTaxEditorInstance(PreferencePredicate.SearchForIdentifierAndTitleCache, null);
        }
        if (sortTaxaByRankAndNamePref != null){
            sortTaxaByRankAndName = sortTaxaByRankAndNamePref.getValue() != null? Boolean.parseBoolean(sortTaxaByRankAndNamePref.getValue()): null;
        }else{
            sortTaxaByRankAndName = null;
            sortTaxaByRankAndNamePref = CdmPreference.NewTaxEditorInstance(PreferencePredicate.SortTaxaByRankAndName, null);
        }
        if (filterCommonNameReferencesPref != null){
            filterCommonNameReferences = filterCommonNameReferencesPref.getValue() != null? Boolean.parseBoolean(filterCommonNameReferencesPref.getValue()): null;
        }else{
            filterCommonNameReferences = null;
            filterCommonNameReferencesPref = CdmPreference.NewTaxEditorInstance(PreferencePredicate.CommonNameReferencesWithMarker, null);
        }
        if (this.searchTermsByAbbrevPref != null){
            searchTermsByAbbrev = searchTermsByAbbrevPref.getValue() != null? TermSearchField.byKey(searchTermsByAbbrevPref.getValue()): null;
        }else{
            searchTermsByAbbrev = null;
            searchTermsByAbbrevPref = CdmPreference.NewTaxEditorInstance(PreferencePredicate.NamedAreaSearchForAbbrev, null);
        }

    }

    @Override
    public boolean performOk(){
        if (!isApply()){
            return true;
        }
        ICdmRepository controller = CdmStore.getCurrentApplicationConfiguration();
        if (controller != null){
            IPreferenceService service = controller.getPreferenceService();
            CdmPreference pref;
            if (showIdInSelectionDialog != null || !showIdInSelectionDialogPref.isAllowOverride()) {
                pref = CdmPreference.NewTaxEditorInstance(PreferencePredicate.ShowIdInSelectionDialog, showIdInSelectionDialog != null? showIdInSelectionDialog.toString():null);
                pref.setAllowOverride(showIdInSelectionDialogPref.isAllowOverride());
                service.set(pref);
            }else{
                service.remove(showIdInSelectionDialogPref.getKey());
            }


            if(searchForIdentifierAsDefault != null || !searchForIdentifierAsDefaultPref.isAllowOverride()){
                pref = CdmPreference.NewTaxEditorInstance(PreferencePredicate.SearchForIdentifierAsDefault, searchForIdentifierAsDefault!= null? searchForIdentifierAsDefault.toString():null);
                pref.setAllowOverride(searchForIdentifierAsDefaultPref.isAllowOverride());
                service.set(pref);
            }else{
                service.remove(searchForIdentifierAsDefaultPref.getKey());
            }


            if(searchForIdentifierAndTitleCache != null || !searchForIdentifierAndTitleCachePref.isAllowOverride()){
                pref = CdmPreference.NewTaxEditorInstance(PreferencePredicate.SearchForIdentifierAndTitleCache, searchForIdentifierAndTitleCache!= null? searchForIdentifierAndTitleCache.toString():null);
                pref.setAllowOverride(searchForIdentifierAndTitleCachePref.isAllowOverride());
                service.set(pref);
            }else{
                service.remove(searchForIdentifierAndTitleCachePref.getKey());
            }

            if(sortTaxaByRankAndName != null || !sortTaxaByRankAndNamePref.isAllowOverride()){

                pref = CdmPreference.NewTaxEditorInstance(PreferencePredicate.SortTaxaByRankAndName, sortTaxaByRankAndName!= null? sortTaxaByRankAndName.toString():null);
                pref.setAllowOverride(sortTaxaByRankAndNamePref.isAllowOverride());
                service.set(pref);
            }else{
                service.remove(sortTaxaByRankAndNamePref.getKey());
            }

            if(filterCommonNameReferences != null || !filterCommonNameReferencesPref.isAllowOverride()){
                pref = CdmPreference.NewTaxEditorInstance(PreferencePredicate.CommonNameReferencesWithMarker, filterCommonNameReferences!= null? filterCommonNameReferences.toString():null);
                pref.setAllowOverride(filterCommonNameReferencesPref.isAllowOverride());
                service.set(pref);
            }else{
                service.remove(filterCommonNameReferencesPref.getKey());
            }

            if(searchTermsByAbbrev != null || !searchTermsByAbbrevPref.isAllowOverride()){
                pref = CdmPreference.NewTaxEditorInstance(PreferencePredicate.NamedAreaSearchForAbbrev, searchTermsByAbbrev!= null? searchTermsByAbbrev.getKey():null);
                pref.setAllowOverride(searchTermsByAbbrevPref.isAllowOverride());
                service.set(pref);
            }else{
                service.remove(searchTermsByAbbrevPref.getKey());
            }
        }
        PreferencesUtil.updateDBPreferences();
        return true;
    }



}
