/**
* Copyright (C) 2018 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.databaseAdmin.preferencePage;

import org.eclipse.swt.widgets.Composite;

import eu.etaxonomy.cdm.api.application.ICdmRepository;
import eu.etaxonomy.cdm.api.service.IPreferenceService;
import eu.etaxonomy.cdm.model.metadata.CdmPreference;
import eu.etaxonomy.cdm.model.metadata.CdmPreference.PrefKey;
import eu.etaxonomy.cdm.model.metadata.PreferencePredicate;
import eu.etaxonomy.cdm.model.metadata.PreferenceSubject;
import eu.etaxonomy.taxeditor.preference.PreferencesUtil;
import eu.etaxonomy.taxeditor.preference.SpecimenOrObservationPreferences;
import eu.etaxonomy.taxeditor.store.CdmStore;

/**
 * @author k.luther
 * @since 27.04.2018
 *
 */
public class SpecimenAdminPreferences extends SpecimenOrObservationPreferences implements IE4AdminPreferencePage {



    Composite composite;
    CdmPreference showSpecimenPref = null;

    @Override
    public void init() {
        super.init();
        if(!CdmStore.isActive()){
            return;
        }
    }

    @Override
    protected void getValues(){
        isAdminPreference = true;

        ICdmRepository controller = CdmStore.getCurrentApplicationConfiguration();
        if (controller != null){
            IPreferenceService service = controller.getPreferenceService();
            PrefKey key = CdmPreference.NewKey(PreferenceSubject.NewTaxEditorInstance(), PreferencePredicate.ShowSpecimen);
            showSpecimenPref = service.find(key);
            if (showSpecimenPref != null){
                this.isShowSpecimenRelatedIssues = showSpecimenPref.getValue() != null? Boolean.valueOf(showSpecimenPref.getValue()): null;
                this.allowOverrideShowSpecimenRelatedIssues = showSpecimenPref.isAllowOverride();
            }else{
                this.allowOverrideShowSpecimenRelatedIssues = true;
            }

            key = CdmPreference.NewKey(PreferenceSubject.NewTaxEditorInstance(), PreferencePredicate.ShowSpecimenListEditor);
            CdmPreference isShowSpecimenListEditorPref = service.find(key);
            if (isShowSpecimenListEditorPref != null){
                this.isShowListEditor = isShowSpecimenListEditorPref.getValue() != null? Boolean.valueOf(isShowSpecimenListEditorPref.getValue()): null;
                this.allowOverrideShowListEditor = isShowSpecimenListEditorPref.isAllowOverride();
            }else{
//                this.isShowCollectingAreaInGeneralSection = (Boolean) PreferencePredicate.ShowCollectingAreasInGeneralSection.getDefaultValue();
                this.allowOverrideShowListEditor = true;
            }


            key = CdmPreference.NewKey(PreferenceSubject.NewTaxEditorInstance(), PreferencePredicate.ShowCollectingAreasInGeneralSection);
            CdmPreference isShowCollectingAreaInGeneralSectionPref = service.find(key);
            if (isShowCollectingAreaInGeneralSectionPref != null){
                this.isShowCollectingAreaInGeneralSection = isShowCollectingAreaInGeneralSectionPref.getValue() != null? Boolean.valueOf(isShowCollectingAreaInGeneralSectionPref.getValue()): null;
                this.allowOverrideShowCollectingAreaInGeneralSection = isShowCollectingAreaInGeneralSectionPref.isAllowOverride();
            }else{
//                this.isShowCollectingAreaInGeneralSection = (Boolean) PreferencePredicate.ShowCollectingAreasInGeneralSection.getDefaultValue();
                this.allowOverrideShowCollectingAreaInGeneralSection = true;
            }

            key = CdmPreference.NewKey(PreferenceSubject.NewTaxEditorInstance(), PreferencePredicate.DeterminationOnlyForFieldUnits);
            CdmPreference isDeterminationOnlyForFieldUnitsPref = service.find(key);
            if (isDeterminationOnlyForFieldUnitsPref != null){
                this.isDeterminationOnlyForFieldUnits = isDeterminationOnlyForFieldUnitsPref.getValue() != null? Boolean.valueOf(isDeterminationOnlyForFieldUnitsPref.getValue()): null;
                this.allowOverrideDeterminationOnlyForFieldUnits = isDeterminationOnlyForFieldUnitsPref.isAllowOverride();
            }else{
                this.allowOverrideDeterminationOnlyForFieldUnits = true;
            }
            key = CdmPreference.NewKey(PreferenceSubject.NewTaxEditorInstance(), PreferencePredicate.ShowTaxonAssociations);
            CdmPreference isShowTaxonAssociationPref = service.find(key);
            if (isShowTaxonAssociationPref != null){
                this.isShowTaxonAssociation = isShowTaxonAssociationPref.getValue() != null? Boolean.valueOf(isShowTaxonAssociationPref.getValue()): null;
                this.allowOverrideShowTaxonAssociation = isShowTaxonAssociationPref.isAllowOverride();
            }else{
                this.allowOverrideShowTaxonAssociation = true;
            }

            key = CdmPreference.NewKey(PreferenceSubject.NewTaxEditorInstance(), PreferencePredicate.ShowLifeForm);
            CdmPreference isShowLifeFormPref = service.find(key);
            if (isShowLifeFormPref != null){
                this.isShowLifeForm = isShowLifeFormPref.getValue() != null? Boolean.valueOf(isShowLifeFormPref.getValue()): null;
                this.allowOverrideShowLifeForm = isShowLifeFormPref.isAllowOverride();
            } else{
                this.allowOverrideShowLifeForm = true;
            }

        }
    }

    @Override
     public boolean performOk() {
        if (!isApply()){
            return true;
        }
        ICdmRepository controller = CdmStore.getCurrentApplicationConfiguration();
        if (controller != null){
            IPreferenceService service = controller.getPreferenceService();
            CdmPreference pref;
            if (isShowCollectingAreaInGeneralSection == null && allowOverrideShowCollectingAreaInGeneralSection){
                service.remove(CdmPreference.NewKey(PreferenceSubject.NewTaxEditorInstance(), PreferencePredicate.ShowCollectingAreasInGeneralSection));
            }else{
                pref = CdmPreference.NewTaxEditorInstance(PreferencePredicate.ShowCollectingAreasInGeneralSection, this.isShowCollectingAreaInGeneralSection != null? Boolean.toString(this.isShowCollectingAreaInGeneralSection): null);
                pref.setAllowOverride(allowOverrideShowCollectingAreaInGeneralSection);
                service.set(pref);
            }

            if (isShowListEditor == null && allowOverrideShowListEditor){
                service.remove(CdmPreference.NewKey(PreferenceSubject.NewTaxEditorInstance(), PreferencePredicate.ShowSpecimenListEditor));
            }else{
                pref = CdmPreference.NewTaxEditorInstance(PreferencePredicate.ShowSpecimenListEditor, this.isShowListEditor != null? Boolean.toString(this.isShowListEditor): null);
                pref.setAllowOverride(allowOverrideShowListEditor);
                service.set(pref);
            }

            if (isShowSpecimenRelatedIssues == null && allowOverrideShowSpecimenRelatedIssues){
                service.remove(CdmPreference.NewKey(PreferenceSubject.NewTaxEditorInstance(), PreferencePredicate.ShowSpecimen));
            }else{
                pref = CdmPreference.NewTaxEditorInstance(PreferencePredicate.ShowSpecimen, isShowSpecimenRelatedIssues != null? Boolean.toString(this.isShowSpecimenRelatedIssues): null);
                pref.setAllowOverride(allowOverrideShowSpecimenRelatedIssues);
                service.set(pref);
            }

            if (isDeterminationOnlyForFieldUnits == null && allowOverrideDeterminationOnlyForFieldUnits){
                service.remove(CdmPreference.NewKey(PreferenceSubject.NewTaxEditorInstance(), PreferencePredicate.DeterminationOnlyForFieldUnits));
            }else{
                pref = CdmPreference.NewTaxEditorInstance(PreferencePredicate.DeterminationOnlyForFieldUnits, isDeterminationOnlyForFieldUnits != null? Boolean.toString(this.isDeterminationOnlyForFieldUnits): null);
                pref.setAllowOverride(allowOverrideDeterminationOnlyForFieldUnits);
                service.set(pref);
            }
            if (isShowTaxonAssociation == null && allowOverrideShowTaxonAssociation){
                service.remove(CdmPreference.NewKey(PreferenceSubject.NewTaxEditorInstance(), PreferencePredicate.ShowTaxonAssociations));
            }else{
                pref = CdmPreference.NewTaxEditorInstance(PreferencePredicate.ShowTaxonAssociations, isShowTaxonAssociation != null? Boolean.toString(this.isShowTaxonAssociation): null);
                pref.setAllowOverride(allowOverrideShowTaxonAssociation);
                service.set(pref);
            }
            if (isShowLifeForm == null && allowOverrideShowLifeForm){
                service.remove(CdmPreference.NewKey(PreferenceSubject.NewTaxEditorInstance(), PreferencePredicate.ShowLifeForm));
            }else{
                pref = CdmPreference.NewTaxEditorInstance(PreferencePredicate.ShowLifeForm, isShowLifeForm != null? Boolean.toString(this.isShowLifeForm): null);
                pref.setAllowOverride(allowOverrideShowLifeForm);
                service.set(pref);
            }
            PreferencesUtil.updateDBPreferences();


        }
        return true;
    }

}
