/**
* Copyright (C) 2016 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.databaseAdmin.wizard;

import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;

import eu.etaxonomy.cdm.api.service.config.CacheUpdaterConfigurator;
import eu.etaxonomy.cdm.model.agent.TeamOrPersonBase;
import eu.etaxonomy.cdm.model.name.TaxonName;
import eu.etaxonomy.cdm.model.occurrence.SpecimenOrObservationBase;
import eu.etaxonomy.cdm.model.reference.Reference;
import eu.etaxonomy.cdm.model.taxon.TaxonBase;
import eu.etaxonomy.taxeditor.l10n.Messages;

/**
 * @author k.luther
 * @date 14.12.2016
 *
 */
public class CacheUpdaterWizardPage extends WizardPage {

    CacheUpdaterConfigurator config;

    boolean updateAgentBase = false;
    boolean updateReference = false;
    boolean updateTaxonBase = false;
    boolean updateName = false;
    boolean updateSpecimen = false;

    public CacheUpdaterWizardPage(String pageName, CacheUpdaterConfigurator configTitleCache) {
        super(pageName);
        this.setDescription(Messages.DatabaseRepairPage_description);
        config = configTitleCache;

    }

    /**
     * {@inheritDoc}
     */
    @Override
    public void createControl(Composite parent) {


            Composite composite = new Composite(parent, SWT.NONE);

            composite.setLayout(new GridLayout(1, true));
            setControl(composite);
            Button checkBoxTeamOrPerson = new Button(composite, SWT.CHECK);
            checkBoxTeamOrPerson.setSelection(config.getClassList().contains(TeamOrPersonBase.class));
            checkBoxTeamOrPerson.setText(Messages.DatabaseRepairPage_TeamOrPerson);
            checkBoxTeamOrPerson
            .setToolTipText(Messages.DatabaseRepairPage_toolTip_teamOrPerson);
            checkBoxTeamOrPerson.addSelectionListener(new SelectionAdapter() {
                @Override
                public void widgetSelected(SelectionEvent e) {
                	Button b = (Button) e.widget;
                	updateAgentBase =  b.getSelection();

                }
            });

            Button checkBoxReference = new Button(composite, SWT.CHECK);
            checkBoxReference.setSelection(config.getClassList().contains(Reference.class));
            checkBoxReference.setText(Messages.DatabaseRepairPage_Reference);
            checkBoxReference
            .setToolTipText(Messages.DatabaseRepairPage_toolTip_reference);
            checkBoxReference.addSelectionListener(new SelectionAdapter() {
                @Override
                public void widgetSelected(SelectionEvent e) {
                	Button b = (Button) e.widget;
                	updateReference= b.getSelection();
                }
            });

            Button checkBoxTaxonName = new Button(composite, SWT.CHECK);
            checkBoxTaxonName.setSelection(config.getClassList().contains(TaxonName.class));
            checkBoxTaxonName.setText(Messages.DatabaseRepairPage_updateTaxonName);
            checkBoxTaxonName
                    .setToolTipText(Messages.DatabaseRepairPage_toolTip_TaxonName);
            checkBoxTaxonName.addSelectionListener(new SelectionAdapter() {
                @Override
                public void widgetSelected(SelectionEvent e) {
                	Button b = (Button) e.widget;
                	updateName = b.getSelection();

                }
            });

            Button checkBoxTaxonBase = new Button(composite, SWT.CHECK);
            checkBoxTaxonBase.setSelection(config.getClassList().contains(TaxonBase.class));
            checkBoxTaxonBase.setText(Messages.DatabaseRepairPage_TaxonBase);
            checkBoxTaxonBase
            .setToolTipText(Messages.DatabaseRepairPage_toolTip_taxonBase);
            checkBoxTaxonBase.addSelectionListener(new SelectionAdapter() {
                @Override
                public void widgetSelected(SelectionEvent e) {
                	Button b = (Button) e.widget;
                	updateTaxonBase = b.getSelection();

                }
            });


            Button checkBoxSpecimen = new Button(composite, SWT.CHECK);
            checkBoxSpecimen.setSelection(config.getClassList().contains(SpecimenOrObservationBase.class));
            checkBoxSpecimen.setText(Messages.DatabaseRepairPage_Specimen);
            checkBoxSpecimen
            .setToolTipText(Messages.DatabaseRepairPage_toolTip_specimen);
            checkBoxSpecimen.addSelectionListener(new SelectionAdapter() {
                @Override
                public void widgetSelected(SelectionEvent e) {
                	Button b = (Button) e.widget;
                	updateSpecimen = b.getSelection();

                }
            });


    }
}
