/**
* Copyright (C) 2019 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.databaseAdmin.wizard;

import java.util.UUID;

import org.eclipse.jface.wizard.Wizard;
import org.eclipse.swt.widgets.Display;

import eu.etaxonomy.cdm.api.application.CdmApplicationState;
import eu.etaxonomy.cdm.api.service.config.CacheUpdaterConfigurator;
import eu.etaxonomy.cdm.api.service.longrunningService.ILongRunningTasksService;
import eu.etaxonomy.cdm.common.monitor.IRemotingProgressMonitor;
import eu.etaxonomy.cdm.model.agent.TeamOrPersonBase;
import eu.etaxonomy.cdm.model.name.TaxonName;
import eu.etaxonomy.cdm.model.occurrence.SpecimenOrObservationBase;
import eu.etaxonomy.cdm.model.reference.Reference;
import eu.etaxonomy.cdm.model.taxon.TaxonBase;
import eu.etaxonomy.taxeditor.model.AbstractUtility;
import eu.etaxonomy.taxeditor.operation.IPostMoniteredOperationEnabled;

/**
 * @author k.luther
 * @since 05.03.2019
 */
public class DatabaseRepairUpdateCachesWizard extends Wizard implements IPostMoniteredOperationEnabled {

    CacheUpdaterConfigurator cacheUpdaterConfig;
    ILongRunningTasksService longRunningTaskService;
    CacheUpdaterWizardPage page;


    public DatabaseRepairUpdateCachesWizard() {
        setWindowTitle("Update Caches");
        cacheUpdaterConfig = CacheUpdaterConfigurator.NewInstance();
        longRunningTaskService = CdmApplicationState.getLongRunningTasksService();
        page = new CacheUpdaterWizardPage("Update Caches", cacheUpdaterConfig);
    }

    @Override
    public void postOperation(IRemotingProgressMonitor monitor) {
        // TODO Auto-generated method stub
    }

    @Override
    public void addPages() {
        addPage(page);
    }

    @Override
    public boolean performFinish() {
        if (page.updateAgentBase){
            cacheUpdaterConfig.getClassList().add(TeamOrPersonBase.class);
        }
        if (page.updateReference){
            cacheUpdaterConfig.getClassList().add(Reference.class);
        }
        if (page.updateName){
            cacheUpdaterConfig.getClassList().add(TaxonName.class);
        }
        if (page.updateTaxonBase){
            cacheUpdaterConfig.getClassList().add(TaxonBase.class);
        }
        if (page.updateSpecimen){
            cacheUpdaterConfig.getClassList().add(SpecimenOrObservationBase.class);
        }
        if (!cacheUpdaterConfig.getClassList().isEmpty()){

             final UUID uuid = longRunningTaskService.monitLongRunningTask(cacheUpdaterConfig);
             Display.getDefault().asyncExec(()->{
                     AbstractUtility.executeMoniteredOperation("Update Caches",
                             uuid,
                             1000,
                             false,
                             DatabaseRepairUpdateCachesWizard.this,
                             null,
                             true, true);
             });
        }
        return true;
    }
}