/**
* Copyright (C) 2016 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.databaseAdmin.wizard;

import java.util.UUID;

import org.eclipse.jface.wizard.Wizard;
import org.eclipse.swt.widgets.Display;

import eu.etaxonomy.cdm.api.application.CdmApplicationState;
import eu.etaxonomy.cdm.api.service.config.SortIndexUpdaterConfigurator;
import eu.etaxonomy.cdm.api.service.longrunningService.ILongRunningTasksService;
import eu.etaxonomy.cdm.common.monitor.IRemotingProgressMonitor;
import eu.etaxonomy.cdm.config.ICdmSource;
import eu.etaxonomy.taxeditor.model.AbstractUtility;
import eu.etaxonomy.taxeditor.operation.IPostMoniteredOperationEnabled;
import eu.etaxonomy.taxeditor.store.CdmStore;

/**
 * @author k.luther
 * @date 06.12.2016
 */
public class DatabaseRepairUpdateSortIndicesWizard extends Wizard implements IPostMoniteredOperationEnabled{
   // private final DatabaseRepairSortIndexPage repairSortIndexPage;
    private final SortIndexUpdaterWizardPage page;
    SortIndexUpdaterConfigurator config;

    ILongRunningTasksService longRunningTaskService;


    public DatabaseRepairUpdateSortIndicesWizard() {
        setWindowTitle("Update Sort Indices");
        config = SortIndexUpdaterConfigurator.NewInstance();
        page = new SortIndexUpdaterWizardPage("Sort index update", config);
        longRunningTaskService = CdmApplicationState.getLongRunningTasksService();
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public boolean performFinish() {

            if (config == null){
                ICdmSource source= CdmStore.getActiveCdmSource();
                config = SortIndexUpdaterConfigurator.NewInstance();
                config.setDoTermNode(false);
                config.setDoPolytomousKeyNode(false);
             }
            if (config.isDoTermNode() || config.isDoPolytomousKeyNode() || config.isDoTaxonNode()){

                final UUID uuid = longRunningTaskService.monitLongRunningTask(config);
                Display.getDefault().asyncExec(new Runnable() {
                    @Override
                    public void run() {
                        AbstractUtility.executeMoniteredOperation("Update SortIndices: ",
                                uuid,
                                1000,
                                false,
                                DatabaseRepairUpdateSortIndicesWizard.this,
                                null,
                                true, true);
                    }
                });
            }

        return true;
    }

    @Override
    public void addPages() {
        addPage(page);
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public void postOperation(IRemotingProgressMonitor monitor) {
        // TODO Auto-generated method stub
    }
}
