/**
* Copyright (C) 2019 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.databaseAdmin.wizard;

import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;

import eu.etaxonomy.cdm.api.service.config.SortIndexUpdaterConfigurator;
import eu.etaxonomy.taxeditor.l10n.Messages;

/**
 * @author k.luther
 * @since 05.03.2019
 *
 */
public class SortIndexUpdaterWizardPage extends WizardPage {

    SortIndexUpdaterConfigurator sortIndexConfig;

    public SortIndexUpdaterWizardPage(String pageName, SortIndexUpdaterConfigurator sortIndexConfig) {
        super(pageName);
        this.setDescription(Messages.DatabaseRepairPage_description_sortIndex);
        this.sortIndexConfig = sortIndexConfig;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public void createControl(Composite parent) {
        Composite composite = new Composite(parent, SWT.NONE);

        composite.setLayout(new GridLayout(1, true));
        setControl(composite);
        Button checkBoxTaxonNode = new Button(composite, SWT.CHECK);
        checkBoxTaxonNode.setSelection(sortIndexConfig.isDoTaxonNode());
        checkBoxTaxonNode.setText(Messages.DatabaseRepairPage_updateTaxonNodes);
        checkBoxTaxonNode
                .setToolTipText(Messages.DatabaseRepairPage_toolTip_taxonNode);
        checkBoxTaxonNode.addSelectionListener(new SelectionAdapter() {
            @Override
            public void widgetSelected(SelectionEvent e) {
                sortIndexConfig.setDoTaxonNode(!sortIndexConfig.isDoTaxonNode());
            }
        });

        Button checkBoxPolytomousKeyNode = new Button(composite, SWT.CHECK);
        checkBoxPolytomousKeyNode.setSelection(sortIndexConfig.isDoPolytomousKeyNode());
        checkBoxPolytomousKeyNode.setText(Messages.DatabaseRepairPage_PolytomousKeyNode);
        checkBoxPolytomousKeyNode
        .setToolTipText(Messages.DatabaseRepairPage_toolTip_polytomousKeyNode);
        checkBoxPolytomousKeyNode.addSelectionListener(new SelectionAdapter() {
            @Override
            public void widgetSelected(SelectionEvent e) {
                sortIndexConfig.setDoPolytomousKeyNode(!sortIndexConfig.isDoPolytomousKeyNode());
            }
        });

        Button checkBoxFeatureNode = new Button(composite, SWT.CHECK);
        checkBoxFeatureNode.setSelection(sortIndexConfig.isDoTermNode());
        checkBoxFeatureNode.setText(Messages.DatabaseRepairPage_featureNodes);
        checkBoxFeatureNode
        .setToolTipText(Messages.DatabaseRepairPage_toolTipFeatureNodes);
        checkBoxFeatureNode.addSelectionListener(new SelectionAdapter() {
            @Override
            public void widgetSelected(SelectionEvent e) {
                sortIndexConfig.setDoTermNode(!sortIndexConfig.isDoTermNode());
            }
        });

    }

}
