/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.dnd;

import java.util.Collection;
import java.util.Iterator;

import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.swt.dnd.DropTargetEvent;
import org.eclipse.swt.dnd.DropTargetListener;

import eu.etaxonomy.cdm.api.service.ITermService;
import eu.etaxonomy.cdm.model.common.CdmBase;
import eu.etaxonomy.cdm.persistence.dto.TermDto;
import eu.etaxonomy.cdm.persistence.dto.UuidAndTitleCache;
import eu.etaxonomy.taxeditor.store.CdmStore;
import eu.etaxonomy.taxeditor.ui.group.grantedauthority.CdmAuthorityComposite;
import eu.etaxonomy.taxeditor.ui.group.grantedauthority.CdmAuthorityCompositeViewer;

/**
 * Drop listener for the {@link CdmAuthorityComposite}.
 *
 * @author cmathew
 * @created Mar 28, 2013
 *
 */
public class CdmAuthorityTableDropTargetListener implements DropTargetListener {

	private CdmAuthorityCompositeViewer viewer;


	public CdmAuthorityTableDropTargetListener(CdmAuthorityCompositeViewer viewer) {
		this.viewer = viewer;
	}

	@Override
	public void dragEnter(DropTargetEvent event) {
	}

	@Override
	public void dragLeave(DropTargetEvent arg0) {
	}

	@Override
	public void dragOperationChanged(DropTargetEvent arg0) {
	}

	@Override
	public void dragOver(DropTargetEvent arg0) {
	}

	@Override
	public void drop(DropTargetEvent dtevent) {
	    Object data = dtevent.data;
	    if (data instanceof IStructuredSelection) {
	        Iterator<?> selectionIterator = ((IStructuredSelection) dtevent.data).iterator();
	        while (selectionIterator.hasNext()){
	            addAuthority(selectionIterator.next());
	        }
	    }
	    else if(data instanceof Object[]) {
	        for(int i=0;i<((Object[])data).length;i++){
	            addAuthority(((Object[])data)[i]);
	        }
	    }
	    else if(data instanceof Collection) {
	        ((Collection) data).forEach(d->addAuthority(d));
	    }
	    else{
	        addAuthority(data);
	    }
	    viewer.getControl().setFocus();
	}

	private void addAuthority(Object authority){
	    if(CdmBase.class.isAssignableFrom(authority.getClass()) ) {
	        viewer.addCdmAuthority((CdmBase)authority);
	    }
	    else if(authority instanceof UuidAndTitleCache){
	        UuidAndTitleCache uuidAndTitleCache = (UuidAndTitleCache) authority;
            CdmBase cdmBase = CdmStore.getCommonService().find(uuidAndTitleCache.getType(), uuidAndTitleCache.getUuid());
            viewer.addCdmAuthority(cdmBase);
	    }else if (authority instanceof TermDto){
	        TermDto uuidAndTitleCache = (TermDto) authority;
            CdmBase cdmBase = CdmStore.getService(ITermService.class).find(uuidAndTitleCache.getUuid());
            viewer.addCdmAuthority(cdmBase);
	    }
	}

	@Override
	public void dropAccept(DropTargetEvent arg0) {
	}

}
