/**
* Copyright (C) 2015 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.editor;

import java.util.List;
import java.util.Map;

import eu.etaxonomy.cdm.model.common.CdmBase;
import eu.etaxonomy.taxeditor.session.ICdmEntitySession;
import eu.etaxonomy.taxeditor.session.ICdmEntitySessionEnabled;
import eu.etaxonomy.taxeditor.store.CdmStore;

/**
 * @author cmathew
 * @date 5 Feb 2015
 */
public abstract class CdmEntitySessionInput<T extends CdmBase> implements ICdmEntitySessionEnabled<T> {

    private ICdmEntitySession cdmEntitySession;

    public CdmEntitySessionInput(boolean init) {
        if(init) {
            initSession();
        }
    }

    protected void initSession() {
        cdmEntitySession = CdmStore.getCurrentSessionManager().newSession(this, true);
    }

    @Override
    public ICdmEntitySession getCdmEntitySession() {
        return cdmEntitySession;
    }

    public void bind() {
        if(cdmEntitySession!=null){
            cdmEntitySession.bind();
        }
        else {
            initSession();
        }
    }

    public void dispose() {
        if (cdmEntitySession != null){
            cdmEntitySession.dispose();
        }
        cdmEntitySession = null;

    }

    public abstract void merge();

    @Override
    public Map<Object, List<String>> getPropertyPathsMap() {
        // TODO Auto-generated method stub
        return null;
    }

}
