// $Id$
/**
* Copyright (C) 2018 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.editor.definedterm;

import org.eclipse.jface.viewers.Viewer;
import org.eclipse.jface.viewers.ViewerComparator;

import eu.etaxonomy.cdm.api.service.l10n.TermRepresentation_L10n;
import eu.etaxonomy.cdm.model.term.OrderedTermBase;
import eu.etaxonomy.cdm.persistence.dto.TermDto;
import eu.etaxonomy.cdm.persistence.dto.TermVocabularyDto;

/**
 * @author pplitzner
 * @date 13.02.2018
 */
public class DefinedTermSorter extends ViewerComparator {

    @Override
    public int compare(Viewer viewer, Object e1, Object e2) {
     // the comparison value in this method determines the
        // location <-> add term method used in the MoveDefinedTermOperation
        // execute call
        //FIXME: remove this when all viewer are changed to use DTOs
        if(e1 instanceof OrderedTermBase && e2 instanceof OrderedTermBase) {
            OrderedTermBase otbe1 = (OrderedTermBase)e1;
            OrderedTermBase otbe2 = (OrderedTermBase)e2;
            if(otbe1.getOrderIndex() == otbe2.getOrderIndex()) {
                return 0;
            } else if (otbe1.getOrderIndex() < otbe2.getOrderIndex()){
                return -1;
            } else{
                return 1;
            }
        }
        else if(e1 instanceof TermDto && e2 instanceof TermDto) {
            TermDto termDto1 = (TermDto)e1;
            TermDto termDto2 = (TermDto)e2;
            int orderIndexCompare = 0;
            if(termDto1.getOrderIndex()!=null){
                if(termDto2.getOrderIndex()!=null){
                    orderIndexCompare =   termDto1.getOrderIndex() - termDto2.getOrderIndex();
                }
                else{
                    orderIndexCompare = 1;
                }
            }
            else if(termDto2.getOrderIndex()!=null){
                orderIndexCompare = -1;
            }
            if(orderIndexCompare != 0){
                return orderIndexCompare;
            }
            else {
                // order indexes are equal or null -> compare by label
                termDto1.localize(new TermRepresentation_L10n());
                termDto2.localize(new TermRepresentation_L10n());
                if(termDto1.getRepresentation_L10n()!=null){
                    if(termDto2.getRepresentation_L10n()!=null) {
                        return termDto1.getRepresentation_L10n().toLowerCase().compareTo(termDto2.getRepresentation_L10n().toLowerCase());
                    }
                    else{
                        return 1;
                    }
                }
                else if(termDto2.getRepresentation_L10n()!=null){
                    return -1;
                }
            }
        }
        else if(e1 instanceof TermVocabularyDto && e2 instanceof TermVocabularyDto) {
            TermVocabularyDto termVoc1 = (TermVocabularyDto)e1;
            TermVocabularyDto termVoc2 = (TermVocabularyDto)e2;
            termVoc1.localize(new TermRepresentation_L10n());
            termVoc2.localize(new TermRepresentation_L10n());
            if(termVoc1.getRepresentation_L10n()!=null){
                if(termVoc2.getRepresentation_L10n()!=null) {
                    return termVoc1.getRepresentation_L10n().toLowerCase().compareTo(termVoc2.getRepresentation_L10n().toLowerCase());
                }
                else{
                    return 1;
                }
            }
            else if(termVoc2.getRepresentation_L10n()!=null){
                return -1;
            }
        }
        return super.compare(viewer, e1, e2);
    }
}
