/**
* Copyright (C) 2020 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.editor.definedterm;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashSet;

import eu.etaxonomy.cdm.api.service.IVocabularyService;
import eu.etaxonomy.cdm.persistence.dto.FeatureDto;
import eu.etaxonomy.cdm.persistence.dto.TermDto;
import eu.etaxonomy.cdm.persistence.dto.TermVocabularyDto;
import eu.etaxonomy.taxeditor.store.CdmStore;

/**
 * @author k.luther
 * @since Jun 11, 2020
 */
public class FeatureDtoContentProvider extends TermDtoContentProvider {

    boolean isAvailableForTaxon = true;
    boolean isAvailableForTaxonName = true;
    boolean isAvailableForOccurrence = true;

    public FeatureDtoContentProvider(boolean isAvailableForTaxon, boolean isAvailableForTaxonName, boolean isAvailableForOccurrence){
        super();
        this.isAvailableForOccurrence = isAvailableForOccurrence;
        this.isAvailableForTaxon = isAvailableForTaxon;
        this.isAvailableForTaxonName = isAvailableForTaxonName;
    }

    @Override
    public Object[] getChildren(Object parentElement) {
        Collection<TermDto> children = new HashSet<>();
        if(parentElement instanceof TermVocabularyDto){
            children.addAll(getChildTerms((TermVocabularyDto)parentElement));
        } else if(parentElement instanceof FeatureDto){
                if(((FeatureDto) parentElement).getIncludes()!=null){
                    for (TermDto child: ((FeatureDto) parentElement).getIncludes()){
                        if (child instanceof FeatureDto){
                            FeatureDto featureChild= (FeatureDto)child;
                            if (featureChild.isAvailableForOccurrence() && isAvailableForOccurrence) {
                                children.add(featureChild);
                            }
                            else if (featureChild.isAvailableForTaxon() && isAvailableForTaxon) {
                                children.add(featureChild);
                            }else if (featureChild.isAvailableForTaxonName() && isAvailableForTaxonName) {
                                children.add(featureChild);
                            }
                        }
                    }

                }
                if(((FeatureDto) parentElement).getGeneralizationOf()!=null){
                    for (TermDto child: ((FeatureDto) parentElement).getGeneralizationOf()){
                        if (child instanceof FeatureDto){
                            FeatureDto featureChild= (FeatureDto)child;
                            if (featureChild.isAvailableForOccurrence() && isAvailableForOccurrence) {
                                children.add(featureChild);
                            }
                            else if (featureChild.isAvailableForTaxon() && isAvailableForTaxon) {
                                children.add(featureChild);
                            }else if (featureChild.isAvailableForTaxonName() && isAvailableForTaxonName) {
                                children.add(featureChild);
                            }
                        }
                    }

                }
        }
        return children.toArray();
    }



    @Override
    public Collection<TermDto> getChildTerms(TermVocabularyDto voc) {
        Collection<TermDto> children = getVocabularyToChildTermMap().get(voc);
        Collection<TermDto> filteredChildren = new HashSet<>();
        if(children==null || children.isEmpty()){
            children = new ArrayList<>(CdmStore.getService(IVocabularyService.class).getCompleteTermHierarchy(voc));
            for (TermDto child: children){
                if (child instanceof FeatureDto){
                    FeatureDto featureChild= (FeatureDto)child;
                    if (featureChild.isAvailableForOccurrence() && isAvailableForOccurrence) {
                        filteredChildren.add(featureChild);
                    }
                    else if (featureChild.isAvailableForTaxon() && isAvailableForTaxon) {
                        filteredChildren.add(featureChild);
                    }else if (featureChild.isAvailableForTaxonName() && isAvailableForTaxonName) {
                        filteredChildren.add(featureChild);
                    }
                }
            }
            if (filteredChildren.isEmpty()){
                getVocabularyToChildTermMap().remove(voc);
            }else{
                getVocabularyToChildTermMap().put(voc, filteredChildren);
            }
        }else{
            filteredChildren = children;
        }
        return filteredChildren;
    }


}
