/**
* Copyright (C) 2009 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.editor.definedterm;

import org.eclipse.core.expressions.PropertyTester;
import org.eclipse.jface.viewers.IStructuredSelection;

import eu.etaxonomy.cdm.api.service.IVocabularyService;
import eu.etaxonomy.cdm.hibernate.HibernateProxyHelper;
import eu.etaxonomy.cdm.model.common.Marker;
import eu.etaxonomy.cdm.model.common.MarkerType;
import eu.etaxonomy.cdm.model.term.DefinedTermBase;
import eu.etaxonomy.cdm.model.term.TermBase;
import eu.etaxonomy.cdm.model.term.TermVocabulary;
import eu.etaxonomy.cdm.persistence.dto.TermDto;
import eu.etaxonomy.cdm.persistence.dto.TermVocabularyDto;
import eu.etaxonomy.taxeditor.store.CdmStore;

/**
 * @author l.morris
 * @date 9 Jan 2012
 */
public class TermBasePropertyTester extends PropertyTester {

	private static final String IS_MODIFIABLE = "isModifiable";

	@Override
	public boolean test(Object receiver, String property, Object[] args,
			Object expectedValue) {

		IStructuredSelection selection = (IStructuredSelection) receiver;
		Object selectedElement = selection.getFirstElement();
		if (selectedElement != null && HibernateProxyHelper.isInstanceOf(selectedElement, TermBase.class)){
			TermBase term = HibernateProxyHelper.deproxy(selectedElement, TermBase.class);

			if(IS_MODIFIABLE.equals(property)){
				return isModifiable(term);
			}
		}

		return false;
	}


	/**
	 * Checks whether there is a {@link Marker} with the type {@link MarkerType#MODIFIABLE()} and if there is then return its value.
	 *
	 * @return The markers value if it exists
	 */
	public static boolean isModifiable(Object object) {
		if (object == null){
			return true;
		}

		TermVocabulary<?> vocabulary = null;

		if(object instanceof DefinedTermBase){
			vocabulary = ((DefinedTermBase<?>) object).getVocabulary();
		}else if(object instanceof TermVocabulary){
			vocabulary = (TermVocabulary<?>) object;
		}else if(object instanceof TermDto){
            vocabulary = CdmStore.getService(IVocabularyService.class).load(((TermDto) object).getVocabularyUuid());
        }else if(object instanceof TermVocabularyDto){
            vocabulary = CdmStore.getService(IVocabularyService.class).load(((TermVocabularyDto) object).getUuid());
        }

		if(vocabulary == null){
			return true;
		}

		for(Marker vocabularyMarker : vocabulary.getMarkers()){
			if(vocabularyMarker.getMarkerType().equals(MarkerType.MODIFIABLE())){
				return vocabularyMarker.getValue();
			}
		}

		return true;
	}
}
