/**
* Copyright (C) 2009 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.editor.definedterm;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;

import org.eclipse.jface.viewers.TreeNodeContentProvider;

import eu.etaxonomy.cdm.api.service.IVocabularyService;
import eu.etaxonomy.cdm.persistence.dto.TermDto;
import eu.etaxonomy.cdm.persistence.dto.TermVocabularyDto;
import eu.etaxonomy.taxeditor.store.CdmStore;

/**
 *
 * @author pplitzner
 * @since Oct 29, 2018
 *
 */
public class TermDtoContentProvider extends TreeNodeContentProvider {

    private Map<TermVocabularyDto, Collection<TermDto>> vocabularyToChildTermMap = new HashMap<>();

    @Override
	public Object[] getElements(Object inputElement) {
		Collection<TermVocabularyDto> inputElements = (Collection<TermVocabularyDto>) inputElement;
		return inputElements.toArray();
	}

	@Override
	public Object[] getChildren(Object parentElement) {
	    Collection<TermDto> children = new HashSet<>();
	    if(parentElement instanceof TermVocabularyDto){
	        children.addAll(getChildTerms((TermVocabularyDto)parentElement));
	    } else if(parentElement instanceof TermDto){
	            if(((TermDto) parentElement).getIncludes()!=null){
	                children.addAll(((TermDto) parentElement).getIncludes());
	            }
                if(((TermDto) parentElement).getGeneralizationOf()!=null){
                    children.addAll(((TermDto) parentElement).getGeneralizationOf());
                }
	    }
	    return children.toArray();
	}

	@Override
	public Object getParent(Object element) {
		if(element instanceof TermDto){
		    TermDto termDto = (TermDto) element;
		    TermDto partOfDto = termDto.getPartOfDto();
		    if(partOfDto!=null){
		        return partOfDto;
		    }
		    TermDto kindOfDto = termDto.getKindOfDto();
		    if(kindOfDto!=null){
		        return kindOfDto;
		    }
		    TermVocabularyDto vocabularyDto = termDto.getVocabularyDto();
		    if(vocabularyDto!=null){
		        return vocabularyDto;
		    }
		    //parent is the vocabulary
		    return termDto.getVocabularyDto();
		}
		return null;

	}

	@Override
	public boolean hasChildren(Object element) {
	    if(element instanceof TermVocabularyDto){
	        return getChildren(element).length > 0;
	    }
		if (getChildren(element) != null){
			return getChildren(element).length > 0;
		}
		return false;
	}

    public Collection<? extends TermDto> getChildTerms(TermVocabularyDto voc) {
        Collection<TermDto> children = getVocabularyToChildTermMap().get(voc);
        if(children==null){
            children = new ArrayList<>(CdmStore.getService(IVocabularyService.class).getCompleteTermHierarchy(voc));
            getVocabularyToChildTermMap().put(voc, children);
        }
        return children;
    }

    public void removeVocabularyFromCache(TermVocabularyDto voc){
        getVocabularyToChildTermMap().remove(voc);
    }

    public Map<TermVocabularyDto, Collection<TermDto>> getVocabularyToChildTermMap() {
        return vocabularyToChildTermMap;
    }

    public void setVocabularyToChildTermMap(Map<TermVocabularyDto, Collection<TermDto>> vocabularyToChildTermMap) {
        this.vocabularyToChildTermMap = vocabularyToChildTermMap;
    }

}
