/**
 * Copyright (C) 2009 EDIT
 * European Distributed Institute of Taxonomy
 * http://www.e-taxonomy.eu
 *
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */
package eu.etaxonomy.taxeditor.editor.definedterm;

import org.eclipse.jface.viewers.ColumnLabelProvider;
import org.eclipse.jface.viewers.DelegatingStyledCellLabelProvider.IStyledLabelProvider;
import org.eclipse.jface.viewers.StyledString;
import org.eclipse.jface.viewers.StyledString.Styler;
import org.eclipse.jface.viewers.ViewerCell;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.StyleRange;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.TextStyle;
import org.eclipse.swt.widgets.Display;

import eu.etaxonomy.cdm.api.service.l10n.TermRepresentation_L10n;
import eu.etaxonomy.cdm.common.CdmUtils;
import eu.etaxonomy.cdm.model.term.Representation;
import eu.etaxonomy.cdm.persistence.dto.AbstractTermDto;
import eu.etaxonomy.cdm.persistence.dto.TermDto;
import eu.etaxonomy.cdm.persistence.dto.TermTreeDto;
import eu.etaxonomy.cdm.persistence.dto.TermVocabularyDto;
import eu.etaxonomy.taxeditor.preference.IPreferenceKeys;
import eu.etaxonomy.taxeditor.preference.PreferencesUtil;

/**
 *
 * @author pplitzner
 * @since Oct 29, 2018
 *
 */
public class TermDtoLabelProvider //extends StyledCellLabelProvider implements ILabelProvider{
    extends ColumnLabelProvider implements IStyledLabelProvider{

    private static Color vocColor = Display.getCurrent().getSystemColor(SWT.COLOR_BLUE);
    private static Color kindOfColor = Display.getCurrent().getSystemColor(SWT.COLOR_DARK_GRAY);
    private Styler vocStyler;
    private Styler kindOfStyler;
    protected boolean showIdInVoc = false;

    public TermDtoLabelProvider() {
        showIdInVoc = PreferencesUtil.getBooleanValue(IPreferenceKeys.SHOW_VOCABULARY_ID_FOR_TERM_LABELS, true);
    }

    public TermDtoLabelProvider(Styler vocStyler){
        this.vocStyler = vocStyler;
        showIdInVoc = PreferencesUtil.getBooleanValue(IPreferenceKeys.SHOW_VOCABULARY_ID_FOR_TERM_LABELS, true);
    }

    @Override
    public void update(ViewerCell cell) {
        Object element = cell.getElement();
//        boolean showIdInVoc = PreferencesUtil.getBooleanValue(IPreferenceKeys.SHOW_VOCABULARY_ID_FOR_TERM_LABELS, true);
        String text = getText(element);
        cell.setText(text);

        Styler styler = null;
        if (element instanceof TermVocabularyDto && text != null) {
            styler = getVocabularyStyler();
        }
        else if(element instanceof TermDto && ((TermDto) element).getKindOfUuid()!=null){
            styler = getKindOfStyler();
        }
        if(styler!=null){
            StyledString styledString = new StyledString(text, styler);
            StyleRange[] styleRanges;
            styleRanges = styledString.getStyleRanges();
            cell.setStyleRanges(styleRanges);
        }
        super.update(cell);
    }

    @Override
    public String getText(Object element) {
        String label = null;
        if(element instanceof AbstractTermDto){
            AbstractTermDto termDto = (AbstractTermDto)element;            
			Representation rep = termDto.getRepresentation(PreferencesUtil.getGlobalLanguage());			
			label = rep != null? rep.getLabel() :termDto.getTitleCache()  ;

            if(showIdInVoc && element instanceof TermDto && ((TermDto) termDto).getIdInVocabulary()!=null){
                label = CdmUtils.concat(" : ", ((TermDto) termDto).getIdInVocabulary(), label);
            }
            if (label == null){
            	label = termDto.getTitleCache();
            }
        }
        // TODO add fallback for label
        if(label==null){
        	label = element.toString();
        }
        return label;
    }

    protected Styler getVocabularyStyler() {
        if (vocStyler == null) {
            vocStyler = new Styler() {
                @Override
                public void applyStyles(TextStyle textStyle) {
                    textStyle.foreground = vocColor;
                }
            };
        }
        return vocStyler;
    }

    protected Styler getKindOfStyler() {
        if (kindOfStyler == null) {
            kindOfStyler = new Styler() {
                @Override
                public void applyStyles(TextStyle textStyle) {
                    textStyle.foreground = kindOfColor;
                }
            };
        }
        return kindOfStyler;
    }

    @Override
    public Image getImage(Object element) {

        return null;
    }

    @Override
    public StyledString getStyledText(Object element) {
        return new StyledString(getText(element), StyledString.QUALIFIER_STYLER);
    }



}
