/**
 * Copyright (C) 2009 EDIT
 * European Distributed Institute of Taxonomy
 * http://www.e-taxonomy.eu
 *
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */
package eu.etaxonomy.taxeditor.editor.definedterm.e4.handler;

import java.util.UUID;

import javax.inject.Named;

import org.eclipse.core.commands.operations.IUndoContext;
import org.eclipse.e4.core.di.annotations.CanExecute;
import org.eclipse.e4.core.di.annotations.Execute;
import org.eclipse.e4.ui.di.UISynchronize;
import org.eclipse.e4.ui.model.application.ui.basic.MPart;
import org.eclipse.e4.ui.model.application.ui.menu.MHandledMenuItem;
import org.eclipse.e4.ui.model.application.ui.menu.MMenuItem;
import org.eclipse.e4.ui.services.IServiceConstants;
import org.eclipse.jface.viewers.IStructuredSelection;

import eu.etaxonomy.cdm.model.common.Language;
import eu.etaxonomy.cdm.persistence.dto.AbstractTermDto;
import eu.etaxonomy.cdm.persistence.dto.TermDto;
import eu.etaxonomy.cdm.persistence.dto.TermVocabularyDto;
import eu.etaxonomy.taxeditor.editor.definedterm.e4.DefinedTermEditorE4;
import eu.etaxonomy.taxeditor.editor.definedterm.operation.DeleteTermBaseOperation;
import eu.etaxonomy.taxeditor.model.AbstractUtility;
import eu.etaxonomy.taxeditor.operation.AbstractPostOperation;
import eu.etaxonomy.taxeditor.store.StoreUtil;

/**
 *
 * @author pplitzner
 * @since Aug 22, 2017
 *
 */
public class DeleteTermBaseHandlerE4 {

    @Execute
    public void execute(@Named(IServiceConstants.ACTIVE_PART) MPart activePart,
            @Named(IServiceConstants.ACTIVE_SELECTION) IStructuredSelection selection, MMenuItem menuItem,
            UISynchronize sync) {
        DefinedTermEditorE4 termEditor = (DefinedTermEditorE4) activePart.getObject();
        if(StoreUtil.promptCheckIsDirty(termEditor)){
            return;
        }
        if(!StoreUtil.confirmDelete()){
            return;
        }

        String label = menuItem.getLocalizedLabel();
        IUndoContext undoContext = StoreUtil.getUndoContext();
        AbstractPostOperation operation =
                new DeleteTermBaseOperation(label,
                        undoContext,
                        (AbstractTermDto) selection.getFirstElement(),
                        termEditor.getDefinedTermEditorInput(),
                        termEditor);
        AbstractUtility.executeOperation(operation, sync);
    }

    @CanExecute
    public boolean canExecute(@Named(IServiceConstants.ACTIVE_SELECTION) IStructuredSelection selection,
            MHandledMenuItem menuItem){
        boolean canExecute = false;
        Object firstElement = selection.getFirstElement();
        canExecute = selection.size()==1
                && (firstElement instanceof AbstractTermDto);
        UUID vocabularyUuid = null;
        if(firstElement instanceof TermDto){
            vocabularyUuid = ((TermDto) firstElement).getVocabularyUuid();
        }else if(firstElement instanceof TermVocabularyDto){
            vocabularyUuid = ((TermVocabularyDto) firstElement).getUuid();
        }
        canExecute &= vocabularyUuid!=null && !Language.cdmVocabularyUuids().contains(vocabularyUuid);
        menuItem.setVisible(canExecute);
        return canExecute;
    }
}
