/**
 * Copyright (C) 2009 EDIT
 * European Distributed Institute of Taxonomy
 * http://www.e-taxonomy.eu
 *
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */
package eu.etaxonomy.taxeditor.editor.definedterm.e4.handler;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Set;

import javax.inject.Named;

import org.eclipse.core.commands.operations.IUndoContext;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.e4.core.di.annotations.CanExecute;
import org.eclipse.e4.core.di.annotations.Execute;
import org.eclipse.e4.ui.di.UISynchronize;
import org.eclipse.e4.ui.model.application.ui.basic.MPart;
import org.eclipse.e4.ui.model.application.ui.menu.MHandledMenuItem;
import org.eclipse.e4.ui.services.IServiceConstants;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.wizard.WizardDialog;
import org.eclipse.swt.widgets.Shell;

import eu.etaxonomy.cdm.model.common.Language;
import eu.etaxonomy.cdm.model.location.Country;
import eu.etaxonomy.cdm.persistence.dto.AbstractTermDto;
import eu.etaxonomy.cdm.persistence.dto.TermDto;
import eu.etaxonomy.taxeditor.editor.definedterm.e4.DefinedTermEditorE4;
import eu.etaxonomy.taxeditor.editor.definedterm.operation.MoveDefinedTermOperation;
import eu.etaxonomy.taxeditor.model.AbstractUtility;
import eu.etaxonomy.taxeditor.model.MessagingUtils;
import eu.etaxonomy.taxeditor.operation.AbstractPostOperation;
import eu.etaxonomy.taxeditor.store.StoreUtil;
import eu.etaxonomy.taxeditor.termtree.TermChooseWizard;

/**
 *
 * @author pplitzner
 * @date 06.03.2019
 *
 */
public class MoveDefinedTermHandler {

    @Execute
    public void execute(@Named(IServiceConstants.ACTIVE_PART) MPart activePart,
            @Named(IServiceConstants.ACTIVE_SELECTION) IStructuredSelection selection,
            @Named(IServiceConstants.ACTIVE_SHELL) Shell shell,
            MHandledMenuItem menuItem,
            UISynchronize sync) {
        DefinedTermEditorE4 termEditor = (DefinedTermEditorE4) activePart.getObject();
        if(StoreUtil.promptCheckIsDirty(termEditor)){
            return;
        }

        TermDto selectedTerm = (TermDto) selection.getFirstElement();

        TermChooseWizard wizard = new TermChooseWizard(selectedTerm);
        WizardDialog dialog = new WizardDialog(shell, wizard);

        List<AbstractTermDto> selectedTerms = new ArrayList<>();
        if (dialog.open() == IStatus.OK) {
            selectedTerms = wizard.getSelectedTerms();
        }else {
            return;
        }

        if(selectedTerms.size()!=1){
            MessagingUtils.warningDialog("Move not possible", this, "You have to select one single target term");
            return;
        }

        AbstractTermDto target = selectedTerms.get(0);
        String label = menuItem.getLocalizedLabel();
        IUndoContext undoContext = StoreUtil.getUndoContext();
        Set<TermDto> movedTerms = Collections.singleton(selectedTerm);
        AbstractPostOperation operation =
                new MoveDefinedTermOperation(label,
                        undoContext,
                        target,
                        movedTerms,
                        termEditor);
        AbstractUtility.executeOperation(operation, sync);

    }

    @CanExecute
    public boolean canExecute(@Named(IServiceConstants.ACTIVE_SELECTION) IStructuredSelection selection,
            MHandledMenuItem menuItem){
        boolean canExecute = false;
        Object firstElement = selection.getFirstElement();
        canExecute = selection.size()==1
                && firstElement instanceof TermDto
                // TODO temporary fix for #7933
                && !((TermDto) firstElement).getVocabularyUuid().equals(Country.uuidCountryVocabulary)
                && !Language.cdmVocabularyUuids().contains(((TermDto) firstElement).getVocabularyUuid());
        menuItem.setVisible(canExecute);
        return canExecute;
    }

}
