/**
 * Copyright (C) 2009 EDIT
 * European Distributed Institute of Taxonomy
 * http://www.e-taxonomy.eu
 *
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */
package eu.etaxonomy.taxeditor.editor.definedterm.e4.handler;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;
import java.util.UUID;
import java.util.stream.Collectors;

import org.eclipse.e4.core.di.annotations.Execute;
import org.eclipse.e4.ui.model.application.MApplication;
import org.eclipse.e4.ui.model.application.commands.MParameter;
import org.eclipse.e4.ui.model.application.ui.basic.MPart;
import org.eclipse.e4.ui.model.application.ui.basic.MPartStack;
import org.eclipse.e4.ui.model.application.ui.menu.MHandledMenuItem;
import org.eclipse.e4.ui.workbench.modeling.EModelService;
import org.eclipse.e4.ui.workbench.modeling.EPartService;
import org.eclipse.e4.ui.workbench.modeling.EPartService.PartState;
import org.eclipse.jface.viewers.IStructuredSelection;

import eu.etaxonomy.cdm.api.service.ITermService;
import eu.etaxonomy.cdm.model.common.ICdmBase;
import eu.etaxonomy.cdm.model.description.Feature;
import eu.etaxonomy.cdm.model.term.DefinedTermBase;
import eu.etaxonomy.cdm.model.term.TermType;
import eu.etaxonomy.cdm.persistence.dto.FeatureDto;
import eu.etaxonomy.cdm.persistence.dto.TermDto;
import eu.etaxonomy.cdm.persistence.dto.UuidAndTitleCache;
import eu.etaxonomy.taxeditor.editor.definedterm.e4.DefinedTermEditorE4;
import eu.etaxonomy.taxeditor.editor.definedterm.input.TermEditorInput;
import eu.etaxonomy.taxeditor.store.CdmStore;
import eu.etaxonomy.taxeditor.workbench.WorkbenchUtility;


/**
 * @author pplitzner
 * @since Aug 22, 2017
 */
public class OpenDefinedTermEditorHandlerE4 {

	@Execute
	public void execute(EPartService partService, MHandledMenuItem menuItem, MApplication application, EModelService modelService) {

	    String commandId = menuItem.getCommand().getElementId();
	    List<MParameter> parameters = menuItem.getParameters();
	    TermType termType = null;
	    for (MParameter param: parameters){
	        try{
	            termType = TermType.getByKey(param.getValue());
	        }catch(Exception e){

	        }
	    }


//
//		TermType termType = TermType.getByUuid(termTypeUuid);
	    if(termType != null){
    	    TermType type = termType;
            TermEditorInput termEditorInput = new TermEditorInput(termType);
    		List<MPart> alreadyOpenedEditors = partService.getParts().stream()
    		        .filter(part->part.getObject()!=null && part.getObject() instanceof DefinedTermEditorE4)
    		        .filter(part->((DefinedTermEditorE4)part.getObject()).getDefinedTermEditorInput().getTermType().equals(type))
    		        .collect(Collectors.toList());
    		if(!alreadyOpenedEditors.isEmpty()){
    		    //there should never be more than one already opened editor
    		    //so we just open the first
    		    partService.activate(alreadyOpenedEditors.iterator().next());
    		}
    		else{
    		    MPart part = partService.createPart(eu.etaxonomy.taxeditor.store.AppModelId.PARTDESCRIPTOR_EU_ETAXONOMY_TAXEDITOR_EDITOR_DEFINEDTERM);
    		    MPartStack editorAreaPartStack = WorkbenchUtility.getEditorAreaPartStack(application, modelService);
    		    if(editorAreaPartStack!=null){
    		        editorAreaPartStack.getChildren().add(part);
    		    }
    		    part = partService.showPart(part, PartState.ACTIVATE);
    		    DefinedTermEditorE4 termEditor = (DefinedTermEditorE4) part.getObject();
    		    termEditor.init(termEditorInput, menuItem.getLocalizedLabel());
    		}
	    }else{
	        List<UuidAndTitleCache<? extends ICdmBase>>  termUuidAndTitleCaches = (List<UuidAndTitleCache<? extends ICdmBase>> ) menuItem.getTransientData().get(commandId+".uuid");
	        Set<TermType> termTypes = new HashSet<>();
	        Set<UUID> termUuids = new HashSet<>();
	        for (UuidAndTitleCache<?> uuidAndTitleCache: termUuidAndTitleCaches){
	            termUuids.add(uuidAndTitleCache.getUuid());
	        }
	        List<DefinedTermBase> terms = CdmStore.getService(ITermService.class).load(new ArrayList<>(termUuids), null);

	        for (DefinedTermBase term: terms){
//	            TermType type = termType;
	            TermEditorInput termEditorInput = new TermEditorInput(term.getTermType());
	            List<MPart> alreadyOpenedEditors = partService.getParts().stream()
	                    .filter(part->part.getObject()!=null && part.getObject() instanceof DefinedTermEditorE4)
	                    .filter(part->((DefinedTermEditorE4)part.getObject()).getDefinedTermEditorInput().getTermType().equals(term.getTermType()))
	                    .collect(Collectors.toList());
	            if(!alreadyOpenedEditors.isEmpty()){
	                //there should never be more than one already opened editor
	                //so we just open the first
	                MPart openPart = alreadyOpenedEditors.iterator().next();
	                partService.activate(openPart);


	            }
	            else{
	                MPart part = partService.createPart(eu.etaxonomy.taxeditor.store.AppModelId.PARTDESCRIPTOR_EU_ETAXONOMY_TAXEDITOR_EDITOR_DEFINEDTERM);
	                MPartStack editorAreaPartStack = WorkbenchUtility.getEditorAreaPartStack(application, modelService);
	                if(editorAreaPartStack!=null){
	                    editorAreaPartStack.getChildren().add(part);
	                }
	                part = partService.showPart(part, PartState.ACTIVATE);
	                DefinedTermEditorE4 termEditor = (DefinedTermEditorE4) part.getObject();
	                termEditor.init(termEditorInput, menuItem.getLocalizedLabel());
	                TermDto dto = null;
	                FeatureDto featureDto = null;
	                if (term instanceof Feature){
	                    Feature feature = (Feature)term;
	                    featureDto = FeatureDto.fromFeature(feature);
	                }else{
	                    dto = TermDto.fromTerm(term);
	                }


	                IStructuredSelection sel = null;
	                if (featureDto != null){
	                    termEditor.setSelection(featureDto);
	                }else{
	                    termEditor.setSelection(dto);
	                }



	            }
	        }

	    }
	}
}