/**
* Copyright (C) 2009 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.editor.definedterm.operation;

import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.commands.operations.IUndoContext;
import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;

import eu.etaxonomy.cdm.api.service.ITermService;
import eu.etaxonomy.cdm.api.service.IVocabularyService;
import eu.etaxonomy.cdm.model.term.DefinedTermBase;
import eu.etaxonomy.cdm.persistence.dto.AbstractTermDto;
import eu.etaxonomy.cdm.persistence.dto.TermDto;
import eu.etaxonomy.cdm.persistence.dto.TermVocabularyDto;
import eu.etaxonomy.taxeditor.editor.definedterm.input.TermEditorInput;
import eu.etaxonomy.taxeditor.operation.AbstractPostOperation;
import eu.etaxonomy.taxeditor.operation.IPostOperationEnabled;
import eu.etaxonomy.taxeditor.preference.PreferencesUtil;
import eu.etaxonomy.taxeditor.store.CdmStore;

/**
 * @author l.morris
 * @date 21 Dec 2011
 */
public class CreateDefinedTermOperation extends AbstractPostOperation<DefinedTermBase> {

	private final AbstractTermDto parent;
	private final TermEditorInput definedTermInput;
    private boolean addTermAsKindOf;


	public CreateDefinedTermOperation(String label,
			IUndoContext undoContext,
			AbstractTermDto parent,
			TermEditorInput definedTermInput,
			IPostOperationEnabled postOperationEnabled, boolean addTermAsKindOf) {
		super(label, undoContext, null, postOperationEnabled);
		this.parent = parent;
		this.definedTermInput = definedTermInput;
		this.addTermAsKindOf = addTermAsKindOf;
	}

	@Override
	public IStatus execute(IProgressMonitor monitor, IAdaptable info)
			throws ExecutionException {
	    TermDto newTerm = null;
	    if(parent instanceof TermDto){
	        newTerm = CdmStore.getService(ITermService.class).addNewTerm(definedTermInput.getTermType(), parent.getUuid(), addTermAsKindOf, PreferencesUtil.getGlobalLanguage());
	    }
	    else if(parent instanceof TermVocabularyDto){
	        newTerm = CdmStore.getService(IVocabularyService.class).addNewTerm(definedTermInput.getTermType(), parent.getUuid(), PreferencesUtil.getGlobalLanguage());
	    }
//	    newTerm.setTitleCache("_" + newTerm.getTitleCache());
		return postExecute(newTerm);
	}

	@Override
	public IStatus redo(IProgressMonitor monitor, IAdaptable info)
			throws ExecutionException {
		return null;
	}

	@Override
	public IStatus undo(IProgressMonitor monitor, IAdaptable info)
			throws ExecutionException {
		return null;
	}

}
