/**
* Copyright (C) 2009 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.editor.definedterm.operation;

import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.commands.operations.IUndoContext;
import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;

import eu.etaxonomy.cdm.api.service.IVocabularyService;
import eu.etaxonomy.cdm.model.term.DefinedTermBase;
import eu.etaxonomy.cdm.model.term.OrderedTermBase;
import eu.etaxonomy.cdm.model.term.OrderedTermVocabulary;
import eu.etaxonomy.cdm.model.term.TermType;
import eu.etaxonomy.cdm.model.term.TermVocabulary;
import eu.etaxonomy.cdm.persistence.dto.TermVocabularyDto;
import eu.etaxonomy.taxeditor.editor.definedterm.input.TermEditorInput;
import eu.etaxonomy.taxeditor.operation.AbstractPostTaxonOperation;
import eu.etaxonomy.taxeditor.operation.IPostOperationEnabled;
import eu.etaxonomy.taxeditor.preference.PreferencesUtil;
import eu.etaxonomy.taxeditor.store.CdmStore;

/**
 * @author l.morris
 * @date 21 Dec 2011
 *
 */
public class CreateTermVocabularyOperation extends AbstractPostTaxonOperation {

	private final TermEditorInput definedEditorInput;

	public CreateTermVocabularyOperation(String label,
			IUndoContext undoContext,
			TermEditorInput definedEditorInput,
			IPostOperationEnabled postOperationEnabled) {
		super(label, undoContext, postOperationEnabled);
		this.definedEditorInput = definedEditorInput;
	}

	@Override
	public IStatus execute(IProgressMonitor monitor, IAdaptable info)
			throws ExecutionException {

	   TermType type= definedEditorInput.getTermType();
	   DefinedTermBase test = type.getEmptyDefinedTermBase(PreferencesUtil.getGlobalLanguage());
	   TermVocabulary termVocabulary = null;
	   if (test instanceof OrderedTermBase){
	      termVocabulary = OrderedTermVocabulary.NewInstance(definedEditorInput.getTermType());
	      termVocabulary.setLabel("Untitled", PreferencesUtil.getGlobalLanguage());	      
	   }

	   if (termVocabulary == null){
	    termVocabulary =
                TermVocabulary.NewInstance(definedEditorInput.getTermType(),
                    null,null, 
                    "Untitled",
                    null,
                    null, PreferencesUtil.getGlobalLanguage());
	   }

		termVocabulary = CdmStore.getService(IVocabularyService.class).save(termVocabulary);
		//FIXME: implement creation of term vocabularies
//		definedEditorInput.getVocabularies().add(termVocabulary);

		return postExecute(new TermVocabularyDto(termVocabulary.getUuid(), termVocabulary.getRepresentations(), termVocabulary.getTermType(), termVocabulary.getTitleCache(), termVocabulary.isAllowDuplicates(), termVocabulary.isOrderRelevant(), termVocabulary.isFlat()));
	}

	@Override
	public IStatus redo(IProgressMonitor monitor, IAdaptable info)
			throws ExecutionException {
		return null;
	}

	@Override
	public IStatus undo(IProgressMonitor monitor, IAdaptable info)
			throws ExecutionException {
		return null;
	}

}
