/**
* Copyright (C) 2009 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.editor.definedterm.operation;

import java.util.Collection;

import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.commands.operations.IUndoContext;
import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;

import eu.etaxonomy.cdm.api.service.DeleteResult;
import eu.etaxonomy.cdm.api.service.ITermService;
import eu.etaxonomy.cdm.api.service.IVocabularyService;
import eu.etaxonomy.cdm.api.service.config.TermDeletionConfigurator;
import eu.etaxonomy.cdm.persistence.dto.AbstractTermDto;
import eu.etaxonomy.cdm.persistence.dto.TermDto;
import eu.etaxonomy.cdm.persistence.dto.TermVocabularyDto;
import eu.etaxonomy.taxeditor.editor.definedterm.input.TermEditorInput;
import eu.etaxonomy.taxeditor.l10n.Messages;
import eu.etaxonomy.taxeditor.model.MessagingUtils;
import eu.etaxonomy.taxeditor.operation.AbstractPostTaxonOperation;
import eu.etaxonomy.taxeditor.operation.IPostOperationEnabled;
import eu.etaxonomy.taxeditor.store.CdmStore;
import eu.etaxonomy.taxeditor.store.StoreUtil;
import eu.etaxonomy.taxeditor.store.internal.TaxeditorStorePlugin;

/**
 * @author l.morris
 * @date 22 Dec 2011
 *
 */
public class DeleteTermBaseOperation extends AbstractPostTaxonOperation {

	private final TermEditorInput definedEditorInput;
	private final AbstractTermDto termBase;

	public DeleteTermBaseOperation(String label,
			IUndoContext undoContext,
			AbstractTermDto termBase,
			TermEditorInput definedEditorInput,
			IPostOperationEnabled postOperationEnabled) {
		super(label, undoContext, postOperationEnabled);
		this.termBase = termBase;
		this.definedEditorInput = definedEditorInput;
	}

	@Override
	public IStatus execute(IProgressMonitor monitor, IAdaptable info)
			throws ExecutionException {
		if (termBase instanceof TermVocabularyDto) {
			definedEditorInput.getVocabularies().remove(termBase);

			// FIXME this is just a temporary fix for #6952
			Collection<TermDto> topLevelTerms = CdmStore.getService(IVocabularyService.class).getTopLevelTerms(termBase.getUuid());
			if(!topLevelTerms.isEmpty()){
			    MessagingUtils.warningDialog("Deletion not possible", this, "Deletion not possible. Vocabulary is not empty.\nPlease delete or move the child terms first");
			    return Status.CANCEL_STATUS;
			}

			DeleteResult result =	CdmStore.getService(IVocabularyService.class).delete(termBase.getUuid());
			if (!result.isOk()){
			    return showErrorMessage(result);
			}

		} else if (termBase instanceof TermDto) {
			DeleteResult result =	CdmStore.getService(ITermService.class).delete(termBase.getUuid(),
			        new TermDeletionConfigurator());
			if (!result.isOk()){
			    return showErrorMessage(result);
            }
		}
		return postExecute(termBase);
	}

	private IStatus showErrorMessage(DeleteResult result) {
	    StringBuffer errorString = new StringBuffer();
        for (Exception e:result.getExceptions()){
            errorString.append(e.getMessage() + System.lineSeparator());
        }
        IStatus status = new Status(IStatus.CANCEL, StoreUtil.getPluginId(), errorString.toString());
        String delete_failed = Messages.DeleteTermBaseOperation_DELETE_FAILED;
        if (result.isAbort()){
            MessagingUtils.warningDialog(delete_failed, this, status);
        }else{
            if (result.getExceptions().iterator().hasNext()){
                MessagingUtils.errorDialog(delete_failed, this.getClass(),null, TaxeditorStorePlugin.PLUGIN_ID, result.getExceptions().iterator().next(), true);
            }else{
                MessagingUtils.warningDialog(delete_failed, this, status);
            }
        }
        return status;
    }

    @Override
	public IStatus redo(IProgressMonitor monitor, IAdaptable info)
			throws ExecutionException {
		return null;
	}

	@Override
	public IStatus undo(IProgressMonitor monitor, IAdaptable info)
			throws ExecutionException {
		return null;
	}

}
