/**
* Copyright (C) 2009 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.editor.definedterm.operation;

import java.util.Collection;
import java.util.HashSet;
import java.util.UUID;

import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.commands.operations.IUndoContext;
import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.jface.viewers.ViewerDropAdapter;

import eu.etaxonomy.cdm.api.service.ITermService;
import eu.etaxonomy.cdm.api.service.TermServiceImpl.TermMovePosition;
import eu.etaxonomy.cdm.api.service.UpdateResult;
import eu.etaxonomy.cdm.model.common.CdmBase;
import eu.etaxonomy.cdm.model.term.DefinedTermBase;
import eu.etaxonomy.cdm.model.term.TermVocabulary;
import eu.etaxonomy.cdm.persistence.dto.AbstractTermDto;
import eu.etaxonomy.cdm.persistence.dto.TermDto;
import eu.etaxonomy.cdm.persistence.dto.TermVocabularyDto;
import eu.etaxonomy.taxeditor.operation.AbstractPostTaxonOperation;
import eu.etaxonomy.taxeditor.operation.IPostOperationEnabled;
import eu.etaxonomy.taxeditor.store.CdmStore;

/**
 * @author l.morris
 * @date 10 Jan 2012
 *
 */
public class MoveDefinedTermOperation extends AbstractPostTaxonOperation {

	private final Collection<TermDto> sourceTerms;// the actual DefinedTermBase(s) we are moving
	private final Collection<TermVocabularyDto> sourceVocabularies;
	private final AbstractTermDto targetTermOrVocabulary;// the target VOCABULARY or DefinedTerm we are moving these to
	private final int currentLocation;

	public MoveDefinedTermOperation(String label,
	        IUndoContext undoContext,
	        AbstractTermDto target,
	        Collection<TermDto> sourceTerms,
	        IPostOperationEnabled postOperationEnabled) {
	    this(label, undoContext, target, sourceTerms, postOperationEnabled, ViewerDropAdapter.LOCATION_ON);
	}
	public MoveDefinedTermOperation(String label,
	        IUndoContext undoContext,
	        AbstractTermDto target,
	        Collection<TermDto> sourceTerms,
			IPostOperationEnabled postOperationEnabled,
			int currentLocation) {
		super(label, undoContext, postOperationEnabled);

		this.targetTermOrVocabulary = target;
		this.sourceTerms = sourceTerms;
		this.currentLocation = currentLocation;
		sourceVocabularies = new HashSet<>();
	}

	@Override
	public IStatus execute(IProgressMonitor monitor, IAdaptable info)
			throws ExecutionException {
 	    UpdateResult result = new UpdateResult();
	    for(TermDto term:sourceTerms){
	        sourceVocabularies.add(term.getVocabularyDto());
            UUID parentUuid = targetTermOrVocabulary.getUuid();
            TermMovePosition termMovePosition = TermMovePosition.ON;
            if(currentLocation == ViewerDropAdapter.LOCATION_BEFORE) {
                termMovePosition = TermMovePosition.BEFORE;
            }
            else if(currentLocation == ViewerDropAdapter.LOCATION_AFTER) {
                termMovePosition = TermMovePosition.AFTER;
            }
            result.includeResult(CdmStore.getService(ITermService.class).moveTerm(term, parentUuid, termMovePosition));
        }
	    Collection<CdmBase> affectedObjects = result.getUpdatedObjects();
	    Collection<AbstractTermDto> affectedDtos = new HashSet();
	    for (CdmBase base: affectedObjects){
	        if (base instanceof TermVocabulary){
	            TermVocabularyDto dto = new TermVocabularyDto(base.getUuid(), ((TermVocabulary)base).getRepresentations(), ((TermVocabulary)base).getTermType(), ((TermVocabulary)base).getTitleCache(), ((TermVocabulary)base).isAllowDuplicates(), ((TermVocabulary)base).isOrderRelevant(), ((TermVocabulary)base).isFlat());
	            affectedDtos.add(dto);
	        }
	        if (base instanceof DefinedTermBase){
	            TermDto dto = TermDto.fromTerm((DefinedTermBase)base);
                affectedDtos.add(dto);
	        }
	    }


		return postExecute(affectedDtos);
	}

	@Override
	public IStatus redo(IProgressMonitor monitor, IAdaptable info)
			throws ExecutionException {
		return null;
	}

	@Override
	public IStatus undo(IProgressMonitor monitor, IAdaptable info)
			throws ExecutionException {
		return null;
	}

}
