/**
* Copyright (C) 2017 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.event;

import javax.annotation.PostConstruct;
import javax.inject.Inject;
import javax.inject.Named;

import org.eclipse.e4.core.commands.ECommandService;
import org.eclipse.e4.core.di.annotations.Optional;
import org.eclipse.e4.core.services.events.IEventBroker;
import org.eclipse.e4.ui.di.UIEventTopic;
import org.eclipse.e4.ui.di.UISynchronize;
import org.eclipse.e4.ui.model.application.ui.basic.MPart;
import org.eclipse.e4.ui.services.IServiceConstants;
import org.eclipse.swt.widgets.Shell;

import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.taxeditor.editor.ITaxonEditor;

/**
 * @author pplitzner
 * @since Sep 4, 2017
 */
public class EventUtility {

    private static ITaxonEditor editor;

    private static MPart activePart;

    private static Shell shell;

    @Inject
    private static UISynchronize sync;

    @Inject
    private static IEventBroker eventBroker;

    @Inject
    private static ECommandService commandService;

    @PostConstruct
    public void create(){
        //nothing
    }

    public static void postEvent(String topic, Object data){
        eventBroker.post(topic, data);
    }

    public static void postSyncEvent(String topic, Object data){
        sync.syncExec(()->eventBroker.post(topic, data));
    }

    public static void postAsyncEvent(String topic, Object data){
        sync.asyncExec(()->eventBroker.post(topic, data));
    }

    public static ECommandService getCommandService() {
        return commandService;
    }

    @Inject
    @Optional
    private void updateCurrentActiveShell(@Named(IServiceConstants.ACTIVE_SHELL)Shell shell){
        EventUtility.shell = shell;
    }

    @Inject
    @Optional
    private void updateCurrentActivePart(@Named(IServiceConstants.ACTIVE_PART)MPart activePart){
        if(activePart!=null){
            EventUtility.activePart = activePart;
        }
    }

    @Inject
    @Optional
    private void updateCurrentTaxon(@UIEventTopic(WorkbenchEventConstants.CURRENT_ACTIVE_EDITOR)ITaxonEditor editor){
        EventUtility.editor = editor;
    }

    public static Taxon getCurrentTaxon() {
        if(editor!=null){
            return editor.getTaxon();
        }
        return null;
    }

    public static ITaxonEditor getTaxonEditor() {
        return editor;
    }

    public static MPart getActivePart() {
        return activePart;
    }

    public static Shell getShell() {
        return shell;
    }

//    private EventHandler testHandler;
//
//    @Inject
//    public EventUtility(IEventBroker eventBroker) {
//        testHandler = new EventHandler() {
//
//            @Override
//            public void handleEvent(Event event) {
//                Object part = event.getProperty(UIEvents.EventTags.ELEMENT);
//                boolean tbr =(Boolean) event.getProperty(UIEvents.EventTags.NEW_VALUE);
//                if (part instanceof MPart){
//                    System.out.println("Part "+((MPart)part).getElementId()+" is "+(!tbr?"NOT":"")+" visible");
//                }
//                else if(part instanceof MPlaceholder){
//                    System.out.println("Part "+((MPlaceholder)part).getRef().getElementId()+" is "+(!tbr?"NOT":"")+" visible");
//                }
//            }
//        };
//        eventBroker.subscribe(UIEvents.UIElement.TOPIC_TOBERENDERED, testHandler);
//    }

}
