package eu.etaxonomy.taxeditor.handler.defaultHandler;

import java.util.List;

import javax.inject.Named;

import org.eclipse.e4.core.di.annotations.CanExecute;
import org.eclipse.e4.core.di.annotations.Execute;
import org.eclipse.e4.ui.model.application.ui.basic.MPart;
import org.eclipse.e4.ui.model.application.ui.menu.MHandledMenuItem;
import org.eclipse.e4.ui.services.IServiceConstants;
import org.eclipse.e4.ui.workbench.modeling.EPartService;
import org.eclipse.e4.ui.workbench.modeling.EPartService.PartState;

import eu.etaxonomy.cdm.model.common.ICdmBase;
import eu.etaxonomy.cdm.persistence.dto.UuidAndTitleCache;
import eu.etaxonomy.taxeditor.bulkeditor.AppModelId;
import eu.etaxonomy.taxeditor.editor.IReferencingObjectsView;
import eu.etaxonomy.taxeditor.view.CdmViewerUtil;

public class OpenReferencingObjectsViewHandler {

    private List<UuidAndTitleCache<? extends ICdmBase>> selectedObjectList;

    @Execute
    public void execute(EPartService partService) {

        MPart part = partService.createPart(AppModelId.PARTDESCRIPTOR_EU_ETAXONOMY_TAXEDITOR_BULKEDITOR_REFERENCINGOBJECTS_E4_REFERENCINGOBJECTSVIEWE4);
        part = partService.showPart(part, PartState.ACTIVATE);
        IReferencingObjectsView view = (IReferencingObjectsView)part.getObject();

        if (selectedObjectList != null && !selectedObjectList.isEmpty()) {
            UuidAndTitleCache<? extends ICdmBase> dto = selectedObjectList.get(0);
            view.handleNewSelection(dto);
        }
    }

    @CanExecute
    public boolean canExecute(EPartService partService,
            @Named(IServiceConstants.ACTIVE_PART)MPart activePart,
            MHandledMenuItem menuItem) {

        boolean canExecute = false;
        MPart part = partService.findPart(AppModelId.PARTDESCRIPTOR_EU_ETAXONOMY_TAXEDITOR_BULKEDITOR_REFERENCINGOBJECTS_E4_REFERENCINGOBJECTSVIEWE4);
        canExecute = part.equals(activePart)  //we explicitly want to allow calling the view recursively
                || part.getObject() == null ;  //we show the menu item only if the view is not yet open (otherwise by setting the focus data will be shown anyway
                                               //but this may change in future for some data where explicitly calling the view is required

        selectedObjectList = null;
        if (canExecute){
            String commandId = menuItem.getCommand().getElementId();
            Object selectedObject = menuItem.getTransientData().get(commandId+".uuid");
            selectedObjectList = CdmViewerUtil.transformSelectionToUuidAndTitleCacheList(selectedObject);
            canExecute = selectedObjectList.size() == 1;
        }
        menuItem.setVisible(canExecute);
        return canExecute;
    }
}