/**
* Copyright (C) 2020 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.handler.defaultHandler.e4;

import org.eclipse.e4.ui.workbench.modeling.EPartService;
import org.eclipse.jface.wizard.WizardDialog;
import org.eclipse.swt.widgets.Shell;

import eu.etaxonomy.cdm.api.service.IClassificationService;
import eu.etaxonomy.cdm.model.taxon.ITaxonTreeNode;
import eu.etaxonomy.cdm.persistence.dto.TaxonNodeDto;
import eu.etaxonomy.taxeditor.newWizard.NewClassificationWizard;
import eu.etaxonomy.taxeditor.store.CdmStore;

/**
 * Handler to open Classification Wizard from taxon navigator
 */
public class DefaultOpenClassificationWizardHandlerE4 extends DefaultOpenHandlerBaseE4<ITaxonTreeNode, TaxonNodeDto> {

    @Override
    protected void open(TaxonNodeDto entity, Shell shell, EPartService partService) {
        NewClassificationWizard classificationWizard = new NewClassificationWizard();
        classificationWizard.init(null, null);
        classificationWizard.setEntity(CdmStore.getService(IClassificationService.class).load(entity.getClassificationUUID()));
        WizardDialog dialog = new WizardDialog(shell, classificationWizard);
        dialog.open();
    }

    @Override
    protected boolean canExecute(TaxonNodeDto selection) {
        return selection.getTaxonUuid() == null;  //only root notes have no taxon
    }
}