/**
* Copyright (C) 2020 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.handler.defaultHandler.e4;

import java.util.List;

import javax.inject.Named;

import org.eclipse.e4.core.contexts.IEclipseContext;
import org.eclipse.e4.core.di.annotations.CanExecute;
import org.eclipse.e4.core.di.annotations.Execute;
import org.eclipse.e4.ui.model.application.MApplication;
import org.eclipse.e4.ui.model.application.ui.basic.MPart;
import org.eclipse.e4.ui.model.application.ui.menu.MHandledMenuItem;
import org.eclipse.e4.ui.services.IServiceConstants;
import org.eclipse.e4.ui.workbench.modeling.EModelService;
import org.eclipse.e4.ui.workbench.modeling.EPartService;
import org.eclipse.swt.widgets.Shell;

import eu.etaxonomy.cdm.model.common.ICdmBase;
import eu.etaxonomy.cdm.persistence.dto.UuidAndTitleCache;
import eu.etaxonomy.taxeditor.view.CdmViewerUtil;

public abstract class DefaultOpenHandlerBaseE4<S extends ICdmBase, T extends UuidAndTitleCache<S>> {

    protected IEclipseContext context;
    protected MApplication application;
    protected EModelService modelService;

    @Execute
    public void execute(@Named(IServiceConstants.ACTIVE_SHELL)Shell shell, MHandledMenuItem menuItem,
            EModelService modelService, EPartService partService, MApplication application,
            IEclipseContext context) throws Exception {

        this.context = context;
        this.modelService = modelService;
        this.application = application;

        String commandId = menuItem.getCommand().getElementId();
        Object transientData = menuItem.getTransientData().get(commandId+".uuid");

        //FIXME DTO
        List<T> uuidList = (List<T>)CdmViewerUtil.transformSelectionToUuidAndTitleCacheList(transientData);

        handleUuidList(shell, partService, uuidList);
    }

    protected void handleUuidList(Shell shell, EPartService partService, List<T> uuidList) throws Exception {
        if (this.allowsMultiple()){
            for (T uuid : uuidList){
                open(uuid, shell, partService);
            }
        }else if (!uuidList.isEmpty()){
            open(uuidList.get(0), shell, partService);
        }
    }

    @CanExecute
    public boolean canExecute(MHandledMenuItem menuItem,
            @Named(IServiceConstants.ACTIVE_PART) MPart activePart) {

        //check if same part
        String partId = getPartId();
        //check for correct entity
        String commandId = menuItem.getCommand().getElementId();
        Object transientData = menuItem.getTransientData().get(commandId+".uuid");

        //FIXME DTO casting
        List<T> uuidList = (List<T>)CdmViewerUtil.transformSelectionToUuidAndTitleCacheList(transientData);
        boolean canExecute = partId!=null? !partId.equals(activePart.getElementId()):true;
        canExecute &= canExecuteHandleList(uuidList);

        menuItem.setVisible(canExecute);
        return canExecute;
    }

    public boolean canExecuteHandleList(List<T> uuidList) {
        boolean canExecute = false;
        for (T uuid : uuidList){
            canExecute |= canExecute(uuid);
        }
        return canExecute;
    }


    protected abstract void open(T entity, Shell shell, EPartService partService) throws Exception;

    protected abstract boolean canExecute(T entity);

    /**
     * subclasses should override this if it is allowed to open multiple editors in parallel
     */
    protected boolean allowsMultiple(){
        return false;
    }

    /**
     * Returns the part id for the part this handler opens.<br>
     */
    protected String getPartId(){
        return null;
    }
}
