package eu.etaxonomy.taxeditor.handler.update;

import java.net.URI;
import java.net.URISyntaxException;
import java.util.ArrayList;
import java.util.List;

import org.eclipse.equinox.internal.p2.ui.ProvUI;
import org.eclipse.equinox.internal.p2.ui.model.ElementUtils;
import org.eclipse.equinox.internal.p2.ui.model.MetadataRepositoryElement;
import org.eclipse.equinox.p2.repository.IRepository;
import org.eclipse.equinox.p2.repository.IRepositoryManager;
import org.eclipse.equinox.p2.repository.metadata.IMetadataRepositoryManager;
import org.eclipse.equinox.p2.ui.ProvisioningUI;

import eu.etaxonomy.taxeditor.model.MessagingUtils;
import eu.etaxonomy.taxeditor.preference.PreferencesUtil;
import eu.etaxonomy.taxeditor.util.ApplicationUtil;

/**
 * This class is a utility class for updating the editor from a p2 update site,
 * greatly inspired by the links given below.
 *
 * To Do :
 * - Allow configurable update sites
 *
 * Notes :
 *
 * @see http://wiki.eclipse.org/Equinox/p2/Adding_Self-Update_to_an_RCP_Application
 * @see http://bugs.eclipse.org/281226
 * @see http://www.vogella.com/tutorials/EclipseP2Update/article.html
 */
public class P2Util {

    private static String EDIT_NIGHTLY_UPDATE_SITE = "https://cybertaxonomy.eu/download/taxeditor/update/nightly/";
    private static String EDIT_NIGHTLY_UPDATE_SITE_NAME = "Taxonomic Editor Nightly";

    private static String EDIT_STABLE_UPDATE_SITE = "https://cybertaxonomy.eu/download/taxeditor/update/stable/";
    private static String EDIT_STABLE_UPDATE_SITE_NAME = "Taxonomic Editor Stable";

    /**
     * Retrieve and load the saved list of repositories from the preference store,
     * making sure that at least the default repository is always loaded.
     */
    @SuppressWarnings("restriction")
    public static void setP2UpdateRepositories() {
        List<MetadataRepositoryElement> repoElements = new ArrayList<>();
        List<MetadataRepositoryElement> savedRepoElements = new ArrayList<>();

        MetadataRepositoryElement mre = new MetadataRepositoryElement(null, getP2UpdateRepository(), true);
        mre.setNickname(getP2UpdateRepositoryName());
        savedRepoElements.add(mre);

        repoElements.addAll(savedRepoElements);

        ElementUtils.updateRepositoryUsingElements(ProvisioningUI.getDefaultUI(),repoElements
                .toArray(new MetadataRepositoryElement[]{} ));
    }

    public static String getP2UpdateRepositoryName(){
        return ApplicationUtil.isStable()? EDIT_STABLE_UPDATE_SITE_NAME : EDIT_NIGHTLY_UPDATE_SITE_NAME;
    }

    public static URI getP2UpdateRepository(){
        try {
            return ApplicationUtil.isStable()? new URI(EDIT_STABLE_UPDATE_SITE) : new URI(EDIT_NIGHTLY_UPDATE_SITE);
        } catch (URISyntaxException e) {
            MessagingUtils.error(P2Util.class, e);
        }
        return null;
    }

    /**
     * {@link org.eclipse.equinox.p2.ui.RepositoryManipulationPage} which handles the repsitory site list
     * in preferences does not create a preference store and hence the changes are not saved. This means
     * that we need to save it ourselves.
     *
     * This method saves the list of current repositories in the preference store as a string with
     * specific delimiters.
     */

    public static void saveP2RepositoryPreferences() {

        IMetadataRepositoryManager metaManager = ProvUI.getMetadataRepositoryManager(ProvisioningUI.getDefaultUI().getSession());

        URI[] currentlyEnabled = metaManager.getKnownRepositories(IRepositoryManager.REPOSITORIES_ALL);
        URI[] currentlyDisabled = metaManager.getKnownRepositories(IRepositoryManager.REPOSITORIES_DISABLED);

        List<MetadataRepositoryElement> repoElements = new ArrayList<>();

        for(URI repo : currentlyEnabled) {
            boolean enabled = true;
            String nickname = metaManager.getRepositoryProperty(repo, IRepository.PROP_NICKNAME);
            MetadataRepositoryElement element = new MetadataRepositoryElement(null, repo, true);
            element.setNickname(nickname);
            element.setEnabled(enabled);
            repoElements.add(element);
        }

        for(URI repo : currentlyDisabled) {
            boolean enabled = false;
            String nickname = metaManager.getRepositoryProperty(repo, IRepository.PROP_NICKNAME);
            MetadataRepositoryElement element = new MetadataRepositoryElement(null, repo, true);
            element.setNickname(nickname);
            element.setEnabled(enabled);
            repoElements.add(element);
        }
        PreferencesUtil.setP2Repositories(repoElements);
    }
}
