/**
 * Copyright (C) 2007 EDIT
 * European Distributed Institute of Taxonomy
 * http://www.e-taxonomy.eu
 *
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */

package eu.etaxonomy.taxeditor.io;

import java.util.List;

import org.apache.commons.lang3.StringUtils;
import org.eclipse.core.runtime.jobs.Job;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.progress.IProgressConstants;

import eu.etaxonomy.cdm.api.application.ICdmRepository;
import eu.etaxonomy.cdm.common.IoResultBase;
import eu.etaxonomy.cdm.io.common.ExportResult;
import eu.etaxonomy.cdm.io.common.IIoConfigurator;
import eu.etaxonomy.cdm.io.common.ImportResult;
import eu.etaxonomy.taxeditor.store.CdmStore;
import eu.etaxonomy.taxeditor.ui.dialog.ReportTextDialog;

/**
 * <p>
 * Abstract AbstractIOHandler class.
 * </p>
 *
 * @author n.hoffmann
 * @created Sep 11, 2009
 * @version 1.0
 */
public abstract class AbstractIOManager<CONFIGURATOR extends IIoConfigurator> {

	protected ICdmRepository applicationConfiguration;

	public static enum TYPE {
		Jaxb, Tcs, Excel_Taxa, Endnote, Sdd, Abcd, SpecimenCdmExcel, Excel_Name, SpecimenSearch, Gbif, Excel_Distribution, Ris, OWL
	}

	/**
	 * <p>
	 * Constructor for AbstractIOHandler.
	 * </p>
	 *
	 * @param applicationController
	 *            a
	 *            {@link eu.etaxonomy.cdm.api.application.CdmApplicationController}
	 *            object.
	 * @param <CONFIGURATOR>
	 *            a CONFIGURATOR object.
	 */
	protected AbstractIOManager(
			ICdmRepository applicationConfiguration) {
		this.applicationConfiguration = applicationConfiguration;
	}

	/**
	 * Starts the IO process
	 *
	 * @param configurator
	 *            a CONFIGURATOR object.
	 */
	public void run(final CONFIGURATOR configurator) {
	    // create job
	    Job job = createIOJob(configurator);
	    run(job);
	}

	public void run(Job job) {
	    // configure the job
	    job.setProperty(IProgressConstants.KEEP_PROPERTY, true);
	    job.setUser(true);
	    // schedule job
	    job.schedule();
	}

	/**
	 * <p>
	 * createIOJob
	 * </p>
	 *
	 * @param configurator
	 *            a CONFIGURATOR object.
	 * @return a {@link org.eclipse.core.runtime.jobs.Job} object.
	 */
	protected abstract Job createIOJob(CONFIGURATOR configurator);

	 /**
     * @param configurator
     * @param display
     * @param importResult
     */
    protected void showResult(String importName, final Display display,
            IoResultBase ioResult, String successMessage) {
        StringBuffer reportTextTemp = ioResult.createReport();
        final StringBuffer reportText;
        if (StringUtils.isBlank(reportTextTemp.toString()) && ioResult instanceof ImportResult){
            reportTextTemp.append(successMessage);
            //TODO: this is a workaround because the abcd import writes the report in the report part...
        	ImportResult importResult = (ImportResult)ioResult;
            if (!importResult.getReports().isEmpty() && importResult.getReports().get(0) != null){
                reportTextTemp = new StringBuffer();
                if(importResult!=null){
                    List<byte[]> reports = importResult.getReports();
                    for (byte[] bs : reports) {
                        if (bs != null){ reportTextTemp.append(new String(bs));}
                    }
                }
            }
        }
        reportText = reportTextTemp;
        if (StringUtils.isBlank(reportText.toString()) && ioResult instanceof ExportResult ){ reportText.append("The Export was succesfull.");}
        display.asyncExec(new Runnable() {

            @Override
            public void run() {
                // display reports with possibility to save
                ReportTextDialog dialog = new ReportTextDialog(PlatformUI.getWorkbench().getActiveWorkbenchWindow().getShell());
                dialog.setTitle(importName+" Report");
                dialog.setReportText(reportText.toString());
                dialog.open();
                CdmStore.getContextManager().notifyContextRefresh();
            }
        });
    }

}
