/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.io.e4.in;

import org.eclipse.e4.core.contexts.ContextInjectionFactory;
import org.eclipse.e4.core.contexts.IEclipseContext;
import org.eclipse.e4.ui.workbench.modeling.EPartService;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.wizard.IWizardContainer;
import org.eclipse.jface.wizard.Wizard;
import org.eclipse.swt.SWT;

import eu.etaxonomy.cdm.io.common.IImportConfigurator;
import eu.etaxonomy.cdm.io.common.IIoConfigurator;
import eu.etaxonomy.taxeditor.l10n.Messages;

/**
 *
 * @author pplitzner
 * @since Oct 5, 2017
 *
 * @param <CONFIG>
 */
public abstract class AbstractImportWizardE4<CONFIG extends IIoConfigurator> extends Wizard {

	protected GenericConfiguratorWizardPageE4 pageConfiguration;
	private NomenclaturalCodeWizardPageE4 pageNomenclaturalCode;

	protected IStructuredSelection selection;

    protected IEclipseContext context;

    private EPartService partService;

    public AbstractImportWizardE4(IEclipseContext context, EPartService partService) {
        this(context, partService, null);
    }

    public AbstractImportWizardE4(IEclipseContext context, EPartService partService, IStructuredSelection selection) {
	    this.context = context;
	    this.partService = partService;
	    this.selection = selection;
    }

	/** {@inheritDoc} */
	@Override
	public void addPages() {
		super.addPages();

		addConfiguratorPage();

		pageNomenclaturalCode = ContextInjectionFactory.make(NomenclaturalCodeWizardPageE4.class, context);
		pageNomenclaturalCode.setConfigurator((IImportConfigurator)getConfigurator());
		this.addPage(pageNomenclaturalCode);
	}

	protected void addConfiguratorPage(){
		pageConfiguration = ContextInjectionFactory.make(GenericConfiguratorWizardPageE4.class, context);
		pageConfiguration.initImport(getConfigurator(), null, Messages.AbstractImportWizard_ConfigurationLabel);
		this.addPage(pageConfiguration);
	}

	@Override
	public void setContainer(IWizardContainer wizardContainer) {
		if(existUnsavedEditors() && wizardContainer!=null){
		    MessageDialog.open(MessageDialog.WARNING, getShell(), "Unsaved editors", "Please save all open editors before executing " +
		            "an import operation. The import operation will be cancelled", SWT.NONE);
		}
		else{
		    super.setContainer(wizardContainer);
		}
	}

	public abstract void init();

	protected boolean existUnsavedEditors(){
	    return !partService.getDirtyParts().isEmpty();
	}

	@Override
	public boolean canFinish() {
		return !existUnsavedEditors() && super.canFinish();
	}

	public abstract CONFIG getConfigurator();

	public void setSelection(IStructuredSelection selection) {
		this.selection = selection;
	}

	public IStructuredSelection getSelection() {
		return selection;
	}

}
