/**
* Copyright (C) 2017 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.io.e4.in;

import java.util.List;
import java.util.UUID;

import javax.inject.Inject;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;

import eu.etaxonomy.cdm.api.service.IVocabularyService;
import eu.etaxonomy.cdm.common.CdmUtils;
import eu.etaxonomy.cdm.model.term.DefinedTermBase;
import eu.etaxonomy.cdm.model.term.TermType;
import eu.etaxonomy.cdm.model.term.TermVocabulary;
import eu.etaxonomy.taxeditor.l10n.Messages;
import eu.etaxonomy.taxeditor.store.CdmStore;

/**
 * @author k.luther
 * @date 04.05.2017
 *
 */
public class ImportFromFileAndChooseVocIdWizardPageE4 extends ImportFromFileDataSourceWithReferenceWizardPage {

    private Combo vocabularyCombo;
    UUID vocUuid;
    String[][] labelAndValues;

    @Inject
    public ImportFromFileAndChooseVocIdWizardPageE4() {
        super();
        setTitle("Choose Excel File");
        setDescription("Please choose an xls file in the Distribution Update format.");
        setExtensions(new String[]{"*.xlsx", "*.xls", "*.*"});

    }

    @Override
    public void createControl(Composite parent) {
        super.createControl(parent);
        Composite composite = (Composite)getControl();
      //  final Composite composite = new Composite(parent, SWT.NULL);
        GridData gridData = new GridData(SWT.FILL, SWT.TOP, true, false);
        composite.setLayoutData(gridData);
        Label label = new Label(composite, SWT.NONE);
        label.setText(Messages.ImportFromFileAndChooseVocIdWizardPage_AreaVoc);
        label.setToolTipText(Messages.ImportFromFileAndChooseVocIdWizardOage_AreaVoc_toolTip);
        vocabularyCombo = new Combo(composite, SWT.DROP_DOWN);
        List<TermVocabulary<DefinedTermBase>> vocs = CdmStore.getService(IVocabularyService.class).findByTermType(TermType.NamedArea, null);
        for(TermVocabulary voc: vocs){
            vocabularyCombo.add(voc.getLabel());
        }
        getLabelAndValues(vocs);
        vocabularyCombo.select(0);
        setVocUuidFromVocCombo();
        vocabularyCombo.addSelectionListener(new SelectionAdapter(){

            @Override
            public void widgetSelected(SelectionEvent e) {
                String name = vocabularyCombo.getText();
                setVocUuidFromVocCombo();
                if (vocUuid != null){
                    setPageComplete(isPageComplete());
                }
            }

        });
    }

    /**
     * @return
     */
    private void setVocUuidFromVocCombo() {
        String name = vocabularyCombo.getText();

        for (String[] labelAndValue: labelAndValues){
            if (labelAndValue[0].equals(name)){
                String uuidString = labelAndValue[1];
                vocUuid = UUID.fromString(uuidString);
            }
        }

    }

    public UUID getVocUuid() {
        return vocUuid;
    }

    public void setVocUuid(UUID vocUuid) {
        this.vocUuid = vocUuid;
    }

    private String[][] getLabelAndValues(List<TermVocabulary<DefinedTermBase>> vocs) {

        labelAndValues = new String[vocs.size()][2];
        for (int i = 0; i < vocs.size(); i++) {
            labelAndValues[i][0] = vocs.get(i).getLabel();
            labelAndValues[i][1] = vocs.get(i).getUuid().toString();
        }
        return labelAndValues;
    }

    @Override
    public boolean isPageComplete() {
        boolean result = CdmUtils.isNotBlank(text_source.getText()) && vocUuid != null;

        return result;
    }

}
