/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.io.e4.in;

import java.io.File;

import javax.inject.Inject;

import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;

import eu.etaxonomy.cdm.common.URI;
import eu.etaxonomy.cdm.common.CdmUtils;

/**
 * @author pplitzner
 * @since Oct 5, 2017
 */
public class ImportFromFileDataSourceWizardPage extends WizardPage {

    public static final String PAGE_NAME = "ImportDataSourceWizardPage";

	private String[] extensions = {"*.xml"};

	private FileDialog fileDialog;

    protected Text text_source;

    protected Composite composite;

    @Inject
	public ImportFromFileDataSourceWizardPage() {
		super(PAGE_NAME);

		setTitle("Xml File");

		setDescription("Select XML file.");

		setExtensions(new String[]{"*.xml","*.*"});
	}

    public void setExtensions(String[] extensions) {
        this.extensions = extensions;
    }

	@Override
    public void createControl(Composite parent) {
		composite = new Composite(parent, SWT.NULL);

		setPageComplete(false);

		GridLayout gridLayout = new GridLayout();
		gridLayout.numColumns = 2;
		composite.setLayout(gridLayout);

		Label folderLabel = new Label(composite, SWT.NONE);
		folderLabel.setText("File");

		fileDialog = new FileDialog(parent.getShell());

		fileDialog.setFilterExtensions(extensions);
		new Label(composite, SWT.NONE);

		text_source = new Text(composite, SWT.BORDER);
		text_source.setEditable(false);
		text_source.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false));


		Button button = new Button(composite, SWT.PUSH);
		button.setText("Browse...");

		button.addSelectionListener(new SelectionAdapter(){

		    @Override
		    public void widgetSelected(SelectionEvent e) {
		        String path = fileDialog.open();
		        if(path!=null){
		            text_source.setText(path);
		            setPageComplete(isPageComplete());
		        }
		    }
		});

        setControl(composite);
	}

	public File getFile() {
		return new File(text_source.getText());
	}

    public URI getUri() {
		return URI.fromFile(getFile());
	}

	@Override
	public boolean isPageComplete() {
		return CdmUtils.isNotBlank(text_source.getText());
	}
}