/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.io.e4.in;

import javax.inject.Inject;

import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Group;

import eu.etaxonomy.cdm.io.common.IImportConfigurator;
import eu.etaxonomy.cdm.model.name.NomenclaturalCode;
import eu.etaxonomy.taxeditor.model.NomenclaturalCodeHelper;
import eu.etaxonomy.taxeditor.preference.PreferencesUtil;

/**
 *
 * @author p.ciardelli
 * @created 11.09.2009
 * @version 1.0
 */
public class NomenclaturalCodeWizardPageE4 extends WizardPage {

    private IImportConfigurator configurator;

    @Inject
	public NomenclaturalCodeWizardPageE4() {
		super("Choose nomenclatural code");

		this.setTitle("Choose nomenclatural code");

		this.setDescription("Choose which code to use for imported names.");
	}

    public void setConfigurator(IImportConfigurator configurator) {
        this.configurator = configurator;
    }

	/** {@inheritDoc} */
	@Override
    public void createControl(Composite parent) {
		Composite container = new Composite(parent, SWT.NULL);
		container.setLayout(new FillLayout());

		final Group group = new Group(container, SWT.NONE);
		group.setLayout(new GridLayout());

		NomenclaturalCode preferredCode = PreferencesUtil.getPreferredNomenclaturalCode();

		// set preferred code as default
		configurator.setNomenclaturalCode(preferredCode);

		for (final NomenclaturalCode code : NomenclaturalCodeHelper.getSupportedCodes()) {
			Button button = new Button(group, SWT.RADIO);
			button.setText(NomenclaturalCodeHelper.getDescription(code));
			button.setData(code);
			button.setSelection(preferredCode.equals(code));
			button.addSelectionListener(new SelectionAdapter() {
				@Override
                public void widgetSelected(SelectionEvent e) {
					configurator.setNomenclaturalCode((NomenclaturalCode) e.widget.getData());
				}
			});
		}
		setControl(container);
	}

}
