/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.io.e4.in.abcd;

import java.util.ArrayList;
import java.util.List;

import org.apache.commons.lang3.StringUtils;
import org.eclipse.jface.layout.GridDataFactory;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.FontData;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;
import org.eclipse.wb.swt.ResourceManager;

import eu.etaxonomy.cdm.common.URI;
import eu.etaxonomy.cdm.api.service.IClassificationService;
import eu.etaxonomy.cdm.api.service.IReferenceService;
import eu.etaxonomy.cdm.io.specimen.abcd206.in.Abcd206ImportConfigurator;
import eu.etaxonomy.cdm.model.metadata.PreferencePredicate;
import eu.etaxonomy.cdm.model.name.NomenclaturalCode;
import eu.etaxonomy.cdm.model.reference.Reference;
import eu.etaxonomy.cdm.model.taxon.Classification;
import eu.etaxonomy.taxeditor.databaseAdmin.wizard.AbstractPreferenceWizard;
import eu.etaxonomy.taxeditor.l10n.Messages;
import eu.etaxonomy.taxeditor.preference.IPreferenceKeys;
import eu.etaxonomy.taxeditor.preference.PreferencesUtil;
import eu.etaxonomy.taxeditor.store.CdmStore;
import eu.etaxonomy.taxeditor.ui.dialog.selection.SelectionDialogFactory;

/**
 * @author pplitzner
 */
public class AbcdImportConfiguratorWizardPage extends AbstractPreferenceWizard implements SelectionListener {

	public static final String PAGE_NAME = "AbcdImportConfiguratorWizardPage";

    private static final String ACCESSION_NUMBER = "Accession number";

    private static final String BARCODE = "Barcode";

    private static final String CATALOGUE_NUMBER = "Catalogue number";
    private Text textClassification;
    private Button btnBrowseClassification;
    private Button btnClear;

    private Text textSource;
    private Button btnBrowseSource;
    private Button btnClearSource;

	private Abcd206ImportConfigurator abcdImportConfigurator;

	private List<URI> biocaseProvider = new ArrayList<>();

	private boolean isFileImport = false;
	private boolean isPreferencePage = false;

    private Combo nomenclaturalCodeSelectionCombo;
    private Combo checkBoxMapUnitId;
    private Combo textDNAProviderString;

	protected AbcdImportConfiguratorWizardPage(String title, String description, Abcd206ImportConfigurator configurator) {
	    this(title, description, configurator, false, false);
    }

	protected AbcdImportConfiguratorWizardPage(String title, String description, Abcd206ImportConfigurator configurator, boolean isPreferencePage, boolean isFileImport) {
		super(PAGE_NAME);
		this.abcdImportConfigurator = configurator;
		setTitle(title);
		setDescription(description);
		this.isPreferencePage = isPreferencePage;
		this.isFileImport = isFileImport;
	}

	public static AbcdImportConfiguratorWizardPage createPage(Abcd206ImportConfigurator configurator){
		return new AbcdImportConfiguratorWizardPage("Configure import parameters",
				"Tooltips will explain parameters in more detail", configurator);
	}
	public static AbcdImportConfiguratorWizardPage createFileImportPage(Abcd206ImportConfigurator configurator){
        return new AbcdImportConfiguratorWizardPage("Configure import parameters",
                "Tooltips will explain parameters in more detail", configurator, false, true);
    }

	public static AbcdImportConfiguratorWizardPage createPreferencePage(Abcd206ImportConfigurator configurator){
        return new AbcdImportConfiguratorWizardPage("Configure import parameters",
                "Tooltips will explain parameters in more detail", configurator, true, false);
    }

	@Override
    public void createControl(Composite parent) {
	    Composite composite = new Composite(parent, SWT.NULL);
        GridLayout gridLayout = new GridLayout();
        gridLayout.numColumns=2;
        Label labelnewlyCreatedTaxa = new Label(composite, SWT.NONE);
        labelnewlyCreatedTaxa.setText("Newly Created Taxa");
        Display display = Display.getDefault();
        FontData data = display .getSystemFont().getFontData()[0];
        Font font = new Font(display, data.getName(), data.getHeight(), SWT.BOLD);
        labelnewlyCreatedTaxa.setFont(font);
        GridData gridData = new GridData();
        gridData.horizontalAlignment = GridData.FILL;
        gridData.grabExcessHorizontalSpace=true;
        gridData.horizontalSpan = 2;
        labelnewlyCreatedTaxa.setLayoutData(gridData);

        //gridLayout.makeColumnsEqualWidth=true;
        composite.setLayout(gridLayout);

		Composite classificationAndSourceComp = new Composite(composite, SWT.NULL);
		classificationAndSourceComp.setLayoutData(new GridData(SWT.FILL, SWT.TOP, false, false, 2, 1));
		classificationAndSourceComp.setLayout(new GridLayout(4, false));

		Label labelClassification = new Label(classificationAndSourceComp, SWT.TOP);
		labelClassification.setText("Classification");
//		labelClassification.setLayoutData(new GridData(SWT.LEFT, SWT.CENTER, false, false, 3, 1));
        textClassification = new Text(classificationAndSourceComp, SWT.BORDER);
        textClassification.setEnabled(false);
        textClassification.setLayoutData(new GridData(SWT.FILL, SWT.LEFT, true, true));
        if (abcdImportConfigurator.getClassificationUuid() != null){
            if (abcdImportConfigurator.getClassificationName() != null){
                textClassification.setText(abcdImportConfigurator.getClassificationName());
            }else{
                Classification classification = CdmStore.getService(IClassificationService.class).load(abcdImportConfigurator.getClassificationUuid());
                textClassification.setText(classification.getTitleCache());
            }
        }
        btnBrowseClassification = new Button(classificationAndSourceComp, SWT.NONE);
        btnBrowseClassification.setImage(ResourceManager.getPluginImage("eu.etaxonomy.taxeditor.store", "icons/open.gif"));
        btnBrowseClassification.addSelectionListener(new SelectionAdapter() {
            @Override
            public void widgetSelected(SelectionEvent e) {
                Classification classification = SelectionDialogFactory.getSelectionFromDialog(Classification.class,parent.getShell(), null, null);
                if(classification!=null){
                    textClassification.setText(classification.getTitleCache());
                    abcdImportConfigurator.setClassificationUuid(classification.getUuid());
                    abcdImportConfigurator.setClassificationName(classification.getTitleCache());
                }
            }
        });
        btnClear = new Button(classificationAndSourceComp, SWT.NONE);
        btnClear.setImage(ResourceManager.getPluginImage("eu.etaxonomy.taxeditor.store", "icons/trash.gif"));
        btnClear.addSelectionListener(new SelectionAdapter() {
            @Override
            public void widgetSelected(SelectionEvent e) {
                abcdImportConfigurator.setClassificationUuid(null);
                textClassification.setText("");
            }
        });

        Button checkBoxMoveToDefaultClassification = new Button(composite, SWT.CHECK);
        gridData = new GridData();
        gridData.horizontalAlignment = GridData.FILL_HORIZONTAL;
        gridData.horizontalSpan = 2;
        checkBoxMoveToDefaultClassification.setLayoutData(gridData);
        GridDataFactory.fillDefaults();
        checkBoxMoveToDefaultClassification.setSelection(abcdImportConfigurator.isMoveNewTaxaToDefaultClassification());
        checkBoxMoveToDefaultClassification.setText("Create new classification for new taxa");
        checkBoxMoveToDefaultClassification
        .setToolTipText("For taxa that do not exist in the data base "
                + "a new classification will be created");
        checkBoxMoveToDefaultClassification.addSelectionListener(new SelectionAdapter() {
            @Override
            public void widgetSelected(SelectionEvent e) {
                abcdImportConfigurator.setMoveNewTaxaToDefaultClassification(!abcdImportConfigurator.isMoveNewTaxaToDefaultClassification());
            }
        });

        Button checkBoxReuseExistingTaxa = new Button(composite, SWT.CHECK);
        gridData = new GridData();
        gridData.horizontalAlignment = GridData.FILL_HORIZONTAL;
        gridData.horizontalSpan = 2;
        checkBoxReuseExistingTaxa.setLayoutData(gridData);
        GridDataFactory.fillDefaults();
        checkBoxReuseExistingTaxa.setSelection(abcdImportConfigurator.isReuseExistingTaxaWhenPossible());
        checkBoxReuseExistingTaxa.setText("Reuse existing taxa when possible");
        checkBoxReuseExistingTaxa
            .setToolTipText("Reuse existing taxa when the name matches the identified name of the specimen");
        checkBoxReuseExistingTaxa.addSelectionListener(new SelectionAdapter() {
            @Override
            public void widgetSelected(SelectionEvent e) {
                abcdImportConfigurator.setReuseExistingDescriptiveGroups(!abcdImportConfigurator.isReuseExistingDescriptiveGroups());
            }
        });

        Button checkBoxIgnoreAuthorship = new Button(composite, SWT.CHECK);
        gridData = new GridData();
        gridData.horizontalAlignment = GridData.FILL_HORIZONTAL;
        gridData.horizontalSpan = 2;
        checkBoxIgnoreAuthorship.setLayoutData(gridData);
        GridDataFactory.fillDefaults();
        checkBoxIgnoreAuthorship.setSelection(abcdImportConfigurator.isIgnoreAuthorship());
        checkBoxIgnoreAuthorship.setText("Ignore Authorship for name matching");
        checkBoxIgnoreAuthorship
        .setToolTipText("Name matching with existing names will be done without "
                + "the authorship part of the name");
        checkBoxIgnoreAuthorship.addSelectionListener(new SelectionAdapter() {
            @Override
            public void widgetSelected(SelectionEvent e) {
                abcdImportConfigurator.setIgnoreAuthorship(!abcdImportConfigurator.isIgnoreAuthorship());
            }
        });

        Label labelCode = new Label(composite, SWT.NONE);
        labelCode.setText("Nomenclatural Code");
        gridData = new GridData();
        gridData.horizontalAlignment=SWT.BEGINNING;

        labelCode.setLayoutData(gridData);
        GridDataFactory.fillDefaults();
        nomenclaturalCodeSelectionCombo = new Combo(composite, SWT.BORDER| SWT.READ_ONLY);
        gridData = new GridData();
        gridData.horizontalAlignment=SWT.BEGINNING;
        nomenclaturalCodeSelectionCombo.setLayoutData(gridData);
        GridDataFactory.fillDefaults();
        for(NomenclaturalCode code: NomenclaturalCode.values()){
            nomenclaturalCodeSelectionCombo.add(code.getKey());
        }

        nomenclaturalCodeSelectionCombo.addSelectionListener(this);
        int index = 0;
        if (abcdImportConfigurator.getNomenclaturalCode() != null){
            for (String label : nomenclaturalCodeSelectionCombo.getItems()){
                if (label.equals(abcdImportConfigurator.getNomenclaturalCode().getKey())){
                    nomenclaturalCodeSelectionCombo.select(index);
                }
                index++;
            }
        }

        Label labelLinkToTaxon = new Label(composite, SWT.NONE);
        labelLinkToTaxon.setText("Link to Taxon");
        labelLinkToTaxon.setFont(font);

        Button checkBoxAddIndividualsAssociations = new Button(composite, SWT.CHECK);
        gridData = new GridData();
        gridData.horizontalAlignment = GridData.FILL_HORIZONTAL;
        gridData.horizontalSpan = 2;
        checkBoxAddIndividualsAssociations.setLayoutData(gridData);
        GridDataFactory.fillDefaults();
        checkBoxAddIndividualsAssociations.setSelection(abcdImportConfigurator.isAddIndividualsAssociationsSuchAsSpecimenAndObservations());
        checkBoxAddIndividualsAssociations.setText("Create an Individual Association for each Specimen");
        checkBoxAddIndividualsAssociations
            .setToolTipText("For each specimen associated to a taxon an indiviadual association to this taxon is created");
        checkBoxAddIndividualsAssociations.addSelectionListener(new SelectionAdapter() {
            @Override
            public void widgetSelected(SelectionEvent e) {
                abcdImportConfigurator.setAddIndividualsAssociationsSuchAsSpecimenAndObservations(!abcdImportConfigurator.isAddIndividualsAssociationsSuchAsSpecimenAndObservations());
            }
        });

        Button checkBoxReuseDescriptiveGroups = new Button(composite, SWT.CHECK);
        gridData = new GridData();
        gridData.horizontalAlignment = GridData.FILL_HORIZONTAL;
        gridData.horizontalSpan = 2;
        checkBoxReuseDescriptiveGroups.setLayoutData(gridData);
        GridDataFactory.fillDefaults();
        checkBoxReuseDescriptiveGroups.setSelection(abcdImportConfigurator.isReuseExistingDescriptiveGroups());
        checkBoxReuseDescriptiveGroups.setText("Reuse existing descriptive group");
        checkBoxReuseDescriptiveGroups
            .setToolTipText("Reuse one of the existing descriptive groups or create a new one for every import");
        checkBoxReuseDescriptiveGroups.addSelectionListener(new SelectionAdapter() {
            @Override
            public void widgetSelected(SelectionEvent e) {
                abcdImportConfigurator.setReuseExistingDescriptiveGroups(!abcdImportConfigurator.isReuseExistingDescriptiveGroups());
            }
        });


        Label labelSpecimen = new Label(composite, SWT.NONE);
        labelSpecimen.setText("Specimen");
        labelSpecimen.setFont(font);
        gridData = new GridData();
        gridData.horizontalAlignment = GridData.FILL;
        gridData.grabExcessHorizontalSpace=true;
        gridData.horizontalSpan = 2;
        labelSpecimen.setLayoutData(gridData);
        GridDataFactory.fillDefaults();
		Button checkBoxIgnoreExisting = new Button(composite, SWT.CHECK);
		gridData = new GridData();
        gridData.horizontalAlignment = GridData.FILL;
        gridData.grabExcessHorizontalSpace=true;
        gridData.horizontalSpan = 2;

		checkBoxIgnoreExisting.setSelection(abcdImportConfigurator.isIgnoreImportOfExistingSpecimen());
		checkBoxIgnoreExisting.setText("Do not import existing specimens");
		checkBoxIgnoreExisting
		.setToolTipText("Specimens that have previously been imported will be ignored in this import");
		checkBoxIgnoreExisting.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				abcdImportConfigurator.setIgnoreImportOfExistingSpecimen(!abcdImportConfigurator.isIgnoreImportOfExistingSpecimen());
			}
		});

		checkBoxIgnoreExisting.setLayoutData(gridData);
        GridDataFactory.fillDefaults();

		Label unitIdLabel = new Label(composite, SWT.NONE);
        unitIdLabel.setText(Messages.AbcdImportPreference_map_unit_nr_catalog_number);

        GridData gridDataUnitId = new GridData();
        gridDataUnitId.horizontalAlignment=SWT.BEGINNING;
        unitIdLabel.setLayoutData(gridDataUnitId);
        GridDataFactory.fillDefaults();

        checkBoxMapUnitId = new Combo(composite, SWT.BORDER| SWT.READ_ONLY);
        gridDataUnitId = new GridData();
        gridDataUnitId.horizontalAlignment=SWT.BEGINNING;
        checkBoxMapUnitId.setLayoutData(gridDataUnitId);
        GridDataFactory.fillDefaults();

        checkBoxMapUnitId.add(ACCESSION_NUMBER);
        checkBoxMapUnitId.add(BARCODE);
        checkBoxMapUnitId.add(CATALOGUE_NUMBER);
        checkBoxMapUnitId
        .setToolTipText(Messages.AbcdImportPreference_map_unit_number_catalog_number_tooltip);
        checkBoxMapUnitId.addSelectionListener(this);

        if (abcdImportConfigurator.isMapUnitIdToAccessionNumber()){
            checkBoxMapUnitId.select(0);
        }else if (abcdImportConfigurator.isMapUnitIdToBarcode()){
            checkBoxMapUnitId.select(1);
        }else {
            checkBoxMapUnitId.select(2);
        }


        Button checkBoxRemoveCountry = new Button(composite, SWT.CHECK);
        gridData = new GridData();
        gridData.horizontalAlignment = GridData.FILL_HORIZONTAL;
        gridData.horizontalSpan = 2;
		checkBoxRemoveCountry.setLayoutData(gridData);
		GridDataFactory.fillDefaults();
		checkBoxRemoveCountry.setSelection(abcdImportConfigurator.isRemoveCountryFromLocalityText());
		checkBoxRemoveCountry.setText("Remove country from locality text");
		checkBoxRemoveCountry
		.setToolTipText("If the locality text contains information about the "
				+ "country which is additionally stored in other ABCD "
				+ "elements then it is removed from the locality text");
		checkBoxRemoveCountry.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				abcdImportConfigurator.setRemoveCountryFromLocalityText(!abcdImportConfigurator.isRemoveCountryFromLocalityText());
			}
		});



		Button checkBoxImportSiblings = new Button(composite, SWT.CHECK);
		gridData = new GridData();
        gridData.horizontalAlignment = GridData.FILL_HORIZONTAL;
        gridData.horizontalSpan = 2;
		checkBoxImportSiblings.setLayoutData(gridData);
        GridDataFactory.fillDefaults();
		checkBoxImportSiblings.setSelection(abcdImportConfigurator.isGetSiblings());
		checkBoxImportSiblings.setText("Import all children of cultures or tissue samples");
		checkBoxImportSiblings
        	.setToolTipText("For a tissue sample or culture all children will be searched and imported");
		checkBoxImportSiblings.addSelectionListener(new SelectionAdapter() {
            @Override
            public void widgetSelected(SelectionEvent e) {
                abcdImportConfigurator.setGetSiblings(checkBoxImportSiblings.getSelection());
            }
        });





        if (!isFileImport){

            Label labelRef = new Label(composite, SWT.NONE);
            gridDataUnitId = new GridData();
            gridDataUnitId.horizontalAlignment=SWT.BEGINNING;
            labelRef.setLayoutData(gridDataUnitId);
            labelRef.setText("Biocase provider for associated DNA");
            GridDataFactory.fillDefaults();

            textDNAProviderString = new Combo(composite, SWT.BORDER);
            textDNAProviderString.setLayoutData(gridDataUnitId);
            GridDataFactory.fillDefaults();


            String allProviderString = PreferencesUtil.getStringValue(PreferencePredicate.BioCaseProvider.getKey(), false);
            if (StringUtils.isNotBlank(allProviderString)){
                String[] providerArray = allProviderString.split(";"); //$NON-NLS-1$
                for (String providerString : providerArray){
                    if (!StringUtils.isBlank(providerString)){
                        textDNAProviderString.add(providerString);

                    }
                }
            }
            textDNAProviderString.setEnabled(true);
            textDNAProviderString.add("Do not search for DNA");
            textDNAProviderString.setLayoutData(new GridData(SWT.FILL, SWT.TOP, true, true, 1, 1));
            if (abcdImportConfigurator.getDnaSoure() != null){
            	textDNAProviderString.select(0);
            }else{
                textDNAProviderString.select(textDNAProviderString.getItemCount());
            }
            textDNAProviderString.addSelectionListener(this);
        }else{
            abcdImportConfigurator.setDnaSoure(null);
        }

        Label labelMedia = new Label(composite, SWT.NONE);
        labelMedia.setText("Media");
        labelMedia.setFont(font);
        gridData = new GridData();
        gridData.horizontalAlignment = GridData.FILL;
        gridData.grabExcessHorizontalSpace=true;
        gridData.horizontalSpan = 2;
        labelMedia.setLayoutData(gridData);


        Button checkBoxMediaSpecimen = new Button(composite, SWT.CHECK);

        checkBoxMediaSpecimen.setSelection(abcdImportConfigurator.isAddMediaAsMediaSpecimen());
        checkBoxMediaSpecimen.setText("Import media as media specimen");
        checkBoxMediaSpecimen
                .setToolTipText("Any media attached to a ABCD unit will be imported as a sub derivative of the specimen created from this unit");
        checkBoxMediaSpecimen.addSelectionListener(new SelectionAdapter() {
            @Override
            public void widgetSelected(SelectionEvent e) {
                abcdImportConfigurator.setAddMediaAsMediaSpecimen(!abcdImportConfigurator.isAddMediaAsMediaSpecimen());
            }
        });
        gridData = new GridData();
        gridData.horizontalAlignment = GridData.FILL_HORIZONTAL;
        gridData.horizontalSpan = 2;
        checkBoxMediaSpecimen.setLayoutData(gridData);
        GridDataFactory.fillDefaults();

        Label labelSource = new Label(composite, SWT.NONE);
        labelSource.setText("Source");
        labelSource.setFont(font);
        gridData = new GridData();
        gridData.horizontalAlignment = GridData.FILL;
        gridData.grabExcessHorizontalSpace=true;
        gridData.horizontalSpan = 2;
        labelSource.setLayoutData(gridData);

        Composite sourceComp = new Composite(composite, SWT.NULL);
        sourceComp.setLayoutData(new GridData(SWT.FILL, SWT.TOP, false, false, 2, 1));
        sourceComp.setLayout(new GridLayout(4, false));
        Label labelSourceRef = new Label(sourceComp, SWT.TOP);
        labelSourceRef.setText("Source");
//      labelClassification.setLayoutData(new GridData(SWT.LEFT, SWT.CENTER, false, false, 3, 1));
        textSource = new Text(sourceComp, SWT.BORDER);
        textSource.setEnabled(false);
        textSource.setLayoutData(new GridData(SWT.FILL, SWT.LEFT, true, true));
        if (abcdImportConfigurator.getSourceRefUuid() != null){
            if (abcdImportConfigurator.getSourceReferenceTitle() != null){
                textSource.setText(abcdImportConfigurator.getSourceReferenceTitle());
            }else{
                Reference sourceRef = CdmStore.getService(IReferenceService.class).load(abcdImportConfigurator.getSourceRefUuid());
                textClassification.setText(sourceRef.getTitleCache());
            }
        }
        btnBrowseSource = new Button(sourceComp, SWT.NONE);
        btnBrowseSource.setImage(ResourceManager.getPluginImage("eu.etaxonomy.taxeditor.store", "icons/open.gif"));
        btnBrowseSource.addSelectionListener(new SelectionAdapter() {
            @Override
            public void widgetSelected(SelectionEvent e) {
                Reference sourceRef = SelectionDialogFactory.getSelectionFromDialog(Reference.class,parent.getShell(), null, null);
                if(sourceRef!=null){
                    textSource.setText(sourceRef.getTitleCache());
                    abcdImportConfigurator.setSourceRefUuid(sourceRef.getUuid());
                    abcdImportConfigurator.setSourceReferenceTitle(sourceRef.getTitleCache());
                }
            }
        });
        btnClear = new Button(sourceComp, SWT.NONE);
        btnClear.setImage(ResourceManager.getPluginImage("eu.etaxonomy.taxeditor.store", "icons/trash.gif"));
        btnClear.addSelectionListener(new SelectionAdapter() {
            @Override
            public void widgetSelected(SelectionEvent e) {
                abcdImportConfigurator.setClassificationUuid(null);
                textClassification.setText("");
            }
        });

		setControl(composite);
	}

	@Override
    public void widgetSelected(SelectionEvent e) {
        if (e.getSource().equals(nomenclaturalCodeSelectionCombo)){
            this.abcdImportConfigurator.setNomenclaturalCode(NomenclaturalCode.getByKey(nomenclaturalCodeSelectionCombo.getText()));
        }
        if (e.getSource().equals(checkBoxMapUnitId)){
            String text = checkBoxMapUnitId.getText();
            if (text.equals(ACCESSION_NUMBER)){
                abcdImportConfigurator.setMapUnitIdToAccessionNumber(true);
                abcdImportConfigurator.setMapUnitIdToBarcode(false);
                abcdImportConfigurator.setMapUnitIdToCatalogNumber(false);
            }else if (text.equals(BARCODE)){
                abcdImportConfigurator.setMapUnitIdToAccessionNumber(false);
                abcdImportConfigurator.setMapUnitIdToBarcode(true);
                abcdImportConfigurator.setMapUnitIdToCatalogNumber(false);
            }else{
                abcdImportConfigurator.setMapUnitIdToAccessionNumber(false);
                abcdImportConfigurator.setMapUnitIdToBarcode(false);
                abcdImportConfigurator.setMapUnitIdToCatalogNumber(true);
            }
        }
        if (e.getSource().equals(textDNAProviderString)){
            String text = textDNAProviderString.getText();
            try{
                abcdImportConfigurator.setDnaSoure(URI.create(text));
            }catch(IllegalArgumentException illegalArgument){
                abcdImportConfigurator.setDnaSoure(null);
            }
        }
    }

	public void saveConfigToPrefernceStore() {
	     try{
	         abcdImportConfigurator.setDnaSoure(URI.create(textDNAProviderString.getText()));
	     }catch(IllegalArgumentException e){
//TODO: check prior or message
	         abcdImportConfigurator.setDnaSoure(null);
	     }

	     PreferencesUtil.setStringValue(IPreferenceKeys.LAST_USED_ABCD_CONFIG, abcdImportConfigurator.toString());
	    }

	 public String createConfigString(){
	    return abcdImportConfigurator.toString();

	 }

    @Override
    public void widgetDefaultSelected(SelectionEvent e) {
        // TODO Auto-generated method stub
    }

    public void createAbcdImportConfig() {
        this.abcdImportConfigurator = PreferencesUtil.getLocalAbcdImportConfigurator(false);
    }
}
