/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.io.e4.in.abcd;

import javax.inject.Inject;

import org.eclipse.e4.core.contexts.ContextInjectionFactory;
import org.eclipse.e4.core.contexts.IEclipseContext;
import org.eclipse.e4.ui.workbench.modeling.EPartService;

import eu.etaxonomy.cdm.common.URI;
import eu.etaxonomy.cdm.database.DbSchemaValidation;
import eu.etaxonomy.cdm.io.common.IImportConfigurator.SOURCE_TYPE;
import eu.etaxonomy.cdm.io.specimen.abcd206.in.Abcd206ImportConfigurator;
import eu.etaxonomy.taxeditor.io.e4.in.AbstractImportWizardE4;
import eu.etaxonomy.taxeditor.io.e4.in.ImportFromFileDataSourceWithReferenceWizardPage;
import eu.etaxonomy.taxeditor.preference.PreferencesUtil;
import eu.etaxonomy.taxeditor.store.CdmStore;

/**
 * @author pplitzner
 * @since Oct 5, 2017
 */
public class AbcdImportWizardE4 extends AbstractImportWizardE4<Abcd206ImportConfigurator> {

	private Abcd206ImportConfigurator configurator;
	private ImportFromFileDataSourceWithReferenceWizardPage dataSourcePage;
	private ClassificationChooserWizardPageE4 classificationChooserWizardPage;

	@Inject
	public AbcdImportWizardE4(IEclipseContext context, EPartService partService) {
	    super(context, partService);
	}

	@Override
	public Abcd206ImportConfigurator getConfigurator() {
		return configurator;
	}

	@Override
	public boolean performFinish() {
	    URI source = dataSourcePage.getUri();
	    configurator.setDbSchemaValidation(DbSchemaValidation.CREATE);

	    if(classificationChooserWizardPage.getClassification()!=null){
	        configurator.setClassificationUuid(classificationChooserWizardPage.getClassification().getUuid());
	    }

	    CdmStore.getImportManager().runMoniteredOperation(configurator, source.toFile(), SOURCE_TYPE.INPUTSTREAM);

	    return true;
	}

	@Override
    public void init() {
		configurator =  PreferencesUtil.getLocalAbcdImportConfigurator(false);
	}

	@Override
	protected void addConfiguratorPage() {
	    AbcdImportConfiguratorWizardPage configuratorWizardPage = AbcdImportConfiguratorWizardPage.createFileImportPage(configurator);//AbcdImportConfiguratorWizardPage.createPage(configurator);
	    //configuratorWizardPage.setConfigurator(configurator);
		addPage(configuratorWizardPage);
	}

	@Override
	public void addPages() {

		classificationChooserWizardPage = ContextInjectionFactory.make(ClassificationChooserWizardPageE4.class, context);

		dataSourcePage = ContextInjectionFactory.make(ImportFromFileDataSourceWithReferenceWizardPage.class, context);

		addPage(classificationChooserWizardPage);
		addPage(dataSourcePage);
		addConfiguratorPage();
		//super.addPages();
	}
}