/**
* Copyright (C) 2017 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.io.e4.in.excel.distribution;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.UUID;

import javax.inject.Inject;

import org.apache.commons.io.IOUtils;
import org.apache.log4j.Logger;
import org.eclipse.core.runtime.jobs.Job;
import org.eclipse.e4.core.contexts.ContextInjectionFactory;
import org.eclipse.e4.core.contexts.IEclipseContext;
import org.eclipse.e4.ui.workbench.modeling.EPartService;

import eu.etaxonomy.cdm.common.URI;
import eu.etaxonomy.cdm.database.DbSchemaValidation;
import eu.etaxonomy.cdm.io.distribution.excelupdate.ExcelDistributionUpdateConfigurator;
import eu.etaxonomy.taxeditor.io.e4.in.AbstractImportWizardE4;
import eu.etaxonomy.taxeditor.io.e4.in.GenericConfiguratorWizardPageE4;
import eu.etaxonomy.taxeditor.io.e4.in.ImportFromFileAndChooseVocIdWizardPageE4;
import eu.etaxonomy.taxeditor.l10n.Messages;
import eu.etaxonomy.taxeditor.store.CdmStore;

/**
 * @author k.luther
 * @date 03.05.2017
 */
public class ExcelDistributionUpdateWizardE4 extends AbstractImportWizardE4<ExcelDistributionUpdateConfigurator> {


    private ExcelDistributionUpdateConfigurator configurator;

    private ImportFromFileAndChooseVocIdWizardPageE4 dataSourcePage;

    private static final Logger logger = Logger.getLogger(ExcelDistributionUpdateWizardE4.class);

    @Inject
    public ExcelDistributionUpdateWizardE4(IEclipseContext context, EPartService partService) {
        super(context, partService);
    }

    @Override
    public ExcelDistributionUpdateConfigurator getConfigurator() {
              return configurator;
    }

    @Override
    public boolean performFinish() {
        URI source = dataSourcePage.getUri();
       // configurator.setSource(source);
        configurator.setDbSchemaValidation(DbSchemaValidation.CREATE);
        File file = new File(source.getJavaUri());
        FileInputStream fis = null;

        try {
            fis = new FileInputStream(file);
        } catch (FileNotFoundException e) {
            logger.error("Error while reading file" + source.toString());
        }
        try {
            configurator.setStream(IOUtils.toByteArray(fis));
        } catch (IOException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
        UUID vocUuid = dataSourcePage.getVocUuid();

        configurator.setAreaVocabularyUuid(vocUuid);
        //CdmStore.getImportManager().runMoniteredOperation(configurator, new File(source), SOURCE_TYPE.INPUTSTREAM);
        Job job = CdmStore.getImportManager().createIOServiceJob(configurator);
        CdmStore.getImportManager().run(job);

        return true;
    }

    @Override
    public void init() {
        configurator =  CdmStore.getImportManager().ExcelDistributionUpdateConfigurator();

    }

    @Override
    public void addPages() {
        addDataSourcePage();
        addConfiguratorPage();

    }
    @Override
    protected void addConfiguratorPage(){
        List<String> ignoreMethods = new ArrayList<>();
        ignoreMethods.add("setIgnoreNull");
        ignoreMethods.add("setInteractWithUser");
        ignoreMethods.add("setUseClassification");
        ignoreMethods.add("setRegisterAuditing");
        ignoreMethods.add("setDeduplicateAuthors");
        ignoreMethods.add("setDeduplicateReferences");
        pageConfiguration = ContextInjectionFactory.make(GenericConfiguratorWizardPageE4.class, context);
        pageConfiguration.initImport(getConfigurator(), ignoreMethods, Messages.ExcelDistributionUpdateWizard_ConfiguratorWizard_label);
        addPage(pageConfiguration);
    }

    private void addDataSourcePage(){
        dataSourcePage = ContextInjectionFactory.make(ImportFromFileAndChooseVocIdWizardPageE4.class, context);
        addPage(dataSourcePage);
    }
}