/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.io.e4.in.excel.taxa;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import eu.etaxonomy.cdm.common.URI;
import java.util.ArrayList;
import java.util.List;

import javax.inject.Inject;
import javax.inject.Named;

import org.apache.commons.io.IOUtils;
import org.apache.log4j.Logger;
import org.eclipse.core.runtime.jobs.Job;
import org.eclipse.e4.core.contexts.ContextInjectionFactory;
import org.eclipse.e4.core.contexts.IEclipseContext;
import org.eclipse.e4.core.di.annotations.Optional;
import org.eclipse.e4.ui.services.IServiceConstants;
import org.eclipse.e4.ui.workbench.modeling.EPartService;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.TreeSelection;

import eu.etaxonomy.cdm.database.DbSchemaValidation;
import eu.etaxonomy.cdm.io.common.IImportConfigurator.SOURCE_TYPE;
import eu.etaxonomy.cdm.io.excel.taxa.NormalExplicitImportConfigurator;
import eu.etaxonomy.cdm.persistence.dto.TaxonNodeDto;
import eu.etaxonomy.taxeditor.io.e4.in.AbstractImportWizardE4;
import eu.etaxonomy.taxeditor.io.e4.in.GenericConfiguratorWizardPageE4;
import eu.etaxonomy.taxeditor.io.e4.in.ImportFromFileDataSourceWithReferenceWizardPage;
import eu.etaxonomy.taxeditor.l10n.Messages;
import eu.etaxonomy.taxeditor.store.CdmStore;


/**
 * @author pplitzner
 * @since Oct 5, 2017
 */
public class ExcelNormalExplicitTaxaImportWizardE4 extends AbstractImportWizardE4<NormalExplicitImportConfigurator>{

    private NormalExplicitImportConfigurator configurator;

	private ImportFromFileDataSourceWithReferenceWizardPage dataSourcePage;
	private static final Logger logger = Logger.getLogger(ExcelNormalExplicitTaxaImportWizardE4.class);

	@Inject
	public ExcelNormalExplicitTaxaImportWizardE4(IEclipseContext context, EPartService partService,
	        @Optional@Named(IServiceConstants.ACTIVE_SELECTION)IStructuredSelection selection) {
	    super(context, partService, selection);
	}

	@Override
	public NormalExplicitImportConfigurator getConfigurator() {
		return configurator;
	}

	@Override
	public boolean performFinish() {
		URI source = dataSourcePage.getUri();
		configurator.setSource(source);
		configurator.setDbSchemaValidation(DbSchemaValidation.CREATE);
		configurator.setSourceReferenceTitle(dataSourcePage.getTextReferenceString().getText());
		File file = new File(source.getJavaUri());
		FileInputStream fis = null;

		try {
			fis = new FileInputStream(file);
		} catch (FileNotFoundException e) {
			logger.error("Error while reading file" + source.toString());
		}
		try {
            configurator.setStream(IOUtils.toByteArray(fis));
        } catch (IOException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
	    Job job = CdmStore.getImportManager().createIOServiceJob(configurator,file , SOURCE_TYPE.INPUTSTREAM);
	    CdmStore.getImportManager().run(job);

		return true;
	}

	@Override
    public void init() {
		configurator =  CdmStore.getImportManager().NormalExplicitConfigurator();
		if (selection instanceof TreeSelection && !selection.isEmpty()){
			TaxonNodeDto node = (TaxonNodeDto)selection.getFirstElement();
			configurator.setParentUUID(node.getTaxonUuid());
			configurator.setClassificationUuid(node.getClassificationUUID());
		}
	}

	@Override
	public void addPages() {
	    dataSourcePage = ContextInjectionFactory.make(ImportFromFileDataSourceWithReferenceWizardPage.class, context);
		dataSourcePage.setTitle("Choose NormalExplicit");
		dataSourcePage.setDescription("Please choose an xls file in the NormalExplicit format.");
		dataSourcePage.setExtensions(new String[]{"*.xlsx", "*.xls", "*.*"});
		addPage(dataSourcePage);
		super.addPages();
	}
	
	@Override
    protected void addConfiguratorPage(){
         List<String> ignoreMethods = new ArrayList<>();
         ignoreMethods.add("setIgnoreNull");
         ignoreMethods.add("setInteractWithUser");
         ignoreMethods.add("setUseClassification");
         ignoreMethods.add("setRegisterAuditing");
         ignoreMethods.add("setDeduplicateAuthors");
         ignoreMethods.add("setDeduplicateReferences");
         pageConfiguration = ContextInjectionFactory.make(GenericConfiguratorWizardPageE4.class, context);
         pageConfiguration.initImport(getConfigurator(), ignoreMethods, Messages.ExcelTaxonUpdateWizard_ConfiguratorWizard_label);
         addPage(pageConfiguration);
    }
}