/**
* Copyright (C) 2017 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.io.e4.in.ris;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import javax.inject.Inject;

import org.apache.commons.io.IOUtils;
import org.apache.log4j.Logger;
import org.eclipse.core.runtime.jobs.Job;
import org.eclipse.e4.core.contexts.ContextInjectionFactory;
import org.eclipse.e4.core.contexts.IEclipseContext;
import org.eclipse.e4.ui.workbench.modeling.EPartService;

import eu.etaxonomy.cdm.common.URI;
import eu.etaxonomy.cdm.database.DbSchemaValidation;
import eu.etaxonomy.cdm.io.common.IImportConfigurator.SOURCE_TYPE;
import eu.etaxonomy.cdm.io.reference.ris.in.RisReferenceImportConfigurator;
import eu.etaxonomy.taxeditor.io.e4.in.AbstractImportWizardE4;
import eu.etaxonomy.taxeditor.io.e4.in.GenericConfiguratorWizardPageE4;
import eu.etaxonomy.taxeditor.io.e4.in.ImportFromFileDataSourceWithReferenceWizardPage;
import eu.etaxonomy.taxeditor.io.e4.in.excel.distribution.ExcelDistributionUpdateWizardE4;
import eu.etaxonomy.taxeditor.store.CdmStore;

/**
 * @author k.luther
 * @date 12.05.2017
 */
public class RISImportWizardE4 extends AbstractImportWizardE4<RisReferenceImportConfigurator>{

    private RisReferenceImportConfigurator configurator;

    private ImportFromFileDataSourceWithReferenceWizardPage dataSourcePage;

    private static final Logger logger = Logger.getLogger(ExcelDistributionUpdateWizardE4.class);

    @Inject
    public RISImportWizardE4(IEclipseContext context, EPartService partService) {
        super(context, partService);
    }

    @Override
    public RisReferenceImportConfigurator getConfigurator() {
        return configurator;
    }

    @Override
    public boolean performFinish() {
        URI source = dataSourcePage.getUri();
        configurator.setDbSchemaValidation(DbSchemaValidation.CREATE);
        File file = new File(source.getJavaUri());
       FileInputStream fis;

        try {
            fis = new FileInputStream(file);
            byte[] data;
            data = IOUtils.toByteArray(fis);
            configurator.setStream(data);
//TODO: workaround, needs to be changed to source as byte array
            Job job = CdmStore.getImportManager().createIOServiceJob(configurator, data, SOURCE_TYPE.INPUTSTREAM);
            CdmStore.getImportManager().run(job);
        } catch (FileNotFoundException e) {
            logger.error("Error while reading file" + source.toString());
        } catch (IOException e){

        }

        return true;
    }

    @Override
    public void init() {
        configurator = RisReferenceImportConfigurator.NewInstance();

    }

    @Override
    public void addPages() {
        addDataSourcePage();
    }
    @Override
    protected void addConfiguratorPage(){
        List<String> ignoreMethods = new ArrayList<>();
        ignoreMethods.add("setIgnoreNull");
        ignoreMethods.add("setInteractWithUser");
        ignoreMethods.add("setUseClassification");
        pageConfiguration = ContextInjectionFactory.make(GenericConfiguratorWizardPageE4.class, context);
        pageConfiguration.initImport(getConfigurator(), ignoreMethods, "Ris Import");
        addPage(pageConfiguration);
    }

    private void addDataSourcePage(){
        dataSourcePage = ContextInjectionFactory.make(ImportFromFileDataSourceWithReferenceWizardPage.class, context);
        dataSourcePage.setTitle("Choose Ris File");
        dataSourcePage.setDescription("Please choose a file in RIS format.");
        dataSourcePage.setExtensions(new String[]{"*.txt"});
        addPage(dataSourcePage);
    }

}
