/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.io.e4.in.tcs;

import java.io.File;
import java.util.ArrayList;
import java.util.List;

import javax.inject.Inject;

import org.eclipse.core.runtime.jobs.Job;
import org.eclipse.e4.core.contexts.ContextInjectionFactory;
import org.eclipse.e4.core.contexts.IEclipseContext;
import org.eclipse.e4.ui.workbench.modeling.EPartService;

import eu.etaxonomy.cdm.common.URI;
import eu.etaxonomy.cdm.database.DbSchemaValidation;
import eu.etaxonomy.cdm.io.common.IImportConfigurator.SOURCE_TYPE;
import eu.etaxonomy.cdm.io.tcsxml.in.TcsXmlImportConfigurator;
import eu.etaxonomy.taxeditor.io.e4.in.AbstractImportWizardE4;
import eu.etaxonomy.taxeditor.io.e4.in.GenericConfiguratorWizardPageE4;
import eu.etaxonomy.taxeditor.io.e4.in.ImportFromFileDataSourceWithReferenceWizardPage;
import eu.etaxonomy.taxeditor.l10n.Messages;
import eu.etaxonomy.taxeditor.store.CdmStore;

/**
 * @author pplitzner
 * @since Oct 5, 2017
 */
public class TcsImportWizardE4 extends AbstractImportWizardE4<TcsXmlImportConfigurator> {

    private TcsXmlImportConfigurator configurator;

	private ImportFromFileDataSourceWithReferenceWizardPage dataSourcePage;

	@Inject
	public TcsImportWizardE4(IEclipseContext context, EPartService partService) {
	    super(context, partService);
	}

	@Override
	public boolean performFinish() {
	    URI source = dataSourcePage.getUri();
	    configurator.setSource(source);
	    configurator.setDbSchemaValidation(DbSchemaValidation.CREATE);

	    Job job = CdmStore.getImportManager().createIOServiceJob(configurator, new File(source.getJavaUri()), SOURCE_TYPE.URI);
	    CdmStore.getImportManager().run(job);

	    return true;
	}

	@Override
    public void init() {
		configurator = CdmStore.getImportManager().TcsConfigurator();
	}

	@Override
	public TcsXmlImportConfigurator getConfigurator() {
		return configurator;
	}

	 @Override
     protected void addConfiguratorPage(){
         List<String> ignoreMethods = new ArrayList<>();
         ignoreMethods.add("setRegisterAuditing");
         pageConfiguration = ContextInjectionFactory.make(GenericConfiguratorWizardPageE4.class, context);
         pageConfiguration.initImport(getConfigurator(), ignoreMethods, Messages.TCSImportWizard_ConfiguratorWizard_label);
         addPage(pageConfiguration);
    }

	@Override
	public void addPages() {
		dataSourcePage = ContextInjectionFactory.make(ImportFromFileDataSourceWithReferenceWizardPage.class, context);
		addPage(dataSourcePage);
		super.addPages();
	}
}