/**
 * Copyright (C) 2017 EDIT
 * European Distributed Institute of Taxonomy
 * http://www.e-taxonomy.eu
 *
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */
package eu.etaxonomy.taxeditor.io.e4.out.cdmlight;

import java.io.File;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.HashSet;
import java.util.List;
import java.util.UUID;

import javax.inject.Inject;
import javax.inject.Named;

import org.apache.commons.lang.StringUtils;
import org.eclipse.e4.core.contexts.IEclipseContext;
import org.eclipse.e4.core.di.annotations.Optional;
import org.eclipse.e4.ui.services.IServiceConstants;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.TreeSelection;
import org.eclipse.swt.widgets.Combo;

import eu.etaxonomy.cdm.ext.geo.CondensedDistributionRecipe;
import eu.etaxonomy.cdm.filter.TaxonNodeFilter;
import eu.etaxonomy.cdm.io.cdmLight.CdmLightExportConfigurator;
import eu.etaxonomy.cdm.model.metadata.PreferencePredicate;
import eu.etaxonomy.cdm.model.taxon.Classification;
import eu.etaxonomy.cdm.persistence.dto.TaxonNodeDto;
import eu.etaxonomy.taxeditor.io.e4.out.AbstractExportWizardE4;
import eu.etaxonomy.taxeditor.io.wizard.ExportToFileDestinationWizardPage;
import eu.etaxonomy.taxeditor.preference.PreferencesUtil;
import eu.etaxonomy.taxeditor.store.CdmStore;

/**
 * @author k.luther
 * @date 21.03.2017
 */
public class CdmLightExportWizardE4 extends
AbstractExportWizardE4<CdmLightExportConfigurator> {


    private CdmLightExportConfigurator configurator;
    private ExportToFileDestinationWizardPage page;
    private MetaDataConfigurationWizardPage metaDataPage;
    public static final String DATE_FORMAT_NOW = "yyyyMMddHHmm";
    TaxonNodeDto selectedNode;

    @Inject
    public CdmLightExportWizardE4(IEclipseContext context,
            @Optional@Named(IServiceConstants.ACTIVE_SELECTION)IStructuredSelection selection) {
        super(context, selection);
    }

    @Override
    public void init() {
        configurator = CdmLightExportConfigurator.NewInstance();
        String recipeString = PreferencesUtil.getStringValue(PreferencePredicate.CondensedDistributionRecipe.getKey());
        if (StringUtils.isNotBlank(recipeString)){
            configurator.setCondensedDistributionConfiguration(CondensedDistributionRecipe.valueOf(recipeString).toConfiguration());
        }
        if (selection instanceof TreeSelection && !selection.isEmpty() &&  selection.getFirstElement() instanceof TaxonNodeDto){
            selectedNode = (TaxonNodeDto)selection.getFirstElement();

            if (selectedNode.getParentUUID() == null){
                configurator.setTaxonNodeFilter(TaxonNodeFilter.NewClassificationInstance(selectedNode.getClassificationUUID()));
            }else{
                configurator.setTaxonNodeFilter(TaxonNodeFilter.NewSubtreeInstance(selectedNode.getUuid()));
            }
        }
    }


    @Override
    public CdmLightExportConfigurator getConfigurator() {
        return configurator;
    }

    @Override
    public boolean performFinish() {
        String urlString = page.getFolderText() + File.separator ;
        if (configurator.isCreateZipFile()){
            SimpleDateFormat sdf = new SimpleDateFormat(DATE_FORMAT_NOW);
            Calendar cal = Calendar.getInstance();

            if (!configurator.getTaxonNodeFilter().getSubtreeFilter().isEmpty()){
                urlString +=  selectedNode.getNameCache()+"_cdmLight_"+sdf.format(cal.getTime());
            }else if (!configurator.getTaxonNodeFilter().getClassificationFilter().isEmpty()){
                urlString += page.getSelectedClassification().getTitleCache()+"_cdmLight_"+sdf.format(cal.getTime());
                page.getSelectedClassification();
            }

        }

        //+ page.getExportFileName();

        final Combo combo = page.getCombo();
        if (metaDataPage != null){
            metaDataPage.fillConfig();
            configurator = metaDataPage.configurator;
        }

        final List<Classification> listClassifications = CdmStore.getCurrentApplicationConfiguration().getClassificationService().listClassifications(null, null, null, null);
        if(combo != null){
            int selectionIndex = combo.getSelectionIndex();
            HashSet<UUID> set = new HashSet<UUID>();
            if(selectionIndex == -1){
                for(Classification c:listClassifications){
                    set.add(c.getUuid());
                }
            }else{
                for(Classification c:listClassifications){
                    if(c.getTitleCache().equalsIgnoreCase(combo.getItem(selectionIndex))){
                        set.add(c.getUuid());

                        if (!page.getCheckUseSelectedTaxonNode()){
                            configurator.setTaxonNodeFilter(TaxonNodeFilter.NewClassificationInstance(c.getUuid()));
                        }
                    }
                }


            }
        }
        configurator.getTaxonNodeFilter().setIncludeUnpublished(page.isExportUnpublishedData());
        CdmStore.getExportManager().runMoniteredOperation(configurator, urlString);
        // create job
//        Job job = CdmStore.getExportManager().createIOServiceJob(configurator, urlString);
//        // configure the job
//        job.setProperty(IProgressConstants.KEEP_PROPERTY, true);
//        job.setUser(true);
//        // schedule job
//        job.schedule();
        PreferencesUtil.setStringValue("exportFolder", page.getFolderText());
        return true;
    }

    @Override
    public void addPages() {
        //standard page
        page =  ExportToFileDestinationWizardPage.OutputModel(configurator);

        addPage(page);
        super.addPages();
        if (PreferencesUtil.showGfbioMetaData()){
            metaDataPage = new MetaDataConfigurationWizardPage("Add Meta Data", configurator);
            addPage(metaDataPage);
        }

    }

}
