/**
* Copyright (C) 2020 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.io.e4.out.cdmlight;

import java.util.List;

import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;

import eu.etaxonomy.cdm.io.cdmLight.CdmLightExportConfigurator;
import eu.etaxonomy.cdm.model.common.Language;
import eu.etaxonomy.taxeditor.store.CdmStore;

/**
 * @author k.luther
 * @since Aug 5, 2020
 */
public class MetaDataConfigurationWizardPage extends WizardPage {

    CdmLightExportConfigurator configurator;

    private Text text_title;
    private Text text_description;
    private Text text_creator;
    private Text text_contributor;
    private Combo combo_language;
    private Text text_dataset_landingPage;
    private Text uri_datasetDownloadLink;
    private Text base_url;
    private Text text_recommendedCitation;
    private Text text_location;
    private Text text_keyWords;
    private Text text_licence;

    /**
     * @param pageName
     */
    protected MetaDataConfigurationWizardPage(String pageName, CdmLightExportConfigurator config) {
        super(pageName);
        configurator = config;

    }

    public CdmLightExportConfigurator getConfigurator() {
        return configurator;
    }

    public void setConfigurator(CdmLightExportConfigurator configurator) {
        this.configurator = configurator;
    }

    @Override
    public void createControl(Composite parent) {
        Composite composite = new Composite(parent, SWT.NULL);
        composite.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));

        GridLayout gridLayout = new GridLayout();
        gridLayout.numColumns = 2;
        composite.setLayout(gridLayout);
        Label label_title = new Label(composite, SWT.NULL);
        label_title.setText("Title");
        text_title = new Text(composite, SWT.BORDER);
        text_title.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true,
                false));

        Label label_description = new Label(composite, SWT.NULL);
        label_description.setText("Description");
        text_description = new Text(composite, SWT.BORDER);
        text_description.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true,
                false));

        Label label_creator = new Label(composite, SWT.NULL);
        label_creator.setText("Creator");
        text_creator = new Text(composite, SWT.BORDER);
        text_creator.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true,
                false));

        Label label_contributor = new Label(composite, SWT.NULL);
        label_contributor.setText("Contributor");
        text_contributor = new Text(composite, SWT.BORDER);
        text_contributor.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true,
                false));

        Label label_language = new Label(composite, SWT.NULL);
        label_language.setText("Language");
        combo_language = new Combo(composite, SWT.BORDER);

        List<Language> preferredLanguages = CdmStore.getTermManager().getPreferredTerms(Language.class);
        int currentSelectionIndex = 0;
        Language defaultLang = CdmStore.getDefaultLanguage();
        for(int i = 0; i < preferredLanguages.size(); i++){
            Language language = preferredLanguages.get(i);
            combo_language.add(language.getLabel(), i);
            combo_language.setData(language.getLabel(), language);
           if (language.equals(defaultLang)){
               currentSelectionIndex = i;
           }
        }

        combo_language.select(currentSelectionIndex);



        Label label_landingPage = new Label(composite, SWT.NULL);
        label_landingPage.setText("Dataset landing page");
        text_dataset_landingPage= new Text(composite, SWT.BORDER);
        text_dataset_landingPage.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true,
                false));

        Label label_datasetDownloadLink = new Label(composite, SWT.NULL);
        label_datasetDownloadLink.setText("Dataset download link");
        uri_datasetDownloadLink= new Text(composite, SWT.BORDER);
        uri_datasetDownloadLink.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true,
                false));

        Label label_base_url = new Label(composite, SWT.NULL);
        label_base_url.setText("Base URL");
        base_url= new Text(composite, SWT.BORDER);
        base_url.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true,
                false));

        Label label_recommendedCitation = new Label(composite, SWT.NULL);
        label_recommendedCitation.setText("Recommended citation");
        text_recommendedCitation= new Text(composite, SWT.BORDER);
        text_recommendedCitation.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true,
                false));

        Label label_location = new Label(composite, SWT.NULL);
        label_location.setText("Location");
        text_location= new Text(composite, SWT.BORDER);
        text_location.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true,
                false));

        Label label_keyWords = new Label(composite, SWT.NULL);
        label_keyWords.setText("Keywords");
        text_keyWords= new Text(composite, SWT.BORDER);
        text_keyWords.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true,
                false));

        Label label_licence = new Label(composite, SWT.NULL);
        label_licence.setText("Licence");
        text_licence= new Text(composite, SWT.BORDER);
        text_licence.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true,
                false));


        setControl(composite);
    }


    public void fillConfig() {

        configurator.setTitle(text_title.getText());

        configurator.setDescription(text_description.getText());

        configurator.setCreator(text_creator.getText());

        configurator.setContributor(text_contributor.getText());

        configurator.setLanguage((Language)combo_language.getData(combo_language.getText()));

        configurator.setDataSet_landing_page(text_dataset_landingPage.getText());

        configurator.setDataset_download_link(uri_datasetDownloadLink.getText());

        configurator.setBase_url(base_url.getText());

        configurator.setRecommended_citation(text_recommendedCitation.getText());

        configurator.setLocation(text_location.getText());

        configurator.setKeywords(text_keyWords.getText());

        configurator.setLicence(text_licence.getText());
    }



}
