/**
 * Copyright (C) 2007 EDIT
 * European Distributed Institute of Taxonomy
 * http://www.e-taxonomy.eu
 *
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */

package eu.etaxonomy.taxeditor.io.e4.out.csv;

import java.io.File;
import java.util.HashSet;
import java.util.List;
import java.util.UUID;

import javax.inject.Inject;

import org.eclipse.core.runtime.jobs.Job;
import org.eclipse.e4.core.contexts.IEclipseContext;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.ui.progress.IProgressConstants;

import eu.etaxonomy.cdm.io.csv.redlist.demo.CsvDemoExportConfigurator;
import eu.etaxonomy.cdm.model.taxon.Classification;
import eu.etaxonomy.taxeditor.io.e4.out.AbstractExportWizardE4;
import eu.etaxonomy.taxeditor.io.wizard.ExportToFileDestinationWizardPage;
import eu.etaxonomy.taxeditor.store.CdmStore;

/**
 * @author a.oppermann
 * @created Nov. 11, 2014
 * @version 1.0
 */
public class CsvExportWizardE4 extends AbstractExportWizardE4<CsvDemoExportConfigurator> {

	private CsvDemoExportConfigurator configurator;
	private ExportToFileDestinationWizardPage page;

    @Inject
    public CsvExportWizardE4(IEclipseContext context) {
        super(context);
    }
	@Override
	public void init() {
		configurator = CsvDemoExportConfigurator.NewInstance(null, null);
	}

	@Override
	public CsvDemoExportConfigurator getConfigurator() {
		return configurator;
	}

	@Override
	public boolean performFinish() {
		String urlString = page.getFolderText() + File.separator
				+ page.getExportFileName();

		final Combo combo = page.getCombo();
		final List<Classification> listClassifications = CdmStore.getCurrentApplicationConfiguration().getClassificationService().listClassifications(null, null, null, null);
		if(combo != null){
		    int selectionIndex = combo.getSelectionIndex();
		    HashSet<UUID> set = new HashSet<UUID>();
		    if(selectionIndex == -1){
		        for(Classification c:listClassifications){
		            set.add(c.getUuid());
		        }
		    }else{
		        for(Classification c:listClassifications){
		            if(c.getTitleCache().equalsIgnoreCase(combo.getItem(selectionIndex))){
		                set.add(c.getUuid());
		            }
		        }
		        configurator.setClassificationUuids(set);
		        configurator.setDoTaxa(true);
		        configurator.setHasHeaderLines(true);
		        configurator.setClassification(true);
		        configurator.setTaxonName(true);
		        configurator.setAuthor(true);
		        configurator.setRank(true);
		        configurator.setLastChange(true);
		    }
		}

	    // create job
	    Job job = CdmStore.getExportManager().createIOServiceJob(configurator, new File(urlString));
	    // configure the job
	    job.setProperty(IProgressConstants.KEEP_PROPERTY, true);
	    job.setUser(true);
	    // schedule job
	    job.schedule();

		return true;
	}

	@Override
	public void addPages() {
	    //TODO create page with drop down menu for export for single classification.
//		super.addPages();


		//standard page
		page =  ExportToFileDestinationWizardPage.Csv(configurator);

		addPage(page);
	}
}
