/**
 * Copyright (C) 2007 EDIT
 * European Distributed Institute of Taxonomy
 * http://www.e-taxonomy.eu
 *
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */

package eu.etaxonomy.taxeditor.io.e4.out.csv.name;

import java.io.File;

import javax.inject.Inject;
import javax.inject.Named;

import org.eclipse.core.runtime.jobs.Job;
import org.eclipse.e4.core.contexts.IEclipseContext;
import org.eclipse.e4.core.di.annotations.Optional;
import org.eclipse.e4.ui.services.IServiceConstants;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.TreeSelection;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.ui.progress.IProgressConstants;

import eu.etaxonomy.cdm.io.csv.caryophyllales.out.CsvNameExportConfigurator;
import eu.etaxonomy.cdm.persistence.dto.TaxonNodeDto;
import eu.etaxonomy.taxeditor.io.e4.out.AbstractExportWizardE4;
import eu.etaxonomy.taxeditor.io.wizard.ExportToFileDestinationWizardPage;
import eu.etaxonomy.taxeditor.store.CdmStore;

/**
 * @author k.luther
 * @created Apr. 12, 2015
 * @version 1.0
 */
public class CsvNameExportWizardE4 extends AbstractExportWizardE4<CsvNameExportConfigurator> {


    protected CsvNameExportConfigurator configurator;
	protected ExportToFileDestinationWizardPage page;

    @Inject
	public CsvNameExportWizardE4(IEclipseContext context,
            @Optional@Named(IServiceConstants.ACTIVE_SELECTION)IStructuredSelection selection) {
	    super(context, selection);
	}

	@Override
	public void init() {
		configurator = CsvNameExportConfigurator.NewInstance(null,null);
		configurator.setNamesOnly(true);
		if (selection instanceof TreeSelection && !selection.isEmpty()){
            TaxonNodeDto node = (TaxonNodeDto)selection.getFirstElement();
            configurator.setClassificationUUID(node.getClassificationUUID());
        }
	}

	@Override
	public CsvNameExportConfigurator getConfigurator() {
		return configurator;
	}

	@Override
	public boolean performFinish() {
	    String urlString = page.getFolderText() + File.separator
	            + page.getExportFileName();

	    final Combo combo = page.getCombo();
	    configurator.setClassificationUUID(page.getSelectedClassificationUUID());

        // create job
        Job job = CdmStore.getExportManager().createIOServiceJob(configurator, new File(urlString));
        // configure the job
        job.setProperty(IProgressConstants.KEEP_PROPERTY, true);
        job.setUser(true);
        // schedule job
        job.schedule();

		return true;
	}

	@Override
	public void addPages() {
	    //TODO create page with drop down menu for export for single classification.
//		super.addPages();


		//standard page
		page =  ExportToFileDestinationWizardPage.CsvNames(configurator);

		addPage(page);
	}
}

