/**
 * Copyright (C) 2007 EDIT
 * European Distributed Institute of Taxonomy
 * http://www.e-taxonomy.eu
 *
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */

package eu.etaxonomy.taxeditor.io.e4.out.dwca;

import java.io.File;
import java.util.Iterator;
import java.util.UUID;

import javax.inject.Inject;
import javax.inject.Named;

import org.eclipse.e4.core.contexts.IEclipseContext;
import org.eclipse.e4.core.di.annotations.Optional;
import org.eclipse.e4.ui.services.IServiceConstants;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.TreeSelection;

import eu.etaxonomy.cdm.filter.TaxonNodeFilter;
import eu.etaxonomy.cdm.io.dwca.out.DwcaTaxExportConfigurator;
import eu.etaxonomy.cdm.persistence.dto.TaxonNodeDto;
import eu.etaxonomy.taxeditor.io.e4.out.AbstractExportWizardE4;
import eu.etaxonomy.taxeditor.io.wizard.ExportToFileDestinationWizardPage;
import eu.etaxonomy.taxeditor.store.CdmStore;

/**
 *
 * @author pplitzner
 * @since Oct 6, 2017
 *
 */
public class DarwinCoreArchiveExportWizardE4 extends
		AbstractExportWizardE4<DwcaTaxExportConfigurator> {

    private DwcaTaxExportConfigurator configurator;
	private ExportToFileDestinationWizardPage page;

    @Inject
	public DarwinCoreArchiveExportWizardE4(IEclipseContext context,
	        @Optional@Named(IServiceConstants.ACTIVE_SELECTION)IStructuredSelection selection) {
	    super(context, selection);
	}

	@Override
	public void init() {
		configurator = DwcaTaxExportConfigurator.NewInstance(null, null, null);
		if (selection instanceof TreeSelection && !selection.isEmpty()){
			Iterator it = selection.iterator();

			while(it.hasNext()){
				Object obj = it.next();
				if (obj instanceof TaxonNodeDto){
					TaxonNodeDto node = (TaxonNodeDto) obj;
					if (node.getParentUUID() == null){
						if (configurator.getTaxonNodeFilter() == null){
							configurator.setTaxonNodeFilter(TaxonNodeFilter.NewClassificationInstance(node.getClassificationUUID()));
						}else{
							configurator.getTaxonNodeFilter().orClassification(node.getClassificationUUID());
						}
					}else{
						if (configurator.getTaxonNodeFilter() == null){
							configurator.setTaxonNodeFilter(TaxonNodeFilter.NewSubtreeInstance(node.getUuid()));
						}else{
							configurator.getTaxonNodeFilter().orSubtree(node.getUuid());
						}

					}
				}
			}

		}
	}

	@Override
	public DwcaTaxExportConfigurator getConfigurator() {
		return configurator;
	}

	@Override
	public boolean performFinish() {
		String urlString = page.getFolderText() + File.separator
				+ page.getExportFileName();

		if (!page.getCheckUseSelectedTaxonNode()){
			configurator.getTaxonNodeFilter().reset();
			if (!page.getCheckUseAllClassifications()){
				configurator.getTaxonNodeFilter().orClassification(page.getSelectedClassificationUUID());
			}else{
				for (UUID classificationUuid: page.getAllClassificationUuids()){
					configurator.getTaxonNodeFilter().orClassification(classificationUuid);
				}

			}
		}
		CdmStore.getExportManager().runMoniteredOperation(configurator, urlString);
		return true;
	}

	@Override
	public void addPages() {
		page = ExportToFileDestinationWizardPage.Dwca(configurator);
		addPage(page);
		super.addPages();
	}
}
