/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.io.e4.out.jaxb;

import java.io.File;

import javax.inject.Inject;

import org.eclipse.core.runtime.jobs.Job;
import org.eclipse.e4.core.contexts.IEclipseContext;
import org.eclipse.ui.progress.IProgressConstants;

import eu.etaxonomy.cdm.io.jaxb.JaxbExportConfigurator;
import eu.etaxonomy.taxeditor.io.e4.out.AbstractExportWizardE4;
import eu.etaxonomy.taxeditor.io.wizard.ExportToFileDestinationWizardPage;
import eu.etaxonomy.taxeditor.store.CdmStore;

/**
 *
 * @author pplitzner
 * @since Oct 6, 2017
 *
 */
public class JaxbExportWizardE4 extends AbstractExportWizardE4<JaxbExportConfigurator> {

	private ExportToFileDestinationWizardPage page;

	private JaxbExportConfigurator configurator;

    @Inject
    public JaxbExportWizardE4(IEclipseContext context) {
        super(context);
    }

	/** {@inheritDoc} */
	@Override
    public void init() {
		this.setWindowTitle("JAXB Export");
		this.configurator = CdmStore.getExportManager().JaxbConfigurator();
	}

	/** {@inheritDoc} */
	@Override
	public boolean performFinish() {
		boolean performFinish = false;

		File exportFile = new File(page.getFolderText() + File.separator + page.getExportFileName());

		configurator.setDestination(exportFile);

	    // create job
	    Job job = CdmStore.getExportManager().createIOServiceJob(configurator, exportFile);
	    // configure the job
	    job.setProperty(IProgressConstants.KEEP_PROPERTY, true);
	    job.setUser(true);
	    // schedule job
	    job.schedule();

		performFinish = true;
		return performFinish;
	}

	/** {@inheritDoc} */
	@Override
	public void addPages() {
		page = ExportToFileDestinationWizardPage.Jaxb(configurator);
		addPage(page);
		super.addPages();
	}

	/** {@inheritDoc} */
	@Override
	public JaxbExportConfigurator getConfigurator() {
		return configurator;
	}


}
