/**
 * Copyright (C) 2017 EDIT
 * European Distributed Institute of Taxonomy
 * http://www.e-taxonomy.eu
 *
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */
package eu.etaxonomy.taxeditor.io.e4.out.owl;

import java.io.File;

import javax.inject.Inject;
import javax.inject.Named;

import org.eclipse.core.runtime.jobs.Job;
import org.eclipse.e4.core.contexts.IEclipseContext;
import org.eclipse.e4.core.di.annotations.Optional;
import org.eclipse.e4.ui.services.IServiceConstants;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.ui.progress.IProgressConstants;

import eu.etaxonomy.cdm.io.descriptive.owl.out.StructureTreeOwlExportConfigurator;
import eu.etaxonomy.taxeditor.io.e4.out.AbstractExportWizardE4;
import eu.etaxonomy.taxeditor.store.CdmStore;

/**
 *
 * @author pplitzner
 * @since May 23, 2019
 *
 */
public class OwlTermExportWizard extends
AbstractExportWizardE4<StructureTreeOwlExportConfigurator> {

    private StructureTreeOwlExportConfigurator configurator;
    private OwlTermExportWizardPage exportPage;

    @Inject
    public OwlTermExportWizard(IEclipseContext context,
            @Optional@Named(IServiceConstants.ACTIVE_SELECTION)IStructuredSelection selection) {
        super(context, selection);
    }

    @Override
    public void init() {
        configurator = StructureTreeOwlExportConfigurator.NewInstance();
    }

    @Override
    public StructureTreeOwlExportConfigurator getConfigurator() {
        return configurator;
    }

    @Override
    public boolean performFinish() {
        configurator.setFeatureTreeUuids(exportPage.getSelectedTrees());
        configurator.setVocabularyUuids(exportPage.getSelectedVocabularies());
        configurator.setDestination(new File(exportPage.getExportDirectory() + File.separator+ "export.owl"));

        if(exportPage.getExportDirectory()!=null){
            // create job
            Job job = CdmStore.getExportManager().createIOServiceJob(configurator, new File(exportPage.getExportDirectory()+ File.separator+ "export.owl"));
            // configure the job
            job.setProperty(IProgressConstants.KEEP_PROPERTY, true);
            job.setUser(true);
            // schedule job
            job.schedule();
        }
        return true;
    }

    @Override
    public void addPages() {
        exportPage = new OwlTermExportWizardPage("OWL term export");
        addPage(exportPage);
    }

}
