/**
* Copyright (C) 2019 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.io.e4.out.owl;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;
import java.util.UUID;

import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.DirectoryDialog;

import eu.etaxonomy.cdm.api.service.ITermTreeService;
import eu.etaxonomy.cdm.api.service.IVocabularyService;
import eu.etaxonomy.cdm.common.CdmUtils;
import eu.etaxonomy.cdm.model.term.TermTree;
import eu.etaxonomy.cdm.model.term.TermType;
import eu.etaxonomy.cdm.persistence.dto.TermVocabularyDto;
import eu.etaxonomy.taxeditor.preference.PreferencesUtil;
import eu.etaxonomy.taxeditor.store.CdmStore;

/**
 * @author pplitzner
 * @since May 23, 2019
 */
public class OwlTermExportWizardPage extends WizardPage {

    private OwlTermExportComposite composite;
    private DirectoryDialog folderDialog;
    private static final String EXPORT_FOLDER ="EXPORT_FOLDER";
    protected OwlTermExportWizardPage(String pageName) {
        super(pageName);
        setTitle("OWL Term Export");
        setDescription("Select the objects to export");
    }

    @Override
    public void createControl(Composite parent) {
        composite = new OwlTermExportComposite(parent, SWT.NONE);

        Set<TermType> termTypes = new HashSet<>();
        termTypes.add(TermType.Feature);
        termTypes.add(TermType.Structure);
        termTypes.add(TermType.Property);
        termTypes.add(TermType.State);
        termTypes.add(TermType.Character);
        List<TermVocabularyDto> areaVocabularies = CdmStore.getService(IVocabularyService.class).findVocabularyDtoByTermTypes(termTypes);
        composite.getSelectVocabularyComposite().getViewer().setInput(areaVocabularies);
        composite.getSelectVocabularyComposite().getViewer().addCheckStateListener(e->updateButtons());

        List<TermTree> trees = CdmStore.getService(ITermTreeService.class).list(TermTree.class, null, null, null, null);
        composite.getSelectFeatureTreeComposite().getViewer().setInput(trees);
        composite.getSelectFeatureTreeComposite().getViewer().addCheckStateListener(e->updateButtons());

        folderDialog = new DirectoryDialog(parent.getShell());
        String exportFolder = PreferencesUtil.getStringValue(EXPORT_FOLDER, true);
        folderDialog.setFilterPath(exportFolder);
        composite.getBtnSelectDirectory().addSelectionListener(new SelectionAdapter() {
            @Override
            public void widgetSelected(SelectionEvent e) {
                super.widgetSelected(e);

                String path = folderDialog.open();
                if (path != null) { // a folder was selected
                    composite.getTxtExportDirectory().setText(path);
                    PreferencesUtil.setStringValue(EXPORT_FOLDER, path);
                    updateButtons();
                }


//                FileDialog dialog = new FileDialog(parent.getShell());
//                String[] filterNames = {
//                        "Web Ontology Language (*.owl)",
//                        "Extensible Markup Language (*.xml)",
//                        "All files (*.*)"
//                        };
//                String[] filterExtensions = { "*.owl", "*.xml", "*.*"};
//
//                dialog.setOverwrite(true);
//                dialog.setFilterNames(filterNames);
//                dialog.setFilterExtensions(filterExtensions);
//                String fileString = dialog.open();
//                if(fileString!=null){
//                    composite.getTxtExportDirectory().setText(fileString);
//
//                }
            }
        });
        setControl(composite);
    }

    private void updateButtons(){
        this.getWizard().getContainer().updateButtons();
    }

    String getExportDirectory(){
        return composite.getExportDirectory();
    }

    List<UUID> getSelectedTrees(){
        List<UUID> selectedTrees = new ArrayList<>();
        Object[] checkedElements = composite.getSelectFeatureTreeComposite().getViewer().getCheckedElements();
        for (Object object : checkedElements) {
            if(object instanceof TermTree) {
                selectedTrees.add(((TermTree) object).getUuid());
            }
        }
        return selectedTrees;
    }

    List<UUID> getSelectedVocabularies(){
        List<UUID> vocs = new ArrayList<>();
        Object[] checkedElements = composite.getSelectVocabularyComposite().getViewer().getCheckedElements();
        for (Object object : checkedElements) {
            if(object instanceof TermVocabularyDto){
                vocs.add(((TermVocabularyDto) object).getUuid());
            }
        }
        return vocs;
    }

    @Override
    public boolean isPageComplete() {
        return !getSelectedVocabularies().isEmpty()
                || !getSelectedTrees().isEmpty()
                || CdmUtils.isNotBlank(getExportDirectory());
    }

}
