/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.io.e4.out.sdd;

import java.io.File;

import javax.inject.Inject;

import org.eclipse.core.runtime.jobs.Job;
import org.eclipse.e4.core.contexts.IEclipseContext;
import org.eclipse.ui.progress.IProgressConstants;

import eu.etaxonomy.cdm.io.sdd.out.SDDExportConfigurator;
import eu.etaxonomy.taxeditor.io.e4.out.AbstractExportWizardE4;
import eu.etaxonomy.taxeditor.io.wizard.ExportToFileDestinationWizardPage;
import eu.etaxonomy.taxeditor.store.CdmStore;

/**
 *
 * @author pplitzner
 * @since Oct 6, 2017
 *
 */
public class SddExportWizardE4 extends AbstractExportWizardE4<SDDExportConfigurator> {

	private ExportToFileDestinationWizardPage page;

	private SDDExportConfigurator configurator;

    @Inject
    public SddExportWizardE4(IEclipseContext context) {
        super(context);
    }

	/** {@inheritDoc} */
	@Override
	public SDDExportConfigurator getConfigurator() {
		return configurator;
	}

	/** {@inheritDoc} */
	@Override
	public boolean performFinish() {
	    File file = new File(page.getFolderText() + File.separator + page.getExportFileName());
        // create job
        Job job = CdmStore.getExportManager().createIOServiceJob(configurator, file);
        // configure the job
        job.setProperty(IProgressConstants.KEEP_PROPERTY, true);
        job.setUser(true);
        // schedule job
        job.schedule();

	    return true;
	}

	/** {@inheritDoc} */
	@Override
	public void addPages() {
		page = ExportToFileDestinationWizardPage.Sdd(configurator);
		addPage(page);
		super.addPages();
	}

	/** {@inheritDoc} */
	@Override
    public void init() {
		this.setWindowTitle("SDD Export");
		this.configurator = CdmStore.getExportManager().SddConfigurator();
	}
}
