/**
 *
 */
package eu.etaxonomy.taxeditor.labels;

import org.eclipse.swt.graphics.Image;

import eu.etaxonomy.cdm.model.name.NameRelationshipType;
import eu.etaxonomy.cdm.model.name.NomenclaturalCode;
import eu.etaxonomy.cdm.model.name.TaxonName;
import eu.etaxonomy.taxeditor.preference.PreferencesUtil;

/**
 * Returns labels and images that are dependent on a nomenclatural
 * code using a code-specific ILabelImageStrategy.
 *
 * @author p.ciardelli
 */
public class LabelImageProvider implements ILabelImageStrategy {

	private ILabelImageStrategy strategy;

	/**
	 * <p>getLabelStrategy</p>
	 *
	 * @param name a {@link eu.etaxonomy.cdm.model.name.TaxonNameBase} object.
	 * @return a {@link eu.etaxonomy.taxeditor.labels.ILabelImageStrategy} object.
	 */
	public static ILabelImageStrategy getLabelStrategy(TaxonName name) {
		if (name == null) {
			if (PreferencesUtil.getPreferredNomenclaturalCode().
					equals(NomenclaturalCode.ICZN)) {
				return new ZoologicalNameLabelStrategy();
			}
		} else if (name.getNameType() != null &&
				name.getNameType().equals(NomenclaturalCode.ICZN)) {
			return new ZoologicalNameLabelStrategy();
		}
		return new DefaultLabelStrategy();
	}

	/**
	 * <p>Constructor for LabelImageProvider.</p>
	 *
	 * @param strategy a {@link eu.etaxonomy.taxeditor.labels.ILabelImageStrategy} object.
	 */
	public LabelImageProvider(ILabelImageStrategy strategy) {
		this.strategy = strategy;
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.editor.labels.IEditorLabelStrategy#getNameRelationTypeLabel(eu.etaxonomy.cdm.model.name.NameRelationshipType)
	 */
	/** {@inheritDoc} */
	@Override
    public String getNameRelationTypeLabel(NameRelationshipType type) {
		return strategy.getNameRelationTypeLabel(type);
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.editor.labels.IEditorLabelStrategy#getNameRelationInverseTypeLabel(eu.etaxonomy.cdm.model.name.NameRelationshipType)
	 */
	/** {@inheritDoc} */
	@Override
    public String getNameRelationTypeInverseLabel(NameRelationshipType type) {
		return strategy.getNameRelationTypeInverseLabel(type);
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.editor.labels.ILabelImageStrategy#getNameRelationImage(eu.etaxonomy.cdm.model.name.NameRelationship)
	 */
	/** {@inheritDoc} */
	@Override
    public Image getNameRelationTypeImage(NameRelationshipType type) {
		return strategy.getNameRelationTypeImage(type);
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.labels.ILabelImageStrategy#getNameRelationDescription(eu.etaxonomy.cdm.model.name.NameRelationshipType)
	 */
	/** {@inheritDoc} */
	@Override
    public String getNameRelationTypeDescription(NameRelationshipType type) {
		return strategy.getNameRelationTypeDescription(type);
	}
}
