/**
 *
 */
package eu.etaxonomy.taxeditor.model;

import java.io.Serializable;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.NullProgressMonitor;

import eu.etaxonomy.taxeditor.store.StoreUtil;

/**
 * <p>CdmProgressMonitorAdapter class.</p>
 *
 * @author n.hoffmann
 * @created Sep 17, 2010
 */
public class CdmProgressMonitorAdapter implements eu.etaxonomy.cdm.common.monitor.IProgressMonitor, Serializable {

    private static final long serialVersionUID = 2890278753937078663L;
    private IProgressMonitor progressMonitor;

	private CdmProgressMonitorAdapter (IProgressMonitor monitor){
		if(monitor == null){
			monitor = new NullProgressMonitor();
		}

		this.progressMonitor = monitor;
	}

	/**
	 * <p>CreateMonitor</p>
	 *
	 * @param monitor a {@link org.eclipse.core.runtime.IProgressMonitor} object.
	 * @return a {@link eu.etaxonomy.taxeditor.model.CdmProgressMonitorAdapter} object.
	 */
	public static CdmProgressMonitorAdapter CreateMonitor (IProgressMonitor monitor){
		return new CdmProgressMonitorAdapter(monitor);
	}

	/**
	 * <p>CreateSubMonitor</p>
	 *
	 * @param monitor a {@link org.eclipse.core.runtime.IProgressMonitor} object.
	 * @param ticks a int.
	 * @return a {@link eu.etaxonomy.taxeditor.model.CdmProgressMonitorAdapter} object.
	 */
	public static CdmProgressMonitorAdapter CreateSubMonitor (IProgressMonitor monitor, int ticks) {
		IProgressMonitor subMonitor = StoreUtil.getSubProgressMonitor(monitor, ticks);
		return new CdmProgressMonitorAdapter(subMonitor);
	}

	/** {@inheritDoc} */
	@Override
	public void beginTask(String name, int totalWork) {
		progressMonitor.beginTask(name, totalWork);
	}

	/** {@inheritDoc} */
	@Override
	public void done() {
		progressMonitor.done();
	}

	/** {@inheritDoc} */
	@Override
	public boolean isCanceled() {
		return progressMonitor.isCanceled();
	}

	/** {@inheritDoc} */
	@Override
	public void setCanceled(boolean value) {
		progressMonitor.setCanceled(value);
	}

	/** {@inheritDoc} */
	@Override
	public void setTaskName(String name) {
		progressMonitor.setTaskName(name);
	}

	/** {@inheritDoc} */
	@Override
	public void subTask(String name) {
		progressMonitor.subTask(name);
	}

	/** {@inheritDoc} */
	@Override
	public void worked(int work) {
		progressMonitor.worked(work);
	}

	/** {@inheritDoc} */
	@Override
	public void warning(String message) {
		MessagingUtils.warn(this.getClass(), message);
	}

	/** {@inheritDoc} */
	@Override
	public void warning(String message, Throwable throwable) {
		MessagingUtils.error(this.getClass(), message, throwable);
	}

	@Override
	public void internalWorked(double arg0) {
		progressMonitor.internalWorked(arg0);
	}

    /**
     * {@inheritDoc}
     */
    @Override
    public Serializable getFeedback() {
        return null;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public boolean getIsWaitingForFeedback() {
        return false;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public void setFeedback(Serializable arg0) {

    }

    /**
     * {@inheritDoc}
     */
    @Override
    public void waitForFeedback() {

    }

    /**
     * {@inheritDoc}
     */
    @Override
    public String getOwner() {
        return null;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public boolean hasFeedbackWaitTimedOut() {
        return false;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public void interrupt() {


    }

    /**
     * {@inheritDoc}
     */
    @Override
    public void setOwner(String arg0) {


    }

    /**
     * {@inheritDoc}
     */
    @Override
    public void waitForFeedback(long arg0) {

    }

}
