/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.model;

import org.eclipse.jface.resource.ColorRegistry;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.RGB;

import eu.etaxonomy.taxeditor.preference.Resources;

/**
 * @author pplitzner
 * @since Nov 1, 2017
 */
public class ColorResources {

	private static ColorResources instance;

	public static ColorResources getInstance(){
		if(instance == null){
			instance = new ColorResources();
		}
		return instance;
	}

	private static ColorRegistry colorRegistry;

	public ColorRegistry getColorRegistry() {
        if (colorRegistry == null) {
            colorRegistry = new ColorRegistry();
            initializeColorRegistry(colorRegistry);
        }
        return colorRegistry;
    }

	public static Color getColor(String key){
	    return getInstance().getColorRegistry().get(key);
	}

	protected void initializeColorRegistry(ColorRegistry registry) {
	    registerColor(registry, Resources.COLOR_SWT_BG_DEFAULT, 214, 214, 214);
	    registerColor(registry, Resources.COLOR_MATRIX_SUMMARY_BG, 255, 255, 153);
	    registerColor(registry, Resources.COLOR_BULK_EDITOR_CANDIDATE, 243, 240, 165);
	    registerColor(registry, Resources.COLOR_BULK_EDITOR_TARGET, 135, 255, 135);
	    registerColor(registry, Resources.COLOR_COMPOSITE_BACKGROUND, 255,255,255);
	    registerColor(registry, Resources.COLOR_CONTROL_BACKGROUND, 220, 220, 220);
	    registerColor(registry, Resources.COLOR_CONTROL_SELECTED, 210, 210, 210);
	    registerColor(registry, Resources.COLOR_CONTROL_SELECTED_FOCUS, 232,242,254);

	    //colors1 and 2 are computed as next neutral color from the combined color
	    //light colors are computed as second last color tints
	    //see https://htmlcolorcodes.com/color-picker/
		registerColor(registry, Resources.COLOR_COMPOSITE_IRRELEVANT1, 241,156,18);
		registerColor(registry, Resources.COLOR_COMPOSITE_IRRELEVANT1_LIGHT, 251, 230, 196);
        registerColor(registry, Resources.COLOR_COMPOSITE_IRRELEVANT2, 241, 190, 18);
        registerColor(registry, Resources.COLOR_COMPOSITE_IRRELEVANT2_LIGHT, 251, 239, 196);
        registerColor(registry, Resources.COLOR_COMPOSITE_IRRELEVANT1_2, 241, 140, 18);
        registerColor(registry, Resources.COLOR_COMPOSITE_IRRELEVANT1_2_LIGHT, 251, 226, 196);

        registerColor(registry, Resources.SEARCH_VIEW_FOREGROUND, 192, 192, 192);
		registerColor(registry, Resources.SEARCH_VIEW_FOCUS, 0, 0, 0);
		registerColor(registry, Resources.BLACK, 0, 0, 0);
		registerColor(registry, Resources.COLOR_LIST_BACKGROUND, 150,150,150);
		registerColor(registry, Resources.COLOR_FONT_DEFAULT, 0,0,0);
		registerColor(registry, Resources.COLOR_TEXT_DISABLED, 150,150,150);
		registerColor(registry, Resources.COLOR_TEXT_DISABLED_BACKGROUND, 230,230,230);
		registerColor(registry, Resources.COLOR_DRAG_ENTER, 185,255,133);
		registerColor(registry, Resources.COLOR_LIST_EVEN, 255,255,255);
		registerColor(registry, Resources.COLOR_LIST_ODD, 230,230,230);
		registerColor(registry, Resources.COLOR_PARSE_ERROR, 255,0,0);
		registerColor(registry, Resources.COLOR_DISABLED_EDITOR, 105,114,124);
		registerColor(registry, Resources.COLOR_EDITOR_ERROR, 255,0,0);
	}

	private void registerColor(ColorRegistry registry, String key,
	        int red, int green, int blue) {
	    registry.put(key, new RGB(red, green, blue));
	}
}
