/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.model;


/**
 * Interface for handling the current "dirty" state of an object. An object is
 * dirty when changes are done but not saved/persisted yet.
 * 
 * @author n.hoffmann
 * @created Mar 8, 2010
 * @version 1.0
 */
public interface IDirtyMarkable {

	/**
	 * Sets the implementor to the "dirty" state specifying the object that was changed.
	 *
	 * @param element the element that was changed causing the "dirty" state.
	 */
	public abstract void changed(Object element);

	/**
	 * Forces the implementing class to be set to the "dirty" state.
	 */
	public abstract void forceDirty();
}
