/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.model;

import eu.etaxonomy.cdm.hibernate.HibernateProxyHelper;
import eu.etaxonomy.cdm.model.name.TaxonName;
import eu.etaxonomy.cdm.model.taxon.TaxonBase;

/**
 * <p>NameHelper class.</p>
 *
 * @author p.ciardelli
 * @author n.hoffmann
 * @created 18.03.2009
 * @version 1.0
 */
public class NameHelper {	

	/**
	 * <p>getDisplayName</p>
	 *
	 * @param name a {@link eu.etaxonomy.cdm.model.name.TaxonNameBase} object.
	 * @return a {@link java.lang.String} object.
	 */
	public static String getDisplayName(TaxonName name){
		return name.getTitleCache();
	}
	
	/**
	 * <p>getDisplayName</p>
	 *
	 * @param name a {@link eu.etaxonomy.cdm.model.name.TaxonNameBase} object.
	 * @return a {@link java.lang.String} object.
	 */
	public static String getDisplayNameCache(TaxonBase taxon){
		return taxon.getName().getNameCache();
	}
	/**
	 * <p>getDisplayName</p>
	 *
	 * @param taxon a {@link eu.etaxonomy.cdm.model.taxon.TaxonBase} object.
	 * @return a {@link java.lang.String} object.
	 */
	public static String getDisplayName(TaxonBase<?> taxon){
		return getDisplayName(taxon.getName());
	}
	
	/**
	 * <p>getDisplayNameWithRef</p>
	 *
	 * @param taxonBase a {@link eu.etaxonomy.cdm.model.taxon.TaxonBase} object.
	 * @return a {@link java.lang.String} object.
	 */
	public static String getDisplayNameWithRef(TaxonBase taxonBase) {
		TaxonName name = HibernateProxyHelper.deproxy(taxonBase.getName());
		if (name != null) {
			String fullTitle = name.generateFullTitle();
			if (fullTitle != null){  //should never be null
				fullTitle = fullTitle.replaceAll("(\\r|\\n|\\r\\n)", "");  //replaces LineBreaks, maybe we should include U+000C, U+0085, U+2028, U+2029 which are also linebreaks according to UNICODE stardard.
			}
			return fullTitle;
		}
		return "";
	}	
}
