/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.model;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import eu.etaxonomy.cdm.model.name.NomenclaturalCode;
import eu.etaxonomy.taxeditor.preference.PreferencesUtil;

/**
 * @author p.ciardelli
 * @created 11.09.2009
 */
public class NomenclaturalCodeHelper {

	public static List<NomenclaturalCode> getAllCodes() {
		return Arrays.asList(NomenclaturalCode.values());
	}

	public static List<NomenclaturalCode> getSupportedCodes() {
		List<NomenclaturalCode> supportedCodes = new ArrayList<>();
		NomenclaturalCode nomenclaturalCode =PreferencesUtil.getPreferredNomenclaturalCode();
		for (NomenclaturalCode code : getAllCodes()) {
			if (isSupported(code)) {
			    if (nomenclaturalCode != null && nomenclaturalCode.equals(code)){
			        supportedCodes.add(0, code);
			    }else{
			        supportedCodes.add(code);
			    }
			}
		}
		return supportedCodes;
	}

	private static boolean isSupported(NomenclaturalCode code) {
		if (code.equals(NomenclaturalCode.ICNAFP ) || code.equals(NomenclaturalCode.ICZN)) {
			return true;
		} else {
			return false;
		}
	}

	public static String getDescription(NomenclaturalCode code) {
		if (code.equals(NomenclaturalCode.ICNP)) {
			return "International Code of Nomenclature of Prokaryotes (ICNP)";
		}
		if (code.equals(NomenclaturalCode.ICNAFP )) {
			return "International Code of Botanical Nomenclature (ICNAFP )";
		}
		if (code.equals(NomenclaturalCode.ICNCP)) {
			return "International Code of Cultivated Plants (ICNCP)";
		}
		if (code.equals(NomenclaturalCode.ICZN)) {
			return "International Code of Zoological Nomenclature (ICZN)";
		}
		if (code.equals(NomenclaturalCode.ICVCN)) {
			return "International Code for Virus Classification and Nomenclature (ICVCN)";
		}
		return code.name();
	}

	public static NomenclaturalCode getDefaultCode() {
		return NomenclaturalCode.ICNAFP ;
	}
}