package eu.etaxonomy.taxeditor.model;

import org.eclipse.swt.dnd.ByteArrayTransfer;
import org.eclipse.swt.dnd.TransferData;

import eu.etaxonomy.cdm.model.taxon.TaxonBase;

/**
 * For drag and drop - a drag widget with this transfer type can only be dropped onto
 *  a drop widget with the same type.
 *
 * @author p.ciardelli
 */
public class TaxonTransfer extends ByteArrayTransfer {

	private static final String TAXONTYPENAME = "taxon_type";
	private static final int TAXONTYPEID = registerType (TAXONTYPENAME);
	private static final TaxonTransfer INSTANCE = new TaxonTransfer();

	/*
	 * The object associated with this transfer event
	 */
	private TaxonBase<?> taxonBase;

	/*
	 * Returns the singleton
	 */
	/**
	 * <p>getInstance</p>
	 *
	 * @return a {@link eu.etaxonomy.taxeditor.model.TaxonTransfer} object.
	 */
	public static TaxonTransfer getInstance() {
		return INSTANCE;
	}
	/*
	 * Set transfer data for local use
	 */
	/**
	 * <p>setTaxon</p>
	 *
	 * @param taxon a {@link eu.etaxonomy.cdm.model.taxon.TaxonBase} object.
	 */
	public void setTaxon(TaxonBase<?> taxon) {
		this.taxonBase = taxon;
	}
	/*
	 * Returns the local transfer data
	 */
	/**
	 * <p>getTaxon</p>
	 *
	 * @return a {@link eu.etaxonomy.cdm.model.taxon.TaxonBase} object.
	 */
	public TaxonBase<?> getTaxon() {
		return taxonBase;
	}

	/*
	 * The type ID is used to identify this transfer
	 */
	@Override
	protected int[] getTypeIds() {
		return new int[] { TAXONTYPEID };
	}

	@Override
	protected String[] getTypeNames() {
		return new String[] { TAXONTYPENAME } ;
	}

	@Override
	protected void javaToNative(Object object, TransferData transferData) {
		// No encoding needed since this is a hardcoded string read and written
		// in the same process.
		// See nativeToJava below
		byte[] check = TAXONTYPENAME.getBytes();
		super.javaToNative(check, transferData);
	}

	@Override
	protected Object nativeToJava(TransferData transferData) {
		Object result = super.nativeToJava(transferData);
		if (isInvalidNativeType(result)) {
			throw new RuntimeException(); //$NON-NLS-1$
		}
		return taxonBase;
	}

	private boolean isInvalidNativeType(Object result) {
		// No encoding needed since this is a hardcoded string read and written
		// in the same process.
		// See javaToNative above
		return !(result instanceof byte[])
				|| !TAXONTYPENAME.equals(new String((byte[]) result));
	}
}
