/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.newWizard;

import org.eclipse.e4.ui.workbench.IWorkbench;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.wizard.Wizard;
import org.eclipse.swt.widgets.Display;

import eu.etaxonomy.cdm.api.conversation.ConversationHolder;
import eu.etaxonomy.cdm.api.conversation.IConversationEnabled;
import eu.etaxonomy.cdm.persistence.hibernate.CdmDataChangeMap;
import eu.etaxonomy.taxeditor.store.CdmStore;
import eu.etaxonomy.taxeditor.ui.element.CdmFormFactory;

/**
 * @author n.hoffmann
 * @created Jun 1, 2010
 */
public abstract class AbstractNewEntityWizard<T>
        extends Wizard
        implements IConversationEnabled {

	protected ConversationHolder conversation;

	protected CdmFormFactory formFactory;

	private T entity;

	protected IStructuredSelection selection;

	public AbstractNewEntityWizard(){
		setWindowTitle(String.format("New %s", getEntityName()));
	}

	/**
	 * FIXME there might be a smarter way to do this,
	 */
	protected abstract String getEntityName();

	@Override
	public boolean performFinish() {
		saveEntity();

		conversation.commit();
		conversation.close();
		conversation = null;
		return true;
	}

	public T getEntity() {
		return entity;
	}

	public void setEntity(T entity){
		this.entity = entity;
	}

	/**
	 * Adds the entity to the current persistence context
	 */
	protected abstract void saveEntity();

	public void init(IWorkbench workbench, IStructuredSelection selection) {
	    formFactory = new CdmFormFactory(Display.getCurrent(), null);
	    conversation = CdmStore.createConversation();
	    this.selection = selection;
	 	entity = createNewEntity();

	}

	protected abstract T createNewEntity();

	@Override
    public ConversationHolder getConversationHolder() {
		return conversation;
	}

	@Override
    public void update(CdmDataChangeMap changeEvents) {}

	public IStructuredSelection getSelection() {
		return selection;
	}
}