/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.newWizard;

import eu.etaxonomy.cdm.api.service.INameService;
import eu.etaxonomy.cdm.model.name.INonViralName;
import eu.etaxonomy.cdm.model.name.NomenclaturalCode;
import eu.etaxonomy.cdm.model.name.TaxonName;
import eu.etaxonomy.cdm.model.name.TaxonNameFactory;
import eu.etaxonomy.taxeditor.preference.PreferencesUtil;
import eu.etaxonomy.taxeditor.store.CdmStore;
import eu.etaxonomy.taxeditor.ui.section.name.NonViralNameWizardPage;

/**
 * <p>NewNonViralNameWizard class.</p>
 *
 * @author n.hoffmann
 * @created Jun 1, 2010
 */
public class NewNonViralNameWizard extends AbstractNewEntityWizard<TaxonName> {

	/** {@inheritDoc} */
	@Override
	public void addPages() {
		addPage(new NonViralNameWizardPage(formFactory, getConversationHolder(), getEntity()));
	}

	/** {@inheritDoc} */
	@Override
	protected TaxonName createNewEntity() {
		NomenclaturalCode code = PreferencesUtil.getPreferredNomenclaturalCode();
		if(code.equals(NomenclaturalCode.ICNAFP )){
			return TaxonNameFactory.NewBotanicalInstance(null);
		}else if(code.equals(NomenclaturalCode.ICZN)){
			return TaxonNameFactory.NewZoologicalInstance(null);
		}
		return null;
	}

	/** {@inheritDoc} */
	@Override
	protected void saveEntity() {
	   CdmStore.getService(INameService.class).merge(getEntity(), true);
	}

	@Override
	protected String getEntityName() {
		return "Scientific Name";
	}
}
