/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.newWizard;

import java.util.List;

import org.eclipse.e4.ui.workbench.IWorkbench;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.swt.widgets.Display;

import eu.etaxonomy.cdm.api.service.IReferenceService;
import eu.etaxonomy.cdm.model.reference.Reference;
import eu.etaxonomy.cdm.model.reference.ReferenceFactory;
import eu.etaxonomy.cdm.model.reference.ReferenceType;
import eu.etaxonomy.taxeditor.store.CdmStore;
import eu.etaxonomy.taxeditor.ui.element.CdmFormFactory;
import eu.etaxonomy.taxeditor.ui.section.reference.ReferenceWizardPage;

/**
 * @author n.hoffmann
 * @created Jun 1, 2010
 */
public class NewReferenceWizard extends AbstractNewEntityWizard<Reference> {

	private ReferenceWizardPage referencePage;
	private ReferenceType refType = null;

	public NewReferenceWizard(ReferenceType type){
	    super();
	    refType = type;
	}

	@Override
	public void init(IWorkbench workbench, IStructuredSelection selection) {
        formFactory = new CdmFormFactory(Display.getCurrent(), null);
        conversation = CdmStore.createConversation();
        this.selection = selection;
        setEntity(createNewEntity());

    }

	@Override
	public void addPages() {
		// TODO disabling external service wizard page for now
//		addPage(new ExternalReferenceServiceWizardPage(formFactory, getConversationHolder(), entity));
		referencePage = new ReferenceWizardPage(formFactory, getConversationHolder(), getEntity(), false);
		addPage(referencePage);
	}

	@Override
	protected Reference createNewEntity() {
	    if (refType != null){
	        List<ReferenceType> inReferenceTypes = ReferenceType.inReferenceContraints(refType);
	        if (!inReferenceTypes.isEmpty()){
	            return ReferenceFactory.newReference(inReferenceTypes.iterator().next());
	        }
	    }
		return ReferenceFactory.newGeneric();
	}

	@Override
	protected void saveEntity() {
	    CdmStore.getService(IReferenceService.class).save(getEntity());
	}

	@Override
	public void setEntity(Reference entity) {
		super.setEntity(entity);
		if (referencePage != null){
		    referencePage.getDetailElement().setEntity(entity);
		}
	}

	@Override
	protected String getEntityName() {
		return "Reference";
	}
}
