/**
* Copyright (C) 2019 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.newWizard;

import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.wizard.IWizard;
import org.eclipse.jface.wizard.WizardDialog;
import org.eclipse.swt.SWTException;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Shell;

/**
 * @author k.luther
 * @since 23.01.2019
 */
public class NewTaxonNodeWizardDialog extends WizardDialog {

    private Button finishAndOpen;
    private IWizard newTaxonNodeWizard;

    public NewTaxonNodeWizardDialog(Shell parentShell, IWizard newWizard) {
        super(parentShell, newWizard);
        newTaxonNodeWizard = newWizard;
    }

    @Override
    protected void createButtonsForButtonBar(Composite parent) {
        finishAndOpen = createButton(parent, IDialogConstants.FINISH_ID,"Finish and Open", false);

        finishAndOpen.addSelectionListener(new SelectionListener() {
            @Override
            public void widgetSelected(SelectionEvent e) {
                ((NewTaxonNodeWizard)newTaxonNodeWizard).setOpenInEditor(true);
                try{
                    finishPressed();
                }catch(SWTException exception){
                    if ("Widget is disposed".equals(exception.getMessage())){
                        return;
                    }
                }
            }
            @Override
            public void widgetDefaultSelected(SelectionEvent e) {

            }
        });
        super.createButtonsForButtonBar(parent);
    }

    @Override
    public void updateButtons() {
        try{
            boolean canFinish = newTaxonNodeWizard.canFinish();

            finishAndOpen.setEnabled(canFinish);

            super.updateButtons();
         // finishAndOpen is default unless it is disabled and next is enabled
            if (canFinish) {
                getShell().setDefaultButton(finishAndOpen);
            }
        }catch (SWTException e){
            if ("Widget is disposed".equals(e.getMessage())){
                return;
            }
        }
    }

}
