/**
 * Copyright (C) 2013 EDIT
 * European Distributed Institute of Taxonomy
 * http://www.e-taxonomy.eu
 *
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */
package eu.etaxonomy.taxeditor.operation;

import java.util.UUID;

import org.eclipse.core.commands.operations.AbstractOperation;
import org.eclipse.core.commands.operations.IUndoContext;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;

import eu.etaxonomy.cdm.model.common.ICdmBase;
import eu.etaxonomy.taxeditor.session.ICdmEntitySessionEnabled;

/**
 * @author pplitzner
 * @date 03.12.2013
 */
public abstract class AbstractPostOperation<T extends ICdmBase> extends AbstractOperation {

    /**
     * A reference to the {@link ICdmBase} element the concrete operation is working on
     */
    protected T element;

    protected UUID elementUuid;

    protected IPostOperationEnabled postOperationEnabled;

    private ICdmEntitySessionEnabled<T> cdmEntitySessionEnabled;

    public AbstractPostOperation(String label, IUndoContext undoContext,
            T element, IPostOperationEnabled postOperationEnabled) {
        this(label, undoContext, element, postOperationEnabled, null);
    }

    public AbstractPostOperation(String label, IUndoContext undoContext,
            T element, IPostOperationEnabled postOperationEnabled,
            ICdmEntitySessionEnabled<T> cdmEntitySessionEnabled) {
        super(label);
        addContext(undoContext);
        this.element = element;
        if (element != null){
            this.elementUuid = element.getUuid();
        }
        this.postOperationEnabled = postOperationEnabled;
        this.cdmEntitySessionEnabled = cdmEntitySessionEnabled;
    }

    public AbstractPostOperation(String label, UUID elementUuid, IUndoContext undoContext,
            IPostOperationEnabled postOperationEnabled,
            ICdmEntitySessionEnabled<T> cdmEntitySessionEnabled) {
        super(label);
        addContext(undoContext);
        this.element = null;
        this.elementUuid = elementUuid;
        this.postOperationEnabled = postOperationEnabled;
        this.cdmEntitySessionEnabled = cdmEntitySessionEnabled;
    }

    public T getElement() {
        return element;
    }

    public UUID getElementUuid(){
        return elementUuid;
    }


    /**
     * This method will try to call the post operation on a possibly registered
     * IPostOperationEnabled implementor. Objects that were affected by the operation
     * may be passed to the registered IPostOperationEnabled implementor.
     *
     * @param objectAffectedByOperation the affected object. Should be <code>null</code> if not needed
     * @return a {@link org.eclipse.core.runtime.IStatus} object.
     */
    protected IStatus postExecute(Object objectAffectedByOperation) {

        if(postOperationEnabled != null){
            return postOperationEnabled.postOperation(objectAffectedByOperation) ? Status.OK_STATUS : Status.CANCEL_STATUS;
        }
        return Status.OK_STATUS;
    }

    /**
     * <p>Getter for the field <code>postOperationEnabled</code>.</p>
     *
     * @return a {@link eu.etaxonomy.taxeditor.operation.IPostOperationEnabled} object.
     */
    public IPostOperationEnabled getPostOperationEnabled() {
        return postOperationEnabled;
    }

    public ICdmEntitySessionEnabled<T> getCdmEntitySessionEnabled() {
        return cdmEntitySessionEnabled;
    }
}