/**
 * Copyright (C) 2007 EDIT
 * European Distributed Institute of Taxonomy
 * http://www.e-taxonomy.eu
 *
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */
package eu.etaxonomy.taxeditor.operation;

import java.util.UUID;

import org.eclipse.core.commands.operations.IUndoContext;
import org.eclipse.e4.ui.di.UISynchronize;

import eu.etaxonomy.cdm.api.service.ITaxonService;
import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.cdm.persistence.dto.TaxonNodeDto;
import eu.etaxonomy.taxeditor.session.ICdmEntitySessionEnabled;
import eu.etaxonomy.taxeditor.store.CdmStore;

/**
 * <p>Abstract AbstractPostOperation class.</p>
 *
 * @author p.ciardelli
 * @author n.hoffmann
 * @created 14.01.2009
 */
public abstract class AbstractPostTaxonOperation extends AbstractPostOperation<Taxon> {

    /**
     * A reference to the taxons TaxonNode
     */
    protected TaxonNodeDto taxonNode;

    protected UUID parentNodeUuid;

    private UISynchronize sync;

    public AbstractPostTaxonOperation(String label, IUndoContext undoContext,
            Taxon taxon, IPostOperationEnabled postOperationEnabled, UISynchronize sync) {
        this(label, undoContext, taxon, postOperationEnabled, null, sync);
    }

    public AbstractPostTaxonOperation(String label, IUndoContext undoContext,
            Taxon taxon, IPostOperationEnabled postOperationEnabled) {
        this(label, undoContext, taxon, postOperationEnabled, null, null);
    }
    public AbstractPostTaxonOperation(String label, IUndoContext undoContext,
            Taxon taxon, IPostOperationEnabled postOperationEnabled, ICdmEntitySessionEnabled<Taxon> cdmEntitySessionEnabled, UISynchronize sync) {
        super(label, undoContext, taxon, postOperationEnabled, cdmEntitySessionEnabled);
        this.sync = sync;
    }

    public AbstractPostTaxonOperation(String label, IUndoContext undoContext,
            Taxon taxon, IPostOperationEnabled postOperationEnabled, ICdmEntitySessionEnabled<Taxon> cdmEntitySessionEnabled) {
        super(label, undoContext, taxon, postOperationEnabled, cdmEntitySessionEnabled);
    }

    public AbstractPostTaxonOperation(String label, IUndoContext undoContext, TaxonNodeDto taxonNode,
            IPostOperationEnabled postOperationEnabled) {
        this(label, undoContext, taxonNode, postOperationEnabled, null);
    }

    public AbstractPostTaxonOperation(String label, IUndoContext undoContext, TaxonNodeDto taxonNode,
            IPostOperationEnabled postOperationEnabled, ICdmEntitySessionEnabled<Taxon> cdmEntitySessionEnabled) {
        this(label, undoContext, (Taxon) CdmStore.getService(ITaxonService.class).find(taxonNode.getTaxonUuid()), postOperationEnabled, cdmEntitySessionEnabled, null);
        this.taxonNode = taxonNode;
    }

    /**
     * <p>Constructor for AbstractPostOperation.</p>
     *
     * @param label a {@link java.lang.String} object.
     * @param undoContext a {@link org.eclipse.core.commands.operations.IUndoContext} object.
     * @param parentNodeUuid a {@link java.util.UUID} object.
     * @param postOperationEnabled a {@link eu.etaxonomy.taxeditor.operation.IPostOperationEnabled} object.
     */
    public AbstractPostTaxonOperation(String label, IUndoContext undoContext, UUID parentNodeUuid,
            IPostOperationEnabled postOperationEnabled) {
        super(label, undoContext, null, postOperationEnabled, null);
        this.parentNodeUuid = parentNodeUuid;
    }

    /**
     * <p>Constructor for AbstractPostOperation.</p>
     *
     * @param label a {@link java.lang.String} object.
     * @param undoContext a {@link org.eclipse.core.commands.operations.IUndoContext} object.
     * @param postOperationEnabled a {@link eu.etaxonomy.taxeditor.operation.IPostOperationEnabled} object.
     */
    public AbstractPostTaxonOperation(String label, IUndoContext undoContext,
            IPostOperationEnabled postOperationEnabled) {
        this(label, undoContext, postOperationEnabled, null);
    }

    public AbstractPostTaxonOperation(String label, IUndoContext undoContext,
            IPostOperationEnabled postOperationEnabled, ICdmEntitySessionEnabled<Taxon> cdmEntitySessionEnabled) {
        super(label, undoContext, null, postOperationEnabled, cdmEntitySessionEnabled);
    }

    public AbstractPostTaxonOperation(String label, IUndoContext undoContext, UUID taxonUuid) {
        this(label, undoContext, taxonUuid, null);
    }

    public UISynchronize getSync() {
        return sync;
    }

    public void setSync(UISynchronize sync) {
        this.sync = sync;
    }
}