/**
* Copyright (C) 2015 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.operation.e4;

import javax.inject.Named;

import org.eclipse.core.commands.operations.AbstractOperation;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.e4.core.di.annotations.Execute;
import org.eclipse.e4.ui.di.UISynchronize;
import org.eclipse.e4.ui.model.application.MApplication;
import org.eclipse.e4.ui.model.application.ui.basic.MPart;
import org.eclipse.e4.ui.model.application.ui.menu.MHandledMenuItem;
import org.eclipse.e4.ui.services.IServiceConstants;
import org.eclipse.e4.ui.workbench.modeling.EModelService;
import org.eclipse.e4.ui.workbench.modeling.EPartService;
import org.eclipse.jface.dialogs.ErrorDialog;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.swt.widgets.Shell;

import eu.etaxonomy.taxeditor.model.AbstractUtility;
import eu.etaxonomy.taxeditor.model.MessagingUtils;

/**
 * @author cmathew
 * @date 16 Jun 2015
 *
 */
public abstract class CdmHandlerE4 {

    protected String label;
    protected EPartService partService;
	protected MApplication application;
	protected EModelService modelService;

    public CdmHandlerE4(String label) {
        this.label = label;
    }

    @Execute
    public void execute(@Named(IServiceConstants.ACTIVE_SELECTION)IStructuredSelection selection,
            @Named(IServiceConstants.ACTIVE_SHELL)Shell shell,
            @Named(IServiceConstants.ACTIVE_PART)MPart activePart,
            MHandledMenuItem menuItem, EPartService partService, UISynchronize sync, MApplication application,EModelService modelService) {
        this.partService = partService;
        this.application = application;
        this.modelService = modelService;
        IStatus allowStatus = allowOperations(selection, shell, activePart, menuItem);
        if(allowStatus.isOK()) {
            AbstractOperation op = prepareOperation(selection, shell, activePart, menuItem);
            if(op != null) {
                AbstractUtility.executeOperation(op, this, sync);
            }
        } else if(allowStatus.getSeverity() == IStatus.ERROR ||
                allowStatus.getSeverity() == IStatus.WARNING ||
                allowStatus.getSeverity() == IStatus.INFO) {
            MessagingUtils.warningDialog("Can not perform " + label, getTrigger(), allowStatus);
        }
        return;
    }

    public void postOperation(IStatus status) {

        switch(status.getSeverity()) {
        case IStatus.WARNING:
            ErrorDialog.openError(null, "Operation successful but with warnings", null, status);
            break;
        case IStatus.ERROR:
            ErrorDialog.openError(null, "Error executing operation", null, status);
            break;
        default:
        }

        onComplete();
    }

    public abstract IStatus allowOperations(IStructuredSelection selection,
            Shell shell,
            MPart activePart,
            MHandledMenuItem menuItem);

    public abstract AbstractOperation prepareOperation(IStructuredSelection selection,
            Shell shell,
            MPart activePart,
            MHandledMenuItem menuItem);

    public abstract void onComplete();

    protected abstract Object getTrigger();

}
