/**
* Copyright (C) 2018 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.preference;

import org.eclipse.jface.layout.GridLayoutFactory;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CLabel;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;

import eu.etaxonomy.cdm.api.application.ICdmRepository;
import eu.etaxonomy.cdm.common.URI;
import eu.etaxonomy.cdm.io.specimen.abcd206.in.Abcd206ImportConfigurator;
import eu.etaxonomy.cdm.model.metadata.CdmPreference;
import eu.etaxonomy.cdm.model.metadata.CdmPreference.PrefKey;
import eu.etaxonomy.cdm.model.metadata.PreferencePredicate;
import eu.etaxonomy.cdm.model.metadata.PreferenceSubject;
import eu.etaxonomy.cdm.model.name.NomenclaturalCode;
import eu.etaxonomy.taxeditor.l10n.Messages;
import eu.etaxonomy.taxeditor.preference.menu.CdmPreferencePage;
import eu.etaxonomy.taxeditor.store.CdmStore;

/**
 * @author k.luther
 * @since 23.03.2018
 */
public class AbcdImportPreference extends CdmPreferencePage implements IE4PreferencePage, SelectionListener {

    private static final String CATALOGUE_NUMBER = "Catalogue number";

    private static final String BARCODE = "Barcode";

    private static final String ACCESSION_NUMBER = "Accession number";

    protected Abcd206ImportConfigurator configurator;

    protected Combo nomenclaturalCodeSelectionCombo;

    protected boolean allowOverride = true;
    protected boolean override = true;
    protected CdmPreference preference = null;
    Combo textDNAProviderString;
    private Composite composite;


    Button checkBoxMediaSpecimen;
    Button checkBoxIgnoreExisting;
    Button checkBoxIgnoreAuthorship;
    Combo checkBoxMapUnitId;
    Button checkBoxRemoveCountry;
    Button checkBoxMoveToDefaultClassification;
    Button checkBoxImportSiblings;
    Button checkBoxAddIndividualsAssociations;
    Button checkBoxReuseDescriptiveGroups;
    Button checkBoxReuseExistingTaxa;


    Combo useLocalOrAdmin;

    @Override
    protected Control createContents(Composite parent) {
        getValues();

        if (!isAdminPreference && !allowOverride){
            Label label = new Label(parent, SWT.NONE);
            label.setText("The CDM settings don't allow to set the abcd import preference locally. If you need to make local settings, please ask an administrator.");
            this.noDefaultAndApplyButton();
            return parent;
        }
        composite = new Composite(parent, SWT.NULL);
        GridLayout gridLayout = new GridLayout();
        composite.setLayout(gridLayout);

        final CLabel description = new CLabel(composite, SWT.NULL);
        description.setText(Messages.AbcdImportPreference_description);
        checkBoxMediaSpecimen = new Button(composite, SWT.CHECK);
        checkBoxMediaSpecimen.setText(Messages.AbcdImportPreference_media_as_mediaSpecimen);
        checkBoxMediaSpecimen
                .setToolTipText(Messages.AbcdImportPreference_media_as_subUnit);
        checkBoxMediaSpecimen.addSelectionListener(new SelectionAdapter() {
            @Override
            public void widgetSelected(SelectionEvent e) {
                configurator.setAddMediaAsMediaSpecimen(!configurator.isAddMediaAsMediaSpecimen());
                setApply(true);
            }
        });

        checkBoxIgnoreExisting = new Button(composite, SWT.CHECK);

        checkBoxIgnoreExisting.setText(Messages.AbcdImportPreference_not_import_existing_specimen);
        checkBoxIgnoreExisting
        .setToolTipText(Messages.AbcdImportPreference_not_import_existing_specimen_tooltip);
        checkBoxIgnoreExisting.addSelectionListener(new SelectionAdapter() {
            @Override
            public void widgetSelected(SelectionEvent e) {
                configurator.setIgnoreImportOfExistingSpecimen(!configurator.isIgnoreImportOfExistingSpecimen());
                setApply(true);
            }
        });

        checkBoxIgnoreAuthorship = new Button(composite, SWT.CHECK);

        checkBoxIgnoreAuthorship.setText(Messages.AbcdImportPreference_ignore_author);
        checkBoxIgnoreAuthorship
        .setToolTipText(Messages.AbcdImportPreference_ignore_author_tooltip);
        checkBoxIgnoreAuthorship.addSelectionListener(new SelectionAdapter() {
            @Override
            public void widgetSelected(SelectionEvent e) {
                configurator.setIgnoreAuthorship(!configurator.isIgnoreAuthorship());
                setApply(true);
            }
        });


        GridData gridData = new GridData();
        gridData = new GridData(GridData.BEGINNING, GridData.CENTER, true, false);
        gridData.horizontalIndent = 9;
        gridData.minimumWidth=100;
//      classificationSelection.setLayoutData(gridData);
        Label unitIdLabel = new Label(composite, SWT.NONE);
        unitIdLabel.setText(Messages.AbcdImportPreference_map_unit_nr_catalog_number);
        checkBoxMapUnitId = new Combo(composite, SWT.BORDER| SWT.READ_ONLY);
        checkBoxMapUnitId.setLayoutData(gridData);
        checkBoxMapUnitId.add(ACCESSION_NUMBER);
        checkBoxMapUnitId.add(BARCODE);
        checkBoxMapUnitId.add(CATALOGUE_NUMBER);
        checkBoxMapUnitId
        .setToolTipText(Messages.AbcdImportPreference_map_unit_number_catalog_number_tooltip);
        checkBoxMapUnitId.addSelectionListener(this);

        checkBoxRemoveCountry = new Button(composite, SWT.CHECK);

        checkBoxRemoveCountry.setText(Messages.AbcdImportPreference_remove_country_from_locality);
        checkBoxRemoveCountry
        .setToolTipText(Messages.AbcdImportPreference_remove_country_from_locality_tooltip);
        checkBoxRemoveCountry.addSelectionListener(new SelectionAdapter() {
            @Override
            public void widgetSelected(SelectionEvent e) {
                configurator.setRemoveCountryFromLocalityText(!configurator.isRemoveCountryFromLocalityText());
                setApply(true);
            }
        });

        checkBoxMoveToDefaultClassification = new Button(composite, SWT.CHECK);
        checkBoxMoveToDefaultClassification.setSelection(configurator.isMoveNewTaxaToDefaultClassification());
        checkBoxMoveToDefaultClassification.setText(Messages.AbcdImportPreference_create_new_classification_new_taxa);
        checkBoxMoveToDefaultClassification
        .setToolTipText(Messages.AbcdImportPreference_create_new_classification_new_taxa_tooltip);
        checkBoxMoveToDefaultClassification.addSelectionListener(new SelectionAdapter() {
            @Override
            public void widgetSelected(SelectionEvent e) {
                configurator.setMoveNewTaxaToDefaultClassification(!configurator.isMoveNewTaxaToDefaultClassification());
                setApply(true);
            }
        });

        checkBoxImportSiblings = new Button(composite, SWT.CHECK);

        checkBoxImportSiblings.setText(Messages.AbcdImportPreference_import_all_children_for_cultures_or_tissues);
        checkBoxImportSiblings
            .setToolTipText(Messages.AbcdImportPreference_import_all_children_for_cultures_or_tissues_tooltip);
        checkBoxImportSiblings.addSelectionListener(new SelectionAdapter() {
            @Override
            public void widgetSelected(SelectionEvent e) {
                configurator.setGetSiblings(!configurator.isGetSiblings());
                setApply(true);
            }
        });

        checkBoxAddIndividualsAssociations = new Button(composite, SWT.CHECK);

        checkBoxAddIndividualsAssociations.setText(Messages.AbcdImportPreference_create_Individual_Association);
        checkBoxAddIndividualsAssociations
            .setToolTipText(Messages.AbcdImportPreference_create_Individual_Association_tooltip);
        checkBoxAddIndividualsAssociations.addSelectionListener(new SelectionAdapter() {
            @Override
            public void widgetSelected(SelectionEvent e) {
                configurator.setAddIndividualsAssociationsSuchAsSpecimenAndObservations(!configurator.isAddIndividualsAssociationsSuchAsSpecimenAndObservations());
                setApply(true);
            }
        });

        checkBoxReuseDescriptiveGroups = new Button(composite, SWT.CHECK);

        checkBoxReuseDescriptiveGroups.setText(Messages.AbcdImportPreference_reuse_descriptive_group);
        checkBoxReuseDescriptiveGroups
            .setToolTipText(Messages.AbcdImportPreference_reuse_descriptive_group_tooltip);
        checkBoxReuseDescriptiveGroups.addSelectionListener(new SelectionAdapter() {
            @Override
            public void widgetSelected(SelectionEvent e) {
                configurator.setReuseExistingDescriptiveGroups(!configurator.isReuseExistingDescriptiveGroups());
                setApply(true);
            }
        });

        checkBoxReuseExistingTaxa = new Button(composite, SWT.CHECK);

        checkBoxReuseExistingTaxa.setText(Messages.AbcdImportPreference_reuse_existing_taxa);
        checkBoxReuseExistingTaxa
            .setToolTipText(Messages.AbcdImportPreference_reuse_existing_taxa_tooltip);
        checkBoxReuseExistingTaxa.addSelectionListener(new SelectionAdapter() {
            @Override
            public void widgetSelected(SelectionEvent e) {
                configurator.setReuseExistingTaxaWhenPossible(!configurator.isReuseExistingTaxaWhenPossible());
                setApply(true);
            }
        });
        GridData sepGrid = createTextGridData();
        Label separator = new Label(composite, SWT.HORIZONTAL|SWT.SEPARATOR);
        separator.setLayoutData(sepGrid);
        separator.setVisible(false);
        Label labelRef = new Label(composite, SWT.NONE);
        labelRef.setText("Biocase provider for associated DNA");
        textDNAProviderString = new Combo(composite, SWT.BORDER);

        gridData = new GridData(GridData.BEGINNING, GridData.CENTER, true, false);
        gridData.horizontalIndent = 9;
        gridData.minimumWidth=200;

        textDNAProviderString.setLayoutData(gridData);
        GridLayoutFactory.fillDefaults();
        String biocaseProvider = null;
        CdmPreference pref = null;

        pref = PreferencesUtil.getPreferenceFromDB(PreferencePredicate.BioCaseProvider);
        biocaseProvider =  pref!= null? pref.getValue():null;

        if (!isAdminPreference && ((pref != null && pref.isAllowOverride()) || pref == null) ){
            biocaseProvider = PreferencesUtil.getStringValue(PreferencePredicate.BioCaseProvider.getKey(), true);
        }
        if (biocaseProvider != null){
            String items[] = biocaseProvider.split(";");
            textDNAProviderString.setItems(items);
        }

        textDNAProviderString.setEnabled(true);
       // textDNAProviderString.setLayoutData(new GridData(SWT.FILL, SWT.TOP, true, true, 1, 1));
        textDNAProviderString.addSelectionListener(this);

        Label nomenclaturalCodeLabel = new Label(composite, SWT.NONE);
        nomenclaturalCodeLabel.setText("Nomenclatural Code");
        nomenclaturalCodeSelectionCombo = new Combo(composite, SWT.BORDER| SWT.READ_ONLY);
        gridData = new GridData(GridData.BEGINNING, GridData.CENTER, true, false);
        gridData.horizontalIndent = 9;
        nomenclaturalCodeSelectionCombo.setLayoutData(gridData);
        GridLayoutFactory.fillDefaults();
        for(NomenclaturalCode code: NomenclaturalCode.values()){
            nomenclaturalCodeSelectionCombo.add(code.getKey());
        }

        nomenclaturalCodeSelectionCombo.addSelectionListener(this);
        if (preference != null){
            allowOverride = preference.isAllowOverride();
        }

        Label sep = new Label(composite, SWT.HORIZONTAL|SWT.SEPARATOR);
        sep.setLayoutData(new GridData(SWT.DEFAULT, SWT.DEFAULT, true, false));
        GridLayoutFactory.fillDefaults();
        useLocalOrAdmin = new Combo(parent, SWT.BORDER | SWT.READ_ONLY);
        if (!isAdminPreference){
            useLocalOrAdmin.add(LocalOrDefaultEnum.Default.getLabel(), 0);
            useLocalOrAdmin.add(LocalOrDefaultEnum.Local.getLabel(), 1);

        }else{
            useLocalOrAdmin.add(LocalOrDefaultEnum.AllowOverride.getLabel(), 0);
            useLocalOrAdmin.add(LocalOrDefaultEnum.Database.getLabel(), 1);
        }

        if (!isAdminPreference){
            if (override && allowOverride){
                useLocalOrAdmin.select(1);
            }else{
                useLocalOrAdmin.select(0);
                composite.setEnabled(false);
            }
        }else{
            if (allowOverride){
                useLocalOrAdmin.select(0);
            }else{
                useLocalOrAdmin.select(1);
            }
        }
        useLocalOrAdmin.addSelectionListener(this);
        if (!allowOverride && !isAdminPreference){
            useLocalOrAdmin.setEnabled(false);
        }
        GridData gridDataLocal = new GridData();
        gridDataLocal = new GridData(GridData.BEGINNING, GridData.CENTER, true, false);
       // gridDataLocal.horizontalIndent = 9;
        gridDataLocal.minimumWidth=100;
        gridDataLocal.widthHint= nomenclaturalCodeSelectionCombo.getBorderWidth();
        useLocalOrAdmin.setLayoutData(gridDataLocal);
        GridLayoutFactory.fillDefaults();
        setSelections();

        return composite;
    }

    @Override
    public void widgetSelected(SelectionEvent e) {
        if (e.getSource().equals(nomenclaturalCodeSelectionCombo)){
            this.configurator.setNomenclaturalCode(NomenclaturalCode.getByKey(nomenclaturalCodeSelectionCombo.getText()));
        }
        if (e.getSource().equals(checkBoxMapUnitId)){
            String text = checkBoxMapUnitId.getText();
            if (text.equals(ACCESSION_NUMBER)){
                this.configurator.setMapUnitIdToAccessionNumber(true);
                this.configurator.setMapUnitIdToBarcode(false);
                this.configurator.setMapUnitIdToCatalogNumber(false);
            }else if (text.equals(BARCODE)){
                this.configurator.setMapUnitIdToAccessionNumber(false);
                this.configurator.setMapUnitIdToBarcode(true);
                this.configurator.setMapUnitIdToCatalogNumber(false);
            }else{
                this.configurator.setMapUnitIdToAccessionNumber(false);
                this.configurator.setMapUnitIdToBarcode(false);
                this.configurator.setMapUnitIdToCatalogNumber(true);
            }
        }
        if (e.getSource().equals(useLocalOrAdmin)){
            if (useLocalOrAdmin.getText().equals(LocalOrDefaultEnum.Default.getLabel()) || useLocalOrAdmin.getText().equals(LocalOrDefaultEnum.Database.getLabel())){
               composite.setEnabled(false);
               PreferencesUtil.extractAbcdConfiguratorFromPreferenceString(configurator, preference.getValue());
               setSelections();

               if (isAdminPreference){
                   allowOverride = false;
               }else{
                   override = false;
               }
            }else{
               composite.setEnabled(true);
               if (isAdminPreference){
                   allowOverride = true;
               }else{
                   override = true;
               }
            }
        }

        if (e.getSource().equals(textDNAProviderString)){
            String textDnaProvider = textDNAProviderString.getText();
            try{
                this.configurator.setDnaSoure(URI.create(textDnaProvider));
            }catch(IllegalArgumentException iae){
                //do not set as dna source
            }
        }
        this.setApply(true);

       // PreferencesUtil.recursiveSetEnabled(composite, override);
        setApply(true);

    }

    @Override
    public boolean performOk() {
        if (!isApply()){
            return true;
        }
        if (configurator != null){
            try{
                configurator.setDnaSoure(URI.create(textDNAProviderString.getText()));
            }catch(IllegalArgumentException e){
                //dna provider is not an uri
            }
            String configString = configurator.toString();


            if (override){
                PreferencesUtil.setStringValue(PreferencePredicate.AbcdImportConfig.getKey(), configString);
                PreferencesUtil.setBooleanValue( PreferencesUtil.prefOverrideKey(PreferencePredicate.AbcdImportConfig.getKey()), override);

            }else{
                PreferencesUtil.setBooleanValue( PreferencesUtil.prefOverrideKey(PreferencePredicate.AbcdImportConfig.getKey()), override);
            }
            PreferencesUtil.setStringValue(PreferencePredicate.AbcdImportConfig.getKey(), "");
        }
        return true;
    }

    @Override
    public void widgetDefaultSelected(SelectionEvent e) {
        // TODO Auto-generated method stub

    }

    @Override
    protected void performDefaults() {
        configurator = Abcd206ImportConfigurator.NewInstance(null,null);
        configurator.setNomenclaturalCode(PreferencesUtil.getPreferredNomenclaturalCode());
        override = false;
        setSelections();
        setApply(true);
    }

    @Override
    public void getValues() {
        isAdminPreference = false;
        configurator = Abcd206ImportConfigurator.NewInstance(null,null);
        ICdmRepository controller;
        controller = CdmStore.getCurrentApplicationConfiguration();
        PrefKey key = CdmPreference.NewKey(PreferenceSubject.NewTaxEditorInstance(), PreferencePredicate.AbcdImportConfig);
        preference = controller.getPreferenceService().find(key);

        if (preference != null ){
            allowOverride = preference.isAllowOverride();
        }else{
            preference = CdmPreference.NewTaxEditorInstance(PreferencePredicate.AbcdImportConfig, configurator.toString());
        }

        override = PreferencesUtil.getBooleanValue(
                PreferencesUtil.prefOverrideKey(PreferencePredicate.AbcdImportConfig.getKey()), true) != null? PreferencesUtil.getBooleanValue(
                        PreferencesUtil.prefOverrideKey(PreferencePredicate.AbcdImportConfig.getKey()), true):false;
       if (allowOverride && override){
           String configString = PreferencesUtil.getStringValue(PreferencePredicate.AbcdImportConfig.getKey(), true);
           configurator = Abcd206ImportConfigurator.NewInstance(null, null);
           PreferencesUtil.extractAbcdConfiguratorFromPreferenceString(configurator, configString);
       }else{
           PreferencesUtil.extractAbcdConfiguratorFromPreferenceString(configurator, preference.getValue());
       }
    }

    protected void setSelections(){
        checkBoxMediaSpecimen.setSelection(configurator.isAddMediaAsMediaSpecimen());
        checkBoxIgnoreExisting.setSelection(configurator.isIgnoreImportOfExistingSpecimen());
        checkBoxIgnoreAuthorship.setSelection(configurator.isIgnoreAuthorship());
        if (configurator.isMapUnitIdToAccessionNumber()){
            checkBoxMapUnitId.select(0);
        }else if (configurator.isMapUnitIdToBarcode()){
            checkBoxMapUnitId.select(1);
        }else{
            checkBoxMapUnitId.select(2);
        }


        checkBoxRemoveCountry.setSelection(configurator.isRemoveCountryFromLocalityText());
        checkBoxImportSiblings.setSelection(configurator.isGetSiblings());
        checkBoxAddIndividualsAssociations.setSelection(configurator.isAddIndividualsAssociationsSuchAsSpecimenAndObservations());
        checkBoxReuseDescriptiveGroups.setSelection(configurator.isReuseExistingDescriptiveGroups());
        checkBoxReuseExistingTaxa.setSelection(configurator.isReuseExistingTaxaWhenPossible());

        textDNAProviderString.setText(configurator.getDnaSoure()!= null? configurator.getDnaSoure().toString():"");
        int index = 0;
        if (configurator.getNomenclaturalCode() != null){
            for (String label : nomenclaturalCodeSelectionCombo.getItems()){
                if (label.equals(configurator.getNomenclaturalCode().getKey())){
                    nomenclaturalCodeSelectionCombo.select(index);
                }
                index++;
            }
        }else{
            for (String label : nomenclaturalCodeSelectionCombo.getItems()){
                NomenclaturalCode code = PreferencesUtil.getPreferredNomenclaturalCode();
                if (label.equals(code.getKey())){
                    nomenclaturalCodeSelectionCombo.select(index);
                    configurator.setNomenclaturalCode(code);
                }
                index++;
            }
        }
    }
}