/**
* Copyright (C) 2020 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.preference;

import org.apache.commons.lang.StringUtils;
import org.eclipse.jface.layout.GridLayoutFactory;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;

import eu.etaxonomy.cdm.api.application.ICdmRepository;
import eu.etaxonomy.cdm.ext.geo.CondensedDistributionRecipe;
import eu.etaxonomy.cdm.model.metadata.CdmPreference;
import eu.etaxonomy.cdm.model.metadata.CdmPreference.PrefKey;
import eu.etaxonomy.cdm.model.metadata.PreferencePredicate;
import eu.etaxonomy.cdm.model.metadata.PreferenceSubject;
import eu.etaxonomy.taxeditor.l10n.Messages;
import eu.etaxonomy.taxeditor.preference.menu.CdmPreferencePage;
import eu.etaxonomy.taxeditor.store.CdmStore;

/**
 * @author k.luther
 * @since Nov 12, 2020
 */
public class CdmLightExportPreference extends CdmPreferencePage implements SelectionListener {

    protected Combo createCondensedDistributionString;
    protected Combo comboDistributionStringRecipe;

    protected Button allowOverrideCreateCondensedDistributionString;
    protected Button allowOverrideDistributionStringRecipe;

    protected CdmPreference preferenceIsExportCondensedDistribution = null;
    protected boolean isAllowOverrideIsExportCondensedDistribution = false;
    protected boolean overrideExportCondensedDistribution = false;
    protected CdmPreference preferenceCondensedDistribtionRecipe = null;
    protected boolean isAllowOverrideCondensedDistributionRecipe = false;
    protected boolean overrideCondensedDistributionRecipe = false;
    protected Boolean isExportCondensedDistribution = null;
    protected CondensedDistributionRecipe recipe = null;

    @Override
    protected Control createContents(Composite parent) {
        getValues();
        Composite composite = createComposite(parent);
//        GridLayout gridLayout = new GridLayout();
//        composite.setLayout(gridLayout);

        Label description = new Label(composite, SWT.NULL);
        description.setText(Messages.CdmLightPreference_description);

        createCondensedDistributionString = createBooleanCombo(composite, Messages.GeneralPreference_yes, Messages.GeneralPreference_no, PreferencePredicate.CdmLightExportCondensedDistributionString, Messages.CdmLightPreference_distributionString, isAdminPreference);

        createCondensedDistributionString
                .setToolTipText(Messages.CdmLightPreference_distributionString_tooltip);
        int index = 0;
        for (String itemLabel : createCondensedDistributionString.getItems()) {
            if (isExportCondensedDistribution == null && itemLabel.startsWith(Messages.Preference_Use_Default)) {
                createCondensedDistributionString.select(index);
                break;
            }
            if (isExportCondensedDistribution != null && itemLabel.equals(Messages.GeneralPreference_yes) && isExportCondensedDistribution){
                createCondensedDistributionString.select(index);
                break;
            }
            if (isExportCondensedDistribution != null && itemLabel.equals(Messages.GeneralPreference_no) && !isExportCondensedDistribution){
                createCondensedDistributionString.select(index);
                break;
            }
            index++;
        }
        createCondensedDistributionString.addSelectionListener(this);

        if (isAdminPreference){
           allowOverrideCreateCondensedDistributionString = createAllowOverrideButton(composite);
           allowOverrideCreateCondensedDistributionString.setSelection(this.preferenceIsExportCondensedDistribution != null? this.preferenceIsExportCondensedDistribution.isAllowOverride(): isAdminPreference);
           allowOverrideCreateCondensedDistributionString.addSelectionListener(new SelectionAdapter() {
               @Override
               public void widgetSelected(SelectionEvent e) {
                   setApply(true);
//                   if (preferenceIsExportCondensedDistribution == null){
//                       preferenceIsExportCondensedDistribution = CdmPreference.NewTaxEditorInstance(PreferencePredicate.CdmLightExportCondensedDistributionString, PreferencePredicate.CdmLightExportCondensedDistributionString.getDefaultValue() != null ?  PreferencePredicate.CdmLightExportCondensedDistributionString.getDefaultValue().toString(): null);
//                   }
//                   preferenceIsExportCondensedDistribution.setAllowOverride(!preferenceIsExportCondensedDistribution.isAllowOverride());
                   isAllowOverrideIsExportCondensedDistribution = !isAllowOverrideIsExportCondensedDistribution;
               }
           });
        }

        comboDistributionStringRecipe = createCombo(composite, CondensedDistributionRecipe.values(), PreferencePredicate.CondensedDistributionRecipe, "Condensed Distribution String recipe", isAdminPreference);

//        GridData gridData = new GridData(GridData.BEGINNING, GridData.CENTER, true, false);
//        gridData.horizontalIndent = 9;
//        comboDistributionStringRecipe.setLayoutData(gridData);
        GridLayoutFactory.fillDefaults();

        comboDistributionStringRecipe.addSelectionListener(this);
        comboDistributionStringRecipe.setEnabled((isExportCondensedDistribution != null && isExportCondensedDistribution )|| (isExportCondensedDistribution == null) && Boolean.valueOf(preferenceIsExportCondensedDistribution.getValue()));
        index = 0;

        for (String itemLabel : comboDistributionStringRecipe.getItems()) {
            if (itemLabel.startsWith(Messages.Preference_Use_Default) && recipe == null){
                comboDistributionStringRecipe.select(index);
                break;
            }
            if (itemLabel.equals(recipe.getLabel())) {
                comboDistributionStringRecipe.select(index);
                break;
            }

            index++;
        }
        if (isAdminPreference){
            allowOverrideDistributionStringRecipe = createAllowOverrideButton(composite);
            allowOverrideDistributionStringRecipe.setSelection(this.preferenceCondensedDistribtionRecipe != null? this.preferenceCondensedDistribtionRecipe.isAllowOverride(): true);
            allowOverrideDistributionStringRecipe.addSelectionListener(new SelectionAdapter() {
                @Override
                public void widgetSelected(SelectionEvent e) {
                    setApply(true);
//                    if (preferenceCondensedDistribtionRecipe == null){
//                        preferenceCondensedDistribtionRecipe = CdmPreference.NewTaxEditorInstance(PreferencePredicate.CondensedDistributionRecipe, PreferencePredicate.CondensedDistributionRecipe.getDefaultValue() != null ?  PreferencePredicate.CondensedDistributionRecipe.getDefaultValue().toString(): null);
//                    }
//                    preferenceCondensedDistribtionRecipe.setAllowOverride(!preferenceCondensedDistribtionRecipe.isAllowOverride());
                    isAllowOverrideCondensedDistributionRecipe = !isAllowOverrideCondensedDistributionRecipe;
                }
            });
         }

        return composite;
    }





    @Override
    public void widgetSelected(SelectionEvent e) {
        setApply(true);
        if (e.getSource().equals(this.comboDistributionStringRecipe)){
            int selIndex = comboDistributionStringRecipe.getSelectionIndex();
            if (selIndex == 0){
                overrideCondensedDistributionRecipe = false;
            }else{
                overrideCondensedDistributionRecipe = true;
            }
            recipe = (CondensedDistributionRecipe) comboDistributionStringRecipe.getData(comboDistributionStringRecipe.getText());

        }
        if (e.getSource().equals(this.createCondensedDistributionString)){
            String text = createCondensedDistributionString.getText();
            if(text.startsWith(Messages.Preference_Use_Default)){
                isExportCondensedDistribution = null;
                overrideExportCondensedDistribution = false;
            }

            if (text.equals(Messages.GeneralPreference_yes)){
                isExportCondensedDistribution = true;
                overrideExportCondensedDistribution = true;
            }else if (text.equals(Messages.GeneralPreference_no)){
                isExportCondensedDistribution = false;
                overrideExportCondensedDistribution = true;
            }
            comboDistributionStringRecipe.setEnabled((isExportCondensedDistribution != null && isExportCondensedDistribution )|| (isExportCondensedDistribution == null) && Boolean.valueOf(preferenceIsExportCondensedDistribution.getValue()));
        }



    }


    @Override
    public boolean performOk() {
        if (!isApply()){
            return true;
        }
        if(overrideExportCondensedDistribution){
            PreferencesUtil.setBooleanValue(PreferencePredicate.CdmLightExportCondensedDistributionString.getKey(), isExportCondensedDistribution);
            PreferencesUtil.setBooleanValue(PreferencesUtil.prefOverrideKey(PreferencePredicate.CdmLightExportCondensedDistributionString.getKey()), overrideExportCondensedDistribution);
        }else{
           PreferencesUtil.setBooleanValue( PreferencesUtil.prefOverrideKey(PreferencePredicate.CdmLightExportCondensedDistributionString.getKey()), overrideExportCondensedDistribution);
        }
        if(overrideCondensedDistributionRecipe){
            PreferencesUtil.setStringValue(PreferencePredicate.CondensedDistributionRecipe.getKey(), recipe.getKey());
            PreferencesUtil.setBooleanValue(PreferencesUtil.prefOverrideKey(PreferencePredicate.CondensedDistributionRecipe.getKey()), overrideCondensedDistributionRecipe);
        }else{
           PreferencesUtil.setBooleanValue( PreferencesUtil.prefOverrideKey(PreferencePredicate.CondensedDistributionRecipe.getKey()), overrideCondensedDistributionRecipe);
        }



        return true;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public void widgetDefaultSelected(SelectionEvent e) {
        // TODO Auto-generated method stub

    }

    @Override
    protected void performDefaults() {
        isExportCondensedDistribution = Boolean.valueOf(PreferencePredicate.CdmLightExportCondensedDistributionString.getDefaultValue().toString());
        createCondensedDistributionString.select(0);
        recipe = PreferencePredicate.CondensedDistributionRecipe.getDefaultValue() != null? CondensedDistributionRecipe.valueOf(PreferencePredicate.CondensedDistributionRecipe.getDefaultValue().toString()): null;
        comboDistributionStringRecipe.select(0);
        setApply(true);
    }


    @Override
    public void getValues() {
        isAdminPreference = false;

        ICdmRepository controller = CdmStore.getCurrentApplicationConfiguration();
       // general export of condensed string
        PrefKey key = CdmPreference.NewKey(PreferenceSubject.NewTaxEditorInstance(), PreferencePredicate.CdmLightExportCondensedDistributionString);
        preferenceIsExportCondensedDistribution = controller.getPreferenceService().find(key);

        if (preferenceIsExportCondensedDistribution != null ){
            isAllowOverrideIsExportCondensedDistribution = preferenceIsExportCondensedDistribution.isAllowOverride();
        }else{
            preferenceIsExportCondensedDistribution = CdmPreference.NewTaxEditorInstance(PreferencePredicate.CdmLightExportCondensedDistributionString, PreferencePredicate.CdmLightExportCondensedDistributionString.getDefaultValue().toString());
            isAllowOverrideIsExportCondensedDistribution = true;
        }

        overrideExportCondensedDistribution = PreferencesUtil.getBooleanValue(
                PreferencesUtil.prefOverrideKey(PreferencePredicate.CdmLightExportCondensedDistributionString.getKey()), true) != null? PreferencesUtil.getBooleanValue(
                        PreferencesUtil.prefOverrideKey(PreferencePredicate.CdmLightExportCondensedDistributionString.getKey()), true):false;

        if (isAllowOverrideIsExportCondensedDistribution){
            if( overrideExportCondensedDistribution){
                isExportCondensedDistribution = PreferencesUtil.getBooleanValue(PreferencePredicate.CdmLightExportCondensedDistributionString.getKey(), true);
            }else{
                isExportCondensedDistribution = null;
            }

        }else{
            isExportCondensedDistribution = preferenceIsExportCondensedDistribution.getValue()!= null?Boolean.valueOf(preferenceIsExportCondensedDistribution.getValue()): null;
        }
        // recipe for creation of condensed string

        key = CdmPreference.NewKey(PreferenceSubject.NewDatabaseInstance(), PreferencePredicate.CondensedDistributionRecipe);
        preferenceCondensedDistribtionRecipe = controller.getPreferenceService().find(key);

        if (preferenceCondensedDistribtionRecipe != null ){
            isAllowOverrideCondensedDistributionRecipe = preferenceCondensedDistribtionRecipe.isAllowOverride();
        }else{
            preferenceCondensedDistribtionRecipe = CdmPreference.NewDatabaseInstance(PreferencePredicate.CondensedDistributionRecipe,
                    PreferencePredicate.CondensedDistributionRecipe.getDefaultValue() != null? PreferencePredicate.CondensedDistributionRecipe.getDefaultValue().toString(): null);
            isAllowOverrideCondensedDistributionRecipe = true;
        }

        overrideCondensedDistributionRecipe = PreferencesUtil.getBooleanValue(
                PreferencesUtil.prefOverrideKey(PreferencePredicate.CondensedDistributionRecipe.getKey()), true) != null? PreferencesUtil.getBooleanValue(
                        PreferencesUtil.prefOverrideKey(PreferencePredicate.CondensedDistributionRecipe.getKey()), true):false;

        if (isAllowOverrideCondensedDistributionRecipe && overrideCondensedDistributionRecipe){
            String configString = PreferencesUtil.getStringValue(PreferencePredicate.CondensedDistributionRecipe.getKey(), true);
            if (StringUtils.isNotBlank(configString)){
                recipe = CondensedDistributionRecipe.valueOf(configString);
            }
        }else{
            if (preferenceCondensedDistribtionRecipe.getValue() != null){
                recipe = CondensedDistributionRecipe.valueOf(preferenceCondensedDistribtionRecipe.getValue());
            }
        }
    }
}