/**
 * Copyright (C) 2007 EDIT
 * European Distributed Institute of Taxonomy
 * http://www.e-taxonomy.eu
 *
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */

package eu.etaxonomy.taxeditor.preference;

import org.eclipse.core.expressions.PropertyTester;

import eu.etaxonomy.taxeditor.store.CdmStore;

/**
 * @author n.hoffmann
 * @created Jan 26, 2011
 * @version 1.0
 */
public class CdmStorePropertyTester extends PropertyTester {

    private static final String EDITOR_IS_CONNECTED_TO_DB = "isCdmStoreConnected";
    private static final String IS_REMOTING = "isRemoting";
    private static final String IS_STANDALONE = "isStandAlone";
    private static final String IS_CONNECTED_AND_NOT_LOCAL_ACTIVATED = "isCdmStoreConnectedAndNotLocalActivated";
    private static final String IS_CONNECTED_AND_LOCAL_ACTIVATED = "isCdmStoreConnectedAndLocalActive";

    @Override
    public boolean test(Object receiver, String property, Object[] args,
            Object expectedValue) {

        if(EDITOR_IS_CONNECTED_TO_DB.equals(property)){
            return isCdmStoreConnected();
        }

        if(IS_REMOTING.equals(property)){
            if(!isCdmStoreConnected()) {
                return false;
            } else {
                return isRemoting();
            }
        }

        if(IS_STANDALONE.equals(property)){
            if(!isCdmStoreConnected()) {
                return false;
            } else {
                return !isRemoting();
            }
        }
        if(IS_CONNECTED_AND_NOT_LOCAL_ACTIVATED.equals(property)){
            return isCdmStoreConnectedAndNotLocalActive();
        }
        if(IS_CONNECTED_AND_LOCAL_ACTIVATED.equals(property)){
            return isCdmStoreConnectedAndLocalActive();
        }
        return false;
    }

    private boolean isCdmStoreConnected(){
        boolean active = CdmStore.isActive();
        return active;
    }

    private boolean isCdmStoreConnectedAndNotLocalActive(){
        boolean active = CdmStore.isActive();
        active = active & !PreferencesUtil.getLocalActive();
        return active;
    }
    private boolean isCdmStoreConnectedAndLocalActive(){
        boolean active = CdmStore.isActive();
        active = active & PreferencesUtil.getLocalActive();
        return active;
    }

    private boolean isRemoting() {
        return CdmStore.getCurrentSessionManager().isRemoting();
    }
}
