/**
 * Copyright (C) 2014 EDIT
 * European Distributed Institute of Taxonomy
 * http://www.e-taxonomy.eu
 *
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */
package eu.etaxonomy.taxeditor.preference;

import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CLabel;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;

import eu.etaxonomy.cdm.model.metadata.CdmPreference;
import eu.etaxonomy.cdm.model.metadata.PreferencePredicate;
import eu.etaxonomy.cdm.model.metadata.TermComboEnum;
import eu.etaxonomy.cdm.model.metadata.TermDisplayEnum;
import eu.etaxonomy.cdm.model.metadata.TermOrder;
import eu.etaxonomy.taxeditor.l10n.Messages;
import eu.etaxonomy.taxeditor.preference.menu.CdmPreferencePage;

/**
 * @author a.oppermann
 * @date 21.07.2014
 *
 */
public class ChecklistEditorGeneralPreference extends CdmPreferencePage implements SelectionListener {



    protected Boolean isEditorActivated;

    protected boolean overrideActivated;
    boolean isUseLocal;
    protected Boolean isShowRank;
    protected TermOrder orderAreas;
    protected TermOrder orderStatus;
    protected TermDisplayEnum displayStatus;
    protected TermComboEnum displayStatusCombo;
    protected TermDisplayEnum displayArea;
    protected Boolean ownDescriptionForDistributionEditor;
    protected Integer numberOfStatus;

    protected boolean overrideOwnDescriptionForDistributionEditor;
    protected boolean overrideAreaVoc;
    protected boolean overrideRank;
    protected boolean overrideOrderAreas;
    protected boolean overrideAreaDisplay;
    protected boolean overrideStatusDisplay;
    protected boolean overrideStatusDisplayCombo;
    protected boolean overrideNumberOfStatus;
    protected boolean overrideOrderStatus;

    protected Combo areaOrderSelectionCombo;
    protected Button allowOverrideOrderAreasButton;
    protected Combo statusDisplaySelectionCombo;
    protected Button allowOverrideStatusDisplayButton;
    protected Combo statusDisplayInComboSelectionCombo;
    protected Button allowOverrideStatusDisplayInComboButton;
    protected Combo areaDisplaySelectionCombo;
    protected Button allowOverrideAreaDisplayButton;
    protected Combo statusSortSelectionCombo;
    protected Button allowOverrideStatusSortButton;


    protected Text numberOfStatusText;
    protected Button allowOverrideNumberOfStatusButton;
    protected Label numberFormatExceptionLabel;

    protected CdmPreference distributionEditorPref;

    protected CdmPreference prefStatusDisplay;
    protected CdmPreference prefAreaDisplay;

    protected CdmPreference prefStatusDisplayInCombo;
    protected CdmPreference prefAreaSort;
    protected CdmPreference prefRank;
    protected CdmPreference prefOwnDescription;
    protected CdmPreference prefNumberStatus;
    protected CdmPreference prefStatusSort;
    Composite child;

    protected Combo activateCombo;
    protected Button allowOverrideActivatedButton;
    final String ENABLE = "Enable";
    final String DISABLE = "Disable";

    protected Combo activateRankCombo;
    protected Button allowOverrideRankButton;
    final String SHOW_RANK = "Yes";
    final String NO_RANK = "No";




    @Override
    protected Control createContents(Composite parent) {
        getValues();
        Composite composite = new Composite(parent, SWT.NULL);
        composite.setLayout(new GridLayout());
        if (!isAdminPreference && distributionEditorPref != null && !distributionEditorPref.isAllowOverride() ) {
            Label label = new Label(composite, SWT.NONE);
            label.setText(
                    "The CDM settings don't allow to set the preferences for using the distribution editor locally. If you need to make local settings, please ask an administrator.");
            this.setDefaultButtonActivated(false);
        } else {
            Composite activateComp = createComposite(composite);
            Label separator = new Label(activateComp, SWT.HORIZONTAL | SWT.SEPARATOR);
            GridData sepGrid = createTextGridData();
            separator.setLayoutData(sepGrid);
            separator.setVisible(false);

            GridData textGrid = createTextGridData();
            textGrid.verticalSpan = 2;
            separator.setLayoutData(sepGrid);

            activateCombo = createBooleanCombo(activateComp, ENABLE, DISABLE, PreferencePredicate.DistributionEditorActivated, Messages.ChecklistEditorGeneralPreference_enable, isAdminPreference);

            int index = 0;
            for (String itemLabel : activateCombo.getItems()) {
                if (isEditorActivated == null && itemLabel.startsWith(Messages.Preference_Use_Default)) {
                    activateCombo.select(index);
                    break;
                }
                if (isEditorActivated != null && itemLabel.equals(ENABLE) && isEditorActivated){
                    activateCombo.select(index);
                    break;
                }
                if (isEditorActivated != null && itemLabel.equals(DISABLE) && !isEditorActivated){
                    activateCombo.select(index);
                    break;
                }
                index++;
            }


            activateCombo.addSelectionListener(this);
            if (isAdminPreference) {
                allowOverrideActivatedButton = createAllowOverrideButton(activateComp);
                allowOverrideActivatedButton.setSelection(this.distributionEditorPref != null? this.distributionEditorPref.isAllowOverride(): true);
                allowOverrideActivatedButton.addSelectionListener(new SelectionAdapter() {
                    @Override
                    public void widgetSelected(SelectionEvent e) {
                        setApply(true);
                        if (distributionEditorPref == null){
                            distributionEditorPref = CdmPreference.NewTaxEditorInstance(PreferencePredicate.DistributionEditorActivated, PreferencePredicate.DistributionEditorActivated.getDefaultValue() != null ?  PreferencePredicate.DistributionEditorActivated.getDefaultValue().toString(): null);
                        }
                        distributionEditorPref.setAllowOverride(!distributionEditorPref.isAllowOverride());
                    }
                });
            }
            child = createComposite(composite);

            if (!isAdminPreference) {
                if (isEditorActivated != null){
                    child.setEnabled(isEditorActivated);
                }else{
                    child.setEnabled(distributionEditorPref.getValue() != null?Boolean.parseBoolean(distributionEditorPref.getValue().toString()): true);
                }
            }
            separator = new Label(child, SWT.HORIZONTAL | SWT.SEPARATOR);
            sepGrid = createTextGridData();
            separator.setLayoutData(sepGrid);

            activateRankCombo = createBooleanCombo(child, SHOW_RANK, NO_RANK,
                    PreferencePredicate.ShowRankInDistributionEditor, Messages.ChecklistEditorGeneralPreference_show_rank, isAdminPreference);
            index = 0;
            for (String itemLabel : activateRankCombo.getItems()) {
                if (isShowRank == null && itemLabel.startsWith(Messages.Preference_Use_Default)) {
                    activateRankCombo.select(index);
                    break;
                }
                if (isShowRank != null && itemLabel.equals(SHOW_RANK) && isShowRank){
                    activateRankCombo.select(index);
                    break;
                }
                if (isShowRank != null && itemLabel.equals(NO_RANK) && !isShowRank){
                    activateRankCombo.select(index);
                    break;
                }
                index++;
            }
            activateRankCombo.addSelectionListener(this);
            if (isAdminPreference) {
                allowOverrideRankButton = createAllowOverrideButton(child);
                allowOverrideRankButton.setSelection(prefRank != null? prefRank.isAllowOverride():true);
                allowOverrideRankButton.addSelectionListener(new SelectionAdapter() {
                    @Override
                    public void widgetSelected(SelectionEvent e) {
                        setApply(true);
                        if (prefRank == null){
                            prefRank = CdmPreference.NewTaxEditorInstance(PreferencePredicate.ShowRankInDistributionEditor, PreferencePredicate.ShowRankInDistributionEditor.getDefaultValue() != null ?  PreferencePredicate.ShowRankInDistributionEditor.getDefaultValue().toString(): null);
                        }
                        prefRank.setAllowOverride(allowOverrideRankButton.getSelection());
                    }
                });
            }
            separator = new Label(child, SWT.HORIZONTAL | SWT.SEPARATOR);
            sepGrid = createTextGridData();
            separator.setLayoutData(sepGrid);

            areaOrderSelectionCombo = createCombo(child, TermOrder.values(), PreferencePredicate.AreasSortedInDistributionEditor, Messages.ChecklistEditorGeneralPreference_Configure_area_order, isAdminPreference);

            index = 0;

            for (String itemLabel : areaOrderSelectionCombo.getItems()) {
                if (itemLabel.startsWith(Messages.Preference_Use_Default) && orderAreas == null){
                    areaOrderSelectionCombo.select(index);
                    break;
                }
                if (itemLabel.equals(orderAreas.getLabel())) {
                    areaOrderSelectionCombo.select(index);
                    break;
                }

                index++;
            }

            areaOrderSelectionCombo.addSelectionListener(this);

            if (isAdminPreference) {
                allowOverrideOrderAreasButton = createAllowOverrideButton(child);
                allowOverrideOrderAreasButton.setSelection(prefAreaSort != null? prefAreaSort.isAllowOverride(): true);
                allowOverrideOrderAreasButton.addSelectionListener(new SelectionAdapter() {
                    @Override
                    public void widgetSelected(SelectionEvent e) {
                        setApply(true);
                        if (prefAreaSort == null){
                            prefAreaSort = CdmPreference.NewTaxEditorInstance(PreferencePredicate.AreasSortedInDistributionEditor, PreferencePredicate.AreasSortedInDistributionEditor.getDefaultValue() != null ?  PreferencePredicate.AreasSortedInDistributionEditor.getDefaultValue().toString(): null);
                        }
                        prefAreaSort.setAllowOverride(allowOverrideOrderAreasButton.getSelection());
                    }
                });
            }

//
            areaDisplaySelectionCombo = createCombo(child, TermDisplayEnum.values(), PreferencePredicate.DisplayOfAreasInDistributionEditor, Messages.ChecklistEditorGeneralPreference_Configure_display_of_Areas, isAdminPreference);


            index = 0;

            for (String itemLabel : areaDisplaySelectionCombo.getItems()) {
                if (itemLabel.startsWith(Messages.Preference_Use_Default) && displayArea == null){
                    areaDisplaySelectionCombo.select(index);
                    break;
                }else if (itemLabel.equals(displayArea.getLabel())) {
                    areaDisplaySelectionCombo.select(index);
                    break;
                }

                index++;
            }
            areaDisplaySelectionCombo.addSelectionListener(this);

            if (isAdminPreference) {
                allowOverrideAreaDisplayButton = createAllowOverrideButton(child);
                allowOverrideAreaDisplayButton.setSelection(prefAreaDisplay != null? prefAreaDisplay.isAllowOverride(): true);
                allowOverrideAreaDisplayButton.addSelectionListener(new SelectionAdapter() {
                    @Override
                    public void widgetSelected(SelectionEvent e) {
                        setApply(true);
                        if (prefAreaDisplay == null){
                            prefAreaDisplay = CdmPreference.NewTaxEditorInstance(PreferencePredicate.DisplayOfAreasInDistributionEditor, PreferencePredicate.DisplayOfAreasInDistributionEditor.getDefaultValue() != null ?  PreferencePredicate.DisplayOfAreasInDistributionEditor.getDefaultValue().toString(): null);
                        }
                        prefAreaDisplay.setAllowOverride(allowOverrideOrderAreasButton.getSelection());
                    }
                });
            }

            separator = new Label(child, SWT.HORIZONTAL | SWT.SEPARATOR);
            separator.setLayoutData(sepGrid);

            statusDisplaySelectionCombo = createCombo(child, TermDisplayEnum.values(), PreferencePredicate.DisplayOfStatus, Messages.ChecklistEditorGeneralPreference_Configure_display_of_Status, isAdminPreference);

            index = 0;

            for (String itemLabel : statusDisplaySelectionCombo.getItems()) {
                if (itemLabel.startsWith(Messages.Preference_Use_Default) && displayStatus == null){
                    statusDisplaySelectionCombo.select(index);
                    break;
                }
                if (itemLabel.equals(displayStatus.getLabel())) {
                    statusDisplaySelectionCombo.select(index);
                    break;
                }

                index++;
            }

            statusDisplaySelectionCombo.addSelectionListener(this);

            if (isAdminPreference) {
                allowOverrideStatusDisplayButton = createAllowOverrideButton(child);
                allowOverrideStatusDisplayButton.setSelection(prefStatusDisplay != null ? prefStatusDisplay.isAllowOverride(): true);
                allowOverrideStatusDisplayButton.addSelectionListener(new SelectionAdapter() {
                    @Override
                    public void widgetSelected(SelectionEvent e) {
                        setApply(true);
                        if (prefStatusDisplay == null){
                            prefStatusDisplay = CdmPreference.NewTaxEditorInstance(PreferencePredicate.DisplayOfStatus, PreferencePredicate.DisplayOfStatus.getDefaultValue() != null ?  PreferencePredicate.DisplayOfStatus.getDefaultValue().toString(): null);
                        }
                        prefStatusDisplay.setAllowOverride(allowOverrideStatusDisplayButton.getSelection());
                    }
                });
            }

            statusDisplayInComboSelectionCombo = createCombo(child, TermComboEnum.values(), PreferencePredicate.DisplayOfStatusInCombo, Messages.ChecklistEditorGeneralPreference_Configure_display_of_Status_in_Combo, isAdminPreference);

            index = 0;

            for (String itemLabel : statusDisplayInComboSelectionCombo.getItems()) {
                if (itemLabel.startsWith(Messages.Preference_Use_Default) && displayStatusCombo == null){
                    statusDisplayInComboSelectionCombo.select(index);
                    break;
                }
                if (itemLabel.equals(displayStatusCombo.getLabel())) {
                    statusDisplayInComboSelectionCombo.select(index);
                    break;
                }

                index++;
            }

            statusDisplayInComboSelectionCombo.addSelectionListener(this);

            if (isAdminPreference) {
                allowOverrideStatusDisplayInComboButton = createAllowOverrideButton(child);
                allowOverrideStatusDisplayInComboButton.setSelection(prefStatusDisplayInCombo != null ? prefStatusDisplayInCombo.isAllowOverride(): true);
                //allowOverrideStatusDisplayInComboButton.setEnabled(prefStatusDisplayInCombo != null);
                allowOverrideStatusDisplayInComboButton.addSelectionListener(new SelectionAdapter() {
                    @Override
                    public void widgetSelected(SelectionEvent e) {
                        setApply(true);
                        if (prefStatusDisplayInCombo == null){
                            prefStatusDisplayInCombo = CdmPreference.NewTaxEditorInstance(PreferencePredicate.DisplayOfStatusInCombo, PreferencePredicate.DisplayOfStatusInCombo.getDefaultValue() != null ?  PreferencePredicate.DisplayOfStatusInCombo.getDefaultValue().toString(): null);
                        }
                        prefStatusDisplayInCombo.setAllowOverride(allowOverrideStatusDisplayInComboButton.getSelection());
                    }
                });
            }

            statusSortSelectionCombo = createCombo(child, TermOrder.values(), PreferencePredicate.StatusSortedInDistributionEditor, Messages.ChecklistEditorGeneralPreference_Configure_status_order, isAdminPreference);

            index = 0;

            for (String itemLabel : statusSortSelectionCombo.getItems()) {
                if (itemLabel.startsWith(Messages.Preference_Use_Default) && orderStatus == null){
                    statusSortSelectionCombo.select(index);
                    break;
                }
                if (itemLabel.equals(orderStatus.getLabel())) {
                    statusSortSelectionCombo.select(index);
                    break;
                }

                index++;
            }

            statusSortSelectionCombo.addSelectionListener(this);

            if (isAdminPreference) {
                allowOverrideStatusSortButton = createAllowOverrideButton(child);
                allowOverrideStatusSortButton.setSelection(prefStatusSort != null? prefStatusSort.isAllowOverride(): true);
                allowOverrideStatusSortButton.addSelectionListener(new SelectionAdapter() {
                    @Override
                    public void widgetSelected(SelectionEvent e) {
                        setApply(true);
                        if (prefStatusSort == null){
                            prefStatusSort = CdmPreference.NewTaxEditorInstance(PreferencePredicate.StatusSortedInDistributionEditor, PreferencePredicate.StatusSortedInDistributionEditor.getDefaultValue() != null ?  PreferencePredicate.StatusSortedInDistributionEditor.getDefaultValue().toString(): null);
                        }
                        prefStatusSort.setAllowOverride(allowOverrideStatusSortButton.getSelection());
                    }
                });
            }



            final CLabel description = new CLabel(child, SWT.NULL);
            description.setText(Messages.ChecklistEditorGeneralPreference_numberOfStatus);
            GridData numberOfStatusGrid= createTextGridData();
//            numberOfStatusGrid.verticalSpan = 1;
            description.setLayoutData(numberOfStatusGrid);
            description.setToolTipText(Messages.ChecklistEditorGeneralPreference_tooltip_numberOfStatus);

            numberOfStatusText = new Text(child, SWT.BORDER);
//            numberOfStatusText.setSize(30, numberOfStatusText.getSize().y);
            numberOfStatusText.setLayoutData(new GridData(20,15));

            if (numberOfStatus != null){
                numberOfStatusText.setText(this.numberOfStatus.toString());
            }else{
                numberOfStatusText.setText(PreferencePredicate.NumberOfVisibleStatusInDropDown.getDefaultValue().toString());
            }

            numberOfStatusText.addModifyListener(new ModifyListener() {

                @Override
                public void modifyText(ModifyEvent e) {
                   setApply(true);
                   String text = numberOfStatusText.getText();
                   try{
                       Integer tempNumber = Integer.valueOf(text);
                       if(tempNumber <= 0){
                           throw new NumberFormatException();
                       }
                       numberOfStatus = tempNumber;
                       numberFormatExceptionLabel.setVisible(false);
                       numberOfStatusText.setBackground(Display.getCurrent().getSystemColor(SWT.COLOR_WHITE));
                   }catch(NumberFormatException ex){
                       numberOfStatusText.setBackground(Display.getCurrent().getSystemColor(SWT.COLOR_RED));
                       numberFormatExceptionLabel.setVisible(true);
                       numberFormatExceptionLabel.setText(Messages.ChecklistEditorGeneralPreference_numberFormatExceptionLabel);
                   }

                }
            });


            if (isAdminPreference) {
                allowOverrideNumberOfStatusButton = createAllowOverrideButton(child);
                allowOverrideNumberOfStatusButton.setSelection(this.prefNumberStatus != null? this.prefNumberStatus.isAllowOverride(): true);
                allowOverrideNumberOfStatusButton.addSelectionListener(new SelectionAdapter() {
                    @Override
                    public void widgetSelected(SelectionEvent e) {
                        setApply(true);
                        if (prefNumberStatus == null){
                            prefNumberStatus = CdmPreference.NewTaxEditorInstance(PreferencePredicate.NumberOfVisibleStatusInDropDown, PreferencePredicate.NumberOfVisibleStatusInDropDown.getDefaultValue() != null ?  PreferencePredicate.NumberOfVisibleStatusInDropDown.getDefaultValue().toString(): null);
                        }
                        prefNumberStatus.setAllowOverride(!prefNumberStatus.isAllowOverride());
                    }
                });
            }

            numberFormatExceptionLabel = new Label(child, SWT.NULL);
            numberFormatExceptionLabel.setForeground(Display.getCurrent().getSystemColor(SWT.COLOR_RED));
            numberFormatExceptionLabel.setVisible(false);
            description.setText(Messages.ChecklistEditorGeneralPreference_numberOfStatus);
            GridData numberOfStatusExceptionGrid= createTextGridData();
//            numberOfStatusGrid.verticalSpan = 1;
            numberFormatExceptionLabel.setLayoutData(numberOfStatusExceptionGrid);
            numberFormatExceptionLabel.setText(Messages.ChecklistEditorGeneralPreference_numberFormatExceptionLabel);
            if (!isAdminPreference) {

                if (distributionEditorPref != null && !distributionEditorPref.isAllowOverride()){
                    activateComp.setEnabled(false);
                }


                if (prefRank != null && !prefRank.isAllowOverride()) {
                    activateRankCombo.setEnabled(false);
                }
                if (prefAreaSort != null && !prefAreaSort.isAllowOverride()) {
                    areaOrderSelectionCombo.setEnabled(false);
                }
                if (prefAreaDisplay != null && !prefAreaDisplay.isAllowOverride()) {
                    areaDisplaySelectionCombo.setEnabled(false);
                }
                if (prefStatusDisplay != null && !prefStatusDisplay.isAllowOverride()) {
                    statusDisplaySelectionCombo.setEnabled(false);
                }
                if (prefStatusSort != null && !prefStatusSort.isAllowOverride()) {
                    statusSortSelectionCombo.setEnabled(false);
                }
                if (prefStatusDisplayInCombo != null && !prefStatusDisplayInCombo.isAllowOverride()) {
                    statusDisplayInComboSelectionCombo.setEnabled(false);
                }
                if (prefNumberStatus != null && !prefNumberStatus.isAllowOverride()){
                    numberOfStatusText.setEnabled(false);
                }
            }

        }
        setValid(true);
        return composite;
    }

    @Override
    public boolean performOk() {
        if (!isApply()){
            return true;
        }
            boolean override = false;
            if (isEditorActivated != null ) {
                override = true;
                PreferencesUtil.setStringValue(PreferencePredicate.DistributionEditorActivated.getKey(), isEditorActivated.toString());
            }
            PreferencesUtil.setBooleanValue(
                    PreferencesUtil.prefOverrideKey(PreferencePredicate.DistributionEditorActivated.getKey()),
                    override);


            override = false;
            if (orderAreas != null ) {
                override = true;
                PreferencesUtil.setSortNamedAreasInDistributionEditor(orderAreas.getKey());
            }
            PreferencesUtil.setBooleanValue(
                    PreferencesUtil.prefOverrideKey(PreferencePredicate.AreasSortedInDistributionEditor.getKey()),
                    override);

            override = false;
            if (isShowRank != null ) {
                override = true;
                PreferencesUtil.setShowRankInChecklistEditor(isShowRank);
            }
            PreferencesUtil.setBooleanValue(
                    PreferencesUtil.prefOverrideKey(PreferencePredicate.ShowRankInDistributionEditor.getKey()),
                    override);


            override = false;
            if (displayStatus != null) {
               override = true;
               PreferencesUtil.setDisplayStatusInChecklistEditor(displayStatus.getKey());
            }
            PreferencesUtil.setBooleanValue(
                    PreferencesUtil.prefOverrideKey(PreferencePredicate.DisplayOfStatus.getKey()),
                    override);

            override = false;
            if (displayStatusCombo != null) {
               override = true;
               PreferencesUtil.setStringValue(PreferencePredicate.DisplayOfStatusInCombo.getKey(), displayStatusCombo.getKey());
            }
            PreferencesUtil.setBooleanValue(
                    PreferencesUtil.prefOverrideKey(PreferencePredicate.DisplayOfStatusInCombo.getKey()),
                    override);



            override = false;
            if (displayArea != null) {
                override = true;
                PreferencesUtil.setAreaDisplayInChecklistEditor(displayArea.getKey());
            }
            PreferencesUtil.setBooleanValue(PreferencesUtil.prefOverrideKey(
                    PreferencePredicate.DisplayOfAreasInDistributionEditor.getKey()), override);


            PreferencesUtil.firePreferencesChanged(this.getClass());

            override = false;
            if (orderStatus != null) {
               override = true;
               PreferencesUtil.setStringValue(PreferencePredicate.StatusSortedInDistributionEditor.getKey(), orderStatus.getKey());
            }
            PreferencesUtil.setBooleanValue(
                    PreferencesUtil.prefOverrideKey(PreferencePredicate.StatusSortedInDistributionEditor.getKey()),
                    override);

            override = false;
            if (numberOfStatus != null) {
                override = true;
                PreferencesUtil.setIntValue(PreferencePredicate.NumberOfVisibleStatusInDropDown.getKey(), numberOfStatus);
            }
            PreferencesUtil.setBooleanValue(PreferencesUtil.prefOverrideKey(
                    PreferencePredicate.NumberOfVisibleStatusInDropDown.getKey()), override);


            PreferencesUtil.firePreferencesChanged(this.getClass());

        return true;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public void widgetSelected(SelectionEvent e) {
        setApply(true);
        if (e.getSource().equals(statusDisplaySelectionCombo)) {
            String text = statusDisplaySelectionCombo.getText();
            if(text.startsWith(Messages.Preference_Use_Default)){
                displayStatus = null;
                return;
            }

            for (TermDisplayEnum display : TermDisplayEnum.values()) {
                if (text.startsWith(display.getLabel())) {
                    displayStatus = display;
                    break;
                }

            }
        }
        if (e.getSource().equals(statusDisplayInComboSelectionCombo)) {
            String text = statusDisplayInComboSelectionCombo.getText();
            if(text.startsWith(Messages.Preference_Use_Default)){
                displayStatusCombo = null;
                return;
            }

            for (TermComboEnum display : TermComboEnum.values()) {
                if (text.startsWith(display.getLabel())) {
                    displayStatusCombo = display;
                    break;
                }

            }
        }
        if (e.getSource().equals(areaDisplaySelectionCombo)) {
            String text = areaDisplaySelectionCombo.getText();
            if(text.startsWith(Messages.Preference_Use_Default)){
                displayArea = null;
                return;
            }

            for (TermDisplayEnum display : TermDisplayEnum.values()) {
                if (text.startsWith(display.getLabel())) {
                    displayArea = display;
                    break;
                }
            }
        }
        if (e.getSource().equals(areaOrderSelectionCombo)) {
            String text = areaOrderSelectionCombo.getText();
            if(text.startsWith(Messages.Preference_Use_Default)){
                orderAreas = null;
                return;
            }

            for (TermOrder display : TermOrder.values()) {
                if (text.startsWith(display.getLabel())) {
                    orderAreas = display;
                    break;
                }
            }
        }

        if (e.getSource().equals(statusSortSelectionCombo)) {
            String text = statusSortSelectionCombo.getText();
            if(text.startsWith(Messages.Preference_Use_Default)){
                orderStatus = null;
                return;
            }

            for (TermOrder display : TermOrder.values()) {
                if (text.startsWith(display.getLabel())) {
                    orderStatus = display;
                    break;
                }
            }
        }
        if (e.getSource().equals(this.activateRankCombo)) {
            String text = activateRankCombo.getText();
            if(text.startsWith(Messages.Preference_Use_Default)){
                isShowRank = null;
                return;
            }

            if (text.equals(SHOW_RANK)){
                isShowRank = true;
            }else{
                isShowRank = false;
            }
        }


        if (e.getSource().equals(this.activateCombo)) {
            String text = activateCombo.getText();
            if(text.startsWith(Messages.Preference_Use_Default)){
                isEditorActivated = null;
                if (!isAdminPreference) {
                    if (isEditorActivated != null){
                        child.setEnabled(isEditorActivated);
                    }else{
                        child.setEnabled(distributionEditorPref.getValue() != null?Boolean.parseBoolean(distributionEditorPref.getValue().toString()): true);
                    }
                }
                return;
            }

            if (text.equals(ENABLE)){
                isEditorActivated = true;
            }else{
                isEditorActivated = false;
            }
            if (!isAdminPreference) {
                if (isEditorActivated != null){
                    child.setEnabled(isEditorActivated);
                }else{
                    child.setEnabled(distributionEditorPref.getValue() != null?Boolean.parseBoolean(distributionEditorPref.getValue().toString()): true);
                }
            }

        }

    }

    /**
     * {@inheritDoc}
     */
    @Override
    public void widgetDefaultSelected(SelectionEvent e) {
        // TODO Auto-generated method stub

    }

    /**
     * {@inheritDoc}
     */
    @Override
    public void getValues() {
        isAdminPreference = false;
        setPreferenceStore(PreferencesUtil.getPreferenceStore());
        distributionEditorPref = PreferencesUtil.getPreferenceFromDB(PreferencePredicate.DistributionEditorActivated);
        overrideActivated = PreferencesUtil.getBooleanValue(
                PreferencesUtil.prefOverrideKey(PreferencePredicate.DistributionEditorActivated.getKey()), true) != null? PreferencesUtil.getBooleanValue(
                        PreferencesUtil.prefOverrideKey(PreferencePredicate.DistributionEditorActivated.getKey()), true):false;

        if (distributionEditorPref != null) {
            if (distributionEditorPref.isAllowOverride() ) {
                if (overrideActivated){
                    isEditorActivated = PreferencesUtil.getBooleanValue(PreferencePredicate.DistributionEditorActivated.getKey(), true);
                }

            } else {
                isEditorActivated = distributionEditorPref.getValue() != null? Boolean.valueOf(distributionEditorPref.getValue()): null;
            }

        } else {
            if(!overrideActivated){
                isEditorActivated = null;
            }else{
                isEditorActivated = PreferencesUtil.getBooleanValue(PreferencePredicate.DistributionEditorActivated.getKey(), true);
            }
            distributionEditorPref = CdmPreference.NewTaxEditorInstance(PreferencePredicate.DistributionEditorActivated, PreferencePredicate.DistributionEditorActivated.getDefaultValue() != null ?  PreferencePredicate.DistributionEditorActivated.getDefaultValue().toString(): null);
        }




        prefAreaDisplay = PreferencesUtil.getPreferenceFromDB(PreferencePredicate.DisplayOfAreasInDistributionEditor);
        overrideAreaDisplay = PreferencesUtil.getBooleanValue(
                PreferencesUtil.prefOverrideKey(PreferencePredicate.DisplayOfAreasInDistributionEditor.getKey()), true) != null? PreferencesUtil.getBooleanValue(
                        PreferencesUtil.prefOverrideKey(PreferencePredicate.DisplayOfAreasInDistributionEditor.getKey()), true): false;

        if (prefAreaDisplay != null) {
            if (prefAreaDisplay.isAllowOverride()){
                if (overrideActivated) {
                    displayArea = PreferencesUtil.displayAreaInChecklistEditor();
                }else{
                    displayArea = null;
                }
            } else {
                displayArea = prefAreaDisplay.getValue() != null? TermDisplayEnum.byKey(prefAreaDisplay.getValue()): null;
            }
        } else {
            if (overrideAreaDisplay){
                displayArea = PreferencesUtil.displayAreaInChecklistEditor();
            } else{
                displayArea = null;
            }
            //prefAreaDisplay = CdmPreference.NewTaxEditorInstance(PreferencePredicate.DisplayOfAreasInDistributionEditor, PreferencePredicate.DisplayOfAreasInDistributionEditor.getDefaultValue() != null ?  PreferencePredicate.DisplayOfAreasInDistributionEditor.getDefaultValue().toString(): null);
        }

        prefStatusDisplay = PreferencesUtil.getPreferenceFromDB(PreferencePredicate.DisplayOfStatus);
        overrideStatusDisplay = PreferencesUtil
                .getBooleanValue(PreferencesUtil.prefOverrideKey(PreferencePredicate.DisplayOfStatus.getKey()), true) != null? PreferencesUtil
                        .getBooleanValue(PreferencesUtil.prefOverrideKey(PreferencePredicate.DisplayOfStatus.getKey()), true): false;
        if (prefStatusDisplay != null) {
            if (prefStatusDisplay.isAllowOverride()){
                if (overrideStatusDisplay) {
                    displayStatus = PreferencesUtil.displayStatusInChecklistEditor();
                }else{
                    displayStatus = null;
                }
            } else {
                this.displayStatus = prefStatusDisplay.getValue() != null? TermDisplayEnum.byKey(prefStatusDisplay.getValue()): null;
            }
        } else {
            if (overrideStatusDisplay){
                displayStatus = PreferencesUtil.displayStatusInChecklistEditor();
            } else{
                displayStatus = null;
            }
            //prefStatusDisplay = CdmPreference.NewTaxEditorInstance(PreferencePredicate.DisplayOfStatus, PreferencePredicate.DisplayOfStatus.getDefaultValue() != null ?  PreferencePredicate.DisplayOfStatus.getDefaultValue().toString(): null);
        }

        prefStatusDisplayInCombo = PreferencesUtil.getPreferenceFromDB(PreferencePredicate.DisplayOfStatusInCombo);
        overrideStatusDisplayCombo = PreferencesUtil
                .getBooleanValue(PreferencesUtil.prefOverrideKey(PreferencePredicate.DisplayOfStatusInCombo.getKey()), true) != null? PreferencesUtil
                        .getBooleanValue(PreferencesUtil.prefOverrideKey(PreferencePredicate.DisplayOfStatusInCombo.getKey()), true): false;
        if (prefStatusDisplayInCombo != null) {
            if (prefStatusDisplayInCombo.isAllowOverride()){
                if (overrideStatusDisplayCombo) {
                    displayStatusCombo = TermComboEnum.byKey(PreferencesUtil.getStringValue(PreferencePredicate.DisplayOfStatusInCombo.getKey()));
                }else{
                    displayStatusCombo = null;
                }
            } else {
                this.displayStatusCombo = prefStatusDisplayInCombo.getValue() != null? TermComboEnum.byKey(prefStatusDisplayInCombo.getValue()): null;
            }
        } else {
            if (overrideStatusDisplayCombo){
                displayStatusCombo = TermComboEnum.byKey(PreferencesUtil.getStringValue(PreferencePredicate.DisplayOfStatusInCombo.getKey()));
            } else{
                displayStatusCombo = null;
            }

        }


        prefAreaSort = PreferencesUtil.getPreferenceFromDB(PreferencePredicate.AreasSortedInDistributionEditor);
        overrideOrderAreas = PreferencesUtil.getBooleanValue(
                PreferencesUtil.prefOverrideKey(PreferencePredicate.AreasSortedInDistributionEditor.getKey()), true) != null? PreferencesUtil.getBooleanValue(
                        PreferencesUtil.prefOverrideKey(PreferencePredicate.AreasSortedInDistributionEditor.getKey()), true): false;
        if (prefAreaSort != null) {
            if (prefAreaSort.isAllowOverride()){
                if(overrideOrderAreas) {
                    orderAreas = PreferencesUtil.getSortNamedAreasInDistributionEditor();
                }else{
                    orderAreas = null;
                }
            } else {
                if (prefAreaSort.getValue() != null) {
                    orderAreas = prefAreaSort.getValue() != null? TermOrder.valueOf(prefAreaSort.getValue().toString()): null;
                } else {
                    orderAreas = ((TermOrder) PreferencePredicate.AreasSortedInDistributionEditor.getDefaultValue());
                }

            }
        } else {
            if (overrideOrderAreas){
                orderAreas = PreferencesUtil.getSortNamedAreasInDistributionEditor();
            }else{
                orderAreas = null;
            }
            //prefAreaSort = CdmPreference.NewTaxEditorInstance(PreferencePredicate.AreasSortedInDistributionEditor, PreferencePredicate.AreasSortedInDistributionEditor.getDefaultValue() != null ?  PreferencePredicate.AreasSortedInDistributionEditor.getDefaultValue().toString(): null);
        }

        prefStatusSort = PreferencesUtil.getPreferenceFromDB(PreferencePredicate.StatusSortedInDistributionEditor);
        overrideOrderStatus = PreferencesUtil.getBooleanValue(
                PreferencesUtil.prefOverrideKey(PreferencePredicate.StatusSortedInDistributionEditor.getKey()), true) != null? PreferencesUtil.getBooleanValue(
                        PreferencesUtil.prefOverrideKey(PreferencePredicate.StatusSortedInDistributionEditor.getKey()), true): false;
        if (prefStatusSort != null) {
            if (prefStatusSort.isAllowOverride()){
                if(overrideOrderStatus) {
                    orderStatus = TermOrder.valueOf(PreferencesUtil.getStringValue(PreferencePredicate.StatusSortedInDistributionEditor.getKey()));
                }else{
                    orderStatus = null;
                }
            } else {
                if (prefStatusSort.getValue() != null) {
                    orderStatus = prefStatusSort.getValue() != null? TermOrder.valueOf(prefStatusSort.getValue().toString()): null;
                } else {
                    orderStatus = ((TermOrder) PreferencePredicate.StatusSortedInDistributionEditor.getDefaultValue());
                }

            }
        } else {
            if (overrideOrderStatus){
                try{
                    orderStatus = TermOrder.valueOf(PreferencesUtil.getStringValue(PreferencePredicate.StatusSortedInDistributionEditor.getKey()));
                }catch (IllegalArgumentException e){
                    orderStatus = TermOrder.Label;
                }
            }else{
                orderStatus = null;
            }
            //prefAreaSort = CdmPreference.NewTaxEditorInstance(PreferencePredicate.AreasSortedInDistributionEditor, PreferencePredicate.AreasSortedInDistributionEditor.getDefaultValue() != null ?  PreferencePredicate.AreasSortedInDistributionEditor.getDefaultValue().toString(): null);
        }

        prefRank = PreferencesUtil.getPreferenceFromDB(PreferencePredicate.ShowRankInDistributionEditor);
        overrideRank = PreferencesUtil.getBooleanValue(
                PreferencesUtil.prefOverrideKey(PreferencePredicate.ShowRankInDistributionEditor.getKey()), true) != null? PreferencesUtil.getBooleanValue(
                        PreferencesUtil.prefOverrideKey(PreferencePredicate.ShowRankInDistributionEditor.getKey()), true): false;
        if (prefRank != null) {
            if (prefRank.isAllowOverride()){
                if(overrideRank) {
                    isShowRank = PreferencesUtil.isShowRankInChecklistEditor();
                }

            } else {
                this.isShowRank = prefRank.getValue() != null? Boolean.valueOf(prefRank.getValue().toString()): null;
            }
        } else {
            if (overrideRank){
                isShowRank = PreferencesUtil.isShowRankInChecklistEditor();
            } else {
                isShowRank = null;
            }

        }

        prefNumberStatus= PreferencesUtil.getPreferenceFromDB(PreferencePredicate.NumberOfVisibleStatusInDropDown);
        overrideNumberOfStatus = PreferencesUtil.getBooleanValue(
                PreferencesUtil.prefOverrideKey(PreferencePredicate.NumberOfVisibleStatusInDropDown.getKey()), true) != null? PreferencesUtil.getBooleanValue(
                        PreferencesUtil.prefOverrideKey(PreferencePredicate.NumberOfVisibleStatusInDropDown.getKey()), true): false;
        if (prefNumberStatus != null) {
            if (prefNumberStatus.isAllowOverride()){
                if(overrideNumberOfStatus) {
                    numberOfStatus = PreferencesUtil.getIntValue(PreferencePredicate.NumberOfVisibleStatusInDropDown.getKey(), true);
                }

            } else {
                this.numberOfStatus = prefNumberStatus.getValue() != null? Integer.valueOf(prefNumberStatus.getValue().toString()): null;
            }
        } else {
            if (overrideNumberOfStatus){
                numberOfStatus = PreferencesUtil.getIntValue(PreferencePredicate.NumberOfVisibleStatusInDropDown.getKey(), true);
            } else {
                numberOfStatus = null;
            }

        }

        prefOwnDescription = PreferencesUtil
                .getPreferenceFromDB(PreferencePredicate.OwnDescriptionForDistributionEditor);
        if (prefOwnDescription != null) {
            if (prefOwnDescription.isAllowOverride()) {
                ownDescriptionForDistributionEditor = PreferencesUtil.isOwnDescriptionForChecklistEditor();
            } else {
                ownDescriptionForDistributionEditor = prefOwnDescription.getValue() != null? Boolean.valueOf(prefOwnDescription.getValue().toString()): null;
            }
        } else {
            ownDescriptionForDistributionEditor = PreferencesUtil.isShowRankInChecklistEditor();
            // allowOverrideOwnDescriptionForDistributionEditor = true;
            overrideOwnDescriptionForDistributionEditor = PreferencesUtil.getBooleanValue(
                    PreferencesUtil.prefOverrideKey(PreferencePredicate.OwnDescriptionForDistributionEditor.getKey()), true) != null? PreferencesUtil.getBooleanValue(
                            PreferencesUtil.prefOverrideKey(PreferencePredicate.OwnDescriptionForDistributionEditor.getKey()), true): false;
            //prefOwnDescription = CdmPreference.NewTaxEditorInstance(PreferencePredicate.OwnDescriptionForDistributionEditor, PreferencePredicate.OwnDescriptionForDistributionEditor.getDefaultValue() != null ?  PreferencePredicate.OwnDescriptionForDistributionEditor.getDefaultValue().toString(): null);
        }


    }

    @Override
    protected void performDefaults() {

        setApply(true);
        isEditorActivated = null;
        activateCombo.select(0);


        displayArea = null;

        int index = 0;
        for (String itemLabel : areaDisplaySelectionCombo.getItems()) {
            if (itemLabel.startsWith(Messages.Preference_Use_Default)) {
                areaDisplaySelectionCombo.select(index);
                break;
            }
            index++;
        }

        displayStatus = null;



        index = 0;
        for (String itemLabel : statusDisplaySelectionCombo.getItems()) {
            if (itemLabel.startsWith(Messages.Preference_Use_Default)) {
                statusDisplaySelectionCombo.select(index);
                break;
            }
            index++;
        }

        orderAreas = null;


        index = 0;
        for (String itemLabel : areaOrderSelectionCombo.getItems()) {
            if (itemLabel.startsWith(Messages.Preference_Use_Default)) {
                areaOrderSelectionCombo.select(index);
                break;
            }
            index++;
        }
        orderStatus = null;


        index = 0;
        for (String itemLabel : statusSortSelectionCombo.getItems()) {
            if (itemLabel.startsWith(Messages.Preference_Use_Default)) {
                statusSortSelectionCombo.select(index);
                break;
            }
            index++;
        }

        isShowRank = null;

        index = 0;
        for (String itemLabel : activateRankCombo.getItems()) {
            if (itemLabel.startsWith(Messages.Preference_Use_Default)) {
                activateRankCombo.select(index);
                break;
            }
            index++;
        }

        displayStatusCombo = null;
        index = 0;
        for (String itemLabel : statusDisplayInComboSelectionCombo.getItems()) {
            if (itemLabel.startsWith(Messages.Preference_Use_Default)) {
                statusDisplayInComboSelectionCombo.select(index);
                break;
            }
            index++;
        }

        ownDescriptionForDistributionEditor = null;
        if (prefNumberStatus == null){
            numberOfStatusText.setText(PreferencePredicate.NumberOfVisibleStatusInDropDown.getDefaultValue().toString());
        }else{
            numberOfStatusText.setText(prefNumberStatus.getValue().toString());
        }
        numberOfStatus = null;
        if (prefNumberStatus != null && !prefNumberStatus.isAllowOverride() && !isAdminPreference){
            numberOfStatusText.setEnabled(false);
        }

        if (isAdminPreference){
            allowOverrideActivatedButton.setSelection(true);
            allowOverrideAreaDisplayButton.setSelection(true);
            allowOverrideStatusDisplayButton.setSelection(true);
            allowOverrideOrderAreasButton.setSelection(true);
            allowOverrideRankButton.setSelection(true);
            allowOverrideNumberOfStatusButton.setSelection(true);
        }
        overrideActivated = true;
        overrideAreaDisplay = true;
        overrideStatusDisplay = true;
        overrideOrderAreas = true;
        overrideRank = true;
        overrideOwnDescriptionForDistributionEditor = true;
        overrideNumberOfStatus = true;

        if (!isAdminPreference) {
//            PreferencesUtil.recursiveSetEnabled(child, Boolean.parseBoolean(distributionEditorPref.getValue()));
//            if (!prefRank.isAllowOverride()) {
//                activateRankCombo.setEnabled(false);
//
//                if (!prefAreaSort.isAllowOverride()) {
//                    areaOrderSelectionCombo.setEnabled(false);
//                }
//                if (!prefAreaDisplay.isAllowOverride()) {
//                    allowOverrideAreaDisplayButton.setEnabled(false);
//                }
//                if (!prefStatusDisplay.isAllowOverride()) {
//                    allowOverrideStatusDisplayButton.setEnabled(false);
//                }
//            }
            super.performDefaults();
        }
    }

}
