/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.preference;

import java.util.List;

import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.ListViewer;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CLabel;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;

import eu.etaxonomy.cdm.api.service.ITermTreeService;
import eu.etaxonomy.cdm.model.term.TermTree;
import eu.etaxonomy.taxeditor.l10n.Messages;
import eu.etaxonomy.taxeditor.model.MessagingUtils;
import eu.etaxonomy.taxeditor.preference.menu.CdmPreferencePage;
import eu.etaxonomy.taxeditor.store.CdmStore;
import eu.etaxonomy.taxeditor.termtree.TermTreeContentProvider;
import eu.etaxonomy.taxeditor.termtree.TermTreeLabelProvider;

/**
 * <p>DefaultFeatureTreePreferenecs class.</p>
 *
 * @author n.hoffmann
 * @created Sep 16, 2010
 */
public class DefaultFeatureTreePreferenecs extends CdmPreferencePage {

	private TermTree defaultFeatureTreeForTextualDescription;
	private TermTree defaultFeatureTreeForStructuredDescription;
	
	private ListViewer viewerText;
	private  ListViewer viewerStructure;

	/** {@inheritDoc} */
	@Override
	protected Control createContents(Composite parent) {
		defaultFeatureTreeForTextualDescription = PreferencesUtil.getDefaultFeatureTreeForTextualDescription();
		defaultFeatureTreeForStructuredDescription = PreferencesUtil.getDefaultFeatureTreeForStructuredDescription();

		Composite composite = new Composite(parent, SWT.NULL);
		composite.setLayout(new GridLayout());
		if(!CdmStore.isActive()) {
            MessagingUtils.noDataSourceWarningDialog(null);
		}else{
		    createStructureTreeSelection(composite);
		    createTextTreeSelection(composite);
		}

		return composite;
	}

	private void createTextTreeSelection(Composite parent){
		final CLabel label = new CLabel(parent, SWT.NONE);
		label.setText(Messages.DefaultFeatureTreePreferenecs_0);
		viewerText = new ListViewer(parent, SWT.SINGLE | SWT.BORDER | SWT.V_SCROLL | SWT.H_SCROLL);
		
		viewerText.getControl().setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));

		viewerText.setContentProvider(new TermTreeContentProvider());
		viewerText.setLabelProvider(new TermTreeLabelProvider());

		viewerText.addSelectionChangedListener(new ISelectionChangedListener() {

			@Override
			public void selectionChanged(SelectionChangedEvent arg0) {
			    setApply(true);
//			    viewer.setSelection(arg0.getSelection());
				IStructuredSelection selection = (IStructuredSelection) viewerText.getSelection();

				defaultFeatureTreeForTextualDescription = (TermTree)selection.getFirstElement();

			}
		});

		List<TermTree> input = CdmStore.getService(ITermTreeService.class).list(TermTree.class, null, null, null, null);

		viewerText.setInput(input);

		if(defaultFeatureTreeForTextualDescription != null){
			IStructuredSelection selection = new StructuredSelection(defaultFeatureTreeForTextualDescription);
			viewerText.setSelection(selection, true);
		}
	}

	private void createStructureTreeSelection(Composite parent){
		final CLabel label = new CLabel(parent, SWT.NONE);
		label.setText(Messages.DefaultFeatureTreePreferenecs_1);

		viewerStructure = new ListViewer(parent, SWT.SINGLE | SWT.BORDER | SWT.V_SCROLL | SWT.H_SCROLL);

		viewerStructure.getControl().setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));

		viewerStructure.setContentProvider(new TermTreeContentProvider());
		viewerStructure.setLabelProvider(new TermTreeLabelProvider());

		viewerStructure.addSelectionChangedListener(new ISelectionChangedListener() {

			@Override
			public void selectionChanged(SelectionChangedEvent arg0) {
			    setApply(true);
//			    viewer.setSelection(arg0.getSelection());
				IStructuredSelection selection = (IStructuredSelection) viewerStructure.getSelection();

				defaultFeatureTreeForStructuredDescription = (TermTree) selection.getFirstElement();

			}
		});

		List<TermTree> input = CdmStore.getService(ITermTreeService.class).list(TermTree.class, null, null, null, null);

		viewerStructure.setInput(input);

		if(defaultFeatureTreeForStructuredDescription != null){
			IStructuredSelection selection = new StructuredSelection(defaultFeatureTreeForStructuredDescription);
			viewerStructure.setSelection(selection, true);
		}
	}

	/** {@inheritDoc} */
	@Override
	public boolean performOk() {
		String defaultFeatureTreeForTextualDescriptionUuid = ""; //$NON-NLS-1$
		String defaultFeatureTreeForStructuredDescriptionUuid = ""; //$NON-NLS-1$

		if(defaultFeatureTreeForTextualDescription != null){
			defaultFeatureTreeForTextualDescriptionUuid = defaultFeatureTreeForTextualDescription.getUuid().toString();
		}
		if(defaultFeatureTreeForStructuredDescription != null){
			defaultFeatureTreeForStructuredDescriptionUuid = defaultFeatureTreeForStructuredDescription.getUuid().toString();
		}

		getPreferenceStore().setValue(PreferencesUtil.createPreferenceString(IPreferenceKeys.FEATURE_TREE_DEFAULT_TEXT), defaultFeatureTreeForTextualDescriptionUuid);
		getPreferenceStore().setValue(PreferencesUtil.createPreferenceString(IPreferenceKeys.FEATURE_TREE_DEFAULT_STRUCTURE), defaultFeatureTreeForStructuredDescriptionUuid);

		return super.performOk();
	}
	@Override
    protected void performDefaults() {
		defaultFeatureTreeForTextualDescription = null;
		defaultFeatureTreeForTextualDescription = null;
		viewerStructure.setSelection(null);
		viewerText.setSelection(null);
        setApply(true);
        super.performDefaults();
    }

}
