/**
* Copyright (C) 2019 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.preference;

import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;

import eu.etaxonomy.cdm.model.metadata.CdmPreference;
import eu.etaxonomy.cdm.model.metadata.PreferencePredicate;
import eu.etaxonomy.taxeditor.l10n.Messages;
import eu.etaxonomy.taxeditor.preference.menu.CdmPreferencePage;

/**
 * @author k.luther
 * @since 02.07.2019
 *
 */
public class DistributionDetailsViewPreference extends CdmPreferencePage implements SelectionListener{

    protected Combo comboShowIdInVocabularyInText;
    private boolean isOverrideShowIdInVoc = true;
    protected Boolean isShowIdInVoc;
    protected boolean allowOverrideShowIdInVoc;
    protected CdmPreference prefShowIdInVoc = null;
    Button allowOverrideButtonShowIdInVoc;
    /**
     * {@inheritDoc}
     */
    @Override
    protected Control createContents(Composite parent) {
        getValues();
        Composite composite = createComposite(parent);
      //show Id in Vocabulary in textfield
        comboShowIdInVocabularyInText = createBooleanCombo(composite, SHOW, DO_NOT_SHOW, PreferencePredicate.NamedAreaWithIdInVoc, Messages.FactualData_showIdInVocabulary, isAdminPreference);
        comboShowIdInVocabularyInText.setToolTipText(Messages.FactualData_showIdInVocabulary_tooltip);
        comboShowIdInVocabularyInText.addSelectionListener(this);
        if (this.isAdminPreference){
            allowOverrideButtonShowIdInVoc = createAllowOverrideButton(composite);
            allowOverrideButtonShowIdInVoc.setSelection(allowOverrideShowIdInVoc);
            allowOverrideButtonShowIdInVoc.addSelectionListener(new SelectionAdapter(){
                @Override
                public void widgetSelected(SelectionEvent e) {
                    allowOverrideShowIdInVoc = !allowOverrideShowIdInVoc;
                    setApply(true);
                    }
            });
        }
        int index = 0;
        if(isShowIdInVoc==null){
            comboShowIdInVocabularyInText.select(0);
        } else{
            for (String itemLabel : comboShowIdInVocabularyInText.getItems()){
                if (itemLabel.startsWith(this.SHOW) && isShowIdInVoc){
                    comboShowIdInVocabularyInText.select(index);
                    break;
                }
                if (itemLabel.startsWith(DO_NOT_SHOW) && !isShowIdInVoc){
                    comboShowIdInVocabularyInText.select(index);
                    break;
                }
                index++;
            }
        }
        if (!isAdminPreference){
            comboShowIdInVocabularyInText.setEnabled(allowOverrideShowIdInVoc);
        }


        return composite;
    }
    @Override
    public void getValues(){
      //get show IdInVocabulary values
        prefShowIdInVoc = PreferencesUtil.getPreferenceFromDB(PreferencePredicate.NamedAreaWithIdInVoc);
        if (prefShowIdInVoc == null){
            prefShowIdInVoc = CdmPreference.NewTaxEditorInstance(PreferencePredicate.NamedAreaWithIdInVoc, PreferencePredicate.NamedAreaWithIdInVoc.getDefaultValue().toString());
        }
        allowOverrideShowIdInVoc = prefShowIdInVoc.isAllowOverride();
        isShowIdInVoc = null;
        String showIdInVoc = PreferencesUtil.getStringValue(PreferencePredicate.NamedAreaWithIdInVoc.getKey(), true);
        if (showIdInVoc != null){
            isShowIdInVoc = Boolean.valueOf(showIdInVoc);
        }

        isOverrideShowIdInVoc = PreferencesUtil.getBooleanValue(PreferencesUtil.prefOverrideKey(PreferencePredicate.NamedAreaWithIdInVoc.getKey()));
        if (!isOverrideShowIdInVoc){
            isShowIdInVoc = null;
        }

    }

    @Override
    protected void performDefaults() {
       isShowIdInVoc = null;
       allowOverrideShowIdInVoc = true;

       comboShowIdInVocabularyInText.select(0);
       if (isAdminPreference){
           allowOverrideButtonShowIdInVoc.setSelection(allowOverrideShowIdInVoc);
        }

        setApply(true);
        super.performDefaults();
    }
    @Override
    public void widgetSelected(SelectionEvent e) {
        if (e.getSource().equals(this.comboShowIdInVocabularyInText)) {
            String text = comboShowIdInVocabularyInText.getText();
            if (text.startsWith(Messages.Preference_Use_Default)){
                isOverrideShowIdInVoc = false;
                isShowIdInVoc = null;

            }else if (text.equals(SHOW)){
                isShowIdInVoc = true;
                isOverrideShowIdInVoc = true;
            }else{
                isShowIdInVoc = false;
                isOverrideShowIdInVoc = true;
            }

        }
        setApply(true);

    }

    @Override
    public void widgetDefaultSelected(SelectionEvent e) {
    }

    @Override
    public boolean performOk() {
        if (isApply()){
            if (isShowIdInVoc != null){
                PreferencesUtil.setStringValue(PreferencePredicate.NamedAreaWithIdInVoc.getKey(), Boolean.toString(isShowIdInVoc));
                PreferencesUtil.setBooleanValue(PreferencesUtil.prefOverrideKey(PreferencePredicate.NamedAreaWithIdInVoc.getKey()), true);

            }else{
                PreferencesUtil.setBooleanValue(PreferencesUtil.prefOverrideKey(PreferencePredicate.NamedAreaWithIdInVoc.getKey()), false);
            }

        }
        return true;
    }

}
