/**
* Copyright (C) 2019 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.preference;

import org.apache.commons.lang.StringUtils;
import org.eclipse.jface.layout.GridLayoutFactory;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;

import eu.etaxonomy.cdm.api.application.ICdmRepository;
import eu.etaxonomy.cdm.ext.geo.CondensedDistributionRecipe;
import eu.etaxonomy.cdm.model.metadata.CdmPreference;
import eu.etaxonomy.cdm.model.metadata.CdmPreference.PrefKey;
import eu.etaxonomy.cdm.model.metadata.PreferencePredicate;
import eu.etaxonomy.cdm.model.metadata.PreferenceSubject;
import eu.etaxonomy.taxeditor.l10n.Messages;
import eu.etaxonomy.taxeditor.preference.menu.CdmPreferencePage;
import eu.etaxonomy.taxeditor.store.CdmStore;

/**
 * @author k.luther
 * @since 02.07.2019
 *
 */
public class DistributionGeneralPreference extends CdmPreferencePage implements SelectionListener {
    protected CdmPreference preferenceCondensedDistribtionRecipe;
    protected boolean isAllowOverrideCondensedDistributionRecipe;
    protected boolean isOverrideCondensedDistributionRecipe;
    Combo comboDistributionStringRecipe;
    Button allowOverrideDistributionStringRecipe;
    protected CondensedDistributionRecipe recipe;


    @Override
    protected Control createContents(Composite parent) {
        getValues();
        Composite composite = createComposite(parent);
        comboDistributionStringRecipe = createCombo(composite, CondensedDistributionRecipe.values(), PreferencePredicate.CondensedDistributionRecipe, "Condensed Distribution String recipe", isAdminPreference);

//      GridData gridData = new GridData(GridData.BEGINNING, GridData.CENTER, true, false);
//      gridData.horizontalIndent = 9;
//      comboDistributionStringRecipe.setLayoutData(gridData);
      GridLayoutFactory.fillDefaults();

      comboDistributionStringRecipe.addSelectionListener(this);
//      comboDistributionStringRecipe.setEnabled((isExportCondensedDistribution != null && isExportCondensedDistribution )|| (isExportCondensedDistribution == null) && Boolean.valueOf(preferenceIsExportCondensedDistribution.getValue()));
      int index = 0;

      for (String itemLabel : comboDistributionStringRecipe.getItems()) {
          if (itemLabel.startsWith(Messages.Preference_Use_Default) && recipe == null){
              comboDistributionStringRecipe.select(index);
              break;
          }
          if (itemLabel.equals(recipe.getLabel())) {
              comboDistributionStringRecipe.select(index);
              break;
          }

          index++;
      }
      if (isAdminPreference){
          allowOverrideDistributionStringRecipe = createAllowOverrideButton(composite);
          allowOverrideDistributionStringRecipe.setSelection(this.preferenceCondensedDistribtionRecipe != null? this.preferenceCondensedDistribtionRecipe.isAllowOverride(): true);
          allowOverrideDistributionStringRecipe.addSelectionListener(new SelectionAdapter() {
              @Override
              public void widgetSelected(SelectionEvent e) {
                  setApply(true);
                  isAllowOverrideCondensedDistributionRecipe = !isAllowOverrideCondensedDistributionRecipe;
              }
          });
       }else{
          comboDistributionStringRecipe.setEnabled(isAllowOverrideCondensedDistributionRecipe);

       }

      return composite;
    }

    @Override
    protected void getValues(){
        isAdminPreference = false;
        ICdmRepository controller;
        controller = CdmStore.getCurrentApplicationConfiguration();
        PrefKey key = CdmPreference.NewKey(PreferenceSubject.NewDatabaseInstance(), PreferencePredicate.CondensedDistributionRecipe);
        preferenceCondensedDistribtionRecipe = controller.getPreferenceService().find(key);

        if (preferenceCondensedDistribtionRecipe != null ){
            isAllowOverrideCondensedDistributionRecipe = preferenceCondensedDistribtionRecipe.isAllowOverride();
        }else{
            preferenceCondensedDistribtionRecipe = CdmPreference.NewDatabaseInstance(PreferencePredicate.CondensedDistributionRecipe,
                    PreferencePredicate.CondensedDistributionRecipe.getDefaultValue() != null? PreferencePredicate.CondensedDistributionRecipe.getDefaultValue().toString(): null);
            isAllowOverrideCondensedDistributionRecipe = true;
        }

        isOverrideCondensedDistributionRecipe = PreferencesUtil.getBooleanValue(
                PreferencesUtil.prefOverrideKey(PreferencePredicate.CondensedDistributionRecipe.getKey()), true) != null? PreferencesUtil.getBooleanValue(
                        PreferencesUtil.prefOverrideKey(PreferencePredicate.CondensedDistributionRecipe.getKey()), true):false;

        if (isAllowOverrideCondensedDistributionRecipe && isOverrideCondensedDistributionRecipe){
           String configString = PreferencesUtil.getStringValue(PreferencePredicate.CondensedDistributionRecipe.getKey(), true);
           if (StringUtils.isNotBlank(configString)){
               recipe = CondensedDistributionRecipe.valueOf(configString);
           }
        }else{
            if (preferenceCondensedDistribtionRecipe.getValue() != null){
                recipe = CondensedDistributionRecipe.valueOf(preferenceCondensedDistribtionRecipe.getValue());
            }
        }
    }

    @Override
    public void widgetSelected(SelectionEvent e) {
        setApply(true);
        if (e.getSource().equals(this.comboDistributionStringRecipe)){
            int selIndex = comboDistributionStringRecipe.getSelectionIndex();
            if (selIndex == 0){
                isOverrideCondensedDistributionRecipe = false;
            }else{
                isOverrideCondensedDistributionRecipe = true;
            }
            recipe = (CondensedDistributionRecipe) comboDistributionStringRecipe.getData(comboDistributionStringRecipe.getText());

        }

    }

    @Override
    public void widgetDefaultSelected(SelectionEvent e) {
        // TODO Auto-generated method stub

    }

    @Override
    public boolean performOk() {
        if (!isApply()){
            return true;
        }

        if(isOverrideCondensedDistributionRecipe){
            PreferencesUtil.setStringValue(PreferencePredicate.CondensedDistributionRecipe.getKey(), recipe.getKey());
            PreferencesUtil.setBooleanValue(PreferencesUtil.prefOverrideKey(PreferencePredicate.CondensedDistributionRecipe.getKey()), isOverrideCondensedDistributionRecipe);
        }else{
           PreferencesUtil.setBooleanValue( PreferencesUtil.prefOverrideKey(PreferencePredicate.CondensedDistributionRecipe.getKey()), isOverrideCondensedDistributionRecipe);
        }
        return true;
    }

}
