/**
* Copyright (C) 2019 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.preference;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.UUID;

import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.commands.NotEnabledException;
import org.eclipse.core.commands.NotHandledException;
import org.eclipse.core.commands.common.NotDefinedException;
import org.eclipse.jface.wizard.WizardDialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CLabel;
import org.eclipse.swt.custom.TableEditor;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.TableItem;
import org.eclipse.ui.handlers.IHandlerService;

import eu.etaxonomy.cdm.model.description.PresenceAbsenceTerm;
import eu.etaxonomy.cdm.model.location.NamedArea;
import eu.etaxonomy.cdm.model.metadata.CdmPreference;
import eu.etaxonomy.cdm.model.metadata.CdmPreference.PrefKey;
import eu.etaxonomy.cdm.model.metadata.PreferencePredicate;
import eu.etaxonomy.cdm.model.metadata.PreferenceResolver;
import eu.etaxonomy.cdm.model.metadata.PreferenceSubject;
import eu.etaxonomy.cdm.model.term.TermType;
import eu.etaxonomy.taxeditor.l10n.Messages;
import eu.etaxonomy.taxeditor.model.ImageResources;
import eu.etaxonomy.taxeditor.preference.wizard.AvailableDistributionPerAreaStatusWizard;
import eu.etaxonomy.taxeditor.store.CdmStore;
import eu.etaxonomy.taxeditor.store.internal.TaxeditorStorePlugin;

/**
 * @author k.luther
 * @since 02.07.2019
 *
 */
public class DistributionStatusPreferencePage extends AbstractVocabularySelectionPreference {

    protected String commandHandlerStringStatus;

    protected List<CdmPreference> prefStatus;
    protected List<CdmPreference> prefStatusToDelete = new ArrayList<>();

    private Button button_openStatusWizard;

    private Composite composite;

    private Map<UUID, NamedArea> namedAreaUUIDMap = new HashMap();
    CdmPreferenceCache cache;


    /**
     * {@inheritDoc}
     */
    @Override
   protected Control createContents(Composite parent) {
        getValues();
        composite = new Composite(parent, SWT.NULL);
        composite.setLayout(new GridLayout());
        GridData textGrid = createTextGridData();
        textGrid.verticalSpan = 2;


        if (isAdminPreference){
            final CLabel labelPerAreaStatus = new CLabel(composite, SWT.NULL);
            labelPerAreaStatus.setText(Messages.DistributionAdminPreferences_PER_AREA_STATUS);
            labelPerAreaStatus.setLayoutData(textGrid);

        }else{
            final CLabel labelPerAreaStatus = new CLabel(composite, SWT.NULL);
            if (pref != null && !pref.isAllowOverride()){
                labelPerAreaStatus.setText(Messages.DistributionAdminPreferences_DEFAULT_AREA_STATUS_NOT_ALLOWED);
            }else{
                labelPerAreaStatus.setText(Messages.DistributionAdminPreferences_DEFAULT_AREA_STATUS);
                labelPerAreaStatus.setLayoutData(textGrid);
            }

        }
        createStatusTable();

        composite.pack ();


        button_openStatusWizard = new Button(composite, SWT.PUSH);

        button_openStatusWizard.addSelectionListener(new SelectionAdapter() {
            @Override
            public void widgetSelected(SelectionEvent se) {
                // you can execute a command directly ... but to get the proper environment it's better to execute it through the IHandlerService:
                IHandlerService handlerService =  TaxeditorStorePlugin.getDefault().getWorkbench().getService(IHandlerService.class);
                try {
                    Object result = handlerService.executeCommand(commandHandlerStringStatus, null);
                    setApply(true);
                    getValues();
                    refresh();
                } catch (ExecutionException e) {
                    // TODO Auto-generated catch block
                    e.printStackTrace();
                } catch (NotDefinedException e) {
                    // TODO Auto-generated catch block
                    e.printStackTrace();
                } catch (NotEnabledException e) {
                    // TODO Auto-generated catch block
                    e.printStackTrace();
                } catch (NotHandledException e) {
                    // TODO Auto-generated catch block
                    e.printStackTrace();
                }
            }

        });
        button_openStatusWizard.setText(Messages.Distribution_status_selection);
        if (!isAdminPreference && pref != null && !pref.isAllowOverride()){
            button_openStatusWizard.setEnabled(false);
        }


        return composite;
    }



    public void createStatusTable() {

        table = new Table(
                composite, SWT.BORDER /*| SWT.CHECK */| SWT.FULL_SELECTION | SWT.SINGLE
        );

        table.setLinesVisible(true);
        table.setHeaderVisible(true);
        GridData tableGrid = createTextGridData();
        table.setLayoutData(tableGrid);
        Iterator<CdmPreference> prefIterator = prefStatus.iterator();
        CdmPreference pref;
        TableColumn column = new TableColumn(table, SWT.NONE);
        column.setText("Predicate");
        column.setWidth(100);
        column = new TableColumn(table, SWT.NONE);
        column.setText("Subject");
        column.setWidth(100);
        column = new TableColumn(table, SWT.NONE);
        column.setText("Values");
        column.setWidth(100);
        column = new TableColumn(table, SWT.NONE);
        column.setText(Messages.Preference_override_allowed);
        column.setWidth(100);
        if (isAdminPreference){
            column = new TableColumn(table, SWT.NONE);
            column.setText(Messages.Delete);
            column.setWidth(100);
            column = new TableColumn(table, SWT.NONE);
            column.setText("Status");
            column.setWidth(100);
        }


        while (prefIterator.hasNext()){
            pref = prefIterator.next();
            createTableRow(pref);
        }

        composite.pack();
    }

    @Override
    public void createTableRow(CdmPreference pref) {
        TableItem item = new TableItem(table, SWT.NONE);
        String uuidString = getUUIDFromSubject(pref);
        String namedAreaString = null;
        if (uuidString != null){
            try{
            UUID uuid = UUID.fromString(uuidString);
            namedAreaString = namedAreaUUIDMap.get(UUID.fromString(uuidString)).getLabel();
            }catch(IllegalArgumentException e){
                // no UUID
            }
        }
        for (int col = 0; col < 6; col ++) {
            if (col == 0){
              item.setText(col, pref.getPredicate());
            }else if (col == 1){
                if (namedAreaString != null){
                    String subjectString = pref.getSubjectString().replace(pref.getSubject().getLastPart(), namedAreaString);
                    subjectString = subjectString.replace("/", "");
                    item.setText(col, subjectString);
                }else{
                    item.setText(col, "Default");
                }


            }else if (col == 2){
               List<UUID> uuidList = null;
               if ((isAdminPreference || !override)&& pref.getValue() == null){
                   item.setText(col, "All status");
               }else{

                   if (!isAdminPreference && override && ((pref != null && pref.isAllowOverride()) || pref == null)){
                       uuidList = PreferencesUtil.createUUIDListFromStringPref(PreferencePredicate.AvailableDistributionStatus.getKey(), true);
                   }else{
                       if (pref != null && pref.getValue() != null){
                           uuidList = PreferencesUtil.createUuidList(pref.getValue());
                       }else{
                           item.setText(col, "All status");
                       }
                   }
                   if (uuidList != null){
                       List<PresenceAbsenceTerm> terms = CdmStore.getTermManager().getTerms(uuidList, PresenceAbsenceTerm.class);
                       String status = "";
                       int index = 1;
                       for (PresenceAbsenceTerm term: terms){
                           status = status.concat(term.getLabel());
                           if (index < terms.size()){
                               status = status.concat(", ");
                           }
                           index ++;

                       }
                       item.setText(col, status);
                   }
               }


             }else if (col == 3 ){
                Button allowOverrideButton = new Button(table, SWT.CHECK);
                allowOverrideButton.setData(pref);
                TableEditor editor = new TableEditor(table);
                editor.horizontalAlignment = SWT.LEFT;
                editor.grabHorizontal = true;
                editor.setEditor(allowOverrideButton, item, col);
                allowOverrideButton.setSelection(pref.isAllowOverride());
                if (isAdminPreference){

                allowOverrideButton.addSelectionListener(new SelectionAdapter(){
                    @Override
                    public void widgetSelected(SelectionEvent e){
                        setApply(true);
                        Button thisButton= (Button) e.widget;
                        CdmPreference preference=(CdmPreference)thisButton.getData();
                        preference = PreferenceResolver.resolve(prefStatus, preference.getKey());
                        if (isAdminPreference){
                            preference.setAllowOverride(!preference.isAllowOverride());
                        }else{

                        }
                    }
                  });
                }else{
                    allowOverrideButton.setEnabled(false);
                }
            } else if (col == 4 && isAdminPreference){
                Button delButton = new Button(table, SWT.PUSH);
                delButton.setImage(ImageResources
                        .getImage(ImageResources.TRASH_ICON));
                delButton.setData(pref);
                delButton.pack();
                delButton.addSelectionListener(new SelectionAdapter(){
                    @Override
                    public void widgetSelected(SelectionEvent e){
                        Button thisButton= (Button) e.widget;
                        CdmPreference preference=(CdmPreference)thisButton.getData();
                        int index = prefStatus.indexOf(preference);
                        table.getItem(index).setText("Deleted");
                        //prefStatus.remove(preference);
                        prefStatusToDelete.add(preference);
                        setApply(true);

                    }
                  });
                TableEditor editor = new TableEditor(table);
                editor.horizontalAlignment = SWT.LEFT;
                editor.grabHorizontal = true;
                editor.setEditor(delButton, item, col);

            }
            else if (col == 5 && isAdminPreference){
                Button listButton = new Button(table, SWT.PUSH);
                listButton.setImage(ImageResources
                        .getImage(ImageResources.EDIT_ICON));
                listButton.setData(pref);
                listButton.pack();
                listButton.addSelectionListener(new SelectionAdapter(){
                    @Override
                    public void widgetSelected(SelectionEvent e){
                        Button thisButton= (Button) e.widget;
                        AvailableDistributionPerAreaStatusWizard wizard = new AvailableDistributionPerAreaStatusWizard(isAdminPreference);
                        CdmPreference actualPref = CdmPreferenceCache.instance().findBestMatching(pref.getKey());
                        wizard.setActualPref(actualPref);
                       String uuidString = getUUIDFromSubject(actualPref);
                        String namedAreaString = null;
                        if (uuidString != null){
                            try{
                            UUID uuid = UUID.fromString(uuidString);
                            NamedArea area = namedAreaUUIDMap.get(UUID.fromString(uuidString));
                            wizard.setNamedArea(area);

                            }catch(IllegalArgumentException ex){
                                // no UUID
                            }
                        }


                        WizardDialog dialog = new WizardDialog(null,
                                wizard);

                        dialog.open();
                        thisButton.setData(CdmPreferenceCache.instance().findBestMatching(pref.getKey()));
                        refresh();


                    }
                  });
                TableEditor editor = new TableEditor(table);
                editor.horizontalAlignment = SWT.LEFT;
                editor.grabHorizontal = true;
                editor.setEditor(listButton, item, col);

            }
        }
    }

    @Override
    public void getValues() {
        isAdminPreference = false;
        type = TermType.NamedArea;
        cache = CdmPreferenceCache.instance();
        cache.getAllTaxEditorDBPreferences();
        PrefKey defaultPref = CdmPreference.NewKey(PreferenceSubject.NewTaxEditorInstance(), PreferencePredicate.AvailableDistributionStatus);
        this.pref = cache.get(defaultPref);
        prefStatus = cache.getAllForPredicate(PreferencePredicate.AvailableDistributionStatus.getKey());
        if (prefStatus == null){
            prefStatus = new ArrayList();
            CdmPreference statusPref = CdmPreference.NewTaxEditorInstance(PreferencePredicate.AvailableDistributionStatus, PreferencePredicate.AvailableDistributionStatus.getDefaultValue() != null ?  PreferencePredicate.AvailableDistributionStatus.getDefaultValue().toString(): null);
            prefStatus.add(statusPref);
        }
        createNamedAreaUUIDMap();

        override = PreferencesUtil.getOverrideForPreference(PreferencePredicate.AvailableDistributionStatus.getKey());

        commandHandlerStringStatus = "eu.etaxonomy.taxeditor.store.open.OpenDistributionStatusWizardHandler";
       // setApply(true);
    }
    @Override
    protected void refresh() {

       if (table != null){
           table.removeAll();
           disposeTableEditors();
           cache.getAllTaxEditorDBPreferences();
           prefStatus = cache.getAllForPredicate(PreferencePredicate.AvailableDistributionStatus.getKey());
           if (prefStatus != null){
               for (CdmPreference pref:prefStatus){
                    if (pref != null){
                        createTableRow(pref);
                    }
                }
           }
           table.redraw();
           table.pack();

        }
       composite.layout(true);
       composite.redraw();

    }

    private void disposeTableEditors() {
        Control [] controls = table.getChildren();

        for (int i = 0; i < controls.length; i++) {
                if(!controls[i].isDisposed()){
                        controls[i].setVisible(false);
                        controls[i].dispose();

                }
        }
}


    /**
     *
     */
    private void createNamedAreaUUIDMap() {
        List<UUID> uuids = new ArrayList();
        for (CdmPreference pref: prefStatus){
            try{
                String lastPart = getUUIDFromSubject(pref);
                if (lastPart != null){
                    UUID uuid = UUID.fromString(lastPart);
                    uuids.add(uuid);
                }

            }catch (IllegalArgumentException e){
                // no uuid in subject
            }

        }
        if (!uuids.isEmpty()){
            List<NamedArea> areas =CdmStore.getTermManager().getTerms(uuids, NamedArea.class);
            for (NamedArea area: areas){
                namedAreaUUIDMap.put(area.getUuid(), area);
            }
        }

    }


    public String getUUIDFromSubject(CdmPreference pref) {
        if (pref == null){
            return null;
        }
        String lastPart = pref.getSubject().getLastPart();
        if (lastPart.contains("NamedArea[")){
            lastPart = lastPart.replace("NamedArea[", "");
            lastPart = lastPart.replace("]", "");
            return lastPart;
        }
        return null;

    }

    @Override
    protected void performDefaults() {

        prefStatus = null;
        getValues();
        refresh();
        setApply(true);

    }

    @Override
    public boolean performOk(){
        if (!isApply()){
            return true;
        }
        CdmPreferenceCache cache = CdmPreferenceCache.instance();
        if (prefStatus == null){
            List<CdmPreference> prefsToDelete =cache.getAllForPredicate(PreferencePredicate.AvailableDistributionStatus.getKey());
            if (prefsToDelete != null){
                PreferencesUtil.removeFromDB(prefsToDelete);
            }
            return true;
        }

        List<CdmPreference> toDelete = new ArrayList<>();
        for (CdmPreference pref:prefStatusToDelete){
            CdmPreference prefToDelete = CdmPreference.NewInstance(pref.getKey(), null);
            toDelete.add(prefToDelete);
        }
        prefStatus.addAll(toDelete);
        PreferencesUtil.setPreferencesToDB(prefStatus);


        return true;
    }






}
