/**
* Copyright (C) 2021 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.preference;

import org.apache.commons.lang.StringUtils;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;

import eu.etaxonomy.cdm.model.metadata.CdmPreference;
import eu.etaxonomy.cdm.model.metadata.EnabledComputedDescription;
import eu.etaxonomy.cdm.model.metadata.PreferencePredicate;
import eu.etaxonomy.taxeditor.l10n.Messages;
import eu.etaxonomy.taxeditor.preference.menu.CdmPreferencePage;

/**
 * @author k.luther
 * @since May 26, 2021
 */
public class EnableComputedDescriptions extends CdmPreferencePage implements SelectionListener{

    protected CdmPreference computedDescriptionsHandling = null;
    protected EnabledComputedDescription selectedHandling = null;
    protected boolean allowOverride;
    protected boolean isOverride = true;

    protected Combo comboSelectHandling;
    Button allowOverrideHandlingButton;


    @Override
    protected Control createContents(Composite parent) {
        getValues();

        Composite composite = createComposite(parent);
        Label description = new Label(composite, SWT.NULL);
        description.setText(Messages.Computed_factualData_handling_description);

        comboSelectHandling = createCombo(composite, EnabledComputedDescription.values(), PreferencePredicate.EnableComputedDescription, Messages.FactualData_EnableComputedFactualData, isAdminPreference);
        comboSelectHandling.addSelectionListener(this);
        if (this.isAdminPreference){
            allowOverrideHandlingButton = createAllowOverrideButton(composite);
            allowOverrideHandlingButton.setSelection(allowOverride);
            allowOverrideHandlingButton.addSelectionListener(new SelectionAdapter(){
                @Override
                public void widgetSelected(SelectionEvent e) {
                    allowOverride = !allowOverride;
                    setApply(true);
                    }
            });
        }
        int index = 0;

        for (String itemLabel : comboSelectHandling.getItems()) {
            if (itemLabel.startsWith(Messages.Preference_Use_Default) && ((!isOverride && !isAdminPreference) || selectedHandling == null)){
                comboSelectHandling.select(index);
                break;
            }
            if (itemLabel.equals(selectedHandling.getLabel())) {
                comboSelectHandling.select(index);
                break;
            }

            index++;
        }

        if (!isAdminPreference){
            comboSelectHandling.setEnabled(allowOverride);
        }

        return composite;
    }


    @Override
    public void getValues(){
        // get modifier values
        computedDescriptionsHandling = PreferencesUtil.getPreferenceFromDB(PreferencePredicate.EnableComputedDescription);
        if (computedDescriptionsHandling == null){
            computedDescriptionsHandling = CdmPreference.NewTaxEditorInstance(PreferencePredicate.EnableComputedDescription, ((EnabledComputedDescription)PreferencePredicate.EnableComputedDescription.getDefaultValue()).getKey());
        }
        allowOverride = computedDescriptionsHandling.isAllowOverride();



        isOverride = PreferencesUtil.getBooleanValue(PreferencesUtil.prefOverrideKey(PreferencePredicate.EnableComputedDescription.getKey()), true);
        if (allowOverride && isOverride){
            String localHandlingPref = PreferencesUtil.getStringValue(PreferencePredicate.EnableComputedDescription.getKey(), true);
            if (StringUtils.isNotBlank(localHandlingPref)){
                selectedHandling = EnabledComputedDescription.byKey(localHandlingPref);
            }

         }else{
             if (computedDescriptionsHandling.getValue() != null){
                 try{
                     selectedHandling = EnabledComputedDescription.byKey(computedDescriptionsHandling.getValue());
                 }catch (IllegalArgumentException e){
                     selectedHandling = (EnabledComputedDescription) PreferencePredicate.EnableComputedDescription.getDefaultValue();
                 }
             }
         }




    }

    @Override
    protected void performDefaults() {
       selectedHandling = null;
       allowOverride = true;

       comboSelectHandling.select(0);
       if (isAdminPreference){
            allowOverrideHandlingButton.setSelection(allowOverride);
        }
        setApply(true);
        super.performDefaults();
    }


    @Override
    public void widgetSelected(SelectionEvent e) {
        setApply(true);
        if (e.getSource().equals(this.comboSelectHandling)){
            int selIndex = comboSelectHandling.getSelectionIndex();
            if (selIndex == 0){
                isOverride = false;
            }else{
                isOverride = true;
            }
            selectedHandling = (EnabledComputedDescription) comboSelectHandling.getData(comboSelectHandling.getText());

        }

    }


    @Override
    public void widgetDefaultSelected(SelectionEvent e) {
        // TODO Auto-generated method stub

    }

    @Override
    public boolean performOk() {
        if (!isApply()){
            return true;
        }

        if(isOverride){
            PreferencesUtil.setStringValue(PreferencePredicate.EnableComputedDescription.getKey(), this.selectedHandling.getKey());
            PreferencesUtil.setBooleanValue(PreferencesUtil.prefOverrideKey(PreferencePredicate.EnableComputedDescription.getKey()), isOverride);
        }else{
            PreferencesUtil.setBooleanValue( PreferencesUtil.prefOverrideKey(PreferencePredicate.EnableComputedDescription.getKey()), isOverride);
        }
        return true;
    }



}
