/**
* Copyright (C) 2019 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.preference;

import org.eclipse.jface.layout.GridLayoutFactory;
import org.eclipse.jface.preference.IntegerFieldEditor;
import org.eclipse.jface.preference.StringFieldEditor;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;

import eu.etaxonomy.cdm.model.metadata.PreferencePredicate;
import eu.etaxonomy.taxeditor.l10n.Messages;
import eu.etaxonomy.taxeditor.preference.menu.CdmPreferencePage;

/**
 * @author k.luther
 * @since 18.01.2019
 *
 */
public class ExternalServicesPreferences extends CdmPreferencePage {

    StringFieldEditor openUrl;
    IntegerFieldEditor maxWidth;
    IntegerFieldEditor maxHeight;

    StringFieldEditor mapService;



    @Override
    protected Control createContents(Composite parent) {
       Composite top = new Composite(parent, SWT.LEFT);
       GridData gridData = new GridData(GridData.FILL_HORIZONTAL);
       GridLayoutFactory.fillDefaults().applyTo(top);
       top.setLayoutData(gridData);
       Label mobot = new Label(top, SWT.NULL);
       mobot.setText("Mobot Open Url Service"); //$NON-NLS-1$
       GridData gridDataLabel = new GridData(GridData.FILL_HORIZONTAL);
       gridDataLabel.horizontalSpan=2;

       mobot.setLayoutData(gridDataLabel);

       Composite mobotComp = new Composite(top, SWT.NULL);

       GridLayoutFactory.fillDefaults().applyTo(mobotComp);

       GridData data = new GridData();
       data.horizontalIndent = 5;
       data.horizontalSpan = 2;
       mobotComp.setLayoutData(data);


       openUrl = new StringFieldEditor(PreferencesUtil.createPreferenceString(PreferencePredicate.MobotOpenUrlServiceAccessPoint.getKey()),
               "Access Point", mobotComp); //$NON-NLS-1$
       openUrl.setPreferenceStore(getPreferenceStore());
       openUrl.load();

       Label reference = new Label(mobotComp, SWT.NULL);
       reference.setText("BHL Reference Image"); //$NON-NLS-1$

       reference.setLayoutData(gridDataLabel);
       maxWidth = new IntegerFieldEditor(
               PreferencesUtil.createPreferenceString(PreferencePredicate.MobotOpenUrlServiceMaxWidth.getKey()),
               Messages.ExternalServicesPreferences_max_width,
               mobotComp);
       maxWidth.setPreferenceStore(getPreferenceStore());
       maxWidth.load();

       maxHeight = new IntegerFieldEditor(
               PreferencesUtil.createPreferenceString(PreferencePredicate.MobotOpenUrlServiceMaxHeight.getKey()),
               Messages.ExternalServicesPreferences_max_height,
               mobotComp);
       maxHeight.setPreferenceStore(getPreferenceStore());
       maxHeight.load();
       Label separator= new Label(top, SWT.HORIZONTAL | SWT.SEPARATOR);
       GridData sepGrid = createTextGridData();
       separator.setLayoutData(sepGrid);



       Label mapServiceLabel = new Label(top, SWT.NULL);
       mapServiceLabel.setText("EDIT Map Service"); //$NON-NLS-1$
       mapServiceLabel.setLayoutData(gridDataLabel);

       mapService = new StringFieldEditor(PreferencesUtil.createPreferenceString(
               PreferencePredicate.EditMapServiceAccessPoint.getKey()),
               "Access Point", top); //$NON-NLS-1$
       mapService.setPreferenceStore(getPreferenceStore());
       mapService.load();

       parent.layout();

       return top;

    }

    @Override
    public void createControl(Composite parent){
        super.createControl(parent);
        if (getApplyButton() != null){
            this.getApplyButton().setEnabled(true);
        }
    }

    @Override
    protected void performDefaults() {
        openUrl.loadDefault();
        maxHeight.loadDefault();
        maxWidth.loadDefault();
        mapService.loadDefault();
        super.performDefaults();
    }


}
