/**
* Copyright (C) 2018 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.preference;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;

import eu.etaxonomy.cdm.model.metadata.CdmPreference;
import eu.etaxonomy.cdm.model.metadata.CdmPreference.PrefKey;
import eu.etaxonomy.cdm.model.metadata.PreferencePredicate;
import eu.etaxonomy.cdm.model.metadata.PreferenceSubject;
import eu.etaxonomy.taxeditor.l10n.Messages;
import eu.etaxonomy.taxeditor.preference.menu.CdmPreferencePage;

/**
 * @author k.luther
 * @since 29.05.2018
 *
 */
public class GeneralPreferencePage extends CdmPreferencePage implements SelectionListener{

    protected Boolean isShowCheckListPerspective;

    protected boolean isAllowOverrideShowMediaView;
    protected boolean isAllowOverrideShowCheckListPerspective;

    protected boolean isOverrideShowMediaView;
    protected boolean isOverrideShowCheckListPerspective;

    protected Combo showChecklistPerspectiveButton;
    protected Button allowOverrideShowChecklistButton;

    @Override
    protected Control createContents(Composite parent) {
        getValues();
        final Composite composite = createComposite(parent);

        GridData gridData = createTextGridData();

        Label separator= new Label(composite, SWT.HORIZONTAL | SWT.SEPARATOR);
        separator.setLayoutData(gridData);
        separator.setVisible(false);



        showChecklistPerspectiveButton = createBooleanCombo(composite, SHOW, DO_NOT_SHOW, PreferencePredicate.ShowChecklistPerspective, "Set Checklist Perspective as Default Perspective",
                isAdminPreference);

        showChecklistPerspectiveButton.addSelectionListener(this);


        if (isAdminPreference){
            allowOverrideShowChecklistButton = createAllowOverrideButton(composite);
            allowOverrideShowChecklistButton.addSelectionListener(new SelectionAdapter(){
                @Override
                public void widgetSelected(SelectionEvent e) {
                    isAllowOverrideShowCheckListPerspective = allowOverrideShowChecklistButton.getSelection();
                    setApply(true);
                }
            });
        }

        setSelections();
        return composite;
    }



    @Override
    public boolean performOk() {
        if (!isApply()){
            return true;
        }
        if (isAllowOverrideShowCheckListPerspective){
            if (isShowCheckListPerspective != null){
                PreferencesUtil.setBooleanValue(PreferencePredicate.ShowChecklistPerspective.getKey(), isShowCheckListPerspective);
            }
            PreferencesUtil.setBooleanValue(PreferencesUtil.prefOverrideKey(PreferencePredicate.ShowChecklistPerspective.getKey()), isOverrideShowCheckListPerspective);


        }
        return true;
    }

    @Override
    protected void getValues(){
        isAdminPreference = false;

        CdmPreferenceCache cache = CdmPreferenceCache.instance();
        isOverrideShowCheckListPerspective = PreferencesUtil.getBooleanValue(PreferencesUtil.prefOverrideKey(PreferencePredicate.ShowChecklistPerspective.getKey()), true) != null? PreferencesUtil.getBooleanValue(PreferencesUtil.prefOverrideKey(PreferencePredicate.ShowChecklistPerspective.getKey()), true): false;
        if (isOverrideShowCheckListPerspective){
            isShowCheckListPerspective = PreferencesUtil.getBooleanValue(PreferencePredicate.ShowChecklistPerspective.getKey(), true);
        }

        PrefKey key = CdmPreference.NewKey(PreferenceSubject.NewTaxEditorInstance(), PreferencePredicate.ShowChecklistPerspective);
        CdmPreference pref = cache.findBestMatching(key);
        if (pref != null){
            isAllowOverrideShowCheckListPerspective = pref.isAllowOverride();
        }else{
            isAllowOverrideShowCheckListPerspective = true;
        }


    }

    @Override
    public void widgetSelected(SelectionEvent e) {
        if (e.getSource().equals(this.showChecklistPerspectiveButton)) {
            String text = showChecklistPerspectiveButton.getText();
            if (text.startsWith(Messages.Preference_Use_Default)){
                isOverrideShowCheckListPerspective = false;
                isShowCheckListPerspective = null;

            }else if (text.equals(SHOW)){
                isShowCheckListPerspective = true;
                isOverrideShowCheckListPerspective = true;
            }else{
                isShowCheckListPerspective = false;
                isOverrideShowCheckListPerspective = true;
            }
        }

        this.setSelections();
        this.setApply(true);


    }



    /**
     * {@inheritDoc}
     */
    @Override
    public void widgetDefaultSelected(SelectionEvent e) {
        // TODO Auto-generated method stub

    }

    @Override
    protected void performDefaults() {
        isOverrideShowCheckListPerspective = false;
        isShowCheckListPerspective = null;

        setSelections();
        setApply(true);
    }

    protected void setSelections(){
        if (isShowCheckListPerspective == null || (!isOverrideShowCheckListPerspective && !isAdminPreference)){
            showChecklistPerspectiveButton.select(0);
        }else{
            if (isShowCheckListPerspective){
                showChecklistPerspectiveButton.select(1);
            }else{
                showChecklistPerspectiveButton.select(2);
            }
        }

        if (isAdminPreference){
            allowOverrideShowChecklistButton.setSelection(isAllowOverrideShowCheckListPerspective);
        }
    }

}
