/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.preference;

import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;

import eu.etaxonomy.cdm.model.name.NomenclaturalCode;
import eu.etaxonomy.taxeditor.model.NomenclaturalCodeHelper;

/**
 * <p>InitNomenclaturalCodePrefDialog class.</p>
 *
 * @author p.ciardelli
 * @created 05.12.2008
 * @version 1.0
 */
public class InitNomenclaturalCodePrefDialog extends Dialog {

	/**
	 * If the user makes no change, or cancels, preferred nom. code will be set to the default
	 */
	private NomenclaturalCode preferredCode = NomenclaturalCodeHelper.getDefaultCode();

	/**
	 * Create the dialog
	 *
	 * @param parentShell a {@link org.eclipse.swt.widgets.Shell} object.
	 */
	public InitNomenclaturalCodePrefDialog(Shell parentShell) {
		super(parentShell);
	}

	/**
	 * {@inheritDoc}
	 *
	 * Create contents of the dialog
	 */
	@Override
	protected Control createDialogArea(Composite parent) {

		Composite container = (Composite) super.createDialogArea(parent);

		// Create label with description of dialog
		final Label chooseANomenclaturalLabel = new Label(container, SWT.WRAP);
		chooseANomenclaturalLabel.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false));
		chooseANomenclaturalLabel.setText("Welcome to the EDIT Taxonomic Editor.\n\nBefore proceeding, please choose a nomenclatural code. This code will be used to create new names and to parse name text.");

		// Create composite to show radio buttons
		final Composite buttonsComposite = new Composite(container, SWT.NONE);
		final GridData gd_buttonsComposite = new GridData();
		gd_buttonsComposite.horizontalIndent = 20;
		buttonsComposite.setLayoutData(gd_buttonsComposite);
		buttonsComposite.setLayout(new GridLayout());

		NomenclaturalCode defaultCode = NomenclaturalCodeHelper.getDefaultCode();

		for (final NomenclaturalCode code : NomenclaturalCodeHelper.getSupportedCodes()) {
			Button button = new Button(buttonsComposite, SWT.RADIO);
			button.setText(NomenclaturalCodeHelper.getDescription(code));
			button.setSelection(defaultCode.equals(code));
			button.addSelectionListener(new SelectionAdapter() {
				@Override
                public void widgetSelected(SelectionEvent e) {
					setPreferredCode(code);
				}
			});
		}

		// More dialog text
		String defaultCodeDescription = NomenclaturalCodeHelper.getDescription(defaultCode);
		final Label chooseANomenclaturalLabel_1 = new Label(container, SWT.WRAP);
		chooseANomenclaturalLabel_1.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false));
		chooseANomenclaturalLabel_1.setText("If you hit \"Cancel\", " + defaultCodeDescription +
												" will be set as your default nomenclatural code.\n\nYou can change the nomenclatural code at any time in the \"Preferences\" menu.");
		//
		return container;
	}

	/**
	 * <p>Setter for the field <code>preferredCode</code>.</p>
	 *
	 * @param code a {@link eu.etaxonomy.cdm.model.name.NomenclaturalCode} object.
	 */
	protected void setPreferredCode(NomenclaturalCode code) {
		this.preferredCode = code;
	}

	/**
	 * {@inheritDoc}
	 *
	 * Return the initial size of the dialog
	 */
	@Override
	protected Point getInitialSize() {
		return new Point(500, 375);
	}

	/** {@inheritDoc} */
	@Override
    protected void configureShell(Shell newShell) {
		super.configureShell(newShell);
		newShell.setText("Choose a nomenclatural code");
	}

	/**
	 * <p>close</p>
	 *
	 * @return a boolean.
	 */
	@Override
    public boolean close() {

		// Save preferred nomenclatural code before closing
		PreferencesUtil.setPreferredNomenclaturalCode(PreferencesUtil.getPreferenceKey(preferredCode), true);

		return super.close();
	}


}
