/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.preference;

import java.io.IOException;
import java.util.Locale;

import org.apache.commons.lang3.StringUtils;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CCombo;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.ui.PlatformUI;

import eu.etaxonomy.taxeditor.l10n.Messages;
import eu.etaxonomy.taxeditor.preference.menu.CdmPreferencePage;

/**
 * @author n.hoffmann
 * @created Dec 3, 2010
 */
public class LanguageEditorPreferencePage extends CdmPreferencePage {

    private CCombo combo;
    PreferencesUtil preferencesUtil = new PreferencesUtil();
    private boolean isSelectionChanged = false;
    private int initalSelectionIndex;

	@Override
	protected Control createContents(Composite parent) {
		Composite container = new Composite(parent, SWT.NULL);
		final GridLayout gridLayout = new GridLayout();
		container.setLayout(gridLayout);
		createEditorDefaultLanguage(container);
		return container;
	}

	/**
     * @param container
     */
    private void createEditorDefaultLanguage(Composite container) {
        // TODO Auto-generated method stub
        final Label description = new Label(container, SWT.NONE);
        description.setText(Messages.LanguageEditorPreferencePage_RestartRequired);

        final Label label = new Label(container, SWT.NONE);
        label.setText(Messages.LanguageEditorPreferencePage_ChooseDefaultLanguage);

        GridData oneLine = new GridData();
        oneLine.grabExcessHorizontalSpace = true;
        oneLine.horizontalAlignment = GridData.FILL;

        combo = new CCombo(container, SWT.NONE);
        combo.setLayoutData(oneLine);

        combo.add(Language.GERMAN.getLabel(), 0);
        combo.add(Language.ENGLISH.getLabel(), 1);
        restoreSavedSelection();

        combo.addSelectionListener(new SelectionListener() {

            @Override
            public void widgetSelected(SelectionEvent e) {
                setApply(true);
//                try {
//                    preferencesUtil.writePropertyToConfigFile(combo.getSelectionIndex());
                    if(initalSelectionIndex != combo.getSelectionIndex()) {
                        isSelectionChanged = true;
                    }else{
                        isSelectionChanged = false;
                    }
//                } catch (IOException e1) {
//                    MessagingUtils.messageDialog("Failed to write Config.ini", LanguageEditorPreferencePage.class,
//                            "Language switch failed, because could not write to Folder. No writing permissions!", null);
//                }
            }

            @Override
            public void widgetDefaultSelected(SelectionEvent e) {
                // TODO Auto-generated method stub

            }
        });
    }

    /**
     * TODO: This method is not taking advantages of the enum field yet
     */
    private void restoreSavedSelection() {
        String rememberedValue = PreferencesUtil.getStringValue(IPreferenceKeys.DEFAULT_LANGUAGE_EDITOR);
        if(StringUtils.isNotEmpty(rememberedValue)&& StringUtils.isNotBlank(rememberedValue)){
            if(rememberedValue.equalsIgnoreCase("en")){ //$NON-NLS-1$
                initalSelectionIndex = 1;
                combo.select(1);
            }else if(rememberedValue.equalsIgnoreCase("de")){ //$NON-NLS-1$
                initalSelectionIndex = 0;
                combo.select(0);
            }
        }else{
            Locale locale = Locale.getDefault();
            if(locale.getLanguage().equals(new Locale("de").getLanguage())){
                initalSelectionIndex = 0;
                combo.select(0);
            }else if(locale.getLanguage().equals(new Locale("en").getLanguage())){
                initalSelectionIndex = 1;
                combo.select(1);
            }
        }
    }

    private enum Language{

        GERMAN("Deutsch"), ENGLISH("English"); //$NON-NLS-1$ //$NON-NLS-2$
        private final String label;
        private Language(String label){
            this.label = label;
        }

        /**
         * @return the label
         */
        public String getLabel() {
            return label;
        }
    }

	@Override
	public boolean performOk() {
        try {
            if(isSelectionChanged){
                preferencesUtil.writePropertyToConfigFile(combo.getSelectionIndex());
                boolean result = MessageDialog.openConfirm(getShell(), Messages.LanguageEditorPreferencePage_PleaseRestart, Messages.LanguageEditorPreferencePage_EditorHasToRestart);
                if(result){
                    //Press Ok
                    PlatformUI.getWorkbench().restart();
                }
            }
        } catch (IOException e) {
            e.printStackTrace();
        }
		return super.performOk();
	}
}