/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.preference;

import java.util.List;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;

import eu.etaxonomy.cdm.model.common.Language;
import eu.etaxonomy.taxeditor.l10n.Messages;
import eu.etaxonomy.taxeditor.preference.menu.CdmPreferencePage;
import eu.etaxonomy.taxeditor.store.CdmStore;

/**
 * @author n.hoffmann
 * @created Dec 3, 2010
 * @version 1.0
 */
public class LanguageRepresentationPreferencePage extends CdmPreferencePage{


	private Language globalLanguage;

//	private Composite createComposite(Composite parent){
//		Composite composite = new Composite(parent, SWT.NULL);
//		composite.setLayoutData(new GridData(SWT.FILL, SWT.TOP, true, false, 2, 1));
//		composite.setLayout(new RowLayout(SWT.HORIZONTAL));
//		return composite;
//	}

	@Override
	protected Control createContents(Composite parent) {
		Composite container = new Composite(parent, SWT.NULL);
		final GridLayout gridLayout = new GridLayout();
		container.setLayout(gridLayout);

		createSetDefaultLanguage(container);

//		createSetMultiLanguageTextEditingCapability(container);

		return container;


	}

	private void createSetDefaultLanguage(Composite parent) {
		globalLanguage = PreferencesUtil.getGlobalLanguage();

		if(globalLanguage == null) {
		    return;
		}

		int curentSelectionIndex = 0;

		final Composite composite = createComposite(parent);
		composite.setLayout(new RowLayout(1));

		final Label label = new Label(composite, SWT.NONE);
		label.setText(Messages.LanguageRepresentationPreferencePage_global);

		final Combo combo_globalLanguage = new Combo(composite, SWT.NONE);

		List<Language> preferredLanguages = CdmStore.getTermManager().getPreferredTerms(Language.class);

		for(int i = 0; i < preferredLanguages.size(); i++){
			Language language = preferredLanguages.get(i);
			combo_globalLanguage.add(language.getLabel(), i);
			if(language.equals(globalLanguage)){
				curentSelectionIndex = i;
			}
		}

		combo_globalLanguage.select(curentSelectionIndex);

		combo_globalLanguage.addSelectionListener(new SelectionAdapter() {
			/* (non-Javadoc)
			 * @see org.eclipse.swt.events.SelectionAdapter#widgetSelected(org.eclipse.swt.events.SelectionEvent)
			 */
			@Override
			public void widgetSelected(SelectionEvent e) {

				int selectionIndex = combo_globalLanguage.getSelectionIndex();
				globalLanguage = CdmStore.getCurrentApplicationConfiguration().getTermService().getLanguageByLabel(combo_globalLanguage.getItem(selectionIndex));
				setApply(true);
			}
		});
	}

//	private void createSetMultiLanguageTextEditingCapability(Composite parent) {
//		isMultilanguageTextEditingCapability = PreferencesUtil.getBooleanValue(IPreferenceKeys.MULTILANGUAGE_TEXT_EDITING_CAPABILITY);
//
//		final Composite composite = createComposite(parent);
//
//		final Button button_toggle = new Button(composite, SWT.CHECK);
//
//
//		button_toggle.setText(Messages.LanguageRepresentationPreferencePage_enable);
//
//		button_toggle.setSelection(isMultilanguageTextEditingCapability);
//
//
//		button_toggle.addSelectionListener(new SelectionAdapter(){
//			/* (non-Javadoc)
//			 * @see org.eclipse.swt.events.SelectionAdapter#widgetSelected(org.eclipse.swt.events.SelectionEvent)
//			 */
//			@Override
//			public void widgetSelected(SelectionEvent e) {
//			    setApply(true);
//				isMultilanguageTextEditingCapability = button_toggle.getSelection();
//			}
//		});
//	}

	@Override
	public boolean performOk() {
	    if (!isApply()){
	        return true;
	    }
	    PreferencesUtil.setGlobalLanguage(globalLanguage);
		return super.performOk();
	}
}
