package eu.etaxonomy.taxeditor.preference;


import org.eclipse.jface.layout.GridLayoutFactory;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CLabel;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;

import eu.etaxonomy.cdm.model.metadata.CdmPreference;
import eu.etaxonomy.cdm.model.metadata.PreferencePredicate;
import eu.etaxonomy.taxeditor.l10n.Messages;
import eu.etaxonomy.taxeditor.preference.menu.CdmPreferencePage;


public class NameDetailsViewConfiguration extends CdmPreferencePage implements SelectionListener{

	boolean isUseLocalSettings;
	protected NameDetailsConfigurator nameDetailsConfig;
    Composite localSettings ;

    protected CdmPreference nameDetailsPref;

    protected boolean isShowTaxon;
    protected boolean isShowNameApprobiation ;
    protected boolean isShowHybrid;
    protected boolean isShowNameRelationship;
    protected boolean isShowNameRelationshipRuleConsidered;
    protected boolean isShowNameRelationshipRuleConsideredCodeEdition;
    protected boolean isShowTypeDesignation;
    protected boolean isShowProtologue;
    protected boolean isShowNomenclaturalStatus;
    protected boolean isShowNomenclaturalStatusRuleConsidered;
    protected boolean isShowNomenclaturalStatusRuleConsideredCodeEdition;
    protected boolean isShowNomenclaturalRef;
    protected boolean isShowAuthorship;
    protected boolean isShowAuthorCache;
    protected boolean isShowEpithets;
    protected boolean isShowRank;
    protected boolean isShowAppendedPhrase;
    protected boolean isShowNameCache;
    protected boolean isShowNomenclaturalCode;
    protected boolean isShowLSID;
    protected boolean isSecEnabled;
    protected boolean isSecDetailsActivated;

    protected boolean isAllowOverride;
    private boolean isOverride = true;
    private Button showSecDetail;
    private Button secEnabled;

  //  protected Combo activateCheckButton;
    protected Button allowLocalPreference;
    protected Button showTaxon;
    protected Button showLsid;
    protected Button showNomenclaturalCode;
    protected Button showNameCache;
    protected Button showAppendedPhrase;
    protected Button showRank;
    protected Button showEpithets;
    protected Button showAuthorCache;
    protected Button showAuthorship;
    protected Button showNomenclaturalRef;
    protected Button showNomenclaturalStatus;
    protected Button showNomenclaturalStatusRuleConsidered;
    protected Button showNomenclaturalStatusRuleConsideredCodeEdition;
    protected Button showProtologue;
    protected Button showTypeDesignation;
    protected Button showNameRelationship;
    protected Button showNameRelationshipRuleConsidered;
    protected Button showNameRelationshipRuleConsideredCodeEdition;
    protected Button showHybrid;
    protected Button showNameApprobiation;

    Composite dbSettings;

    @Override
    protected Control createContents(Composite parent) {
        getValues();
        if (isAllowOverride || isAdminPreference ){

            final CLabel description = new CLabel(parent, SWT.NULL);
            description.setText(Messages.NameDetailsViewConfiguration_description);

            Composite title  = createComposite(parent);
            if (isAdminPreference){
                allowLocalPreference = createAllowOverrideButton(title);

                allowLocalPreference.addSelectionListener(new SelectionAdapter(){
                    @Override
                    public void widgetSelected(SelectionEvent e) {
                        setApply(true);
                        isAllowOverride = allowLocalPreference.getSelection();
                    }
                });

            }
            dbSettings = new Composite(parent, SWT.NONE);
            dbSettings.setLayout(new GridLayout());

            Label separator= new Label(dbSettings, SWT.HORIZONTAL | SWT.SEPARATOR);
            separator.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
            showTaxon = new Button(dbSettings, SWT.CHECK);
            showTaxon.setText(Messages.NameDetailsViewComposite_Show_Taxon);
            showTaxon.addSelectionListener(this);
            isShowTaxon = nameDetailsConfig.isTaxonSectionActivated();

            showTaxon.addSelectionListener(new SelectionAdapter(){
                 @Override
                 public void widgetSelected(SelectionEvent e) {
                     setApply(true);
                     isShowTaxon = showTaxon.getSelection();
                     nameDetailsConfig.setTaxonSectionActivated(isShowTaxon);
                     showSecDetail.setEnabled(isShowTaxon);
                     secEnabled.setEnabled(isShowTaxon);
                  }
             });

            Composite taxonComp = new Composite(dbSettings, SWT.NULL);

            GridLayoutFactory.fillDefaults().applyTo(taxonComp);

            GridData data = new GridData();
            data.horizontalIndent = 10;
            data.horizontalAlignment = SWT.FILL;
            taxonComp.setLayoutData(data);
            GridLayoutFactory.fillDefaults().applyTo(taxonComp);
            secEnabled = new Button(taxonComp, SWT.CHECK);
            secEnabled.setText(Messages.NameDetailsViewComposite_SecEnabled);
            secEnabled.addSelectionListener(this);
            isSecEnabled = nameDetailsConfig.isSecEnabled();

            secEnabled.addSelectionListener(new SelectionAdapter(){
                 @Override
                 public void widgetSelected(SelectionEvent e) {
                     setApply(true);
                     isSecEnabled = secEnabled.getSelection();
                     nameDetailsConfig.setSecEnabled(isSecEnabled);
                  }
             });
            showSecDetail = new Button(taxonComp, SWT.CHECK);
            showSecDetail.setText(Messages.NameDetailsViewComposite_Show_SecDetail);
            showSecDetail.addSelectionListener(this);
            isSecDetailsActivated = nameDetailsConfig.isSecDetailsActivated();

            showSecDetail.addSelectionListener(new SelectionAdapter(){
                 @Override
                 public void widgetSelected(SelectionEvent e) {
                     setApply(true);
                     isSecDetailsActivated = showSecDetail.getSelection();
                     nameDetailsConfig.setSecDetailsActivated(isSecDetailsActivated);
                  }
             });

            showLsid = new Button(dbSettings, SWT.CHECK);
            showLsid.setText(Messages.NameDetailsViewComposite_Show_LSID);
            isShowLSID = nameDetailsConfig.isLSIDActivated();
            showLsid.addSelectionListener(this);
            showLsid.addSelectionListener(new SelectionAdapter(){
                 @Override
                 public void widgetSelected(SelectionEvent e) {
                     setApply(true);
                     isShowLSID = showLsid.getSelection();
                     nameDetailsConfig.setLSIDActivated(isShowLSID);
                  }
             });

            showNomenclaturalCode = new Button(dbSettings, SWT.CHECK);
            showNomenclaturalCode.setText(Messages.NameDetailsViewComposite_Show_NomenclaturalCode);
            isShowNomenclaturalCode = nameDetailsConfig.isNomenclaturalCodeActived();
            showNomenclaturalCode.addSelectionListener(this);
            showNomenclaturalCode.addSelectionListener(new SelectionAdapter(){
                 @Override
                 public void widgetSelected(SelectionEvent e) {
                     setApply(true);
                     isShowNomenclaturalCode = showNomenclaturalCode.getSelection();
                     nameDetailsConfig.setNomenclaturalCodeActived(isShowNomenclaturalCode);
                  }
             });

            showNameCache = new Button(dbSettings, SWT.CHECK);
            showNameCache.setText(Messages.NameDetailsViewComposite_Show_NameCache);
            isShowNameCache = nameDetailsConfig.isNameCacheActivated();
            showNameCache.addSelectionListener(this);
            showNameCache.addSelectionListener(new SelectionAdapter(){
                 @Override
                 public void widgetSelected(SelectionEvent e) {
                     setApply(true);
                     isShowNameCache = showNameCache.getSelection();
                     nameDetailsConfig.setNameCacheActivated(isShowNameCache);
                  }
             });
            showAppendedPhrase = new Button(dbSettings, SWT.CHECK);
            isShowAppendedPhrase= nameDetailsConfig.isAppendedPhraseActivated();
            showAppendedPhrase.setText(Messages.NameDetailsViewComposite_Show_AppendedPhrase);
            showAppendedPhrase.addSelectionListener(this);
            showAppendedPhrase.addSelectionListener(new SelectionAdapter(){
                 @Override
                 public void widgetSelected(SelectionEvent e) {
                     setApply(true);
                     isShowAppendedPhrase = showAppendedPhrase.getSelection();
                     nameDetailsConfig.setAppendedPhraseActivated(isShowAppendedPhrase);
                  }
             });

            showRank = new Button(dbSettings, SWT.CHECK);
            isShowRank = nameDetailsConfig.isRankActivated();
            showRank.setText(Messages.NameDetailsViewComposite_Show_Rank);
            showRank.addSelectionListener(this);
            showRank.addSelectionListener(new SelectionAdapter(){
                 @Override
                 public void widgetSelected(SelectionEvent e) {
                     setApply(true);
                     isShowRank = showRank.getSelection();
                     nameDetailsConfig.setRankActivated(isShowRank);
                  }
             });
            showEpithets = new Button(dbSettings, SWT.CHECK);
            isShowEpithets = nameDetailsConfig.isAtomisedEpithetsActivated();
            showEpithets.setText(Messages.NameDetailsViewComposite_Show_AtomisedEpithets);
            showEpithets.addSelectionListener(this);
            showEpithets.addSelectionListener(new SelectionAdapter(){
                 @Override
                 public void widgetSelected(SelectionEvent e) {
                     setApply(true);
                     isShowEpithets = showEpithets.getSelection();
                     nameDetailsConfig.setAtomisedEpithetsActivated(isShowEpithets);
                  }
             });

            showAuthorCache = new Button(dbSettings, SWT.CHECK);
            isShowAuthorCache = nameDetailsConfig.isAuthorCacheActivated();
            showAuthorCache.setText(Messages.NameDetailsViewComposite_Show_AuthorCache);
            showAuthorCache.addSelectionListener(this);
            showAuthorCache.addSelectionListener(new SelectionAdapter(){
                 @Override
                 public void widgetSelected(SelectionEvent e) {
                     setApply(true);
                     isShowAuthorCache = showAuthorCache.getSelection();
                     nameDetailsConfig.setAuthorCacheActivated(isShowAuthorCache);
                  }
             });

            showAuthorship = new Button(dbSettings, SWT.CHECK);
            isShowAuthorship = nameDetailsConfig.isAuthorshipSectionActivated();
            showAuthorship.setText(Messages.NameDetailsViewComposite_Show_Author);
            showAuthorship.addSelectionListener(this);
            showAuthorship.addSelectionListener(new SelectionAdapter(){
                 @Override
                 public void widgetSelected(SelectionEvent e) {
                     setApply(true);
                     isShowAuthorship = showAuthorship.getSelection();
                     nameDetailsConfig.setAuthorshipSectionActivated(isShowAuthorship);
                  }
             });

            showNomenclaturalRef = new Button(dbSettings, SWT.CHECK);
            isShowNomenclaturalRef = nameDetailsConfig.isNomenclaturalReferenceSectionActivated();
            showNomenclaturalRef.setText(Messages.NameDetailsViewComposite_Show_NomenclaturalReference);
            showNomenclaturalRef.addSelectionListener(this);
            showNomenclaturalRef.addSelectionListener(new SelectionAdapter(){
                 @Override
                 public void widgetSelected(SelectionEvent e) {
                     setApply(true);
                     isShowNomenclaturalRef = showNomenclaturalRef.getSelection();
                     nameDetailsConfig.setNomenclaturalReferenceSectionActivated(isShowNomenclaturalRef);
                  }
             });

            showNomenclaturalStatus = new Button(dbSettings, SWT.CHECK);
            isShowNomenclaturalStatus = nameDetailsConfig.isNomenclaturalStatusSectionActivated();
            showNomenclaturalStatus.setText(Messages.NameDetailsViewComposite_Show_NomenclaturalStatus);
            showNomenclaturalStatus.addSelectionListener(this);
            showNomenclaturalStatus.addSelectionListener(new SelectionAdapter(){
                 @Override
                 public void widgetSelected(SelectionEvent e) {
                     setApply(true);
                     isShowNomenclaturalStatus = showNomenclaturalStatus.getSelection();
                     nameDetailsConfig.setNomenclaturalStatusSectionActivated(isShowNomenclaturalStatus);

                     showNomenclaturalStatusRuleConsidered.setEnabled(isShowNomenclaturalStatus);
                     if (isShowNomenclaturalStatus){
                         showNomenclaturalStatusRuleConsideredCodeEdition.setEnabled(isShowNomenclaturalStatusRuleConsidered);
                     }else{
                         showNomenclaturalStatusRuleConsideredCodeEdition.setEnabled(isShowNomenclaturalStatus);
                     }
                  }
             });


            Composite nomenlaturalStatusComp = new Composite(dbSettings, SWT.NULL);

            GridLayoutFactory.fillDefaults().applyTo(nomenlaturalStatusComp);

            nomenlaturalStatusComp.setLayoutData(data);

            showNomenclaturalStatusRuleConsidered = new Button(nomenlaturalStatusComp, SWT.CHECK);
            isShowNomenclaturalStatusRuleConsidered = nameDetailsConfig.isNomenclaturalStatusRuleConsideredActivated();
            showNomenclaturalStatusRuleConsidered.setText(Messages.NameDetailsViewComposite_Show_NomenclaturalStatus_RuleConsidered);
            showNomenclaturalStatusRuleConsidered.addSelectionListener(this);
            showNomenclaturalStatusRuleConsidered.addSelectionListener(new SelectionAdapter(){
                 @Override
                 public void widgetSelected(SelectionEvent e) {
                     setApply(true);
                     isShowNomenclaturalStatusRuleConsidered = showNomenclaturalStatusRuleConsidered.getSelection();
                     nameDetailsConfig.setNomenclaturalStatusRuleConsideredActivated(isShowNomenclaturalStatusRuleConsidered);
                     showNomenclaturalStatusRuleConsideredCodeEdition.setEnabled(isShowNomenclaturalStatusRuleConsidered);
                 }
             });

            showNomenclaturalStatusRuleConsideredCodeEdition = new Button(nomenlaturalStatusComp, SWT.CHECK);
            isShowNomenclaturalStatusRuleConsideredCodeEdition = nameDetailsConfig.isNomenclaturalStatusRuleConsideredCodeEditionActivated();
            showNomenclaturalStatusRuleConsideredCodeEdition.setText(Messages.NameDetailsViewComposite_Show_NomenclaturalStatus_RuleConsideredCodeEdition);
            showNomenclaturalStatusRuleConsideredCodeEdition.addSelectionListener(this);
            showNomenclaturalStatusRuleConsideredCodeEdition.addSelectionListener(new SelectionAdapter(){
                 @Override
                 public void widgetSelected(SelectionEvent e) {
                     setApply(true);
                     isShowNomenclaturalStatusRuleConsideredCodeEdition = showNomenclaturalStatusRuleConsideredCodeEdition.getSelection();
                     nameDetailsConfig.setNomenclaturalStatusRuleConsideredCodeEditionActivated(isShowNomenclaturalStatusRuleConsideredCodeEdition);
                  }
             });

            showProtologue = new Button(dbSettings, SWT.CHECK);
            isShowProtologue = nameDetailsConfig.isProtologueActivated();
            showProtologue.setText(Messages.NameDetailsViewComposite_Show_Protologue);
            showProtologue.addSelectionListener(this);
            showProtologue.addSelectionListener(new SelectionAdapter(){
                 @Override
                 public void widgetSelected(SelectionEvent e) {
                     setApply(true);
                     isShowProtologue = showProtologue.getSelection();
                     nameDetailsConfig.setProtologueActivated(isShowProtologue);
                  }
             });

            showTypeDesignation = new Button(dbSettings, SWT.CHECK);
            isShowTypeDesignation = nameDetailsConfig.isTypeDesignationSectionActivated();
            showTypeDesignation.setText(Messages.NameDetailsViewComposite_Show_TypeDesignation);
            showTypeDesignation.addSelectionListener(this);
            showTypeDesignation.addSelectionListener(new SelectionAdapter(){
                 @Override
                 public void widgetSelected(SelectionEvent e) {
                     setApply(true);
                     isShowTypeDesignation = showTypeDesignation.getSelection();
                     nameDetailsConfig.setTypeDesignationSectionActivated(isShowTypeDesignation);
                  }
             });


            showNameRelationship = new Button(dbSettings, SWT.CHECK);
            isShowNameRelationship = nameDetailsConfig.isNameRelationsSectionActivated();
            showNameRelationship.setText(Messages.NameDetailsViewComposite_Show_Namerelationships);
            showNameRelationship.addSelectionListener(this);
            showNameRelationship.addSelectionListener(new SelectionAdapter(){
                 @Override
                 public void widgetSelected(SelectionEvent e) {
                     setApply(true);
                     isShowNameRelationship = showNameRelationship.getSelection();
                     nameDetailsConfig.setNameRelationsSectionActivated(isShowNameRelationship);
                     showNameRelationshipRuleConsidered.setEnabled(isShowNameRelationship);
                     if (isShowNameRelationship){
                         showNameRelationshipRuleConsideredCodeEdition.setEnabled(isShowNameRelationshipRuleConsidered);
                     }else {
                         showNameRelationshipRuleConsideredCodeEdition.setEnabled(isShowNameRelationship);
                     }

                  }
             });

            Composite nameRelationComp = new Composite(dbSettings, SWT.NULL);
            GridLayoutFactory.fillDefaults().applyTo(nameRelationComp);
            nameRelationComp.setLayoutData(data);

            showNameRelationshipRuleConsidered = new Button(nameRelationComp, SWT.CHECK);
            isShowNameRelationshipRuleConsidered = nameDetailsConfig.isNameRelationsRuleConsideredActivated();
            showNameRelationshipRuleConsidered.setText(Messages.NameDetailsViewComposite_Show_Namerelationships_RuleConsidered);
            showNameRelationshipRuleConsidered.addSelectionListener(this);
            showNameRelationshipRuleConsidered.addSelectionListener(new SelectionAdapter(){
                 @Override
                 public void widgetSelected(SelectionEvent e) {
                     setApply(true);
                     isShowNameRelationshipRuleConsidered = showNameRelationshipRuleConsidered.getSelection();
                     nameDetailsConfig.setNameRelationsRuleConsideredActivated(isShowNameRelationshipRuleConsidered);
                     showNameRelationshipRuleConsideredCodeEdition.setEnabled(isShowNameRelationshipRuleConsidered);

                  }
             });

            showNameRelationshipRuleConsideredCodeEdition = new Button(nameRelationComp, SWT.CHECK);
            isShowNameRelationshipRuleConsideredCodeEdition = nameDetailsConfig.isNameRelationsRuleConsideredCodeEditionActivated();
            showNameRelationshipRuleConsideredCodeEdition.setText(Messages.NameDetailsViewComposite_Show_Namerelationships_RuleConsideredCodeEdition);
            showNameRelationshipRuleConsideredCodeEdition.addSelectionListener(this);
            showNameRelationshipRuleConsideredCodeEdition.addSelectionListener(new SelectionAdapter(){
                 @Override
                 public void widgetSelected(SelectionEvent e) {
                     setApply(true);
                     isShowNameRelationshipRuleConsideredCodeEdition = showNameRelationshipRuleConsideredCodeEdition.getSelection();
                     nameDetailsConfig.setNameRelationsRuleConsideredCodeEditionActivated(isShowNameRelationshipRuleConsideredCodeEdition);
                  }
             });

            showHybrid = new Button(dbSettings, SWT.CHECK);
            isShowHybrid = nameDetailsConfig.isHybridActivated();
            showHybrid.setText(Messages.NameDetailsViewComposite_Show_Hybrid);
            showHybrid.addSelectionListener(this);
            showHybrid.addSelectionListener(new SelectionAdapter(){
                 @Override
                 public void widgetSelected(SelectionEvent e) {
                     setApply(true);
                     isShowHybrid = showHybrid.getSelection();
                     nameDetailsConfig.setHybridActivated(isShowHybrid);
                  }
             });
            showNameApprobiation = new Button(dbSettings, SWT.CHECK);
            isShowNameApprobiation = nameDetailsConfig.isNameApprobiationActivated();
            showNameApprobiation.setText(Messages.NameDetailsViewComposite_Show_NameApprobiation);
            showNameApprobiation.addSelectionListener(this);
            showNameApprobiation.addSelectionListener(new SelectionAdapter(){
                 @Override
                 public void widgetSelected(SelectionEvent e) {
                     setApply(true);
                     isShowNameApprobiation = showNameApprobiation.getSelection();
                     nameDetailsConfig.setNameApprobiationActivated(isShowNameApprobiation);
                  }
             });
            showSecDetail.setEnabled(isShowTaxon);
            secEnabled.setEnabled(isShowTaxon);

            setButtonSelections();

        }else{
            final CLabel description = new CLabel(parent, SWT.NULL);
            description.setText(Messages.NameDetailsViewConfiguration_description_not_available);
            this.setDefaultButtonActivated(false);
        }




        return parent;
	}

    @Override
    public boolean performOk() {
        if (isApply()){
            if (isOverride){
                PreferencesUtil.setStringValue(PreferencePredicate.NameDetailsView.getKey(), nameDetailsConfig.toString());
                if (nameDetailsPref == null || !nameDetailsConfig.toString().equals(nameDetailsPref.getValue())) {
                    PreferencesUtil.setBooleanValue(PreferencesUtil.prefOverrideKey(PreferencePredicate.NameDetailsView.getKey()), true);
                }
            }else {
                PreferencesUtil.setBooleanValue(PreferencesUtil.prefOverrideKey(PreferencePredicate.NameDetailsView.getKey()), false);
            }

        }
        PreferencesUtil.firePreferencesChanged(this.getClass());
        return true;

    }

    @Override
    public void getValues(){
        if (nameDetailsConfig == null){
            nameDetailsPref = PreferencesUtil.getPreferenceFromDB(PreferencePredicate.NameDetailsView);
            isAdminPreference = false;
            isAllowOverride = true;
            if (nameDetailsPref != null){
                isAllowOverride = nameDetailsPref.isAllowOverride();

            }
            isOverride = PreferencesUtil.getBooleanValue(PreferencesUtil.prefOverrideKey(PreferencePredicate.NameDetailsView.getKey()), true) != null? PreferencesUtil.getBooleanValue(PreferencesUtil.prefOverrideKey(PreferencePredicate.NameDetailsView.getKey()), true): false;
            if (isOverride){
                nameDetailsConfig = PreferencesUtil.getPreferredNameDetailsConfiguration(true);
            }else{
                nameDetailsConfig = PreferencesUtil.getPreferredNameDetailsConfiguration(false);
            }
            if (nameDetailsConfig == null){
                nameDetailsConfig = new NameDetailsConfigurator();
            }
        }
    }

    protected void setButtonSelections(){
        if (isAdminPreference){
            allowLocalPreference.setSelection(isAllowOverride);
        }
        showTaxon.setSelection(isShowTaxon);
        showLsid.setSelection(isShowLSID);
        showNomenclaturalCode.setSelection(isShowNomenclaturalCode);
        showNameCache.setSelection(isShowNameCache);
        showAppendedPhrase.setSelection(isShowAppendedPhrase);
        showRank.setSelection(isShowRank);
        showEpithets.setSelection(isShowEpithets);
        showAuthorCache.setSelection(isShowAuthorCache);
        showAuthorship.setSelection(isShowAuthorship);
        showNomenclaturalRef.setSelection(isShowNomenclaturalRef);
        showNomenclaturalStatus.setSelection(isShowNomenclaturalStatus);

        showNomenclaturalStatusRuleConsidered.setSelection(isShowNomenclaturalStatusRuleConsidered);
        showNomenclaturalStatusRuleConsidered.setEnabled(isShowNomenclaturalStatus);
        showNomenclaturalStatusRuleConsideredCodeEdition.setSelection(isShowNomenclaturalStatusRuleConsideredCodeEdition);
        if (isShowNomenclaturalStatus){
            showNomenclaturalStatusRuleConsideredCodeEdition.setEnabled(isShowNomenclaturalStatusRuleConsidered);
        }else{
            showNomenclaturalStatusRuleConsideredCodeEdition.setEnabled(isShowNomenclaturalStatus);
        }

        showProtologue.setSelection(isShowProtologue);
        showTypeDesignation.setSelection(isShowTypeDesignation);
        showNameRelationship.setSelection(isShowNameRelationship);
        showNameRelationshipRuleConsidered.setSelection(isShowNameRelationshipRuleConsidered);
        showNameRelationshipRuleConsideredCodeEdition.setSelection(isShowNameRelationshipRuleConsideredCodeEdition);
        showNameRelationshipRuleConsidered.setEnabled(isShowNameRelationship);
        if (isShowNameRelationship){
            showNameRelationshipRuleConsideredCodeEdition.setEnabled(isShowNameRelationshipRuleConsidered);
        }else{
            showNameRelationshipRuleConsideredCodeEdition.setEnabled(isShowNameRelationship);
        }
        showHybrid.setSelection(isShowHybrid);
        showNameApprobiation.setSelection(isShowNameApprobiation);
        secEnabled.setSelection(isSecEnabled);
        //secEnabled.setEnabled(isShowTaxon);
        showSecDetail.setSelection(isSecDetailsActivated);
        //showSecDetail.setEnabled(isShowTaxon);

    }

    @Override
    protected void performDefaults() {
        if (isAdminPreference){
            nameDetailsConfig = new NameDetailsConfigurator();
        }else{
            nameDetailsConfig = PreferencesUtil.getPreferredNameDetailsConfiguration(false);
        }

        isShowAppendedPhrase = nameDetailsConfig.isAppendedPhraseActivated();
        isShowEpithets = nameDetailsConfig.isAtomisedEpithetsActivated();
        isShowAuthorship = nameDetailsConfig.isAuthorshipSectionActivated();
        isShowAuthorCache = nameDetailsConfig.isAuthorCacheActivated();
        isShowLSID = nameDetailsConfig.isLSIDActivated();
        isShowNameApprobiation = nameDetailsConfig.isNameApprobiationActivated();
        isShowNameCache = nameDetailsConfig.isNameCacheActivated();
        isShowNameRelationship = nameDetailsConfig.isNameRelationsSectionActivated();
        isShowNameRelationshipRuleConsidered = nameDetailsConfig.isNameRelationsRuleConsideredActivated();
        isShowNameRelationshipRuleConsideredCodeEdition = nameDetailsConfig.isNameRelationsRuleConsideredCodeEditionActivated();
        isShowNomenclaturalCode = nameDetailsConfig.isNomenclaturalCodeActived();
        isShowNomenclaturalStatus = nameDetailsConfig.isNomenclaturalStatusSectionActivated();
        isShowNomenclaturalStatusRuleConsidered = nameDetailsConfig.isNomenclaturalStatusRuleConsideredActivated();
        isShowNomenclaturalStatusRuleConsideredCodeEdition = nameDetailsConfig.isNomenclaturalStatusRuleConsideredCodeEditionActivated();
        isShowNomenclaturalRef = nameDetailsConfig.isNomenclaturalReferenceSectionActivated();
        isShowProtologue = nameDetailsConfig.isProtologueActivated();
        isShowRank = nameDetailsConfig.isRankActivated();
        isShowTaxon = nameDetailsConfig.isTaxonSectionActivated();
        isSecDetailsActivated = nameDetailsConfig.isSecDetailsActivated();
        isSecEnabled = nameDetailsConfig.isSecEnabled();
        isShowTypeDesignation = nameDetailsConfig.isTypeDesignationSectionActivated();
        isShowHybrid = nameDetailsConfig.isHybridActivated();
//        isSimpleDetailsViewActivated = null;
        isAllowOverride = true;
        isOverride = false;
        this.setButtonSelections();
//        PreferencesUtil.recursiveSetEnabled(dbSettings, isSimpleDetailsViewActivated!= null? isSimpleDetailsViewActivated:false);
       // activateCheckButton.select(0);
        setApply(true);
        super.performDefaults();
    }

    @Override
    public void widgetSelected(SelectionEvent e) {
        if (e.getSource() != getApplyButton() || e.getSource() != getDefaultsButton()){
            isOverride = true;
        }

    }

    @Override
    public void widgetDefaultSelected(SelectionEvent e) {
        // TODO Auto-generated method stub

    }



}
