/**
 * Copyright (C) 2007 EDIT
 * European Distributed Institute of Taxonomy
 * http://www.e-taxonomy.eu
 *
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */
package eu.etaxonomy.taxeditor.preference;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;

import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CLabel;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;

import eu.etaxonomy.cdm.model.metadata.CdmPreference;
import eu.etaxonomy.cdm.model.metadata.PreferencePredicate;
import eu.etaxonomy.cdm.model.name.NomenclaturalCode;
import eu.etaxonomy.taxeditor.databaseAdmin.preferencePage.IE4AdminPreferencePage;
import eu.etaxonomy.taxeditor.l10n.Messages;
import eu.etaxonomy.taxeditor.model.NomenclaturalCodeHelper;
import eu.etaxonomy.taxeditor.preference.menu.CdmPreferencePage;
import eu.etaxonomy.taxeditor.store.CdmStore;

/**
 * <p>
 * NomenclaturalCodePreferences class.
 * </p>
 *
 * @author k.luther
 * @created 14.09.2018
 */
public class NomenclaturalCodePreferences extends CdmPreferencePage implements IE4AdminPreferencePage{
    protected CdmPreference pref;
    protected NomenclaturalCode actualCode = null;
    protected Map<NomenclaturalCode, String>  labelAndValues;
    protected Map<NomenclaturalCode, Integer> keyAndIndex;
    protected Combo nomenclaturalCode;
    protected Button allowOverrideButton;
    protected boolean allowOverride;
    private boolean override;


	@Override
	protected Control createContents(Composite parent) {
//	    PreferencesUtil.setPreferredNomenclaturalCode(null, false);

	    getValues();
	    Composite composite = createComposite(parent);
	    if (pref != null && !pref.isAllowOverride() && !isAdminPreference){
	        final CLabel description = new CLabel(composite, SWT.NULL);
	        description.setText(Messages.NomenclaturalCodePreferences_localChangesNotAllowed);
	        description.setLayoutData(createTextGridData());
	        return composite;
	    }
        final CLabel description = new CLabel(composite, SWT.NULL);
        description.setText(Messages.NomenclaturalCodePreferences_description);
        description.setLayoutData(createTextGridData());

        nomenclaturalCode= new Combo(composite, SWT.READ_ONLY);
        nomenclaturalCode.setText(Messages.NomenclaturalCodePreferences_available_codes);



        nomenclaturalCode.setFont(parent.getFont());
        getLabelAndValues();
        keyAndIndex = new HashMap<>();
        Integer index = 0;
        if (isAdminPreference){
            nomenclaturalCode.add(Messages.Preference_Use_Default + " (" +PreferencePredicate.NomenclaturalCode.getDefaultValue().toString()+")", index);
        }else{
            if (pref != null){
                if (pref.getValue() == null){
                    nomenclaturalCode.add(Messages.Preference_Use_Default + " (" +PreferencePredicate.NomenclaturalCode.getDefaultValue().toString()+")", index);
                }else{
                    nomenclaturalCode.add(Messages.Preference_Use_Default + " (" +pref.getValue()+")", index);
                }
            }
        }
        index++;
        for (NomenclaturalCode key: labelAndValues.keySet()) {
            nomenclaturalCode.add(labelAndValues.get(key));
            keyAndIndex.put(key, index);
            index++;
        }
        index = keyAndIndex.get(actualCode);
        if(index!=null){
            nomenclaturalCode.select(index);
        }else{
            nomenclaturalCode.select(0);

        }
        if (isAdminPreference){
            allowOverrideButton = createAllowOverrideButton(composite);
            allowOverrideButton.setText(Messages.GeneralPreference_allowOverride);
            allowOverrideButton.setSelection(allowOverride);
            if (actualCode == null){
//                allowOverrideButton.setEnabled(false);
            }
            allowOverrideButton.addSelectionListener(new SelectionAdapter(){
                @Override
                public void widgetSelected(SelectionEvent e) {
                    allowOverride = allowOverrideButton.getSelection();
                    setApply(true);
                }
            });
        }

        nomenclaturalCode.addSelectionListener(new SelectionAdapter() {
            @Override
            public void widgetSelected(SelectionEvent evt) {
                setApply(true);
                String name = nomenclaturalCode.getText();
                actualCode = null;
                for (Entry<NomenclaturalCode, String> label:labelAndValues.entrySet()){
                    if (label.getValue().equals(name)){
                        actualCode = label.getKey();
                    }
                }



            }
        });

        return composite;

	}

	/**
	 * @return
	 */
	private void getLabelAndValues() {
		List<NomenclaturalCode> supportedCodes = NomenclaturalCodeHelper
				.getSupportedCodes();
		labelAndValues = new HashMap<>();

		for (int i = 0; i < supportedCodes.size(); i++) {
			labelAndValues.put(supportedCodes.get(i), NomenclaturalCodeHelper
                    .getDescription(supportedCodes.get(i))) ;

		}

	}

	/**
	 * {@inheritDoc}
	 *
	 * Initialize the preference page
	 */
    @Override
    public void init() {
        super.init();
        if(!CdmStore.isActive()){
            return;
        }



	}

	@Override
	 public boolean performOk() {
	    if (!isApply()){
            return true;
        }

	    if (actualCode != null && pref.isAllowOverride()){
	        PreferencesUtil.setStringValue(PreferencePredicate.NomenclaturalCode.getKey(), actualCode.getKey());
	        PreferencesUtil.setBooleanValue(PreferencesUtil.createOverridePreferenceString(PreferencePredicate.NomenclaturalCode.getKey()), true);
	    }else{
	        PreferencesUtil.setBooleanValue(PreferencesUtil.createOverridePreferenceString(PreferencePredicate.NomenclaturalCode.getKey()), false);
	    }
        return true;
    }

	@Override
	protected void performDefaults() {
	    actualCode = null;
	    allowOverride = true;

	    nomenclaturalCode.select(0);

        if (isAdminPreference){
            allowOverrideButton.setSelection(allowOverride);
        }
        setApply(true);
	    super.performDefaults();
	}

	@Override
    protected void getValues(){
	    isAdminPreference = false;
	    pref = PreferencesUtil.getPreferenceFromDB(PreferencePredicate.NomenclaturalCode);
        override = PreferencesUtil.getBooleanValue(
                PreferencesUtil.prefOverrideKey(PreferencePredicate.NomenclaturalCode.getKey()), true) != null? PreferencesUtil.getBooleanValue(
                        PreferencesUtil.prefOverrideKey(PreferencePredicate.NomenclaturalCode.getKey()), true):false;
        String nomString = null;
        if (pref != null) {
            if (pref.isAllowOverride() ) {
                if (override){
                    nomString = PreferencesUtil.getStringValue(PreferencePredicate.NomenclaturalCode.getKey(), true);
                }

            } else {
                nomString = pref.getValue() != null? String.valueOf(pref.getValue()): null;
            }

        } else {
            if(!override){
                actualCode = null;
            }else{
                nomString = PreferencesUtil.getStringValue(PreferencePredicate.NomenclaturalCode.getKey(), true);

            }
            pref = CdmPreference.NewTaxEditorInstance(PreferencePredicate.NomenclaturalCode, PreferencePredicate.NomenclaturalCode.getDefaultValue() != null ?  PreferencePredicate.NomenclaturalCode.getDefaultValue().toString(): null);
        }
        if (nomString != null){
            actualCode = NomenclaturalCode.fromString(nomString);
        }




	}

}
