package eu.etaxonomy.taxeditor.preference;

import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CLabel;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;

import eu.etaxonomy.cdm.model.metadata.CdmPreference;
import eu.etaxonomy.cdm.model.metadata.PreferencePredicate;
import eu.etaxonomy.taxeditor.event.EventUtility;
import eu.etaxonomy.taxeditor.event.WorkbenchEventConstants;
import eu.etaxonomy.taxeditor.l10n.Messages;
import eu.etaxonomy.taxeditor.preference.menu.CdmPreferencePage;

public class OrderPreferences extends CdmPreferencePage implements SelectionListener{


        protected NavigatorOrderEnum orderActivated;
        private Combo orderSelectionCombo;
	    boolean isRestoreTaxonNavigator;


	    Button activateRestoreNavigatorButton;
	    protected Button allowOverrideButton;
	    protected CdmPreference orderPref;
	    protected CdmPreference nodePref;
	    boolean override = false;
	    protected boolean allowOverride;

	    protected Combo showTaxonNodeWizardButton;
	    protected Button allowOverrideShowTaxonNodeWizardButton;
	    protected boolean isOverrideShowTaxonNodeWizard;
	    protected boolean isAllowOverrideShowTaxonNodeWizard;
	    protected Boolean isShowTaxonNodeWizard;



	    @Override
	    protected Control createContents(Composite parent) {

	        isRestoreTaxonNavigator = PreferencesUtil.isStoreNavigatorState();
	        getValues();
            Composite secondComposite = createComposite(parent);

            if(!isAdminPreference) {
                activateRestoreNavigatorButton = new Button(secondComposite, SWT.CHECK | SWT.LEFT);
                activateRestoreNavigatorButton.setText(Messages.OrderPreferences_Restore);
                activateRestoreNavigatorButton.setSelection(isRestoreTaxonNavigator);
                activateRestoreNavigatorButton.addSelectionListener(new SelectionAdapter(){
                    @Override
                    public void widgetSelected(SelectionEvent e) {
                        setApply(true);
                        if(isRestoreTaxonNavigator != activateRestoreNavigatorButton.getSelection()) {
                            isRestoreTaxonNavigator = activateRestoreNavigatorButton.getSelection();

                        }
                    }

                });
            }

	        orderSelectionCombo = createCombo(secondComposite, NavigatorOrderEnum.values(), PreferencePredicate.TaxonNodeOrder, Messages.OrderPreferences_Sorting, isAdminPreference);

            int index = 0;

            if (orderActivated == null){
                orderSelectionCombo.select(index);
            }else{
                for (String itemLabel : orderSelectionCombo.getItems()){
                    if (itemLabel.equalsIgnoreCase(orderActivated.getLabel())){
                        orderSelectionCombo.select(index);
                        break;
                    }
                    index++;
                }
            }


            orderSelectionCombo.addSelectionListener(this);
            if (isAdminPreference){
                allowOverrideButton = createAllowOverrideButton(secondComposite);
                allowOverrideButton.setSelection(allowOverride);
                allowOverrideButton.addSelectionListener(new SelectionAdapter(){
                    @Override
                    public void widgetSelected(SelectionEvent e) {
                        allowOverride = !allowOverride;
                        setApply(true);
                        }
                });
            }

            if (orderPref != null && !isAdminPreference){
                if (!orderPref.isAllowOverride()){
                    orderSelectionCombo.setEnabled(false);
                    final CLabel description = new CLabel(secondComposite, SWT.NULL);
                  description.setText(Messages.OrderPreferencePage_NotAllowed);
                  GridData textGrid = createTextGridData();
                  textGrid.verticalSpan = 2;
                  description.setLayoutData(textGrid);
                }
            }

            showTaxonNodeWizardButton = createBooleanCombo(secondComposite, SHOW, DO_NOT_SHOW, PreferencePredicate.ShowTaxonNodeWizard, "Show Taxonnode Wizard",
                    isAdminPreference);
            showTaxonNodeWizardButton.setText(Messages.DatabasePreferncesPage_Show_TaxonNodeWizard);
            showTaxonNodeWizardButton.addSelectionListener(this);


            if (isAdminPreference){
                allowOverrideShowTaxonNodeWizardButton = createAllowOverrideButton(secondComposite);
                allowOverrideShowTaxonNodeWizardButton.addSelectionListener(new SelectionAdapter(){
                    @Override
                    public void widgetSelected(SelectionEvent e) {
                        isAllowOverrideShowTaxonNodeWizard = allowOverrideShowTaxonNodeWizardButton.getSelection();
                        setApply(true);
                    }
                });
                allowOverrideShowTaxonNodeWizardButton.setSelection(isAllowOverrideShowTaxonNodeWizard);
            }else{
                if (!isAllowOverrideShowTaxonNodeWizard){
                    showTaxonNodeWizardButton.setEnabled(false);

                }
            }

            if (isShowTaxonNodeWizard == null || (!isOverrideShowTaxonNodeWizard && !isAdminPreference)){
                showTaxonNodeWizardButton.select(0);
            }else{
                if (isShowTaxonNodeWizard){
                    showTaxonNodeWizardButton.select(1);
                }else{
                    showTaxonNodeWizardButton.select(2);
                }
            }

	        return secondComposite;
	    }


		@Override
		public boolean performOk() {
		    if (isApply()){
    		    PreferencesUtil.setSortNodes(this.orderActivated);
    		    if (orderActivated != null){
    		        PreferencesUtil.setBooleanValue(PreferencesUtil.createOverridePreferenceString(PreferencePredicate.TaxonNodeOrder.getKey()), true);
    		    }else{
    		        PreferencesUtil.setBooleanValue(PreferencesUtil.createOverridePreferenceString(PreferencePredicate.TaxonNodeOrder.getKey()), false);
    		    }
    		    PreferencesUtil.setStoreNavigatorState(isRestoreTaxonNavigator);


    	    	if (isAllowOverrideShowTaxonNodeWizard){
    	    	    if (isShowTaxonNodeWizard != null){
    	                  PreferencesUtil.setBooleanValue(PreferencePredicate.ShowTaxonNodeWizard.getKey(), isShowTaxonNodeWizard);
    	            }
    	            PreferencesUtil.setBooleanValue(PreferencesUtil.prefOverrideKey(PreferencePredicate.ShowTaxonNodeWizard.getKey()), isOverrideShowTaxonNodeWizard);
    	    	}
    	    	EventUtility.postEvent(WorkbenchEventConstants.REFRESH_NAVIGATOR, true);
		    }
		    
			return true;

		}


        /**
         * {@inheritDoc}
         */
        @Override
        public void widgetSelected(SelectionEvent e) {
            setApply(true);
            if (e.getSource().equals(orderSelectionCombo)){
                for (NavigatorOrderEnum item : NavigatorOrderEnum.values()){
                    if (!orderSelectionCombo.getText().startsWith(Messages.Preference_Use_Default)){
                        orderActivated = (NavigatorOrderEnum)orderSelectionCombo.getData(orderSelectionCombo.getText());
                    }else{
                        orderActivated = null;
                    }

                }
                MessageDialog.openInformation(getShell(), null, Messages.OrderPreferencePage_PleaseReopenNavigator);
            }
            if (e.getSource().equals(this.showTaxonNodeWizardButton)) {
                String text = showTaxonNodeWizardButton.getText();
                if (text.startsWith(Messages.Preference_Use_Default)){
                    isOverrideShowTaxonNodeWizard = false;
                    isShowTaxonNodeWizard = null;

                }else if (text.equals(SHOW)){
                    isShowTaxonNodeWizard = true;
                    isOverrideShowTaxonNodeWizard = true;
                }else{
                    isShowTaxonNodeWizard = false;
                    isOverrideShowTaxonNodeWizard = true;
                }

            }
        }


        /**
         * {@inheritDoc}
         */
        @Override
        public void widgetDefaultSelected(SelectionEvent e) {
            // TODO Auto-generated method stub

        }

        @Override
        protected void performDefaults() {
            if (!isAdminPreference) {
                activateRestoreNavigatorButton.setSelection(false);
            }

            isRestoreTaxonNavigator = false;
            orderActivated = null;

            orderSelectionCombo.select(0);
            isOverrideShowTaxonNodeWizard = false;
            isShowTaxonNodeWizard = null;
            showTaxonNodeWizardButton.select(0);

            super.performDefaults();
        }

        @Override
        protected void getValues(){
            orderPref = PreferencesUtil.getPreferenceFromDB(PreferencePredicate.TaxonNodeOrder);
            override = PreferencesUtil.getOverrideForPreference(PreferencePredicate.TaxonNodeOrder.getKey());
            allowOverride = orderPref != null? orderPref.isAllowOverride():true;
            if (orderPref == null || allowOverride  ){
                if (override){
                    orderActivated = NavigatorOrderEnum.valueOf(PreferencesUtil.getStringValue(PreferencePredicate.TaxonNodeOrder.getKey()));
                }
            }else{
                orderActivated = orderPref.getValue()!= null? NavigatorOrderEnum.valueOf(orderPref.getValue()):null;
            }

            isOverrideShowTaxonNodeWizard = PreferencesUtil.getBooleanValue(PreferencesUtil.prefOverrideKey(PreferencePredicate.ShowTaxonNodeWizard.getKey()), true) != null? PreferencesUtil.getBooleanValue(PreferencesUtil.prefOverrideKey(PreferencePredicate.ShowTaxonNodeWizard.getKey()), true): false;
            if (isOverrideShowTaxonNodeWizard){
                isShowTaxonNodeWizard = PreferencesUtil.getBooleanValue(PreferencePredicate.ShowTaxonNodeWizard.getKey(), true);
            }


            nodePref = PreferencesUtil.getPreferenceFromDB(PreferencePredicate.ShowTaxonNodeWizard);
            if (nodePref != null){
                isAllowOverrideShowTaxonNodeWizard = nodePref.isAllowOverride();
            }else{
                isAllowOverrideShowTaxonNodeWizard = true;
            }

        }

}
