/**
* Copyright (C) 2019 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.preference;

import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CLabel;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;

import eu.etaxonomy.cdm.model.metadata.CdmPreference;
import eu.etaxonomy.cdm.model.metadata.PreferencePredicate;
import eu.etaxonomy.cdm.model.metadata.PublishEnum;
import eu.etaxonomy.taxeditor.l10n.Messages;
import eu.etaxonomy.taxeditor.preference.menu.CdmPreferencePage;

/**
 * @author k.luther
 * @since 08.01.2019
 *
 */
public class PublishFlagLocalPreference extends CdmPreferencePage {

    protected PublishEnum publishBehaviour;

    protected Combo publishFlagBehaviour;
    protected CdmPreference pref = null;
    Button allowOverrideButton;

    protected boolean allowOverride;
    private boolean override = true;


   @Override
   public void init() {
       super.init();


   }
    /**
     * {@inheritDoc}
     */
    @Override
    protected Control createContents(Composite parent) {
        getValues();
        pref = PreferencesUtil.getPreferenceFromDB(PreferencePredicate.DefaultBehaviourForPublishFlag);
        if (pref == null){
            pref = CdmPreference.NewTaxEditorInstance(PreferencePredicate.DefaultBehaviourForPublishFlag, PreferencePredicate.DefaultBehaviourForPublishFlag.getDefaultValue().toString());
        }
        allowOverride = pref.isAllowOverride();
        Composite composite = createComposite(parent);

        if (!allowOverride && !isAdminPreference){
            final CLabel description = new CLabel(composite, SWT.NULL);
            description.setText(Messages.PublishFlagPreference_description_not_allowed);
            description.setLayoutData(createTextGridData());
            return composite;
        }

        publishFlagBehaviour = createCombo(composite, PublishEnum.values(), PreferencePredicate.DefaultBehaviourForPublishFlag, Messages.PublishFlagPreference_description, isAdminPreference);
        publishFlagBehaviour.addSelectionListener(new SelectionListener() {

            @Override
            public void widgetSelected(SelectionEvent e) {
                setApply(true);
                if (!publishFlagBehaviour.getText().startsWith(Messages.Preference_Use_Default)){
                    publishBehaviour = (PublishEnum)publishFlagBehaviour.getData(publishFlagBehaviour.getText());
                }else{
                    publishBehaviour = null;
                }

            }

            @Override
            public void widgetDefaultSelected(SelectionEvent e) {
                // TODO Auto-generated method stub

            }
        });
        if (this.isAdminPreference){
            allowOverrideButton = createAllowOverrideButton(composite);
            allowOverrideButton.setSelection(allowOverride);
            allowOverrideButton.addSelectionListener(new SelectionAdapter(){
                @Override
                public void widgetSelected(SelectionEvent e) {
                    allowOverride = !allowOverride;
                    setApply(true);
                    }
            });
        }
        int index = 0;

        if(publishBehaviour==null){
            publishFlagBehaviour.select(0);
        }
        else{
            for (String itemLabel : publishFlagBehaviour.getItems()){
                if (itemLabel.startsWith(publishBehaviour.getLabel())){
                    publishFlagBehaviour.select(index);
                    break;
                }
                index++;
            }
        }

        return composite;

    }


    @Override
    public boolean performOk() {
        if (isApply()){
            if (publishBehaviour != null){
                String text = publishFlagBehaviour.getText();
                for (PublishEnum display: PublishEnum.values()){
                    if (text.startsWith(display.getLabel())){
                        text = display.getKey();
                        break;
                    }
                }
                PreferencesUtil.setStringValue(PreferencePredicate.DefaultBehaviourForPublishFlag.getKey(), text);
                if (pref == null || !pref.getValue().equals(text)){
                    PreferencesUtil.setBooleanValue(PreferencesUtil.prefOverrideKey(PreferencePredicate.DefaultBehaviourForPublishFlag.getKey()), true);
                }
            }else{
                PreferencesUtil.setBooleanValue(PreferencesUtil.prefOverrideKey(PreferencePredicate.DefaultBehaviourForPublishFlag.getKey()), false);
            }
        }
        return true;
    }

    @Override
    public void getValues(){
        publishBehaviour = null;
        try{
            String publishString = PreferencesUtil.getStringValue(PreferencePredicate.DefaultBehaviourForPublishFlag.getKey(), true);

            if (publishString != null){
                publishBehaviour = PublishEnum.valueOf(publishString);
            }
        }catch (IllegalArgumentException e){
            publishBehaviour = PublishEnum.InheritFromParent;
        }

        override = PreferencesUtil.getBooleanValue(PreferencesUtil.prefOverrideKey(PreferencePredicate.DefaultBehaviourForPublishFlag.getKey()));
        if (!override){
            publishBehaviour = null;
        }
    }

    @Override
    protected void performDefaults() {
        publishBehaviour = null;
        allowOverride = true;
        int index = 0;

        for (String itemLabel : publishFlagBehaviour.getItems()){
            if (itemLabel.startsWith(Messages.Preference_Use_Default)){
                publishFlagBehaviour.select(index);
                break;
            }
            index++;
        }
        if (isAdminPreference){
            allowOverrideButton.setSelection(allowOverride);
        }
        setApply(true);
        super.performDefaults();
    }



}
