/**
* Copyright (C) 2020 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.preference;

import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;

import eu.etaxonomy.cdm.model.metadata.CdmPreference;
import eu.etaxonomy.cdm.model.metadata.CdmPreference.PrefKey;
import eu.etaxonomy.cdm.model.metadata.PreferencePredicate;
import eu.etaxonomy.cdm.model.metadata.PreferenceSubject;
import eu.etaxonomy.cdm.model.metadata.SecReferenceHandlingEnum;
import eu.etaxonomy.cdm.model.metadata.SecReferenceHandlingSwapEnum;
import eu.etaxonomy.taxeditor.l10n.Messages;
import eu.etaxonomy.taxeditor.preference.menu.CdmPreferencePage;

/**
 * @author k.luther
 * @since Dec 1, 2020
 */
public class SecundumReferenceHandling extends CdmPreferencePage {

    protected SecReferenceHandlingEnum secundumBehaviour;
    protected SecReferenceHandlingSwapEnum secundumSwapBehaviour;


    protected Combo synonymSecundumBehaviour;
    protected CdmPreference pref = null;
    Button allowOverrideButton;

    protected boolean allowOverride;
    private boolean override = true;

    protected Combo synonymSecundumSwapBehaviour;
    protected CdmPreference prefSwap = null;
    Button allowOverrideSwapButton;

    protected boolean allowOverrideSwap;
    private boolean overrideSwap = true;


   @Override
   public void init() {
       super.init();
       isAdminPreference = false;


   }
    /**
     * {@inheritDoc}
     */
    @Override
    protected Control createContents(Composite parent) {
        getValues();

        Composite composite = createComposite(parent);
        synonymSecundumBehaviour = createCombo(composite, SecReferenceHandlingEnum.values(), PreferencePredicate.DefaultBehaviourForSecundum, Messages.SecundumPreference_description, isAdminPreference);

        synonymSecundumBehaviour.addSelectionListener(new SelectionListener() {

            @Override
            public void widgetSelected(SelectionEvent e) {
                setApply(true);
                if (!synonymSecundumBehaviour.getText().startsWith(Messages.Preference_Use_Default)){
                    secundumBehaviour = (SecReferenceHandlingEnum)synonymSecundumBehaviour.getData(synonymSecundumBehaviour.getText());
                }else{
                    secundumBehaviour = null;
                }

            }



            @Override
            public void widgetDefaultSelected(SelectionEvent e) {
                // TODO Auto-generated method stub

                }
            });
            if (this.isAdminPreference){
                allowOverrideButton = createAllowOverrideButton(composite);
                allowOverrideButton.setSelection(allowOverride);
                allowOverrideButton.addSelectionListener(new SelectionAdapter(){
                    @Override
                    public void widgetSelected(SelectionEvent e) {
                        allowOverride = !allowOverride;
                        setApply(true);
                        }
                });
            }
            int index = 0;

            if(secundumBehaviour==null){
                synonymSecundumBehaviour.select(0);
            }
            else{
                for (String itemLabel : synonymSecundumBehaviour.getItems()){
                    if (itemLabel.startsWith(secundumBehaviour.getLabel())){
                        synonymSecundumBehaviour.select(index);
                        break;
                    }
                    index++;
                }
            }
            //swap secundum behaviour
            synonymSecundumSwapBehaviour = createCombo(composite, SecReferenceHandlingSwapEnum.values(), PreferencePredicate.DefaultBehaviourForSecundumWhenSwap, Messages.SecundumPreferenceSwap_description, isAdminPreference);

            synonymSecundumSwapBehaviour.addSelectionListener(new SelectionListener() {

                @Override
                public void widgetSelected(SelectionEvent e) {
                    setApply(true);
                    if (!synonymSecundumSwapBehaviour.getText().startsWith(Messages.Preference_Use_Default)){
                        secundumSwapBehaviour = (SecReferenceHandlingSwapEnum)synonymSecundumSwapBehaviour.getData(synonymSecundumSwapBehaviour.getText());
                    }else{
                        secundumSwapBehaviour = null;
                    }

                }

                @Override
                public void widgetDefaultSelected(SelectionEvent e) {
                    // TODO Auto-generated method stub

                    }
                });
                if (this.isAdminPreference){
                    allowOverrideSwapButton = createAllowOverrideButton(composite);
                    allowOverrideSwapButton.setSelection(allowOverrideSwap);
                    allowOverrideSwapButton.addSelectionListener(new SelectionAdapter(){
                        @Override
                        public void widgetSelected(SelectionEvent e) {
                            allowOverrideSwap = !allowOverrideSwap;
                            setApply(true);
                            }
                    });
                }
                index = 0;

                if(secundumSwapBehaviour==null){
                    synonymSecundumSwapBehaviour.select(0);
                }
                else{
                    for (String itemLabel : synonymSecundumSwapBehaviour.getItems()){
                        if (itemLabel.startsWith(secundumSwapBehaviour.getLabel())){
                            synonymSecundumSwapBehaviour.select(index);
                            break;
                        }
                        index++;
                    }
                }
                if (!allowOverride && !isAdminPreference){
                    synonymSecundumBehaviour.setEnabled(false);
                }
                if (!allowOverrideSwap && !isAdminPreference){
                    synonymSecundumSwapBehaviour.setEnabled(false);
                }
            return composite;

        }


        @Override
        public boolean performOk() {
            if (isApply()){
                if (secundumBehaviour != null){
                    String text = secundumBehaviour.getKey();
                    PreferencesUtil.setStringValue(PreferencePredicate.DefaultBehaviourForSecundum.getKey(), text);
                    if (pref == null || !pref.getValue().equals(text)){
                        PreferencesUtil.setBooleanValue(PreferencesUtil.prefOverrideKey(PreferencePredicate.DefaultBehaviourForSecundum.getKey()), true);
                    }
                }else{
                    PreferencesUtil.setBooleanValue(PreferencesUtil.prefOverrideKey(PreferencePredicate.DefaultBehaviourForSecundum.getKey()), false);
                }

                //for swap operations
                if (secundumSwapBehaviour != null){
                    String text = secundumSwapBehaviour.getKey();
                    PreferencesUtil.setStringValue(PreferencePredicate.DefaultBehaviourForSecundumWhenSwap.getKey(), text);
                    if (prefSwap == null || !prefSwap.getValue().equals(text)){
                        PreferencesUtil.setBooleanValue(PreferencesUtil.prefOverrideKey(PreferencePredicate.DefaultBehaviourForSecundumWhenSwap.getKey()), true);
                    }
                }else{
                    PreferencesUtil.setBooleanValue(PreferencesUtil.prefOverrideKey(PreferencePredicate.DefaultBehaviourForSecundumWhenSwap.getKey()), false);
                }
            }
            return true;
        }

        @Override
        public void getValues(){
            secundumBehaviour = null;
            PrefKey key = CdmPreference.NewKey(PreferenceSubject.NewInstance("/MoveSynOrAccepted/"), PreferencePredicate.DefaultBehaviourForSecundum);
            pref = PreferencesUtil.getPreferenceFromDB(key);
            if (pref == null){
                pref = CdmPreference.NewInstance(key, PreferencePredicate.DefaultBehaviourForSecundum.getDefaultValue().toString());
            }
            allowOverride = pref.isAllowOverride();
            try{
                if (!isAdminPreference && allowOverride){
                    String secundumString = PreferencesUtil.getStringValue(PreferencePredicate.DefaultBehaviourForSecundum.getKey(), true);
                    if (secundumString != null){
                        secundumBehaviour = SecReferenceHandlingEnum.valueOf(secundumString);
                    }
                }else{
                    secundumBehaviour = SecReferenceHandlingEnum.valueOf(pref.getValue());
                }


            }catch (IllegalArgumentException e){
                secundumBehaviour = SecReferenceHandlingEnum.KeepOrWarn;
            }
            String prefString = PreferencesUtil.prefOverrideKey(PreferencePredicate.DefaultBehaviourForSecundum.getKey());
            override = PreferencesUtil.getBooleanValue(PreferencesUtil.prefOverrideKey(PreferencePredicate.DefaultBehaviourForSecundum.getKey()), true);
            if (!override && !isAdminPreference){
                secundumBehaviour = null;
            }

            //for swap operation

            secundumSwapBehaviour = null;
            key = CdmPreference.NewKey(PreferenceSubject.NewInstance("/SwapSynToAccepted/"), PreferencePredicate.DefaultBehaviourForSecundumWhenSwap);

            prefSwap = PreferencesUtil.getPreferenceFromDB(key);
            if (prefSwap == null){
                prefSwap = CdmPreference.NewInstance(key, PreferencePredicate.DefaultBehaviourForSecundumWhenSwap.getDefaultValue().toString());
            }
            allowOverrideSwap = prefSwap.isAllowOverride();
            try{
                if (!isAdminPreference && allowOverrideSwap){
                    String secundumString = PreferencesUtil.getStringValue(PreferencePredicate.DefaultBehaviourForSecundumWhenSwap.getKey(), !isAdminPreference);

                    if (secundumString != null){
                        secundumSwapBehaviour = SecReferenceHandlingSwapEnum.valueOf(secundumString);
                    }
                }else{
                    secundumSwapBehaviour = SecReferenceHandlingSwapEnum.valueOf(prefSwap.getValue());
                }


            }catch (IllegalArgumentException e){
                secundumSwapBehaviour = SecReferenceHandlingSwapEnum.KeepOrWarn;
            }
            prefString = PreferencesUtil.prefOverrideKey(PreferencePredicate.DefaultBehaviourForSecundumWhenSwap.getKey());
            overrideSwap = PreferencesUtil.getBooleanValue(PreferencesUtil.prefOverrideKey(PreferencePredicate.DefaultBehaviourForSecundumWhenSwap.getKey()), true);
            if (!overrideSwap){
                secundumSwapBehaviour = null;
            }
        }

        @Override
        protected void performDefaults() {
            secundumBehaviour = null;
            allowOverride = true;
            int index = 0;

            for (String itemLabel : synonymSecundumBehaviour.getItems()){
                if (itemLabel.startsWith(Messages.Preference_Use_Default)){
                    synonymSecundumBehaviour.select(index);
                    break;
                }
                index++;
            }
            if (isAdminPreference){
                allowOverrideButton.setSelection(allowOverride);
            }

            //for swap operation

            secundumSwapBehaviour = null;
            allowOverrideSwap = true;
            index = 0;

            for (String itemLabel : synonymSecundumSwapBehaviour.getItems()){
                if (itemLabel.startsWith(Messages.Preference_Use_Default)){
                    synonymSecundumSwapBehaviour.select(index);
                    break;
                }
                index++;
            }
            if (isAdminPreference){
                allowOverrideSwapButton.setSelection(allowOverrideSwap);
            }

            setApply(true);
            super.performDefaults();
        }





}
