/**
* Copyright (C) 2019 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.preference;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;

import eu.etaxonomy.cdm.model.metadata.CdmPreference;
import eu.etaxonomy.cdm.model.metadata.CdmPreference.PrefKey;
import eu.etaxonomy.cdm.model.metadata.PreferencePredicate;
import eu.etaxonomy.cdm.model.metadata.PreferenceSubject;
import eu.etaxonomy.taxeditor.l10n.Messages;
import eu.etaxonomy.taxeditor.preference.menu.CdmPreferencePage;

/**
 * @author k.luther
 * @since 18.01.2019
 *
 */
public class SupplementalDataSourcePreferences extends CdmPreferencePage implements SelectionListener{

    protected Boolean isShowIDNamespace;
    protected Boolean isShowIDSource;

    protected boolean isAllowOverrideShowIDNamespace;
    protected boolean isAllowOverrideShowIDSource;

    protected boolean isOverrideShowIDNamespace;
    protected boolean isOverrideShowIDSource;

    protected Combo showIDNamespaceButton;
    protected Button allowOverrideShowIDNamespaceButton;
    protected Combo showIDSourceButton;
    protected Button allowOverrideIDSourceButton;

    final String SHOW = Messages.GeneralPreference_yes;
    final String DO_NOT_SHOW = Messages.GeneralPreference_no;

    @Override
    protected Control createContents(Composite parent) {
        getValues();
        final Composite composite = createComposite(parent);

        GridData gridData = createTextGridData();

        Label separator= new Label(composite, SWT.HORIZONTAL | SWT.SEPARATOR);
        separator.setLayoutData(gridData);
        separator.setVisible(false);

        showIDSourceButton = createBooleanCombo(composite, SHOW, DO_NOT_SHOW, PreferencePredicate.ShowIdInSource, Messages.SupplementalDataSourcePreferences_SHOW_ID,
                isAdminPreference);
        showIDSourceButton.addSelectionListener(this);
        if (isShowIDSource == null || (!isOverrideShowIDSource && !isAdminPreference)){
            showIDSourceButton.select(0);
        }else if (isShowIDSource){
            showIDSourceButton.select(1);
        }else{
            showIDSourceButton.select(2);
        }

        if (isAdminPreference){
            allowOverrideIDSourceButton = createAllowOverrideButton(composite);
            allowOverrideIDSourceButton.setSelection(isAllowOverrideShowIDSource);
            allowOverrideIDSourceButton.addSelectionListener(new SelectionAdapter(){
                @Override
                public void widgetSelected(SelectionEvent e) {
                    isAllowOverrideShowIDSource = allowOverrideIDSourceButton.getSelection();
                    setApply(true);
                }
            });
        }

        showIDNamespaceButton = createBooleanCombo(composite, SHOW, DO_NOT_SHOW, PreferencePredicate.ShowNamespaceInSource, Messages.SupplementalDataSourcePreferences_SHOW_NAMESPACE,
                isAdminPreference);

        showIDNamespaceButton.addSelectionListener(this);

        if (isShowIDNamespace == null || (!isOverrideShowIDNamespace && !isAdminPreference)){
            showIDNamespaceButton.select(0);
        }else if (isShowIDNamespace){
            showIDNamespaceButton.select(1);
        }else{
            showIDNamespaceButton.select(2);
        }
        if (isAdminPreference){
            allowOverrideShowIDNamespaceButton = createAllowOverrideButton(composite);
            allowOverrideShowIDNamespaceButton.setSelection(isAllowOverrideShowIDNamespace);
            allowOverrideShowIDNamespaceButton.addSelectionListener(new SelectionAdapter(){
                @Override
                public void widgetSelected(SelectionEvent e) {
                    isAllowOverrideShowIDNamespace = allowOverrideShowIDNamespaceButton.getSelection();
                    setApply(true);
                }
            });
        }

        if (!isAllowOverrideShowIDNamespace && !isAdminPreference){
            showIDNamespaceButton.setEnabled(false);
        }
        if (!isAllowOverrideShowIDSource && !isAdminPreference){
            showIDSourceButton.setEnabled(false);
        }
        return composite;
    }

    @Override
    public boolean performOk() {
        if (isAllowOverrideShowIDSource){
            PreferencesUtil.setBooleanValue(PreferencesUtil.prefOverrideKey(PreferencePredicate.ShowIdInSource.getKey()), isOverrideShowIDSource);
            if(isShowIDSource != null){
                PreferencesUtil.setBooleanValue(PreferencePredicate.ShowIdInSource.getKey(), isShowIDSource);
            }
        }
        if (isAllowOverrideShowIDNamespace){
            PreferencesUtil.setBooleanValue(PreferencesUtil.prefOverrideKey(PreferencePredicate.ShowNamespaceInSource.getKey()), isOverrideShowIDNamespace);
            if (isShowIDNamespace != null){
                PreferencesUtil.setBooleanValue(PreferencePredicate.ShowNamespaceInSource.getKey(), isShowIDNamespace);
            }
        }
       return true;
   }
    @Override
    protected void getValues(){
        isAdminPreference = false;

        CdmPreferenceCache cache = CdmPreferenceCache.instance();
        isShowIDSource = PreferencesUtil.getBooleanValue(PreferencePredicate.ShowIdInSource.getKey(), true);
        isOverrideShowIDSource = PreferencesUtil.getBooleanValue(PreferencesUtil.prefOverrideKey(PreferencePredicate.ShowIdInSource.getKey()), true) != null?PreferencesUtil.getBooleanValue(PreferencesUtil.prefOverrideKey(PreferencePredicate.ShowIdInSource.getKey()), true): false;
        PrefKey key = CdmPreference.NewKey(PreferenceSubject.NewTaxEditorInstance(), PreferencePredicate.ShowIdInSource);
        CdmPreference pref = cache.findBestMatching(key);
        if (pref != null){
            isAllowOverrideShowIDSource = pref.isAllowOverride();
        }else{
            isAllowOverrideShowIDSource = true;
        }

        isShowIDNamespace = PreferencesUtil.getBooleanValue(PreferencePredicate.ShowNamespaceInSource.getKey(), true);
        isOverrideShowIDNamespace = PreferencesUtil.getBooleanValue(PreferencesUtil.prefOverrideKey(PreferencePredicate.ShowNamespaceInSource.getKey()), true) != null? PreferencesUtil.getBooleanValue(PreferencesUtil.prefOverrideKey(PreferencePredicate.ShowNamespaceInSource.getKey()), true):false;
        key = CdmPreference.NewKey(PreferenceSubject.NewTaxEditorInstance(), PreferencePredicate.ShowNamespaceInSource);
        pref = cache.findBestMatching(key);
        if (pref != null){
            isAllowOverrideShowIDNamespace= pref.isAllowOverride();
        }else{
            isAllowOverrideShowIDNamespace = true;
        }
    }

    @Override
    public void widgetSelected(SelectionEvent e) {
        if (e.getSource().equals(this.showIDNamespaceButton)) {
            String text = showIDNamespaceButton.getText();

            if (text.startsWith(Messages.Preference_Use_Default)){
                isShowIDNamespace = null;
                isOverrideShowIDNamespace = false;
            }else{
                isOverrideShowIDNamespace = true;
                if (text.equals(SHOW)){
                    isShowIDNamespace = true;

                }else{
                    isShowIDNamespace = false;
                }
            }

        }
        if (e.getSource().equals(this.showIDSourceButton)) {
            String text = showIDSourceButton.getText();
            if (text.startsWith(Messages.Preference_Use_Default)){
                isShowIDSource = null;
                isOverrideShowIDSource = false;
            }else{
                isOverrideShowIDSource = true;
                if (text.equals(SHOW)){
                    isShowIDSource = true;
                }else{
                    isShowIDSource = false;
                }
            }

        }

        this.setApply(true);
    }

    @Override
    public void widgetDefaultSelected(SelectionEvent e) {
    }

    @Override
    protected void performDefaults() {
        showIDNamespaceButton.select(0);
        isShowIDNamespace = null;
        isAllowOverrideShowIDNamespace=true;
        isOverrideShowIDNamespace = false;
        isOverrideShowIDSource = false;
        if (isAdminPreference){
            allowOverrideIDSourceButton.setSelection(true);
            allowOverrideShowIDNamespaceButton.setSelection(true);

        }

        showIDSourceButton.select(0);
        isShowIDSource = null;
        isAllowOverrideShowIDSource=true;
        this.setApply(true);
        super.performDefaults();
    }



}
