/**
* Copyright (C) 2015 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.preference.menu;

import org.eclipse.jface.preference.PreferencePage;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CLabel;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;

import eu.etaxonomy.cdm.model.metadata.CdmPreference;
import eu.etaxonomy.cdm.model.metadata.IKeyLabel;
import eu.etaxonomy.cdm.model.metadata.IPreferencePredicate;
import eu.etaxonomy.taxeditor.l10n.Messages;
import eu.etaxonomy.taxeditor.preference.IE4PreferencePage;
import eu.etaxonomy.taxeditor.preference.PreferencesUtil;

/**
 * @author cmathew
 * @date 30 Jul 2015
 */
public abstract class CdmPreferencePage extends PreferencePage implements IE4PreferencePage {

    private boolean isDefaultButtonActivated = true;

    protected boolean isAdminPreference = false;

    private boolean isApply;

    protected final String SHOW = Messages.GeneralPreference_yes;
    protected final String DO_NOT_SHOW = Messages.GeneralPreference_no;

    public CdmPreferencePage() {
        this(null, null);
    }

    public CdmPreferencePage(String title) {
        this(title, null);
    }

    public CdmPreferencePage(String title, ImageDescriptor image) {
        super(title, image);

        init();
    }

    public void init() {
        setPreferenceStore(PreferencesUtil.getPreferenceStore());
    }

    @Override
    public void createControl(Composite parent){
        super.createControl(parent);
        if (getApplyButton() != null){
            this.getApplyButton().setEnabled(false);
        }
        if (getDefaultsButton() != null){
            this.getDefaultsButton().setEnabled(isDefaultButtonActivated);
        }
    }

    @Override
    public boolean performOk() {
        if (isValid()){
            return super.performOk();
        }return false;
    }

    @Override
    public boolean performCancel() {

        return super.performCancel();
    }

    @Override
    public void performApply() {
        setApply(true);
        super.performApply();
        setApply(false);
    }

    protected Composite createComposite(Composite composite) {
        Composite activateComp = new Composite(composite, SWT.NULL);
        GridLayout layout = new GridLayout();
        layout.numColumns = 2;

        activateComp.setLayout(layout);
        GridData gridData = new GridData(GridData.FILL_HORIZONTAL);

        activateComp.setLayoutData(gridData);

        return activateComp;
    }

    protected GridData createTextGridData() {
        GridData sepGrid = new GridData(GridData.FILL_HORIZONTAL);
        sepGrid.horizontalSpan = 2;
        return sepGrid;
    }

    protected void getValues(){
    }

    protected Button createAllowOverrideButton(Composite activateComp) {
        final Button allowOverrideCheckButton = new Button(activateComp, SWT.CHECK);
        String overrideString;
        if (isAdminPreference){
            overrideString = Messages.GeneralPreference_allowOverride;
        }else{
            overrideString = Messages.GeneralPreference_override;
        }
        allowOverrideCheckButton.setText(overrideString);
        GridData grid = new GridData();
        grid.horizontalAlignment = SWT.END;
        allowOverrideCheckButton.setLayoutData(grid);
        return allowOverrideCheckButton;
    }

    @Override
    public boolean isApply() {
        return isApply;
    }

    public void setApply(boolean isApply) {
        this.isApply = isApply;
        if (this.getApplyButton() != null){
            this.getApplyButton().setEnabled(this.isApply);
        }
    }

    public boolean isDefaultButtonActivated() {
        return isDefaultButtonActivated;
    }

    public void setDefaultButtonActivated(boolean isDefaultButtonActivated) {
        this.isDefaultButtonActivated = isDefaultButtonActivated;
    }

    public Combo createBooleanCombo(Composite parent, String textTrue, String textFalse, IPreferencePredicate predicate, String labelText, boolean isAdminPage) {
        final CLabel rankDescription = new CLabel(parent, SWT.NULL);
        rankDescription.setText(labelText);
        GridData textGrid = createTextGridData();
        textGrid.verticalSpan = 2;
        rankDescription.setLayoutData(textGrid);

        Combo booleanCombo = new Combo(parent, SWT.BORDER | SWT.READ_ONLY);

        CdmPreference dbPref = PreferencesUtil.getPreferenceFromDB(predicate);
        boolean defaultValue = true;
        if (dbPref != null && !isAdminPage){
            defaultValue = Boolean.valueOf(dbPref.getValue());
        }else{
            try{
                defaultValue = (Boolean)predicate.getDefaultValue();
            }catch(NullPointerException e){
                defaultValue = false;
            }

        }
        String defaultString;
        if (defaultValue){
            defaultString = " ("+textTrue+")";
        }else{
            defaultString = " ("+textFalse+")";
        }
        booleanCombo.add(Messages.Preference_Use_Default + defaultString);
        booleanCombo.add(textTrue);
        booleanCombo.add(textFalse);
        if (!isAdminPage && dbPref != null){
            booleanCombo.setEnabled(dbPref.isAllowOverride());
        }
        return booleanCombo;
    }

    public Combo createCombo(Composite parent, IKeyLabel[] enumValues, IPreferencePredicate predicate, String labelText, boolean isAdminPage) {
        final CLabel description = new CLabel(parent, SWT.NULL);
        description.setText(labelText);
        GridData textGrid = createTextGridData();
        textGrid.verticalSpan = 2;
        description.setLayoutData(textGrid);

        Combo booleanCombo = new Combo(parent, SWT.BORDER | SWT.READ_ONLY);

        CdmPreference dbPref = PreferencesUtil.getPreferenceFromDB(predicate);
        String defaultValue = " - ";
        if (dbPref != null && !isAdminPage && dbPref.getValue() != null){
            defaultValue = dbPref.getValue();
            if (enumValues != null){
                for (IKeyLabel value: enumValues){
                    if (value.getKey().equals(defaultValue)){
                        defaultValue = value.getLabel();
                    }
                }
            }
        }else {
            try{
                Object defaultObject = predicate.getDefaultValue();
                if (defaultObject != null){
                    if (defaultObject instanceof IKeyLabel){
                        defaultValue = ((IKeyLabel)defaultObject).getLabel();
                    }else{
                        defaultValue = defaultObject.toString();
                    }
                }
            }catch(NullPointerException e){
                defaultValue = "";
            }
        }

        booleanCombo.add(Messages.Preference_Use_Default + " ("+defaultValue+")");
        if (enumValues != null){
            for (IKeyLabel value: enumValues){
                booleanCombo.add(value.getLabel());
                booleanCombo.setData(value.getLabel(), value);
            }
        }

        return booleanCombo;
    }

    public Boolean getBooleanPrefValue(CdmPreference pref) {
        return pref != null? (pref.getValue()!= null?Boolean.valueOf(pref.getValue()): null): null;
    }

    public String getStringPrefValue(CdmPreference pref) {
        return pref != null? (pref.getValue()!= null? pref.getValue(): null): null;
    }

    public boolean getPrefAllowOverride(CdmPreference pref) {
        return pref != null? pref.isAllowOverride(): true;
    }
}
