/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.preference.menu;

import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;

import eu.etaxonomy.cdm.model.metadata.CdmPreference;
import eu.etaxonomy.cdm.model.metadata.PreferencePredicate;
import eu.etaxonomy.cdm.model.term.TermType;
import eu.etaxonomy.taxeditor.l10n.Messages;
import eu.etaxonomy.taxeditor.preference.GeneralTermPreferencePage;
import eu.etaxonomy.taxeditor.preference.PreferencesUtil;

/**
 * @author p.ciardelli
 * @created 24.08.2009
 */
public class LanguageMenuPreferences extends GeneralTermPreferencePage {

	/** Constant <code>ID="eu.etaxonomy.taxeditor.preferences.lang"{trunked}</code> */
	public final static String ID = "eu.etaxonomy.taxeditor.preferences.languages"; //$NON-NLS-1$
    protected Boolean isMultilanguageTextEditingCapability;
    protected CdmPreference multiLangugeActivatedPref =null;
    private boolean overrideActivated;

    private Combo multiLanguageEditing;

    private Button allowOverrideOrderButton;

	public LanguageMenuPreferences() {
	    super();
        setPredicate(PreferencePredicate.AvailableLanguages);
        type = TermType.Language;
	}

	@Override
    protected Control createContents(Composite parent) {
	    createSetMultiLanguageTextEditingCapability(parent);
	    Composite result = (Composite) super.createContents(parent);
	    return result;
	}

	private void createSetMultiLanguageTextEditingCapability(Composite parent) {
        //isMultilanguageTextEditingCapability = PreferencesUtil.getBooleanValue(PreferencePredicate.MultiLanguageTextEditing.getKey());
	    getValues();
        final Composite composite = createComposite(parent);
        GridLayout layout = new GridLayout();
        layout.numColumns = 2;

        composite.setLayout(layout);

        multiLanguageEditing = createBooleanCombo(composite, Messages.GeneralPreference_yes, Messages.GeneralPreference_no, PreferencePredicate.MultiLanguageTextEditing, "Allow multi language editing", isAdminPreference);
        int index = 0;
        for (String itemLabel : multiLanguageEditing.getItems()) {
            if (isMultilanguageTextEditingCapability == null && itemLabel.startsWith(Messages.Preference_Use_Default)) {
                multiLanguageEditing.select(index);
                break;
            }
            if (isMultilanguageTextEditingCapability != null && itemLabel.equals(Messages.GeneralPreference_yes) && isMultilanguageTextEditingCapability){
                multiLanguageEditing.select(index);
                break;
            }
            if (isMultilanguageTextEditingCapability != null && itemLabel.equals(Messages.GeneralPreference_no) && !isMultilanguageTextEditingCapability){
                multiLanguageEditing.select(index);
                break;
            }
            index++;
        }
        if (!isAdminPreference && multiLangugeActivatedPref != null){
            multiLanguageEditing.setEnabled(multiLangugeActivatedPref.isAllowOverride());
        }
        multiLanguageEditing.addSelectionListener(this);

        if (isAdminPreference) {
            allowOverrideOrderButton = createAllowOverrideButton(composite);
            allowOverrideOrderButton.setSelection(multiLangugeActivatedPref != null? multiLangugeActivatedPref.isAllowOverride(): true);
            //allowOverrideOrderAreasButton.setEnabled(prefAreaSort != null);
            allowOverrideOrderButton.addSelectionListener(new SelectionAdapter() {
                @Override
                public void widgetSelected(SelectionEvent e) {
                    setApply(true);
                    if (multiLangugeActivatedPref == null){
                        multiLangugeActivatedPref = CdmPreference.NewTaxEditorInstance(PreferencePredicate.MultiLanguageTextEditing, PreferencePredicate.MultiLanguageTextEditing.getDefaultValue() != null ?  PreferencePredicate.MultiLanguageTextEditing.getDefaultValue().toString(): null);
                    }
                    multiLangugeActivatedPref.setAllowOverride(allowOverrideOrderButton.getSelection());
                }
            });
        }
    }

	 @Override
	 public void getValues() {
	        isAdminPreference = false;
	        setPreferenceStore(PreferencesUtil.getPreferenceStore());
	        multiLangugeActivatedPref = PreferencesUtil.getPreferenceFromDB(PreferencePredicate.MultiLanguageTextEditing);
	        overrideActivated = PreferencesUtil.getBooleanValue(
	                PreferencesUtil.prefOverrideKey(PreferencePredicate.MultiLanguageTextEditing.getKey()), true) != null? PreferencesUtil.getBooleanValue(
	                        PreferencesUtil.prefOverrideKey(PreferencePredicate.MultiLanguageTextEditing.getKey()), true):false;
            if (multiLangugeActivatedPref != null) {
                if (multiLangugeActivatedPref.isAllowOverride() ) {
                    if (overrideActivated){
                        isMultilanguageTextEditingCapability = PreferencesUtil.getBooleanValue(PreferencePredicate.MultiLanguageTextEditing.getKey(), true);
                    }

                } else {
                    isMultilanguageTextEditingCapability = multiLangugeActivatedPref.getValue() != null? Boolean.valueOf(multiLangugeActivatedPref.getValue()): null;
                }

            } else {
                if(!overrideActivated){
                    isMultilanguageTextEditingCapability = null;
                }else{
                    isMultilanguageTextEditingCapability = PreferencesUtil.getBooleanValue(PreferencePredicate.MultiLanguageTextEditing.getKey(), true);
                }
                multiLangugeActivatedPref = CdmPreference.NewTaxEditorInstance(PreferencePredicate.MultiLanguageTextEditing, PreferencePredicate.MultiLanguageTextEditing.getDefaultValue() != null ?  PreferencePredicate.MultiLanguageTextEditing.getDefaultValue().toString(): null);
            }
	 }
	 @Override
	 public void widgetSelected(SelectionEvent e) {
        setApply(true);
        if (e.getSource().equals(multiLanguageEditing)) {
            String text = multiLanguageEditing.getText();
            if(text.startsWith(Messages.Preference_Use_Default)){
                isMultilanguageTextEditingCapability = null;
                treeComposite.setEnabled(multiLangugeActivatedPref == null? true: multiLangugeActivatedPref.getValue() != null?Boolean.valueOf(multiLangugeActivatedPref.getValue().toString()): true);
                if (!isAdminPreference) {
                    overrideActivated = false;
                }
                return;
            }
            overrideActivated = true;

            if (text.equals(Messages.GeneralPreference_yes)){
                isMultilanguageTextEditingCapability = true;
            }else{
                isMultilanguageTextEditingCapability = false;
            }

            if (isMultilanguageTextEditingCapability != null){
                treeComposite.setEnabled(isMultilanguageTextEditingCapability);
            }else{
                treeComposite.setEnabled(multiLangugeActivatedPref == null? true: multiLangugeActivatedPref.getValue() != null?Boolean.valueOf(multiLangugeActivatedPref.getValue().toString()): true);
            }

        }
     }

	@Override
    public boolean performOk() {
        if (!isApply()){
            return true;
        }
        if (isMultilanguageTextEditingCapability!= null){
            PreferencesUtil.setBooleanValue(PreferencePredicate.MultiLanguageTextEditing.getKey(), isMultilanguageTextEditingCapability);
        }
        PreferencesUtil.setBooleanValue(
                PreferencesUtil.prefOverrideKey(PreferencePredicate.MultiLanguageTextEditing.getKey()),
                overrideActivated);

        return super.performOk();
    }
}
