/**
 * Copyright (C) 2007 EDIT
 * European Distributed Institute of Taxonomy
 * http://www.e-taxonomy.eu
 *
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */
package eu.etaxonomy.taxeditor.preference.menu;

import java.util.List;

import org.eclipse.jface.preference.ComboFieldEditor;

import eu.etaxonomy.cdm.api.application.ICdmRepository;
import eu.etaxonomy.cdm.api.service.IPreferenceService;
import eu.etaxonomy.cdm.model.metadata.CdmPreference;
import eu.etaxonomy.cdm.model.metadata.CdmPreference.PrefKey;
import eu.etaxonomy.cdm.model.metadata.PreferencePredicate;
import eu.etaxonomy.cdm.model.metadata.PreferenceSubject;
import eu.etaxonomy.cdm.model.name.NomenclaturalCode;
import eu.etaxonomy.taxeditor.l10n.Messages;
import eu.etaxonomy.taxeditor.model.NomenclaturalCodeHelper;
import eu.etaxonomy.taxeditor.preference.PreferencesUtil;
import eu.etaxonomy.taxeditor.store.CdmStore;

/**
 * <p>
 * NomenclaturalCodePreferences class.
 * </p>
 *
 * @author p.ciardelli
 * @created 16.09.2008
 */
public class NomenclaturalCodePreferences extends FieldEditorPreferencePageE4 {
    CdmPreference pref;

	@Override
	protected void createFieldEditors() {
	    PreferencesUtil.setPreferredNomenclaturalCode(null, false);
	    isApply = true;
	    if (pref.isAllowOverride()) {
	        addField(new ComboFieldEditor(
	                PreferencesUtil.createPreferenceString(PreferencePredicate.NomenclaturalCode.getKey()),
				Messages.NomenclaturalCodePreferences_available_codes, getLabelAndValues(),
				getFieldEditorParent()));


        } else {
            setDescription(Messages.NomenclaturalCodePreferences_localChangesNotAllowed);
        }

	}

	/**
	 * @return
	 */
	private String[][] getLabelAndValues() {
		List<NomenclaturalCode> supportedCodes = NomenclaturalCodeHelper
				.getSupportedCodes();
		String[][] labelAndValues = new String[supportedCodes.size()][2];
		String label;
		labelAndValues[0][0] = Messages.Preference_Use_Default + "(" + pref.getValue() + ")";
        labelAndValues[0][1] = null;
		for (int i = 1; i < supportedCodes.size(); i++) {
		    label = NomenclaturalCodeHelper
                    .getDescription(supportedCodes.get(i));
		    String value = supportedCodes.get(i).toString();
		    String prefValue = pref.getValue();
//		    if (pref.getValue().equals(supportedCodes.get(i).toString())) {
//		         label +=Messages.Preference_Use_Default;
//            }
		    labelAndValues[i][0] = label;
			labelAndValues[i][1] = PreferencesUtil
					.getPreferenceKey(supportedCodes.get(i));
		}
		return labelAndValues;
	}

	/**
	 * {@inheritDoc}
	 *
	 * Initialize the preference page
	 */
    @Override
    public void init() {
        super.init();
        if(!CdmStore.isActive()){
            return;
        }
	      ICdmRepository controller = CdmStore.getCurrentApplicationConfiguration();
          if (controller != null){
              IPreferenceService service = controller.getPreferenceService();
              PrefKey key = CdmPreference.NewKey(PreferenceSubject.NewDatabaseInstance(), PreferencePredicate.NomenclaturalCode);
              pref = service.find(key);
              //getPreferenceStore().setValue(IPreferenceKeys.ALLOW_OVERRIDE_NOMENCLATURAL_CODE_KEY, pref == null? true : pref.isAllowOverride());
          }
          if (pref == null){
              pref = CdmPreference.NewDatabaseInstance(PreferencePredicate.NomenclaturalCode, PreferencePredicate.NomenclaturalCode.getDefaultValue().toString());
          }

	    if (pref.isAllowOverride()) {
	        setDescription(Messages.NomenclaturalCodePreferences_choose);
	    }else{
	        setDescription(Messages.NomenclaturalCodePreferences_localChangesNotAllowed);

	    }
	}

	@Override
	 public boolean performOk() {
	    if (!isApply()){
	        return true;
	    }
		boolean result = super.performOk();
		String value = pref.getValue();

		String test = PreferencesUtil.getStringValue(PreferencePredicate.NomenclaturalCode.getKey(), true);
		if (!pref.getValue().equals(PreferencesUtil.getStringValue(PreferencePredicate.NomenclaturalCode.getKey(), true))) {
            PreferencesUtil.setBooleanValue(PreferencesUtil.createOverridePreferenceString(PreferencePredicate.NomenclaturalCode.getKey()), true);
        }else{
            PreferencesUtil.setBooleanValue(PreferencesUtil.createOverridePreferenceString(PreferencePredicate.NomenclaturalCode.getKey()), false);
        }
		return result;
	}


}
