/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.preference.menu;

import java.util.ArrayList;
import java.util.List;
import java.util.UUID;

import org.apache.commons.lang3.StringUtils;

import eu.etaxonomy.cdm.model.metadata.PreferencePredicate;
import eu.etaxonomy.cdm.model.term.TermType;
import eu.etaxonomy.cdm.persistence.dto.TermDto;
import eu.etaxonomy.taxeditor.preference.GeneralTermPreferencePage;
import eu.etaxonomy.taxeditor.preference.LocalOrDefaultEnum;
import eu.etaxonomy.taxeditor.preference.PreferencesUtil;
import eu.etaxonomy.taxeditor.store.CdmStore;

/**
 * <p>SpecimenTypeDesignationStatusMenuPreferences class.</p>
 *
 * @author n.hoffmann
 * @created 25.06.2009
 * @version 1.0
 */
public class SpecimenTypeDesignationStatusMenuPreferences extends
		GeneralTermPreferencePage {
    /**
    * <p>Constructor for SpecimenTypeDesignationStatusMenuPreferences.</p>
    */
   public SpecimenTypeDesignationStatusMenuPreferences() {
       super();
       setPredicate(PreferencePredicate.SpecimenTypeDesignationStatus);
       type = TermType.SpecimenTypeDesignationStatus;
   }






   @Override
   public boolean performOk() {
       if (!isApply()){
           return true;
       }
       Object[] checkedElements = treeComposite.getViewer().getCheckedElements();
       List<UUID> listUIIDChecked = new ArrayList<>();
       List<TermDto> preferredTerms = new ArrayList<>();
       for (Object o : checkedElements) {
           if(o instanceof TermDto){
               TermDto termDto = (TermDto) o;
               listUIIDChecked.add(termDto.getUuid());
               preferredTerms.add(termDto);
           }
       }
       String saveCheckedElements = StringUtils.join(listUIIDChecked, ";"); //$NON-NLS-1$

       PreferencesUtil.setStringValue(PreferencePredicate.SpecimenTypeDesignationStatus.getKey(), saveCheckedElements);
       String text = this.useLocalOrAdmin.getText();
       if (text.equals(LocalOrDefaultEnum.Database.getLabel()) || text.equals(LocalOrDefaultEnum.Default.getLabel())){
           PreferencesUtil.setBooleanValue(PreferencesUtil.createOverridePreferenceString(PreferencePredicate.SpecimenTypeDesignationStatus.getKey()), false);
       }else if (text.equals(LocalOrDefaultEnum.Local.getLabel())){
           PreferencesUtil.setBooleanValue(PreferencesUtil.createOverridePreferenceString(PreferencePredicate.SpecimenTypeDesignationStatus.getKey()), true);
       }

       CdmStore.getTermManager().setPreferredTermsByType(preferredTerms, TermType.SpecimenTypeDesignationStatus);
       PreferencesUtil.firePreferencesChanged(this.getClass());
       return true;

   }
}
