/**
* Copyright (C) 2018 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.preference.wizard;


import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import org.eclipse.jface.viewers.CheckboxTreeViewer;
import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;

import eu.etaxonomy.cdm.api.service.IVocabularyService;
import eu.etaxonomy.cdm.model.metadata.CdmPreference;
import eu.etaxonomy.cdm.model.term.TermType;
import eu.etaxonomy.cdm.persistence.dto.AbstractTermDto;
import eu.etaxonomy.cdm.persistence.dto.TermVocabularyDto;
import eu.etaxonomy.taxeditor.editor.definedterm.DefinedTermSorter;
import eu.etaxonomy.taxeditor.editor.definedterm.TermDtoContentProvider;
import eu.etaxonomy.taxeditor.editor.definedterm.TermDtoLabelProvider;
import eu.etaxonomy.taxeditor.store.CdmStore;

/**
 * @author k.luther
 * @since 04.06.2018
 *
 */
public abstract class AbstractTermSelectionWizardPage<T extends AbstractTermDto> extends WizardPage {

    protected CheckBoxTreeComposite treeComposite;
    private List<TermVocabularyDto> vocabularies = new ArrayList<>();
    boolean localPref;
    protected CdmPreference pref;
    protected boolean override;

    protected TermType type;

    protected AbstractTermSelectionWizardPage(String pageName, TermType type) {
        super(pageName);
        this.type = type;

    }

    @Override
    public void createControl(Composite parent){
        parent.setLayout(new GridLayout());
        treeComposite = new CheckBoxTreeComposite(parent, new TermDtoContentProvider(), new TermDtoLabelProvider(), SWT.NONE);
        treeComposite.getViewer().setComparator(new DefinedTermSorter());
        rememberCheckedValues(getCheckedValuesFromPreferences());
        treeComposite.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1));
        setControl(treeComposite);
    }

    protected abstract String getCheckedValuesFromPreferences();

    public CheckboxTreeViewer getViewer(){
        return treeComposite.getViewer();
    }

    public List<TermVocabularyDto> getVocabularies() {
        return vocabularies;
    }

    public void addVocabularies(TermVocabularyDto vocabulary) {
        this.vocabularies.add(vocabulary);
    }

    protected void setVocabularies(List<TermVocabularyDto> vocs) {

        vocabularies = vocs;
    }

    public CdmPreference getPreference() {
        return pref;
    }

    protected void rememberCheckedValues(String checkedValues) {
        initialiseVocabularies();

        treeComposite.getViewer().setInput(getVocabularies());
        List<T> termsFromStringValues = null;
        if (checkedValues != null && checkedValues != "") { //$NON-NLS-1$
            String[] listChecked = checkedValues.split(";"); //$NON-NLS-1$
            String[] listCheckedComma = checkedValues.split(","); //$NON-NLS-1$
            List<String> checked = new ArrayList<>();
            if (listChecked != null ){
                checked = Arrays.asList(listChecked);
            }
            if (listCheckedComma != null && checkedValues.contains(",")){ //$NON-NLS-1$
                checked = Arrays.asList(listCheckedComma);
            }
            termsFromStringValues = getTermsFromStringValues(checked);
            if (termsFromStringValues != null){
                treeComposite.setCheckedElements(termsFromStringValues.toArray());
            }

        }
        if (termsFromStringValues == null){
            termsFromStringValues = getTermsFromStringValues(new ArrayList<String>());
            if (termsFromStringValues != null){
                treeComposite.setCheckedElements(termsFromStringValues.toArray());
            }
        }
    }



    protected void initialiseVocabularies() {
        if (getVocabularies() != null) {
            getVocabularies().clear();
        }
        List<TermVocabularyDto> vocs = new ArrayList<>();
        vocs = CdmStore.getService(IVocabularyService.class).findVocabularyDtoByTermType(type);

        setVocabularies(vocs);
    }

    protected abstract List<TermVocabularyDto> getVocabulariesFromPreference();

    /**
     * @param listValue
     * @return
     */
    protected abstract List<T> getTermsFromStringValues(List<String> listValue);

    protected Button createAllowOverrideButton(Composite parent) {
       Button activateCheckButton = new Button(parent, SWT.CHECK);
       if (localPref){
           activateCheckButton.setText("Override");
           activateCheckButton.setSelection(override);
       }else{
           activateCheckButton.setText("Allow Override");
           activateCheckButton.setSelection(override);
       }



        activateCheckButton.addSelectionListener(new SelectionAdapter(){
                @Override
                public void widgetSelected(SelectionEvent e) {
                    pref.setAllowOverride(activateCheckButton.getSelection());
                    override = activateCheckButton.getSelection();

                }
            });
        return activateCheckButton;
    }

}
