/**
* Copyright (C) 2018 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.preference.wizard;

import org.eclipse.jface.viewers.Viewer;
import org.eclipse.jface.viewers.ViewerComparator;

/**
 * @author k.luther
 * @since 04.10.2018
 *
 */
public class AreaViewerComparator extends ViewerComparator {

    /**
     * Returns a negative, zero, or positive number depending on whether
     * the first element is less than, equal to, or greater than
     * the second element.
     * <p>
     * The default implementation of this method is based on
     * comparing the elements' categories as computed by the <code>category</code>
     * framework method. Elements within the same category are further
     * subjected to a case insensitive compare of their label strings, either
     * as computed by the content viewer's label provider, or their
     * <code>toString</code> values in other cases. Subclasses may override.
     * </p>
     *
     * @param viewer the viewer
     * @param e1 the first element
     * @param e2 the second element
     * @return a negative number if the first element is less  than the
     *  second element; the value <code>0</code> if the first element is
     *  equal to the second element; and a positive number if the first
     *  element is greater than the second element
     */
    @Override
    public int compare(Viewer viewer, Object e1, Object e2) {
        int cat1 = category(e1);
        int cat2 = category(e2);

        if (cat1 != cat2) {
            return cat1 - cat2;
        }

        if (e1 instanceof NamedAreaWrapper && e2 instanceof NamedAreaWrapper){
            NamedAreaWrapper wrapper1 = (NamedAreaWrapper)e1;
            NamedAreaWrapper wrapper2 = (NamedAreaWrapper)e2;

            if (wrapper1.isBaseElement == wrapper2.isBaseElement){
               return super.compare(viewer, e1, e2);
            }else if (wrapper1.isBaseElement){
                return -1;
            }else{
                return 1;
            }
        }else{
            return super.compare(viewer, e1, e2);
        }

    }


}
