/**
* Copyright (C) 2018 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.preference.wizard;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.UUID;

import org.apache.commons.lang3.StringUtils;
import org.eclipse.swt.widgets.Composite;

import eu.etaxonomy.cdm.api.service.IVocabularyService;
import eu.etaxonomy.cdm.model.metadata.CdmPreference;
import eu.etaxonomy.cdm.model.term.TermType;
import eu.etaxonomy.cdm.persistence.dto.TermVocabularyDto;
import eu.etaxonomy.taxeditor.editor.definedterm.RootElementsOnlyTreeContentProvider;
import eu.etaxonomy.taxeditor.l10n.Messages;
import eu.etaxonomy.taxeditor.preference.PreferencesUtil;
import eu.etaxonomy.taxeditor.store.CdmStore;


/**
 * @author k.luther
 * @since 04.06.2018
 *
 */
public class AvailableAreaVocabulariesPage extends AbstractTermSelectionWizardPage<TermVocabularyDto>{

    String featureTitle;

    public AvailableAreaVocabulariesPage(String pageName, boolean localPref, CdmPreference pref, String featureTitle) {
        super(pageName, TermType.NamedArea);
        this.localPref = localPref;
        this.featureTitle = featureTitle;
        this.pref = pref;


    }

//    public AvailableAreaVocabulariesPage(String pageName, boolean localPref, CdmPreference pref, String featureTitle) {
//        this(pageName, localPref, pref, featureTitle);
//
//    }

    @Override
    public void createControl(Composite parent) {

        setTitle(String.format(Messages.AvailableAreaVocabulariesPage_PAGE_TITLE, featureTitle));
        setDescription(String.format(Messages.AvailableAreaVocabulariesPage_PAGE_DESCRIPTION, featureTitle));
        if (!localPref){
            createAllowOverrideButton(parent);
        }

        super.createControl(parent);

        treeComposite.getViewer().setContentProvider(new RootElementsOnlyTreeContentProvider());
        treeComposite.setAllowTopLevelSelection(true);
    }



    @Override
    protected String getCheckedValuesFromPreferences() {
        String vocString = null;
        if (localPref){
            vocString = PreferencesUtil.getPreferredVocabulariesForDistributionEditor(localPref);
            if (StringUtils.isBlank(vocString) && pref != null){
                vocString = pref.getValue();
            }
            this.override = PreferencesUtil.getOverrideForPreference(pref.getPredicate());
        }else if (pref != null){
            vocString = pref.getValue();
        }
        return vocString;
    }





    /**
     * {@inheritDoc}
     */
    @Override
    protected List<TermVocabularyDto> getVocabulariesFromPreference() {
        // TODO Auto-generated method stub
        return null;
    }
    @Override
    protected void rememberCheckedValues(String checkedValues) {
        initialiseVocabularies();

        treeComposite.getViewer().setInput(getVocabularies());

        if (checkedValues != null && checkedValues != "") { //$NON-NLS-1$
            String[] listChecked = checkedValues.split(";"); //$NON-NLS-1$
            String[] listCheckedComma = checkedValues.split(","); //$NON-NLS-1$
            List<String> checked = new ArrayList<>();
            if (listChecked != null ){
                checked = Arrays.asList(listChecked);
            }
            if (listCheckedComma != null && checkedValues.contains(",")){ //$NON-NLS-1$
                checked = Arrays.asList(listCheckedComma);
            }
            List<TermVocabularyDto> termsFromStringValues = getTermsFromStringValues(checked);
            treeComposite.setCheckedElements(termsFromStringValues.toArray());

        }else{

            treeComposite.getViewer().setAllChecked(true);
        }
    }

    @Override
    protected List<TermVocabularyDto> getTermsFromStringValues(List<String> listValue) {
        List<TermVocabularyDto> termlist = new ArrayList<>();
        List<UUID> uuidList = new ArrayList<>();
        for (String s : listValue) {
            if (!StringUtils.isBlank(s)){
                UUID uuid = UUID.fromString(s);
                uuidList.add(uuid);
            }
        }
        IVocabularyService vocabularyService = CdmStore.getService(IVocabularyService.class);

        termlist = vocabularyService.findVocabularyDtoByVocabularyUuids(uuidList);



        return termlist;

    }




}
