/**
 * Copyright (C) 2014 EDIT
 * European Distributed Institute of Taxonomy
 * http://www.e-taxonomy.eu
 *
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */
package eu.etaxonomy.taxeditor.preference.wizard;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.UUID;

import org.apache.commons.lang3.StringUtils;
import org.eclipse.swt.widgets.Composite;

import eu.etaxonomy.cdm.api.service.ITermService;
import eu.etaxonomy.cdm.api.service.IVocabularyService;
import eu.etaxonomy.cdm.model.metadata.PreferencePredicate;
import eu.etaxonomy.cdm.model.term.DefinedTermBase;
import eu.etaxonomy.cdm.model.term.TermType;
import eu.etaxonomy.cdm.model.term.TermVocabulary;
import eu.etaxonomy.cdm.persistence.dto.TermDto;
import eu.etaxonomy.cdm.persistence.dto.TermVocabularyDto;
import eu.etaxonomy.taxeditor.l10n.Messages;
import eu.etaxonomy.taxeditor.preference.PreferencesUtil;
import eu.etaxonomy.taxeditor.store.CdmStore;

/**
 * @author a.oppermann
 * @date 21.07.2014
 *
 */
public class AvailableDistributionPage extends AbstractTermSelectionWizardPage<TermDto> {



    public AvailableDistributionPage(String pageName) {
        super(pageName, TermType.NamedArea);
        this.localPref = true;


    }


    @Override
    public void createControl(Composite parent) {
        setTitle(Messages.AvailableDistributionPage_PAGE_TITLE);
        setDescription(Messages.AvailableDistributionPage_PAGE_DESCRIPTION);

        super.createControl(parent);
    }

    @Override
    protected String getCheckedValuesFromPreferences() {
        String checkedValues = PreferencesUtil.getStringValue(PreferencePredicate.AvailableDistributionAreaTerms.getKey(), true);

        return checkedValues;
    }

    @Override
    protected void rememberCheckedValues(String checkedValues) {
        initialiseVocabularies();

        treeComposite.getViewer().setInput(getVocabularies());

        if (checkedValues != null && checkedValues != "") { //$NON-NLS-1$
            String[] listChecked = checkedValues.split(";"); //$NON-NLS-1$
            String[] listCheckedComma = checkedValues.split(","); //$NON-NLS-1$
            List<String> checked = new ArrayList<>();
            if (listChecked != null ){
                checked = Arrays.asList(listChecked);
            }
            if (listCheckedComma != null && checkedValues.contains(",")){ //$NON-NLS-1$
                checked = Arrays.asList(listCheckedComma);
            }
            List<TermDto> termsFromStringValues = getTermsFromStringValues(checked);
            treeComposite.setCheckedElements(termsFromStringValues.toArray());
        }

    }

    @Override
    public void dispose() {
       // CdmStore.getCurrentSessionManager().dispose(this);
        super.dispose();
    }

//    @Override
//    public ICdmEntitySession getCdmEntitySession() {
//        return CdmStore.getCurrentSessionManager().getNullSession();
//    }
//
//    @Override
//    public <T extends CdmBase> Collection<T> getRootEntities() {
//        return null;
//    }
//
//    @Override
    public Map<Object, List<String>> getPropertyPathsMap() {
        Map<Object, List<String>> propertyPathsMap = new HashMap<Object, List<String>>();
        List<String> termsPropertyPaths = Arrays.asList(new String[] {
                "includes" //$NON-NLS-1$
        });
        propertyPathsMap.put("includes", termsPropertyPaths); //$NON-NLS-1$
        propertyPathsMap.put("terms", termsPropertyPaths); //$NON-NLS-1$
        return propertyPathsMap;
    }

    @Override
    protected  List<TermVocabularyDto> getVocabulariesFromPreference(){
        List<TermVocabularyDto> vocs = new ArrayList<>();

        if (PreferencesUtil.getPreferenceFromDB(PreferencePredicate.AvailableDistributionAreaVocabularies) == null && PreferencesUtil.getStringValue(PreferencePredicate.AvailableDistributionAreaVocabularies.getKey()) == null){
            vocs = CdmStore.getService(IVocabularyService.class).findVocabularyDtoByTermType(type);
        }else{
            String vocString = PreferencesUtil.getStringValue(PreferencePredicate.AvailableDistributionAreaVocabularies.getKey());
            if (vocString == null || vocString.equals("")){
                vocs = CdmStore.getService(IVocabularyService.class).findVocabularyDtoByTermType(type);
                return vocs;
            }
            String[] arrayVocs = vocString.split(";"); //$NON-NLS-1$

            Set<UUID> uuidVocs = new HashSet<>();
            for (String voc: arrayVocs){
                if (!StringUtils.isBlank(voc)){
                    uuidVocs.add(UUID.fromString(voc));
                }
            }
            List<TermVocabulary> tempVocs = CdmStore.getService(IVocabularyService.class).find(uuidVocs);
            for (TermVocabulary<?> voc: tempVocs){
                vocs.add(new TermVocabularyDto(voc.getUuid(), voc.getRepresentations(), voc.getTermType(), voc.getTitleCache(), voc.isAllowDuplicates(), voc.isOrderRelevant(), voc.isFlat()));
            }
        }
        return vocs;
    }
    @Override
    protected void initialiseVocabularies() {
        if (getVocabularies() != null) {
            getVocabularies().clear();
        }
        List<TermVocabularyDto> vocs = new ArrayList<>();
        vocs = getVocabulariesFromPreference();

        setVocabularies(vocs);
    }

    @Override
    protected List<TermDto> getTermsFromStringValues(List<String> listValue) {

            List<TermDto> termlist = new ArrayList<>();
            ITermService termService = CdmStore.getService(ITermService.class);
            List<UUID> uuidList = new ArrayList<>();
            for (String s : listValue) {
                if (!StringUtils.isBlank(s)){
                    UUID uuid = UUID.fromString(s);
                    uuidList.add(uuid);
                }
            }
            termlist = new ArrayList<>(termService.findByUUIDsAsDto(uuidList));


            if (listValue.isEmpty()){
                List<DefinedTermBase> terms = CdmStore.getTermManager().getAllTerms(type, null);
                for (DefinedTermBase<?> term: terms){
                    termlist.add(TermDto.fromTerm(term, true));
                }
            }

            return termlist;


    }

}
