/**
* Copyright (C) 2018 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.preference.wizard;

import java.util.ArrayList;
import java.util.List;
import java.util.UUID;

import org.apache.commons.lang3.StringUtils;
import org.eclipse.jface.util.IPropertyChangeListener;
import org.eclipse.jface.util.PropertyChangeEvent;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;

import eu.etaxonomy.cdm.api.service.ITermService;
import eu.etaxonomy.cdm.api.service.IVocabularyService;
import eu.etaxonomy.cdm.model.location.NamedArea;
import eu.etaxonomy.cdm.model.metadata.CdmPreference;
import eu.etaxonomy.cdm.model.metadata.CdmPreference.PrefKey;
import eu.etaxonomy.cdm.model.metadata.PreferencePredicate;
import eu.etaxonomy.cdm.model.metadata.PreferenceResolver;
import eu.etaxonomy.cdm.model.metadata.PreferenceSubject;
import eu.etaxonomy.cdm.model.term.DefinedTermBase;
import eu.etaxonomy.cdm.persistence.dto.TermDto;
import eu.etaxonomy.cdm.persistence.dto.TermVocabularyDto;
import eu.etaxonomy.taxeditor.l10n.Messages;
import eu.etaxonomy.taxeditor.model.ImageResources;
import eu.etaxonomy.taxeditor.preference.PreferencesUtil;
import eu.etaxonomy.taxeditor.preference.Resources;
import eu.etaxonomy.taxeditor.store.CdmStore;
import eu.etaxonomy.taxeditor.store.StoreUtil;
import eu.etaxonomy.taxeditor.ui.dialog.selection.SelectionDialogFactory;

/**
 * @author k.luther
 * @since 05.10.2018
 *
 */
public class AvailableDistributionPerAreaStatusPage extends AvailableDistributionStatusPage implements IPropertyChangeListener{

    List<CdmPreference> perAreaPreferences = new ArrayList();
    List<CdmPreference> perAreaPreferencesDeleteList = new ArrayList();
    List<TermVocabularyDto> vocs;
    NamedArea actualArea;
    CdmPreference actualPref;
    Text areaText;
    String namedAreaString;



    Button btnSelectArea;
    Button btnRemove;
    Label status;
    int countTerms;


    public AvailableDistributionPerAreaStatusPage(String pageName, List<CdmPreference> prefs, boolean localPref) {
        super(pageName, null, localPref);

        perAreaPreferences = prefs;
        if(PreferencesUtil.getPreferenceFromDB(PreferencePredicate.AvailableDistributionAreaVocabularies) == null && PreferencesUtil.getStringValue(PreferencePredicate.AvailableDistributionAreaVocabularies.getKey()) == null){
            vocs = CdmStore.getService(IVocabularyService.class).findVocabularyDtoByTermType(type);

        }
        pref = PreferencesUtil.getPreferenceFromDB(PreferencePredicate.AvailableDistributionStatus);
        if (pref == null){
            pref = CdmPreference.NewTaxEditorInstance(PreferencePredicate.AvailableDistributionStatus, null);
        }
        initialiseVocabularies();
        countTerms = CdmStore.getTermManager().getPreferredTerms(type).size();
    }



    @Override
    public void createControl(Composite parent) {
        setTitle(Messages.AvailableDistributionStatusPage_PAGE_TITLE);
        setDescription(Messages.AvailableDistributionStatusPage_PAGE_DESCRIPTION);
        createAreaSelectionCombo(parent);

        super.createControl(parent);



    }
    @Override
    protected Button createAllowOverrideButton(Composite parent) {
        Button activateCheckButton = new Button(parent, SWT.CHECK);
        if (localPref){
            activateCheckButton.setText("Override");
            activateCheckButton.setSelection(override);
        }else{
            activateCheckButton.setText("Allow Override");
            activateCheckButton.setSelection(override);
        }

        activateCheckButton.addSelectionListener(new SelectionAdapter(){
            @Override
            public void widgetSelected(SelectionEvent e) {
                if (actualPref != null){
                   actualPref.setAllowOverride(activateCheckButton.getSelection());
                }
                override = activateCheckButton.getSelection();

            }
        });

         return activateCheckButton;
     }

    protected void createAreaSelectionCombo(Composite parent) {

        Composite toolbar = new Composite(parent, SWT.NONE);
        toolbar.setLayoutData(new GridData(SWT.LEFT, SWT.CENTER, true, false, 1, 1));
        GridLayout gl_toolbar = new GridLayout(4, false);
        gl_toolbar.horizontalSpacing = 4;
        gl_toolbar.marginHeight = 0;
        gl_toolbar.verticalSpacing = 0;
        toolbar.setLayout(gl_toolbar);

        Label areaLabel = new Label(toolbar, SWT.NONE);
        areaLabel.setText("Select area for area specific status");

        GridData textData = new GridData();
        textData.minimumWidth = 200;
        textData.widthHint = 200;
        areaText = new Text(toolbar,  SWT.NONE);
        areaText.setEditable(false);
        areaText.setBackground(StoreUtil
                .getColor(Resources.COLOR_TEXT_DISABLED_BACKGROUND));
        areaText.setLayoutData(textData);
        if (actualArea != null){
            areaText.setText(actualArea.getLabel());
            areaText.setEnabled(false);
        }
        btnSelectArea = new Button(toolbar, SWT.PUSH | SWT.FLAT );
        btnSelectArea.setImage(ImageResources
                .getImage(ImageResources.BROWSE_ICON));
        btnSelectArea.setToolTipText("Browse Area");

        btnSelectArea.addSelectionListener(new SelectionListener(){

            @Override
            public void widgetSelected(SelectionEvent e) {
                refreshActualPrefernce();

                NamedArea selection = SelectionDialogFactory.getSelectionFromDialog(NamedArea.class, getShell(), //getConversationHolder(),
                        actualArea, null);
                actualArea = selection;
                PrefKey key;
                if (actualArea != null){
                    areaText.setText(actualArea.getLabel());
                    key = newAreaSpecificStatusPreference();
                    actualPref = PreferenceResolver.resolve(perAreaPreferences, key);
                    if (actualPref == null){
                        actualPref = CdmPreference.NewInstance(key, null);
                        perAreaPreferences.add(actualPref);
                    }
                    override = actualPref.isAllowOverride();
                }else{
                    actualPref = null;
                }

                status.setVisible(false);
                rememberCheckedValues(getCheckedValuesFromPreferences());

            }

            @Override
            public void widgetDefaultSelected(SelectionEvent e) {
                // TODO Auto-generated method stub

            }

        });
//        btnRemove = new Button(toolbar, SWT.NONE);
//        btnRemove.addSelectionListener(new SelectionListener(){
//
//            @Override
//            public void widgetSelected(SelectionEvent e) {
//
//                PrefKey key = newAreaSpecificStatusPreference();
//                actualPref = PreferenceResolver.resolve(perAreaPreferences, key);
//                actualPref = CdmPreference.NewInstance(key, null);
//                if (actualPref.getKey().equals(pref.getKey())){
//                    pref = actualPref;
//                }
//
//                perAreaPreferences.add(actualPref);
//                if (actualArea != null){
//                    status.setText("Preference for area " + actualArea.getLabel() + " removed. Now the status defined for all areas are displayed");
//                }else{
//                    status.setText("Preference for all areas is removed. Now all status will be displayed");
//                }
//                status.setVisible(true);
//
//                actualArea = null;
//
//                areaText.setText("");
//                rememberCheckedValues(getCheckedValuesFromPreferences());
//
//            }
//
//            @Override
//            public void widgetDefaultSelected(SelectionEvent e) {
//                // TODO Auto-generated method stub
//
//            }
//
//        });
//        btnRemove.setImage(ImageResources.getImage(ImageResources.ACTIVE_DELETE_ICON));
//        btnRemove.setToolTipText("");
        status = new Label(toolbar, SWT.NONE);
        status.setVisible(false);
    }


    @Override
    public void propertyChange(PropertyChangeEvent event) {
        if(event == null){
            return;
        }
        Object eventSource = event.getSource();
        if(eventSource != this && btnSelectArea.equals(eventSource)){
            if (actualArea != null){
                PrefKey key = newAreaSpecificStatusPreference();
                actualPref = PreferenceResolver.resolve(perAreaPreferences, key);
                areaText.setText(actualArea.getLabel());
                override = actualPref.isAllowOverride();
            }else{
                actualPref = null;
            }

        }
    }

    @Override
    protected String getCheckedValuesFromPreferences() {
       String checkedValues = null;
       if (actualArea == null){
           checkedValues = pref.getValue();
           override = pref.isAllowOverride();
       }else{
           if (actualPref == null){
               PrefKey key = newAreaSpecificStatusPreference();
               actualPref = PreferenceResolver.resolve(perAreaPreferences, key);

               if (actualPref == null){
                   actualPref = CdmPreference.NewInstance(key, null);
                   checkedValues = PreferencesUtil.getStringValue(PreferencePredicate.AvailableDistributionStatus.getKey(), true);

               }
           }
           checkedValues = actualPref.getValue();
           override = actualPref.isAllowOverride();
       }

       return checkedValues;
    }

    protected void refreshActualPrefernce(){
        String saveCheckedElements;
        getViewer().expandAll();


        if (countTerms == getViewer().getCheckedElements().length){
            saveCheckedElements = null;
        }else{
            Object[] checkedElements = getViewer().getCheckedElements();
            List<UUID> listUIIDChecked = new ArrayList<>();
            List<DefinedTermBase> preferredTerms = new ArrayList<>();
            for (Object o : checkedElements) {
                 if(o instanceof TermDto){
                     TermDto termDto = (TermDto) o;
                     listUIIDChecked.add(termDto.getUuid());
                     preferredTerms.add(CdmStore.getService(ITermService.class).load(termDto.getUuid()));
                 }
             }
             saveCheckedElements = StringUtils.join(listUIIDChecked, ";"); //$NON-NLS-1$
             if (StringUtils.isBlank(saveCheckedElements)){
                 saveCheckedElements = null;
         }
        }
        if (actualArea == null){
            CdmPreference savePref = CdmPreference.NewTaxEditorInstance(PreferencePredicate.AvailableDistributionStatus, saveCheckedElements);
            savePref.setAllowOverride(override);
            pref = savePref;
        }else{
            PrefKey key = newAreaSpecificStatusPreference();
            actualPref = PreferenceResolver.resolve(perAreaPreferences, key);
            CdmPreference savePref = CdmPreference.NewInstance(PreferenceSubject.fromKey(key), PreferencePredicate.AvailableDistributionStatus, saveCheckedElements);
            if (actualPref != null){
               perAreaPreferences.remove(actualPref);
            }
            perAreaPreferences.add(savePref);
            savePref.setAllowOverride(override);
            actualPref = savePref;

        }
    }

    @Override
    protected  List<TermVocabularyDto> getVocabulariesFromPreference(){
     return null;
    }

    private PrefKey newAreaSpecificStatusPreference(){
        PrefKey prefKey;
        if (actualArea != null){
            prefKey  = CdmPreference.NewKey(PreferenceSubject.NewInstance(PreferenceSubject.ROOT+"NamedArea["+actualArea.getUuid().toString()+"]"+PreferenceSubject.SEP), PreferencePredicate.AvailableDistributionStatus);
        }else{
            prefKey = CdmPreference.NewKey(PreferenceSubject.NewTaxEditorInstance(), PreferencePredicate.AvailableDistributionStatus);
        }
        return prefKey;
    }


    public void setActualPreference(CdmPreference pref){
        this.actualPref = pref;
        if (actualPref != null){
            override = actualPref.isAllowOverride();
        }else{
            override = true;
        }
    }


}
