/**
* Copyright (C) 2018 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.preference.wizard;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.jface.wizard.IWizardPage;
import org.eclipse.jface.wizard.Wizard;
import org.osgi.service.prefs.BackingStoreException;
import org.osgi.service.prefs.Preferences;

import eu.etaxonomy.cdm.api.conversation.ConversationHolder;
import eu.etaxonomy.cdm.api.conversation.IConversationEnabled;
import eu.etaxonomy.cdm.model.location.NamedArea;
import eu.etaxonomy.cdm.model.metadata.CdmPreference;
import eu.etaxonomy.cdm.model.metadata.CdmPreference.PrefKey;
import eu.etaxonomy.cdm.model.metadata.PreferencePredicate;
import eu.etaxonomy.cdm.model.metadata.PreferenceSubject;
import eu.etaxonomy.cdm.persistence.hibernate.CdmDataChangeMap;
import eu.etaxonomy.taxeditor.l10n.Messages;
import eu.etaxonomy.taxeditor.preference.CdmPreferenceCache;
import eu.etaxonomy.taxeditor.preference.PreferencesUtil;
import eu.etaxonomy.taxeditor.store.CdmStore;

/**
 * @author k.luther
 * @since 05.10.2018
 *
 */
public class AvailableDistributionPerAreaStatusWizard extends Wizard implements IConversationEnabled{
    private ConversationHolder conversation;
    private final AvailableDistributionPerAreaStatusPage aPage;
    List<CdmPreference> prefs;
    private boolean isAdmin;


    public AvailableDistributionPerAreaStatusWizard(boolean isAdmin) {
        this.isAdmin = isAdmin;
        setWindowTitle(Messages.AvailableDistributionStatusWizard_WIZARD_TITLE);
        // TODO if preferenceStore has elements checked load elements in wizard
        if (isAdmin){
            CdmPreferenceCache cache = CdmPreferenceCache.instance();
            prefs = new ArrayList<>();
            cache.getAllTaxEditorDBPreferences();
            List<CdmPreference> distrPrefs = cache.getAllForPredicate(PreferencePredicate.AvailableDistributionStatus.getKey());
            if (distrPrefs != null){
                prefs.addAll(distrPrefs);
            }
            if (prefs == null || prefs.isEmpty()) {
                prefs = new ArrayList();
                prefs.add(CdmPreference.NewTaxEditorInstance(PreferencePredicate.AvailableDistributionStatus, null));
            }
        }else{
            Preferences predicateNode = PreferencesUtil.getEditorPreferences().node(PreferencePredicate.AvailableDistributionStatus.getKey());
            try {
                if (prefs == null){
                    prefs = new ArrayList();
                }
                for (String childName: predicateNode.childrenNames()){
                    PrefKey key = CdmPreference.NewKey(PreferenceSubject.NewInstance(childName), PreferencePredicate.AvailableDistributionStatus);
                    prefs.add(CdmPreference.NewInstance(key, predicateNode.get(childName, null)));
                }
            } catch (BackingStoreException e) {
                // TODO Auto-generated catch block
                e.printStackTrace();
            }
        }
        aPage = new AvailableDistributionPerAreaStatusPage(Messages.AvailableDistributionStatusWizard_PAGE_TITLE, prefs, false);


    }

    public void setActualPref(CdmPreference pref){
        this.aPage.setActualPreference(pref);
    }

    public void setNamedArea(NamedArea namedArea){
        this.aPage.actualArea = namedArea;

    }

    @Override
    public void addPages() {
        addPage(aPage);
    }

    @Override
    public IWizardPage getPage(String pageName) {

        return aPage;
    }

    @Override
    public int getPageCount() {
       return 1;
    }

    @Override
    public IWizardPage[] getPages() {
        IWizardPage[] pages = new IWizardPage[1];
        pages [0] = aPage;
        return pages;
    }

    @Override
    public String getWindowTitle() {

        return Messages.AvailableDistributionStatusWizard_WINDOW_TITLE;
    }

    @Override
    public boolean performCancel() {
        // TODO Auto-generated method stub
        return true;
    }

    @Override
    public boolean performFinish() {
        aPage.refreshActualPrefernce();
        if (isAdmin){
            PreferencesUtil.setPreferencesToDB(aPage.perAreaPreferences);
            PreferencesUtil.setPreferenceToDB(aPage.pref);
        }else{

            //how to create a new preference(s) or save to a list????
//            PreferencesUtil.getEditorPreferences().put(PreferencesUtil.createSubjectStringForChildNodes("", aPage.pref), aPage.getPreference().getValue());
        }


       // PreferencesUtil.setBooleanValue(PreferencesUtil.createOverridePreferenceString(PreferencePredicate.AvailableDistributionStatus.getKey()), aPage.override);
        PreferencesUtil.firePreferencesChanged(this.getClass());

        return true;
    }

    @Override
    public void update(CdmDataChangeMap changeEvents) {

    }

    @Override
    public ConversationHolder getConversationHolder() {
        if (conversation == null) {
            conversation = CdmStore.createConversation();
        }
        return conversation;
    }

}
