/**
* Copyright (C) 2018 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.preference.wizard;

import java.util.ArrayList;
import java.util.List;
import java.util.UUID;

import org.apache.commons.lang3.StringUtils;
import org.eclipse.jface.wizard.IWizardPage;
import org.eclipse.jface.wizard.Wizard;

import eu.etaxonomy.cdm.api.conversation.ConversationHolder;
import eu.etaxonomy.cdm.api.conversation.IConversationEnabled;
import eu.etaxonomy.cdm.model.metadata.CdmPreference;
import eu.etaxonomy.cdm.model.metadata.PreferencePredicate;
import eu.etaxonomy.cdm.model.term.TermType;
import eu.etaxonomy.cdm.persistence.dto.TermDto;
import eu.etaxonomy.cdm.persistence.hibernate.CdmDataChangeMap;
import eu.etaxonomy.taxeditor.l10n.Messages;
import eu.etaxonomy.taxeditor.preference.PreferencesUtil;
import eu.etaxonomy.taxeditor.store.CdmStore;

/**
 * @author k.luther
 * @since 05.10.2018
 *
 */
public class AvailableDistributionStatusWizard extends Wizard implements IConversationEnabled{
    private ConversationHolder conversation;
    private final AvailableDistributionStatusPage aPage;
    CdmPreference pref;

    public AvailableDistributionStatusWizard() {
        setWindowTitle(Messages.AvailableDistributionStatusWizard_WIZARD_TITLE);
        // TODO if preferenceStore has elements checked load elements in wizard
        pref = PreferencesUtil.getPreferenceFromDB(PreferencePredicate.AvailableDistributionStatus);
        if (pref == null) {
            pref = CdmPreference.NewTaxEditorInstance(PreferencePredicate.AvailableDistributionStatus, null);
        }
        aPage = new AvailableDistributionStatusPage(Messages.AvailableDistributionStatusWizard_PAGE_TITLE, pref, true);

    }

    public AvailableDistributionStatusWizard(CdmPreference pref) {
        setWindowTitle(Messages.AvailableDistributionStatusWizard_WIZARD_TITLE);
        // TODO if preferenceStore has elements checked load elements in wizard

        if (pref == null) {
            pref = CdmPreference.NewTaxEditorInstance(PreferencePredicate.AvailableDistributionStatus, null);
        }
        aPage = new AvailableDistributionStatusPage(Messages.AvailableDistributionStatusWizard_PAGE_TITLE, pref, true);

    }

    @Override
    public void addPages() {
        addPage(aPage);
    }

    @Override
    public IWizardPage getPage(String pageName) {

        return aPage;
    }

    @Override
    public int getPageCount() {
       return 1;
    }

    @Override
    public IWizardPage[] getPages() {
        IWizardPage[] pages = new IWizardPage[1];
        pages [0] = aPage;
        return pages;
    }

    @Override
    public String getWindowTitle() {

        return Messages.AvailableDistributionStatusWizard_WINDOW_TITLE;
    }

    @Override
    public boolean performCancel() {
        // TODO Auto-generated method stub
        return true;
    }

    @Override
    public boolean performFinish() {
        Object[] checkedElements = aPage.getViewer().getCheckedElements();

        List<UUID> listUIIDChecked = new ArrayList<>();
        List<TermDto> preferredTerms = new ArrayList<>();
        for (Object o : checkedElements) {
            if(o instanceof TermDto){
                TermDto termDto = (TermDto) o;
                listUIIDChecked.add(termDto.getUuid());
                preferredTerms.add(termDto);
            }
        }
        String saveCheckedElements = StringUtils.join(listUIIDChecked, ";"); //$NON-NLS-1$

        PreferencesUtil.setStringValue(PreferencePredicate.AvailableDistributionStatus.getKey(), saveCheckedElements);
        PreferencesUtil.setBooleanValue(PreferencesUtil.createOverridePreferenceString(PreferencePredicate.AvailableDistributionStatus.getKey()), aPage.override);

        CdmStore.getTermManager().setPreferredTermsByType(preferredTerms, TermType.PresenceAbsenceTerm);
        PreferencesUtil.firePreferencesChanged(this.getClass());
        return true;
    }

    @Override
    public void update(CdmDataChangeMap changeEvents) {

    }

    @Override
    public ConversationHolder getConversationHolder() {
        if (conversation == null) {
            conversation = CdmStore.createConversation();
        }
        return conversation;
    }

}
