/**
 * Copyright (C) 2014 EDIT
 * European Distributed Institute of Taxonomy
 * http://www.e-taxonomy.eu
 *
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */
package eu.etaxonomy.taxeditor.preference.wizard;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.UUID;

import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.eclipse.jface.wizard.Wizard;

import eu.etaxonomy.cdm.model.metadata.PreferencePredicate;
import eu.etaxonomy.cdm.persistence.dto.TermDto;
import eu.etaxonomy.cdm.persistence.dto.TermVocabularyDto;
import eu.etaxonomy.taxeditor.l10n.Messages;
import eu.etaxonomy.taxeditor.preference.PreferencesUtil;

/**
 * @author alex
 * @date 21.07.2014
 *
 */
public class AvailableDistributionWizard extends Wizard {

   // private ConversationHolder conversation;
    private final AvailableDistributionPage aPage;

    public AvailableDistributionWizard() {
        setWindowTitle(Messages.AvailableDistributionWizard_WINDOW_TITLE);
        // TODO if preferenceStore has elements checked load elements in wizard
        aPage = new AvailableDistributionPage(Messages.AvailableDistributionWizard_PAGE_TITLE);
    }

    @Override
    public boolean performFinish() {
        // TODO: get Selection and save in EditorPreferences
        if (!checkNoneChecked()) {
            Object[] checkedElements = aPage.getViewer().getCheckedElements();
            Object[] grayedElements = aPage.getViewer().getGrayedElements();
            ArrayList<Object> checkedList = new ArrayList<>(Arrays.asList(checkedElements));
            ArrayList<Object> grayedList = new ArrayList<>(Arrays.asList(grayedElements));
            checkedList = (ArrayList) CollectionUtils.subtract(checkedList, grayedList);
            ArrayList<UUID> listUIIDChecked = new ArrayList<>();
            ArrayList<UUID> listUIIDGrayed = new ArrayList<>();
            for (Object o : checkedList) {
                if (o instanceof TermDto) {
                    listUIIDChecked.add(((TermDto) o).getUuid());

                }else if(o instanceof TermVocabularyDto){
                    TermVocabularyDto termVocDto = (TermVocabularyDto) o;
                    listUIIDGrayed.add(termVocDto.getUuid());
                }
            }
            for (Object o : grayedList) {
                if (o instanceof TermDto) {
                    listUIIDChecked.add(((TermDto) o).getUuid());

                }else if(o instanceof TermVocabularyDto){
                    TermVocabularyDto termVocDto = (TermVocabularyDto) o;
                    listUIIDGrayed.add(termVocDto.getUuid());
                }
            }

            String saveCheckedElements = StringUtils.join(listUIIDChecked, ";"); //$NON-NLS-1$

            PreferencesUtil.setStringValue(PreferencePredicate.AvailableDistributionAreaTerms.getKey(), saveCheckedElements);
            return true;
        } else {
            return false;
        }
    }

    @Override
    public void addPages() {
        addPage(aPage);
    }

    private boolean checkNoneChecked() {

        if (aPage.getViewer().getCheckedElements().length == 0) {
            aPage.setMessage(Messages.AvailableDistributionWizard_CHECK_MESSAGE, aPage.WARNING);
            return true;
        } else {
            aPage.setMessage(null);
            return false;
        }
    }
//
//    @Override
//    public ConversationHolder getConversationHolder() {
//        if (conversation == null) {
//            conversation = CdmStore.createConversation();
//        }
//        return conversation;
//    }
//
//    @Override
//    public void update(CdmDataChangeMap changeEvents) {
//    }
}
