/**
* Copyright (C) 2018 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.preference.wizard;

import java.util.ArrayList;
import java.util.UUID;

import org.apache.commons.lang3.StringUtils;
import org.eclipse.jface.wizard.IWizardPage;
import org.eclipse.jface.wizard.Wizard;

import eu.etaxonomy.cdm.api.conversation.ConversationHolder;
import eu.etaxonomy.cdm.api.conversation.IConversationEnabled;
import eu.etaxonomy.cdm.model.metadata.CdmPreference;
import eu.etaxonomy.cdm.model.metadata.PreferencePredicate;
import eu.etaxonomy.cdm.model.metadata.PreferenceSubject;
import eu.etaxonomy.cdm.persistence.dto.TermVocabularyDto;
import eu.etaxonomy.cdm.persistence.hibernate.CdmDataChangeMap;
import eu.etaxonomy.taxeditor.l10n.Messages;
import eu.etaxonomy.taxeditor.preference.PreferencesUtil;
import eu.etaxonomy.taxeditor.store.CdmStore;

/**
 * @author k.luther
 * @since 04.06.2018
 *
 */
public class AvailableVocabularyWizard extends Wizard implements IConversationEnabled {

    private ConversationHolder conversation;
    private final AvailableAreaVocabulariesPage aPage;
    private boolean localPref = true;
    private CdmPreference pref;

    public AvailableVocabularyWizard(boolean localPref, CdmPreference pref, String featureTitle) {
        setWindowTitle(Messages.AvailableVocabularyWizard_WIZARD_TITLE);
        // TODO if preferenceStore has elements checked load elements in wizard
        this.localPref = localPref;
        this.pref = pref;
        aPage = new AvailableAreaVocabulariesPage(Messages.AvailableVocabularyWizard_PAGE_TITLE, localPref, pref, featureTitle);
    }
//    public AvailableVocabularyWizard(boolean localPref, CdmPreference pref, String featureTitle) {
//        setWindowTitle(Messages.AvailableVocabularyWizard_WIZARD_TITLE);
//        // TODO if preferenceStore has elements checked load elements in wizard
//        this.localPref = localPref;
//        this.pref = pref;
//        aPage = new AvailableAreaVocabulariesPage(Messages.AvailableVocabularyWizard_PAGE_TITLE, localPref, pref, featureTitle);
//    }

    @Override
    public void addPages() {
        addPage(aPage);
    }

    @Override
    public IWizardPage getPage(String pageName) {

        return aPage;
    }

    @Override
    public int getPageCount() {
       return 1;
    }

    @Override
    public IWizardPage[] getPages() {
        IWizardPage[] pages = new IWizardPage[1];
        pages [0] = aPage;
        return pages;
    }

    @Override
    public String getWindowTitle() {
        return Messages.AvailableVocabularyWizard_WINDOW_TITLE;
    }

    @Override
    public boolean performCancel() {
        return true;
    }

    @Override
    public boolean performFinish() {
        Object[] checkedElements = aPage.getViewer().getCheckedElements();

        ArrayList<UUID> listUIIDChecked = new ArrayList<UUID>();
        int countChecked = 0;
        for (Object o : checkedElements) {
            if(o instanceof TermVocabularyDto){
                TermVocabularyDto vocDto = (TermVocabularyDto) o;
                listUIIDChecked.add(vocDto.getUuid());
                countChecked++;
            }
        }
        String saveCheckedElements = StringUtils.join(listUIIDChecked, ";"); //$NON-NLS-1$
        int countVoc = aPage.getVocabularies().size();
        String predicate = null;
        pref = aPage.getPreference();
        if (pref != null){
            predicate = pref.getPredicate();
        }

        if (!localPref){
            if (StringUtils.isBlank(saveCheckedElements) || countChecked == countVoc){
                saveCheckedElements = null;
            }
            CdmPreference savePref = CdmPreference.NewInstance(PreferenceSubject.NewTaxEditorInstance(), PreferencePredicate.getByKey(predicate), saveCheckedElements);
            savePref.setAllowOverride(pref.isAllowOverride());
            PreferencesUtil.setPreferenceToDB(savePref);

            PreferencesUtil.updateDBPreferences();
        }else{
            PreferencesUtil.setStringValue(predicate, saveCheckedElements);
            PreferencesUtil.setBooleanValue(PreferencesUtil.createOverridePreferenceString(predicate), this.aPage.override);
        }
        return true;
    }

    @Override
    public void update(CdmDataChangeMap changeEvents) {
    }

    @Override
    public ConversationHolder getConversationHolder() {
        if (conversation == null) {
            conversation = CdmStore.createConversation();
        }
        return conversation;
    }

}
