/**
* Copyright (C) 2019 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.preference.wizard;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.UUID;

import org.apache.commons.lang3.StringUtils;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CLabel;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;

import eu.etaxonomy.cdm.api.service.IVocabularyService;
import eu.etaxonomy.cdm.model.metadata.CdmPreference;
import eu.etaxonomy.cdm.persistence.dto.TermVocabularyDto;
import eu.etaxonomy.taxeditor.editor.definedterm.DefinedTermSorter;
import eu.etaxonomy.taxeditor.editor.definedterm.RootElementsOnlyTreeContentProvider;
import eu.etaxonomy.taxeditor.editor.definedterm.TermDtoLabelProvider;
import eu.etaxonomy.taxeditor.l10n.Messages;
import eu.etaxonomy.taxeditor.preference.AbstractGeneralTermPreference;
import eu.etaxonomy.taxeditor.preference.LocalOrDefaultEnum;
import eu.etaxonomy.taxeditor.preference.PreferencesUtil;
import eu.etaxonomy.taxeditor.store.CdmStore;

/**
 * @author k.luther
 * @since 13.09.2019
 */
public class GeneralVocabularyPreferencePage extends AbstractGeneralTermPreference<TermVocabularyDto> implements ISelectionChangedListener, SelectionListener{


    protected List<TermVocabularyDto> getVocabulariesFromPreference() {

        return null;
    }

    @Override
    protected Control createContents(Composite parent) {
        Composite container = new Composite(parent, SWT.NULL);
       // getValues();
        getPreferenceValues();
        if (!isAdminPreference && !allowOverride){
            final CLabel description = new CLabel(parent, SWT.NULL);
            description.setText(Messages.NameDetailsViewConfiguration_description_not_available);
            this.setDefaultButtonActivated(false);
            return parent;

        }
        final GridLayout gridLayout = new GridLayout();
        gridLayout.numColumns = 2;
        container.setLayout(gridLayout);
        parent.setLayout(new GridLayout());
        useLocalOrAdmin = new Combo(parent, SWT.BORDER | SWT.READ_ONLY);
        if (!isAdminPreference){
            useLocalOrAdmin.add(LocalOrDefaultEnum.Default.getLabel(), 0);
            useLocalOrAdmin.add(LocalOrDefaultEnum.Local.getLabel(), 1);

        }else{
            useLocalOrAdmin.add(LocalOrDefaultEnum.AllowOverride.getLabel(), 0);
            useLocalOrAdmin.add(LocalOrDefaultEnum.Database.getLabel(), 1);
        }


        treeComposite = new CheckBoxTreeComposite(container, new RootElementsOnlyTreeContentProvider(), new TermDtoLabelProvider(), SWT.NONE);
        treeComposite.getViewer().setComparator(new DefinedTermSorter());
        treeComposite.setAllowTopLevelSelection(true);
        rememberCheckedValues(getCheckedValuesFromPreferences());

        if (!isAdminPreference){
            if (isOverride){
                useLocalOrAdmin.select(1);
            }else{
                useLocalOrAdmin.select(0);
                treeComposite.setEnabled(false);
            }
        }else{
            if (allowOverride){
                useLocalOrAdmin.select(0);
            }else{
                useLocalOrAdmin.select(1);
            }
        }
        useLocalOrAdmin.addSelectionListener(this);

        GridData data = new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1);
        data.heightHint = 300;
        treeComposite.setLayoutData(data);
        treeComposite.getViewer().addSelectionChangedListener(this);

        return container;

    }

    @Override
    protected void rememberCheckedValues(String checkedValues) {
        initialiseVocabularies();
        treeComposite.getViewer().getTree().deselectAll();
        treeComposite.getViewer().refresh();
        treeComposite.getViewer().setInput(getVocabularies());

        if (checkedValues != null && checkedValues != "") { //$NON-NLS-1$
            String[] listChecked = checkedValues.split(";"); //$NON-NLS-1$
            String[] listCheckedComma = checkedValues.split(","); //$NON-NLS-1$
            List<String> checked = new ArrayList<>();
            if (listChecked != null ){
                checked = Arrays.asList(listChecked);
            }
            if (listCheckedComma != null && checkedValues.contains(",")){ //$NON-NLS-1$
                checked = Arrays.asList(listCheckedComma);
            }
            List<TermVocabularyDto> termsFromStringValues = getTermsFromStringValues(checked);
            treeComposite.setCheckedElements(termsFromStringValues.toArray());

        }else{
            List<TermVocabularyDto> termsFromStringValues = getTermsFromStringValues(new ArrayList<String>());
            treeComposite.setCheckedElements(termsFromStringValues.toArray());
            treeComposite.getViewer().refresh();
            treeComposite.redraw();

        }
    }


    @Override
    protected List<TermVocabularyDto> getTermsFromStringValues(List<String> listValue) {
        List<TermVocabularyDto> vocList = new ArrayList<>();
        List<UUID> uuids = new ArrayList();
        IVocabularyService vocService = CdmStore.getService(IVocabularyService.class);
        for (String s : listValue) {
            if (!StringUtils.isBlank(s)){
                UUID uuid = UUID.fromString(s);
                uuids.add(uuid);
            }
        }
        vocList = vocService.findVocabularyDtoByVocabularyUuids(uuids);


        if (listValue.isEmpty()){
            vocList =vocService.findVocabularyDtoByTermType(type);
        }

        return vocList;
    }

    @Override
    public boolean performOk() {
        if (!isApply()){
            return true;
        }
        Object[] checkedElements = treeComposite.getViewer().getCheckedElements();
        List<UUID> listUIIDChecked = new ArrayList<>();

        for (Object o : checkedElements) {
            if(o instanceof TermVocabularyDto){
                TermVocabularyDto termDto = (TermVocabularyDto) o;
                listUIIDChecked.add(termDto.getUuid());
             }
        }
        String saveCheckedElements = StringUtils.join(listUIIDChecked, ";"); //$NON-NLS-1$
        if (isAdminPreference){
            CdmPreference savePref = CdmPreference.NewTaxEditorInstance(predicate, saveCheckedElements);
            String text = this.useLocalOrAdmin.getText();
            if (text.equals(LocalOrDefaultEnum.Database.getLabel()) ){
                savePref.setAllowOverride(false);
            }else if (text.equals(LocalOrDefaultEnum.AllowOverride.getLabel())){
                savePref.setAllowOverride(true);
            }
            PreferencesUtil.setPreferenceToDB(savePref);
            PreferencesUtil.updateDBPreferences();
        }else{
            PreferencesUtil.setStringValue(predicate.getKey(), saveCheckedElements);
            String text = this.useLocalOrAdmin.getText();
            if (text.equals(LocalOrDefaultEnum.Database.getLabel()) || text.equals(LocalOrDefaultEnum.Default.getLabel())){
                PreferencesUtil.setBooleanValue(PreferencesUtil.createOverridePreferenceString(predicate.getKey()), false);
            }else if (text.equals(LocalOrDefaultEnum.Local.getLabel())){
                PreferencesUtil.setBooleanValue(PreferencesUtil.createOverridePreferenceString(predicate.getKey()), true);
            }
        }

        PreferencesUtil.firePreferencesChanged(this.getClass());
        return true;
    }

    @Override
    protected void performDefaults(){
        if (isAdminPreference){
            rememberCheckedValues("");
            treeComposite.setEnabled(true);
        }else{
            isOverride = false;
            rememberCheckedValues(getCheckedValuesFromPreferences());
            treeComposite.setEnabled(false);
        }
       pref = null;
       useLocalOrAdmin.select(0);

       setApply(true);
       super.performDefaults();

    }

}
