/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.termtree;

import java.util.Arrays;
import java.util.HashSet;
import java.util.List;
import java.util.Set;
import java.util.stream.Collectors;

import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Composite;

import eu.etaxonomy.cdm.api.service.ITermService;
import eu.etaxonomy.cdm.model.term.DefinedTermBase;
import eu.etaxonomy.cdm.persistence.dto.TermDto;
import eu.etaxonomy.cdm.persistence.dto.TermVocabularyDto;
import eu.etaxonomy.taxeditor.editor.definedterm.DefinedTermSorter;
import eu.etaxonomy.taxeditor.editor.definedterm.TermDtoContentProvider;
import eu.etaxonomy.taxeditor.editor.definedterm.TermDtoLabelProvider;
import eu.etaxonomy.taxeditor.preference.wizard.CheckBoxTreeComposite;
import eu.etaxonomy.taxeditor.store.CdmStore;

/**
 * @author n.hoffmann
 * @created Aug 5, 2010
 * @version 1.0
 */
public class AvailableTermsWizardPage extends WizardPage {

    private CheckBoxTreeComposite treeComposite;
    private List<TermVocabularyDto> vocabularies;

	protected AvailableTermsWizardPage(List<TermVocabularyDto> vocabularies, boolean singleSelection) {
		super("Available Terms Wizard Page");
		this.vocabularies = vocabularies;
		setTitle("Available Terms");
		if(singleSelection){
		    setDescription("Please select one single term");
		}
		else{
		    setDescription("Please select terms.");
		}
	}

	/** {@inheritDoc} */
	@Override
	public void createControl(Composite parent) {
        treeComposite = new CheckBoxTreeComposite(parent, new TermDtoContentProvider(), new TermDtoLabelProvider(), SWT.NONE);
        treeComposite.getViewer().setComparator(new DefinedTermSorter());
        treeComposite.getViewer().setInput(vocabularies);
        treeComposite.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1));
        treeComposite.getViewer().addSelectionChangedListener(e->getWizard().getContainer().updateButtons());

		setControl(treeComposite);
	}

	public Set<DefinedTermBase> getSelectedFeatures() {

        List<Object> checkedElements = Arrays.asList(treeComposite.getViewer().getCheckedElements());
        checkedElements = checkedElements.stream().filter(element->element instanceof TermDto).collect(Collectors.toList());

		Set<DefinedTermBase> terms = new HashSet<>();

		for(Object element : checkedElements){
		    if(element instanceof TermDto){
		        DefinedTermBase term = CdmStore.getService(ITermService.class).load(((TermDto) element).getUuid());
		        terms.add(term);
		    }
		}
		return terms;
	}

}
