/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.termtree;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.UUID;

import org.eclipse.jface.viewers.ITreeContentProvider;
import org.eclipse.jface.viewers.Viewer;

import eu.etaxonomy.cdm.model.term.DefinedTermBase;
import eu.etaxonomy.cdm.model.term.TermNode;
import eu.etaxonomy.cdm.model.term.TermTree;
import eu.etaxonomy.cdm.persistence.dto.TermNodeDto;
import eu.etaxonomy.cdm.persistence.dto.TermTreeDto;

/**
 * <p>TermTreeContentProvider class.</p>
 *
 * @author n.hoffmann
 * @created Aug 5, 2010
 */
public class TermTreeContentProvider<T extends DefinedTermBase> implements ITreeContentProvider {

    private Viewer viewer;
    private TermNodeDto oldInput;
    private TermNodeDto newInput;
    private Map<UUID, TermNodeDto> uuidTermMap = new HashMap<>();

	@Override
	public void dispose() {}

	@Override
	public void inputChanged(Viewer viewer, Object oldInput, Object newInput) {
	    this.viewer = viewer;
	    if (oldInput instanceof TermNodeDto){
	        this.oldInput = (TermNodeDto)oldInput;
	    }
	    if (newInput instanceof TermNodeDto){
	        this.newInput = (TermNodeDto)newInput;
	    }
	    if (oldInput instanceof TermTreeDto){
	        this.oldInput = ((TermTreeDto)oldInput).getRoot();
	    }
	    if (newInput instanceof TermTreeDto){
            this.newInput = ((TermTreeDto)newInput).getRoot();
        }
	}

	@Override
	public Object[] getElements(Object inputElement) {
		return this.getChildren(inputElement);
	}

	@Override
	public Object[] getChildren(Object parentElement) {
	    if(parentElement instanceof TermTree){
            List<TermNode<?>> children = ((TermTree) parentElement).getRootChildren();
            return children.toArray();
        }else if(parentElement instanceof TermNode){
            List<TermNode<?>> children = ((TermNode) parentElement).getChildNodes();
            return children.toArray();
        }
		if(parentElement instanceof TermTreeDto){
		    if (((TermTreeDto) parentElement).getRoot() != null){
		        List<TermNodeDto> children = ((TermTreeDto) parentElement).getRoot().getChildren();
		        return children.toArray();
		    }
			return new Object[]{};
		}else if(parentElement instanceof TermNodeDto){
			List<TermNodeDto> children = ((TermNodeDto) parentElement).getChildren();
			return children.toArray();
		}else if(parentElement instanceof List){
			return ((List<TermNodeDto>) parentElement).toArray();
		}else{
			return new Object[]{};
		}
	}

	@Override
	public Object getParent(Object element) {
	    if(element instanceof TermNode){
            return ((TermNode<?>) element).getParent();
        }
	    if(element instanceof TermNodeDto){
	        TermNodeDto parent = uuidTermMap.get(((TermNodeDto) element).getParentUuid());
	        return parent;
	    }
		return null;
	}

	public Map<UUID, TermNodeDto> getUuidTermMap() {
        return uuidTermMap;
    }

	public void setUuidTermMap(Map<UUID, TermNodeDto> uuidTermMap) {
        this.uuidTermMap = uuidTermMap;
    }

	@Override
	public boolean hasChildren(Object element) {
		return getChildren(element).length != 0;
	}
}