/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.termtree;

import java.util.ArrayList;

import org.eclipse.swt.graphics.Image;

import eu.etaxonomy.cdm.model.common.Language;
import eu.etaxonomy.cdm.model.term.DefinedTermBase;
import eu.etaxonomy.cdm.model.term.Representation;
import eu.etaxonomy.cdm.model.term.TermNode;
import eu.etaxonomy.cdm.model.term.TermTree;
import eu.etaxonomy.cdm.persistence.dto.TermDto;
import eu.etaxonomy.cdm.persistence.dto.TermNodeDto;
import eu.etaxonomy.cdm.persistence.dto.TermTreeDto;
import eu.etaxonomy.taxeditor.editor.definedterm.TermDtoLabelProvider;
import eu.etaxonomy.taxeditor.model.ImageResources;
import eu.etaxonomy.taxeditor.preference.IPreferenceKeys;
import eu.etaxonomy.taxeditor.preference.PreferencesUtil;

/**
 * @author n.hoffmann
 * @created Aug 5, 2010
 */
public class TermTreeLabelProvider extends TermDtoLabelProvider {

    public TermTreeLabelProvider() {
        showIdInVoc = PreferencesUtil.getBooleanValue(IPreferenceKeys.SHOW_VOCABULARY_ID_FOR_TERM_LABELS_IN_TREE_EDITOR, true);
    }


	@Override
	public String getText(Object element) {
		if(element instanceof TermTreeDto){
			TermTreeDto treeElement = (TermTreeDto)element;
			Representation rep = treeElement.getRepresentation(PreferencesUtil.getGlobalLanguage());
			return rep != null? rep.getLabel() :((TermTreeDto) element).getTitleCache()  ;
		}else if(element instanceof TermNodeDto){
		    TermDto term = ((TermNodeDto) element).getTerm();
		    if(term!=null){
//		        boolean showIdInVoc = PreferencesUtil.getBooleanValue(IPreferenceKeys.SHOW_VOCABULARY_ID_FOR_TERM_LABELS_IN_TREE_EDITOR, true);
		        return super.getText(term);
		    }

		}else if(element instanceof TermNode){
            DefinedTermBase<?> term = ((TermNode<?>) element).getTerm();
            if(term!=null){
                Representation rep = term.getRepresentation(PreferencesUtil.getGlobalLanguage());
                if (rep == null){
                    rep = term.getPreferredRepresentation(new ArrayList<Language>());
                }
                return rep != null? rep.getLabel() : term.getTitleCache();
            }

        }else if(element instanceof TermTree){
            return ((TermTree<?>) element).getTitleCache();

        }
		return "No label provided";
	}

	 @Override
	 public Image getImage(Object element) {
	     if (element instanceof TermTreeDto){
	            TermTreeDto dto = (TermTreeDto)element;
	            if (!dto.isFlat()){
	                return ImageResources.getImage(ImageResources.HIERARCHICAL);
	            }else{
	                return ImageResources.getImage(ImageResources.FLAT);
	            }

	        }
	        return null;
	 }
}