/**
* Copyright (C) 2017 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.termtree.e4;

import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.TreeSelection;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.swt.dnd.DragSourceAdapter;
import org.eclipse.swt.dnd.DragSourceEvent;

import eu.etaxonomy.cdm.model.term.TermNode;
import eu.etaxonomy.cdm.persistence.dto.TermNodeDto;
import eu.etaxonomy.taxeditor.termtree.TermNodeDtoTransfer;

public class TermNodeDtoDragListener extends DragSourceAdapter {

	private final TreeViewer viewer;

	public TermNodeDtoDragListener(TreeViewer viewer) {
		this.viewer = viewer;
	}

    /**
     * Method declared on DragSourceListener
     */
    @Override
    public void dragSetData(DragSourceEvent event) {
        IStructuredSelection selection = (IStructuredSelection) viewer.getSelection();
        if(selection!=null && !selection.isEmpty()){
            if (selection instanceof TreeSelection && ((TreeSelection)selection).getFirstElement() instanceof TermNodeDto){
                TermNodeDto[] featureNodes = (TermNodeDto[]) selection.toList().toArray(new TermNodeDto[selection.size()]);
                if (TermNodeDtoTransfer.getInstance().isSupportedType(event.dataType)) {
                    event.data = featureNodes;
                }
            }
        }

    }

	/**
	 * Method declared on DragSourceListener
	 */
	@Override
	public void dragStart(DragSourceEvent event) {
	    ISelection sel = viewer.getSelection();
	    if (sel instanceof TreeSelection && ((TreeSelection)sel).getFirstElement() instanceof TermNodeDto){
            event.doit = !viewer.getSelection().isEmpty()
                    && ((IStructuredSelection) viewer.getSelection()).toList()
                    .stream().allMatch(element -> element instanceof TermNodeDto);
	    }else {
	        event.doit = !viewer.getSelection().isEmpty()
                    && ((IStructuredSelection) viewer.getSelection()).toList()
                    .stream().allMatch(element -> element instanceof TermNode);
	    }
	}

}