/**
* Copyright (C) 2021 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.termtree.e4;

import org.eclipse.jface.viewers.Viewer;
import org.eclipse.jface.viewers.ViewerComparator;

import eu.etaxonomy.cdm.common.CdmUtils;
import eu.etaxonomy.cdm.persistence.dto.TermNodeDto;
import eu.etaxonomy.taxeditor.preference.IPreferenceKeys;
import eu.etaxonomy.taxeditor.preference.PreferencesUtil;


/**
 * @author k.luther
 * @since Mar 1, 2021
 */
public class TermTreeSorter extends ViewerComparator {


    @Override
    public int compare(Viewer viewer, Object e1, Object e2) {
        if (e1 instanceof TermNodeDto && e2 instanceof TermNodeDto){
            boolean showIdInVoc = PreferencesUtil.getBooleanValue(IPreferenceKeys.SHOW_VOCABULARY_ID_FOR_TERM_LABELS, true);
            String label1 = ((TermNodeDto) e1).getTerm().getRepresentation_L10n();
            String label2 = ((TermNodeDto) e2).getTerm().getRepresentation_L10n();
            if(showIdInVoc){
                label1 = CdmUtils.concat(" : ", ((TermNodeDto) e1).getTerm().getIdInVocabulary(), label1);
                label2 = CdmUtils.concat(" : ", ((TermNodeDto) e1).getTerm().getIdInVocabulary(), label2);
            }
            return label1.compareTo(label2);
        }
        return super.compare(viewer, e1, e2);
    }
}
