/**
* Copyright (C) 2020 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.termtree.e4.handler;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import javax.inject.Named;

import org.eclipse.core.runtime.IStatus;
import org.eclipse.e4.core.di.annotations.Execute;
import org.eclipse.e4.ui.di.UISynchronize;
import org.eclipse.e4.ui.model.application.ui.basic.MPart;
import org.eclipse.e4.ui.services.IServiceConstants;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.wizard.WizardDialog;
import org.eclipse.swt.widgets.Shell;

import eu.etaxonomy.cdm.model.description.Character;
import eu.etaxonomy.cdm.model.term.DefinedTermBase;
import eu.etaxonomy.cdm.model.term.TermType;
import eu.etaxonomy.cdm.persistence.dto.CharacterDto;
import eu.etaxonomy.cdm.persistence.dto.CharacterNodeDto;
import eu.etaxonomy.cdm.persistence.dto.TermDto;
import eu.etaxonomy.cdm.persistence.dto.TermNodeDto;
import eu.etaxonomy.taxeditor.l10n.Messages;
import eu.etaxonomy.taxeditor.model.MessagingUtils;
import eu.etaxonomy.taxeditor.termtree.AvailableTermsWizard;
import eu.etaxonomy.taxeditor.termtree.e4.ICharacterEditor;
import eu.etaxonomy.taxeditor.termtree.e4.ITermTreeEditor;
import eu.etaxonomy.taxeditor.termtree.e4.TermTreeEditor;
import eu.etaxonomy.taxeditor.termtree.e4.operation.AddFeatureOperation;

/**
 * @author k.luther
 * @since Nov 10, 2020
 */
public abstract class AbstractAddTermHandler {
    @Execute
    public void execute(@Named(IServiceConstants.ACTIVE_SHELL) Shell shell,
            @Named(IServiceConstants.ACTIVE_PART)MPart thisPart,
            @Named(IServiceConstants.ACTIVE_SELECTION)IStructuredSelection selection,
            UISynchronize sync) {
        ITermTreeEditor editor = ((ITermTreeEditor) thisPart.getObject());

        TermNodeDto parent = getParent(selection);
        TermType type = null;
        if (parent.getTerm() == null){
            type = parent.getTree().getTermType();
        }else{
            type = parent.getTerm().getTermType();
        }
        AvailableTermsWizard wizard = new AvailableTermsWizard(type);
        WizardDialog dialog = new WizardDialog(shell, wizard);

        if (dialog.open() == IStatus.OK) {
            Collection<DefinedTermBase> additionalTerms = wizard.getAdditionalFeatures();
            List<DefinedTermBase> duplicates = new ArrayList<>();
            for (DefinedTermBase term : additionalTerms) {

                boolean isDuplicate = false;
                if (!parent.getTree().isAllowDuplicate()){
                    isDuplicate = editor.checkDuplicates(term.getUuid(), parent.getTree().getUuid());
                    if (isDuplicate){
                        duplicates.add(term);
                    }
                }
                if (isDuplicate){
                    continue;
                }
                AddFeatureOperation operation = new AddFeatureOperation(term.getUuid(), parent, editor, editor);
//                AbstractUtility.executeOperation(operation, sync);
                editor.addOperation(operation);
                editor.setDirty();

                if (editor instanceof ICharacterEditor){
                    CharacterNodeDto newDto = new CharacterNodeDto(CharacterDto.fromCharacter((Character)term), parent, 0, parent.getTree(), null, null, null);
                }else{
                    TermNodeDto newDto = new TermNodeDto(TermDto.fromTerm(term), parent, 0, parent.getTree(), null, null, null);
                }

//                ((AbstractTermTreeEditor)editor).getViewer().refresh();
                if (editor instanceof ICharacterEditor){
                    editor.refresh();
                }else{
                    Object[] expandedElements = ((TermTreeEditor)editor).getViewer().getExpandedElements();
                    ((TermTreeEditor)editor).getViewer().setInput(((TermTreeEditor)editor).getTrees());
                    ((TermTreeEditor)editor).getViewer().setExpandedElements(expandedElements);
                }
            }
            if (!duplicates.isEmpty()){
                String termsTitles = "";
                for (DefinedTermBase term: duplicates){
                    termsTitles = termsTitles + term.getTitleCache() +"\n";
                }
                MessagingUtils.informationDialog(Messages.AddFeatureHandler_Duplicates_not_allowed, Messages.AddFeatureHandler_Duplicates_not_allowed_message +  "\n"+termsTitles);
            }
        }
    }

    /**
     * @param selection
     * @return
     */
    abstract protected TermNodeDto getParent(IStructuredSelection selection);
}
