/**
* Copyright (C) 2017 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.termtree.e4.handler;

import javax.inject.Named;

import org.eclipse.e4.core.di.annotations.CanExecute;
import org.eclipse.e4.core.di.annotations.Execute;
import org.eclipse.e4.ui.di.UISynchronize;
import org.eclipse.e4.ui.model.application.ui.basic.MPart;
import org.eclipse.e4.ui.model.application.ui.menu.MHandledMenuItem;
import org.eclipse.e4.ui.services.IServiceConstants;
import org.eclipse.jface.dialogs.InputDialog;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.window.Window;

import eu.etaxonomy.cdm.model.term.TermTree;
import eu.etaxonomy.cdm.persistence.dto.TermTreeDto;
import eu.etaxonomy.taxeditor.store.StoreUtil;
import eu.etaxonomy.taxeditor.termtree.e4.ITermTreeEditor;
import eu.etaxonomy.taxeditor.termtree.e4.TermTreeEditor;
import eu.etaxonomy.taxeditor.termtree.e4.operation.CreateFeatureTreeOperation;

/**
 * @author pplitzner
 * @since Jul 12, 2017
 */
public class CreateTermTreeHandler {

    @Execute
    public void execute(@Named(IServiceConstants.ACTIVE_PART)MPart thisPart, UISynchronize sync){
        ITermTreeEditor editor = (ITermTreeEditor) thisPart.getObject();

        if (StoreUtil.promptCheckIsDirty(editor)) {
            return;
        }
        Object[] expandedElements =  ((TermTreeEditor)editor).getViewer().getExpandedElements();

        InputDialog dialog = new InputDialog(null,
                "Create new term tree", "Enter new term tree title", "",
                null);
        if (dialog.open() == Window.CANCEL){
            return;
        }
        TermTree<?> newTree = TermTree.NewInstance(editor.getTermType());
        newTree.setAllowDuplicates(false);
        newTree.setFlat(false);
        newTree.setOrderRelevant(true);
        newTree.setTitleCache(dialog.getValue(), true);

        CreateFeatureTreeOperation operation = new CreateFeatureTreeOperation(newTree, editor, editor);
        TermTreeDto newDto = TermTreeDto.fromTree(newTree);
        editor.setDirty();

        editor.addOperation(operation);
        ((TermTreeEditor)editor).putTree(newDto);
        ((TermTreeEditor)editor).getViewer().setInput(((TermTreeEditor)editor).getTrees());

        ((TermTreeEditor)editor).getViewer().setExpandedElements(expandedElements);
        ((TermTreeEditor)editor).getViewer().refresh();
        IStructuredSelection sel = new StructuredSelection(newDto);
        ((TermTreeEditor)editor).getViewer().setSelection(sel);



    }

    @CanExecute
    public boolean canExecute(
            @Named(IServiceConstants.ACTIVE_PART)MPart thisPart,
            MHandledMenuItem menuItem) {
        boolean canExecute = false;
        canExecute = thisPart.getObject() instanceof ITermTreeEditor;
        menuItem.setVisible(canExecute);
        return canExecute;
    }
}