/**
* Copyright (C) 2020 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.termtree.e4.handler;

import java.util.List;
import java.util.stream.Collectors;

import org.eclipse.e4.core.di.annotations.Execute;
import org.eclipse.e4.ui.model.application.MApplication;
import org.eclipse.e4.ui.model.application.commands.MParameter;
import org.eclipse.e4.ui.model.application.ui.basic.MPart;
import org.eclipse.e4.ui.model.application.ui.basic.MPartStack;
import org.eclipse.e4.ui.model.application.ui.menu.MHandledMenuItem;
import org.eclipse.e4.ui.workbench.modeling.EModelService;
import org.eclipse.e4.ui.workbench.modeling.EPartService;
import org.eclipse.e4.ui.workbench.modeling.EPartService.PartState;

import eu.etaxonomy.cdm.model.term.TermType;
import eu.etaxonomy.taxeditor.termtree.e4.TermTreeEditor;
import eu.etaxonomy.taxeditor.workbench.WorkbenchUtility;

/**
 * @author k.luther
 * @since Dec 11, 2020
 */
public class OpenTermTreeEditorHandler {
    @Execute
    public void execute(EPartService partService, MHandledMenuItem menuItem, MApplication application, EModelService modelService) {

        String commandId = menuItem.getCommand().getElementId();
        List<MParameter> parameters = menuItem.getParameters();
        TermType termType = null;
        for (MParameter param: parameters){
            termType = TermType.getByKey(param.getValue());
        }

        if(termType != null){
        TermType type = termType;

        List<MPart> alreadyOpenedEditors = partService.getParts().stream()
        .filter(part->part.getObject()!=null && part.getObject() instanceof TermTreeEditor)
        .filter(part->((TermTreeEditor)part.getObject()).getTermType().equals(type))
        .collect(Collectors.toList());
        if(!alreadyOpenedEditors.isEmpty()){
            //there should never be more than one already opened editor
            //so we just open the first
            partService.activate(alreadyOpenedEditors.iterator().next());
        }
        else{
            MPart part = partService.createPart(eu.etaxonomy.taxeditor.store.AppModelId.PARTDESCRIPTOR_EU_ETAXONOMY_TAXEDITOR_EDITOR_TERMTREE);
            MPartStack editorAreaPartStack = WorkbenchUtility.getEditorAreaPartStack(application, modelService);
            if(editorAreaPartStack!=null){
                editorAreaPartStack.getChildren().add(part);
            }
            part = partService.showPart(part, PartState.ACTIVATE);
            TermTreeEditor termEditor = (TermTreeEditor) part.getObject();
            termEditor.init(termType, menuItem.getLocalizedLabel());
        }
        }
    }

}
