/**
* Copyright (C) 2017 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.termtree.e4.handler;

import javax.inject.Named;

import org.eclipse.e4.core.di.annotations.CanExecute;
import org.eclipse.e4.core.di.annotations.Execute;
import org.eclipse.e4.core.di.annotations.Optional;
import org.eclipse.e4.ui.di.UISynchronize;
import org.eclipse.e4.ui.model.application.ui.basic.MPart;
import org.eclipse.e4.ui.model.application.ui.menu.MHandledMenuItem;
import org.eclipse.e4.ui.services.IServiceConstants;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.TreeSelection;

import eu.etaxonomy.cdm.persistence.dto.TermNodeDto;
import eu.etaxonomy.cdm.persistence.dto.TermTreeDto;
import eu.etaxonomy.taxeditor.store.StoreUtil;
import eu.etaxonomy.taxeditor.termtree.e4.ICharacterEditor;
import eu.etaxonomy.taxeditor.termtree.e4.ITermTreeEditor;
import eu.etaxonomy.taxeditor.termtree.e4.TermTreeEditor;
import eu.etaxonomy.taxeditor.termtree.e4.operation.RemoveFeatureOperation;

/**
 * @author pplitzner
 * @since Jul 12, 2017
 *
 */
public class RemoveTermHandler {

    @Execute
    public void execute(@Named(IServiceConstants.ACTIVE_PART)MPart thisPart,
            @Named(IServiceConstants.ACTIVE_SELECTION)IStructuredSelection selection, UISynchronize sync){
        ITermTreeEditor editor = (ITermTreeEditor) thisPart.getObject();

//        if (StoreUtil.promptCheckIsDirty(editor)) {
//            return;
//        }
        if(!StoreUtil.confirmDelete()){
            return;
        }

        for (Object selectedObject : selection.toArray()) {
        	TermNodeDto featureNode = (TermNodeDto) selectedObject;
        	TermTreeDto tree = featureNode.getTree();
        	if (featureNode.getUuid() != null){
        	    RemoveFeatureOperation operation = new RemoveFeatureOperation(featureNode.getUuid(), editor, editor);
        	    editor.addOperation(operation);
        	}else{
//        	    editor.removeAddOperation(featureNode);
        	}
            ISelection sel = editor.getViewer().getSelection();
            if (sel instanceof TreeSelection){
//                Object o =((TreeSelection) sel).getFirstElement();
//                if (o instanceof TermNodeDto && ((TermNodeDto)o).getUuid().equals(featureNode.getUuid())){
                    TermNodeDto parent = editor.getNodeDtoForUuid(featureNode.getParentUuid());
                    parent.removeChild(featureNode, false);
//                }
            }
//            editor.refresh();
//            }else{
//                for (Object treeDto: ((AbstractTermTreeEditor)editor).getTrees()){
//                    if (treeDto instanceof TermTreeDto && ((TermTreeDto)treeDto).getUuid().equals(featureNode.getTree().getUuid())){
//                        ((TermTreeDto)treeDto).removeChild(featureNode);
//                    }
//
//                }
                if (editor instanceof TermTreeEditor){
                    Object[] expandedElements = ((TermTreeEditor)editor).getViewer().getExpandedElements();
                    ((TermTreeEditor)editor).getViewer().setInput(((TermTreeEditor)editor).getTrees());
                    ((TermTreeEditor)editor).getViewer().setExpandedElements(expandedElements);
                }else{
                    ((ICharacterEditor)editor).getViewer().setInput(tree);
                }
                editor.refresh();
//            }

            editor.setDirty();
        }
    }

    @CanExecute
    public boolean canExecute(
            @Optional@Named(IServiceConstants.ACTIVE_SELECTION)IStructuredSelection selection,
            @Named(IServiceConstants.ACTIVE_PART)MPart thisPart,
            MHandledMenuItem menuItem) {
        boolean canExecute = selection!=null && !selection.isEmpty();
        if(selection!=null){
            for(Object object:selection.toArray()){
                canExecute &= object instanceof TermNodeDto;
            }
        }
        canExecute &= thisPart.getObject() instanceof ITermTreeEditor;
        menuItem.setVisible(canExecute);
        return canExecute;
    }

}
