/**
* Copyright (C) 2017 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.termtree.e4.handler;

import javax.inject.Named;

import org.eclipse.e4.core.di.annotations.CanExecute;
import org.eclipse.e4.core.di.annotations.Execute;
import org.eclipse.e4.core.di.annotations.Optional;
import org.eclipse.e4.ui.di.UISynchronize;
import org.eclipse.e4.ui.model.application.ui.basic.MPart;
import org.eclipse.e4.ui.model.application.ui.menu.MHandledMenuItem;
import org.eclipse.e4.ui.services.IServiceConstants;
import org.eclipse.jface.viewers.IStructuredSelection;

import eu.etaxonomy.cdm.persistence.dto.TermTreeDto;
import eu.etaxonomy.taxeditor.store.StoreUtil;
import eu.etaxonomy.taxeditor.termtree.e4.ITermTreeEditor;
import eu.etaxonomy.taxeditor.termtree.e4.TermTreeEditor;
import eu.etaxonomy.taxeditor.termtree.e4.operation.RemoveFeatureTreeOperation;

/**
 * @author pplitzner
 * @since Jul 12, 2017
 *
 */
public class RemoveTermTreeHandler {

    @Execute
    public void execute(@Named(IServiceConstants.ACTIVE_PART)MPart thisPart,
            @Named(IServiceConstants.ACTIVE_SELECTION)IStructuredSelection selection, UISynchronize sync){
        ITermTreeEditor editor = (ITermTreeEditor) thisPart.getObject();

        if (StoreUtil.promptCheckIsDirty(editor)) {
            return;
        }
        if(!StoreUtil.confirmDelete()){
            return;
        }

        for (Object selectedObject : selection.toArray()) {
        	TermTreeDto featureTree = (TermTreeDto) selectedObject;
        	((TermTreeEditor)editor).removeTree(featureTree);
        	((TermTreeEditor)editor).getViewer().setInput(((TermTreeEditor)editor).getTrees());
            RemoveFeatureTreeOperation operation = new RemoveFeatureTreeOperation(featureTree.getUuid(), editor, editor);
//            AbstractUtility.executeOperation(operation, sync);
            editor.addOperation(operation);
            editor.setDirty();
            editor.refresh();
        }
    }

    @CanExecute
    public boolean canExecute(
            @Optional@Named(IServiceConstants.ACTIVE_SELECTION)IStructuredSelection selection,
            @Named(IServiceConstants.ACTIVE_PART)MPart thisPart,
            MHandledMenuItem menuItem) {
        boolean canExecute = selection!=null && !selection.isEmpty();
        if(canExecute){
            for(Object object:selection.toArray()){
                canExecute &= object instanceof TermTreeDto;
            }
        }
        canExecute &= thisPart.getObject() instanceof ITermTreeEditor;
        menuItem.setVisible(canExecute);
        return canExecute;
    }

}
