/**
* Copyright (C) 2017 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.termtree.e4.handler;

import java.io.File;
import java.util.ArrayList;
import java.util.List;
import java.util.UUID;

import javax.inject.Named;

import org.eclipse.core.runtime.jobs.Job;
import org.eclipse.e4.core.di.annotations.CanExecute;
import org.eclipse.e4.core.di.annotations.Execute;
import org.eclipse.e4.core.di.annotations.Optional;
import org.eclipse.e4.ui.model.application.ui.basic.MPart;
import org.eclipse.e4.ui.model.application.ui.menu.MHandledMenuItem;
import org.eclipse.e4.ui.services.IServiceConstants;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.progress.IProgressConstants;

import eu.etaxonomy.cdm.io.descriptive.owl.out.StructureTreeOwlExportConfigurator;
import eu.etaxonomy.cdm.model.term.TermTree;
import eu.etaxonomy.cdm.persistence.dto.TermTreeDto;
import eu.etaxonomy.taxeditor.store.CdmStore;
import eu.etaxonomy.taxeditor.store.StoreUtil;
import eu.etaxonomy.taxeditor.termtree.e4.ITermTreeEditor;

/**
 * @author pplitzner
 * @since Jul 12, 2017
 *
 */
public class TermTreeExportOntologyHandler {

    @Execute
    public void execute(@Named(IServiceConstants.ACTIVE_SHELL) Shell shell,
            @Named(IServiceConstants.ACTIVE_SELECTION)IStructuredSelection selection,
            @Named(IServiceConstants.ACTIVE_PART)MPart thisPart){
        ITermTreeEditor editor = (ITermTreeEditor) thisPart.getObject();

        if(StoreUtil.promptCheckIsDirty(editor)){
            return;
        }

        List<UUID> featureTreeUuids = new ArrayList<>();
        selection.toList().stream().filter(o->o instanceof TermTree).forEach(tree->featureTreeUuids.add(((TermTree)tree).getUuid()));

        FileDialog dialog = new FileDialog(shell);
        String[] filterNames = {
                "Web Ontology Language (*.owl)",
                "Extensible Markup Language (*.xml)",
                "All files (*.*)"
                };
        String[] filterExtensions = { "*.owl", "*.xml", "*.*"};

        dialog.setOverwrite(true);
        dialog.setFilterNames(filterNames);
        dialog.setFilterExtensions(filterExtensions);
        String fileString = dialog.open();
        if(fileString!=null){
            // create job
            StructureTreeOwlExportConfigurator configurator = StructureTreeOwlExportConfigurator.NewInstance();

            configurator.setFeatureTreeUuids(featureTreeUuids);

            Job job = CdmStore.getExportManager().createIOServiceJob(configurator, new File(fileString));
            // configure the job
            job.setProperty(IProgressConstants.KEEP_PROPERTY, true);
            job.setUser(true);
            // schedule job
            job.schedule();
        }
    }

    @CanExecute
    public boolean canExecute(
            @Optional@Named(IServiceConstants.ACTIVE_SELECTION)IStructuredSelection selection,
            MHandledMenuItem menuItem) {
        boolean canExecute = false;
        canExecute = selection!=null
                && selection.size()==1
                && selection.getFirstElement() instanceof TermTreeDto;
        menuItem.setVisible(canExecute);
        return canExecute;
    }

}
