/**
* Copyright (C) 2018 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.termtree.e4.operation;

import java.util.UUID;

import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;

import eu.etaxonomy.cdm.api.service.ITermNodeService;
import eu.etaxonomy.cdm.api.service.UpdateResult;
import eu.etaxonomy.cdm.model.description.Feature;
import eu.etaxonomy.cdm.persistence.dto.TermNodeDto;
import eu.etaxonomy.taxeditor.model.MessagingUtils;
import eu.etaxonomy.taxeditor.operation.AbstractPostOperation;
import eu.etaxonomy.taxeditor.operation.IPostOperationEnabled;
import eu.etaxonomy.taxeditor.session.ICdmEntitySessionEnabled;
import eu.etaxonomy.taxeditor.store.CdmStore;
import eu.etaxonomy.taxeditor.store.StoreUtil;
import eu.etaxonomy.taxeditor.store.internal.TaxeditorStorePlugin;

/**
 * @author pplitzner
 * @since Apr 30, 2018
 */
public class AddFeatureOperation extends AbstractPostOperation<Feature> {

    private UUID nodeUuid;

    private int position;
    private UUID featureUuid;

    public AddFeatureOperation(UUID featureUuid, TermNodeDto node,
            IPostOperationEnabled postOperationEnabled, ICdmEntitySessionEnabled<Feature> cdmEntitySessionEnabled) {
        this(featureUuid, node.getUuid(), 0, postOperationEnabled, cdmEntitySessionEnabled);
    }
    public AddFeatureOperation(UUID featureUuid, UUID nodeUuid, int position,
            IPostOperationEnabled postOperationEnabled, ICdmEntitySessionEnabled<Feature> cdmEntitySessionEnabled) {
        super("Add Feature", StoreUtil.getUndoContext(), null, postOperationEnabled, cdmEntitySessionEnabled);

        this.nodeUuid = nodeUuid;
        this.featureUuid = featureUuid;
        this.position = position;
    }

    @Override
    public IStatus execute(IProgressMonitor monitor, IAdaptable info) throws ExecutionException {
        UpdateResult updateResult = CdmStore.getService(ITermNodeService.class).addChildNode(nodeUuid, featureUuid, position);
        if(updateResult.isError()){
            Exception exception = StoreUtil.mergeUpdateResultExceptions(updateResult);
            MessagingUtils.errorDialog("Error during operation", this, "Could not add term", TaxeditorStorePlugin.PLUGIN_ID, exception, false);
            return Status.CANCEL_STATUS;
        }
        return postExecute(updateResult.getCdmEntity());
    }

    @Override
    public IStatus redo(IProgressMonitor monitor, IAdaptable info) throws ExecutionException {
        return null;
    }

    @Override
    public IStatus undo(IProgressMonitor monitor, IAdaptable info) throws ExecutionException {
        return null;
    }
}