/**
* Copyright (C) 2018 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.termtree.e4.operation;

import java.util.UUID;

import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;

import eu.etaxonomy.cdm.api.service.ITermNodeService;
import eu.etaxonomy.cdm.api.service.UpdateResult;
import eu.etaxonomy.cdm.common.URI;
import eu.etaxonomy.cdm.model.description.Feature;
import eu.etaxonomy.cdm.model.term.TermNode;
import eu.etaxonomy.cdm.model.term.TermVocabulary;
import eu.etaxonomy.taxeditor.operation.AbstractPostOperation;
import eu.etaxonomy.taxeditor.operation.IPostOperationEnabled;
import eu.etaxonomy.taxeditor.session.ICdmEntitySessionEnabled;
import eu.etaxonomy.taxeditor.store.CdmStore;
import eu.etaxonomy.taxeditor.store.StoreUtil;
import eu.etaxonomy.taxeditor.view.webimport.termimport.wrapper.OntologyTermWrapper;

/**
 * @author pplitzner
 * @since Apr 30, 2018
 */
public class AddOntologyTermOperation extends AbstractPostOperation<TermNode> {

    private UUID targetUuid;
    private OntologyTermWrapper wrapper;
    private TermVocabulary<?> vocabulary;

    public AddOntologyTermOperation(OntologyTermWrapper wrapper, UUID targetUuid, TermVocabulary vocabulary,
            IPostOperationEnabled postOperationEnabled, ICdmEntitySessionEnabled<TermNode> cdmEntitySessionEnabled) {
        super("Add ontology term", StoreUtil.getUndoContext(), null, postOperationEnabled, cdmEntitySessionEnabled);
        this.wrapper = wrapper;
        this.targetUuid = targetUuid;
        this.vocabulary = vocabulary;
    }

    @Override
    public IStatus execute(IProgressMonitor monitor, IAdaptable info) throws ExecutionException {
        Feature feature = Feature.NewInstance(wrapper.getDescription(), wrapper.getLabel(), null);
        feature.setUri(URI.create(wrapper.getUri()));
        UpdateResult updateResult = CdmStore.getService(ITermNodeService.class).createChildNode(targetUuid, feature, vocabulary.getUuid());

//        UpdateResult updateResult = CdmStore.getService(ITermNodeService.class).addChildNode(targetUuid, element.getUuid());
        return postExecute(updateResult.getCdmEntity());
    }

    @Override
    public IStatus redo(IProgressMonitor monitor, IAdaptable info) throws ExecutionException {
        return null;
    }

    @Override
    public IStatus undo(IProgressMonitor monitor, IAdaptable info) throws ExecutionException {
        return null;
    }

}
