/**
 * Copyright (C) 2018 EDIT
 * European Distributed Institute of Taxonomy
 * http://www.e-taxonomy.eu
 *
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */
package eu.etaxonomy.taxeditor.termtree.e4.operation;

import java.util.UUID;

import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.swt.dnd.DND;

import eu.etaxonomy.cdm.api.service.ITermNodeService;
import eu.etaxonomy.cdm.api.service.UpdateResult;
import eu.etaxonomy.cdm.model.term.TermNode;
import eu.etaxonomy.taxeditor.operation.AbstractPostOperation;
import eu.etaxonomy.taxeditor.operation.IPostOperationEnabled;
import eu.etaxonomy.taxeditor.session.ICdmEntitySessionEnabled;
import eu.etaxonomy.taxeditor.store.CdmStore;
import eu.etaxonomy.taxeditor.store.StoreUtil;

/**
 * @author pplitzner
 * @date 21.01.2019
 */
public class MoveFeatureOperation extends AbstractPostOperation<TermNode>{

    private UUID droppedNodeUuid;
    private UUID targetUuid;
    private UUID termUuid;
    private int currentOperation;
    private int position;

    public MoveFeatureOperation(UUID droppedNodeUuid, UUID termUuid, UUID targetUuid, int position,
            int currentOperation,
            IPostOperationEnabled postOperationEnabled, ICdmEntitySessionEnabled<TermNode> cdmEntitySessionEnabled) {
        super(currentOperation == DND.DROP_MOVE ? "Move Feature" : "Copy Feature",StoreUtil.getUndoContext(),null, postOperationEnabled, cdmEntitySessionEnabled);
        this.droppedNodeUuid = droppedNodeUuid;
        this.targetUuid = targetUuid;
        this.position = position;
        this.currentOperation = currentOperation;
    }

    @Override
    public IStatus execute(IProgressMonitor monitor, IAdaptable info) throws ExecutionException {
        UpdateResult updateResult = null;
        //move operation
        if(currentOperation==DND.DROP_MOVE){
            updateResult = CdmStore.getService(ITermNodeService.class).moveNode(droppedNodeUuid, targetUuid, position);
        }
        //copy operation
        else if(currentOperation==DND.DROP_COPY){
            updateResult = CdmStore.getService(ITermNodeService.class).addChildNode(targetUuid, termUuid);
        }
        if(updateResult!=null){
            return postExecute(updateResult.getCdmEntity());
        }
        return Status.CANCEL_STATUS;
    }

    @Override
    public IStatus redo(IProgressMonitor monitor, IAdaptable info) throws ExecutionException {
        return null;
    }

    @Override
    public IStatus undo(IProgressMonitor monitor, IAdaptable info) throws ExecutionException {
        return null;
    }

    /**
     * This method will try to call the post operation on a possibly registered
     * IPostOperationEnabled implementor. Objects that were affected by the operation
     * may be passed to the registered IPostOperationEnabled implementor.
     *
     * @param objectAffectedByOperation the affected object. Should be <code>null</code> if not needed
     * @return a {@link org.eclipse.core.runtime.IStatus} object.
     */
    @Override
    protected IStatus postExecute(Object objectAffectedByOperation) {

        if(postOperationEnabled != null){
            return postOperationEnabled.postOperation(objectAffectedByOperation) ? Status.OK_STATUS : Status.CANCEL_STATUS;
        }
        return Status.OK_STATUS;
    }

    /**
     * <p>Getter for the field <code>postOperationEnabled</code>.</p>
     *
     * @return a {@link eu.etaxonomy.taxeditor.operation.IPostOperationEnabled} object.
     */
    @Override
    public IPostOperationEnabled getPostOperationEnabled() {
        return postOperationEnabled;
    }
}
