/**
* Copyright (C) 2017 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.ui.combo;

import org.eclipse.jface.viewers.ComboViewer;
import org.eclipse.jface.viewers.ILabelProvider;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;

/**
 *
 * Abstract generic combo.
 *
 * @author pplitzner
 * @since Apr 10, 2018
 *
 * @param <T> the type of the elements to show in the combo
 */
public abstract class AbstractComboComposite<T> extends Composite{

    private ComboViewer viewer;

    public AbstractComboComposite(Composite parent, int style){
        super(parent, style);
        setLayout(new FillLayout());
        viewer = new ComboViewer(new Combo(this, SWT.READ_ONLY | SWT.SINGLE));
        viewer.setContentProvider(getContentProvider());
        viewer.setLabelProvider(getLabelProvider());
    }

    public void setInput(Object input){
        viewer.setInput(input);
    }

    public T getSelection(){
        IStructuredSelection structuredSelection = viewer.getStructuredSelection();
        if(structuredSelection!=null && !structuredSelection.isEmpty()){
            return (T) structuredSelection.getFirstElement();
        }
        return null;
    }

    public void setText(String string){
        viewer.getCombo().setText(string);
    }

    public void setElement(T element){
        if(element==null){
            viewer.getCombo().deselectAll();
        }
        int index = viewer.getCombo().indexOf(getLabelProvider().getText(element));
        viewer.getCombo().select(index);
    }


    public void addSelectionChangedListener(ISelectionChangedListener listener){
        viewer.addSelectionChangedListener(listener);
    }

    public abstract IStructuredContentProvider getContentProvider();
    public abstract ILabelProvider getLabelProvider();

}
