/**
* Copyright (C) 2018 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.ui.combo;

import java.util.ArrayList;
import java.util.List;

import eu.etaxonomy.cdm.model.common.Language;
import eu.etaxonomy.cdm.model.name.NameRelationshipType;
import eu.etaxonomy.cdm.model.term.Representation;
import eu.etaxonomy.taxeditor.preference.IPreferenceKeys;
import eu.etaxonomy.taxeditor.preference.PreferencesUtil;
import eu.etaxonomy.taxeditor.store.CdmStore;

/**
 * @author k.luther
 * @since 15.10.2018
 */
public class InverseTermWrapper {

    private String label;
    private NameRelationshipType term;
    boolean isInverse;

    boolean useAbbrevLabel;

    InverseTermWrapper(NameRelationshipType term, boolean isInverse, boolean useAbbrevLabel){
        this.term = term;
        this.isInverse = isInverse;
        this.useAbbrevLabel = useAbbrevLabel;
        if (isInverse){
            this.label = getInverseLabel(term);
        }else{
            this.label = getLabel( term);
        }
    }

    public NameRelationshipType getTerm() {
        return term;
    }
    public void setTerm(NameRelationshipType term) {
        this.term = term;
    }

    public String getLabel() {
        return label;
    }
    public void setLabel(String label) {
        this.label = label;
    }

    public boolean isInverse() {
        return isInverse;
    }
    public void setInverse(boolean isInverse) {
        this.isInverse = isInverse;
    }

    private String getInverseLabel(NameRelationshipType term) {
        if (term == null){
            return "";
        }else{
            String termLabel = null;
            List<Language> languages = new ArrayList<>();

            languages.add(CdmStore.getDefaultLanguage());
            Representation rep = term.getPreferredInverseRepresentation(languages);
            if (rep != null){
                if (useAbbrevLabel){
                    termLabel = rep.getAbbreviatedLabel();
                    if (termLabel == null){
                        termLabel = rep.getLabel();
                    }
                }else{
                    termLabel = rep.getLabel();
                }
            }
            if (termLabel == null){
                termLabel = term.getInverseLabel();
            }

            return termLabel;
        }
    }

    /**
     * May be overridden by derived classes if the desired label string does not
     * reside in term.getLabel();
     *
     * @param term
     *            a T object.
     * @return a {@link java.lang.String} object.
     */
    protected String getLabel(NameRelationshipType term) {
        if (term == null){
            return "";
        }else{
            String termLabel = null;
            Representation rep = term.getPreferredRepresentation(CdmStore.getDefaultLanguage());
            if (rep != null){
                if (useAbbrevLabel){
                    termLabel = rep.getAbbreviatedLabel();
                    if (termLabel == null){
                        termLabel = rep.getLabel();
                    }
                }else{
                    termLabel = rep.getLabel();
                }
            }
            if (termLabel == null){
                termLabel = term.getLabel();
            }
            if(PreferencesUtil.getBooleanValue(IPreferenceKeys.SHOW_VOCABULARY_ID_FOR_TERM_LABELS)
                && term.getVocabulary()!=null){
                String vocLabel = term.getVocabulary().getLabel(CdmStore.getDefaultLanguage());
                if (vocLabel == null){
                    vocLabel = term.getVocabulary().getLabel();
                }
                termLabel += " ["+vocLabel+"]";
            }
            return termLabel;
        }
    }
}