/**
* Copyright (C) 2018 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.ui.combo;

import java.util.ArrayList;
import java.util.Comparator;
import java.util.List;

import org.eclipse.jface.resource.FontDescriptor;
import org.eclipse.jface.util.PropertyChangeEvent;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;

import eu.etaxonomy.cdm.model.common.Language;
import eu.etaxonomy.cdm.model.metadata.PreferencePredicate;
import eu.etaxonomy.cdm.model.name.NameRelationshipType;
import eu.etaxonomy.cdm.model.name.Rank;
import eu.etaxonomy.cdm.model.term.DefinedTermBase;
import eu.etaxonomy.cdm.model.term.Representation;
import eu.etaxonomy.cdm.model.term.TermType;
import eu.etaxonomy.taxeditor.preference.PreferencesUtil;
import eu.etaxonomy.taxeditor.store.CdmStore;
import eu.etaxonomy.taxeditor.store.TermManager;
import eu.etaxonomy.taxeditor.ui.element.CdmFormFactory;
import eu.etaxonomy.taxeditor.ui.element.CdmPropertyChangeEvent;
import eu.etaxonomy.taxeditor.ui.element.ICdmFormElement;

/**
 * @author k.luther
 * @since 08.10.2018
 *
 */
public class NameRelationshipTypeCombo<T extends InverseTermWrapper> extends AbstractComboElement<T> {


   List<InverseTermWrapper> wrapperList = new ArrayList<>();
   T wrapperSelection;
   List<NameRelationshipType> terms = new ArrayList();
   boolean useAbbrevLabel;


    /**
     * @param cdmFormFactory
     * @param parentElement
     * @param termType
     * @param labelString
     * @param selection
     * @param addEmptyElement
     * @param style
     * @param useAbbrevLabel
     * @param comparator
     */
    public NameRelationshipTypeCombo(CdmFormFactory cdmFormFactory, ICdmFormElement parentElement,
            String labelString,  boolean addEmptyElement, int style, boolean useAbbrevLabel,
            Comparator<T> comparator) {
     super(cdmFormFactory, parentElement);
     // labelString, selection, addEmptyElement, style,  comparator
     
     String availableTypes = PreferencesUtil.getStringValue(PreferencePredicate.AvailableNameRelationshipType.getKey());
     List<NameRelationshipType> relTypes = CdmStore.getTermManager().createTermListFromString(availableTypes, TermType.NameRelationshipType);
     
//     terms = getTermManager().getPreferredTerms(TermType.NameRelationshipType);
     
     terms = relTypes;
//     PreferencesUtil.get
     this.useAbbrevLabel = useAbbrevLabel;
     populateTerms();
     label.setText(labelString);
     addContentProposalAdapter();
     FontDescriptor descriptor = FontDescriptor.createFrom(label.getFont());
  // setStyle method returns a new font descriptor for the given style
     descriptor = descriptor.setStyle(SWT.ITALIC);
     label.setFont(descriptor.createFont(label.getDisplay()));

     combo.addSelectionListener(this);
     combo.addDisposeListener(this);
     PreferencesUtil.getPreferenceStore().addPropertyChangeListener(this);


    }

    /**
     * Fills the combo with elements and sets up the convenience functions
     * for selection index
     *
     * @param preferredTerms
     */

    protected void populateTerms() {

        combo.removeAll();


        fillInverseTermWrapperList();

        int i = 1;

        for (InverseTermWrapper termWrapper : wrapperList) {
            String label = termWrapper.getLabel();
            combo.add(label);
            combo.setData(label,termWrapper);
            i++;

        }

    }

    /**
     *
     */
    private void fillInverseTermWrapperList() {
        for (NameRelationshipType term: terms){
            wrapperList.add(new InverseTermWrapper(term, false, useAbbrevLabel));
            List<Language> languages = new ArrayList();
            languages.add(CdmStore.getDefaultLanguage());
            Representation rep = term.getPreferredInverseRepresentation(languages);

            if (rep != null && !rep.getLabel().equals(term.getLabel(CdmStore.getDefaultLanguage()))){
                wrapperList.add(new InverseTermWrapper(term, true, useAbbrevLabel));
            }
        }

    }


    /** {@inheritDoc} */
    @Override
    public void widgetSelected(SelectionEvent e) {
    	if (combo.getSelectionIndex() > -1){
    		wrapperSelection = (T) wrapperList.get(combo.getSelectionIndex());
    	}
        firePropertyChangeEvent(new CdmPropertyChangeEvent(this, e));
    }

    /** {@inheritDoc} */
    @Override
    public void propertyChange(PropertyChangeEvent event) {
        super.propertyChange(event);
        if (event != null
                && PreferencesUtil.PREFERRED_TERMS_CHANGE.equals(event
                        .getProperty())) {
            populateTerms();
        }
    }


    @Override
    public T getSelection() {
        return wrapperSelection;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public void setSelection(T selection) {
        wrapperSelection = selection;

    }
    protected TermManager getTermManager() {
        return CdmStore.getTermManager();
    }
}
