/**
* Copyright (C) 2019 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.ui.combo;

import java.util.Comparator;

import eu.etaxonomy.cdm.model.reference.OriginalSourceBase;
import eu.etaxonomy.cdm.model.reference.OriginalSourceType;

/**
 * @author pplitzner, k.luther
 * @since 04.12.2019
 */
/**
 * Sorts source type combo alphabetically <b>except</b> for PrimaryTaxonomicSource resp.
 * PrimaryMediaSource which are always on top depending on the IdentifiableSource;
 * <br>
 * The last two are always "Other" followed by "Unknown"
 * @author pplitzner
 * @date Jan 6, 2017
 *
 */
public class OriginalSourceTypeComparator implements Comparator<OriginalSourceType>{

    private OriginalSourceBase entity;

    public OriginalSourceTypeComparator(OriginalSourceBase entity) {
        this.entity = entity;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public int compare(OriginalSourceType o1, OriginalSourceType o2) {
        if(o1==null){
            if(o2==null){
                return 0;
            }
            else{
                return -1;
            }
        }
        if(o2==null){
            return 1;
        }
        //both are either taxonomic or media
        else if(o1.equals(OriginalSourceType.PrimaryTaxonomicSource)
                && o2.equals(OriginalSourceType.PrimaryMediaSource)){
            if(entity != null && entity.getType().equals(OriginalSourceType.PrimaryMediaSource)){
                return 1;
            }
            else{
                return -1;
            }
        }
        else if(o2.equals(OriginalSourceType.PrimaryTaxonomicSource)
                && o1.equals(OriginalSourceType.PrimaryMediaSource)){
            if(entity != null && entity.getType().equals(OriginalSourceType.PrimaryMediaSource)){
                return -1;
            }
            else{
                return 1;
            }
        }
        //one is not taxonomic or media
        else if(o1.equals(OriginalSourceType.PrimaryTaxonomicSource)
                ||o1.equals(OriginalSourceType.PrimaryMediaSource)){
            return -1;

        }
        else if(o2.equals(OriginalSourceType.PrimaryTaxonomicSource)
                || o2.equals(OriginalSourceType.PrimaryMediaSource)){
            return 1;
        }
        //The last two are always "Other" followed by "Unknown"
        else if(o1.equals(OriginalSourceType.Other)){
            if(o2.equals(OriginalSourceType.Unknown)){
                return -11;
            }
            else{
                return 1;
            }
        }
        if(o2.equals(OriginalSourceType.Other)){
            return 1;
        }
        else{
            String message1 = o1.getKey();
            String message2 = o2.getKey();
            return message1.compareTo(message2);
        }
    }

}