/**
 * Copyright (C) 2015 EDIT
 * European Distributed Institute of Taxonomy
 * http://www.e-taxonomy.eu
 *
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */
package eu.etaxonomy.taxeditor.ui.dialog;

import java.io.IOException;
import java.net.SocketTimeoutException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.commons.lang3.StringUtils;
import org.apache.log4j.Logger;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.core.runtime.jobs.Job;
import org.eclipse.core.runtime.preferences.ConfigurationScope;
import org.eclipse.core.runtime.preferences.IEclipsePreferences;
import org.eclipse.jface.layout.GridDataFactory;
import org.eclipse.swt.SWT;
import org.eclipse.swt.SWTException;
import org.eclipse.swt.custom.StyledText;
import org.eclipse.swt.events.KeyAdapter;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Dialog;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.forms.events.ExpansionEvent;
import org.eclipse.ui.forms.events.IExpansionListener;
import org.eclipse.ui.forms.widgets.ExpandableComposite;
import org.eclipse.wb.swt.SWTResourceManager;
import org.osgi.service.prefs.BackingStoreException;
import org.osgi.service.prefs.Preferences;
import org.springframework.security.authentication.BadCredentialsException;
import org.springframework.security.authentication.LockedException;
import org.springframework.security.authentication.UsernamePasswordAuthenticationToken;

import eu.etaxonomy.cdm.api.application.CdmApplicationRemoteConfiguration;
import eu.etaxonomy.cdm.api.application.CdmApplicationState;
import eu.etaxonomy.cdm.api.service.IUserService;
import eu.etaxonomy.cdm.common.CdmUtils;
import eu.etaxonomy.cdm.config.CdmSourceException;
import eu.etaxonomy.cdm.config.ICdmSource;
import eu.etaxonomy.cdm.model.metadata.CdmMetaData;
import eu.etaxonomy.cdm.persistence.permission.Role;
import eu.etaxonomy.taxeditor.l10n.Messages;
import eu.etaxonomy.taxeditor.model.MessagingUtils;
import eu.etaxonomy.taxeditor.preference.IPreferenceKeys;
import eu.etaxonomy.taxeditor.preference.PreferencesUtil;
import eu.etaxonomy.taxeditor.remoting.server.CdmServerException;
import eu.etaxonomy.taxeditor.remoting.source.CdmRemoteLocalhostSource;
import eu.etaxonomy.taxeditor.remoting.source.CdmServerInfo;
import eu.etaxonomy.taxeditor.remoting.source.CdmServerInfo.CdmInstanceInfo;
import eu.etaxonomy.taxeditor.remoting.source.ICdmRemoteSource;
import eu.etaxonomy.taxeditor.store.CdmStore;
import eu.etaxonomy.taxeditor.store.LoginManager;
import eu.etaxonomy.taxeditor.store.internal.TaxeditorStorePlugin;
import eu.etaxonomy.taxeditor.workbench.datasource.ICdmServerError;

/**
 * @author cmathew
 * @date 20 Jan 2015
 */
public class RemotingLoginDialog extends Dialog implements ICdmServerError {

    private Logger logger = Logger.getLogger(getClass());

    public static final String DEFAULT_PASS = "00000"; //$NON-NLS-1$
    public static final String DEFAULT_USER = "admin"; //$NON-NLS-1$

    public final static String STATUS_AVAILABLE = Messages.RemotingLoginDialog_STATUS_AVAILABLE;
    public final static String STATUS_NOT_AVAILABLE = Messages.RemotingLoginDialog_STATUS_NOT_AVAILABLE;
    public final static String STATUS_STARTED = Messages.RemotingLoginDialog_STATUS_STARTED;
    public final static String STATUS_NOT_STARTED = Messages.RemotingLoginDialog_STATUS_NOT_STARTED;
    public final static String STATUS_RETRIEVING = Messages.RemotingLoginDialog_STATUS_RETRIEVING;
    public final static String STATUS_CHECKING_AVAILABILITY = Messages.RemotingLoginDialog_STATUS_CHECKING;
    public final static String STATUS_NO_INSTANCES = Messages.RemotingLoginDialog_STATUS_NO_INSTANCES_FOUND;
    public final static String STATUS_NOT_COMPATIBLE = Messages.RemotingLoginDialog_STATUS_NOT_COMPATIBLE;

    public final static String MESG_COMPATIBLE_EDITOR_OLD = Messages.RemotingLoginDialog_UPDATE_EDITOR;
    public final static String MESG_COMPATIBLE_SERVER_OLD = Messages.RemotingLoginDialog_CHOOSE_COMPATIBLE_CDM_SERVER;
    public final static String MESG_SCHEMA_MISSING = Messages.RemotingLoginDialog_SCHEMA_MISSING;
    public final static String MESG_NO_SCHEMA = Messages.RemotingLoginDialog_NO_SCHEMA;

    public final static String LABEL_CONNECT = Messages.RemotingLoginDialog_LABEL_CONNECT;
    public final static String LABEL_CREATE_SCHEMA = Messages.RemotingLoginDialog_LABEL_CREATE_SCHEMA;
    public final static String LABEL_UPDATE_SCHEMA_VERSION = Messages.RemotingLoginDialog_LABEL_UPDATE_SCHEMA_VERSION;
    public final static String MESG_UPDATE_SCHEMA_VERSION = Messages.RemotingLoginDialog_MSG_UPDATE_SCHEMA_VERSION;
    public final static String LABEL_LOGIN = Messages.RemotingLoginDialog_LABEL_LOGIN;
    public final static String LABEL_CDM_SERVER = Messages.RemotingLoginDialog_LABEL_CDM_SERVER;
    public final static String LABEL_REFRESH = Messages.RemotingLoginDialog_LABEL_REFRESH;
    public final static String LABEL_CDM_INSTANCE = Messages.RemotingLoginDialog_LABEL_CDM_INSTANCE;
    public final static String LABEL_LOGIN_COLON = Messages.RemotingLoginDialog_LABEL_LOGIN_COLON;
    public final static String LABEL_PASSWORD = Messages.RemotingLoginDialog_LABEL_PASSWORD;
    public final static String LABEL_EDITOR_CDM_VERSION = Messages.RemotingLoginDialog_LABEL_EDITOR_CDM_VERSION;
    public final static String LABEL_SERVER_CDM_VERSION = Messages.RemotingLoginDialog_LABEL_SERVER_CDM_VERSION;
    public final static String LABEL_EDITOR_CDMLIB_VERSION = Messages.RemotingLoginDialog_LABEL_EDITOR_CDMLIB_VERSION;
    public final static String LABEL_SERVER_CDMLIB_VERSION = Messages.RemotingLoginDialog_LABEL_SERVER_CDMLIB_VERSION;
    public final static String LABEL_STOP_MANAGED_SERVER = Messages.RemotingLoginDialog_LABEL_STOP_MANAGED_SERVER;
    public final static String LABEL_PORT = Messages.RemotingLoginDialog_LABEL_PORT;
    public final static String LABEL_ADVANCED = Messages.RemotingLoginDialog_LABEL_ADVANCED;
    public final static String LABEL_DEFAULT_LOGIN = Messages.RemotingLoginDialog_DEFAULT_LOGIN;
    public final static String LABEL_REMEMBER_ME = Messages.RemotingLoginDialog_LABEL_REMEMBER_ME;

    public final static String MESG_LOGIN_CANNOT_BE_EMPTY = Messages.RemotingLoginDialog_LOGIN_CANNOT_BE_EMPTY;
    public final static String STATUS_GENERATING_CONFIG_FILE = Messages.RemotingLoginDialog_GENERATING_CONFIG_FILE;
    public final static String STATUS_STARTING_MGD_SERVER = Messages.RemotingLoginDialog_STARTING_MGD_SERVER;

    public final static String ERR_GENERATING_CONFIG_FILE = Messages.RemotingLoginDialog_ERROR_GENERATING_CONFIG_FILE;
    public final static String ERR_STARTING_SERVER = Messages.RemotingLoginDialog_ERROR_STARTING_SERVER;
    public final static String ERR_SERVER_LAUNCH = Messages.RemotingLoginDialog_SERVER_LAUNCH_ERROR;
    public final static String ERR_STOPPING_SERVER = Messages.RemotingLoginDialog_ERROR_STOPPING_SERVER;
    public final static String ERR_COULD_NOT_STOP_SERVER = Messages.RemotingLoginDialog_COULD_NOT_STOP_SERVER;
    public final static String RETRIEVE_SERVER_INSTANCES = Messages.RemotingLoginDialog_RETRIEVE_SERVER_INSTANCES;
    public final static String TASK_LAUNCHING_SERVER = Messages.RemotingLoginDialog_TASK_LAUNCHING_SERVER;

    public static final int BTN_COLOR_ATTENTION = SWT.COLOR_RED;

    public final static String STORE_PREFERENCES_NODE = "eu.etaxonomy.taxeditor.store"; //$NON-NLS-1$

    public final static String LOGIN_NODE = "login"; //$NON-NLS-1$
    public final static String USERNAME_SUFFIX = "_username"; //$NON-NLS-1$
    public final static String PASSWORD_SUFFIX = "_password"; //$NON-NLS-1$

    public final static String LAST_SERVER_INSTANCE_NODE = "lastServerInstance"; //$NON-NLS-1$
    public final static String LAST_SERVER_KEY = "lastServerKey"; //$NON-NLS-1$
    public final static String LAST_INSTANCE_KEY = "lastInstanceKey"; //$NON-NLS-1$

    protected Shell shlConnect;
    protected Text txtCdmServerStatus;
    protected Text txtCdmInstanceStatus;
    protected Combo comboCdmServer;
    protected Combo comboCdmInstance;
    protected Button btnConnect;

    protected Composite remotingComposite;
    protected Button btnCdmServerRefresh;
    protected Button btnCdmInstanceRefresh;
    protected Button btnStopServer;
    protected Composite loginComposite;
    protected Label lblLogin;
    protected Text txtLogin;
    protected Label lblPassword;
    protected Text txtPassword;
    protected Button btnRememberMe;
    protected Label lblDefaultLogin;
    protected Composite compAdvanced;
    protected Label lblPort;
    protected Text txtPort;
    protected Label lblServerCdmlibVersion;
    protected Text txtServerCdmlibVersion;
    protected ExpandableComposite expandableCompositeAdvanced;
    protected StyledText styledTxtMessage;

    protected Label lblEditorCdmlibVersion;
    protected Text txtEditorCdmlibVersion;
    protected Label lblServerSchemaVersion;
    protected Text txtServerSchemaVersion;
    protected Label lblEditorCdmVersion;
    protected Text txtEditorCdmVersion;

    protected final int MESSAGE_HEIGHT = 50;
    protected final int COMBO_MIN_WIDTH =200;
    protected final int CONTROLS_MIN_HEIGHT = 23; //does not seem to work yet
//  private final int MIN_EXP_HEIGHT = 380;

    protected boolean autoConnect = false;
    protected boolean loadLoginPrefs = true;
    protected boolean isDevRemoteSource = false;
    protected boolean isLocal = false;

    protected Job serverJob;
    protected Object result;

    protected final Map<String, CdmServerInfo> serverInfoMap = new HashMap<>();

    protected String serverName;
    protected String instanceName;
    protected CdmServerInfo selectedServerInfo;
    protected CdmInstanceInfo selectedCdmInstance;
    protected String login;
    protected String password;

    /**
     * Create the dialog.
     * @param parent
     * @param style
     */
    public RemotingLoginDialog(Shell parent, int style) {
        super(parent, style);
        setText(LABEL_LOGIN);
    }

    public Object open(ICdmRemoteSource source, boolean loadLoginPrefs, boolean autoConnect) {
        this.loadLoginPrefs = loadLoginPrefs;
        this.setServerName(source.getName());
        if (source instanceof CdmRemoteLocalhostSource){
            this.setInstanceName(((CdmRemoteLocalhostSource)source).getDatasourceName());
        }else{
            String contextPath = source.getContext();
            this.setInstanceName(contextPath == null? "" : contextPath.substring(contextPath.lastIndexOf("/") + 1)); //$NON-NLS-1$
        }
        return open(getServerName(), getInstanceName(), loadLoginPrefs, autoConnect);
    }

    public Object open(String serverName, String instanceName, boolean loadLoginPrefs, boolean autoConnect) {
        this.setServerName(serverName);
        this.setInstanceName(instanceName);
        this.loadLoginPrefs = loadLoginPrefs;
        this.setAutoConnect(autoConnect);
        return open();
    }

    /**
     * Open the dialog.
     * @return the dialog result
     */
    public Object open() {

        createContents();
        if(getServerName() == null && getInstanceName() == null) {
            readPrefLastServerInstance();
        }
        expandableCompositeAdvanced.setExpanded(false);

        setEditorInfo();
        populateCdmServerCombo();
        shlConnect.pack(true);

        setCenterPoint();
        shlConnect.open();

        Display display = getParent().getDisplay();
        while (!shlConnect.isDisposed()) {
            if (!display.isDisposed()){
                if (!display.readAndDispatch()) {
                    display.sleep();
                }
            }
        }
        return result;
    }

    /**
     * Create contents of the dialog.
     */
    protected void createContents() {
        shlConnect = new Shell(getParent(), SWT.DIALOG_TRIM | SWT.APPLICATION_MODAL | SWT.RESIZE | SWT.CENTER);
        shlConnect.setText(LABEL_CONNECT);
        shlConnect.setLayout(new FillLayout(SWT.HORIZONTAL));

        remotingComposite = new Composite(shlConnect, SWT.NONE);
        remotingComposite.setLayout(new GridLayout(1, false));

        //server
        Composite cdmServerComposite = new Composite(remotingComposite, SWT.NONE);
        GridData gd_cdmServerComposite = new GridData(SWT.LEFT, SWT.CENTER, false, false, 1, 1);
        cdmServerComposite.setLayoutData(gd_cdmServerComposite);
        cdmServerComposite.setLayout(new GridLayout(4, false));

        Label lblCdmServer = new Label(cdmServerComposite, SWT.NONE);
        lblCdmServer.setText(LABEL_CDM_SERVER);
        lblCdmServer.setLayoutData(new GridData(SWT.RIGHT, SWT.CENTER, false, false, 1, 1));

        comboCdmServer = new Combo(cdmServerComposite, SWT.READ_ONLY);
        comboCdmServer.addSelectionListener(new SelectionAdapter() {
            @Override
            public void widgetSelected(SelectionEvent e) {
                onRefreshCdmServer();
            }
        });

        GridData gd_comboCdmServer = new GridData(SWT.FILL, SWT.CENTER, true, true, 1, 1);

        comboCdmServer.setLayoutData(gd_comboCdmServer);
        comboCdmServer.select(0);

        txtCdmServerStatus = new Text(cdmServerComposite, SWT.BORDER);
        txtCdmServerStatus.setBackground(SWTResourceManager.getColor(SWT.COLOR_INFO_BACKGROUND));
        txtCdmServerStatus.setEditable(false);
        GridData gd_txtCdmServerStatus = new GridData(SWT.FILL, SWT.CENTER, true, true, 1, 1);
        gd_txtCdmServerStatus.minimumHeight = CONTROLS_MIN_HEIGHT;
        txtCdmServerStatus.setLayoutData(gd_txtCdmServerStatus);

        btnCdmServerRefresh = new Button(cdmServerComposite, SWT.NONE);
        btnCdmServerRefresh.addSelectionListener(new SelectionAdapter() {
            @Override
            public void widgetSelected(SelectionEvent e) {
                onRefreshCdmServer();
            }
        });
        GridData gd_btnCdmServerRefresh = new GridData(SWT.FILL, SWT.CENTER, true, true, 1, 1);
        gd_btnCdmServerRefresh.minimumHeight = CONTROLS_MIN_HEIGHT;
        btnCdmServerRefresh.setLayoutData(gd_btnCdmServerRefresh);
        btnCdmServerRefresh.setText(LABEL_REFRESH);

        //instance
        Label lblCdmInstance = new Label(cdmServerComposite, SWT.NONE);
        GridData gd_lblCdmInstance = new GridData(SWT.RIGHT, SWT.CENTER, false, true, 1, 1);
        //gd_lblCdmInstance.heightHint = 30;
        lblCdmInstance.setLayoutData(gd_lblCdmInstance);
        lblCdmInstance.setText(LABEL_CDM_INSTANCE);

        comboCdmInstance = new Combo(cdmServerComposite, SWT.READ_ONLY);
        comboCdmInstance.addSelectionListener(new SelectionAdapter() {
            @Override
            public void widgetSelected(SelectionEvent e) {
                onRefreshCdmInstance();
            }
        });

        GridData gd_comboCdmInstance = new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1);
        gd_comboCdmInstance.minimumWidth = COMBO_MIN_WIDTH;

        comboCdmInstance.setLayoutData(gd_comboCdmInstance);
        comboCdmInstance.select(0);

        txtCdmInstanceStatus = new Text(cdmServerComposite, SWT.BORDER);
        txtCdmInstanceStatus.setBackground(SWTResourceManager.getColor(SWT.COLOR_INFO_BACKGROUND));
        txtCdmInstanceStatus.setEditable(false);
        GridData gd_txtCdmInstanceStatus = new GridData(SWT.FILL, SWT.CENTER, true, true, 1, 1);
        gd_txtCdmInstanceStatus.minimumHeight = CONTROLS_MIN_HEIGHT;
        txtCdmInstanceStatus.setLayoutData(gd_txtCdmInstanceStatus);

        btnCdmInstanceRefresh = new Button(cdmServerComposite, SWT.FLAT);
        btnCdmInstanceRefresh.addSelectionListener(new SelectionAdapter() {
            @Override
            public void widgetSelected(SelectionEvent e) {
                onRefreshCdmInstance();
            }
        });
        GridData gd_btnCdmInstanceRefresh = new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1);
        gd_btnCdmInstanceRefresh.minimumHeight = CONTROLS_MIN_HEIGHT;
        btnCdmInstanceRefresh.setLayoutData(gd_btnCdmInstanceRefresh);
        btnCdmInstanceRefresh.setText(Messages.RemotingLoginDialog_LABEL_REFRESH);

        //login + connect
        loginComposite = new Composite(remotingComposite, SWT.NONE);
        GridData gd_loginComposite = new GridData(SWT.LEFT, SWT.CENTER, false, false, 1, 1);
        loginComposite.setLayoutData(gd_loginComposite);
        GridLayout gl_loginComposite = new GridLayout(6, false);
        gl_loginComposite.marginTop = 5;
        loginComposite.setLayout(gl_loginComposite);

        lblLogin = new Label(loginComposite, SWT.CENTER);
        GridData gd_lblLogin = new GridData(SWT.RIGHT, SWT.CENTER, false, false, 1, 1);
        lblLogin.setLayoutData(gd_lblLogin);
        lblLogin.setText(Messages.RemotingLoginDialog_LABEL_LOGIN_COLON);

        txtLogin = new Text(loginComposite, SWT.BORDER);
        GridData gd_txtLogin = new GridData(SWT.FILL, SWT.FILL, true, false, 1, 1);
        gd_txtLogin.minimumWidth = 80;
        txtLogin.setLayoutData(gd_txtLogin);

        lblPassword = new Label(loginComposite, SWT.CENTER);
        lblPassword.setLayoutData(new GridData(SWT.RIGHT, SWT.CENTER, false, false, 1, 1));
        lblPassword.setText(Messages.RemotingLoginDialog_LABEL_PASSWORD);

        txtPassword = new Text(loginComposite, SWT.BORDER | SWT.PASSWORD);
        GridData gd_txtPassword = new GridData(SWT.FILL, SWT.FILL, true, false, 1, 1);
        gd_txtPassword.minimumWidth = 80;
        txtPassword.setLayoutData(gd_txtPassword);
        @SuppressWarnings("unused")
        Label nope = new Label(loginComposite, SWT.NONE);
        txtPassword.addKeyListener(new KeyAdapter() {
        	@Override
        	public void keyPressed(KeyEvent e) {
        		if(e.character==SWT.CR){
        		    if(btnConnect.getText().equals(LABEL_CONNECT)){
                        onConnectButtonPressed();
                    }
        		}
        	}
        });

        btnConnect = new Button(loginComposite, SWT.FLAT);
        btnConnect.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, false, 1, 1));
        btnConnect.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				onConnectButtonPressed();
			}
		});
        btnConnect.setText(LABEL_CONNECT);

        //rememberMe + default values
        btnRememberMe = new Button(loginComposite, SWT.CHECK);
        btnRememberMe.setSelection(true);
        GridData gd_btnRememberMe = new GridData(SWT.LEFT, SWT.CENTER, false, false, 2, 1);
        btnRememberMe.setLayoutData(gd_btnRememberMe);
        btnRememberMe.setText(Messages.RemotingLoginDialog_LABEL_REMEMBER_ME);

        lblDefaultLogin = new Label(loginComposite, SWT.NONE);
        GridDataFactory.fillDefaults().span(4, 1).align(SWT.TRAIL, SWT.CENTER).applyTo(lblDefaultLogin);
        lblDefaultLogin.setText(String.format(Messages.RemotingLoginDialog_DEFAULT_LOGIN, DEFAULT_USER, DEFAULT_PASS));
        lblDefaultLogin.setVisible(false);

        styledTxtMessage = new StyledText(remotingComposite, SWT.NONE);
        styledTxtMessage.setBackground(SWTResourceManager.getColor(SWT.COLOR_INFO_BACKGROUND));
        styledTxtMessage.setForeground(SWTResourceManager.getColor(SWT.COLOR_DARK_RED));
        styledTxtMessage.setSelectionBackground(SWTResourceManager.getColor(SWT.COLOR_LIST_SELECTION_TEXT));
        styledTxtMessage.setSelectionForeground(SWTResourceManager.getColor(SWT.COLOR_DARK_RED));
        styledTxtMessage.setDoubleClickEnabled(false);
        styledTxtMessage.setEditable(false);
        styledTxtMessage.setWordWrap(true);

        GridData gd_styledTxtMessage = new GridData(SWT.FILL, SWT.FILL, true, false, 1, 1);
        gd_styledTxtMessage.exclude = true;
        gd_styledTxtMessage.minimumHeight = MESSAGE_HEIGHT;

        styledTxtMessage.setLayoutData(gd_styledTxtMessage);

        //Advanced
        expandableCompositeAdvanced = new ExpandableComposite(remotingComposite, SWT.NONE, ExpandableComposite.TWISTIE);
        GridData gd_xpndblcmpstAdvanced = new GridData(SWT.FILL, SWT.FILL, false, true, 1, 1);
        expandableCompositeAdvanced.setLayoutData(gd_xpndblcmpstAdvanced);
        expandableCompositeAdvanced.addExpansionListener(new IExpansionListener() {
            @Override
            public void expansionStateChanged(ExpansionEvent e) {
                shlConnect.pack(true);
            }
            @Override
            public void expansionStateChanging(ExpansionEvent e) {
            }
        });
        expandableCompositeAdvanced.setText(Messages.RemotingLoginDialog_LABEL_ADVANCED);
        expandableCompositeAdvanced.setExpanded(true);

        compAdvanced = new Composite(expandableCompositeAdvanced, SWT.NONE);
        expandableCompositeAdvanced.setClient(compAdvanced);
        compAdvanced.setLayout(new GridLayout(4, false));

        lblPort = new Label(compAdvanced, SWT.CENTER);
        lblPort.setLayoutData(new GridData(SWT.RIGHT, SWT.CENTER, false, false, 1, 1));
        lblPort.setText(Messages.RemotingLoginDialog_LABEL_PORT);

        txtPort = new Text(compAdvanced, SWT.BORDER);
        GridData gd_txtPort = new GridData(SWT.FILL, SWT.CENTER, false, false, 1, 1);
        gd_txtPort.minimumWidth = 50;
        txtPort.setLayoutData(gd_txtPort);

        lblServerCdmlibVersion = new Label(compAdvanced, SWT.CENTER);
        lblServerCdmlibVersion.setLayoutData(new GridData(SWT.RIGHT, SWT.CENTER, false, false, 1, 1));
        lblServerCdmlibVersion.setText(Messages.RemotingLoginDialog_LABEL_SERVER_CDMLIB_VERSION);

        txtServerCdmlibVersion = new Text(compAdvanced, SWT.BORDER);
        txtServerCdmlibVersion.setEditable(false);
        txtServerCdmlibVersion.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, false, false, 1, 1));
/*
        btnStopServer = new Button(compAdvanced, SWT.FLAT);
        btnStopServer.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, false, 2, 1));
        btnStopServer.addMouseListener(new MouseAdapter() {
            @Override
            public void mouseUp(MouseEvent e) {
                onStopManagedServer();
            }
        });
        btnStopServer.setText(Messages.RemotingLoginDialog_LABEL_STOP_MANAGED_SERVER);
*/
        lblEditorCdmlibVersion = new Label(compAdvanced, SWT.CENTER);
        lblEditorCdmlibVersion.setLayoutData(new GridData(SWT.RIGHT, SWT.CENTER, false, false, 1, 1));
        lblEditorCdmlibVersion.setText(Messages.RemotingLoginDialog_LABEL_EDITOR_CDMLIB_VERSION);

        txtEditorCdmlibVersion = new Text(compAdvanced, SWT.BORDER);
        txtEditorCdmlibVersion.setEditable(false);
        txtEditorCdmlibVersion.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
        addEmptyCells(2);

        lblServerSchemaVersion = new Label(compAdvanced, SWT.CENTER);
        lblServerSchemaVersion.setLayoutData(new GridData(SWT.RIGHT, SWT.CENTER, false, false, 1, 1));
        lblServerSchemaVersion.setText(Messages.RemotingLoginDialog_LABEL_SERVER_CDM_VERSION);

        txtServerSchemaVersion = new Text(compAdvanced, SWT.BORDER);
        txtServerSchemaVersion.setEditable(false);
        txtServerSchemaVersion.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
        addEmptyCells(2);

        lblEditorCdmVersion = new Label(compAdvanced, SWT.CENTER);
        lblEditorCdmVersion.setLayoutData(new GridData(SWT.RIGHT, SWT.CENTER, false, false, 1, 1));
        lblEditorCdmVersion.setText(Messages.RemotingLoginDialog_LABEL_EDITOR_CDM_VERSION);

        txtEditorCdmVersion = new Text(compAdvanced, SWT.BORDER);
        txtEditorCdmVersion.setEditable(false);
        txtEditorCdmVersion.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
    }

    public void setCenterPoint() {
        Rectangle parentSize = getParent().getBounds();
        Rectangle shellSize = shlConnect.getBounds();
        int locationX = (parentSize.width - shellSize.width)/2+parentSize.x;
        int locationY = (parentSize.height - shellSize.height)/2+parentSize.y;
        shlConnect.setLocation(new Point(locationX, locationY));

    }

    @SuppressWarnings("unused")
    private void addEmptyCells(int cnt) {
        for (int i = 0; i < cnt; i++) {
            new Label(compAdvanced, SWT.NONE);
        }
    }

	protected void onConnectButtonPressed() {
		/*if(selectedServerInfo.isLocalhostMgd() && !isSelectedCdmInstanceRunningInManagedServer()) {
			startManagedServer();
		} else {*/
			connect();
			if (CdmStore.isActive() && CdmStore.currentAuthentiationHasOneOfRoles(Role.ROLE_REMOTING)){
			    PreferencesUtil.setNomenclaturalCodePreferences();
			}
//		}
	}

    protected void populateCdmServerCombo() {
        Job job = new Job(RETRIEVE_SERVER_INSTANCES) {
            @Override
            protected IStatus run(IProgressMonitor monitor) {
                Display.getDefault().syncExec(new Runnable() {
                    @Override
                    public void run() {
                        //TODO performance
                        putAllServers();
                        int serverIndex = -1;
                        if(getServerName() != null) {
                            serverIndex = comboCdmServer.indexOf(getServerName());
                        }
                        if(serverIndex == -1) {
                            comboCdmServer.select(0);
                            setAutoConnect(false);
                        } else {
                            comboCdmServer.select(serverIndex);
                        }
                        ICdmSource devRemoteSource = CdmServerInfo.getDevServerRemoteSource();
                        if(devRemoteSource != null) {
                            setDevRemoteSource(true);
                            String username = System.getProperty("cdm.server.dev.username"); //$NON-NLS-1$
                            String password = System.getProperty("cdm.server.dev.password"); //$NON-NLS-1$
                            if(username != null && !username.isEmpty() && password != null && !password.isEmpty()) {
                                txtLogin.setText(username);
                                txtPassword.setText(password);
                                CdmStore.connect(devRemoteSource, RemotingLoginDialog.this);
                            }
                        } else {
                            onRefreshCdmServer();
                        }
                    }


                });
                return Status.OK_STATUS;
            }
        };
        job.schedule();
    }
    public void putAllServers() {
        for(CdmServerInfo csii : CdmServerInfo.getCdmServers(isLocal)) {
            putServerInfo(csii);
            comboCdmServer.add(csii.getName());
        }
    }

    public void putServerInfo(CdmServerInfo csii) {
        getServerInfoMap().put(csii.getName(), csii);
    }
    public void onRefreshCdmServer() {
        try{
            txtCdmServerStatus.setText(STATUS_CHECKING_AVAILABILITY);
            clearOnServerChange();
            emptyCredentials();
            updateSelectedCdmServer();
            Display.getDefault().asyncExec(new Runnable() {
                @Override
                public void run() {
                    checkSelectedCdmServer();
                }
            });
        }
        catch(SWTException e){
            //catch widget is disposed exception which may occur if
            //dialog is closed but the runnable tries to update
        }
    }

    protected void updateSelectedCdmServer() {
        int selIndex = comboCdmServer.getSelectionIndex();
        if(selIndex != -1) {
            selectedServerInfo = getServerInfoMap().get(comboCdmServer.getItem(selIndex));
        }
    }

    protected void updatePort() {
        txtPort.setText(""); //$NON-NLS-1$
        if(selectedServerInfo != null) {
            int port = selectedServerInfo.getPort();
            if(port == CdmServerInfo.NULL_PORT) {
                txtPort.setText(CdmServerInfo.NULL_PORT_STRING);
            } else {
                txtPort.setText(String.valueOf(port));
            }
        }
    }

    protected int getPort() {
        int port = CdmServerInfo.NULL_PORT;
        try {
            port = Integer.valueOf(txtPort.getText());
        } catch (NumberFormatException nfe) {
            if(!CdmServerInfo.NULL_PORT_STRING.equals(txtPort.getText())) {
                setMessage(Messages.RemotingLoginDialog_MESSAGE_PORT_SHOULD_BE_INTEGER);
            }
        }
        return port;
    }

    private void checkSelectedCdmServer() {
        try{
            if(selectedServerInfo != null) {
                if(selectedServerInfo.isLocalhost()) {
                    txtPort.setEditable(true);
                    txtPort.setEnabled(true);
                }
                try {
                    if(selectedServerInfo.pingServer()) {
                        txtCdmServerStatus.setText(STATUS_AVAILABLE);
                        populateCdmInstanceCombo(true);
                        if(selectedServerInfo.isLocalhost()){
                            setCdmlibVersion(txtServerCdmlibVersion);
                        }else{
                            String serverVersionTimestamp = generateLastModifiedTooltip(selectedServerInfo.getCdmlibLastModified());
                            txtServerCdmlibVersion.setText(selectedServerInfo.getCdmlibServicesVersion() + ":" + serverVersionTimestamp); //$NON-NLS-1$
                        }
                    } else {
                        txtCdmServerStatus.setText(STATUS_NOT_AVAILABLE);
                        comboCdmInstance.removeAll();
                        disableCdmInstanceControls("", ""); //$NON-NLS-1$ //$NON-NLS-2$
                    }
                } catch (CdmServerException | IOException e) {
                    txtCdmServerStatus.setText(STATUS_NOT_AVAILABLE);
                    // TODO distinguish java.net.ConnectException: Connection refused, java.net.SocketTimeoutException: Read timed out
                    Throwable cause = e;
                    if(e instanceof CdmServerException && e.getCause() != null){
                        cause = e.getCause();
                    }
                    String message = Messages.RemotingLoginDialog_CONNECTION_FAILED_MESSAGE;
                    if(cause instanceof SocketTimeoutException){
                        message = Messages.RemotingLoginDialog_CONNECTION_TIMEOUT_MESSAGE;
                    }
                    MessagingUtils.warningDialog(Messages.RemotingLoginDialog_CONNECTION_FAILED_TITLE, this,
                            message);
                    logger.warn(Messages.RemotingLoginDialog_CONNECTION_FAILED_TITLE, e);
                }
            }
            updatePort();
        }
        catch(SWTException e){
            //catch widget is disposed exception which may occur if
            //dialog is closed but the runnable tries to update
        }
    }

    protected void populateCdmInstanceCombo(final boolean forceRefresh) {
        comboCdmInstance.removeAll();
        comboCdmInstance.setEnabled(false);
        btnConnect.setEnabled(false);
        txtCdmInstanceStatus.setText(STATUS_RETRIEVING);
        txtCdmInstanceStatus.setToolTipText(""); //$NON-NLS-1$

        serverJob = new Job(Messages.RemotingLoginDialog_RETRIEVE_SERVER_INSTANCES) {
            @Override
            protected IStatus run(IProgressMonitor monitor) {
                try {
                    if(selectedServerInfo != null) {
                        if(forceRefresh) {
                            selectedServerInfo.refreshInstances();
                        }
                        final List<CdmInstanceInfo> instances = selectedServerInfo.getInstances();
                        Display.getDefault().asyncExec(()->{
                            if(!instances.isEmpty()) {
                                for(CdmInstanceInfo cdmInstance : instances) {
                                    comboCdmInstance.add(cdmInstance.getName());
                                }
                                int instanceIndex = -1;
                                if(getInstanceName() != null) {
                                    instanceIndex = comboCdmInstance.indexOf(getInstanceName());
                                }
                                if(instanceIndex == -1) {
                                    comboCdmInstance.select(0);
                                    setAutoConnect(false);
                                } else {
                                    comboCdmInstance.select(instanceIndex);
                                }
                                onRefreshCdmInstance();
                                comboCdmInstance.setEnabled(true);
                                if(isAutoConnect()) {
                                    connect();
                                }
                            } else {
                                txtCdmInstanceStatus.setText(STATUS_NO_INSTANCES);
                                btnConnect.setEnabled(false);
                            }
                        });
                    }
                } catch (final CdmServerException e) {
                    MessagingUtils.warn(getClass(), e);
                    Display.getDefault().asyncExec(new Runnable() {
                        @Override
                        public void run() {
                            disableCdmInstanceControls(STATUS_NOT_AVAILABLE, e.getMessage());
                        }
                    });
                }
                return Status.OK_STATUS;
            }
        };

        if(txtCdmServerStatus.getText().equals(STATUS_AVAILABLE) && !isDevRemoteSource()) {
            // Start the Job
            serverJob.schedule();
        }
    }

    public void disableCdmInstanceControls(String cdmInstanceStatus, String tooltip) {
        txtCdmInstanceStatus.setText(cdmInstanceStatus);
        txtCdmInstanceStatus.setToolTipText(tooltip);
        comboCdmInstance.setEnabled(false);
        btnConnect.setEnabled(false);
    }

    public void onRefreshCdmInstance() {
        txtCdmInstanceStatus.setText(STATUS_CHECKING_AVAILABILITY);
        clearOnInstanceChange();
        updateSelectedCdmInstance();
        checkSelectedCdmInstance();
//        updateManagedServerControls();
    }

    protected void updateSelectedCdmInstance() {
        int selIndex = comboCdmInstance.getSelectionIndex();
        if(selIndex != -1) {
            selectedCdmInstance = selectedServerInfo.getInstanceFromName(comboCdmInstance.getItem(selIndex));
            if(loadLoginPrefs) {
                readPrefCredentials();
            }
        }
    }
/*
    private void updateManagedServerControls() {
        if(selectedServerInfo.isLocalhostMgd()) {
            if(isSelectedCdmInstanceRunningInManagedServer()) {
                txtCdmInstanceStatus.setText(STATUS_STARTED);
            } else {
                txtCdmInstanceStatus.setText(STATUS_NOT_STARTED);
            }
            btnConnect.setEnabled(true);
            selectedServerInfo.setPort(getManagedServerPort());
            updatePort();
        }

        if(isManagedServerRunning()) {
            btnStopServer.setEnabled(true);
        } else {
            btnStopServer.setEnabled(false);
        }
    }

    private boolean isManagedServerRunning() {
        return CdmStore.getManagedServer() != null && CdmStore.getManagedServer().isAlive();
    }

    private boolean isSelectedCdmInstanceRunningInManagedServer() {
        return CdmStore.getManagedServer() != null &&
                CdmStore.getManagedServer().isAlive() &&
                selectedServerInfo.isLocalhostMgd() &&
                CdmStore.getManagedServer().getDataSourceName().equals(selectedCdmInstance.getName());
    }

    private void startManagedServer() {
        /*if(isManagedServerRunning()) {
            if(CdmStore.getManagedServer().getDataSourceName().equals(selectedCdmInstance.getName())) {
                return;
            } else {
                Display.getDefault().syncExec(new Runnable() {
                    @Override
                    public void run() {
                        onStopManagedServer();
                    }
                });
            }
        }
        boolean forceSchemaCreate = !schemaExists(selectedCdmInstance);
        boolean forceSchemaUpdate = LABEL_UPDATE_SCHEMA_VERSION.equals(btnConnect.getText());

        Job job = new Job(Messages.RemotingLoginDialog_JOB_SERVER_LAUNCH) {

            @Override
            public IStatus run(IProgressMonitor monitor) {
                String mgdServerConfigFileName = "mgd.datasources.xml"; //$NON-NLS-1$
                String config = CdmServerUtils.convertEditorToServerConfig();
                int maxUnits = 50;
                monitor.beginTask(Messages.RemotingLoginDialog_TASK_LAUNCHING_SERVER, maxUnits);
                try {
                    monitor.subTask(String.format(Messages.RemotingLoginDialog_GENERATING_CONFIG_FILE, selectedCdmInstance.getName()));
                    File managedServerConfigFile = CdmServerUtils.writeManagedServerConfig(config, mgdServerConfigFileName);
                    monitor.worked(1);
                    CdmServer cdmServer = new CdmServer(selectedCdmInstance.getName(), managedServerConfigFile);
                    cdmServer.setForceSchemaCreate(forceSchemaCreate);
                    cdmServer.setForceSchemaUpdate(forceSchemaUpdate);
                    CdmStore.setManagedServer(cdmServer);
                    monitor.subTask(Messages.RemotingLoginDialog_STARTING_MGD_SERVER);
                    cdmServer.start(false, RemotingLoginDialog.this);
                    int serverUnits = 0;

                    // the following loop is a 'fake' progress monitoring where the progress
                    // bar is advanced by one unit every second until maxUnits -2
                    while(!CdmStore.getManagedServer().isStarted() && !CdmStore.getManagedServer().isFailed()) {
                        if(serverUnits < maxUnits - 2) {
                            try {
                                Thread.sleep(1000);
                            } catch (InterruptedException e) {
                            }
                            monitor.worked(1);
                            serverUnits++;
                        }
                    }
                    Display.getDefault().syncExec(()->{
                                hide(false);
                                updateManagedServerControls();
                                connect();
                    });
                } catch (IOException ioe) {
                    MessagingUtils.errorDialog(Messages.RemotingLoginDialog_ERROR_GENERATING_CONFIG_FILE,
                            this,
                            ioe.getMessage(),
                            TaxeditorStorePlugin.PLUGIN_ID,
                            ioe,
                            true);
                } catch (CdmEmbeddedServerException cese) {
                    MessagingUtils.errorDialog(Messages.RemotingLoginDialog_ERROR_STARTING_SERVER,
                            this,
                            cese.getMessage(),
                            TaxeditorStorePlugin.PLUGIN_ID,
                            cese,
                            true);
                } finally {
                    monitor.done();
                }
                // NOTE: Errors thrown during server startup are passed to the handleError() implementation
                // TODO: is the above catch clause for CdmEmbeddedServerException still valuable?

                return Status.OK_STATUS;
            }
       };

        // configure the job
        job.setProperty(IProgressConstants.KEEP_PROPERTY, true);
        job.setUser(true);
        // schedule job
        hide(true);
        job.schedule();
    }
    */
    private boolean schemaExists(CdmInstanceInfo instanceInfo) {
        if (instanceInfo.getDataSource() != null){
            try {
                //TODO we need a new method in cdmlib to check for existing schema
                String schemaVersion = instanceInfo.getDataSource().getDbSchemaVersion();
                return schemaVersion != null;
            } catch (CdmSourceException e) {
                return false;
            }
        }else{
            return true;
        }
    }

    private String managedSchemaVersion(CdmInstanceInfo instanceInfo) throws CdmSourceException{
        if (instanceInfo.getDataSource() != null){
            return instanceInfo.getDataSource().getDbSchemaVersion();
        }else{
            throw new RuntimeException("ManagedSchemaVersion only available for managed server instances.");
        }
    }
    private int compareSchemaVersion(CdmInstanceInfo instanceInfo) throws CdmSourceException {
        return CdmMetaData.compareVersion(managedSchemaVersion(instanceInfo), CdmMetaData.getDbSchemaVersion(), 3, null);
    }
/*
    @Override
    public void handleError(final Throwable t) {

        Display.getDefault().syncExec(()-> {
            serverJob.cancel();

            String title = Messages.RemotingLoginDialog_SERVER_LAUNCH_ERROR;
            String  message = t.getMessage();

            MessagingUtils.errorDialog(title,
                    this,
                    message,
                    TaxeditorStorePlugin.PLUGIN_ID,
                    t,
                    true);
        });
    }

    private void onStopManagedServer() {
        try {
            CdmStore.getManagedServer().stop();
        } catch (Exception e) {
            MessagingUtils.errorDialog(Messages.RemotingLoginDialog_ERROR_STOPPING_SERVER,
                    this,
                    String.format(Messages.RemotingLoginDialog_COULD_NOT_STOP_SERVER, CdmStore.getManagedServer().getPort()),
                    TaxeditorStorePlugin.PLUGIN_ID,
                    e,
                    true);
        }
        CdmStore.setManagedServer(null);
        updateManagedServerControls();
    }

    private int getManagedServerPort() {
        return CdmStore.getManagedServer() == null ? CdmServerInfo.NULL_PORT : CdmStore.getManagedServer().getPort();
    }
*/
    protected void checkSelectedCdmInstance() {
        try {
            if (selectedCdmInstance != null && selectedCdmInstance.getDataSource() != null){
                String schemaVersion = selectedCdmInstance.getDataSource().getDbSchemaVersion();
            }
        } catch (CdmSourceException e1) {
            // TODO Auto-generated catch block
            e1.printStackTrace();
        }
       if (txtCdmServerStatus.getText().equals(STATUS_AVAILABLE)) {

            boolean available = false;
            String instanceStatus = STATUS_NOT_AVAILABLE;
            String message = null;
            try {
                if(selectedServerInfo.pingInstance(selectedCdmInstance, getPort())) {
                    instanceStatus = STATUS_AVAILABLE;
                    available = true;
                } else {
                    instanceStatus = STATUS_NOT_AVAILABLE;
                    available = false;
                }

                if(available) {
                    txtServerSchemaVersion.setText(selectedServerInfo.getCdmRemoteSource(selectedCdmInstance, getPort()).getDbSchemaVersion());

                    int compareDbSchemaVersion = selectedServerInfo.compareDbSchemaVersion(selectedCdmInstance, getPort());

                    int compareCdmlibServicesVersion = 0;
                    boolean disableServicesApiTimestampCheck =
                            PreferencesUtil.getBooleanValue((IPreferenceKeys.DISABLE_SERVICES_API_TIMESTAMP_CHECK));
                    if(!disableServicesApiTimestampCheck) {
                        compareCdmlibServicesVersion = selectedServerInfo.compareCdmlibServicesVersion();
                    }

                    if(compareDbSchemaVersion > 0 || compareCdmlibServicesVersion > 0) {
                        instanceStatus = STATUS_NOT_COMPATIBLE;
                        available = false;
                        message = MESG_COMPATIBLE_EDITOR_OLD;
                    } else if(compareDbSchemaVersion < 0 || compareCdmlibServicesVersion < 0) {
                        instanceStatus = STATUS_NOT_COMPATIBLE;
                        available = false;
                        message = MESG_COMPATIBLE_SERVER_OLD;
                    } else {
                        instanceStatus =  STATUS_AVAILABLE;
                        available = true;
                        message = ""; //$NON-NLS-1$
                    }
                }
            } catch (Exception e) {
                txtCdmInstanceStatus.setToolTipText(e.getMessage());
            } finally {
                lblDefaultLogin.setVisible(false);
                btnConnect.setEnabled(available);
                txtCdmInstanceStatus.setText(instanceStatus);
                if(!StringUtils.isBlank(message)) {
                    setMessage(message);
                }
            }
        }
    }

//    private void checkManagedSelectedInstance() {
//        boolean available = false;
//        String instanceStatus = STATUS_NOT_AVAILABLE;
//        String message = null;
//        String connect = LABEL_CONNECT;
//        Integer color = null;
//        String schemaVersion = CdmMetaData.getDbSchemaVersion();
//
//        if (!schemaExists(selectedCdmInstance)){
//            message = MESG_SCHEMA_MISSING;
//            connect = LABEL_CREATE_SCHEMA;
//            color = BTN_COLOR_ATTENTION;
//            schemaVersion = MESG_NO_SCHEMA;
//        } else {
//            int compare;
//            try {
//                schemaVersion = managedSchemaVersion(selectedCdmInstance);
//                compare = compareSchemaVersion(selectedCdmInstance);
//                if (compare > 0){
//                    instanceStatus = STATUS_NOT_COMPATIBLE;
//                    message = MESG_COMPATIBLE_EDITOR_OLD;
//                    available = false;
//                }else if (compare < 0){
//                    instanceStatus = STATUS_NOT_COMPATIBLE;
//                    message = MESG_UPDATE_SCHEMA_VERSION;
//                    available = true;
//                    connect = LABEL_UPDATE_SCHEMA_VERSION;
//                    color = BTN_COLOR_ATTENTION;
//                }else{
//                    available = true;
//                    instanceStatus = STATUS_AVAILABLE;
//                }
//            } catch (CdmSourceException e) {
//                available = false;
//            }
//        }
//
//        setManagedValues(available, instanceStatus, connect, color, schemaVersion);
//        setMessage(message);
//    }
//
//    private void setManagedValues(boolean available, String instanceStatus, String connect, Integer color,
//            String schemaVersion) {
//        Display.getDefault().syncExec(()->{
//            btnConnect.setEnabled(available);
//            btnConnect.setText(connect);
//            Color systemColor = null;
//            if(color != null) {
//                systemColor = Display.getCurrent().getSystemColor(color);
//            }
//            btnConnect.setBackground(systemColor);
//            btnConnect.requestLayout();
//            txtCdmInstanceStatus.setText(instanceStatus);
//            txtServerSchemaVersion.setText(schemaVersion);
//            lblDefaultLogin.setVisible(true);
//         });
//    }

    public void connect() {
        checkSelectedCdmInstance();

        if(!txtCdmInstanceStatus.getText().equals(STATUS_AVAILABLE)) {
            return;
        }

        ICdmRemoteSource source = selectedServerInfo.getCdmRemoteSource(selectedCdmInstance, getPort());

        if(!validateLogin(source)) {
            return;
        }

        try {
            CdmStore.connect(source, this);
        } catch (Exception e) {
            // Do not expect anything to go wrong at this point, so we throw a runtime exception
            // if any problems
            throw new RuntimeException(e);
        }
    }

    public boolean isRememberMe() {
        return btnRememberMe.getSelection();
    }

    private void persistPrefLastServerInstance() {
        IEclipsePreferences preferences = ConfigurationScope.INSTANCE.getNode(STORE_PREFERENCES_NODE);
        Preferences lastServerInstancePrefs = preferences.node(LAST_SERVER_INSTANCE_NODE);

        lastServerInstancePrefs.put(LAST_SERVER_KEY, selectedServerInfo.getName());
        lastServerInstancePrefs.put(LAST_INSTANCE_KEY, selectedCdmInstance.getName());

        flushPreferences(lastServerInstancePrefs);
    }

    private void persistPrefCredentials() {
         IEclipsePreferences preferences = ConfigurationScope.INSTANCE.getNode(STORE_PREFERENCES_NODE);
         Preferences credentialsPrefs = preferences.node(LOGIN_NODE);
         credentialsPrefs.put(getUsernamePrefKey(), txtLogin.getText());
         credentialsPrefs.put(getPasswordPrefKey(), txtPassword.getText());
         flushPreferences(credentialsPrefs);
    }

    private void removePrefCredentials() {
        IEclipsePreferences preferences = ConfigurationScope.INSTANCE.getNode(STORE_PREFERENCES_NODE);
        Preferences credentialsPrefs = preferences.node(LOGIN_NODE);
        credentialsPrefs.put(getUsernamePrefKey(), txtLogin.getText());
        credentialsPrefs.put(getPasswordPrefKey(), txtPassword.getText());
        removePreferences(credentialsPrefs);
    }

    private void removePreferences(Preferences prefs) {
        try {
            prefs.removeNode();
        } catch (BackingStoreException bse) {
            setMessage(bse.getMessage());
        }
    }

    private void flushPreferences(Preferences prefs) {
        try {
            prefs.flush();
        } catch (BackingStoreException bse) {
            setMessage(bse.getMessage());
        }
    }

    protected void readPrefCredentials() {
        String username, password;
        IEclipsePreferences preferences = ConfigurationScope.INSTANCE.getNode(STORE_PREFERENCES_NODE);
        Preferences credentialsPrefs = preferences.node(LOGIN_NODE);
        username = credentialsPrefs.get(getUsernamePrefKey(), ""); //$NON-NLS-1$
        txtLogin.setText(username);
        password = credentialsPrefs.get(getPasswordPrefKey(),""); //$NON-NLS-1$
        txtPassword.setText(password);
        if(username.isEmpty() || password.isEmpty()) {
            setAutoConnect(false);
        }
    }

    private void readPrefLastServerInstance() {
        IEclipsePreferences preferences = ConfigurationScope.INSTANCE.getNode(STORE_PREFERENCES_NODE);
        Preferences lastServerInstancePrefs = preferences.node(LAST_SERVER_INSTANCE_NODE);

        setServerName(lastServerInstancePrefs.get(LAST_SERVER_KEY, null));
        setInstanceName(lastServerInstancePrefs.get(LAST_INSTANCE_KEY, null));
    }

    private void emptyCredentials() {
        txtLogin.setText(""); //$NON-NLS-1$
        txtPassword.setText(""); //$NON-NLS-1$
    }

    private String getUsernamePrefKey() {
        return selectedServerInfo.toString(selectedCdmInstance.getName(), isDevRemoteSource()?getPort():-1) + USERNAME_SUFFIX;
    }

    private String getPasswordPrefKey() {
        return selectedServerInfo.toString(selectedCdmInstance.getName(), isDevRemoteSource()?getPort():-1) + PASSWORD_SUFFIX;
    }

    private boolean validateLogin(ICdmRemoteSource remoteSource) {
        if(getUsername() == null || getUsername().isEmpty()) {
            setMessage(Messages.RemotingLoginDialog_LOGIN_CANNOT_BE_EMPTY);
            return false;
        }
        if(getPassword() == null || getPassword().isEmpty()) {
            setMessage(Messages.RemotingLoginDialog_PASSWORD_CANNOT_BE_EMPTY);
            return false;
        }

        try {
            IUserService userService = CdmApplicationRemoteConfiguration.getUserService(remoteSource);
            UsernamePasswordAuthenticationToken token = new UsernamePasswordAuthenticationToken(getUsername(), getPassword());
            CdmApplicationRemoteConfiguration.getAuthenticationManager(userService).authenticate(token);
        } catch(BadCredentialsException e){
            setMessage(LoginManager.INCORRECT_CREDENTIALS_MESSAGE);
            return false;
        } catch(LockedException e){
            setMessage(LoginManager.ACCOUNT_LOCKED_MESSAGE);
            return false;
        } catch(IllegalArgumentException e){
            setMessage(LoginManager.INCORRECT_CREDENTIALS_MESSAGE);
            return false;
        }
        return true;
    }

    public String getUsername() {
        return txtLogin.getText();
    }

    public String getPassword() {
        return txtPassword.getText();
    }

    public void setMessage(String message) {
        if(CdmUtils.isNotBlank(message)) {
//            if(message.length() > 50) {
//                styledTxtMessage.setToolTipText(message);
//                message = message.substring(0,50) + "..."; //$NON-NLS-1$
//            }
            styledTxtMessage.setText(message);
            styledTxtMessage.setVisible(true);
            ((GridData)styledTxtMessage.getLayoutData()).exclude = false;
//            shlConnect.setMinimumSize(MIN_WIDTH, getHeightWithoutMessage() + MESSAGE_HEIGHT);
            shlConnect.pack(true);
        } else {
            if (!styledTxtMessage.isDisposed()){
                styledTxtMessage.setText(""); //$NON-NLS-1$
                styledTxtMessage.setVisible(false);
                ((GridData)styledTxtMessage.getLayoutData()).exclude = true;
            }
            if(!shlConnect.isDisposed()){
                shlConnect.pack(true);
            }
        }
        if (!remotingComposite.isDisposed()){
            remotingComposite.layout();
        }
    }

//    private int getHeightWithoutMessage() {
//        if(xpndblcmpstAdvanced.isExpanded()) {
//            return MIN_EXP_HEIGHT;
//        } else {
//            return MIN_HEIGHT;
//        }
//    }

    public void hide(boolean isHidden) {
        if (!shlConnect.isDisposed()){
            if(shlConnect != null && shlConnect.getDisplay() != null) {
                shlConnect.setVisible(!isHidden);
            }
        }
    }

    public void dispose() {
        if (!shlConnect.isDisposed()){
            if(shlConnect != null && shlConnect.getDisplay() != null) {
                shlConnect.dispose();
            }
        }
    }

    public void onComplete() {
        Display.getDefault().asyncExec(new Runnable() {
            @Override
            public void run() {
                if(selectedCdmInstance!=null){
                    if(isRememberMe()) {
                        persistPrefCredentials();
                    }else{
                        removePrefCredentials();
                    }
                    persistPrefLastServerInstance();
                }
                dispose();
            }
        });
    }

    private String generateLastModifiedTooltip(String cdmlibLastModified) {
        if(StringUtils.isBlank(cdmlibLastModified)) {
            return ""; //$NON-NLS-1$
        }
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy.MM.dd.HH.mm.z"); //$NON-NLS-1$
        Date cdmlibLastModifiedDate;
        String cdmlibLastModifiedTimestamp = ""; //$NON-NLS-1$

        cdmlibLastModifiedDate = new Date(Long.valueOf(cdmlibLastModified));
        cdmlibLastModifiedTimestamp = sdf.format(cdmlibLastModifiedDate);

        return cdmlibLastModifiedTimestamp;
    }

    private void setEditorInfo() {
        txtEditorCdmVersion.setText(CdmMetaData.getDbSchemaVersion());
        setCdmlibVersion(txtEditorCdmlibVersion);
    }

    private void setCdmlibVersion(Text txt) {
        String editorVersionTimestamp = generateLastModifiedTooltip(CdmApplicationState.getCdmlibLastModified());
        txt.setText(CdmApplicationState.getCdmlibVersion() + ":" + editorVersionTimestamp); //$NON-NLS-1$
    }

    private void clearOnServerChange() {
        setMessage(""); //$NON-NLS-1$
        txtServerSchemaVersion.setText(""); //$NON-NLS-1$
        txtServerCdmlibVersion.setText(""); //$NON-NLS-1$
        txtServerCdmlibVersion.setToolTipText(""); //$NON-NLS-1$
        txtServerSchemaVersion.setText(""); //$NON-NLS-1$
        comboCdmInstance.removeAll();
        txtCdmInstanceStatus.setText(""); //$NON-NLS-1$
        txtPort.setEditable(false);
        txtPort.setEnabled(false);
    }

    private void clearOnInstanceChange() {
        setMessage(""); //$NON-NLS-1$
        txtServerSchemaVersion.setText(""); //$NON-NLS-1$
        btnConnect.setText(LABEL_CONNECT);
        btnConnect.setBackground(null);
        btnConnect.requestLayout();
    }

    @SuppressWarnings("unused")  //in work, for future centralized handling of control's status
    private void updateControls(){

        //server
//      comboCdmServer; //nothing to change
        String strServerStatus = this.selectedServerInfo != null? STATUS_NOT_AVAILABLE : STATUS_AVAILABLE;
        txtCdmServerStatus.setText(strServerStatus);
//      btnCdmServerRefresh;  //nothing to change

        //database
//        comboCdmInstance
        String strInstanceStatus = this.selectedCdmInstance != null? STATUS_NOT_AVAILABLE : STATUS_AVAILABLE;
        txtCdmInstanceStatus.setText(strInstanceStatus);
//        btnCdmInstanceRefresh;  //nothing to change

        //login + connect
        txtLogin.setText(Nz(login));
        txtPassword.setText(Nz(password));
        btnConnect.setText(LABEL_CONNECT);  //TODO
        btnConnect.setEnabled(true);

        //rememberMe + defaultValues
//        btnRememberMe;
//        lblDefaultLogin;

        //message
//        styledTxtMessage;

        //Advanced
//        txtPort
//        btnStopServer;

//        txtServerCdmlibVersion;
//        txtEditorCdmlibVersion;
//        txtServerSchemaVersion;
//        txtEditorCdmVersion;
    }

    private String Nz(String value) {
        return CdmUtils.Nz(value);
    }

    public Map<String, CdmServerInfo> getServerInfoMap() {
        return serverInfoMap;
    }

    public String getServerName() {
        return serverName;
    }

    public void setServerName(String serverName) {
        this.serverName = serverName;
    }

    public boolean isAutoConnect() {
        return autoConnect;
    }

    public void setAutoConnect(boolean autoConnect) {
        this.autoConnect = autoConnect;
    }

    public boolean isDevRemoteSource() {
        return isDevRemoteSource;
    }

    public void setDevRemoteSource(boolean isDevRemoteSource) {
        this.isDevRemoteSource = isDevRemoteSource;
    }

    public String getInstanceName() {
        return instanceName;
    }

    public void setInstanceName(String instanceName) {
        this.instanceName = instanceName;
    }

    @Override
    public void handleError(Throwable t) {
        Display.getDefault().syncExec(()-> {
            serverJob.cancel();

            String title = Messages.RemotingLoginDialog_SERVER_LAUNCH_ERROR;
            String  message = t.getMessage();

            MessagingUtils.errorDialog(title,
                    this,
                    message,
                    TaxeditorStorePlugin.PLUGIN_ID,
                    t,
                    true);
        });

    }
}
