/**
 * Copyright (C) 2015 EDIT
 * European Distributed Institute of Taxonomy
 * http://www.e-taxonomy.eu
 *
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */
package eu.etaxonomy.taxeditor.ui.dialog;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;

import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.StyledText;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.swt.widgets.Shell;

/**
 * @author cmathew
 * @date 22 Oct 2015
 *
 */
public class ReportTextDialog extends Dialog {

    private String reportText = "";
    private String title = "";
    /**
     * Create the dialog.
     * @param parentShell
     */
    public ReportTextDialog(Shell parentShell) {
        super(parentShell);
    }

    /**
     * Create contents of the dialog.
     * @param parent
     */
    @Override
    protected Control createDialogArea(final Composite parent) {
        Composite container = (Composite) super.createDialogArea(parent);
        GridLayout gridLayout = (GridLayout) container.getLayout();
        gridLayout.numColumns = 2;

        StyledText styledText = new StyledText(container, SWT.BORDER | SWT.MULTI | SWT.V_SCROLL | SWT.BORDER);
        styledText.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1));
        styledText.setText(reportText);

        Button btnSave = new Button(container, SWT.NONE);
        btnSave.addSelectionListener(new SelectionAdapter() {
            @Override
            public void widgetSelected(SelectionEvent e) {
                FileDialog fileDialog = new FileDialog(parent.getShell(), SWT.SAVE);
                // Set filter on .txt files
                fileDialog.setFilterExtensions(new String[] { "*.txt" });
                // Put in a readable name for the filter
                fileDialog.setFilterNames(new String[] { "Textfiles(*.txt)" });
                String fileName = fileDialog.open();
                if(fileName != null) {
                    File file = new File(fileName);
                    if (file.exists()) {
                        MessageBox mb = new MessageBox(fileDialog.getParent(), SWT.ICON_WARNING
                                | SWT.YES | SWT.NO);
                        mb.setMessage(fileName + " already exists. Do you want to replace it?");
                        boolean override = mb.open() == SWT.YES;
                        if(!override) {
                            return;
                        }
                    }
                    try {
                        FileWriter fileWriter = new FileWriter(fileName, false);
                        fileWriter.write(getReportText());
                        fileWriter.close();
                    } catch (IOException ioe) {
                        throw new IllegalStateException(ioe);
                    }
                }
            }
        });
        btnSave.setLayoutData(new GridData(SWT.LEFT, SWT.TOP, false, false, 1, 1));
        btnSave.setText("save");

        return container;
    }

    /**
     * Create contents of the button bar.
     * @param parent
     */
    @Override
    protected void createButtonsForButtonBar(Composite parent) {
        createButton(parent, IDialogConstants.OK_ID, IDialogConstants.OK_LABEL, true);
    }

    @Override
    protected void configureShell(Shell shell) {
        super.configureShell(shell);
        shell.setText(title);
     }
    /**
     * Return the initial size of the dialog.
     */
    @Override
    protected Point getInitialSize() {
        return new Point(669, 501);
    }

    public void setReportText(String text) {
        this.reportText = text;
    }

    public String getReportText() {
        return reportText;
    }

    public void setTitle(String title) {
        this.title = title;
    }
}
