/**
* Copyright (C) 2019 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.ui.dialog;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.UUID;

import org.apache.commons.collections.CollectionUtils;
import org.eclipse.jface.wizard.Wizard;

import eu.etaxonomy.cdm.api.service.description.DistributionAggregationConfiguration;
import eu.etaxonomy.cdm.model.location.NamedAreaLevel;
import eu.etaxonomy.cdm.persistence.dto.TermDto;
import eu.etaxonomy.cdm.persistence.dto.TermVocabularyDto;
import eu.etaxonomy.taxeditor.l10n.Messages;
import eu.etaxonomy.taxeditor.ui.dialog.configurator.wizard.SuperAreaPage;

/**
 * @author k.luther
 * @since 25.11.2019
 */
public class SuperAreaSelectionWizard extends Wizard {

        private final SuperAreaPage aPage;
        DistributionAggregationConfiguration configurator;

        public SuperAreaSelectionWizard(DistributionAggregationConfiguration configurator, NamedAreaLevel level) {
            setWindowTitle(Messages.AvailableDistributionWizard_WINDOW_TITLE);
            // TODO if preferenceStore has elements checked load elements in wizard
            aPage = new SuperAreaPage(Messages.AvailableDistributionWizard_PAGE_TITLE, level);
            this.configurator = configurator;
        }

        @Override
        public boolean performFinish() {
            // TODO: get Selection and save in EditorPreferences
            if (!checkNoneChecked()) {
                Object[] checkedElements = aPage.getViewer().getCheckedElements();
                Object[] grayedElements = aPage.getViewer().getGrayedElements();
                ArrayList checkedList = new ArrayList(Arrays.asList(checkedElements));
                ArrayList grayedList = new ArrayList(Arrays.asList(grayedElements));
                checkedList = (ArrayList) CollectionUtils.subtract(checkedList, grayedList);
                ArrayList<UUID> listUIIDChecked = new ArrayList<>();
                ArrayList<UUID> listUIIDGrayed = new ArrayList<UUID>();
                for (Object o : checkedList) {
                    if (o instanceof TermDto) {
                        listUIIDChecked.add(((TermDto) o).getUuid());

                    }else if(o instanceof TermVocabularyDto){
                        TermVocabularyDto termVocDto = (TermVocabularyDto) o;
                        listUIIDGrayed.add(termVocDto.getUuid());
                    }
                }
//                for (Object o : grayedList) {
//                    if (o instanceof TermDto) {
//                        listUIIDChecked.add(((TermDto) o).getUuid());
//
//                    }else if(o instanceof TermVocabularyDto){
//                        TermVocabularyDto termVocDto = (TermVocabularyDto) o;
//                        listUIIDGrayed.add(termVocDto.getUuid());
//                    }
//                }

               configurator.setSuperAreas(listUIIDChecked);


                return true;
            } else {
                return false;
            }
        }

        @Override
        public void addPages() {
            addPage(aPage);
        }

        private boolean checkNoneChecked() {

            if (aPage.getViewer().getCheckedElements().length == 0) {
                aPage.setMessage(Messages.AvailableDistributionWizard_CHECK_MESSAGE, aPage.WARNING);
                return true;
            } else {
                aPage.setMessage(null);
                return false;
            }
        }
    //
//        @Override
//        public ConversationHolder getConversationHolder() {
//            if (conversation == null) {
//                conversation = CdmStore.createConversation();
//            }
//            return conversation;
//        }
    //
//        @Override
//        public void update(CdmDataChangeMap changeEvents) {
//        }
    }

