/**
* Copyright (C) 2019 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.ui.dialog.configurator;

import java.util.ArrayList;
import java.util.Comparator;
import java.util.EnumSet;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.commons.lang3.StringUtils;
import org.eclipse.jface.viewers.CheckboxTableViewer;
import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;

import eu.etaxonomy.cdm.api.service.description.AggregationMode;
import eu.etaxonomy.cdm.api.service.description.AggregationSourceMode;
import eu.etaxonomy.cdm.api.service.description.DescriptionAggregationBase;
import eu.etaxonomy.cdm.api.service.description.DescriptionAggregationConfigurationBase;
import eu.etaxonomy.cdm.filter.LogicFilter;
import eu.etaxonomy.cdm.model.name.Rank;
import eu.etaxonomy.cdm.model.name.TaxonName;
import eu.etaxonomy.cdm.model.reference.OriginalSourceType;
import eu.etaxonomy.cdm.model.taxon.TaxonNode;
import eu.etaxonomy.taxeditor.store.CdmStore;

/**
 * @author k.luther
 * @since 13.12.2019
 */
public abstract class AggregationConfigurationWizardPage<T extends DescriptionAggregationBase<T,S>,
            S extends DescriptionAggregationConfigurationBase<T>>
            extends WizardPage {

    protected S configurator;

    protected Button checkUseSelectedTaxonNode;
    protected Button checkUseHigherLevel;
    protected Button checkUseSelectedSubtree;

    protected Combo comboHigherRank;
    protected Combo comboLowerRank;

    protected Combo comboSourceModeChildParent;
    protected Combo comboSourceModeWithinTaxon;

    protected CheckboxTableViewer aggregationModeViewer;
    protected TaxonNode subTreeNode;

    protected CheckboxTableViewer sourceTypeViewer;
    protected Map<String, OriginalSourceType> typeMap = new HashMap<>();

    protected Button checkIncludeUnpublishedTaxa;

    private Comparator<Rank> comparator = new Comparator<Rank>(){
        @Override
        public int compare(Rank o1, Rank o2) {
            return o2.compareTo(o1);
        }
    };

    protected AggregationConfigurationWizardPage(String pageName) {
        super(pageName);
        // TODO Auto-generated constructor stub
    }

    protected void updateHigherRankCombo() {
    	int index = comboHigherRank.getSelectionIndex();
    	String item = null;
    	if (index > 0){
    		item = comboHigherRank.getItem(index);
    	}
        comboHigherRank.removeAll();
        List<Rank> rankList = new ArrayList<>();
        TaxonName name = null;
        if (subTreeNode != null){
            if (subTreeNode.getTaxon() != null){
                name = subTreeNode.getTaxon().getName();
            }
        }
        comboHigherRank.add(""); //$NON-NLS-1$
        if (name != null && name.getRank() != null){
            for (Rank rank: CdmStore.getTermManager().getPreferredTerms(Rank.class)){
                if (useHigherLevel()
                        || (name.getRank().isHigher(rank) || name.getRank().equals(rank))){
                    rankList.add(rank);
                }
            }
        }else{
            for (Rank rank: CdmStore.getTermManager().getPreferredTerms(Rank.class)){
                rankList.add(rank);
            }
        }

        rankList.sort(comparator);
        for (Rank rank: rankList){
            comboHigherRank.add(rank.getLabel());
            comboHigherRank.setData(rank.getLabel(), rank);
        }
        int i = 1;
        if (item != null){
        	for (String itemString: comboHigherRank.getItems()){
        		if (itemString.equals(item)){
        			comboHigherRank.select(i);
        			break;
        		}
        		i++;
        	}
        }
        
    }

   protected void updateLowerRankCombo() {
	   int index = comboLowerRank.getSelectionIndex();
	   String item = null;
	   if (index > 0){
		   item = comboLowerRank.getItem(index);
	   }
       comboLowerRank.removeAll();
       TaxonName name = null;
       List<Rank> rankList = new ArrayList<>();
       if (subTreeNode != null){
           if (subTreeNode.getTaxon() != null){
               name = subTreeNode.getTaxon().getName();
           }
       }
       comboLowerRank.add(""); //$NON-NLS-1$
       Rank higherRank = null;
       if (comboHigherRank.getText() != null && comboHigherRank.getData(comboHigherRank.getText()) != null){
           higherRank = (Rank)comboHigherRank.getData(comboHigherRank.getText());
       }
       for (Rank rank: CdmStore.getTermManager().getPreferredTerms(Rank.class)){
           if (higherRank != null){
               if (higherRank.isHigher(rank) || higherRank.equals(rank)){
                   rankList.add(rank);
               }
           } else if (name == null || (name.getRank().isHigher(rank) || name.getRank().equals(rank))) {
               rankList.add(rank);
           }
       }
       rankList.sort(comparator);
       for (Rank rank: rankList){
    	   comboLowerRank.add(rank.getLabel());
    	   comboLowerRank.setData(rank.getLabel(), rank);
       }
       int i = 0;
       if (item != null){
       	for (String itemString: comboLowerRank.getItems()){
       		if (itemString.equals(item)){
       			comboLowerRank.select(i);
       			break;
       		}
       		i++;
       	}
       }
   }

   public Rank getHigherRank(){
       if (StringUtils.isNotBlank(comboHigherRank.getText())){
           return (Rank)comboHigherRank.getData(comboHigherRank.getText());
       }
       return null;
   }

   public Rank getLowerRank(){
       if (StringUtils.isNotBlank(comboLowerRank.getText())){
           return (Rank)comboLowerRank.getData(comboLowerRank.getText());
       }
       return null;
   }

   public boolean useHigherLevel(){
       return checkUseHigherLevel.getSelection();
   }

   public boolean useSubtree(){
       return checkUseSelectedSubtree.getSelection();
   }

   public boolean useTaxonNode(){
       return checkUseSelectedTaxonNode.getSelection();
   }

   public AggregationSourceMode getSourceModeChildParent(){
       if (StringUtils.isNotBlank(comboSourceModeChildParent.getText())){
           return (AggregationSourceMode)comboSourceModeChildParent.getData(comboSourceModeChildParent.getText());
       }
       return null;
   }

   public AggregationSourceMode getSourceModeWithinTaxon(){
       if (StringUtils.isNotBlank(comboSourceModeWithinTaxon.getText())){
           return (AggregationSourceMode)comboSourceModeWithinTaxon.getData(comboSourceModeWithinTaxon.getText());

       }
       return null;
   }

   public EnumSet<OriginalSourceType> getSourceTypes(){
       List<OriginalSourceType> list = new ArrayList<>();
       if (sourceTypeViewer == null) return null;
       for (Object o: sourceTypeViewer.getCheckedElements()){
           String string = null;
           if (o instanceof String){
               string = (String)o;
           }
           if (string != null){
               list.add(typeMap.get(string));
           }
       }

       return EnumSet.copyOf(list);
   }


   public List<AggregationMode> getAggregationMode(){

       List<AggregationMode> result = new ArrayList<>();
       for (Object mode: aggregationModeViewer.getCheckedElements()){
           if (mode instanceof AggregationMode){
               result.add((AggregationMode)mode);
           }
       }
       return result;
   }
   
   protected void setConfigValueToRankCombo(Combo rankCombo) {
		int index = 0;
       if (configurator.getTaxonNodeFilter().getRankMax() == null){
       	rankCombo.select(index);
       }else{        
	        Rank rank = null;
	        for (String item: rankCombo.getItems()){
	        	if (rankCombo.getData(item) instanceof Rank){
	        		rank = (Rank) rankCombo.getData(item);
	        		if (rank.getUuid().equals(configurator.getTaxonNodeFilter().getRankMax().getUuid())){
	        			rankCombo.select(index);
	        			break;
	        		}
	        	}
	        	index++;
	        }
       }
	}

}