/**
* Copyright (C) 2019 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.ui.dialog.configurator;

import java.util.UUID;

import org.eclipse.jface.wizard.Wizard;
import org.eclipse.swt.widgets.Display;

import eu.etaxonomy.cdm.api.application.CdmApplicationState;
import eu.etaxonomy.cdm.api.service.description.DistributionAggregationConfiguration;
import eu.etaxonomy.cdm.api.service.longrunningService.ILongRunningTasksService;
import eu.etaxonomy.cdm.common.monitor.IRemotingProgressMonitor;
import eu.etaxonomy.cdm.filter.TaxonNodeFilter;
import eu.etaxonomy.taxeditor.l10n.Messages;
import eu.etaxonomy.taxeditor.model.AbstractUtility;
import eu.etaxonomy.taxeditor.operation.IPostMoniteredOperationEnabled;

/**
 * @author k.luther
 * @since 15.11.2019
 */
public class DistributionAggregationConfiguratorWizard extends Wizard implements IPostMoniteredOperationEnabled {
    private final DistributionAggregationConfiguration configurator;
    private DistributionAggregationWizardPage page;
    private ILongRunningTasksService longRunningTaskService;

    public DistributionAggregationConfiguratorWizard(DistributionAggregationConfiguration config) {
        super();
        this.configurator = config;
        setNeedsProgressMonitor(true);
        this.setWindowTitle(Messages.SetAggregationConfiguration_Title);
        longRunningTaskService = CdmApplicationState.getLongRunningTasksService();
    }

    @Override
    public boolean performFinish() {

        configurator.setAggregatingSourceTypes(page.getSourceTypes());
        configurator.setStatusOrder(page.getStatusOrder());
        if (page.useSubtree()){
            configurator.setTaxonNodeFilter(TaxonNodeFilter.NewSubtreeInstance(page.getSubTreeNode()));
        } else if (page.useHigherLevel()){
            configurator.setTaxonNodeFilter(TaxonNodeFilter.NewClassificationInstance(page.getSelectedClassification()));
        } else if (page.useTaxonNode()){
            configurator.setTaxonNodeFilter(TaxonNodeFilter.NewTaxonNodeInstance(page.getSubTreeNode()));
        }
        configurator.getTaxonNodeFilter().setIncludeUnpublished(page.useUnpublishedData());
        configurator.getTaxonNodeFilter().setRankMax(page.getHigherRank());
        configurator.getTaxonNodeFilter().setRankMin(page.getLowerRank());
        configurator.setAggregationMode(page.getAggregationMode());
        configurator.setToParentSourceMode(page.getSourceModeChildParent());
        configurator.setWithinTaxonSourceMode(page.getSourceModeWithinTaxon());

        UUID uuid = longRunningTaskService.invoke(configurator);

        Display.getDefault().asyncExec(()-> {
                AbstractUtility.executeMoniteredOperation("Aggregate Distributions",
                        uuid,
                        1000,
                        false,
                        DistributionAggregationConfiguratorWizard.this,
                        null,
                        true, false);
        });

        return true;
    }


    @Override
    public void addPages() {
        page = new DistributionAggregationWizardPage(configurator);
        addPage(page);
    }


    @Override
    public void postOperation(IRemotingProgressMonitor monitor) {
        // TODO Auto-generated method stub
    }
}