// $Id$
/**
* Copyright (C) 2017 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.ui.dialog.configurator;

import org.eclipse.core.databinding.DataBindingContext;
import org.eclipse.core.databinding.beans.PojoProperties;
import org.eclipse.core.databinding.observable.value.IObservableValue;
import org.eclipse.jface.databinding.swt.WidgetProperties;
import org.eclipse.jface.wizard.IWizardContainer;
import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Label;

import eu.etaxonomy.cdm.api.service.config.PublishForSubtreeConfigurator;
import eu.etaxonomy.taxeditor.l10n.Messages;
import eu.etaxonomy.taxeditor.ui.element.CdmFormFactory;

/**
 * @author k.luther
 * @date 10.10.2017
 *
 */
public class PublishSubTreeConfiguratorWizardPage extends WizardPage implements SelectionListener {

    private DataBindingContext m_bindingContext;

    private final static CdmFormFactory toolkit = new CdmFormFactory(Display.getCurrent());
    private final PublishForSubtreeConfigurator configurator;

    private Button btnIncludeAcceptedTaxa;
    private Button btnIncludeSynonyms;
    private Button btnIncludeSharedTaxa;
    private Button btnIncludeMisappliedNames;
    private Button btnIncludeProParteSynonyms;
    private Button btnIncludeHybrids;

    private Combo btnPublish;


    /**
     * @param string
     */
    public PublishSubTreeConfiguratorWizardPage(PublishForSubtreeConfigurator configurator) {
        super("Set Publish Flag Configuration");
        this.configurator = configurator;

        this.setDescription(Messages.SetPublishConfiguration_Description_Configurator);

    }



    /* (non-Javadoc)
     * @see org.eclipse.jface.dialogs.IDialogPage#createControl(org.eclipse.swt.widgets.Composite)
     */
    @Override
    public void createControl(Composite parent) {

        final Composite composite = new Composite(parent, SWT.NULL);

        GridLayout gridLayout = new GridLayout();
        gridLayout.numColumns = 1;

        composite.setLayout(gridLayout);
        final Composite control = new Composite(composite, SWT.NULL);
        GridLayout gridLayoutControl = new GridLayout();

        control.setLayout(gridLayoutControl);
        control.setLayoutData(new GridData(SWT.FILL, SWT.LEFT, true, true));

        btnPublish = new Combo(control, SWT.BORDER| SWT.READ_ONLY);
        btnPublish.setText(Messages.SetPublishConfiguration_Publish);
        btnPublish.setToolTipText(Messages.SetPublishConfiguration_Publish_tooltip);
        btnPublish.add(Messages.SetPublishConfiguration_publish, 0);
        btnPublish.add(Messages.SetPublishConfiguration_dont_publish,1);
        if (configurator.isPublish()){
            btnPublish.select(0);
        }else{
            btnPublish.select(1);
        }
        btnPublish.addSelectionListener(this);

        btnIncludeAcceptedTaxa = new Button(control, SWT.CHECK);
        btnIncludeAcceptedTaxa.setText(Messages.SetPublishConfiguration_IncludeAcceptedTaxa);
        btnIncludeAcceptedTaxa.addSelectionListener(this);

        btnIncludeSynonyms = new Button(control, SWT.CHECK);
        btnIncludeSynonyms.setText(Messages.SetPublishConfiguration_IncludeSynonyms);
        btnIncludeSynonyms.addSelectionListener(this);

        btnIncludeMisappliedNames = new Button(control, SWT.CHECK);
        btnIncludeMisappliedNames.setText(Messages.SetPublishConfiguration_IncludeMisappliedNames);
        btnIncludeMisappliedNames.addSelectionListener(this);

        btnIncludeProParteSynonyms = new Button(control, SWT.CHECK);
        btnIncludeProParteSynonyms.setText(Messages.SetPublishConfiguration_IncludeProParteSynonyms);
        btnIncludeProParteSynonyms.addSelectionListener(this);

        Label space = new Label(control, SWT.NONE);

        btnIncludeSharedTaxa = new Button(control, SWT.CHECK);
        btnIncludeSharedTaxa.setText(Messages.SetPublishConfiguration_IncludeSharedtaxa);

        btnIncludeHybrids = new Button(control, SWT.CHECK);
        btnIncludeHybrids.setText(Messages.SetPublishConfiguration_IncludeHybrids);


        m_bindingContext = initDataBindings();

        setControl(composite);
    }



    protected DataBindingContext initDataBindings() {
        DataBindingContext bindingContext = new DataBindingContext();

        IObservableValue observeSelectionBtnIncludeAcceptedTaxa = WidgetProperties.selection().observe(btnIncludeAcceptedTaxa);
        IObservableValue includeAcceptedTaxaConfiguratorObserveValue = PojoProperties.value("includeAcceptedTaxa").observe(configurator);
        bindingContext.bindValue(observeSelectionBtnIncludeAcceptedTaxa, includeAcceptedTaxaConfiguratorObserveValue);

        IObservableValue observeSelectionBtnIncludeSharedTaxal = WidgetProperties.selection().observe(btnIncludeSharedTaxa);
        IObservableValue includeSharedTaxaConfiguratorObserveValue = PojoProperties.value("includeSharedTaxa").observe(configurator);
        bindingContext.bindValue(observeSelectionBtnIncludeSharedTaxal, includeSharedTaxaConfiguratorObserveValue);

        IObservableValue observeSelectionBtnIncludeMisappliedNames = WidgetProperties.selection().observe(btnIncludeMisappliedNames);
        IObservableValue includeMisappliedNamesConfiguratorObserveValue = PojoProperties.value("includeMisapplications").observe(configurator);
        bindingContext.bindValue(observeSelectionBtnIncludeMisappliedNames, includeMisappliedNamesConfiguratorObserveValue);

        IObservableValue observeSelectionBtnIncludeProParteSynonyms = WidgetProperties.selection().observe(btnIncludeProParteSynonyms);
        IObservableValue includeProParteSynonymsConfiguratorObserveValue = PojoProperties.value("includeProParteSynonyms").observe(configurator);
        bindingContext.bindValue(observeSelectionBtnIncludeProParteSynonyms, includeProParteSynonymsConfiguratorObserveValue);

        IObservableValue observeSelectionBtnIncludeHybrids = WidgetProperties.selection().observe(btnIncludeHybrids);
        IObservableValue includeHybridsConfiguratorObserveValue = PojoProperties.value("includeHybrids").observe(configurator);
        bindingContext.bindValue(observeSelectionBtnIncludeHybrids, includeHybridsConfiguratorObserveValue);

        IObservableValue observeSelectionBtnIncludeSynonyms = WidgetProperties.selection().observe(btnIncludeSynonyms);
        IObservableValue includeSynonymsConfiguratorObserveValue = PojoProperties.value("includeSynonyms").observe(configurator);
        bindingContext.bindValue(observeSelectionBtnIncludeSynonyms, includeSynonymsConfiguratorObserveValue);



//        IObservableValue observeSelectionBtnPublish = WidgetProperties.selection().observe(btnPublish);
//        IObservableValue publishConfiguratorObserveValue = PojoProperties.value("publish").observe(configurator);
//        bindingContext.bindValue(observeSelectionBtnPublish, publishConfiguratorObserveValue, null, null);
        return bindingContext;

    }

    public void updateContext(){
        m_bindingContext.updateTargets();
    }



    /* (non-Javadoc)
     * @see org.eclipse.swt.events.SelectionListener#widgetSelected(org.eclipse.swt.events.SelectionEvent)
     */
    @Override
    public void widgetSelected(SelectionEvent e) {
        if (btnPublish.getSelectionIndex() > 0){
            configurator.setPublish(false);
        } else{
            configurator.setPublish(true);
        }
        if (!btnIncludeMisappliedNames.getSelection() && !btnIncludeAcceptedTaxa.getSelection() && !btnIncludeProParteSynonyms.getSelection() ){
            btnIncludeSharedTaxa.setEnabled(false);
        }else{
            btnIncludeSharedTaxa.setEnabled(true);

        }
        if (!btnIncludeMisappliedNames.getSelection() && !btnIncludeAcceptedTaxa.getSelection() && !btnIncludeProParteSynonyms.getSelection() && !btnIncludeSynonyms.getSelection()){
            btnIncludeHybrids.setEnabled(false);
        }else{
            btnIncludeHybrids.setEnabled(true);

        }

        IWizardContainer container = this.getWizard().getContainer();
        container.updateButtons();
    }



    /* (non-Javadoc)
     * @see org.eclipse.swt.events.SelectionListener#widgetDefaultSelected(org.eclipse.swt.events.SelectionEvent)
     */
    @Override
    public void widgetDefaultSelected(SelectionEvent e) {
        // TODO Auto-generated method stub

    }

    @Override
    public boolean isPageComplete(){

         return (btnIncludeAcceptedTaxa.getSelection() || btnIncludeSynonyms.getSelection() || btnIncludeMisappliedNames.getSelection() || btnIncludeProParteSynonyms.getSelection());


    }
}
