/**
* Copyright (C) 2017 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.ui.dialog.configurator;

import org.eclipse.core.databinding.DataBindingContext;
import org.eclipse.core.databinding.beans.PojoProperties;
import org.eclipse.core.databinding.observable.value.IObservableValue;
import org.eclipse.jface.databinding.swt.WidgetProperties;
import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Text;
import org.eclipse.wb.swt.ResourceManager;

import eu.etaxonomy.cdm.api.service.config.SecundumForSubtreeConfigurator;
import eu.etaxonomy.cdm.model.reference.Reference;
import eu.etaxonomy.taxeditor.l10n.Messages;
import eu.etaxonomy.taxeditor.preference.Resources;
import eu.etaxonomy.taxeditor.store.StoreUtil;
import eu.etaxonomy.taxeditor.ui.dialog.selection.SelectionDialogFactory;
import eu.etaxonomy.taxeditor.ui.selection.EntitySelectionElement;

/**
 * @author k.luther
 * @date 10.02.2017
 *
 */
public class SetSecundumConfiguratorWizardPage extends WizardPage implements Listener {
    private DataBindingContext m_bindingContext;

    private final SecundumForSubtreeConfigurator configurator;

    private  Button btnBrowseReference = null;

    private Text textReference = null;
    private Button btnClear = null;
    private EntitySelectionElement<Reference> selectReference;
    private Button btnIncludeAcceptedTaxa;
    private Button btnIncludeSynonyms;
    private Button btnOverwriteExisting;
//    private Button btnOverwriteExistingSynonyms;
//    private Button btnOverwriteExistingRelations;
    private Button btnEmptySecundumDetail;
    private Button btnIncludeSharedTaxa;
    private Button btnIncludeMisappliedNamed;
    private Button btnIncludeProParteSynonyms;


    /**
     * Create the composite.
     * @param parent
     * @param style
     */
    public SetSecundumConfiguratorWizardPage(SecundumForSubtreeConfigurator configurator) {
        super("Set Secundum Reference Configuration");
        this.configurator = configurator;

        this.setDescription(Messages.SetSecundumConfiguration_Description_Configurator);
    }

    /**
     * @return
     */
    private DataBindingContext initDataBindings() {
        DataBindingContext bindingContext = new DataBindingContext();

        IObservableValue observeSelectionBtnIncludeAcceptedTaxa = WidgetProperties.selection().observe(btnIncludeAcceptedTaxa);
        IObservableValue includeAcceptedTaxaConfiguratorObserveValue = PojoProperties.value("includeAcceptedTaxa").observe(configurator);
        bindingContext.bindValue(observeSelectionBtnIncludeAcceptedTaxa, includeAcceptedTaxaConfiguratorObserveValue, null, null);

        IObservableValue observeSelectionBtnIncludeSynonyms = WidgetProperties.selection().observe(btnIncludeSynonyms);
        IObservableValue includeSynonymsConfiguratorObserveValue = PojoProperties.value("includeSynonyms").observe(configurator);
        bindingContext.bindValue(observeSelectionBtnIncludeSynonyms, includeSynonymsConfiguratorObserveValue, null, null);

        IObservableValue observeSelectionBtnIncludeMisappliedNames = WidgetProperties.selection().observe(btnIncludeMisappliedNamed);
        IObservableValue includeMisappliedNamesConfiguratorObserveValue = PojoProperties.value("includeMisapplications").observe(configurator);
        bindingContext.bindValue(observeSelectionBtnIncludeMisappliedNames, includeMisappliedNamesConfiguratorObserveValue, null, null);

        IObservableValue observeSelectionBtnIncludeProparteSynonyms = WidgetProperties.selection().observe(btnIncludeProParteSynonyms);
        IObservableValue includeProparteSynonymsConfiguratorObserveValue = PojoProperties.value("includeProParteSynonyms").observe(configurator);
        bindingContext.bindValue(observeSelectionBtnIncludeProparteSynonyms, includeProparteSynonymsConfiguratorObserveValue, null, null);

        IObservableValue observeSelectionBtnOverwriteExisting = WidgetProperties.selection().observe(btnOverwriteExisting);
        IObservableValue overwriteExistingConfiguratorObserveValue = PojoProperties.value("overwriteExisting").observe(configurator);
        bindingContext.bindValue(observeSelectionBtnOverwriteExisting, overwriteExistingConfiguratorObserveValue, null, null);

        IObservableValue observeSelectionBtnEmptySecundumDetail = WidgetProperties.selection().observe(btnEmptySecundumDetail);
        IObservableValue emptySecundumDetailConfiguratorObserveValue = PojoProperties.value("emptySecundumDetail").observe(configurator);
        bindingContext.bindValue(observeSelectionBtnEmptySecundumDetail, emptySecundumDetailConfiguratorObserveValue, null, null);

        IObservableValue observeSelectionBtnIncludeSharedTaxal = WidgetProperties.selection().observe(btnIncludeSharedTaxa);
        IObservableValue includeSharedTaxaConfiguratorObserveValue = PojoProperties.value("includeSharedTaxa").observe(configurator);
        bindingContext.bindValue(observeSelectionBtnIncludeSharedTaxal, includeSharedTaxaConfiguratorObserveValue, null, null);
        return bindingContext;
    }

    @Override
    public void createControl(Composite parent) {

        final Composite composite = new Composite(parent, SWT.NULL);
        GridData gridComp = new GridData(SWT.BEGINNING, SWT.BEGINNING, true, true);
        gridComp.minimumWidth = 200;
        GridLayout gridLayout = new GridLayout();

        gridLayout.numColumns = 1;
        composite.setLayoutData(gridComp);
        composite.setLayout(gridLayout);
        final Composite compositeRef = new Composite(composite, SWT.NULL);

        GridLayout gridLayoutRef = new GridLayout();
        gridLayoutRef.numColumns = 4;


        compositeRef.setLayout(gridLayoutRef);
        GridData gridRef = new GridData(SWT.BEGINNING, SWT.BEGINNING, true, true);
        compositeRef.setLayoutData(gridRef);
        Label label = new Label(compositeRef, SWT.WRAP);
        label.setText(Messages.SetSecundumConfiguration_NewSecundum_Label);
        label.setLayoutData(new GridData(SWT.BEGINNING, SWT.BEGINNING, false, false));

        textReference = new Text(compositeRef, SWT.BORDER);
        textReference.setEditable(false);
        textReference.setLayoutData(new GridData(SWT.FILL, SWT.LEFT, true, true));
//        textReference.setTextLimit(Text.LIMIT);
        textReference.setBackground(StoreUtil
                .getColor(Resources.COLOR_TEXT_DISABLED_BACKGROUND));
        GridData gridTextReference = new GridData(SWT.FILL, SWT.FILL, true, true);
        gridTextReference.widthHint = 20;
        gridTextReference.grabExcessHorizontalSpace = true;
        textReference.setLayoutData(gridTextReference);

        btnBrowseReference = new Button(compositeRef, SWT.NONE);
        btnBrowseReference.setImage(ResourceManager.getPluginImage("eu.etaxonomy.taxeditor.store", "icons/prj_obj.gif"));
        btnBrowseReference.addListener(SWT.Selection, this);
        btnBrowseReference.setFocus();

        btnClear = new Button(compositeRef, SWT.NONE);
        btnClear.setImage(ResourceManager.getPluginImage("eu.etaxonomy.taxeditor.store", "icons/trash.gif"));
        btnClear.addListener(SWT.Selection, this);
        Label labelDescription = new Label(compositeRef, SWT.WRAP);
        labelDescription.setText(Messages.SetSecundumConfiguration_Description);
        GridData gd_labelDescription = new GridData(SWT.FILL, SWT.BEGINNING, true, false);
        gd_labelDescription.horizontalSpan = 4;
        labelDescription.setLayoutData(gd_labelDescription);
//        new Label(compositeRef, SWT.NONE);
//        new Label(compositeRef, SWT.NONE);
//        new Label(compositeRef, SWT.NONE);

       // selectReference = toolkit.createSelectionElement(Rights.class, getConversationHolder(), composite, "Rights", null, EntitySelectionElement.SELECTABLE);
        final Composite control = new Composite(composite, SWT.WRAP);
        GridLayout gridLayoutControl = new GridLayout();

        control.setLayout(gridLayoutControl);
        control.setLayoutData(new GridData(SWT.FILL, SWT.LEFT, true, true));
      //  Composite control = toolkit.createComposite(composite);
        btnIncludeAcceptedTaxa = new Button(control, SWT.CHECK);
        btnIncludeAcceptedTaxa.setText(Messages.SetSecundumConfiguration_IncludeAcceptedTaxa);
        btnIncludeAcceptedTaxa.setSelection(configurator.isIncludeAcceptedTaxa());
        btnIncludeAcceptedTaxa.addListener(SWT.Selection, new Listener() {
            @Override
         public void handleEvent(Event e) {
                Button b = (Button) e.widget;
                GridData data = (GridData)  btnIncludeAcceptedTaxa.getLayoutData();
                data.exclude = b.getSelection();
                boolean isEnabled = true;
                if (!btnIncludeSynonyms.getSelection() && !data.exclude){
                    isEnabled = false;
                }
                btnOverwriteExisting.setVisible(isEnabled);
                btnEmptySecundumDetail.setVisible(isEnabled);
                btnIncludeSharedTaxa.setVisible(isEnabled);
                setPageComplete(isEnabled);
            }
        });

//                btnOverwriteExistingAccepted = new Button(control, SWT.CHECK);
//                GridData gd_btnOverwriteExistingAccepted = new GridData(SWT.LEFT, SWT.CENTER, false, false, 1, 1);
//                gd_btnOverwriteExistingAccepted.horizontalIndent = 10;
//                btnOverwriteExistingAccepted.setLayoutData(gd_btnOverwriteExistingAccepted);
//                btnOverwriteExistingAccepted.setText(Messages.SetSecundumConfiguration_OverwriteExistingAccepted);



        btnIncludeSynonyms = new Button(control, SWT.CHECK);
        btnIncludeSynonyms.setText(Messages.SetSecundumConfiguration_IncludeSynonyms);
        btnIncludeSynonyms.setSelection(configurator.isIncludeSynonyms());
        btnIncludeSynonyms.addListener(SWT.Selection, new Listener() {
            @Override
         public void handleEvent(Event e) {
                Button b = (Button) e.widget;
                GridData data = (GridData)  btnIncludeSynonyms.getLayoutData();
                data.exclude = b.getSelection();
                boolean isEnabled = true;
                if (!btnIncludeAcceptedTaxa.getSelection() && !data.exclude && !btnIncludeMisappliedNamed.getSelection() && !btnIncludeProParteSynonyms.getSelection()){
                    isEnabled = false;
                }
                btnEmptySecundumDetail.setVisible(isEnabled);
                btnOverwriteExisting.setVisible(isEnabled);
                btnIncludeSharedTaxa.setVisible(isEnabled);
                setPageComplete(isEnabled);

            }
        });

//                btnOverwriteExistingSynonyms = new Button(control, SWT.CHECK);
//                GridData gd_btnOverwriteExistingSynonyms = new GridData(SWT.LEFT, SWT.CENTER, false, false, 1, 1);
//                gd_btnOverwriteExistingSynonyms.horizontalIndent = 10;
//                btnOverwriteExistingSynonyms.setLayoutData(gd_btnOverwriteExistingSynonyms);
//                btnOverwriteExistingSynonyms.setText(Messages.SetSecundumConfiguration_OverwriteExistingSynonyms);
//                btnOverwriteExistingSynonyms.setSelection(configurator.isOverwriteExistingSynonyms());

        btnIncludeMisappliedNamed = new Button(control, SWT.CHECK);
        btnIncludeMisappliedNamed.setText(Messages.SetSecundumConfiguration_IncludeMisapplications);
        btnIncludeMisappliedNamed.setSelection(configurator.isIncludeMisapplications());
        btnIncludeMisappliedNamed.addListener(SWT.Selection, new Listener() {
            @Override
         public void handleEvent(Event e) {
                Button b = (Button) e.widget;
                GridData data = (GridData)  btnIncludeMisappliedNamed.getLayoutData();
                data.exclude = b.getSelection();
                boolean isEnabled = true;
                if (!btnIncludeAcceptedTaxa.getSelection() && !btnIncludeProParteSynonyms.getSelection() && !btnIncludeSynonyms.getSelection() && !data.exclude){
                    isEnabled = false;
                }
                btnEmptySecundumDetail.setVisible(isEnabled);
                btnOverwriteExisting.setVisible(isEnabled);
                btnIncludeSharedTaxa.setVisible(isEnabled);
                setPageComplete(isEnabled);
            }
        });



        btnIncludeProParteSynonyms = new Button(control, SWT.CHECK);
        btnIncludeProParteSynonyms.setText(Messages.SetSecundumConfiguration_IncludeProParteSynonyms);
        btnIncludeProParteSynonyms.setSelection(configurator.isIncludeProParteSynonyms());
        btnIncludeProParteSynonyms.addListener(SWT.Selection, new Listener() {
            @Override
         public void handleEvent(Event e) {
                Button b = (Button) e.widget;
                GridData data = (GridData)  btnIncludeProParteSynonyms.getLayoutData();
                data.exclude = b.getSelection();
                boolean isEnabled = true;
                if (!btnIncludeAcceptedTaxa.getSelection() && !btnIncludeMisappliedNamed.getSelection() && !btnIncludeSynonyms.getSelection() && !data.exclude){
                    isEnabled = false;
                }
                btnEmptySecundumDetail.setVisible(isEnabled);
                btnOverwriteExisting.setVisible(isEnabled);
                btnIncludeSharedTaxa.setVisible(isEnabled);
                setPageComplete(isEnabled);
            }
        });

        Label space = new Label(control, SWT.NONE);
        btnIncludeSharedTaxa = new Button(control, SWT.CHECK);
        btnIncludeSharedTaxa.setText(Messages.SetSecundumConfiguration_IncludeSharedtaxa);
        btnIncludeSharedTaxa.setSelection(configurator.isIncludeSharedTaxa());

        btnOverwriteExisting = new Button(control, SWT.CHECK);

        btnOverwriteExisting.setText(Messages.SetSecundumConfiguration_OverwriteExisting);
        btnOverwriteExisting.setSelection(configurator.isOverwriteExisting());


//        btnOverwriteExistingAccepted.setSelection(configurator.isOverwriteExistingAccepted());

        btnEmptySecundumDetail = new Button(control, SWT.CHECK);
        btnEmptySecundumDetail.setText(Messages.SetSecundumConfiguration_EmptySecundumDetail);
        btnEmptySecundumDetail.setSelection(configurator.isEmptySecundumDetail());

        m_bindingContext = initDataBindings();
        setControl(composite);
    }



    @Override
    public void handleEvent(Event event) {
        Reference reference;
        if(event.widget==btnBrowseReference){
            reference = SelectionDialogFactory.getSelectionFromDialog(Reference.class, this.getShell(), null, null);
            if(reference!=null){
                this.configurator.setNewSecundum(reference);
                textReference.setText(reference.getTitleCache());

            }

        } else if(event.widget==btnClear){
            reference = null;
            textReference.setText("");
            this.configurator.setNewSecundum(null);
        } else{
            reference = null;
            textReference.setText("");
            this.configurator.setNewSecundum(null);
        }

    }






}
