/**
* Copyright (C) 2019 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.ui.dialog.configurator;

import java.util.List;
import java.util.UUID;

import org.eclipse.jface.wizard.Wizard;

import eu.etaxonomy.cdm.api.application.CdmApplicationState;
import eu.etaxonomy.cdm.api.service.description.StructuredDescriptionAggregationConfiguration;
import eu.etaxonomy.cdm.api.service.longrunningService.ILongRunningTasksService;
import eu.etaxonomy.cdm.common.monitor.IRemotingProgressMonitor;
import eu.etaxonomy.cdm.filter.TaxonNodeFilter;
import eu.etaxonomy.cdm.model.taxon.TaxonNode;
import eu.etaxonomy.cdm.persistence.dto.DescriptiveDataSetBaseDto;
import eu.etaxonomy.cdm.persistence.dto.TaxonNodeDto;
import eu.etaxonomy.cdm.persistence.dto.TermDto;
import eu.etaxonomy.taxeditor.l10n.Messages;
import eu.etaxonomy.taxeditor.operation.IPostMoniteredOperationEnabled;

/**
 * @author k.luther
 * @since 11.12.2019
 */
public class StructuredAggregationConfigurationWizard extends Wizard implements IPostMoniteredOperationEnabled {

    private final StructuredDescriptionAggregationConfiguration configurator;
    private StructuredDescriptionAggregationConfigurationWizardPage page;
    private ILongRunningTasksService longRunningTaskService;
    private DescriptiveDataSetBaseDto dataSet;
    private List<TaxonNodeDto> selectedNodes;


    public StructuredAggregationConfigurationWizard(StructuredDescriptionAggregationConfiguration config, DescriptiveDataSetBaseDto dataSet, List<TaxonNodeDto> nodes) {
        this.configurator = config;
        this.dataSet = dataSet;
        setNeedsProgressMonitor(true);
        this.setWindowTitle(Messages.SetAggregationConfiguration_Title);
        longRunningTaskService = CdmApplicationState.getLongRunningTasksService();
        this.selectedNodes = nodes;
    }

    @Override
    public void postOperation(IRemotingProgressMonitor monitor) {
        // TODO Auto-generated method stub
    }

    @Override
    public boolean performFinish() {
        configurator.setAggregatingSourceTypes(page.getSourceTypes());
        configurator.setAggregationMode(page.getAggregationMode());
        configurator.setToParentSourceMode(page.getSourceModeChildParent());
        configurator.setIncludeDefault(page.checkIncludeDefaultDescriptions.getSelection());
        configurator.setIncludeLiterature(page.checkIncludeLiteratureDescriptions.getSelection());
        configurator.getTaxonNodeFilter().setIncludeUnpublished(page.checkIncludeUnpublishedTaxa.getSelection());
//        for (TaxonNode taxonNode : matrix.getDescriptiveDataSet().getTaxonSubtreeFilter()) {
//            filter = filter.orSubtree(taxonNode);
//        }
        TaxonNodeFilter filter = configurator.getTaxonNodeFilter();
        if (!page.useHigherLevel()) {
            //TODO: check whether this is correct!!!!
            for (TaxonNode node: page.getSelectedTaxonNodes()) {
                if (page.useSubtree()){
                    filter = filter.orSubtree(node);
                }else{
                    filter = filter.orTaxon(node.getTaxon());
                }
            }
            configurator.setTaxonNodeFilter(filter);
        }else{
            if (dataSet.getSubTreeFilter() != null){
	            for (TaxonNodeDto taxonNode : dataSet.getSubTreeFilter()) {
	                filter = filter.orSubtree(taxonNode.getUuid());
	            }
        	}
            configurator.setTaxonNodeFilter(filter);
        }

        //ranks
        UUID minRankUuid = null;
        UUID maxRankUuid = null;
        if (page.getHigherRank() != null){
            maxRankUuid = page.getHigherRank().getUuid();
        }else{
            maxRankUuid = dataSet.getMaxRank() != null? dataSet.getMaxRank().getUuid(): null;

        }
        if (page.getLowerRank() != null){
            minRankUuid = page.getLowerRank().getUuid();
        }else{
            minRankUuid = dataSet.getMinRank() != null? dataSet.getMinRank().getUuid(): null;
        }
        filter.setRankMax(maxRankUuid);
        filter.setRankMin(minRankUuid);

        //geo filter
        if (dataSet.getGeoFilter() != null){
            for (TermDto namedArea : dataSet.getGeoFilter()) {
                filter = filter.orArea(namedArea.getUuid());
            }
        }
        return true;
    }

    @Override
    public void addPages() {
        page = new StructuredDescriptionAggregationConfigurationWizardPage(configurator, selectedNodes);
        addPage(page);
    }
}