/**
* Copyright (C) 2020 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.ui.dialog.configurator.deleteConfigurator;

import org.eclipse.core.databinding.DataBindingContext;
import org.eclipse.core.databinding.beans.PojoProperties;
import org.eclipse.core.databinding.observable.value.IObservableValue;
import org.eclipse.jface.databinding.swt.WidgetProperties;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.DisposeEvent;
import org.eclipse.swt.events.DisposeListener;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.forms.widgets.FormToolkit;

import eu.etaxonomy.cdm.api.service.config.RemoveDescriptionsFromDescriptiveDataSetConfigurator;
import eu.etaxonomy.taxeditor.l10n.Messages;

/**
 * @author k.luther
 * @since Jun 19, 2020
 */
public class DeleteDescriptionFromDescriptiveDatasetConfiguratorComposite extends Composite{

    private final DataBindingContext m_bindingContext;

    private final FormToolkit toolkit = new FormToolkit(Display.getCurrent());
    private final RemoveDescriptionsFromDescriptiveDataSetConfigurator configurator;
    private Button btnOnlyRemoveFromDataset;

    public DeleteDescriptionFromDescriptiveDatasetConfiguratorComposite(RemoveDescriptionsFromDescriptiveDataSetConfigurator configurator, Composite parent, int style) {
        super(parent, style);

        this.configurator = configurator;
        addDisposeListener(new DisposeListener() {
            @Override
            public void widgetDisposed(DisposeEvent e) {
                toolkit.dispose();
            }
        });
        toolkit.paintBordersFor(this);
        setLayout(new RowLayout(SWT.VERTICAL));
        setBackground(getBackground());




        btnOnlyRemoveFromDataset = new Button(this, SWT.CHECK);
        btnOnlyRemoveFromDataset.setText(Messages.DeleteConfiguration_descriptionFromDescriptiveDataSet_onlyRemove);
        btnOnlyRemoveFromDataset.setEnabled(true);
        btnOnlyRemoveFromDataset.setSelection(configurator.isOnlyRemoveDescriptionsFromDataSet());
        m_bindingContext = initDataBindings();
    }

    protected DataBindingContext initDataBindings() {
        DataBindingContext bindingContext = new DataBindingContext();

        IObservableValue observeSelectionBtnDeleteAllSpecimen = WidgetProperties.selection().observe(btnOnlyRemoveFromDataset);
        IObservableValue deleteAllSpecimenConfiguratorObserveValue = PojoProperties.value("onlyRemoveDescriptionsFromDataSet").observe(configurator);
        bindingContext.bindValue(observeSelectionBtnDeleteAllSpecimen, deleteAllSpecimenConfiguratorObserveValue, null, null);
        return bindingContext;
    }

}
