/**
* Copyright (C) 2020 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.ui.dialog.configurator.deleteConfigurator;

import org.eclipse.core.databinding.DataBindingContext;
import org.eclipse.core.databinding.beans.PojoProperties;
import org.eclipse.core.databinding.observable.value.IObservableValue;
import org.eclipse.jface.databinding.swt.WidgetProperties;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.DisposeEvent;
import org.eclipse.swt.events.DisposeListener;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Label;
import org.eclipse.ui.forms.widgets.FormToolkit;

import eu.etaxonomy.cdm.api.service.config.DeleteDescriptiveDataSetConfigurator;
import eu.etaxonomy.taxeditor.l10n.Messages;

/**
 * @author k.luther
 * @since Jun 16, 2020
 */
public class DeleteDescriptiveDatasetConfiguratorComposite extends Composite {
    private final DataBindingContext m_bindingContext;

    private final FormToolkit toolkit = new FormToolkit(Display.getCurrent());
    private final DeleteDescriptiveDataSetConfigurator configurator;
    private Button btnDeleteAllSpecimenDescriptions;
    private Button btnDeleteAllLiteratureDescriptions;
    private Button btnDeleteAllDefaultDescriptions;
    private Button btnDeleteAllAggregatedDescriptions;




    public DeleteDescriptiveDatasetConfiguratorComposite(DeleteDescriptiveDataSetConfigurator configurator, Composite parent, int style) {
        super(parent, style);

        this.configurator = configurator;
        addDisposeListener(new DisposeListener() {
            @Override
            public void widgetDisposed(DisposeEvent e) {
                toolkit.dispose();
            }
        });
        toolkit.paintBordersFor(this);
        setLayout(new RowLayout(SWT.VERTICAL));
        setBackground(getBackground());

        Label lblDelete = new Label(this, SWT.NONE);
        lblDelete.setText(Messages.DeleteConfiguration_descriptiveDataSet_deleteSelection);


        btnDeleteAllSpecimenDescriptions = new Button(this, SWT.CHECK);
        btnDeleteAllSpecimenDescriptions.setText(Messages.DeleteConfiguration_descriptiveDataSet_deleteAllSpecimenDesc);
        btnDeleteAllSpecimenDescriptions.setEnabled(true);
        btnDeleteAllSpecimenDescriptions.setSelection(configurator.isDeleteAllSpecimenDescriptions());

        btnDeleteAllDefaultDescriptions = new Button(this, SWT.CHECK);
        btnDeleteAllDefaultDescriptions.setText(Messages.DeleteConfiguration_descriptiveDataSet_deleteAllDefaultDesc);
        btnDeleteAllDefaultDescriptions.setEnabled(true);
        btnDeleteAllDefaultDescriptions.setSelection(configurator.isDeleteAllDefaultDescriptions());

        btnDeleteAllLiteratureDescriptions = new Button(this, SWT.CHECK);
        btnDeleteAllLiteratureDescriptions.setText(Messages.DeleteConfiguration_descriptiveDataSet_deleteAllLiteratureDesc);
        btnDeleteAllLiteratureDescriptions.setEnabled(true);
        btnDeleteAllLiteratureDescriptions.setSelection(configurator.isDeleteAllLiteratureDescriptions());

        btnDeleteAllAggregatedDescriptions = new Button(this, SWT.CHECK);
        btnDeleteAllAggregatedDescriptions.setText(Messages.DeleteConfiguration_descriptiveDataSet_deleteAllAggregatedDesc);
        btnDeleteAllAggregatedDescriptions.setEnabled(true);
        btnDeleteAllAggregatedDescriptions.setSelection(configurator.isDeleteAllAggregatedDescriptions());

        m_bindingContext = initDataBindings();

    }

    protected DataBindingContext initDataBindings() {
        DataBindingContext bindingContext = new DataBindingContext();

        IObservableValue observeSelectionBtnDeleteAllSpecimen = WidgetProperties.selection().observe(btnDeleteAllSpecimenDescriptions);
        IObservableValue deleteAllSpecimenConfiguratorObserveValue = PojoProperties.value("deleteAllSpecimenDescriptions").observe(configurator);
        bindingContext.bindValue(observeSelectionBtnDeleteAllSpecimen, deleteAllSpecimenConfiguratorObserveValue, null, null);

        IObservableValue observeSelectionBtnDeleteAllDefault = WidgetProperties.selection().observe(btnDeleteAllDefaultDescriptions);
        IObservableValue deleteAlldefaultConfiguratorObserveValue = PojoProperties.value("deleteAllDefaultDescriptions").observe(configurator);
        bindingContext.bindValue(observeSelectionBtnDeleteAllDefault, deleteAlldefaultConfiguratorObserveValue, null, null);

        IObservableValue observeSelectionBtnDeleteAllLiterature = WidgetProperties.selection().observe(btnDeleteAllLiteratureDescriptions);
        IObservableValue deleteAllLiteratureConfiguratorObserveValue = PojoProperties.value("deleteAllLiteratureDescriptions").observe(configurator);
        bindingContext.bindValue(observeSelectionBtnDeleteAllLiterature, deleteAllLiteratureConfiguratorObserveValue, null, null);


        IObservableValue observeSelectionBtnDeleteAllAggregated = WidgetProperties.selection().observe(btnDeleteAllAggregatedDescriptions);
        IObservableValue deleteAllAggregatedConfiguratorObserveValue = PojoProperties.value("deleteAllAggregatedDescriptions").observe(configurator);
        bindingContext.bindValue(observeSelectionBtnDeleteAllAggregated, deleteAllAggregatedConfiguratorObserveValue, null, null);





        return bindingContext;
    }


}
