/**
* Copyright (C) 2016 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.ui.dialog.configurator.deleteConfigurator;

import org.eclipse.core.databinding.DataBindingContext;
import org.eclipse.core.databinding.beans.PojoProperties;
import org.eclipse.core.databinding.observable.value.IObservableValue;
import org.eclipse.jface.databinding.swt.WidgetProperties;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.DisposeEvent;
import org.eclipse.swt.events.DisposeListener;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Event;
import org.eclipse.ui.forms.widgets.FormToolkit;

import eu.etaxonomy.cdm.api.service.config.MediaDeletionConfigurator;
import eu.etaxonomy.taxeditor.l10n.Messages;

/**
 * @author k.luther
 * @date 02.11.2016
 *
 */
public class DeleteMediaConfiguratorComposite extends Composite {
    private final DataBindingContext m_bindingContext;

    private final FormToolkit toolkit = new FormToolkit(Display.getCurrent());
    private final MediaDeletionConfigurator configurator;
    private Button btnDelete;
//    private Button btnDeleteIfUsedInTaxonDescription;
//    private Button btnDeleteIfUsedInSpecimenDescription;
    private Button btnDeleteIfUsedInSomeWhereElse;
    private Button btnRemoveFromImageGallery;
    
    private boolean inBulkEditor;


    public DeleteMediaConfiguratorComposite(MediaDeletionConfigurator configurator, Composite parent, int style, boolean inBulkEditor) {
        super(parent, style);
        this.inBulkEditor = inBulkEditor;
        this.configurator = configurator;
        addDisposeListener(new DisposeListener() {
            @Override
            public void widgetDisposed(DisposeEvent e) {
                toolkit.dispose();
            }
        });
        toolkit.paintBordersFor(this);
        setLayout(new RowLayout(SWT.VERTICAL));
        setBackground(getBackground());
        
        
//        if (inBulkEditor){
//            btnDeleteIfUsedInTaxonDescription = new Button(this, SWT.CHECK);
//            btnDeleteIfUsedInTaxonDescription.setText(Messages.DeleteConfiguration_media_deleteIfUsedInTaxonDescription);
//            btnDeleteIfUsedInTaxonDescription.setEnabled(true);
//            btnDeleteIfUsedInTaxonDescription.setSelection(configurator.isDeleteIfUsedInTaxonDescription());
//            btnDeleteIfUsedInSpecimenDescription = new Button(this, SWT.CHECK);
//            btnDeleteIfUsedInSpecimenDescription.setText(Messages.DeleteConfiguration_media_deleteIfUsedInSpecimenDescription);
//            btnDeleteIfUsedInSpecimenDescription.setEnabled(true);
//            btnDeleteIfUsedInSpecimenDescription.setSelection(configurator.isDeleteIfUsedInSpecimenDescription());
//        }
//        else {
        	
        	
        	//btnDelete.setSelection(configurator.setDeleteFrom(deleteFrom););
        	
        	btnDeleteIfUsedInSomeWhereElse = new Button(this, SWT.RADIO);
        	btnDeleteIfUsedInSomeWhereElse.setText(Messages.DeleteConfiguration_media_deleteIfUsedSomeWhereElse);
        	btnDeleteIfUsedInSomeWhereElse.setEnabled(true);
        	btnDeleteIfUsedInSomeWhereElse.setSelection(configurator.isDeleteFromEveryWhere());
        	
//            btnDeleteIfUsedInSpecimenDescription = new Button(this, SWT.CHECK);
//            btnDeleteIfUsedInSpecimenDescription.setText(Messages.DeleteConfiguration_media_deleteIfUsedInSpecimenDescription);
//            btnDeleteIfUsedInTaxonDescription = new Button(this, SWT.CHECK);
//            btnDeleteIfUsedInTaxonDescription.setText(Messages.DeleteConfiguration_media_deleteIfUsedInTaxonDescription);
//            btnDelete.addListener(SWT.Selection, (Event e) -> {
//                Button b = (Button) e.widget;
//                btnDeleteIfUsedInSpecimenDescription.setEnabled(b.getSelection());
//                btnDeleteIfUsedInTaxonDescription.setEnabled(b.getSelection());
//            });
        	if (!inBulkEditor){
        		btnDelete = new Button(this, SWT.RADIO);
            	btnDelete.setText(Messages.DeleteConfiguration_media_delete);
            	
	            btnRemoveFromImageGallery = new Button(this, SWT.RADIO);
	            btnRemoveFromImageGallery.setText(Messages.DeleteConfiguration_media_removeFromGallery);
	            btnRemoveFromImageGallery.setSelection(configurator.isOnlyRemoveFromGallery());
        	}
//        }

        m_bindingContext = initDataBindings();

    }

    protected DataBindingContext initDataBindings() {
        DataBindingContext bindingContext = new DataBindingContext();
        //
//        if (inBulkEditor){
//	        IObservableValue observeSelectionBtnDeleteIfUsedInTaxonDescription = WidgetProperties.selection().observe(btnDeleteIfUsedInTaxonDescription);
//	        IObservableValue deleteIfUsedInTaxonDescriptionConfiguratorObserveValue = PojoProperties.value("deleteIfUsedInTaxonDescription").observe(configurator);
//	        bindingContext.bindValue(observeSelectionBtnDeleteIfUsedInTaxonDescription, deleteIfUsedInTaxonDescriptionConfiguratorObserveValue, null, null);
//	        
//	        IObservableValue observeSelectionBtnDeleteIfUsedInSpecimenDescription = WidgetProperties.selection().observe(btnDeleteIfUsedInSpecimenDescription);
//	        IObservableValue deleteIfUsedInSpecimenDescriptionConfiguratorObserveValue = PojoProperties.value("deleteIfUsedInSpecimenDescription").observe(configurator);
//	        bindingContext.bindValue(observeSelectionBtnDeleteIfUsedInSpecimenDescription, deleteIfUsedInSpecimenDescriptionConfiguratorObserveValue, null, null);
//        } else{       
        	IObservableValue observeSelectionBtnDeleteIfUsedInSomeWhereElse = WidgetProperties.selection().observe(btnDeleteIfUsedInSomeWhereElse);
 	        IObservableValue deleteIfUsedInSomeWhereElseConfiguratorObserveValue = PojoProperties.value("deleteFromEveryWhere").observe(configurator);
 	        bindingContext.bindValue(observeSelectionBtnDeleteIfUsedInSomeWhereElse, deleteIfUsedInSomeWhereElseConfiguratorObserveValue, null, null);
 	        
//	        IObservableValue observeSelectionBtnDelete = WidgetProperties.selection().observe(btnDelete);
//	        IObservableValue deleteConfiguratorObserveValue = PojoProperties.value("deleteIfUsedInTaxonDescription").observe(configurator);
//	        bindingContext.bindValue(observeSelectionBtnDelete, deleteConfiguratorObserveValue, null, null);
 	       if (!inBulkEditor){
 	    	  IObservableValue observeSelectionBtnDelete = WidgetProperties.selection().observe(btnDelete);
	            IObservableValue deleteConfiguratorObserveValue = PojoProperties.value("deleteFromDescription").observe(configurator);
	            bindingContext.bindValue(observeSelectionBtnDelete, deleteConfiguratorObserveValue, null, null);
 	    	   
	            IObservableValue observeSelectionBtnRemoveFromImageGallery = WidgetProperties.selection().observe(btnRemoveFromImageGallery);
	            IObservableValue removeFromImageGalleryConfiguratorObserveValue = PojoProperties.value("onlyRemoveFromGallery").observe(configurator);
	            bindingContext.bindValue(observeSelectionBtnRemoveFromImageGallery, removeFromImageGalleryConfiguratorObserveValue, null, null);
 	       }

       

        return bindingContext;
    }


}
