/**
* Copyright (C) 2015 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.ui.dialog.configurator.deleteConfigurator;

import java.util.List;

import org.eclipse.core.databinding.DataBindingContext;
import org.eclipse.core.databinding.beans.PojoProperties;
import org.eclipse.core.databinding.observable.value.IObservableValue;
import org.eclipse.jface.databinding.swt.WidgetProperties;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.DisposeEvent;
import org.eclipse.swt.events.DisposeListener;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.ui.forms.widgets.FormToolkit;
import org.eclipse.wb.swt.SWTResourceManager;

import eu.etaxonomy.cdm.api.service.config.NameDeletionConfigurator;
import eu.etaxonomy.cdm.api.service.config.TaxonBaseDeletionConfigurator;
import eu.etaxonomy.cdm.model.taxon.Classification;


/**
 * @author pplitzner
 * @date Feb 18, 2015
 *
 */
public class DeleteTaxonBaseConfiguratorComposite extends Composite implements SelectionListener{

    protected  DataBindingContext m_bindingContext;

    private final FormToolkit toolkit = new FormToolkit(Display.getCurrent());
    protected final TaxonBaseDeletionConfigurator configurator;
    protected Button btnDeleteTaxonName;


    //name related
    protected Button btnRemoveAllNameRelationships;
    protected Button btnIgnoreIsBasionym;
    protected Button btnIgnoreIsOriginalSpelling;
    protected Button btnIgnoreIsReplacedSynonymFor;
    protected Button btnIgnoreHasBasionym;
    protected Button btnIgnoreHasReplacedSynonym;
    protected Button btnRemoveAllTypeDesignations;

    private Combo classificationSelectionCombo;


    Classification selectedClassification;
    private List<Classification> classifications;
    private Composite composite;


    /**
     * Create the composite.
     * @param parent
     * @param style
     */
    public DeleteTaxonBaseConfiguratorComposite(TaxonBaseDeletionConfigurator configurator, Composite parent, int style) {
        super(parent, style);
        this.configurator = configurator;
        addDisposeListener(new DisposeListener() {
            @Override
            public void widgetDisposed(DisposeEvent e) {
                toolkit.dispose();
            }
        });
        toolkit.paintBordersFor(this);
        setLayout(new GridLayout());
        setBackground(getBackground());
        createContent();

    }

    protected void createContent(){
        btnDeleteTaxonName = new Button(this, SWT.CHECK);
        btnDeleteTaxonName.setLayoutData(new GridData(SWT.LEFT, SWT.BOTTOM, false, false, 1, 1));
        btnDeleteTaxonName.setText("Delete name if possible");
        btnDeleteTaxonName.setSelection(true);
        btnDeleteTaxonName.addListener(SWT.Selection, new Listener() {
               @Override
            public void handleEvent(Event e) {
                   Button b = (Button) e.widget;
                   GridData data = (GridData)  btnDeleteTaxonName.getLayoutData();
                   data.exclude = b.getSelection();
                   composite.setEnabled(data.exclude);
                   btnRemoveAllNameRelationships.setEnabled(data.exclude);
                   btnIgnoreIsBasionym.setEnabled(data.exclude);
                   btnIgnoreIsOriginalSpelling.setEnabled(data.exclude);
                   btnIgnoreIsReplacedSynonymFor.setEnabled(data.exclude);
                   btnIgnoreHasBasionym.setEnabled(data.exclude);
                   btnIgnoreHasReplacedSynonym.setEnabled(data.exclude);
                   btnRemoveAllTypeDesignations.setEnabled(data.exclude);
               }
        });

        composite = new Composite(this, SWT.NO_BACKGROUND);
        GridLayout gl_composite = new GridLayout(1, false);
        gl_composite.horizontalSpacing = 1;
        gl_composite.marginHeight = 1;
        composite.setLayout(gl_composite);
        GridData gd_composite = new GridData(SWT.LEFT, SWT.CENTER, false, false, 1, 1);
        gd_composite.heightHint = 166;
        gd_composite.widthHint = 294;
        composite.setLayoutData(gd_composite);
        composite.setBackground(SWTResourceManager.getColor(SWT.COLOR_WIDGET_LIGHT_SHADOW));
        toolkit.adapt(composite);
        toolkit.paintBordersFor(composite);


        //DeleteConfigurator for the name


                btnRemoveAllNameRelationships = new Button(composite, SWT.CHECK);
                GridData gd_btnRemoveAllNameRelationships = new GridData(SWT.LEFT, SWT.CENTER, false, false, 1, 1);
                gd_btnRemoveAllNameRelationships.horizontalIndent = 5;
                btnRemoveAllNameRelationships.setLayoutData(gd_btnRemoveAllNameRelationships);
                btnRemoveAllNameRelationships.setText("Remove all name relations");
                btnRemoveAllNameRelationships.setSelection(true);

                btnRemoveAllTypeDesignations = new Button(composite, SWT.CHECK);
                GridData gd_btnRemoveAllTypeDesignations = new GridData(SWT.LEFT, SWT.CENTER, false, false, 1, 1);
                gd_btnRemoveAllTypeDesignations.horizontalIndent = 5;
                btnRemoveAllTypeDesignations.setLayoutData(gd_btnRemoveAllTypeDesignations);
                btnRemoveAllTypeDesignations.setText("Remove all type designations");
                btnRemoveAllTypeDesignations.setSelection(true);

                Label lblDelete = new Label(composite, SWT.NONE);
                GridData gd_lblDelete = new GridData(SWT.LEFT, SWT.CENTER, false, false, 1, 1);
                gd_lblDelete.horizontalIndent = 5;
                lblDelete.setLayoutData(gd_lblDelete);
                toolkit.adapt(lblDelete, true, true);
                lblDelete.setText("delete even if ");

                        btnIgnoreHasReplacedSynonym = new Button(composite, SWT.CHECK);
                        GridData gd_btnIgnoreHasReplacedSynonym = new GridData(SWT.LEFT, SWT.CENTER, false, false, 1, 1);
                        gd_btnIgnoreHasReplacedSynonym.horizontalIndent = 10;
                        btnIgnoreHasReplacedSynonym.setLayoutData(gd_btnIgnoreHasReplacedSynonym);
                        btnIgnoreHasReplacedSynonym.setText("it has replaced synonym(s)");
                        btnIgnoreHasReplacedSynonym.setSelection(true);

                        btnIgnoreHasBasionym = new Button(composite, SWT.CHECK);
                        GridData gd_btnIgnoreHasBasionym = new GridData(SWT.LEFT, SWT.CENTER, false, false, 1, 1);
                        gd_btnIgnoreHasBasionym.horizontalIndent = 10;
                        btnIgnoreHasBasionym.setLayoutData(gd_btnIgnoreHasBasionym);
                        btnIgnoreHasBasionym.setText("it has basionym(s)");
                        btnIgnoreHasBasionym.setSelection(true);


                        btnIgnoreIsReplacedSynonymFor = new Button(composite, SWT.CHECK);
                        GridData gd_btnIgnoreIsReplacedSynonymFor = new GridData(SWT.LEFT, SWT.CENTER, false, false, 1, 1);
                        gd_btnIgnoreIsReplacedSynonymFor.horizontalIndent = 10;
                        btnIgnoreIsReplacedSynonymFor.setLayoutData(gd_btnIgnoreIsReplacedSynonymFor);
                        btnIgnoreIsReplacedSynonymFor.setSelection(true);
                        btnIgnoreIsReplacedSynonymFor.setText("it is a replaced synonym");

        btnIgnoreIsBasionym = new Button(composite, SWT.CHECK);
        GridData gd_btnIgnoreIsBasionym = new GridData(SWT.LEFT, SWT.CENTER, false, false, 1, 1);
        gd_btnIgnoreIsBasionym.horizontalIndent = 10;
        btnIgnoreIsBasionym.setLayoutData(gd_btnIgnoreIsBasionym);
        btnIgnoreIsBasionym.setText("it is a basionym");
        btnIgnoreIsBasionym.setSelection(true);

        btnIgnoreIsOriginalSpelling = new Button(composite, SWT.CHECK);
        GridData gd_btnIgnoreIsOriginalSpelling = new GridData(SWT.LEFT, SWT.CENTER, false, false, 1, 1);
        gd_btnIgnoreIsOriginalSpelling.horizontalIndent = 10;
        btnIgnoreIsOriginalSpelling.setLayoutData(gd_btnIgnoreIsOriginalSpelling);
        btnIgnoreIsOriginalSpelling.setText("it is original spelling");
        btnIgnoreIsOriginalSpelling.setSelection(true);




       // createClassificationSelectionCombo(this);
        initDataBindings();
    }

    protected void initDataBindings() {
        if (m_bindingContext == null){
            m_bindingContext = new DataBindingContext();
        }
        //
        IObservableValue<?> observeSelectionBtnDeleteTaxonNameObserveWidget = WidgetProperties.selection().observe(btnDeleteTaxonName);
        IObservableValue<?> deleteNameIfPossibleConfiguratorObserveValue = PojoProperties.value("deleteNameIfPossible").observe(configurator);
        m_bindingContext.bindValue(observeSelectionBtnDeleteTaxonNameObserveWidget, deleteNameIfPossibleConfiguratorObserveValue, null, null);

        NameDeletionConfigurator nameConfig = configurator.getNameDeletionConfig();
        IObservableValue<?> observeSelectionbtnRemoveAllNameRelationshipsObserveWidget = WidgetProperties.selection().observe(btnRemoveAllNameRelationships);
        IObservableValue<?> deleteRemoveAllNameRelationshipsConfiguratorObserveValue = PojoProperties.value("removeAllNameRelationships").observe(nameConfig);
        m_bindingContext.bindValue(observeSelectionbtnRemoveAllNameRelationshipsObserveWidget, deleteRemoveAllNameRelationshipsConfiguratorObserveValue, null, null);

        IObservableValue<?> observeSelectionbtnIgnoreIsBasionymObserveWidget = WidgetProperties.selection().observe(btnIgnoreIsBasionym);
        IObservableValue<?> deleteIgnoreIsBasionymConfiguratorObserveValue = PojoProperties.value("ignoreIsBasionymFor").observe(nameConfig);
        m_bindingContext.bindValue(observeSelectionbtnIgnoreIsBasionymObserveWidget, deleteIgnoreIsBasionymConfiguratorObserveValue, null, null);

        IObservableValue<?> observeSelectionbtnIgnoreIsOriginalSpellingObserveWidget = WidgetProperties.selection().observe(btnIgnoreIsOriginalSpelling);
        IObservableValue<?> deleteIgnoreIsOriginalSpellingConfiguratorObserveValue = PojoProperties.value("ignoreIsOriginalSpellingFor").observe(nameConfig);
        m_bindingContext.bindValue(observeSelectionbtnIgnoreIsOriginalSpellingObserveWidget, deleteIgnoreIsOriginalSpellingConfiguratorObserveValue, null, null);

        IObservableValue<?> observeSelectionbtnIgnoreIsReplacedSynonymForObserveWidget = WidgetProperties.selection().observe(btnIgnoreIsReplacedSynonymFor);
        IObservableValue<?> deleteIgnoreIsReplacedSynonymForConfiguratorObserveValue = PojoProperties.value("ignoreIsReplacedSynonymFor").observe(nameConfig);
        m_bindingContext.bindValue(observeSelectionbtnIgnoreIsReplacedSynonymForObserveWidget, deleteIgnoreIsReplacedSynonymForConfiguratorObserveValue, null, null);

        IObservableValue<?> observeSelectionbtnIgnoreHasBasionymObserveWidget = WidgetProperties.selection().observe(btnIgnoreHasBasionym);
        IObservableValue<?> deleteIgnoreHasBasionymConfiguratorObserveValue = PojoProperties.value("ignoreHasBasionym").observe(nameConfig);
        m_bindingContext.bindValue(observeSelectionbtnIgnoreHasBasionymObserveWidget, deleteIgnoreHasBasionymConfiguratorObserveValue, null, null);

        IObservableValue<?> observeSelectionbtnIgnoreHasReplacedSynonymObserveWidget = WidgetProperties.selection().observe(btnIgnoreHasReplacedSynonym);
        IObservableValue<?> deleteIgnoreHasReplacedSynonymConfiguratorObserveValue = PojoProperties.value("ignoreHasReplacedSynonym").observe(nameConfig);
        m_bindingContext.bindValue(observeSelectionbtnIgnoreHasReplacedSynonymObserveWidget, deleteIgnoreHasReplacedSynonymConfiguratorObserveValue, null, null);

        IObservableValue<?> observeSelectionbtnRemoveAllTypeDesignationsObserveWidget = WidgetProperties.selection().observe(btnRemoveAllTypeDesignations);
        IObservableValue<?> deleteRemoveAllTypeDesignationsConfiguratorObserveValue = PojoProperties.value("removeAllTypeDesignations").observe(nameConfig);
        m_bindingContext.bindValue(observeSelectionbtnRemoveAllTypeDesignationsObserveWidget, deleteRemoveAllTypeDesignationsConfiguratorObserveValue, null, null);


    }

  /* private Control createClassificationSelectionCombo(Composite parent){
//		classifications = CdmStore.getTaxonTreeService().list(null, null, null, null, null);

		Composite classificationSelection = new Composite(parent, SWT.NULL);
		classificationSelection.setLayoutData(new GridData(SWT.FILL, SWT.TOP, true, true));

		GridLayout layout = new GridLayout();
		classificationSelection.setLayout(layout);

		Label label = new Label(classificationSelection, SWT.NULL);
		// TODO not working is not really true but leave it here to remind everyone that this is under construction
		label.setText("Select Classification");
		classificationSelectionCombo = new Combo(classificationSelection, SWT.BORDER | SWT.READ_ONLY);
		classificationSelectionCombo.setLayoutData(new GridData(SWT.FILL, SWT.TOP, true, true));
		classifications = CdmStore.getService(IClassificationService.class).list(null, null, null, null, null);

		for(Classification tree : classifications){
			classificationSelectionCombo.add(tree.getName().getText(), classifications.indexOf(tree));

		}

		classificationSelectionCombo.select(0);
		classificationSelectionCombo.addSelectionListener(this);

		return classificationSelection;
	}*/

@Override
public void widgetSelected(SelectionEvent e) {
	selectedClassification = classifications.get(classificationSelectionCombo.getSelectionIndex());
}

@Override
public void widgetDefaultSelected(SelectionEvent e) {
	// TODO Auto-generated method stub

}
}
