/**
* Copyright (C) 2016 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.ui.dialog.configurator.deleteConfigurator;

import org.eclipse.core.databinding.DataBindingContext;
import org.eclipse.core.databinding.beans.PojoProperties;
import org.eclipse.core.databinding.observable.value.IObservableValue;
import org.eclipse.jface.databinding.swt.WidgetProperties;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Listener;

import eu.etaxonomy.cdm.api.service.config.TaxonDeletionConfigurator;
import eu.etaxonomy.cdm.api.service.config.TaxonNodeDeletionConfigurator;

/**
 * @author k.luther
 * @date 19.04.2016
 *
 */
public class DeleteTaxonConfiguratorComposite extends DeleteTaxonBaseConfiguratorComposite {

    private Button btnDeleteSynonyms;
    private Button btnDeleteMisappliedNames;
    private Button btnTaxonRelationships;
    private Button btnConceptRelationships;

    private Button btnDeleteTaxon;

    /**
     * @param configurator
     * @param parent
     * @param style
     */
    public DeleteTaxonConfiguratorComposite(TaxonDeletionConfigurator configurator, Composite parent, int style) {
        super(configurator, parent, style);
    }

    @Override
    protected void createContent(){
        btnDeleteTaxon = new Button(this, SWT.CHECK);
        btnDeleteTaxon.setText("Delete taxon if possible");
//        btnDeleteTaxon.setSelection(true);
        btnDeleteTaxon.addListener(SWT.Selection, new Listener() {
            @Override
         public void handleEvent(Event e) {
                Button b = (Button) e.widget;
                GridData data = (GridData)  btnDeleteTaxon.getLayoutData();
                data.exclude = b.getSelection();

                btnDeleteSynonyms.setEnabled(data.exclude);
                btnDeleteMisappliedNames.setEnabled(data.exclude);
                btnTaxonRelationships.setEnabled(data.exclude);
                btnConceptRelationships.setEnabled(data.exclude);
                btnDeleteTaxonName.setEnabled(data.exclude);
                btnIgnoreHasBasionym.setEnabled(data.exclude);
                btnIgnoreHasReplacedSynonym.setEnabled(data.exclude);
                btnIgnoreIsBasionym.setEnabled(data.exclude);
                btnIgnoreIsReplacedSynonymFor.setEnabled(data.exclude);
                btnRemoveAllNameRelationships.setEnabled(data.exclude);
                btnRemoveAllTypeDesignations.setEnabled(data.exclude);

            }
        });

        btnDeleteSynonyms = new Button(this, SWT.CHECK);
        GridData gd_btnDeleteSynonyms = new GridData(SWT.LEFT, SWT.CENTER, false, false, 1, 1);
        gd_btnDeleteSynonyms.horizontalIndent = 10;
        btnDeleteSynonyms.setLayoutData(gd_btnDeleteSynonyms);
        btnDeleteSynonyms.setText("Delete synonyms if possible");
        btnDeleteSynonyms.setSelection(true);

        btnDeleteMisappliedNames = new Button(this, SWT.CHECK);
        GridData gd_btnDeleteMisappliedNames = new GridData(SWT.LEFT, SWT.CENTER, false, false, 1, 1);
        gd_btnDeleteMisappliedNames.horizontalIndent = 10;
        btnDeleteMisappliedNames.setLayoutData(gd_btnDeleteMisappliedNames);
        btnDeleteMisappliedNames.setText("Delete misapplied names if possible");
        btnDeleteMisappliedNames.setSelection(true);


        btnTaxonRelationships = new Button(this, SWT.CHECK);
        GridData gd_btnTaxonRelationships = new GridData(SWT.LEFT, SWT.CENTER, false, false, 1, 1);
        gd_btnTaxonRelationships.horizontalIndent = 10;
        btnTaxonRelationships.setLayoutData(gd_btnTaxonRelationships);
        btnTaxonRelationships.setText("Delete taxon relations if possible");
        btnTaxonRelationships.setSelection(true);

        btnConceptRelationships = new Button(this, SWT.CHECK);
        GridData gd_btnConceptRelationships = new GridData(SWT.LEFT, SWT.CENTER, false, false, 1, 1);
        gd_btnConceptRelationships.horizontalIndent = 10;
        btnConceptRelationships.setLayoutData(gd_btnConceptRelationships);
        btnConceptRelationships.setText("Delete concept relations if possible");
        btnConceptRelationships.setSelection(true);
        super.createContent();
        initDataBindingsTaxon();
    }

    protected void initDataBindingsTaxon() {
        if (m_bindingContext == null){
            m_bindingContext = new DataBindingContext();
        }
        TaxonNodeDeletionConfigurator nodeConfig = ((TaxonDeletionConfigurator)configurator).getTaxonNodeConfig();
        if (nodeConfig == null){
            nodeConfig = new TaxonNodeDeletionConfigurator();
            ((TaxonDeletionConfigurator)configurator).setTaxonNodeConfig(nodeConfig);
        }
        IObservableValue observeSelectionBtnDeleteTaxonObserveWidget = WidgetProperties.selection().observe(btnDeleteTaxon);
        IObservableValue deleteTaxonConfiguratorObserveValue = PojoProperties.value("deleteTaxon").observe(nodeConfig);
        m_bindingContext.bindValue(observeSelectionBtnDeleteTaxonObserveWidget, deleteTaxonConfiguratorObserveValue, null, null);

        IObservableValue observeSelectionBtnDeleteSynonymsObserveWidget = WidgetProperties.selection().observe(btnDeleteSynonyms);
        IObservableValue deleteSynonymsConfiguratorObserveValue = PojoProperties.value("deleteSynonymsIfPossible").observe(configurator);
        m_bindingContext.bindValue(observeSelectionBtnDeleteSynonymsObserveWidget, deleteSynonymsConfiguratorObserveValue, null, null);

        IObservableValue observeSelectionbtnDeleteMisappliedNamesObserveWidget = WidgetProperties.selection().observe(btnDeleteMisappliedNames);
        IObservableValue deleteMisappliedNamesConfiguratorObserveValue = PojoProperties.value("deleteMisappliedNames").observe(configurator);
        m_bindingContext.bindValue(observeSelectionbtnDeleteMisappliedNamesObserveWidget, deleteMisappliedNamesConfiguratorObserveValue, null, null);

        IObservableValue observeSelectionbtnTaxonRelationshipsObserveWidget = WidgetProperties.selection().observe(btnTaxonRelationships);
        IObservableValue deleteTaxonRelationshipsConfiguratorObserveValue = PojoProperties.value("deleteTaxonRelationships").observe(configurator);
        m_bindingContext.bindValue(observeSelectionbtnTaxonRelationshipsObserveWidget, deleteTaxonRelationshipsConfiguratorObserveValue, null, null);

        IObservableValue observeSelectionbtnConceptRelationshipsObserveWidget = WidgetProperties.selection().observe(btnConceptRelationships);
        IObservableValue deleteConceptRelationshipsConfiguratorObserveValue = PojoProperties.value("deleteConceptRelationships").observe(configurator);
        m_bindingContext.bindValue(observeSelectionbtnConceptRelationshipsObserveWidget, deleteConceptRelationshipsConfiguratorObserveValue, null, null);

    }
}
