/**
* Copyright (C) 2019 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.ui.dialog.configurator.wizard;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.List;
import java.util.UUID;

import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;

import eu.etaxonomy.cdm.api.service.IVocabularyService;
import eu.etaxonomy.cdm.model.location.NamedAreaLevel;
import eu.etaxonomy.cdm.persistence.dto.TermDto;
import eu.etaxonomy.taxeditor.editor.definedterm.DefinedTermSorter;
import eu.etaxonomy.taxeditor.editor.definedterm.TermDtoContentProvider;
import eu.etaxonomy.taxeditor.editor.definedterm.TermDtoLabelProvider;
import eu.etaxonomy.taxeditor.preference.wizard.AvailableDistributionPage;
import eu.etaxonomy.taxeditor.preference.wizard.CheckBoxTreeComposite;
import eu.etaxonomy.taxeditor.store.CdmStore;

/**
 * @author k.luther
 * @since 28.11.2019
 */
public class SuperAreaPage extends AvailableDistributionPage {

    private NamedAreaLevel level;
    /**
     * @param pageName
     * @param level
     */
    public SuperAreaPage(String pageName, NamedAreaLevel level) {
        super(pageName);
        this.level = level;
    }

    @Override
    public void createControl(Composite parent) {
        setTitle("Select super areas");
        setDescription("Select areas to which the sub areas should be aggregated");

        parent.setLayout(new GridLayout());
        treeComposite = new CheckBoxTreeComposite(parent, new TermDtoContentProvider(), new TermDtoLabelProvider(), SWT.NONE);
        treeComposite.getViewer().setComparator(new DefinedTermSorter());

        treeComposite.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1));
        setControl(treeComposite);
        setLevelChecked();

    }

    /**
     *
     */
    private void setLevelChecked() {
        initialiseVocabularies();
        List<TermDto> areasChecked= new ArrayList();
        List<UUID> vocUuidList = new ArrayList<>();
        getVocabularies().stream().forEach(p -> vocUuidList.add(p.getUuid()));
        Collection<TermDto> terms = CdmStore.getService(IVocabularyService.class).getNamedAreaTerms(vocUuidList);
        for (TermDto term: terms) {
            if (term.getLevel() != null && term.getLevel().equals(level)){
                areasChecked.add(term);
            }
        }

        treeComposite.getViewer().setInput(getVocabularies());
        Collections.sort(areasChecked, (o1, o2)->o1.getOrderIndex()-o2.getOrderIndex());

        treeComposite.setCheckedElements(areasChecked.toArray());
        treeComposite.update();

    }

    @Override
    protected String getCheckedValuesFromPreferences() {

        return null;
    }

}
