/**
* Copyright (C) 2021 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.ui.dialog.selection;

import java.lang.reflect.Member;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;
import java.util.UUID;

import org.eclipse.jface.viewers.ILabelProvider;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.swt.widgets.Shell;

import eu.etaxonomy.cdm.api.service.IAgentService;
import eu.etaxonomy.cdm.api.service.dto.EntityDTOBase;
import eu.etaxonomy.cdm.api.service.dto.IdentifiedEntityDTO;
import eu.etaxonomy.cdm.model.agent.AgentBase;
import eu.etaxonomy.cdm.model.agent.Person;
import eu.etaxonomy.cdm.model.agent.Team;
import eu.etaxonomy.cdm.model.agent.TeamOrPersonBase;
import eu.etaxonomy.cdm.model.metadata.PreferencePredicate;
import eu.etaxonomy.cdm.persistence.dto.TeamOrPersonUuidAndTitleCache;
import eu.etaxonomy.cdm.persistence.dto.UuidAndTitleCache;
import eu.etaxonomy.taxeditor.newWizard.AbstractNewEntityWizard;
import eu.etaxonomy.taxeditor.newWizard.NewPersonWizard;
import eu.etaxonomy.taxeditor.newWizard.NewTeamWizard;
import eu.etaxonomy.taxeditor.preference.PreferencesUtil;
import eu.etaxonomy.taxeditor.store.CdmStore;

/**
 * @author k.luther
 * @since Jul 1, 2021
 */
public class CollectorSelectionDialog extends AgentSelectionDialog<AgentBase<?>> {
	Team collectorTeam = null;

    /**
     * @param shell
     * @param title
     * @param multi
     * @param settings
     * @param agent
     * @param selectTeamMember
     */
    protected CollectorSelectionDialog(Shell shell, String title, boolean multi, String settings,
            AgentBase<?> agent, boolean selectTeamMember) {
        super(shell, title, multi, settings, agent, selectTeamMember);
        if (selectTeamMember && agent instanceof Team){
    		this.collectorTeam = (Team)agent;
    		
    	}

    }
    public static <S extends AgentBase> S select(Shell shell,
            S entity, boolean selectTeamMember) {
    	
        CollectorSelectionDialog dialog = new CollectorSelectionDialog(shell,
                "Choose Collector", false, CollectorSelectionDialog.class.getCanonicalName(), entity, selectTeamMember);
        return  (S) getSelectionFromDialog(dialog);
    }

    @Override
    protected ILabelProvider createListLabelProvider() {
        return new CollectorLabelProvider();
    }
    
    @Override
	protected AbstractNewEntityWizard getNewEntityWizard(String parameter) {
    	if(TEAM.equals(parameter)){
            return new NewTeamWizard(false, true);
        }
        else if(PERSON.equals(parameter)){
            return new NewPersonWizard();
        }
        else{
            throw new IllegalArgumentException("Could not determine the desired wizard.");
        }
    
	}

    public class CollectorLabelProvider extends LabelProvider {
        @Override
        public String getText(Object element) {
            if (element == null) {
                return null;
            }
            UuidAndTitleCache uuidAndTitleCache = (UuidAndTitleCache) element;
            String title = "";
            String abbrevTitleCache = uuidAndTitleCache.getAbbrevTitleCache();
            String titleCache = uuidAndTitleCache.getTitleCache();
            String collectorTitleCache = null;
            if (element instanceof TeamOrPersonUuidAndTitleCache && ((TeamOrPersonUuidAndTitleCache)element).getCollectorTitleCache() != null){
                title = ((TeamOrPersonUuidAndTitleCache)element).getCollectorTitleCache();
            }
            if (!title.equals(titleCache)){
                title += " - " + titleCache;
            }
            if (abbrevTitleCache != null && !uuidAndTitleCache.getTitleCache().equals(abbrevTitleCache) && !abbrevTitleCache.equals(titleCache)){
                title += " - " + abbrevTitleCache;
            }
            if(PreferencesUtil.getBooleanValue(PreferencePredicate.ShowIdInSelectionDialog.getKey())){
                title += " ["+uuidAndTitleCache.getId()+"]";
            }
            if (element instanceof EntityDTOBase){
                title += "(" + ((IdentifiedEntityDTO)element).getIdentifier().getTypeLabel() +": " + ((IdentifiedEntityDTO)element).getIdentifier().getIdentifier() + ")";
            }

            return title;
        }


    }
    @Override
    protected void callService(String pattern) {
        Class clazz = AgentBase.class;
        if (selectTeamMember){
            clazz = Person.class;
        }
        Set<UUID> memberUuids = new HashSet<>();
        if (collectorTeam != null && selectTeamMember){
        	List<Person> teamMembers = this.collectorTeam.getTeamMembers();
        	
            teamMembers.forEach(member -> memberUuids.add(member.getUuid()));
        }
        
        model = CdmStore.getService(IAgentService.class).getUuidAndTitleCacheWithCollectorTitleCache(clazz, limitOfInitialElements, pattern);
        if (collectorTeam == null){
        	return;
        }
        //filter
        Iterator<UuidAndTitleCache<AgentBase<?>>> modelIterator = model.iterator();
        List<UuidAndTitleCache<AgentBase<?>>> tempModel = new ArrayList<>();
        while(modelIterator.hasNext()){
        	UuidAndTitleCache<AgentBase<?>> person = modelIterator.next();
        	if (memberUuids.contains(person.getUuid())){
        		tempModel.add(person);
        	}
        }
        model.clear();
        model.addAll(tempModel);
    }

    @Override
    protected String getTitle(AgentBase<?> cdmObject) {
		if(cdmObject == null){
			return "";
		}

		if (cdmObject instanceof TeamOrPersonBase) {
			return ((TeamOrPersonBase<?>) cdmObject).getCollectorTitleCache();
		}

		throw new IllegalArgumentException("Generic method only" +
				" supports cdmObject of type TeamOrPersonBase." +
				" Please implement specific method in subclass.");
	}
}
