/**
* Copyright (C) 2018 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.ui.dialog.selection;

import java.util.ArrayList;
import java.util.List;
import java.util.UUID;

import org.apache.commons.lang3.StringUtils;
import org.eclipse.swt.widgets.Shell;

import eu.etaxonomy.cdm.api.service.IVocabularyService;
import eu.etaxonomy.cdm.model.location.NamedArea;
import eu.etaxonomy.cdm.model.metadata.CdmPreference;
import eu.etaxonomy.cdm.model.metadata.CdmPreference.PrefKey;
import eu.etaxonomy.cdm.model.metadata.PreferencePredicate;
import eu.etaxonomy.cdm.model.metadata.PreferenceSubject;
import eu.etaxonomy.cdm.model.term.TermType;
import eu.etaxonomy.cdm.model.term.TermVocabulary;
import eu.etaxonomy.taxeditor.preference.CdmPreferenceCache;
import eu.etaxonomy.taxeditor.preference.PreferencesUtil;
import eu.etaxonomy.taxeditor.store.CdmStore;

/**
 * @author k.luther
 * @since 09.10.2018
 *
 */
public class CommonNameNamedAreaSelectionDialog extends NamedAreaSelectionDialog {

    /**
     * @param shell
     * @param title
     * @param multi
     * @param namedArea
     * @param preferenceId
     * @param preselectedVocabularyUuids
     */
    protected CommonNameNamedAreaSelectionDialog(Shell shell, String title, boolean multi, NamedArea namedArea,
            Object preferenceId) {
        super(shell, title, multi, namedArea, preferenceId, createVocabularyUuidList());


    }

    private static UUID[] createVocabularyUuidList() {
        String preselectedVocString = PreferencesUtil.getStringValue(PreferencePredicate.CommonNameAreaVocabularies.getKey(), false);
        if (StringUtils.isBlank(preselectedVocString)){
            return null;
        }
        String[] preselectedVocArray = preselectedVocString.split(";");
        UUID[] uuidList = new UUID[preselectedVocArray.length];
        int i = 0;
        for (String uuidString: preselectedVocArray){
            uuidList[i]= UUID.fromString(uuidString);
            i++;
        }
        return uuidList;
    }
    @Override
    protected void init() {
        vocabularies = getAvailableVocabularies();
    }


    protected List<TermVocabulary> getAvailableVocabularies(){
        List<TermVocabulary> vocabularies = new ArrayList();

        UUID[] preselectedVocabularyUuids = createVocabularyUuidList();
        CdmPreferenceCache cache = CdmPreferenceCache.instance();
        PrefKey key = CdmPreference.NewKey(PreferenceSubject.NewTaxEditorInstance(), PreferencePredicate.CommonNameAreaVocabularies);
        CdmPreference pref = cache.findBestMatching(key);
        if (preselectedVocabularyUuids != null){
            for(int i=0;i<preselectedVocabularyUuids.length;i++){
                TermVocabulary preselectedVocabulary = CdmStore.getService(IVocabularyService.class).find(preselectedVocabularyUuids[i]);
                vocabularies.add(preselectedVocabulary);
            }
        }
        if ((pref != null && !pref.isAllowOverride()) && preselectedVocabularyUuids!=null) {
             return vocabularies;
        }else{
            selectedVocabularies = vocabularies;

            return CdmStore.getService(IVocabularyService.class).listByTermType(TermType.NamedArea, true, null, null, null, null);
        }


    }

    /**
     * Creates a filtered selection dialog to select a named area.
     *
     * @param shell
     *              The shell for displaying this widget
     * @param namedArea
     *              A namedArea that should be selected when the dialog opens
     * @param conversation a {@link eu.etaxonomy.cdm.api.conversation.ConversationHolder} object.
     * @param preferenceId a class which is used for generating the preference key so that every
     * dialogs can be grouped to have their own preferences depending on this id
     * @param preselectedVocabularyUuids the {@link UUID}s of the pre-selected vocabularies
     * @return a {@link eu.etaxonomy.cdm.model.location.NamedArea} object.
     */
    public static NamedArea select(Shell shell, //ConversationHolder conversation,
            NamedArea namedArea, String preferenceId) {
        CommonNameNamedAreaSelectionDialog dialog = new CommonNameNamedAreaSelectionDialog(shell, //conversation,
                "Choose an area", false, namedArea, preferenceId);
        return getSelectionFromDialog(dialog);
    }


}
