/**
 * Copyright (C) 2014 EDIT
 * European Distributed Institute of Taxonomy
 * http://www.e-taxonomy.eu
 *
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */
package eu.etaxonomy.taxeditor.ui.dialog.selection;

import java.util.UUID;

import org.eclipse.jface.dialogs.InputDialog;
import org.eclipse.jface.window.Window;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Shell;

import eu.etaxonomy.cdm.api.service.IDescriptiveDataSetService;
import eu.etaxonomy.cdm.hibernate.HibernateProxyHelper;
import eu.etaxonomy.cdm.model.description.DescriptiveDataSet;
import eu.etaxonomy.taxeditor.model.MessagingUtils;
import eu.etaxonomy.taxeditor.newWizard.AbstractNewEntityWizard;
import eu.etaxonomy.taxeditor.store.CdmStore;

/**
 * @author pplitzner
 * @date 11.03.2014
 *
 */
public class DescriptiveDataSetSelectionDialog extends
AbstractFilteredCdmResourceSelectionDialog<DescriptiveDataSet> {

    public static DescriptiveDataSet select(Shell shell, DescriptiveDataSet descriptiveDataSet){
        DescriptiveDataSetSelectionDialog dialog = new DescriptiveDataSetSelectionDialog(shell,
                "Choose descriptive dataset", false, DescriptiveDataSetSelectionDialog.class.getCanonicalName(), descriptiveDataSet);
        return getSelectionFromDialog(dialog);
    }

    /**
     * <p>Constructor for FilteredDerivedUnitSelectionDialog.</p>
     */
    protected DescriptiveDataSetSelectionDialog(Shell shell, String title,
            boolean multi, String settings, DescriptiveDataSet cdmObject) {
        super(shell, title, multi, settings, cdmObject);
    }

    /** {@inheritDoc} */
    @Override
    protected DescriptiveDataSet getPersistentObject(UUID uuid) {
        Object object = CdmStore.getService(IDescriptiveDataSetService.class).load(uuid);

        DescriptiveDataSet descriptiveDataSet = (DescriptiveDataSet) HibernateProxyHelper.deproxy(object);

        if(descriptiveDataSet != null){
            return descriptiveDataSet;
        }
        MessagingUtils.error(this.getClass(), "Selected element is not a descriptive dataset", null);
        return null;
    }

    /** {@inheritDoc} */
    @Override
    protected void callService(String pattern) {
        model = CdmStore.getService(IDescriptiveDataSetService.class).getDescriptiveDataSetUuidAndTitleCache(limitOfInitialElements, pattern);
    }

    @Override
    protected String getTitle(DescriptiveDataSet cdmObject) {
        if(cdmObject!=null){
            return cdmObject.getLabel();
        }
        return super.getTitle(cdmObject);
    }

    /** {@inheritDoc} */
    @Override
    protected AbstractNewEntityWizard getNewEntityWizard(String parameter) {
        return null;
    }

    @Override
    protected SelectionListener getNewWizardButtonSelectionListener(){
        return new SelectionAdapter() {

            @Override
            public void widgetSelected(SelectionEvent e) {
                Object source = e.getSource();
                if (source instanceof Button){
                    Button sourceButton = (Button) source;
                }
                InputDialog dialog = new InputDialog(getShell(), "Descriptive Dataset label", "Enter label for descriptive dataset", null, null);
                if (dialog.open() == Window.OK) {
                    // User clicked OK; update the label with the input
                	DescriptiveDataSet descriptiveDataSet = DescriptiveDataSet.NewInstance();
                    CdmStore.getService(IDescriptiveDataSetService.class).merge(descriptiveDataSet,true);
                    descriptiveDataSet.setLabel(dialog.getValue());
                    refresh();
                    setPattern(descriptiveDataSet);
                  }
            }
        };
    }

    /** {@inheritDoc} */
    @Override
    protected String[] getNewWizardText() {
        return new String[]{"Descriptive Dataset "};
    }

}
