/**
* Copyright (C) 2017 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.ui.dialog.selection;

import java.util.UUID;

import org.eclipse.swt.widgets.Shell;

import eu.etaxonomy.cdm.api.service.IReferenceService;
import eu.etaxonomy.cdm.config.ICdmSource;
import eu.etaxonomy.cdm.model.agent.Person;
import eu.etaxonomy.cdm.model.agent.Team;
import eu.etaxonomy.cdm.model.agent.TeamOrPersonBase;
import eu.etaxonomy.cdm.model.reference.OriginalSourceType;
import eu.etaxonomy.cdm.model.reference.Reference;
import eu.etaxonomy.cdm.model.reference.ReferenceFactory;
import eu.etaxonomy.taxeditor.newWizard.NewReferenceWizard;
import eu.etaxonomy.taxeditor.remoting.source.CdmServerInfo;
import eu.etaxonomy.taxeditor.store.CdmStore;

/**
 * @author k.luther
 * @date 02.05.2017
 */
public class ExtReferenceSelectionDialog extends
        AbstractFilteredCdmResourceSelectionDialog<Reference> {

    protected static boolean isInReference = false;
    private Reference currentReference;
    private ICdmSource remoteSource ;
    private Reference remoteSourceRef;
    private IReferenceService service = null;

    public static Reference select(Shell shell,
            Reference reference, boolean isInReference) {
        ExtReferenceSelectionDialog dialog = new ExtReferenceSelectionDialog(shell,
                "Choose a reference", false, reference, isInReference);
        return getSelectionFromDialog(dialog);
    }

    public static Reference select(Shell shell,
            Reference reference) {
        ExtReferenceSelectionDialog dialog = new ExtReferenceSelectionDialog(shell,
                "Choose a reference", false, reference);
        return getSelectionFromDialog(dialog);
    }

    protected ExtReferenceSelectionDialog(Shell shell,
            String title, boolean multi, Reference reference) {
        super(shell, title, multi, ExtReferenceSelectionDialog.class.getCanonicalName());
        this.currentReference = reference;
//            controller = CdmStore.getCurrentApplicationConfiguration();
        remoteSource = CdmServerInfo.getDevServerRemoteSource();
        remoteSourceRef = ReferenceFactory.newDatabase();
        remoteSourceRef.setTitle(remoteSource.getName());
    }

    protected ExtReferenceSelectionDialog(Shell shell,
            String title, boolean multi, Reference reference, boolean isInReference) {
        super(shell, title, multi, ExtReferenceSelectionDialog.class.getCanonicalName());
        ExtReferenceSelectionDialog.isInReference = isInReference;
        this.currentReference = reference;

//            controller = CdmStore.getCurrentApplicationConfiguration();
//            CdmServerInfoConfig parameterObject = new CdmServerInfoConfig("localhost", "localhost", 8080, "cdmserver/", true);
//            CdmServerInfo selectedCsii = new CdmServerInfo(parameterObject);
//            CdmInstanceInfo selectedCdmInstance = selectedCsii.getInstanceFromName("local-cuba") ;
//            Integer port;
//            remoteSource = selectedCsii.getCdmRemoteSource(selectedCdmInstance, 80);
    }

    @Override
    protected Reference getPersistentObject(UUID cdmUuid) {
        Reference extRef = service.load(cdmUuid);

        Reference newRef = ReferenceFactory.newReference( extRef.getType());
        if (extRef.getTitle() != null){
            newRef.setTitle(extRef.getTitle());
        }else{
            newRef.setTitleCache(extRef.getTitleCache(), true);
        }
        if (extRef.getAbbrevTitle() != null){
            newRef.setAbbrevTitle(extRef.getAbbrevTitle());
        }else{
            newRef.setAbbrevTitleCache(extRef.getAbbrevTitleCache(), true);
        }

        if (extRef.getInReference() != null){
            Reference inRef = extRef.getInReference();
            Reference newRefInRef = ReferenceFactory.newReference( extRef.getInBook().getType());
            if (inRef.getTitle() != null){
                newRefInRef.setTitle(inRef.getTitle());
            }else{
                newRefInRef.setTitleCache(inRef.getTitleCache(), true);
            }
            if (inRef.getAbbrevTitle() != null){
                newRefInRef.setAbbrevTitle(inRef.getAbbrevTitle());
            }else{
                newRefInRef.setAbbrevTitleCache(inRef.getAbbrevTitleCache(), true);
            }
            newRefInRef.addSource(OriginalSourceType.Other, String.valueOf(newRefInRef.getId()), "Reference", remoteSourceRef, null);
            newRef.setInReference(newRefInRef);
            newRef.setProtectedTitleCache(false);
            newRef.setProtectedAbbrevTitleCache(false);
            newRef.addSource(OriginalSourceType.Other, String.valueOf(newRef.getId()), "Reference", remoteSourceRef, null);
        }
        TeamOrPersonBase<?> author;
        if (extRef.getAuthorship() != null){
            if (extRef.getAuthorship() instanceof Person){
                 author = Person.NewTitledInstance(extRef.getAuthorship().getTitleCache());
                 author.setProtectedTitleCache(true);
            }else{
               author = Team.NewInstance();
               Person newMember;

               for (Person member:((Team)extRef.getAuthorship()).getTeamMembers()){
                   newMember = Person.NewTitledInstance(member.getTitleCache());
                   ((Team)author).addTeamMember(newMember);
               }
            }
            newRef.setAuthorship(author);
        }
        newRef = CdmStore.getService(IReferenceService.class).save(newRef);
        return newRef;
    }

    @Override
    protected void callService(String pattern) {

        if (remoteSource != null){
//               if (service == null){
//                   service = CdmApplicationRemoteConfiguration.getExtReferenceService(remoteSource);
//               }
           if (pattern.equals("?")){ //$NON-NLS-1$
                if (isInReference && currentReference != null){
                    model = service.getUuidAndTitleCache(null, null, currentReference.getType());
                }else{
                    model = service.getUuidAndTitleCache(null, null);
                }
            }else{
                if (isInReference && currentReference != null){
                    model = service.getUuidAndTitleCache(limitOfInitialElements, pattern, currentReference.getType());
                }else{
                    model = service.getUuidAndTitleCache(limitOfInitialElements,pattern);
                }
            }
         }
    }

    @Override
    protected String getTitle(Reference cdmObject) {
        if(cdmObject == null){
            return "";
        }else{
            return super.getTitle(cdmObject);
        }
//          }else{
//              return DefaultReferenceCacheStrategy.putAuthorToEndOfString(cdmObject.getTitleCache(), cdmObject.getAuthorship().getTitleCache());
//          }
    }

    @Override
    protected NewReferenceWizard getNewEntityWizard(String parameter) {
        return new NewReferenceWizard(null);
    }

    @Override
    protected String[] getNewWizardText() {
        return new String[]{"Reference "};
    }
}
