/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.ui.dialog.selection;

import java.util.ArrayList;
import java.util.List;
import java.util.UUID;

import org.eclipse.jface.dialogs.InputDialog;
import org.eclipse.jface.window.Window;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.widgets.Shell;

import eu.etaxonomy.cdm.api.service.ITermTreeService;
import eu.etaxonomy.cdm.model.term.TermTree;
import eu.etaxonomy.cdm.model.term.TermType;
import eu.etaxonomy.taxeditor.l10n.Messages;
import eu.etaxonomy.taxeditor.newWizard.AbstractNewEntityWizard;
import eu.etaxonomy.taxeditor.store.CdmStore;

/**
 * @author n.hoffmann
 * @created Sep 17, 2010
 */
public class FeatureTreeSelectionDialog extends
		AbstractFilteredCdmResourceSelectionDialog<TermTree> {

    private final TermType termType;

    public static TermTree select(Shell shell,
    		TermTree featureTree) {
        return select(shell, featureTree, null);

    }
	public static TermTree select(Shell shell,
	        TermTree featureTree, TermType termType) {
		FeatureTreeSelectionDialog dialog = new FeatureTreeSelectionDialog(shell,
				Messages.FeatureTreeSelectionDialog_CHOOSE_TREE, false, featureTree, termType);
		return getSelectionFromDialog(dialog);
	}

	protected FeatureTreeSelectionDialog(Shell shell,
			 String title, boolean multi,
			 TermTree cdmObject, TermType termType) {
		super(shell, title, multi, FeatureTreeSelectionDialog.class.getCanonicalName(), cdmObject);
		this.termType = termType;
	}

	@Override
	protected TermTree getPersistentObject(UUID uuid) {
	    List<String> propertyPath = new ArrayList<>();
	    propertyPath.add("root.*");
		return CdmStore.getService(ITermTreeService.class).load(uuid, propertyPath);
	}

	@Override
	protected void callService(String pattern) {
        model = CdmStore.getService(ITermTreeService.class).getUuidAndTitleCacheByTermType(TermTree.class, termType, limitOfInitialElements, pattern);
	}

	@Override
    protected SelectionListener getNewWizardButtonSelectionListener(){
        return new SelectionAdapter() {

            @Override
            public void widgetSelected(SelectionEvent e) {
                InputDialog dialog = new InputDialog(getShell(), Messages.FeatureTreeSelectionDialog_TREE_LABEL, Messages.FeatureTreeSelectionDialog_ENTER_LABEL, null, null);
                if (dialog.open() == Window.OK) {
                    // User clicked OK; update the label with the input
                	TermTree tree = null;
                    if(termType!=null){
                        tree = TermTree.NewInstance(termType);
                    }
                    else{
                        tree = TermTree.NewInstance();
                    }
                    tree.setTitleCache(dialog.getValue(), true);
                    CdmStore.getService(ITermTreeService.class).merge(tree,true);
                    refresh();
                    setPattern(tree);
                  }
            }
        };
    }

	@Override
	protected String[] getNewWizardText() {
		return new String[]{ Messages.FeatureTreeSelectionDialog_NEW_TREE};
	}

	@Override
	protected AbstractNewEntityWizard getNewEntityWizard(String parameter) {
		return null;
	}
}
