// $Id$
/**
* Copyright (C) 2017 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.ui.dialog.selection;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Shell;

import eu.etaxonomy.cdm.model.term.TermVocabulary;
import eu.etaxonomy.taxeditor.preference.PreferencesUtil;

/**
 * @author k.luther
 * @date 21.11.2017
 *
 */
public class FilterDialog extends Dialog {

    List<TermVocabulary> vocabularies = new ArrayList<TermVocabulary>();

    List<TermVocabulary> selectedVocabularies = new ArrayList<TermVocabulary>();
    List<TermVocabulary> tempSelectedVocabularies = new ArrayList<TermVocabulary>();
    boolean performOk = true;

    Object preferenceId;

    /**
     * @param parentShell
     */
    protected FilterDialog(Shell parentShell, Object preferenceId, List<TermVocabulary> selectedVocabularies, List<TermVocabulary> vocabularies) {
        super(parentShell);
        this.vocabularies = vocabularies;
        this.preferenceId = preferenceId;
        this.selectedVocabularies = selectedVocabularies;
        this.tempSelectedVocabularies = new ArrayList();
        this.tempSelectedVocabularies.addAll(selectedVocabularies);


    }

    @Override
    protected Control createDialogArea(Composite parent) {
        Composite dialogArea = new Composite(parent, SWT.NULL);
        dialogArea.setLayout(new GridLayout(1, false));
        for (TermVocabulary voc: vocabularies){
            Button btnCheckButton = new Button(dialogArea, SWT.CHECK);
            btnCheckButton.setText(voc.getLabel());
            btnCheckButton.setData(voc);
            if (selectedVocabularies.contains(voc)){
                btnCheckButton.setSelection(true);
            }else {
                btnCheckButton.setSelection(false);
            }

            btnCheckButton.addSelectionListener(new SelectionListener(){

                @Override
                public void widgetSelected(SelectionEvent e) {
                   if (btnCheckButton.getSelection()){
                       if (!tempSelectedVocabularies.contains(btnCheckButton.getData())){
                    	   tempSelectedVocabularies.add((TermVocabulary)btnCheckButton.getData());
                    	   if (!tempSelectedVocabularies.isEmpty()) {
                    		   getButton(IDialogConstants.OK_ID).setEnabled(true);
                    	   }
                       }
                   }else{
                       if (tempSelectedVocabularies.contains(btnCheckButton.getData())){
                    	   tempSelectedVocabularies.remove(btnCheckButton.getData());
                    	   if (tempSelectedVocabularies.isEmpty()) {
                    		   getButton(IDialogConstants.OK_ID).setEnabled(false);
                    	   }
                       }
                   }

                }

                @Override
                public void widgetDefaultSelected(SelectionEvent e) {
                    // TODO Auto-generated method stub

                }

            });
        }

        GridData gd_table = new GridData(SWT.CENTER, SWT.CENTER, true, true, 2, 1);
        gd_table.heightHint = 231;
        gd_table.widthHint = 543;

        return dialogArea;

    }

    @Override
    protected void okPressed(){

        for (TermVocabulary voc: vocabularies){
            if (tempSelectedVocabularies.contains(voc)){
                PreferencesUtil.setBooleanValue(getPrefKey(voc), false);
                selectedVocabularies.add(voc);
            } else{
                PreferencesUtil.setBooleanValue(getPrefKey(voc), true);
                selectedVocabularies.remove(voc);
            }
        }
        super.okPressed();
    }

    private String getPrefKey(TermVocabulary vocabulary){
        return "hide_"+NamedAreaSelectionDialog.class.getCanonicalName()+vocabulary.getUuid()+preferenceId;
    }

}
