/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.ui.dialog.selection;

import java.util.ArrayList;
import java.util.List;
import java.util.UUID;

import org.eclipse.swt.widgets.Shell;

import eu.etaxonomy.cdm.api.service.IGroupService;
import eu.etaxonomy.cdm.common.CdmUtils;
import eu.etaxonomy.cdm.model.permission.Group;
import eu.etaxonomy.cdm.model.permission.User;
import eu.etaxonomy.cdm.persistence.dto.UuidAndTitleCache;
import eu.etaxonomy.taxeditor.newWizard.AbstractNewEntityWizard;
import eu.etaxonomy.taxeditor.newWizard.NewGroupWizard;
import eu.etaxonomy.taxeditor.store.CdmStore;

/**
 * @author n.hoffmann
 * @created Mar 10, 2011
 * @version 1.0
 */
public class GroupSelectionDialog extends
		AbstractFilteredCdmResourceSelectionDialog<Group> {

    User user;

	public static Group select(Shell shell, //ConversationHolder conversation,
	        Group group, User user){
		GroupSelectionDialog dialog = new GroupSelectionDialog(shell, //conversation,
				"Choose Group", false, GroupSelectionDialog.class.getCanonicalName(), group, user);
		return getSelectionFromDialog(dialog);
	}

	/**
	 * @param shell
	 * @param conversation
	 * @param title
	 * @param multi
	 * @param settings
	 * @param cdmObject
	 */
	protected GroupSelectionDialog(Shell shell,//ConversationHolder conversation,
	        String title, boolean multi,
			String settings, Group cdmObject, User user) {
		super(shell, //conversation,
		        title, multi, settings, cdmObject);
		this.user = user;
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.ui.dialogs.filteredSelection.AbstractFilteredCdmResourceSelectionDialog#search
	 */
	@Override
	protected void callService(String pattern) {
		List<Group> groups = CdmStore.getService(IGroupService.class).list(Group.class, null, null, null, null);

		model = new ArrayList<UuidAndTitleCache<Group>>();

		for(Group group : groups){
		    if (pattern == null || group.getName().matches("(?i)"+CdmUtils.quoteRegExWithWildcard(pattern) + ".*")){
		        if (user != null && !user.getGroups().isEmpty() && user.getGroups().contains(group)){
		            continue;
		        }
		        model.add(new UuidAndTitleCache<Group>(Group.class, group.getUuid(), group.getId(), String.format("%s", group.getName())));
		    }
		}
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.ui.dialogs.filteredSelection.AbstractFilteredCdmResourceSelectionDialog#getNewWizardLinkText()
	 */
	@Override
	protected String[] getNewWizardText() {
		return null;
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.ui.dialogs.filteredSelection.AbstractFilteredCdmResourceSelectionDialog#getNewEntityWizard(java.lang.String)
	 */
	@Override
	protected AbstractNewEntityWizard getNewEntityWizard(String parameter) {
		return new NewGroupWizard();
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.ui.dialogs.filteredSelection.AbstractFilteredCdmResourceSelectionDialog#getPersistentObject(java.util.UUID)
	 */
	@Override
	protected Group getPersistentObject(UUID uuid) {
		return CdmStore.getService(IGroupService.class).load(uuid);
	}

	@Override
	protected String getTitle(Group group) {
		return group != null ? group.getName() : "";
	}

}
