/**
 * Copyright (C) 2014 EDIT
 * European Distributed Institute of Taxonomy
 * http://www.e-taxonomy.eu
 *
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */
package eu.etaxonomy.taxeditor.ui.dialog.selection;

import java.util.UUID;

import org.eclipse.swt.widgets.Shell;

import eu.etaxonomy.cdm.api.service.IMediaService;
import eu.etaxonomy.cdm.hibernate.HibernateProxyHelper;
import eu.etaxonomy.cdm.model.media.Media;
import eu.etaxonomy.taxeditor.model.MessagingUtils;
import eu.etaxonomy.taxeditor.newWizard.AbstractNewEntityWizard;
import eu.etaxonomy.taxeditor.store.CdmStore;

/**
 * @author pplitzner
 * @date 11.03.2014
 *
 */
public class MediaSelectionDialog extends
AbstractFilteredCdmResourceSelectionDialog<Media> {

    public static Media select(Shell shell, //ConversationHolder conversation,
            Media media){
        MediaSelectionDialog dialog = new MediaSelectionDialog(shell, //conversation,
                "Choose Media", false, MediaSelectionDialog.class.getCanonicalName(), media);
        return getSelectionFromDialog(dialog);
    }

    /**
     * <p>Constructor for FilteredDerivedUnitSelectionDialog.</p>
     */
    protected MediaSelectionDialog(Shell shell, //ConversationHolder conversation,
            String title,
            boolean multi, String settings, Media cdmObject) {
        super(shell, //conversation,
                title, multi, settings, cdmObject);
    }

    /** {@inheritDoc} */
    @Override
    protected Media getPersistentObject(UUID uuid) {
        Object object = CdmStore.getService(IMediaService.class).load(uuid);

        Media media = (Media) HibernateProxyHelper.deproxy(object);

        if(media != null){
            return media;
        }
        MessagingUtils.error(this.getClass(), "Selected element is not media", null);
        return null;
    }

    /** {@inheritDoc} */
    @Override
    protected void callService(String pattern) {
        model = CdmStore.getService(IMediaService.class).getUuidAndTitleCache(limitOfInitialElements, pattern);

    }

    /* (non-Javadoc)
     * @see eu.etaxonomy.taxeditor.ui.dialog.selection.AbstractFilteredCdmResourceSelectionDialog#getTitle(eu.etaxonomy.cdm.model.common.ICdmBase)
     */
    @Override
    protected String getTitle(Media cdmObject) {
        if(cdmObject!=null){
            return cdmObject.getTitleCache();
        }
        return super.getTitle(cdmObject);
    }

    /** {@inheritDoc} */
    @Override
    protected AbstractNewEntityWizard getNewEntityWizard(String parameter) {
        return null;
    }

    /** {@inheritDoc} */
    @Override
    protected String[] getNewWizardText() {
        return null;
    }
}
