/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.ui.dialog.selection;

import java.util.UUID;

import org.eclipse.swt.widgets.Shell;

import eu.etaxonomy.cdm.api.service.INameService;
import eu.etaxonomy.cdm.model.name.TaxonName;
import eu.etaxonomy.taxeditor.newWizard.AbstractNewEntityWizard;
import eu.etaxonomy.taxeditor.newWizard.NewNonViralNameWizard;
import eu.etaxonomy.taxeditor.store.CdmStore;

/**
 * @author n.hoffmann
 * @created 04.06.2009
 */
public class NameSelectionDialog extends AbstractFilteredCdmResourceSelectionDialog<TaxonName> {

	public static TaxonName select(Shell shell,
	        TaxonName name) {
		NameSelectionDialog dialog = new NameSelectionDialog(shell,
				"Choose a name", false, name);
		return getSelectionFromDialog(dialog);
	}

	protected NameSelectionDialog(Shell shell,
	        String title, boolean multi, TaxonName name) {
		super(shell,
		        title, multi, NameSelectionDialog.class.getCanonicalName(), name);
	}

	@Override
	protected TaxonName getPersistentObject(UUID cdmUuid) {
		return CdmStore.getService(INameService.class).load(cdmUuid);
	}

	@Override
	protected AbstractNewEntityWizard getNewEntityWizard(String parameter) {
		return new NewNonViralNameWizard();
	}

	@Override
	protected String[] getNewWizardText() {
		return new String[]{"New Name"};
	}

    @Override
    protected void callService(String pattern) {
        model = CdmStore.getService(INameService.class).getUuidAndTitleCache(limitOfInitialElements, pattern);
    }
}