/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.ui.dialog.selection;

import java.util.ArrayList;
import java.util.List;
import java.util.UUID;

import org.apache.commons.lang3.StringUtils;
import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Shell;

import eu.etaxonomy.cdm.api.service.ITermService;
import eu.etaxonomy.cdm.api.service.IVocabularyService;
import eu.etaxonomy.cdm.model.location.NamedArea;
import eu.etaxonomy.cdm.model.metadata.PreferencePredicate;
import eu.etaxonomy.cdm.model.metadata.TermSearchField;
import eu.etaxonomy.cdm.model.term.DefinedTermBase;
import eu.etaxonomy.cdm.model.term.TermType;
import eu.etaxonomy.cdm.model.term.TermVocabulary;
import eu.etaxonomy.taxeditor.model.ImageResources;
import eu.etaxonomy.taxeditor.model.MessagingUtils;
import eu.etaxonomy.taxeditor.newWizard.AbstractNewEntityWizard;
import eu.etaxonomy.taxeditor.preference.CdmPreferenceCache;
import eu.etaxonomy.taxeditor.preference.PreferencesUtil;
import eu.etaxonomy.taxeditor.store.CdmStore;

/**
 * @author n.hoffmann
 * @created May 11, 2010
 */
public class NamedAreaSelectionDialog extends
		AbstractFilteredCdmResourceSelectionDialog<NamedArea> {

//	private class IncludeNamedAreaVocabulary extends Action {
//		private final TermVocabulary<NamedArea> vocabulary;
//
//		/**
//		 * Creates a new instance of the class.
//		 */
//		public IncludeNamedAreaVocabulary(TermVocabulary<NamedArea> vocabulary) {
//			super(vocabulary.getTitleCache(), IAction.AS_CHECK_BOX);
//			this.vocabulary = vocabulary;
//		}
//
//		@Override
//        public void run(){
//			if(isChecked()){
//				selectedVocabularies.add(vocabulary);
//			}else{
//				selectedVocabularies.remove(vocabulary);
//			}
//			PreferencesUtil.setBooleanValue(getPrefKey(vocabulary), !isChecked());
//			search();
//		}
//	}

	protected List<NamedArea> selectedAreas;
	protected List<TermVocabulary> selectedVocabularies;
    protected List<TermVocabulary> vocabularies;

	/**
	 * Creates a filtered selection dialog to select a named area.
	 *
	 * @param shell
	 * 				The shell for displaying this widget
	 * @param namedArea
	 * 				A namedArea that should be selected when the dialog opens
	 * @param conversation a {@link eu.etaxonomy.cdm.api.conversation.ConversationHolder} object.
	 * @param preferenceId a class which is used for generating the preference key so that every
	 * dialogs can be grouped to have their own preferences depending on this id
	 * @param preselectedVocabularyUuids the {@link UUID}s of the pre-selected vocabularies
	 * @return a {@link eu.etaxonomy.cdm.model.location.NamedArea} object.
	 */
	public static NamedArea select(Shell shell,
	        NamedArea namedArea, String preferenceId, UUID... preselectedVocabularyUuids) {
		NamedAreaSelectionDialog dialog = new NamedAreaSelectionDialog(shell,
				"Choose an area", false, namedArea, preferenceId, preselectedVocabularyUuids);
		return getSelectionFromDialog(dialog);
	}

	protected NamedAreaSelectionDialog(Shell shell,
	        String title, boolean multi, NamedArea namedArea, Object preferenceId, UUID... preselectedVocabularyUuids) {
		super(shell,
		        title, multi, NamedAreaSelectionDialog.class.getCanonicalName(), namedArea);

		selectedVocabularies = new ArrayList<TermVocabulary>();
		this.preferenceID = preferenceId;

		if (preselectedVocabularyUuids != null && preselectedVocabularyUuids.length > 0){
    		for(int i=0;i<preselectedVocabularyUuids.length;i++){
    			TermVocabulary preselectedVocabulary = CdmStore.getService(IVocabularyService.class).find(preselectedVocabularyUuids[i]);
    			selectedVocabularies.add(preselectedVocabulary);
    		}
		}else{
		    selectedVocabularies = createSelectedVocabularies();
		}
	}

    protected List<TermVocabulary> createSelectedVocabularies() {
        List<TermVocabulary> tempSelectedVocabularies = new ArrayList<TermVocabulary>();
        for(TermVocabulary vocabulary:vocabularies){
			if((selectedVocabularies.contains(vocabulary) && !PreferencesUtil.getBooleanValue(getPrefKey(vocabulary)))
					|| !PreferencesUtil.getBooleanValue(getPrefKey(vocabulary))){
				tempSelectedVocabularies.add(vocabulary);
			}
		}
        return tempSelectedVocabularies;
    }

//    private static UUID[] createVocabularyUuidList() {
//        String preselectedVocString = PreferencesUtil.getStringValue(PreferencePredicate.AvailableDistributionAreaVocabularies.getKey());
//        if (StringUtils.isBlank(preselectedVocString)){
//            return null;
//        }
//        String[] preselectedVocArray = preselectedVocString.split(";");
//        UUID[] uuidList = new UUID[preselectedVocArray.length];
//        int i = 0;
//        for (String uuidString: preselectedVocArray){
//            uuidList[i]= UUID.fromString(uuidString);
//            i++;
//        }
//        return uuidList;
//    }

	private String getPrefKey(TermVocabulary vocabulary){
		return "hide_"+NamedAreaSelectionDialog.class.getCanonicalName()+vocabulary.getUuid()+preferenceID;
	}

	@Override
	protected NamedArea getPersistentObject(UUID uuid) {

	    DefinedTermBase area =  CdmStore.getService(ITermService.class).find(uuid);
	    if (area instanceof NamedArea){
	        return (NamedArea) area;
	    }

		return null;
	}

	@Override
	protected void init() {
		vocabularies = getAvailableVocabularies();
	}

	private List<TermVocabulary> getAvailableVocabularies(){
	    vocabularies = new ArrayList<>();
	    CdmPreferenceCache cache = CdmPreferenceCache.instance();
	    vocabularies = CdmStore.getService(IVocabularyService.class).listByTermType(TermType.NamedArea, true, null, null, null, null);
	    return vocabularies;
	}

	@Override
	protected String getTitle(NamedArea namedArea) {
		try {
			String result = NamedArea.labelWithLevel(namedArea, CdmStore.getDefaultLanguage());
			return result;
		} catch (Exception e) {
			MessagingUtils.error(NamedAreaSelectionDialog.class, "Error occurred when trying retrieve title for Named Area: " + namedArea.getUuid(), e);
			return namedArea.getTitleCache();
		}
	}

	@Override
	protected AbstractNewEntityWizard getNewEntityWizard(String parameter) {
		return null;
	}

	@Override
	protected String[] getNewWizardText() {
		return null;
	}

	@Override
    void createFilterButton(Composite searchAndFilter){
        filterButton = new Button(searchAndFilter, SWT.NONE);
//            filterButton.setText("Filter");
        filterButton.setImage(ImageResources.getImage(ImageResources.FUNNEL_ICON));
//            SelectionListener filterSelectionListener = new FilterSelectionListener(preferenceID, this);
        filterButton.addSelectionListener(new SelectionListener(){
            @Override
            public void widgetSelected(SelectionEvent e) {
                Object source = e.getSource();
                if (source instanceof Button){
                    Dialog dialog = new FilterDialog(getShell(), preferenceID, selectedVocabularies, vocabularies);
                    dialog.open();
                    createSelectedVocabularies();
                    search();
                }
            }

            @Override
            public void widgetDefaultSelected(SelectionEvent e) {
                // TODO Auto-generated method stub
            }
        });
    }

    @Override
    protected void callService(String pattern) {
        String abbrevTypeString = PreferencesUtil.getStringValue(PreferencePredicate.NamedAreaSearchForAbbrev.getKey());
        TermSearchField abbrevType;
        try{
            abbrevType = StringUtils.isNotBlank(abbrevTypeString)? TermSearchField.byKey(abbrevTypeString): null;
        }catch(IllegalArgumentException e){
            abbrevType = null;
        }
        if (selectedVocabularies == null || selectedVocabularies.size() == 0){
            if (abbrevType != null && !abbrevType.equals(TermSearchField.NoAbbrev)){
                model = CdmStore.getService(ITermService.class).getUuidAndTitleCacheNamedAreaByAbbrev(vocabularies, limitOfInitialElements, pattern, PreferencesUtil.getGlobalLanguage(), abbrevType);
            }else{
                model = CdmStore.getService(ITermService.class).getUuidAndTitleCacheNamedArea(vocabularies, limitOfInitialElements, pattern, PreferencesUtil.getGlobalLanguage());
            }
        }else{
            if (abbrevType != null  && !abbrevType.equals(TermSearchField.NoAbbrev)){
                model = CdmStore.getService(ITermService.class).getUuidAndTitleCacheNamedAreaByAbbrev(selectedVocabularies, limitOfInitialElements, pattern, PreferencesUtil.getGlobalLanguage(), abbrevType);
            }else{
                model = CdmStore.getService(ITermService.class).getUuidAndTitleCacheNamedArea(selectedVocabularies, limitOfInitialElements, pattern, PreferencesUtil.getGlobalLanguage());
            }
        }
    }
}