/**
* Copyright (C) 2016 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.ui.dialog.selection;

import java.text.Collator;
import java.util.Comparator;
import java.util.List;

import org.eclipse.jface.viewers.ILabelProvider;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.swt.widgets.Shell;

import eu.etaxonomy.cdm.api.service.IAgentService;
import eu.etaxonomy.cdm.api.service.dto.EntityDTOBase;
import eu.etaxonomy.cdm.api.service.dto.IdentifiedEntityDTO;
import eu.etaxonomy.cdm.model.agent.INomenclaturalAuthor;
import eu.etaxonomy.cdm.model.agent.TeamOrPersonBase;
import eu.etaxonomy.cdm.model.metadata.PreferencePredicate;
import eu.etaxonomy.cdm.persistence.dto.TeamOrPersonUuidAndTitleCache;
import eu.etaxonomy.cdm.persistence.dto.UuidAndTitleCache;
import eu.etaxonomy.taxeditor.newWizard.AbstractNewEntityWizard;
import eu.etaxonomy.taxeditor.newWizard.NewPersonWizard;
import eu.etaxonomy.taxeditor.newWizard.NewTeamWizard;
import eu.etaxonomy.taxeditor.preference.PreferencesUtil;
import eu.etaxonomy.taxeditor.store.CdmStore;

/**
 * @author k.luther
 * @date 25.05.2016
 */
public class NomenclaturalAuthorSelectionDialog extends TeamOrPersonBaseSelectionDialog {

    protected NomenclaturalAuthorSelectionDialog(Shell shell,
            String title,
            boolean multi, String settings, TeamOrPersonBase<?> agent, boolean teamMemberSelection) {
        super(shell, title, multi, settings, agent, teamMemberSelection);

    }

    @Override
    protected void callService(String pattern) {
        model = (List)CdmStore.getService(IAgentService.class).getUuidAndAbbrevTitleCache(TeamOrPersonBase.class, limitOfInitialElements, pattern);
    }

    @Override
    protected AbstractNewEntityWizard<? extends TeamOrPersonBase<?>> getNewEntityWizard(String parameter) {
        if(TEAM.equals(parameter)){
            return new NewTeamWizard(true, false);
        }
        else if(PERSON.equals(parameter)){
            return new NewPersonWizard();
        }
        else{
            throw new IllegalArgumentException("Could not determine the desired wizard.");
        }
    }

//    @Override
//	protected ILabelProvider createListLabelProvider() {
//		return new FilteredCdmResourceAbbrevLabelProvider();
//	}

    /**
     * <p>select</p>
     *
     * @param shell a {@link org.eclipse.swt.widgets.Shell} object.
     * @param conversation a {@link eu.etaxonomy.cdm.api.conversation.ConversationHolder} object.
     * @param entity a {@link eu.etaxonomy.cdm.model.agent.AgentBase} object.
     * @return a {@link eu.etaxonomy.cdm.model.agent.AgentBase} object.
     */
    public static TeamOrPersonBase<?> select(Shell shell,
            TeamOrPersonBase<?> entity, boolean teamMemberSelection) {
        NomenclaturalAuthorSelectionDialog dialog = new NomenclaturalAuthorSelectionDialog(shell, //conversation,
                "Choose Agent", false, NomenclaturalAuthorSelectionDialog.class.getCanonicalName(), entity, teamMemberSelection);
        return getSelectionFromDialog(dialog);
    }

    @Override
    protected String getTitle(TeamOrPersonBase<?> cdmObject) {
        if(cdmObject == null){
            return "";
        }

        return ((INomenclaturalAuthor)cdmObject).getNomenclaturalTitleCache();
    }

    @Override
    protected ILabelProvider createListLabelProvider() {
        return new NomenclaturalLabelProvider();
    }

    public class NomenclaturalLabelProvider extends LabelProvider {
        @Override
        public String getText(Object element) {
            if (element == null) {
                return null;
            }
            UuidAndTitleCache<?> uuidAndTitleCache = (UuidAndTitleCache<?>) element;
            String title = uuidAndTitleCache.getAbbrevTitleCache();
            String titleCache = uuidAndTitleCache.getTitleCache();
            String collectorTitleCache = null;
            if (element instanceof TeamOrPersonUuidAndTitleCache){
                collectorTitleCache = ((TeamOrPersonUuidAndTitleCache)element).getCollectorTitleCache();
            }
            if (!titleCache.equals(title)){
                title += " - " + titleCache;
            }
            if (collectorTitleCache != null && !uuidAndTitleCache.getTitleCache().equals(collectorTitleCache) && !titleCache.equals(collectorTitleCache)){
                title += " - " + collectorTitleCache;
            }
            if(PreferencesUtil.getBooleanValue(PreferencePredicate.ShowIdInSelectionDialog.getKey())){
                title += " ["+uuidAndTitleCache.getId()+"]";
            }
            if (element instanceof EntityDTOBase){
                title += "(" + ((IdentifiedEntityDTO)element).getIdentifier().getTypeLabel() +": " + ((IdentifiedEntityDTO)element).getIdentifier().getIdentifier() + ")";
            }

            return title;
        }
    }

	@Override
	protected Comparator<UuidAndTitleCache<TeamOrPersonBase<?>>> getItemsComparator() {
		return new Comparator<UuidAndTitleCache<TeamOrPersonBase<?>>>() {
			@Override
			public int compare(UuidAndTitleCache<TeamOrPersonBase<?>> entity1,
					UuidAndTitleCache<TeamOrPersonBase<?>> entity2) {

			    if (entity1.getUuid().equals(entity2.getUuid())){
			        return 0;
			    }
				Collator collator = Collator.getInstance();
				String compareString1 = "";
				if (entity1.getAbbrevTitleCache() != null){
					compareString1 = entity1.getAbbrevTitleCache();
				}
				if (entity1.getTitleCache() != null){
					compareString1 += entity1.getTitleCache();
				}

				String compareString2 = "";
				if (entity2.getAbbrevTitleCache() != null){
					compareString2 = entity2.getAbbrevTitleCache();
				}
				if (entity2.getTitleCache() != null){
					compareString2 += entity2.getTitleCache();
				}


				int result =collator.compare(compareString1, compareString2);
				if (result == 0){
				    result = entity1.getUuid().compareTo(entity2.getUuid());
				}
				return result;
			}
		};
	}


}
