/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.ui.dialog.selection;

import java.util.List;
import java.util.UUID;

import org.eclipse.swt.widgets.Shell;

import eu.etaxonomy.cdm.api.service.IAgentService;
import eu.etaxonomy.cdm.model.agent.Person;
import eu.etaxonomy.taxeditor.newWizard.AbstractNewEntityWizard;
import eu.etaxonomy.taxeditor.newWizard.NewPersonWizard;
import eu.etaxonomy.taxeditor.store.CdmStore;

/**
 * <p>FilteredPersonSelectionDialog class.</p>
 *
 * @author n.hoffmann
 * @created Sep 24, 2009
 * @version 1.0
 */
public class PersonSelectionDialog extends
			AgentSelectionDialog<Person> {

	/**
	 * <p>select</p>
	 *
	 * @param shell a {@link org.eclipse.swt.widgets.Shell} object.
	 * @param conversation a {@link eu.etaxonomy.cdm.api.conversation.ConversationHolder} object.
	 * @param person a {@link eu.etaxonomy.cdm.model.agent.Person} object.
	 * @return a {@link eu.etaxonomy.cdm.model.agent.Person} object.
	 */
	public static Person select(Shell shell, //onversationHolder conversation,
	        Person person){
		PersonSelectionDialog dialog = new PersonSelectionDialog(shell, //conversation,
				"Choose a person", false, PersonSelectionDialog.class.getCanonicalName(), person);
		return getSelectionFromDialog(dialog);
	}

	/**
	 * <p>Constructor for FilteredPersonSelectionDialog.</p>
	 *
	 * @param shell a {@link org.eclipse.swt.widgets.Shell} object.
	 * @param title a {@link java.lang.String} object.
	 * @param multi a boolean.
	 * @param settings a {@link java.lang.String} object.
	 * @param cdmObject a {@link eu.etaxonomy.cdm.model.agent.Person} object.
	 * @param conversation a {@link eu.etaxonomy.cdm.api.conversation.ConversationHolder} object.
	 */
	protected PersonSelectionDialog(Shell shell,// ConversationHolder conversation,
	        String title,
			boolean multi, String settings, Person cdmObject) {
		super(shell, title, multi, settings, cdmObject, true);
	}



	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.dialogs.filteredSelection.AbstractFilteredCdmResourceSelectionDialog#getPersistentObject(java.util.UUID)
	 */
	/** {@inheritDoc} */
	@Override
	protected Person getPersistentObject(UUID uuid) {
		return (Person) CdmStore.getService(IAgentService.class).load(uuid);
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.dialogs.filteredSelection.AbstractFilteredCdmResourceSelectionDialog#search
	 */
	/** {@inheritDoc} */
	@Override
	protected void callService(String pattern) {
		model = CdmStore.getService(IAgentService.class).getUuidAndTitleCache(Person.class, limitOfInitialElements, pattern);
	}

	/** {@inheritDoc} */
	@Override
	protected AbstractNewEntityWizard getNewEntityWizard(String parameter) {
		return new NewPersonWizard();
	}

	/** {@inheritDoc} */
	@Override
	protected String[] getNewWizardText() {
		return new String[]{"New Person"};
	}
}
