/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.ui.dialog.selection;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.UUID;

import org.apache.commons.lang3.StringUtils;
import org.eclipse.jface.viewers.ILabelProvider;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;

import eu.etaxonomy.cdm.api.service.IReferenceService;
import eu.etaxonomy.cdm.api.service.dto.IdentifiedEntityDTO;
import eu.etaxonomy.cdm.api.service.dto.IdentifiedEntityDTO.AlternativeIdentifier;
import eu.etaxonomy.cdm.model.metadata.PreferencePredicate;
import eu.etaxonomy.cdm.model.reference.Reference;
import eu.etaxonomy.cdm.persistence.dto.UuidAndTitleCache;
import eu.etaxonomy.cdm.persistence.query.MatchMode;
import eu.etaxonomy.taxeditor.newWizard.AbstractNewEntityWizard;
import eu.etaxonomy.taxeditor.newWizard.NewReferenceWizard;
import eu.etaxonomy.taxeditor.preference.PreferencesUtil;
import eu.etaxonomy.taxeditor.store.CdmStore;

/**
 * <p>FilteredReferenceSelectionDialog class.</p>
 *
 * @author n.hoffmann
 * @created 04.06.2009
 * @version 1.0
 */
public class ReferenceSelectionDialog extends AbstractFilteredCdmResourceSelectionDialog<Reference> {

    protected static boolean isInReference = false;
    private Reference currentReference;
    List<String> lastSelectedReferences = null;
    Map<UUID, AlternativeIdentifier> identifierMap;
    Set<Reference> preSelectedReferences = new HashSet<>();

	/**
	 * <p>select</p>
	 *
	 * @param reference a {@link eu.etaxonomy.cdm.model.reference.ReferenceBase} object.
	 * @param shell a {@link org.eclipse.swt.widgets.Shell} object.
	 * @param conversation a {@link eu.etaxonomy.cdm.api.conversation.ConversationHolder} object.
	 * @return a {@link eu.etaxonomy.cdm.model.reference.ReferenceBase} object.
	 */
	public static Reference select(Shell shell, //ConversationHolder conversation,
	        Reference reference, boolean isInReference) {
		ReferenceSelectionDialog dialog = new ReferenceSelectionDialog(shell, //conversation,
				"Choose a reference", false, reference, isInReference);
		return getSelectionFromDialog(dialog);
	}

	/**
     * <p>select</p>
     *
     * @param reference a {@link eu.etaxonomy.cdm.model.reference.ReferenceBase} object.
     * @param shell a {@link org.eclipse.swt.widgets.Shell} object.
     * @param conversation a {@link eu.etaxonomy.cdm.api.conversation.ConversationHolder} object.
     * @return a {@link eu.etaxonomy.cdm.model.reference.ReferenceBase} object.
     */
    public static Reference select(Shell shell, //ConversationHolder conversation,
            Reference reference) {
        ReferenceSelectionDialog dialog = new ReferenceSelectionDialog(shell, //conversation,
                "Choose a reference", false, reference, null);
        return getSelectionFromDialog(dialog);
    }

    public static Reference select(Shell shell, String title,
            Reference reference, Set<Reference> preSelectedReferences) {
        ReferenceSelectionDialog dialog = new ReferenceSelectionDialog(shell, //conversation,
                title, false, reference, preSelectedReferences);
        return getSelectionFromDialog(dialog);
    }




	/**
	 * <p>Constructor for FilteredReferenceSelectionDialog.</p>
	 *
	 * @param shell a {@link org.eclipse.swt.widgets.Shell} object.
	 * @param title a {@link java.lang.String} object.
	 * @param reference a {@link eu.etaxonomy.cdm.model.reference.ReferenceBase} object.
	 * @param conversation a {@link eu.etaxonomy.cdm.api.conversation.ConversationHolder} object.
	 * @param multi a boolean.
	 */
	protected ReferenceSelectionDialog(Shell shell, //ConversationHolder conversation,
	        String title, boolean multi, Reference reference, Set<Reference> preSelectedReferences) {
		super(shell, //conversation,
		        title, multi, ReferenceSelectionDialog.class.getCanonicalName(), reference);
		this.currentReference = reference;
		this.preSelectedReferences = preSelectedReferences;

	}




	/**
     * <p>Constructor for FilteredReferenceSelectionDialog.</p>
     *
     * @param shell a {@link org.eclipse.swt.widgets.Shell} object.
     * @param title a {@link java.lang.String} object.
     * @param reference a {@link eu.etaxonomy.cdm.model.reference.ReferenceBase} object.
     * @param conversation a {@link eu.etaxonomy.cdm.api.conversation.ConversationHolder} object.
     * @param multi a boolean.
     */
    protected ReferenceSelectionDialog(Shell shell, //ConversationHolder conversation,
            String title, boolean multi, Reference reference, boolean isInReference) {
        super(shell, //conversation,
                title, multi, ReferenceSelectionDialog.class.getCanonicalName());
        this.isInReference = isInReference;
        this.currentReference = reference;

    }


	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.dialogs.AbstractFilteredCdmResourceSelectionDialog#getPersistentObject(java.util.UUID)
	 */
	/** {@inheritDoc} */
	@Override
	protected Reference getPersistentObject(UUID cdmUuid) {
	    if (lastSelectedReferences == null){
	        lastSelectedReferences = new ArrayList<>();
	    }
	    if (lastSelectedReferences.size()<6){
	        lastSelectedReferences.add(cdmUuid.toString());
	    }else{
	        lastSelectedReferences.remove(0);
	        lastSelectedReferences.add(cdmUuid.toString());
	    }

	    PreferencesUtil.setLastSelectedReference(lastSelectedReferences);
		return CdmStore.getService(IReferenceService.class).load(cdmUuid);
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.dialogs.AbstractFilteredCdmResourceSelectionDialog#search
	 */
	/** {@inheritDoc} */
	@Override
	protected void callService(String pattern) {

	    if (StringUtils.isBlank(pattern) ){
	        lastSelectedReferences = PreferencesUtil.getLastSelectedReferences();
	        Set<UUID> uuids = new HashSet<>();
	        for (String uuidString: lastSelectedReferences){
	            uuids.add(UUID.fromString(uuidString));
	        }
	        if (preSelectedReferences != null){
    	        for (Reference ref: preSelectedReferences){
    	            if (ref != null){
    	                uuids.add(ref.getUuid());
    	            }
                }
	        }
	        if (!uuids.isEmpty()){
	            if (currentReference != null){
	                uuids.remove(currentReference.getUuid());
	                if (this.isInReference){
	                    model = CdmStore.getService(IReferenceService.class).getUuidAndTitleCacheForUUIDS(uuids, currentReference.getType());
	                }else{
	                    model = CdmStore.getService(IReferenceService.class).getUuidAndTitleCacheForUUIDS(uuids);
	                }
	            }else{
	                model = CdmStore.getService(IReferenceService.class).getUuidAndTitleCacheForUUIDS(uuids);
	            }

	        }

	    } else if (isInReference && currentReference != null){

            if (isUseIdentifier()){
                List<IdentifiedEntityDTO<Reference>> list = CdmStore.getService(IReferenceService.class).listByIdentifierAbbrev(pattern, null, MatchMode.BEGINNING, limitOfInitialElements);
                if (model!= null){
                    model.clear();
                }
                for (IdentifiedEntityDTO dto: list){

                    model.add(dto.getCdmEntity());
                }
            }else{

                model = CdmStore.getService(IReferenceService.class).getUuidAndTitleCache(limitOfInitialElements,pattern, currentReference.getType());
            }

        }else{
            if (isUseIdentifier() && PreferencesUtil.getBooleanValue(PreferencePredicate.SearchForIdentifierAndTitleCache.getKey())){
                List<IdentifiedEntityDTO<Reference>> list = CdmStore.getService(IReferenceService.class).listByIdentifierAndTitleCacheAbbrev(pattern, null, MatchMode.BEGINNING, limitOfInitialElements);
                if (model!= null){
                    model.clear();
                }
                identifierMap = new HashMap();
                for (IdentifiedEntityDTO dto: list){
                    if (dto.getIdentifier() != null){
                        identifierMap.put(dto.getCdmEntity().getUuid(), dto.getIdentifier());
                    }
                    model.add(dto.getCdmEntity());

                }
            }else if (isUseIdentifier() ){
                List<IdentifiedEntityDTO<Reference>> list = CdmStore.getService(IReferenceService.class).listByIdentifierAbbrev(pattern, null, MatchMode.BEGINNING, limitOfInitialElements);
                if (model!= null){
                    model.clear();
                }
                identifierMap = new HashMap();
                for (IdentifiedEntityDTO dto: list){
                    identifierMap.put(dto.getCdmEntity().getUuid(), dto.getIdentifier());
                    model.add(dto.getCdmEntity());

                }
            }else {
                model = CdmStore.getService(IReferenceService.class).getUuidAndTitleCache(limitOfInitialElements,pattern);
            }

        }

	}

	@Override
    protected void addIdentifierCheckButton(Composite searchAndFilter) {
        Button btnCheckButton = new Button(searchAndFilter, SWT.CHECK);
        btnCheckButton.setText("Use Identifier");
        if (PreferencesUtil.getBooleanValue(PreferencePredicate.SearchForIdentifierAsDefault.getKey())){
            btnCheckButton.setSelection(true);
            useIdentifier = true;
        }
        btnCheckButton.addSelectionListener(new SelectionAdapter() {
            @Override
            public void widgetSelected(SelectionEvent e) {
                useIdentifier = btnCheckButton.getSelection();
                search();
            }
        });
    }
	/** {@inheritDoc} */
	@Override
	protected String getTitle(Reference cdmObject) {
		if(cdmObject == null){
			return "";
		}else{
		    return super.getTitle(cdmObject);
		}
//		}else{
//			return DefaultReferenceCacheStrategy.putAuthorToEndOfString(cdmObject.getTitleCache(), cdmObject.getAuthorship().getTitleCache());
//		}
	}

	/** {@inheritDoc} */
	@Override
	protected AbstractNewEntityWizard getNewEntityWizard(String parameter) {
	    if (isInReference){
	        if (currentReference != null) {
                return new NewReferenceWizard(this.currentReference.getType());
            }
	    }
		return new NewReferenceWizard(null);
	}

	/** {@inheritDoc} */
	@Override
	protected String[] getNewWizardText() {
		return new String[]{"New Reference"};
	}

	/** {@inheritDoc} */
    @Override
    protected void search() {
        Control control =getSearchField();
        String pattern = null;
        if (control != null){
            pattern = ((Text)control).getText();
            callService(pattern);

            fillContentProvider(null);
        }
    }
    @Override
    protected ILabelProvider createListLabelProvider() {
        return new FilteredReferenceLabelProvider();
    }

    public class FilteredReferenceLabelProvider extends FilteredCdmResourceLabelProvider {
        @Override
        public String getText(Object element) {
            if (element == null) {
                return null;
            }
            UuidAndTitleCache uuidAndTitleCache = (UuidAndTitleCache) element;
            String titleCache = uuidAndTitleCache.getTitleCache();
            if(PreferencesUtil.getBooleanValue(PreferencePredicate.ShowIdInSelectionDialog.getKey())){
                titleCache += " ["+uuidAndTitleCache.getId()+"]";
            }
            if (isUseIdentifier()){
                if (identifierMap != null){
                    AlternativeIdentifier identifier = identifierMap.get(uuidAndTitleCache.getUuid());
                    if (identifier != null){
                        titleCache += " (" + identifier.getTypeLabel() +": " + identifier.getIdentifier() + ")";
                    }else{
                        titleCache += " (-)";
                    }
                }

            }

            return titleCache;
        }
    };

}
