/**
* Copyright (C) 2009 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.ui.dialog.selection;

import java.util.List;
import java.util.UUID;

import org.eclipse.swt.widgets.Shell;

import eu.etaxonomy.cdm.api.facade.DerivedUnitFacade;
import eu.etaxonomy.cdm.model.agent.AgentBase;
import eu.etaxonomy.cdm.model.agent.Institution;
import eu.etaxonomy.cdm.model.agent.Person;
import eu.etaxonomy.cdm.model.agent.Team;
import eu.etaxonomy.cdm.model.agent.TeamOrPersonBase;
import eu.etaxonomy.cdm.model.common.ICdmBase;
import eu.etaxonomy.cdm.model.description.Feature;
import eu.etaxonomy.cdm.model.description.PolytomousKey;
import eu.etaxonomy.cdm.model.description.PolytomousKeyNode;
import eu.etaxonomy.cdm.model.location.Country;
import eu.etaxonomy.cdm.model.location.NamedArea;
import eu.etaxonomy.cdm.model.media.Media;
import eu.etaxonomy.cdm.model.media.Rights;
import eu.etaxonomy.cdm.model.metadata.PreferencePredicate;
import eu.etaxonomy.cdm.model.molecular.Amplification;
import eu.etaxonomy.cdm.model.molecular.Primer;
import eu.etaxonomy.cdm.model.name.TaxonName;
import eu.etaxonomy.cdm.model.occurrence.Collection;
import eu.etaxonomy.cdm.model.occurrence.DerivedUnit;
import eu.etaxonomy.cdm.model.occurrence.FieldUnit;
import eu.etaxonomy.cdm.model.occurrence.SpecimenOrObservationBase;
import eu.etaxonomy.cdm.model.permission.GrantedAuthorityImpl;
import eu.etaxonomy.cdm.model.permission.Group;
import eu.etaxonomy.cdm.model.permission.User;
import eu.etaxonomy.cdm.model.reference.Reference;
import eu.etaxonomy.cdm.model.taxon.Classification;
import eu.etaxonomy.cdm.model.taxon.Synonym;
import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.cdm.model.taxon.TaxonBase;
import eu.etaxonomy.cdm.model.taxon.TaxonNode;
import eu.etaxonomy.cdm.model.term.TermTree;
import eu.etaxonomy.taxeditor.model.MessagingUtils;
import eu.etaxonomy.taxeditor.preference.PreferencesUtil;
import eu.etaxonomy.taxeditor.ui.element.ICdmFormElement;
import eu.etaxonomy.taxeditor.ui.element.IEntityElement;
import eu.etaxonomy.taxeditor.ui.section.agent.TeamMemberElement;
import eu.etaxonomy.taxeditor.ui.section.description.CommonNameSourceElement;
import eu.etaxonomy.taxeditor.ui.section.description.detail.CommonNameDetailElement;
import eu.etaxonomy.taxeditor.ui.section.group.MemberDetailElement;
import eu.etaxonomy.taxeditor.ui.section.group.MemberDetailSection;
import eu.etaxonomy.taxeditor.ui.section.name.AuthorshipDetailElement;
import eu.etaxonomy.taxeditor.ui.section.occurrence.DerivedUnitGeneralDetailElement;
import eu.etaxonomy.taxeditor.ui.section.occurrence.FieldUnitFacadeGeneralDetailElement;
import eu.etaxonomy.taxeditor.ui.section.reference.NomenclaturalSourceDetailElement;
import eu.etaxonomy.taxeditor.ui.section.reference.ReferenceDetailElement;
import eu.etaxonomy.taxeditor.ui.section.user.GroupsByUserDetailElement;
import eu.etaxonomy.taxeditor.ui.section.user.GroupsByUserDetailSection;
import eu.etaxonomy.taxeditor.ui.selection.CollectorSelectionElement;

/**
 * @author n.hoffmann
 * @date Jan 25, 2012
 */
public class SelectionDialogFactory {

    public static <T extends ICdmBase> T getSelectionFromDialog(Class<T> clazz, Shell shell,
            T currentSelection){
        return getSelectionFromDialog(clazz, shell, //conversation,
                currentSelection, null);
    }

	public static <T extends ICdmBase> T getSelectionFromDialog(Class<T> clazz, Shell shell,
	        T currentSelection, ICdmFormElement parentElement){

		if(clazz.equals(Taxon.class)){
			return (T) TaxonSelectionDialog.selectTaxon(shell, (Taxon) currentSelection);
		}
		if(clazz.equals(Synonym.class)){
			return (T) TaxonBaseSelectionDialog.selectSynonym(shell );
		}
		if(clazz.equals(TaxonBase.class)){
			return (T) TaxonBaseSelectionDialog.selectTaxonBase(shell);
		}
		if(clazz.equals(Classification.class)){
			return (T) ClassificationSelectionDialog.select(shell, (Classification) currentSelection);
		}
		if(clazz.equals(TaxonNode.class)){
			return (T) TaxonNodeSelectionDialog.select(shell,
			        null, null, (TaxonNode) currentSelection, ((TaxonNode) currentSelection).getClassification().getUuid());
		}
		if(clazz.equals(Reference.class) && parentElement != null && parentElement.getParentElement() instanceof NomenclaturalSourceDetailElement){
			return (T) NomenclaturalReferenceSelectionDialog.select(shell,
			        (Reference) currentSelection, ((NomenclaturalSourceDetailElement)parentElement.getParentElement()).getEntity().getCombinationAuthorship());
		}
		if(clazz.equals(Reference.class) && parentElement instanceof CommonNameSourceElement && PreferencesUtil.getFilterCommonNameReferences()){
            return (T) CommonNameReferenceSelectionDialog.select(shell,
                    (Reference) currentSelection);
        }
		if(clazz.equals(Reference.class)){
		    if (parentElement instanceof ReferenceDetailElement){
		        return (T) ReferenceSelectionDialog.select(shell,
		                ((ReferenceDetailElement) parentElement).getEntity(), true);
		    }
			return (T) ReferenceSelectionDialog.select(shell, (Reference) currentSelection);
		}

		if(clazz.equals(TaxonName.class)){
			return (T) NameSelectionDialog.select(shell, (TaxonName)currentSelection);
		}
		if(clazz.equals(Team.class)){
			return (T) TeamSelectionDialog.select(shell, (Team)currentSelection);
		}
		if(clazz.equals(TeamOrPersonBase.class) && (parentElement instanceof AuthorshipDetailElement 
				|| (parentElement instanceof ReferenceDetailElement && ((ReferenceDetailElement)parentElement).isNomenclaturalReference()))){
            return (T) NomenclaturalAuthorSelectionDialog.select(shell,
                    (TeamOrPersonBase) currentSelection, false);
        }else if(clazz.equals(AgentBase.class) && (parentElement instanceof FieldUnitFacadeGeneralDetailElement || parentElement instanceof DerivedUnitGeneralDetailElement)){
		    return (T) CollectorSelectionDialog.select(shell,
		            (TeamOrPersonBase) currentSelection, false);
		}else if (clazz.equals(AgentBase.class) && parentElement instanceof CollectorSelectionElement){
			return (T) CollectorSelectionDialog.select(shell,
		            (TeamOrPersonBase) ((CollectorSelectionElement)parentElement).getCollectorTeam(), true);
		}else if(clazz.equals(TeamOrPersonBase.class)){
            //TODO: add TeamOrPersonBaseSelectionDialog (see ticket #4545)
            return (T) TeamOrPersonBaseSelectionDialog.select(shell,
                    (TeamOrPersonBase) currentSelection, false);
        }
		if(clazz.equals(Person.class) && parentElement instanceof TeamMemberElement){
		    return (T) NomenclaturalPersonAuthorSelectionDialog.select(shell,
		            (TeamOrPersonBase) currentSelection, true);
        }
		if(clazz.equals(AgentBase.class)){
			return (T) AgentSelectionDialog.select(shell, (AgentBase)currentSelection, false);
		}
		if(clazz.equals(Feature.class)){
			return (T) FeatureSelectionDialog.select(shell, (Feature)currentSelection);
		}
		if(clazz.equals(TermTree.class)){
			return (T) FeatureTreeSelectionDialog.select(shell, (TermTree)currentSelection);
		}
		if(clazz.equals(PolytomousKey.class)){
			return (T) PolytomousKeySelectionDialog.select(shell, (PolytomousKey)currentSelection);
		}
		if(clazz.equals(PolytomousKeyNode.class)){
			MessagingUtils.warningDialog("Not implemented yet", SelectionDialogFactory.class,
					"This functionality is not implemented yet.");
			return null;
		}
		if(clazz.equals(DerivedUnit.class)){
		    return (T) DerivedUnitSelectionDialog.select(shell, (DerivedUnit) currentSelection);
		}
		if(clazz.equals(FieldUnit.class)){
		    return (T) FieldUnitSelectionDialog.select(shell, (FieldUnit) currentSelection);
		}
		if(clazz.equals(SpecimenOrObservationBase.class)){
		    return (T) SpecimenOrObservationBaseSelectionDialog.select(shell,
		            (SpecimenOrObservationBase) currentSelection);
		}
		if(clazz.equals(NamedArea.class)){
		    if(parentElement instanceof IEntityElement && ((IEntityElement) parentElement).getEntity() instanceof DerivedUnitFacade){
		        return (T) NamedAreaSelectionDialog.select(shell,
		                (NamedArea) currentSelection, DerivedUnit.class.getCanonicalName(), Country.uuidCountryVocabulary);
		    }
		    else if(parentElement instanceof CommonNameDetailElement ){
                return (T) CommonNameNamedAreaSelectionDialog.select(shell,
                        (NamedArea) currentSelection, parentElement.getClass().getCanonicalName());
            }
		    else{
	            List<UUID> uuidList = PreferencesUtil.createUUIDListFromStringPref(PreferencePredicate.AvailableDistributionAreaVocabularies.getKey(), false);
	            UUID[] uuidArray = null;
	            if (uuidList != null){
	                uuidArray = new UUID[uuidList.size()];
	                uuidList.toArray(uuidArray);
	            }
	            return (T)NamedAreaSelectionDialog.select(shell, (NamedArea) currentSelection, null, uuidArray);
		    }
		}
		if(clazz.equals(Collection.class)){
			return (T) CollectionSelectionDialog.select(shell, (Collection) currentSelection);
		}
		if(clazz.equals(User.class) && parentElement instanceof MemberDetailElement){
		    MemberDetailSection section = (MemberDetailSection)((MemberDetailElement)parentElement).getParentElement();
			return (T) UserSelectionDialog.select(shell,(User) currentSelection, section.getEntity());
		}
		if(clazz.equals(GrantedAuthorityImpl.class)){
			return (T) GrantedAuthoritySelectionDialog.select(shell, (GrantedAuthorityImpl)currentSelection);
		}
		if (clazz.equals(Person.class)){
			return (T) PersonSelectionDialog.select(shell, (Person)currentSelection);
		}
		if(clazz.equals(Group.class) &&  parentElement instanceof GroupsByUserDetailElement){
		    GroupsByUserDetailSection section =(GroupsByUserDetailSection)((GroupsByUserDetailElement)parentElement).getParentElement();
			return (T) GroupSelectionDialog.select(shell, (Group)currentSelection, section.getEntity());
		}
		if(clazz.equals(Institution.class)){
			return (T) InstitutionSelectionDialog.select(shell, (Institution)currentSelection);
		}
		if(clazz.equals(Primer.class)){
		    return (T) PrimerSelectionDialog.select(shell, (Primer)currentSelection);
		}
		if(clazz.equals(Amplification.class)){
		    return (T) AmplificationSelectionDialog.select(shell, (Amplification)currentSelection);
		}
		if(clazz.equals(Media.class)){
		    return (T) MediaSelectionDialog.select(shell, (Media)currentSelection);
		}
		if(clazz.equals(Rights.class)){
            return (T) RightsSelectionDialog.select(shell, (Rights)currentSelection);
        }

		return null;
	}

	public static <T extends ICdmBase> T getSelectionFromExtDialog(Class<T> clazz, Shell shell,
	        ICdmFormElement parentElement){
	    return (T) ExtReferenceSelectionDialog.select(shell, null);
	}
}