/**
 * Copyright (C) 2007 EDIT
 * European Distributed Institute of Taxonomy
 * http://www.e-taxonomy.eu
 *
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */

package eu.etaxonomy.taxeditor.ui.dialog.selection;

import java.util.Collections;
import java.util.HashSet;
import java.util.UUID;

import org.eclipse.swt.widgets.Shell;

import eu.etaxonomy.cdm.api.service.ITaxonService;
import eu.etaxonomy.cdm.model.taxon.Synonym;
import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.cdm.model.taxon.TaxonBase;
import eu.etaxonomy.taxeditor.newWizard.AbstractNewEntityWizard;
import eu.etaxonomy.taxeditor.newWizard.NewTaxonWizard;
import eu.etaxonomy.taxeditor.preference.PreferencesUtil;
import eu.etaxonomy.taxeditor.store.CdmStore;

/**
 * @author n.hoffmann
 * @created Sep 21, 2009
 * @version 1.0
 */
public class TaxonBaseSelectionDialog<T extends TaxonBase> extends AbstractFilteredCdmResourceSelectionDialog {


    public static TaxonBase selectTaxonBase(Shell shell){//, ConversationHolder conversation) {
        AbstractFilteredCdmResourceSelectionDialog<TaxonBase> dialog = new TaxonBaseSelectionDialog(TaxonBase.class, shell, //conversation,
                "Choose a taxon/synonym", false, null, null);
        return getSelectionFromDialog(dialog);
    }

    public static Taxon selectTaxon(Shell shell, //ConversationHolder conversation,
            Taxon taxonToBeFiltered) {


        AbstractFilteredCdmResourceSelectionDialog<TaxonBase> dialog = new TaxonBaseSelectionDialog(Taxon.class, shell, //conversation,
                "Choose a taxon", false, null, taxonToBeFiltered);
        return (Taxon) getSelectionFromDialog(dialog);
    }

    public static Synonym selectSynonym(Shell shell){//, ConversationHolder conversation) {
        AbstractFilteredCdmResourceSelectionDialog<TaxonBase> dialog = new TaxonBaseSelectionDialog(Synonym.class, shell, //conversation,
                "Choose a synonym", false, null, null);
        return (Synonym) getSelectionFromDialog(dialog);
    }

    private final Class<? extends TaxonBase> clazz;

    protected TaxonBaseSelectionDialog(Class<? extends TaxonBase> clazz, Shell shell, //ConversationHolder conversation,
            String title, boolean multi, TaxonBase taxon, TaxonBase taxonToBeFiltered) {
        super(shell, //conversation,
                title, multi, TaxonBaseSelectionDialog.class.getCanonicalName(), taxon);
        if (taxonToBeFiltered != null){
            this.cdmBaseToBeFiltered = new HashSet<>();
            this.cdmBaseToBeFiltered.add(taxonToBeFiltered.getUuid());
        }
        this.clazz = clazz;
        search();
    }

    /** {@inheritDoc} */
    @Override
    protected TaxonBase getPersistentObject(UUID uuid) {
        return CdmStore.getService(ITaxonService.class).load(uuid);
    }

    /** {@inheritDoc} */
    @SuppressWarnings({ "unchecked", "rawtypes" })
	@Override
    protected void callService(String pattern) {

        model = CdmStore.getService(ITaxonService.class).getUuidAndTitleCache(limitOfInitialElements, pattern);
    }

    /** {@inheritDoc} */
    @Override
    protected AbstractNewEntityWizard getNewEntityWizard(String parameter) {
        return new NewTaxonWizard();
    }

    /** {@inheritDoc} */
    @Override
    protected String[] getNewWizardText() {
        return new String[]{"New Taxon"};
    }

    @Override
    protected void sort() {
        if(!PreferencesUtil.isSortTaxaByRankAndName()){
            Collections.sort(model, getItemsComparator());
        }
      // otherwise result is already sorted
    }
}
