/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.ui.dialog.selection;

import java.util.Collections;
import java.util.Set;
import java.util.UUID;

import org.eclipse.swt.widgets.Shell;

import eu.etaxonomy.cdm.api.service.IVocabularyService;
import eu.etaxonomy.cdm.model.term.TermType;
import eu.etaxonomy.cdm.model.term.TermVocabulary;
import eu.etaxonomy.taxeditor.newWizard.AbstractNewEntityWizard;
import eu.etaxonomy.taxeditor.newWizard.NewFeatureVocabularyWizard;
import eu.etaxonomy.taxeditor.store.CdmStore;

/**
 *
 * @author pplitzner
 * @since Jun 1, 2018
 *
 */
public class TermVocabularySelectionDialog extends
		AbstractFilteredCdmResourceSelectionDialog<TermVocabulary> {

    private TermType termType;

    public static TermVocabulary select(String dialogTitle, Shell shell, Set<TermVocabulary> vocabulariesToBeFiltered,
            TermType termType){
        return select_internal(dialogTitle, shell, vocabulariesToBeFiltered, termType);
    }

    public static TermVocabulary select(String dialogTitle, Shell shell, TermVocabulary voc){
        return select_internal(dialogTitle, shell, voc!=null?Collections.singleton(voc):null, null);
    }

    public static TermVocabulary select(Shell shell, TermVocabulary voc){
        return select_internal("Choose Vocabulary", shell, voc!=null?Collections.singleton(voc):null, null);
    }

    private static TermVocabulary select_internal(String dialogTitle, Shell shell, Set<TermVocabulary> vocabulariesToBeFiltered,
            TermType termType){
		TermVocabularySelectionDialog dialog = new TermVocabularySelectionDialog(shell,
				dialogTitle!=null?dialogTitle:"Choose Vocabulary",
				false,
				TermVocabularySelectionDialog.class.getCanonicalName(),
				vocabulariesToBeFiltered, termType);
		return getSelectionFromDialog(dialog);
	}

    protected TermVocabularySelectionDialog(Shell shell,
            String title, boolean multi,
            String settings, Set<TermVocabulary> vocabulariesToBeFiltered, TermType termType) {
        super(shell, title, multi, settings, vocabulariesToBeFiltered);
        this.termType = termType;
    }

	@Override
	protected TermVocabulary getPersistentObject(UUID uuid) {
		return CdmStore.getService(IVocabularyService.class).load(uuid);
	}

	@Override
	protected void callService(String pattern) {
        model = CdmStore.getService(IVocabularyService.class).getUuidAndTitleCache(TermVocabulary.class, termType, limitOfInitialElements, pattern);
	}

	@Override
	protected String[] getNewWizardText() {
        return new String[]{ "New Vocabulary"};
	}

	@Override
	protected AbstractNewEntityWizard getNewEntityWizard(String parameter) {
        return new NewFeatureVocabularyWizard(termType);
	}

}
