/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.ui.dialog.selection;

import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.UUID;

import org.eclipse.swt.widgets.Shell;

import eu.etaxonomy.cdm.api.service.IUserService;
import eu.etaxonomy.cdm.common.CdmUtils;
import eu.etaxonomy.cdm.model.permission.Group;
import eu.etaxonomy.cdm.model.permission.User;
import eu.etaxonomy.cdm.persistence.dto.UuidAndTitleCache;
import eu.etaxonomy.taxeditor.newWizard.AbstractNewEntityWizard;
import eu.etaxonomy.taxeditor.newWizard.NewUserWizard;
import eu.etaxonomy.taxeditor.store.CdmStore;

/**
 * @author n.hoffmann
 * @created Mar 10, 2011
 * @version 1.0
 */
public class UserSelectionDialog extends
		AbstractFilteredCdmResourceSelectionDialog<User> {

    Group group;
    Set<User> groupMembers;

	/**
	 *
	 * @param shell
	 * @param conversation
	 * @param user
	 * @return
	 */
	public static User select(Shell shell, //ConversationHolder conversation,
	        User user, Group group){
		UserSelectionDialog dialog = new UserSelectionDialog(shell, //conversation,
				"Choose User", false, UserSelectionDialog.class.getCanonicalName(), user, group);
		return getSelectionFromDialog(dialog);
	}

	/**
	 * @param shell
	 * @param conversation
	 * @param title
	 * @param multi
	 * @param settings
	 * @param cdmObject
	 */
	protected UserSelectionDialog(Shell shell, //ConversationHolder conversation,
			String title, boolean multi, String settings, User cdmObject, Group group) {
		super(shell, //conversation,
		        title, multi, settings, cdmObject);
		this.group = group;
		groupMembers = group.getMembers();
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.ui.dialogs.filteredSelection.AbstractFilteredCdmResourceSelectionDialog#search
	 */
	@Override
	protected void callService(String pattern) {
		List<User> users = CdmStore.getService(IUserService.class).list(User.class, null, null, null, null);

		model = new ArrayList<UuidAndTitleCache<User>>();

		for(User user : users){
		    if (pattern == null || user.getUsername().matches(CdmUtils.quoteRegExWithWildcard(pattern) + ".*")) {
		        if (groupMembers != null && !groupMembers.isEmpty() && groupMembers.contains(user)){
		           continue;
		        }
                model.add(new UuidAndTitleCache<User>(User.class, user.getUuid(), user.getId(), String.format("%s, %s", user.getUsername(), user.getPerson())));
            }
		}
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.ui.dialogs.filteredSelection.AbstractFilteredCdmResourceSelectionDialog#getNewWizardLinkText()
	 */
	@Override
	protected String[] getNewWizardText() {
		return null;
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.ui.dialogs.filteredSelection.AbstractFilteredCdmResourceSelectionDialog#getNewEntityWizard(java.lang.String)
	 */
	@Override
	protected AbstractNewEntityWizard getNewEntityWizard(String parameter) {
		return new NewUserWizard();
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.ui.dialogs.filteredSelection.AbstractFilteredCdmResourceSelectionDialog#getPersistentObject(java.util.UUID)
	 */
	@Override
	protected User getPersistentObject(UUID uuid) {
		return CdmStore.getService(IUserService.class).load(uuid);
	}

	@Override
	protected String getTitle(User user) {
		return user != null ? user.getUsername() : "";
	}

}
