/**
* Copyright (C) 2020 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.ui.element;

import java.math.BigDecimal;

import org.apache.commons.lang3.StringUtils;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.widgets.Display;

/**
 * @author k.luther
 * @since May 27, 2020
 */
public class BigDecimalWithLabelElement extends NumberWithLabelElement {

    public BigDecimalWithLabelElement(CdmFormFactory toolkit, ICdmFormElement parentElement, String labelString,
            Number initialNumber, int style) {
        super(toolkit, parentElement, labelString, initialNumber, style);
    }

    private BigDecimal start;
    private BigDecimal end;

    @Override
    public void modifyText(ModifyEvent event) {
        String value = text.getText();
        if(StringUtils.isBlank(value)){
            text.setBackground(Display.getCurrent().getSystemColor(SWT.COLOR_WHITE));
            super.modifyText(event);
            return;
        }

        try{
            String pattern = "-?\\d{0,9}(.\\d{0,9})?";
            if (value.matches(pattern)){
                if ("-".equals(value)){
                    //accept entry
                }else{
                    BigDecimal number = new BigDecimal(value);
                    if((start != null && number.compareTo(start) < 0 || (end != null && number.compareTo(end) > 0))){
                        exception = new NumberFormatException("You entered a number that is not within the allowed bounds.");
                        throw exception;
                    }
                }
            }else{
                exception = new NumberFormatException("You entered a number that is not within the allowed bounds.");
                throw exception;
            }
        }catch(NumberFormatException e){
            text.setBackground(Display.getCurrent().getSystemColor(SWT.COLOR_RED));
            firePropertyChangeEvent(new CdmPropertyChangeEvent(this, event));
            exception = e;
            return;
        }

        exception = null;
        text.setBackground(Display.getCurrent().getSystemColor(SWT.COLOR_WHITE));

        super.modifyText(event);
    }

    /**
     * <p>setLimits</p>
     *
     * @param numberOfDigits a int.
     * @param start a {@link java.lang.Integer} object.
     * @param end a {@link java.lang.Integer} object.
     */

    public void setLimits(int numberOfDigits, BigDecimal start, BigDecimal end){
        text.setTextLimit(numberOfDigits);
        this.start = start;
        this.end = end;
    }

    public BigDecimal getBigDecimal(){
        String text = super.getText();
        if (StringUtils.isBlank(text) || "-".equals(text)){
            return null;
        }
        try {
            return new BigDecimal(text);
        } catch (NumberFormatException e) {
            exception = e;
            return null;
        }
    }
}
