/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.ui.element;

import org.eclipse.swt.SWT;
import org.eclipse.swt.browser.Browser;
import org.eclipse.swt.events.ControlEvent;
import org.eclipse.swt.events.ControlListener;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.ui.forms.widgets.TableWrapData;

import eu.etaxonomy.cdm.common.URI;

/**
 * <p>BrowserElement class.</p>
 *
 * @author n.hoffmann
 */
public class BrowserElement extends AbstractCdmFormElement implements ControlListener{

	private Browser browser;

	private String imageUriString;

	/**
	 * <p>Constructor for BrowserElement.</p>
	 *
	 * @param style a int.
	 * @param formFactory a {@link eu.etaxonomy.taxeditor.ui.element.CdmFormFactory} object.
	 * @param parentElement a {@link eu.etaxonomy.taxeditor.ui.element.ICdmFormElement} object.
	 * @param imageUri a {@link eu.etaxonomy.cdm.common.URI} object.
	 */
	protected BrowserElement(CdmFormFactory formFactory, ICdmFormElement parentElement, URI imageUri, int style) {
		super(formFactory, parentElement);
		formFactory.createLabel(getLayoutComposite(), "");

		browser = new Browser(getLayoutComposite(), SWT.NONE);
		addControl(browser);

		getLayoutComposite().setLayoutData(LayoutConstants.FILL());


		layoutBrowser();

		showBrowser();
		browser.addControlListener(this);

		this.setImageUri(imageUri);
	}

	private void layoutBrowser() {
		TableWrapData layoutData = LayoutConstants.FILL();
		layoutData.heightHint = 1000;

		browser.setLayoutData(layoutData);
	}

	private void showBrowser() {

		String html = "";
		html = "<HTML><HEAD><TITLE></TITLE></HEAD><BODY marginwidth=0 marginheight=0 leftmargin=0 topmargin=0>";
		html += "<img width=\"" + calculateWidth() + "\" src=\"" + getImageUriString() + "\" />";
		html += "</BODY></HTML>";

		browser.setText(html);
	}

	/**
	 * <p>calculateWidth</p>
	 *
	 * @return a int.
	 */
	public int calculateWidth() {
		Rectangle boundaries = browser.getBounds();
		return boundaries.width;
	}

	@Override
    public void controlMoved(ControlEvent e) {
		showBrowser();
	}

	@Override
    public void controlResized(ControlEvent e) {
		showBrowser();
	}

	/**
	 * <p>setImageUri</p>
	 *
	 * @param imageUri the imageUri to set
	 */
	public void setImageUri(URI imageUri) {
		setImageUriString(imageUri != null ? imageUri.toString() : "");
	}

	/**
	 * <p>Setter for the field <code>imageUriString</code>.</p>
	 *
	 * @param imageUriString a {@link java.lang.String} object.
	 */
	public void setImageUriString(String imageUriString){
		this.imageUriString = imageUriString;
		showBrowser();
	}

	/**
	 * <p>Getter for the field <code>imageUriString</code>.</p>
	 *
	 * @return the imageUri
	 */
	public String getImageUriString() {
		return imageUriString;
	}
}
