/**
* Copyright (C) 2021 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.ui.element;

import org.eclipse.swt.graphics.Color;

import eu.etaxonomy.taxeditor.model.AbstractUtility;
import eu.etaxonomy.taxeditor.preference.Resources;

/**
 * @author a.mueller
 * @since 20.01.2021
 */
public enum CacheRelevance{
    CACHE1(Resources.COLOR_COMPOSITE_IRRELEVANT1, null),
    CACHE2(Resources.COLOR_COMPOSITE_IRRELEVANT2, null),

    CACHE1_2(Resources.COLOR_COMPOSITE_IRRELEVANT1_2, null),
    CACHE1_LIGHT(Resources.COLOR_COMPOSITE_IRRELEVANT1_LIGHT, CACHE1),
    CACHE2_LIGHT(Resources.COLOR_COMPOSITE_IRRELEVANT2_LIGHT, CACHE2),
    CACHE1_2_LIGHT(Resources.COLOR_COMPOSITE_IRRELEVANT1_2_LIGHT, CACHE1_2),
    NONE(Resources.COLOR_COMPOSITE_BACKGROUND, null),
    ;

    private final Color color;
    private CacheRelevance light;


    private CacheRelevance(String colorStr, CacheRelevance main){
        this.color = AbstractUtility.getColor(colorStr);
        light = this;
        if (main != null){
            main.light = this;
        }
    }

    public Color getColor() {
        return color;
    }
    public Color getColorLight() {
        return color;
    }

    public Color getColor(String alternativeColorForNone) {
        if (this == NONE){
            return AbstractUtility.getColor(alternativeColorForNone);
        }else{
            return color;
        }
    }

    public CacheRelevance addCacheRelevance(CacheRelevance that){
        if (this == CACHE1 && that == CACHE2 || this == CACHE2 && that == CACHE1){
            return CACHE1_2;
        } else if (this == CACHE1_LIGHT && that == CACHE2_LIGHT || this == CACHE2_LIGHT && that == CACHE1_LIGHT){
            return CACHE1_2_LIGHT;
        }else {
            return CacheRelevance.values()[Math.min(this.ordinal(), that.ordinal())];
        }
    }

    public CacheRelevance getLight(){
        return light;
    }

}