/**
* Copyright (C) 2021 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.ui.element;

import java.util.HashSet;
import java.util.Set;

/**
 * @author a.mueller
 * @date 22.01.2021
 */
public class CacheRelevanceHelper {

    protected Set<ToggleableTextElement> cacheDependencies = new HashSet<>();

    public CacheRelevance cacheRelevance() {
        CacheRelevance result = CacheRelevance.NONE;
        for (ToggleableTextElement cache : this.cacheDependencies){
            if (cache != null){  //null should not happen anymore but just in case
                if (cache.getState()){
                    result = result.addCacheRelevance(cache.getRelevance());
                }
                //add recursion for light dependencies
                result = result.addCacheRelevance(cache.cacheRelevance().getLight());
            }
        }
        return result;
    }

    public void addDependsOnCache(ToggleableTextElement toggleElement) {
        if (toggleElement != null){
            cacheDependencies.add(toggleElement);
        }
    }

    /**
     * For hierarchical caching
     */
    public ToggleableTextElement[] getDependenciesArray() {
        return this.cacheDependencies.toArray(new ToggleableTextElement[0]);
    }

    public static void addDependsOnCacheForSubElements(AbstractFormSection<?> section,
            ToggleableTextElement toggleElement) {
        section.getElements().stream()
            .filter(x -> x instanceof ICacheRelevantFormElement)
            .forEach(
                x->((ICacheRelevantFormElement)x).addDependsOnCache(toggleElement));
    }

    public static void updateCacheRelevanceForSubelements(AbstractFormSection<?> section) {
        updateCacheRelevanceForSubelements(section.getElements());
    }

    public static void updateCacheRelevanceForSubelements(Set<ICdmFormElement> elements) {
//        System.out.print("setIrrelevant: " + this.getClass().getName());
//        System.out.println("");
        elements.stream()
            .filter(x -> x instanceof ICacheRelevantFormElement)
            .forEach(
                    x->((ICacheRelevantFormElement)x).updateCacheRelevance());

    }


}
