/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.ui.element;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Label;
import org.eclipse.ui.forms.widgets.TableWrapData;

/**
 * <p>CheckboxElement class.</p>
 *
 * @author n.hoffmann
 * @created Nov 5, 2009
 */
public class CheckboxElement
        extends AbstractRelevanceFormElement
        implements SelectionListener, IEnableableFormElement, ISelectable {

	private final Button checkbox;

	private Label label;

	/**
	 * <p>Constructor for CheckboxElement.</p>
	 *
	 * @param initialState a boolean.
	 * @param style a int.
	 * @param formFactory a {@link eu.etaxonomy.taxeditor.ui.element.CdmFormFactory} object.
	 * @param parentElement a {@link eu.etaxonomy.taxeditor.ui.element.ICdmFormElement} object.
	 * @param labelString a {@link java.lang.String} object.
	 */
	protected CheckboxElement(CdmFormFactory formFactory, ICdmFormElement parentElement, String labelString, boolean initialState,
			int style) {
		super(formFactory, parentElement);

		if(labelString != null){
			label = formFactory.createLabel(getLayoutComposite(), labelString);
			addControl(label);
		}

		checkbox = formFactory.createButton(getLayoutComposite(), null, SWT.CHECK | style);
		checkbox.setLayoutData(LayoutConstants.FILL_HORIZONTALLY());

		checkbox.setSelection(initialState);

		checkbox.addSelectionListener(this);
		addControl(checkbox);
	}

	/**
	 * <p>setSelection</p>
	 */
	public void setSelection(boolean selected){
		checkbox.removeSelectionListener(this);
		checkbox.setSelection(selected);
		checkbox.addSelectionListener(this);
	}
	/**
	 * <p>getSelection</p>
	 */
	public boolean getSelection(){
		return checkbox.getSelection();
	}

	@Override
    public void widgetSelected(SelectionEvent e) {
		firePropertyChangeEvent(new CdmPropertyChangeEvent(this, e));
	}

	public Button getMainControl(){
	    return checkbox;
	}

	@Override
    public void widgetDefaultSelected(SelectionEvent e) {}

	@Override
    public void setSelected(boolean selected) {
	    setBackground(selected ? SELECTED : getPersistentBackground());
	}

	@Override
    public void setEnabled(boolean enabled) {
		if (!checkbox.isDisposed()){
			checkbox.setEnabled(enabled);
			if (label != null){
				label.setEnabled(enabled);
			}
		}
	}

	@Override
	public boolean isEnabled() {
	    return checkbox.isEnabled();
	}

	@Override
    public void updateCacheRelevance() {
	    Color color = cacheRelevance().getColor();
		setCheckboxBackgroundOnly(color);
	}

    public void setCheckboxBackgroundOnly(Color color) {
        if (checkbox.isDisposed()){
            return;
        }
        checkbox.setBackground(color);
    }

	@Override
	public void setBackground(Color color) {
	    setCheckboxBackgroundOnly(color);
        if (label != null && !label.isDisposed()) {
            label.setBackground(color);
        }
	}

    public void setIndent(int indent){
	    TableWrapData tableWrapData = (TableWrapData)label.getLayoutData();
        if (tableWrapData == null){
            tableWrapData = new TableWrapData();

        }
        tableWrapData.indent = indent;
        label.setLayoutData(tableWrapData);
        getLayoutComposite().layout();
    }
}