package eu.etaxonomy.taxeditor.ui.element;

import java.util.Observable;
import java.util.Observer;

import org.eclipse.core.commands.Command;
import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.commands.NotEnabledException;
import org.eclipse.core.commands.NotHandledException;
import org.eclipse.core.commands.common.NotDefinedException;
import org.eclipse.e4.core.commands.ECommandService;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.DisposeEvent;
import org.eclipse.swt.events.DisposeListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.handlers.IHandlerService;

import eu.etaxonomy.taxeditor.event.EventUtility;
import eu.etaxonomy.taxeditor.store.CdmStore;
import eu.etaxonomy.taxeditor.store.LoginManager;
import eu.etaxonomy.taxeditor.store.internal.TaxeditorStorePlugin;

/**
 * A Button which executes a <code>IHandler</code> managed through a
 * <code>org.eclipse.ui.handlers</code> extension point
 *
 * see http://wiki.eclipse.org/Platform_Command_Framework#Commands
 *
 * @author a.kohlbecker
 *
 */
public class CommandHandlerButton extends Composite implements Observer {

	String commandId;
	Button button;

	/**
	 * @param parent
	 * @param style
	 * @param commandId the id of the command handler as defined in a <code>org.eclipse.ui.handlers</code> extension point
	 */
	public CommandHandlerButton(Composite parent, int style, String commandId)  {
		super(parent, SWT.NONE);
		this.commandId = commandId;

		setLayout(new GridLayout());
		setLayoutData(new GridData());

		button = new Button(this, style);
		button.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent se) {
				// you can execute a command directly ... but to get the proper environment it's better to execute it through the IHandlerService:
				IHandlerService handlerService =  TaxeditorStorePlugin.getDefault().getWorkbench().getService(IHandlerService.class);
				try {
					handlerService.executeCommand(CommandHandlerButton.this.commandId, null);
				} catch (ExecutionException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				} catch (NotDefinedException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				} catch (NotEnabledException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				} catch (NotHandledException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			}

		});
		updateButtonEnabledState();

		CdmStore.getLoginManager().addObserver(this);
        addDisposeListener(new DisposeListener() {
			@Override
			public void widgetDisposed(DisposeEvent e) {
				CdmStore.getLoginManager().deleteObserver(CommandHandlerButton.this);
			}
        });
	}


	/* (non-Javadoc)
	 * @see java.util.Observer#update(java.util.Observable, java.lang.Object)
	 */
	@Override
	public void update(Observable o, Object arg) {
		if(o instanceof LoginManager){
			updateButtonEnabledState();
		}

	}

	private void updateButtonEnabledState() {
		ECommandService commandService = EventUtility.getCommandService();
		Command command = commandService.getCommand(commandId);
		if(!button.isDisposed()){
			button.setEnabled(command != null && command.isEnabled());
		}
	}

	public void setText(String string) {
		button.setText(string);
	}

	public void setButtonEnabled(boolean enabled){
	    button.setEnabled(enabled);
	}
}
