/**
* Copyright (C) 2018 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.ui.element;

import org.eclipse.jface.util.PropertyChangeEvent;
import org.eclipse.swt.widgets.Text;
import org.joda.time.Partial;

import eu.etaxonomy.cdm.common.CdmUtils;
import eu.etaxonomy.cdm.model.common.TimePeriod;
import eu.etaxonomy.cdm.model.common.VerbatimTimePeriod;
import eu.etaxonomy.cdm.strategy.parser.TimePeriodParser;
import eu.etaxonomy.taxeditor.l10n.Messages;

public class DateDetailSection<T extends TimePeriod>
		extends AbstractFormSection<T>
        implements ICacheRelevantFormElement, IEnableableFormElement{

	protected TextWithLabelElement text_freeText;
	protected PartialElement partialElement_start;
	protected PartialElement partialElement_end;
	protected CheckboxElement period_continue;
	protected TextWithLabelElement text_parseText;
    protected TextWithLabelElement text_VerbatimDate = null;
	protected int cursorPosition;
	protected boolean includeVerbatim;

	boolean isEnabled = true;

	public int getCursorPosition() {
        return cursorPosition;
    }

    public void setCursorPosition(int cursorPosition) {
        this.cursorPosition = cursorPosition;
    }

    /**
	 * Constructor for DateDetailSection.
	 */
	protected DateDetailSection(CdmFormFactory formFactory,
			ICdmFormElement parentElement, boolean includeVerbatim, int style) {
		super(formFactory, parentElement, style);

		this.includeVerbatim = includeVerbatim;
		text_parseText = formFactory.createTextWithLabelElement(this, "Parse",
                null, style);
		text_parseText.getMainControl().setLayoutData(
                LayoutConstants.FILL_HORIZONTALLY(6, 1));
		text_parseText.getMainControl().setToolTipText(Messages.DateDetail_parseText_tooltip);
		partialElement_start = formFactory.createPartialElement(this,
				"Start ", null, style);

		period_continue = formFactory.createCheckbox(this, "Continue", false, style);
        period_continue.getMainControl().setLayoutData(LayoutConstants.FILL_HORIZONTALLY(6,1));
		partialElement_end = formFactory.createPartialElement(this, "End ",
				null, style);

		if (includeVerbatim){
	        text_VerbatimDate = formFactory.createTextWithLabelElement(this,
	                "Verbatim Date", null, style);

	        text_VerbatimDate.getMainControl().setLayoutData(
	                LayoutConstants.FILL_HORIZONTALLY(6, 1));
		}

		text_freeText = formFactory.createTextWithLabelElement(this,
				"Freetext", null, style);
		text_freeText.getMainControl().setLayoutData(
				LayoutConstants.FILL_HORIZONTALLY(6, 1));


		formFactory.addPropertyChangeListener(this);
	}

	@Override
	public final T getEntity() {
		if(super.getEntity() == null){
			T newInstance = newInstance();
			this.setEntity(newInstance);
		}
		return super.getEntity();
	}

	@SuppressWarnings("unchecked")
	protected T newInstance(){
		if (includeVerbatim){
			return (T)VerbatimTimePeriod.NewVerbatimInstance();
		}else{
			return (T)TimePeriod.NewInstance();
		}
	}

	@SuppressWarnings("unchecked")
	protected T parseNewInstance() {
		if (includeVerbatim){
			T result = (T)TimePeriodParser.parseStringVerbatim(getText_parseText().getText());
			return result;
		}else{
			T result = (T)TimePeriodParser.parseString(getText_parseText().getText());
			return result;
		}
	}


	/**
	 * Setter for the field <code>timePeriod</code>.
	 */
	@Override
	public void setEntity(T timePeriod) {
		setEntityInternally(timePeriod);
		updateTitle();
		getText_parseText().setText(timePeriod.toString());
	}

	/**
	 * When setting the entity through parsing we do not want
	 * to alter the parse field
	 */
	protected void setEntityInternally(T timePeriod){
		Partial start = timePeriod.getStart();
		partialElement_start.setPartial(start);
		Partial end = timePeriod.getEnd();
		partialElement_end.setPartial(end);
		period_continue.setSelection(timePeriod.isContinued());
		if (timePeriod.isContinued()){
		    partialElement_end.setEnabled(false);

		}
		((Text) getText_parseText().getMainControl()).setSelection(cursorPosition);
		text_freeText.setText(timePeriod.getFreeText());
		if (includeVerbatim){
	        ((Text) text_VerbatimDate.getMainControl()).setSelection(cursorPosition);
	        text_VerbatimDate.setText(((VerbatimTimePeriod)timePeriod).getVerbatimDate());
		}

		super.setEntity(timePeriod);
	}

	@Override
	public void propertyChange(PropertyChangeEvent event) {
		if (event == null) {
			return;
		}
		Object eventSource = event.getSource();

		if (getElements().contains(eventSource)) {
			if (event instanceof CdmPropertyChangeEvent) {
				if (((CdmPropertyChangeEvent) event).hasException()) {
					handleException((CdmPropertyChangeEvent) event);
					return;
				}
			}
			handleEvent(eventSource);
		}
	}

	private void handleException(CdmPropertyChangeEvent event) {
		firePropertyChangeEvent(new CdmPropertyChangeEvent(this,
				event.getException()));
	}

	protected void handleEvent(Object eventSource) {
		if (eventSource == partialElement_start) {
			Partial start = partialElement_start.getPartial();
			getEntity().setStart(start);
		} else if (eventSource == partialElement_end) {
			Partial end = partialElement_end.getPartial();
			getEntity().setEnd(end);

		} else if (eventSource == getText_parseText()) {
			cursorPosition = ((Text) getText_parseText().getMainControl())
					.getCaretPosition();
			T newInstance = parseNewInstance();
			setEntityInternally(newInstance);
		} else if (eventSource == text_freeText) {
			getEntity().setFreeText(text_freeText.getText());
		}else if (eventSource == period_continue) {
		    boolean continued = period_continue.getSelection();
            getEntity().setContinued(continued);
            if (continued){
                partialElement_end.setAllNull();
                partialElement_end.setEnabled(false);
            }else{
                partialElement_end.setEnabled(true);
            }
        }else if (eventSource == text_VerbatimDate) {

            ((VerbatimTimePeriod)getEntity()).setVerbatimDate(text_VerbatimDate.getText());
        }
		updateTitle();
		if (!(eventSource == getText_parseText())){
		    updateParseField();
		}

		firePropertyChangeEvent(new CdmPropertyChangeEvent(this, null));
	}

	protected void updateTitle(){
		String title = CdmUtils.Nz(getEntity().toString());
		this.setText(title);
		layout();
	}
	protected void updateParseField(){
	    getText_parseText().setText(getEntity().toString());
	    layout();
    }

	@Override
	public void dispose() {
		formFactory.removePropertyChangeListener(this);
		super.dispose();
	}

    public TextWithLabelElement getText_parseText() {
        return text_parseText;
    }

    public void setText_parseText(TextWithLabelElement text_parseText) {
        this.text_parseText = text_parseText;
    }

    @Override
    public void updateCacheRelevance() {
        CacheRelevanceHelper.updateCacheRelevanceForSubelements(this);
    }

    @Override
    public void addDependsOnCache(ToggleableTextElement toggleElement) {
        CacheRelevanceHelper.addDependsOnCacheForSubElements(this, toggleElement);
    }

    @Override
    public CacheRelevance cacheRelevance() {
        return null;
    }

    @Override
    public void setEnabled(boolean enabled){
        this.isEnabled = enabled;
        for (ICdmFormElement element: getElements()){
            if (element instanceof IEnableableFormElement){
                IEnableableFormElement enableableElement = (IEnableableFormElement) element;
                enableableElement.setEnabled(enabled);
            }
        }
    }

    @Override
    public boolean isEnabled() {
       return isEnabled;
    }

}