/**
* Copyright (C) 2020 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.ui.element;

import org.eclipse.jface.util.PropertyChangeEvent;
import org.joda.time.Partial;

import eu.etaxonomy.cdm.common.CdmUtils;
import eu.etaxonomy.cdm.model.common.ExtendedTimePeriod;

/**
 * @author k.luther
 * @since Jun 12, 2020
 */
public class ExtendedTimeDetailSection extends AbstractFormSection<ExtendedTimePeriod> {

    private PartialElement partialElement_extremeStart;
    private PartialElement partialElement_start;
    private PartialElement partialElement_end;
    private PartialElement partialElement_extremeEnd;
//    private TextWithLabelElement text_freeText;
    /**
     * @param formFactory
     * @param parentElement
     * @param includeVerbatim
     * @param style
     */
    protected ExtendedTimeDetailSection(CdmFormFactory formFactory, ICdmFormElement parentElement,
             int style) {
        super(formFactory, parentElement, style);

//        this.includeVerbatim = includeVerbatim;


        partialElement_extremeStart = formFactory.createPartialElement(this, "Extreme Start ", null, false, style);

        partialElement_start = formFactory.createPartialElement(this, "Start ", null, false, style);


        partialElement_end = formFactory.createPartialElement(this, "End ", null, false, style);

        partialElement_extremeEnd = formFactory.createPartialElement(this, "Extreme End ",
                null, false, style);

//        text_freeText = formFactory.createTextWithLabelElement(this,
//                "Freetext", null,  style);
//        text_freeText.getMainControl().setLayoutData(
//                LayoutConstants.FILL_HORIZONTALLY(5, 1));


        formFactory.addPropertyChangeListener(this);

    }

    @Override
    public void setEntity(ExtendedTimePeriod timePeriod) {
        Partial extremeStart = timePeriod.getExtremeStart();
        partialElement_extremeStart.setPartial(extremeStart);
        Partial start = timePeriod.getStart();
        partialElement_start.setPartial(start);
        Partial end = timePeriod.getEnd();
        partialElement_end.setPartial(end);

        Partial extremeEnd = timePeriod.getExtremeEnd();
        partialElement_extremeEnd.setPartial(extremeEnd);

//        text_freeText.setText(timePeriod.getFreeText());

        super.setEntity(timePeriod);

    }

    @Override
    public void propertyChange(PropertyChangeEvent event) {
        if (event == null) {
            return;
        }
        Object eventSource = event.getSource();

        if (getElements().contains(eventSource)) {
            if (event instanceof CdmPropertyChangeEvent) {
                if (((CdmPropertyChangeEvent) event).hasException()) {
                    handleException((CdmPropertyChangeEvent) event);
                    return;
                }
            }
            handleEvent(eventSource);
        }
    }

    /**
     * @param event
     */
    private void handleException(CdmPropertyChangeEvent event) {
        firePropertyChangeEvent(new CdmPropertyChangeEvent(this,
                event.getException()));
    }

    protected void handleEvent(Object eventSource) {
        if (eventSource == partialElement_start) {
            Partial start = partialElement_start.getPartial();
            getEntity().setStart(start);
        } else if (eventSource == partialElement_end) {
            Partial end = partialElement_end.getPartial();
            getEntity().setEnd(end);
        } else if (eventSource == partialElement_extremeEnd) {
            Partial extremeEnd = partialElement_extremeEnd.getPartial();
            getEntity().setExtremeEnd(extremeEnd);
        } else if (eventSource == partialElement_extremeStart) {
            Partial extremeStart = partialElement_extremeStart.getPartial();
            getEntity().setExtremeStart(extremeStart);
//        } else if (eventSource == text_freeText) {
//            getEntity().setFreeText(text_freeText.getText());
        }
//        updateTitle();


        firePropertyChangeEvent(new CdmPropertyChangeEvent(this, null));
    }

    protected void updateTitle(){
        String title = CdmUtils.Nz(getEntity().toString());
        this.setText(title);
        layout();
    }


}
