/**
* Copyright (C) 2020 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.ui.element;

import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.widgets.Label;

import eu.etaxonomy.cdm.model.common.ExtendedTimePeriod;

/**
 * @author k.luther
 * @since Jun 12, 2020
 */
public class ExtendedTimePeriodElement extends AbstractCdmFormElement implements ISelectable {

    protected ExtendedTimePeriod timePeriod;
    protected Label label;
    protected ExtendedTimeDetailSection section_dateDetails;

    /**
     * @param formFactory
     * @param parentElement
     * @param labelString
     * @param timePeriod
     * @param style
     */
    public ExtendedTimePeriodElement(CdmFormFactory formFactory, ICdmFormElement parentElement, String labelString,
            ExtendedTimePeriod timePeriod, int style) {
        super(formFactory, parentElement);
        label = formFactory.createLabel(getLayoutComposite(), labelString);

        addControl(label);

        section_dateDetails = createDateDetailSection();
        addControl(section_dateDetails);

        setTimePeriod(timePeriod);

        formFactory.addPropertyChangeListener(this);
    }

    protected ExtendedTimeDetailSection createDateDetailSection() {
        return formFactory.createExtendedTimeDetailSection(this,
                SWT.NULL);
    }

    @Override
    public void setBackground(Color color) {
        label.setBackground(color);
        section_dateDetails.setBackground(color);
    }

    @Override
    public void setSelected(boolean selected) {
        setBackground(selected ? SELECTED : getPersistentBackground());
    }

    public void setLabel(String string) {
        label.setText(string);
    }

    public void setTimePeriod(ExtendedTimePeriod timePeriod) {
        this.timePeriod = timePeriod;
        if (timePeriod != null) {
            section_dateDetails.setEntity(timePeriod);
        }
    }

}
