/**
* Copyright (C) 2020 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.ui.element;

import org.apache.commons.lang3.StringUtils;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.widgets.Display;

/**
 * @author k.luther
 * @since May 27, 2020
 */
public class FloatWithLabelElement extends NumberWithLabelElement {

    private Float start;
    private Float end;

    public FloatWithLabelElement(CdmFormFactory toolkit, ICdmFormElement parentElement, String labelString,
            Number initialNumber, int style) {
        super(toolkit, parentElement, labelString, initialNumber, style);
    }

    @Override
    public void modifyText(ModifyEvent event) {
        String value = text.getText();
        if(StringUtils.isBlank(value)){
            text.setBackground(Display.getCurrent().getSystemColor(SWT.COLOR_WHITE));
            super.modifyText(event);
            return;
        }

        try{
            Float number = Float.parseFloat(value);
            if((start != null && number < start) || (end != null && number > end)){
                exception = new NumberFormatException("You entered a number that is not within the allowed bounds.");
                throw exception;
            }
        }catch(NumberFormatException e){
            text.setBackground(Display.getCurrent().getSystemColor(SWT.COLOR_RED));
            firePropertyChangeEvent(new CdmPropertyChangeEvent(this, event));
            exception = e;
            return;
        }

        exception = null;
        text.setBackground(Display.getCurrent().getSystemColor(SWT.COLOR_WHITE));

        super.modifyText(event);
    }

    /**
     * <p>setLimits</p>
     *
     * @param numberOfDigits a int.
     * @param start a {@link java.lang.Float} object.
     * @param end a {@link java.lang.Float} object.
     */
    public void setLimits(int numberOfDigits, Float start, Float end){
        text.setTextLimit(numberOfDigits);
        this.start = start;
        this.end = end;
    }

    public void setLimits(int numberOfDigits, Integer start, Integer end){
        setLimits(numberOfDigits, start.floatValue(), end.floatValue());
    }
}
