/**
 * Copyright (C) 2007 EDIT
 * European Distributed Institute of Taxonomy
 * http://www.e-taxonomy.eu
 *
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */

package eu.etaxonomy.taxeditor.ui.element;

import org.eclipse.jface.util.PropertyChangeEvent;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.widgets.Label;
import org.eclipse.ui.forms.widgets.Section;

import eu.etaxonomy.cdm.api.facade.DerivedUnitFacade;

/**
 * <p>
 * GatheringEventUnitElement class.
 * </p>
 *
 * @author c.mathew
 * @created 23 Jul. 2013
 * @version 1.0
 */
public class GatheringEventUnitElement extends AbstractCdmFormElement implements ISelectable {

	private final Label label;
	private final MinMaxTextSection section_minMaxText;

	/**
	 * <p>
	 * Constructor for TimePeriodElement.
	 * </p>
	 *
	 * @param style
	 *            a int.
	 * @param formFactory
	 *            a {@link eu.etaxonomy.taxeditor.ui.element.CdmFormFactory}
	 *            object.
	 * @param parentElement
	 *            a {@link eu.etaxonomy.taxeditor.ui.element.ICdmFormElement}
	 *            object.
	 * @param labelString
	 *            a {@link java.lang.String} object.
	 * @param timePeriod
	 *            a {@link eu.etaxonomy.cdm.model.common.TimePeriod} object.
	 */
	public GatheringEventUnitElement(CdmFormFactory formFactory,
			ICdmFormElement parentElement,
			String labelString,
			DerivedUnitFacade gatheringEvent,
			MinMaxTextSection.UnitType unitType,
			final int style) {
		super(formFactory, parentElement);

		label = formFactory.createLabel(getLayoutComposite(), labelString);
		addControl(label);

		section_minMaxText = formFactory.createMinMaxTextSection(this,
				unitType,
				Section.TWISTIE);

		addControl(section_minMaxText);

		section_minMaxText.setEntity(gatheringEvent);

		formFactory.addPropertyChangeListener(this);
	}


	/** {@inheritDoc} */
	@Override
	public void propertyChange(PropertyChangeEvent event) {
		if (event == null) {
			return;
		}
		Object eventSource = event.getSource();

		if (getElements().contains(eventSource)) {
			handleEvent(eventSource);
		}
	}

	private void handleEvent(Object eventSource) {
		if (eventSource == section_minMaxText) {
			firePropertyChangeEvent(new CdmPropertyChangeEvent(this, null));
		}
	}

	/** {@inheritDoc} */
	@Override
	public void setBackground(Color color) {
		label.setBackground(color);
		section_minMaxText.setBackground(color);
	}

	@Override
	public void setSelected(boolean selected) {
		setBackground(selected ? SELECTED : getPersistentBackground());
	}

	public void setLabel(String string) {
		label.setText(string);
	}
}

