/**
 * Copyright (C) 2007 EDIT
 * European Distributed Institute of Taxonomy
 * http://www.e-taxonomy.eu
 *
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */

package eu.etaxonomy.taxeditor.ui.element;

import org.eclipse.jface.util.PropertyChangeEvent;
import org.eclipse.swt.widgets.Text;

import eu.etaxonomy.cdm.model.common.LanguageString;
import eu.etaxonomy.cdm.model.description.KeyStatement;

/**
 * @author n.hoffmann
 * @created Dec 3, 2010
 * @version 1.0
 */
public class KeyStatementElement extends MultilanguageTextElement {

	private KeyStatement keyStatement;

	/**
	 * @param formFactory
	 * @param formElement
	 */
	public KeyStatementElement(CdmFormFactory formFactory,
			ICdmFormElement formElement, String labelString,
			KeyStatement keyStatement, int textHeight, int style) {
		super(formFactory, formElement, labelString, null, textHeight, style);
		setKeyStatement(keyStatement);
	}

	/**
	 * @param keyStatement
	 *            the keyStatement to set
	 */
	public void setKeyStatement(KeyStatement keyStatement) {
		this.keyStatement = keyStatement;
		if (keyStatement != null) {
			setMultilanguageText(keyStatement.getLabel());
		}
	}

	/**
	 * @return the keyStatement
	 */
	public KeyStatement updateKeyStatement(KeyStatement ks) {
	    keyStatement = ks;
	    if(keyStatement == null) {
	        keyStatement = KeyStatement.NewInstance(element_languageString.getLanguageString().getLanguage(),
	                element_languageString.getLanguageString().getText());

	    } else {
	        LanguageString languageString = keyStatement.getLabel().get(element_languageString.getLanguageString().getLanguage());
	        if(languageString != null) {
	            languageString.setText(element_languageString.getText());
	        } else {
	            keyStatement.getLabel().put(element_languageString.getLanguageString().getLanguage(),
	                    element_languageString.getLanguageString());
	        }
	    }
	    return keyStatement;
	}


	/*
	 * (non-Javadoc)
	 *
	 * @see
	 * eu.etaxonomy.taxeditor.ui.forms.MultilanguageTextElement#propertyChange
	 * (org.eclipse.jface.util.PropertyChangeEvent)
	 */
	@Override
	public void propertyChange(PropertyChangeEvent event) {
		if (event == null) {
			return;
		}

		if (keyStatement == null
		        && event.getSource() == element_languageString
		        && element_languageString.getLanguageString() != null) {
		    // for the case when the user copy / pastes text into the
		    // (LanguageStringWithLabelElement) element_languageString text field
		    String text = element_languageString.getLanguageString().getText();
            setKeyStatement(KeyStatement.NewInstance(element_languageString.getLanguageString().getLanguage(),
		            text));
            // fix for setting the cursor to 0 when no key statement existed before
		    ((Text)element_languageString.getMainControl()).setSelection(text.length());
		}

		super.propertyChange(event);

	}

}
