/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.ui.element;

import org.apache.commons.lang3.StringUtils;

/**
 * <p>NumberWithLabelElement class.</p>
 *
 * @author n.hoffmann
 * @created Mar 22, 2010
 */
public abstract class NumberWithLabelElement extends TextWithLabelElement {


	protected NumberFormatException exception;

	/**
	 * <p>Constructor for NumberWithLabelElement.</p>
	 *
	 * @param toolkit a {@link eu.etaxonomy.taxeditor.ui.element.CdmFormFactory} object.
	 * @param parentElement a {@link eu.etaxonomy.taxeditor.ui.element.ICdmFormElement} object.
	 * @param labelString a {@link java.lang.String} object.
	 * @param initialNumber a {@link java.lang.Number} object.
	 * @param style a int.
	 */
	public NumberWithLabelElement(CdmFormFactory toolkit,
			ICdmFormElement parentElement, String labelString,
			Number initialNumber, int style) {
		super(toolkit, parentElement, labelString, null, null, style);
		setNumber(initialNumber);
	}

	/**
     * <p>Constructor for NumberWithLabelElement.</p>
     *
     * @param toolkit a {@link eu.etaxonomy.taxeditor.ui.element.CdmFormFactory} object.
     * @param parentElement a {@link eu.etaxonomy.taxeditor.ui.element.ICdmFormElement} object.
     * @param labelString a {@link java.lang.String} object.
     * @param initialNumber a {@link java.lang.Number} object.
     * @param style a int.
     */
    public NumberWithLabelElement(CdmFormFactory toolkit,
            ICdmFormElement parentElement, String labelString,
            Number initialNumber, Integer height, Integer length, int style) {
        super(toolkit, parentElement, labelString, null, null, style);
        setNumber(initialNumber);
    }


	/**
	 * <p>setNumber</p>
	 *
	 * @param number a {@link java.lang.Number} object.
	 */
	public void setNumber(Number number) {
		super.setText(getStringRepresentation(number));
	}

	/**
	 * Get the value of this field as an {@link Integer}.
	 * @return the Integer value or null if {@link NumberFormatException} occurs.
	 */
	public Integer getInteger() {
	    if(super.getText()!=null){
	        String text = super.getText().trim();
	        try {
	            return StringUtils.isBlank(text) ? null : new Integer(text);
	        } catch (NumberFormatException e) {
	            exception = e;
	        }
	    }
		return null;
	}

	/**
	 * Get the value of this field as a {@link Float}.
	 * @return the Float value or null if {@link NumberFormatException} occurs.
	 */
	public Float getFloat(){
	    String text = super.getText();
	    try {
	    	return StringUtils.isBlank(text) ? null : new Float(text);
	    } catch (NumberFormatException e) {
	        exception = e;
	    }
	    return null;
	}

	/**
	 * Get the value of this field as an {@link Double}.
	 * @return the Double value or null if {@link NumberFormatException} occurs.
	 */
	public Double getDouble(){
	    String text = super.getText();
	    try {
	    	return StringUtils.isBlank(text) ? null : new Double(text);
	    } catch (NumberFormatException e) {
	        exception = e;
	    }
	    return null;
	}

	private String getStringRepresentation(Object number){
		if(number != null){
			return number.toString();
		}
		return null;
	}





	/**
	 * <p>Getter for the field <code>exception</code>.</p>
	 *
	 * @return the exception
	 */
	public NumberFormatException getException() {
		return exception;
	}
}
