/**
* Copyright (C) 2019 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.ui.element;

import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;

import eu.etaxonomy.cdm.model.agent.ORCID;
import eu.etaxonomy.taxeditor.l10n.Messages;
import eu.etaxonomy.taxeditor.model.ImageResources;
import eu.etaxonomy.taxeditor.model.MessagingUtils;
import eu.etaxonomy.taxeditor.store.internal.TaxeditorStorePlugin;
import eu.etaxonomy.taxeditor.workbench.WorkbenchUtility;

/**
 * @author k.luther
 * @since 11.11.2019
 */
public class OrcidWithLabelElement extends AbstractUriWithExceptionLabelElement<ORCID> {

    private Button btnOpenBrowser;

    /**
     * @param formFactory
     * @param parentElement
     * @param labelString
     * @param initialObject
     * @param textHeight
     * @param style
     */
    protected OrcidWithLabelElement(CdmFormFactory formFactory, ICdmFormElement parentElement, String labelString,
            ORCID initialObject, Integer textHeight, int style) {
        super(formFactory, parentElement, labelString, initialObject, textHeight, style);
        exceptionString = Messages.OrcidWithLabelElement_ORCID_NOT_SAVED;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public void setParsedText(ORCID object) {
        if(object != null){
            super.setText(object.toString());
        }

    }

    /**
     * {@inheritDoc}
     */
    @Override
    protected ORCID getParsedText() throws Exception {
        String orcidText = super.getText();
        if(orcidText!=null){
            return ORCID.fromString(super.getText());
        }
        return null;
    }

    @Override
    protected void init(CdmFormFactory formFactory, String labelString, ORCID initialObject, Integer textHeight, int style) {

        //label
        initLabel(formFactory, labelString, false, getLayoutComposite());
        Composite textAndButton = formFactory.createComposite(getLayoutComposite(), style);
        addControl(textAndButton);
        textAndButton.setLayout(LayoutConstants.LAYOUT(2, false));
        textAndButton.setLayoutData(LayoutConstants.FILL_HORIZONTALLY());

        //uri text
        initText(formFactory, null, textHeight, null, false, style, textAndButton);
        setParsedText(initialObject);
      //button
        btnOpenBrowser = formFactory.createButton(textAndButton, "", SWT.NONE); //$NON-NLS-1$
        btnOpenBrowser.setImage(ImageResources.getImage(ImageResources.WEB));
        btnOpenBrowser.setToolTipText(Messages.UriWithLabelElement_OPEN_EXTERNAL_BROWSER);
        btnOpenBrowser.addSelectionListener(new SelectionAdapter() {
            @Override
            public void widgetSelected(SelectionEvent e) {
                String errorTitle = Messages.UriWithLabelElement_INVALID_URL;
                String errorText = Messages.UriWithLabelElement_COULD_NOT_OPEN_BROWSER;

                ORCID orcid = parseText();
                if(orcid!=null){
                    try {
                        WorkbenchUtility.openWebpage(orcid.asURI());
                    } catch (IllegalArgumentException iae) {
                        MessagingUtils.informationDialog(errorTitle, new Status(IStatus.WARNING, TaxeditorStorePlugin.PLUGIN_ID, errorText, iae));
                    }
                }
                else{
                    MessagingUtils.informationDialog(errorTitle, new Status(IStatus.WARNING, TaxeditorStorePlugin.PLUGIN_ID, errorText, null));
                }
            }
        });
        btnOpenBrowser.setLayoutData(LayoutConstants.RIGHT());
        initExceptionLabel(getLayoutComposite(), formFactory);
    }


//    @Override
//    public ORCID parseText(){
//        try {
//            labelException.setText("");
//            labelException.setFont(JFaceResources.getFontRegistry().get(JFaceResources.DEFAULT_FONT));
//            labelException.setForeground(getPersistentBackground());
//            return getParsedText();
//        } catch (Exception e) {
//            labelException.setFont(JFaceResources.getFontRegistry().getBold(JFaceResources.DEFAULT_FONT));
//            labelException.setForeground(Display.getCurrent().getSystemColor(SWT.COLOR_RED));
//            labelException.setText(Messages.OrcidWithLabelElement_ORCID_NOT_SAVED + " " + e.getLocalizedMessage());
//            labelException.requestLayout();
//            return null;
//        }
//    }

}

