/**
* Copyright (C) 2021 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.ui.element;

import org.apache.commons.lang3.StringUtils;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;

import eu.etaxonomy.taxeditor.model.ImageResources;

/**
 * @author k.luther
 * @since Apr 1, 2021
 */
public class RemovableTextElement extends AbstractCdmFormElement implements SelectionListener {

    private final Text text_cache;
    private final Button btnRemoveElement;
    private final Label label;
    private final Composite container;
    private boolean state;

    protected RemovableTextElement(CdmFormFactory formFactory, ICdmFormElement parentElement,
            String labelString, String initialText, int style) {
        super(formFactory, parentElement);

        label = formFactory.createLabel(getLayoutComposite(), labelString, style);
        addControl(label);

        container = formFactory.createComposite(getLayoutComposite(), SWT.WRAP);
        container.setLayout(LayoutConstants.LAYOUT(2, false));
        container.setLayoutData(LayoutConstants.FILL_HORIZONTALLY());
        addControl(container);

        text_cache = formFactory.createText(container, initialText, SWT.WRAP | SWT.MULTI);
        addControl(text_cache);
        text_cache.setEnabled(false);
        text_cache.setLayoutData(LayoutConstants.FILL());

        btnRemoveElement = formFactory.createButton(container, "Edit", SWT.TOGGLE);
        btnRemoveElement.setText("");
        btnRemoveElement.setImage(ImageResources.getImage(ImageResources.TRASH_ICON));
        addControl(btnRemoveElement);
        btnRemoveElement.addSelectionListener(this);
        if (initialText != null){
            setState(true);
        }else {
            setState(false);
        }

    }


    @Override
    public void widgetSelected(SelectionEvent e) {
       setText(null);
       firePropertyChangeEvent(new CdmPropertyChangeEvent(this, e));
    }

    @Override
    public void widgetDefaultSelected(SelectionEvent e) {
        // TODO Auto-generated method stub

    }

    public void setText(String text){
        if (text == null){
            this.text_cache.setText("");
        }
        if (StringUtils.isBlank(text)){
            setState(false);
        }else{
            setState(true);
        }
    }


    public boolean isState() {
        return state;
    }


    public void setState(boolean state) {
        this.state = state;
    }

}
