/**
* Copyright (C) 2016 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.ui.element;

/**
 * @author k.luther
 * @date 22.06.2016
 *
 */

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Set;

import org.eclipse.jface.util.PropertyChangeEvent;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.widgets.Button;

import eu.etaxonomy.cdm.model.common.Language;
import eu.etaxonomy.cdm.model.term.Representation;
import eu.etaxonomy.cdm.model.term.TermBase;
import eu.etaxonomy.cdm.model.term.TermType;
import eu.etaxonomy.cdm.persistence.dto.TermDto;
import eu.etaxonomy.taxeditor.model.DefaultTermComparator;
import eu.etaxonomy.taxeditor.model.ImageResources;
import eu.etaxonomy.taxeditor.preference.PreferencesUtil;
import eu.etaxonomy.taxeditor.store.CdmStore;
import eu.etaxonomy.taxeditor.ui.combo.term.TermComboElement;


public class RepresentationElement extends AbstractCdmFormElement implements IEnableableFormElement{

        protected TermComboElement<Language> combo_language;

        protected TextWithLabelElement element_Label;
        protected TextWithLabelElement element_abbrevLabel;
        protected TextWithLabelElement element_plural;
        protected TextWithLabelElement element_text;

        protected Button button;

        protected Representation selectedRepresentation;
        protected TermBase term;
        protected TermDto termDto;

		private Button removeRepresentation;

		boolean isEnabled = true;

        public RepresentationElement(CdmFormFactory formFactory,
                ICdmFormElement formElement, TermBase term,
                Integer textHeight, int style, boolean fill) {
            this(formFactory, formElement, null, term, textHeight, style, fill);
        }

        public RepresentationElement(CdmFormFactory formFactory,
                ICdmFormElement formElement, Representation representation, TermBase term,
                Integer textHeight, int style, boolean fill) {
            super(formFactory, formElement);

            formFactory.addPropertyChangeListener(this);

            element_Label = this.formFactory.createTextWithLabelElement(formElement, "Label", null, style);
            element_abbrevLabel = this.formFactory.createTextWithLabelElement(formElement, "abbrev. Label", null, style);
            element_plural = this.formFactory.createTextWithLabelElement(formElement, "Plural", null, style);
            element_text = this.formFactory.createMultiLineTextWithLabel(formElement, "Description", textHeight, style);
            if (PreferencesUtil.isMultilanguageTextEditingCapability()) {
                createRepresentationEditingElements(formElement, style);
            }
            setTerm(term, fill);
            if(representation!=null){
                setSelectedRepresentation(representation);
            }

        }

        public RepresentationElement(CdmFormFactory formFactory,
                ICdmFormElement formElement, Representation representation, TermDto term,
                Integer textHeight, int style, boolean fill) {
            super(formFactory, formElement);

            formFactory.addPropertyChangeListener(this);

            element_Label = this.formFactory.createTextWithLabelElement(formElement, "Label", null, style);
            element_abbrevLabel = this.formFactory.createTextWithLabelElement(formElement, "abbrev. Label", null, style);
            element_plural = this.formFactory.createTextWithLabelElement(formElement, "Plural", null, style);
            element_text = this.formFactory.createMultiLineTextWithLabel(formElement, "Description", textHeight, style);
            if (PreferencesUtil.isMultilanguageTextEditingCapability()) {
                createRepresentationEditingElements(formElement, style);
            }
            setTermDto(term, fill);
            if(representation!=null){
                setSelectedRepresentation(representation);
            }

        }

        protected void createRepresentationEditingElements(
                ICdmFormElement formElement, int style) {
        	removeRepresentation = formFactory.createButton(getLayoutComposite(), null,
    				SWT.PUSH);
    		removeRepresentation.setImage(ImageResources
    				.getImage(ImageResources.TRASH_ICON));
    		removeRepresentation.setToolTipText("Remove representation");
    		removeRepresentation.addSelectionListener(new DeleteListener(this));
    		addControl(removeRepresentation);

            combo_language = formFactory.createDefinedTermComboElement(TermType.Language, formElement,
                            "", null, false, style);

       }

        public List<Language> getLanguages() {

            ArrayList<Language> languageList = new ArrayList<Language>();
            Set<Representation> reps = null;
            if (term != null){
                reps = term.getRepresentations();
            }else{
                reps = termDto.getRepresentations();
            }

            for (Representation rep: reps){
                languageList.add(rep.getLanguage());
            }

            Collections.sort(languageList, new DefaultTermComparator<Language>());

            return languageList;
        }

        public void setTerm(
                TermBase term, boolean update) {
            this.term = term;
            if (term.getRepresentations().isEmpty()){
            	//if the term has no representation at all, create a default one.
            	Representation rep = Representation.NewInstance("", "empty representation", "", PreferencesUtil.getGlobalLanguage());
            	term.addRepresentation(rep);
            }

            if (selectedRepresentation != null) {
               combo_language.setTerms(getLanguages());

            } else{
            	 setEnabledControls(false);
                 if (PreferencesUtil.isMultilanguageTextEditingCapability()) {
                	 removeRepresentation.setEnabled(false);
                 }
            }
            if (update){
            	updateControls();
            }
        }

        public void setTermDto(
                TermDto term, boolean update) {
            this.termDto = term;
            if (term.getRepresentations().isEmpty()){
                //if the term has no representation at all, create a default one.
                Representation rep = Representation.NewInstance("", "empty representation", "", PreferencesUtil.getGlobalLanguage());
                term.addRepresentation(rep);
            }

            if (selectedRepresentation != null) {
               combo_language.setTerms(getLanguages());

            } else{
                 setEnabledControls(false);
                 if (PreferencesUtil.isMultilanguageTextEditingCapability()) {
                     removeRepresentation.setEnabled(false);
                 }
            }
            if (update){
                updateControls();
            }
        }

        private void setEnabledControls(boolean enabled){
            element_Label.setEnabled(enabled);
            element_abbrevLabel.setEnabled(enabled);
            element_plural.setEnabled(enabled);
            element_text.setEnabled(enabled);
        }

        @Override
        public void setEnabled(boolean isEnabled){
            this.isEnabled = isEnabled;
            setEnabledControls(isEnabled);

        }

        @Override
        public boolean isEnabled(){
            return isEnabled;
        }

        protected void updateControls() {
            Representation preferredRepresentation = null;
            if (term != null){
                 preferredRepresentation = term
                    .getPreferredRepresentation(PreferencesUtil.getGlobalLanguage() );
            }else{
                 preferredRepresentation = termDto
                        .getPreferredRepresentation(PreferencesUtil.getGlobalLanguage() );
            }

            element_Label.setText(preferredRepresentation != null? preferredRepresentation.getLabel(): null);
            element_abbrevLabel.setText(preferredRepresentation != null? preferredRepresentation.getAbbreviatedLabel(): null);
            element_plural.setText(preferredRepresentation != null? preferredRepresentation.getPlural(): null);
            element_text.setText(preferredRepresentation != null? preferredRepresentation.getDescription(): null);
            selectedRepresentation = preferredRepresentation;
            if (PreferencesUtil.isMultilanguageTextEditingCapability()) {
                combo_language.setSelection(preferredRepresentation == null?CdmStore.getDefaultLanguage():preferredRepresentation.getLanguage());
                boolean removePossible = false;
                if (term != null){
                    removePossible = term.getRepresentations().size() > 1 ;
                }else{
                    removePossible = termDto.getRepresentations().size() > 1 ;
                }
                removeRepresentation.setEnabled(removePossible);

            }
        }

        public TermBase getTerm() {
            return term;
        }

        @Override
        public void propertyChange(PropertyChangeEvent event) {
            if (event == null) {
                return;
            }
            Object eventSource = event.getSource();

            if (eventSource == element_abbrevLabel) {
                String abbrevLabel = ((TextWithLabelElement) eventSource)
                        .getText();
                selectedRepresentation.setAbbreviatedLabel(abbrevLabel);
                firePropertyChangeEvent(new CdmPropertyChangeEvent(this, event));
            }
            else if (eventSource == element_plural){
                String plural = ((TextWithLabelElement) eventSource)
                        .getText();
                selectedRepresentation.setPlural(plural);
                firePropertyChangeEvent(new CdmPropertyChangeEvent(this, event));
            } else if (eventSource == element_Label){
                String label = ((TextWithLabelElement) eventSource)
                        .getText();
                selectedRepresentation.setLabel(label);
                firePropertyChangeEvent(new CdmPropertyChangeEvent(this, event));
            } else if (eventSource == element_text){
                String text = ((TextWithLabelElement) eventSource)
                        .getText();
                selectedRepresentation.setText(text);
                firePropertyChangeEvent(new CdmPropertyChangeEvent(this, event));
            } else if (eventSource == button){
            	firePropertyChangeEvent(new CdmPropertyChangeEvent(this, event));
            } else if (eventSource == combo_language){
                if (combo_language.getSelection() != null){
                    setEnabledControls(true);
                    boolean removePossible = false;
                    if (term != null){
                        removePossible = term.getRepresentations().size() > 1 ;
                    }else{
                        removePossible = termDto.getRepresentations().size() > 1 ;
                    }
                    removeRepresentation.setEnabled(removePossible);

                    Language selectedLanguage = combo_language.getSelection();
                    if (selectedLanguage != null) {
                        selectedRepresentation = getTerm().getRepresentation(selectedLanguage);
                        if (selectedRepresentation == null){
                            selectedRepresentation = Representation.NewInstance("", "", "", selectedLanguage);
                        }
                        element_Label.setText(selectedRepresentation.getLabel());
                        element_abbrevLabel.setText(selectedRepresentation.getAbbreviatedLabel());
                        element_plural.setText(selectedRepresentation.getPlural());
                        element_text.setText(selectedRepresentation.getDescription());

                    }else {
                        setEnabledControls(false);
                    }
                } else{
                    setEnabledControls(false);
                    removeRepresentation.setEnabled(false);
                }
                firePropertyChangeEvent(new CdmPropertyChangeEvent(this, event));
            }
        }

        public Representation getSelectedRepresentation() {
            return selectedRepresentation;
        }

        public void setSelectedRepresentation(Representation selectedRepresentation) {
            if (selectedRepresentation == null){
                selectedRepresentation = Representation.NewInstance("", "", "", null);
            }
            this.selectedRepresentation = selectedRepresentation;
        }

        private class DeleteListener extends SelectionAdapter {

    		private final RepresentationElement element;

    		public DeleteListener(RepresentationElement element) {
    			this.element = element;
    		}

    		@Override
    		public void widgetSelected(SelectionEvent e) {
    		    if (term != null){
    		        term.removeRepresentation(selectedRepresentation);
    		    }else {
    		        termDto.getRepresentations().remove(selectedRepresentation);
    		    }
    			firePropertyChangeEvent(new CdmPropertyChangeEvent(
    					this, e));
    			updateControls();
    		}
    	}
    }


