/**
* Copyright (C) 2018 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.ui.element;

import org.eclipse.jface.util.PropertyChangeEvent;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.widgets.Label;

import eu.etaxonomy.cdm.model.common.TimePeriod;

public abstract class TimePeriodElementBase<T extends TimePeriod>
		extends AbstractRelevanceFormElement
		implements ISelectable {

    protected T timePeriod;
    protected Label label;
    protected DateDetailSection<T> section_dateDetails;

	/**
	 * Constructor for TimePeriodElement.
	 */
	public TimePeriodElementBase(CdmFormFactory formFactory,
			ICdmFormElement parentElement, String labelString,
			T timePeriod, final int style) {
		super(formFactory, parentElement);
		label = formFactory.createLabel(getLayoutComposite(), labelString);

		addControl(label);

		section_dateDetails = createDateDetailSection();
		addControl(section_dateDetails);

		setTimePeriod(timePeriod);

		formFactory.addPropertyChangeListener(this);
	}


	protected abstract DateDetailSection<T> createDateDetailSection();

	public void setEntity(T timePeriod) {
		setTimePeriod(timePeriod);
	}

	@Override
	public void propertyChange(PropertyChangeEvent event) {
		if (event == null) {
			return;
		}
		Object eventSource = event.getSource();

		if (getElements().contains(eventSource)) {
			handleEvent(eventSource);
		}
	}

	private void handleEvent(Object eventSource) {
		if (eventSource == section_dateDetails) {
			timePeriod = section_dateDetails.getEntity();
			firePropertyChangeEvent(new CdmPropertyChangeEvent(this, null));
		}
	}

	public void setTimePeriod(T timePeriod) {
		this.timePeriod = timePeriod;
		if (timePeriod != null) {
			section_dateDetails.setEntity(timePeriod);
		}
	}

	public T getTimePeriod() {
		return timePeriod;
	}

	@Override
	public void setBackground(Color color) {
		label.setBackground(color);
		section_dateDetails.setBackground(color);
	}

	@Override
	public void setSelected(boolean selected) {
		setBackground(selected ? SELECTED : getPersistentBackground());
	}

	public void setLabel(String string) {
		label.setText(string);
	}

    @Override
    public void updateCacheRelevance() {
        if (section_dateDetails != null){
            section_dateDetails.updateCacheRelevance();
        }
    }

    @Override
    public void addDependsOnCache(ToggleableTextElement cacheElement) {
        if (section_dateDetails != null){
            section_dateDetails.addDependsOnCache(cacheElement);
        }
    }
}