/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.ui.element;

import java.io.IOException;

import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;

import eu.etaxonomy.cdm.common.URI;
import eu.etaxonomy.taxeditor.l10n.Messages;
import eu.etaxonomy.taxeditor.model.ImageResources;
import eu.etaxonomy.taxeditor.model.MessagingUtils;
import eu.etaxonomy.taxeditor.store.internal.TaxeditorStorePlugin;
import eu.etaxonomy.taxeditor.workbench.WorkbenchUtility;

/**
 * @author n.hoffmann
 * @created Dec 20, 2010
 */
public class UriWithLabelElement extends AbstractUriWithExceptionLabelElement<URI> {

    private Button btnOpenBrowser;
    private boolean isValidUri = false;

    protected UriWithLabelElement(CdmFormFactory formFactory, ICdmFormElement parentElement, String labelString,
            URI initialObject, Integer textHeight, int style) {
        super(formFactory, parentElement, labelString, initialObject, textHeight, style);
        exceptionString = Messages.UriWithLabelElement_URL_NOT_SAVED;
	}

    @Override
    protected void init(CdmFormFactory formFactory, String labelString, URI initialObject, Integer textHeight, int style) {

        //label
        initLabel(formFactory, labelString, false, getLayoutComposite());

        //composite(uri + button)
        Composite textAndButton = formFactory.createComposite(getLayoutComposite(), style);
        addControl(textAndButton);
        textAndButton.setLayout(LayoutConstants.LAYOUT(2, false));
        textAndButton.setLayoutData(LayoutConstants.FILL_HORIZONTALLY());

        //uri text
        initText(formFactory, null, textHeight, null, false, style, textAndButton);
        setParsedText(initialObject);
        //button
        btnOpenBrowser = formFactory.createButton(textAndButton, "", SWT.NONE); //$NON-NLS-1$
        btnOpenBrowser.setImage(ImageResources.getImage(ImageResources.WEB));
        btnOpenBrowser.setToolTipText(Messages.UriWithLabelElement_OPEN_EXTERNAL_BROWSER);
        btnOpenBrowser.addSelectionListener(new SelectionAdapter() {
            @Override
            public void widgetSelected(SelectionEvent e) {
                String errorTitle = Messages.UriWithLabelElement_INVALID_URL;
                String errorText = Messages.UriWithLabelElement_COULD_NOT_OPEN_BROWSER;

                URI uri = parseText();
                if(uri!=null){
                    try {
                        WorkbenchUtility.openWebpage(uri.toURL());
                    } catch (IOException pie) {
                        MessagingUtils.informationDialog(errorTitle, new Status(IStatus.WARNING, TaxeditorStorePlugin.PLUGIN_ID, errorText, pie));
                    } catch (IllegalArgumentException iae) {
                        MessagingUtils.informationDialog(errorTitle, new Status(IStatus.WARNING, TaxeditorStorePlugin.PLUGIN_ID, errorText, iae));
                    }
                }
                else{
                    MessagingUtils.informationDialog(errorTitle, new Status(IStatus.WARNING, TaxeditorStorePlugin.PLUGIN_ID, errorText, null));
                }
            }
        });
        btnOpenBrowser.setLayoutData(LayoutConstants.RIGHT());
        initExceptionLabel(getLayoutComposite(), formFactory);
    }

    @Override
    public void setParsedText(URI object) {
        if(object != null){
            super.setText(object.toString());
        }


    }

    @Override
    protected URI getParsedText() throws Exception {
        String uriText = super.getText();
        if(uriText!=null){
            btnOpenBrowser.setEnabled(true);
            try{
                URI uri = new URI(super.getText());
                return uri;
            }catch (Exception e){
                btnOpenBrowser.setEnabled(false);
                throw e;
            }
        }else{
            btnOpenBrowser.setEnabled(false);
        	return null;
        }
    }
}
