/**
* Copyright (C) 2014 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.ui.mvc.combo;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.DisposeEvent;
import org.eclipse.swt.events.DisposeListener;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.forms.widgets.FormToolkit;
import org.eclipse.ui.forms.widgets.TableWrapData;
import org.eclipse.ui.forms.widgets.TableWrapLayout;

import eu.etaxonomy.cdm.model.term.IEnumTerm;
import eu.etaxonomy.taxeditor.ui.element.CdmFormFactory;
import eu.etaxonomy.taxeditor.ui.element.ICdmFormElement;
import eu.etaxonomy.taxeditor.ui.mvc.AbstractCdmComposite;

/**
 * @author pplitzner
 * @date 29.01.2014
 *
 */
public class EnumTermCombo extends AbstractCdmComposite<EnumTermComboController> {

    private final FormToolkit toolkit = new FormToolkit(Display.getCurrent());
    private final Combo combo;

    private final Class<? extends IEnumTerm<?>> enumType;

    /**
     * Create the composite.
     * @param parent
     * @param style
     */
    public EnumTermCombo(Composite parent, int style, Class<? extends IEnumTerm<?>> enumType) {
        super(parent, style);
        this.enumType = enumType;
        addDisposeListener(new DisposeListener() {
            @Override
            public void widgetDisposed(DisposeEvent e) {
                toolkit.dispose();
            }
        });
        toolkit.adapt(this);
        toolkit.paintBordersFor(this);
        {
            TableWrapLayout tableWrapLayout = new TableWrapLayout();
            tableWrapLayout.verticalSpacing = 0;
            tableWrapLayout.topMargin = 0;
            tableWrapLayout.horizontalSpacing = 0;
            tableWrapLayout.bottomMargin = 0;
            tableWrapLayout.rightMargin = 0;
            tableWrapLayout.leftMargin = 0;
            setLayout(tableWrapLayout);
        }

        combo = new Combo(this, SWT.READ_ONLY);
        combo.setLayoutData(new TableWrapData(TableWrapData.FILL_GRAB, TableWrapData.TOP, 1, 1));
        toolkit.adapt(combo);
        toolkit.paintBordersFor(combo);

    }
    public Combo getCombo() {
        return combo;
    }
    /* (non-Javadoc)
     * @see eu.etaxonomy.taxeditor.ui.mvc.interfaces.CdmComposite#initController(java.lang.Object, eu.etaxonomy.taxeditor.ui.element.CdmFormFactory, eu.etaxonomy.taxeditor.ui.element.ICdmFormElement)
     */
    @Override
    protected void initInternalController(CdmFormFactory formFactory, ICdmFormElement parentElement) {
        controller = new EnumTermComboController(combo, formFactory, parentElement, enumType, SWT.NONE);
    }

}
