/**
* Copyright (C) 2014 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.ui.mvc.element;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.DisposeEvent;
import org.eclipse.swt.events.DisposeListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.DateTime;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.forms.widgets.FormToolkit;
import org.eclipse.ui.forms.widgets.TableWrapLayout;
import org.joda.time.DateTimeZone;

import eu.etaxonomy.taxeditor.model.ImageResources;
import eu.etaxonomy.taxeditor.ui.element.CacheRelevance;
import eu.etaxonomy.taxeditor.ui.element.CacheRelevanceHelper;
import eu.etaxonomy.taxeditor.ui.element.CdmFormFactory;
import eu.etaxonomy.taxeditor.ui.element.ICacheRelevantFormElement;
import eu.etaxonomy.taxeditor.ui.element.ICdmFormElement;
import eu.etaxonomy.taxeditor.ui.element.ToggleableTextElement;
import eu.etaxonomy.taxeditor.ui.mvc.AbstractCdmComposite;

/**
 * @author pplitzner
 * @date 16.06.2014
 */
public class DateElement
        extends AbstractCdmComposite<DateElementController>
        implements ICacheRelevantFormElement {

    private static final String pattern = "yyyy-MM-dd HH:mm";

    private final FormToolkit toolkit = new FormToolkit(Display.getCurrent());
    private org.joda.time.DateTime initialDateTime;
    private final Text textDate;
    private final Button openDateDialog;

    protected CacheRelevanceHelper cacheRelevanceHelper = new CacheRelevanceHelper();

    /**
     * Create the composite.
     */
    public DateElement(Composite parent, org.joda.time.DateTime initDateTime, String label, int style, boolean editableText) {
        super(parent, style);
        if (initDateTime != null){
        	this.initialDateTime = initDateTime.toDateTime(DateTimeZone.UTC);
        }

        addDisposeListener(new DisposeListener() {
            @Override
            public void widgetDisposed(DisposeEvent e) {
                toolkit.dispose();
            }
        });
        toolkit.adapt(this);
        toolkit.paintBordersFor(this);
        {
            TableWrapLayout tableWrapLayout = new TableWrapLayout();
            tableWrapLayout.verticalSpacing = 0;
            tableWrapLayout.topMargin = 0;
            tableWrapLayout.rightMargin = 0;
            tableWrapLayout.leftMargin = 0;
            tableWrapLayout.horizontalSpacing = 0;
            tableWrapLayout.bottomMargin = 0;
            setLayout(tableWrapLayout);
        }

        Composite composite = new Composite (this, SWT.NONE);
    	GridLayout layout = new GridLayout ();

    	layout.numColumns = 2;
    	layout.marginWidth = 0;

    	composite.setLayout(layout);


     	textDate = toolkit.createText(composite, null, SWT.BORDER | SWT.SINGLE | SWT.FULL_SELECTION);

        textDate.setEditable(editableText);

        GridData gridData = new GridData(GridData.FILL_HORIZONTAL);
        gridData.minimumWidth = 100;

        textDate.setLayoutData(gridData);
        openDateDialog = new Button (composite, SWT.PUSH);
        Image image = ImageResources.getImage(ImageResources.DATE);
   //     openDateDialog.setText ("Open Date Dialog");
        openDateDialog.setImage(image);

        openDateDialog.addSelectionListener (new SelectionAdapter() {
        	@Override
        	public void widgetSelected(SelectionEvent e){
        		final Shell dialog = new Shell (parent.getShell(), SWT.DIALOG_TRIM);

    			dialog.setLayout (new GridLayout (3, false));
    			dialog.setText(label);

    			final DateTime calendar = new DateTime (dialog, SWT.CALENDAR | SWT.BORDER);

    			final DateTime time = new DateTime (dialog, SWT.TIME | SWT.SHORT);
    			if (initialDateTime != null){
    				calendar.setDate(initialDateTime.getYear(), initialDateTime.getMonthOfYear()-1, initialDateTime.getDayOfMonth());
    				time.setHours(initialDateTime.getHourOfDay());
        			time.setMinutes(initialDateTime.getMinuteOfHour());
//        			time.setSeconds(initialDateTime.getSecondOfMinute());
    			}

//    			new Label (dialog, SWT.NONE);
//    			new Label (dialog, SWT.NONE);
    			Button ok = new Button (dialog, SWT.PUSH);
    			ok.setText ("OK");
    			ok.setLayoutData(new GridData (SWT.FILL, SWT.CENTER, false, false));
    			ok.addSelectionListener (new SelectionAdapter() {
    	        	@Override
    	        	public void widgetSelected(SelectionEvent e){
    	        		setData(makeZonedTimeFromSWT(calendar, time));
    	        		dialog.close ();
    				}
    			});
    			dialog.setDefaultButton (ok);
    			dialog.pack ();
    			dialog.open ();
    		}
        });

        toolkit.adapt(textDate, true, true);
    }

    @Override
    protected void initInternalController(CdmFormFactory formFactory, ICdmFormElement parentElement) {
        controller = new DateElementController(this, formFactory, parentElement, initialDateTime);
        formFactory.adapt(controller);
    }
    public Text getTextDate() {
        return textDate;
    }

    public static org.joda.time.DateTime makeZonedTimeFromSWT(
	                                org.eclipse.swt.widgets.DateTime widget,
	                                org.eclipse.swt.widgets.DateTime dateTime) {
		  //the first month of the year is 0
		  return new org.joda.time.DateTime(widget.getYear(),
				  widget.getMonth()+1,
				  widget.getDay(),
				  dateTime.getHours(),
				  dateTime.getMinutes(),
                  0,
                  0,
                  DateTimeZone.UTC);
	}

	public static void updateSWTwithJoda(
	                                org.eclipse.swt.widgets.DateTime widget,
	                                DateTime dateTime) {
		  widget.setYear(dateTime.getYear());
		  widget.setMonth(dateTime.getMonth() -1);
		  widget.setDay(dateTime.getDay());
		  widget.setHours(dateTime.getHours());
		  widget.setMinutes(dateTime.getMinutes());
	}

	public void setData (org.joda.time.DateTime data) {
			this.initialDateTime = data;
			this.controller.setDateTime(data);
			this.textDate.setText(initialDateTime.toString(pattern));
	}

    @Override
    public void updateCacheRelevance() {
        textDate.setBackground(cacheRelevance().getColor());
    }

    @Override
    public void addDependsOnCache(ToggleableTextElement cacheElement) {
        cacheRelevanceHelper.addDependsOnCache(cacheElement);
    }

    @Override
    public CacheRelevance cacheRelevance() {
        return cacheRelevanceHelper.cacheRelevance();
    }
}