/**
* Copyright (C) 2014 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.ui.mvc.element;

import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.widgets.Text;
import org.joda.time.DateTime;

import eu.etaxonomy.taxeditor.ui.element.AbstractRelevanceFormElement;
import eu.etaxonomy.taxeditor.ui.element.CdmFormFactory;
import eu.etaxonomy.taxeditor.ui.element.CdmPropertyChangeEvent;
import eu.etaxonomy.taxeditor.ui.element.ICdmFormElement;
import eu.etaxonomy.taxeditor.ui.element.IEnableableFormElement;
import eu.etaxonomy.taxeditor.ui.mvc.interfaces.CdmCompositeController;

/**
 * @author pplitzner
 * @date 16.06.2014
 */
public class DateElementController
        extends AbstractRelevanceFormElement
        implements IEnableableFormElement, CdmCompositeController, ModifyListener {

    private final Text textDate;
    private DateTime dateTime;

    public DateElementController(DateElement dateTimeElement, CdmFormFactory formFactory,
    		ICdmFormElement parentElement, DateTime initialDateTime) {
        super(formFactory, parentElement);
        this.textDate = dateTimeElement.getTextDate();
//        if (initialDateTime != null){
//        	this.textDate.setText(initialDateTime.format(DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm")));
//        }
        this.textDate.setText(initialDateTime!=null?initialDateTime.toString("yyyy-MM-dd HH:mm"):"");
        addControl(textDate);
        dateTime = initialDateTime;
        textDate.addModifyListener(this);
    }

//    public DateElementController(DateElementFormElement dateTimeElement, CdmFormFactory formFactory,  DateTime initialDateTime) {
//        super(formFactory, dateTimeElement);
//        this.textDate = dateTimeElement.getTextDate();
//        this.textDate.setText(initialDateTime!=null?initialDateTime.toString("yyyy-MM-dd"):"");
//        addControl(textDate);
//        textDate.addModifyListener(this);
//    }


    @Override
    public void modifyText(ModifyEvent e) {
        try {
//            String text = textDate.getText();
//            DateTime parsedDateTime = DateTime.parse(text);
//            this.dateTime = parsedDateTime;
            firePropertyChangeEvent(new CdmPropertyChangeEvent(this, e));
        } catch (IllegalArgumentException iae) {
            System.err.println(iae.getMessage());
        }
    }

    public DateTime getDateTime(){
        return dateTime;
    }
    public void setDateTime(DateTime date){
       dateTime = date;
    }

    @Override
    public void setEnabled(boolean enabled) {
        textDate.setEnabled(enabled);
    }

    @Override
    public boolean isEnabled() {
        return textDate.isEnabled();
    }

    @Override
    public void updateCacheRelevance() {
        System.out.println("updateCacheRelevance not yet implemented for " + this.getClass().getName());
    }
}