/**
* Copyright (C) 2014 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.ui.mvc.element;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.KeyAdapter;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Text;

import eu.etaxonomy.taxeditor.preference.Resources;
import eu.etaxonomy.taxeditor.store.StoreUtil;
import eu.etaxonomy.taxeditor.ui.element.AbstractRelevanceFormElement;
import eu.etaxonomy.taxeditor.ui.element.CdmFormFactory;
import eu.etaxonomy.taxeditor.ui.element.CdmPropertyChangeEvent;
import eu.etaxonomy.taxeditor.ui.element.ICdmFormElement;
import eu.etaxonomy.taxeditor.ui.element.IEnableableFormElement;
import eu.etaxonomy.taxeditor.ui.element.ISelectable;
import eu.etaxonomy.taxeditor.ui.mvc.interfaces.CdmCompositeController;

/**
 * TextWithLabelElement class.
 *
 * @author n.hoffmann
 */
public class TextWithLabelElementController
        extends AbstractRelevanceFormElement
        implements ModifyListener, IEnableableFormElement, ISelectable, CdmCompositeController {

    protected Text text;
//    private Label label;

    /** Constant <code>MAX_HEIGHT=0</code> */
    public static final int MAX_HEIGHT = 0;
    /** Constant <code>SINGLE=-1</code> */
    public static final int SINGLE = -1;

    /**
     * Constructor for TextWithLabelElement.
     */
    public TextWithLabelElementController(TextWithLabelElementMVC textWithLabelElement, CdmFormFactory formFactory, ICdmFormElement parentElement,
            String initialText, Integer textHeight, int style) {
        super(formFactory, parentElement);

        int scrollStyle = textHeight == null ? SWT.NULL : (SWT.V_SCROLL | SWT.MULTI);

        int combinedStyle = style | SWT.BORDER | scrollStyle;

        // SWT.PASSWORD does not work when SWT.WRAP is set.
        if (style != SWT.PASSWORD) {
            combinedStyle = combinedStyle | SWT.WRAP;
        }

        text = textWithLabelElement.getText();

        addControl(text);

        if (textHeight == null) {
            text.addKeyListener(new KeyAdapter() {
                @Override
                public void keyPressed(KeyEvent e) {
                    if (e.character == SWT.CR) {
                        // Don't accept carriage returns as input when in single
                        // line mode
                        e.doit = false;
                    } else if (e.character == SWT.TAB) {
                        // traverse is not working for wrapped text widgets so
                        // we reintroduce it here
                        e.doit = false;
                        TextWithLabelElementController.this.text.traverse(SWT.TRAVERSE_TAB_NEXT);
                    }
                }
            });
        }

        text.addModifyListener(this);
    }

    /**
     * Get the text of this composites text composite
     */
    public String getText() {
        return text.getText();
    }

    /**
     * Set the text of this composites text composite
     */
    public void setText(String string) {
        StoreUtil.setTextWithoutModifyListeners(text, string);
    }

    @Override
    public void modifyText(ModifyEvent e) {
        firePropertyChangeEvent(new CdmPropertyChangeEvent(this, e));
    }

    @Override
    public void setEnabled(boolean enabled) {
        text.setEnabled(enabled);
        String symbolicName = enabled ? Resources.COLOR_FONT_DEFAULT : Resources.COLOR_TEXT_DISABLED;
        text.setForeground(getColor(symbolicName));
    }

    @Override
    public boolean isEnabled() {
        return text.isEnabled();
    }

    @Override
    public void updateCacheRelevance() {
        Color color = cacheRelevance().getColor();
        text.setBackground(color);
    }

    @Override
    public void setBackground(Color color) {
//        if (label != null) {
//            label.setBackground(color);
//        }
    }

    @Override
    public void setSelected(boolean selected) {
        setBackground(selected ? SELECTED : getPersistentBackground());
    }

    @Override
    public void setFocus() {
        text.setFocus();
    }

    /**
     * getMainControl
     */
    public Control getMainControl() {
        return text;
    }

    /**
     * setTextLimit
     */
    public void setTextLimit(int limit) {
        text.setTextLimit(limit);
    }
}
