/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.ui.openurl;

import org.eclipse.core.runtime.IStatus;
import org.eclipse.jface.wizard.WizardDialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Label;

import eu.etaxonomy.cdm.model.reference.Reference;
import eu.etaxonomy.taxeditor.model.MessagingUtils;
import eu.etaxonomy.taxeditor.ui.element.AbstractCdmFormElement;
import eu.etaxonomy.taxeditor.ui.element.CdmFormFactory;
import eu.etaxonomy.taxeditor.ui.element.ICdmFormElement;
import eu.etaxonomy.taxeditor.ui.element.LayoutConstants;

/**
 * @author n.hoffmann
 * @created Jan 31, 2011
 */
public class OpenUrlSelectorElement extends AbstractCdmFormElement implements SelectionListener{

	private Button button;
	
	private Reference reference;
	
	private String referenceDetail;

	private IOpenUrlEnabled openUrlEnabled;
	
	/**
	 * @param formFactory
	 * @param parentElement
	 * @param labelString
	 * @param initialUri
	 * @param textHeight
	 * @param style
	 */
	public OpenUrlSelectorElement(CdmFormFactory formFactory,
			ICdmFormElement parentElement, String labelString, IOpenUrlEnabled openUrlEnabled, int style) {
		super(formFactory, parentElement);
		this.openUrlEnabled = openUrlEnabled;
		
		Label label = formFactory.createLabel(getLayoutComposite(), null);
		addControl(label);
		
		button = formFactory.createButton(getLayoutComposite(), labelString, SWT.PUSH);
		button.setLayoutData(LayoutConstants.RIGHT());
		addControl(button);
		button.addSelectionListener(this);
	}

	@Override
	public void widgetSelected(SelectionEvent e) {
		if(openUrlEnabled.getReference() == null){
			MessagingUtils.warningDialog("No reference", getClass(), "The given reference is empty");
			return;
		}
		
		OpenUrlSelectorWizard wizard = new OpenUrlSelectorWizard(openUrlEnabled.getReference(), openUrlEnabled.getDetail());
		
		WizardDialog dialog = new WizardDialog(getLayoutComposite().getShell(), wizard);
		
		if(dialog.open() == IStatus.OK){
			openUrlEnabled.setOpenUrl(wizard.getOpenUrlReference().getUri());
		}
	}

	public String getDetail() {
		return referenceDetail;
	}
	
	public void setDetail(String referenceDetail){
		this.referenceDetail = referenceDetail;
	}

	@Override
	public void widgetDefaultSelected(SelectionEvent e) {}

	public void setReference(Reference reference) {
		this.reference = reference;
	}

	public Reference getReference() {
		return reference;
	}
}