/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.ui.openurl;

import java.util.List;

import org.eclipse.jface.wizard.Wizard;
import org.eclipse.swt.widgets.Display;

import eu.etaxonomy.cdm.common.URI;
import eu.etaxonomy.cdm.ext.openurl.MobotOpenUrlQuery;
import eu.etaxonomy.cdm.ext.openurl.MobotOpenUrlServiceWrapper;
import eu.etaxonomy.cdm.ext.openurl.MobotOpenUrlServiceWrapper.ReferenceType;
import eu.etaxonomy.cdm.ext.openurl.OpenUrlReference;
import eu.etaxonomy.cdm.model.metadata.PreferencePredicate;
import eu.etaxonomy.cdm.model.reference.Reference;
import eu.etaxonomy.taxeditor.preference.PreferencesUtil;
import eu.etaxonomy.taxeditor.ui.element.CdmFormFactory;

/**
 * @author n.hoffmann
 * @created Jan 31, 2011
 */
public class OpenUrlSelectorWizard extends Wizard {

	private final MobotOpenUrlServiceWrapper openUrlServiceWrapper;

	private final MobotOpenUrlQuery query;

	private OpenUrlReference openUrlReference;

	private final CdmFormFactory formFactory;

	private URI openUrlReferenceImageUri;

	private final Reference reference;

	private final String referenceDetail;

	/**
	 * @param openUrlSelectorElement
	 * @param reference
	 */
	public OpenUrlSelectorWizard(Reference reference, String referenceDetail) {
		if(reference == null){
			throw new IllegalArgumentException("Reference may not be empty");
		}

		this.reference = reference;
		this.referenceDetail = referenceDetail;

		setWindowTitle("Mobot Open Url");
		setNeedsProgressMonitor(true);

		formFactory = new CdmFormFactory(Display.getDefault());

		openUrlServiceWrapper = new MobotOpenUrlServiceWrapper();
		openUrlServiceWrapper.setBaseUrl(PreferencesUtil.getStringValue(PreferencePredicate.MobotOpenUrlServiceAccessPoint.getKey()));

		query  = new MobotOpenUrlQuery();

		query.refType = ReferenceType.getReferenceType(reference);
		if(reference.getAuthorship() != null){
			query.authorName = reference.getAuthorship().getTitleCache();
		}
		query.abbreviation = reference.getTitle();
		if(reference.getInReference() != null){
			query.journalTitle = reference.getInReference().getTitle();
		}
		query.ISBN = reference.getIsbn();
		query.ISSN = reference.getIssn();
		if(reference.getDatePublished() != null){
			query.publicationDate = reference.getDatePublished().toString();
		}
		query.publicationPlace = reference.getPlacePublished();
		query.publisherName = reference.getPublisher();
		query.volume = reference.getVolume();

		query.startPage = referenceDetail;
	}

	@Override
	public void addPages() {

		addPage(new OpenUrlResultPage());
		addPage(new OpenUrlReferencePage());

		super.addPages();
	}

	@Override
	public boolean performFinish() {
		return true;
	}

	public void setOpenUrlReference(OpenUrlReference openUrlReference) {
		this.openUrlReference = openUrlReference;
		this.openUrlReferenceImageUri = openUrlReference.getJpegImage(PreferencesUtil.getIntValue(PreferencePredicate.MobotOpenUrlServiceMaxWidth.getKey(), true)
				, PreferencesUtil.getIntValue(PreferencePredicate.MobotOpenUrlServiceMaxHeight.getKey(), true));
	}
	public OpenUrlReference getOpenUrlReference() {
		return openUrlReference;
	}

	public List<OpenUrlReference> getResult() {
		return openUrlServiceWrapper.doResolve(query);
	}

	public CdmFormFactory getFormFactory() {
		return formFactory;
	}

	public MobotOpenUrlServiceWrapper getOpenUrlServiceWrapper() {
		return openUrlServiceWrapper;
	}

	public URI getOpenUrlReferenceImageUri() {
		return openUrlReferenceImageUri;
	}

	public Reference getReference() {
		return reference;
	}

	public String getReferenceDetail() {
		return referenceDetail;
	}
}