/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.ui.password;

import org.eclipse.jface.wizard.Wizard;
import org.eclipse.swt.widgets.Display;
import org.springframework.security.access.AccessDeniedException;
import org.springframework.security.core.AuthenticationException;

import eu.etaxonomy.cdm.api.conversation.ConversationHolder;
import eu.etaxonomy.cdm.api.conversation.IConversationEnabled;
import eu.etaxonomy.cdm.api.service.IUserService;
import eu.etaxonomy.cdm.model.permission.User;
import eu.etaxonomy.cdm.persistence.hibernate.CdmDataChangeMap;
import eu.etaxonomy.taxeditor.l10n.Messages;
import eu.etaxonomy.taxeditor.model.MessagingUtils;
import eu.etaxonomy.taxeditor.store.CdmStore;
import eu.etaxonomy.taxeditor.ui.element.CdmFormFactory;

/**
 * @author n.hoffmann
 * @created Mar 10, 2011
 * @version 1.0
 */
public class PasswordWizard extends Wizard implements IConversationEnabled{

	private PasswordWizardPage passwordPage;
	private final User user;
	private final ConversationHolder conversation;

	/**
	 * @param user the user to change the password for
	 * @param conversation this optional parameter can be null. Only supply conversation if you
	 * need to run this wizard is a nested conversation
	 */
	public PasswordWizard(User user, ConversationHolder conversation){
		this.conversation = conversation;
		this.user = user;
		setWindowTitle(Messages.PasswordWizardPage_CHANGE_PASSWORD);
	}

	@Override
	public void addPages() {
		CdmFormFactory formFactory = new CdmFormFactory(Display.getDefault());

		passwordPage = new PasswordWizardPage(formFactory, getConversationHolder(), user);
		addPage(passwordPage);
	}

	@Override
	public boolean performFinish() {

		ConversationHolder internalConversation = CdmStore.getCurrentApplicationConfiguration().NewConversation();
	    internalConversation.bind();
	    internalConversation.startTransaction();
		String warningTitle = Messages.PasswordWizard_COULD_NOT_CHANGE_PWD;
        String warningMessage = Messages.PasswordWizard_OLD_PWD_INCORRECT;
        try{
			if(passwordPage.isChangingOwnPassword()){
			    // change own password with validating of old one
				CdmStore.getService(IUserService.class).changePassword(passwordPage.getOldPassword(), passwordPage.getNewPassword());
			}else{
			    // change others passwords
			    CdmStore.getService(IUserService.class).changePasswordForUser(user.getUsername(), passwordPage.getNewPassword());
			}
			CdmStore.getService(IUserService.class).loadWithUpdate(user.getUuid());
			if (user.equals(CdmStore.getLoginManager().getAuthenticatedUser())){
				CdmStore.getLoginManager().authenticate(user.getUsername(), passwordPage.getNewPassword());
			}
		    internalConversation.commit(false);
		    internalConversation.unbind();
		    internalConversation.close();
			return true;
		}catch(AccessDeniedException e){
			MessagingUtils.warningDialog(warningTitle, this, warningMessage);
			return false;
		}catch (AuthenticationException e){
			MessagingUtils.warningDialog(warningTitle, this, warningMessage);

			return false;
		}catch(Exception e){
			MessagingUtils.warningDialog(Messages.PasswordWizard_PROBLEM_WITH_CHANGING_PWD, this, Messages.PasswordWizard_PWD_COULD_NOT_BE_CHANGED + e.getMessage());
			return false;
		}finally{
		    if(conversation != null){
		        conversation.bind();
		    }
		}
	}

	@Override
	public void update(CdmDataChangeMap arg0) {
	}

	@Override
	public ConversationHolder getConversationHolder() {
		return conversation;
	}

}
