/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.ui.password;

import org.eclipse.jface.dialogs.IInputValidator;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Text;

import eu.etaxonomy.cdm.api.conversation.ConversationHolder;
import eu.etaxonomy.cdm.model.permission.User;
import eu.etaxonomy.taxeditor.l10n.Messages;
import eu.etaxonomy.taxeditor.store.CdmStore;
import eu.etaxonomy.taxeditor.ui.element.AbstractCdmEntityWizardPage;
import eu.etaxonomy.taxeditor.ui.element.CdmFormFactory;
import eu.etaxonomy.taxeditor.ui.element.ICdmFormElement;
import eu.etaxonomy.taxeditor.ui.element.LayoutConstants;
import eu.etaxonomy.taxeditor.ui.element.TextWithLabelElement;
import eu.etaxonomy.taxeditor.ui.section.AbstractCdmDetailElement;

/**
 * @author n.hoffmann
 * @created 02.07.2009
 * @version 1.0
 */
public class PasswordWizardPage extends AbstractCdmEntityWizardPage<User> implements ModifyListener{

	private TextWithLabelElement text_oldPassword;
	private TextWithLabelElement text_password;
	private TextWithLabelElement text_passwordRepeat;
	private final PasswordValidator passwordValidator;

	protected PasswordWizardPage(CdmFormFactory formFactory,
			ConversationHolder conversation, User entity) {
		super(formFactory, conversation, entity);

		passwordValidator = new PasswordValidator();

		setTitle(Messages.PasswordWizardPage_CHANGE_PASSWORD);
		setDescription(Messages.PasswordWizardPage_CHANGE_PASSWORD_AND_CONFIRM);
	}

	@Override
	public void createControl(Composite parent) {
		Composite control = formFactory.createComposite(parent);
		control.setLayoutData(LayoutConstants.FILL());

		setPageComplete(false);

		control.setLayout(LayoutConstants.LAYOUT(2, false));
		WizardPageRootElement rootElement = new WizardPageRootElement(formFactory, control, getConversationHolder());

		if(isChangingOwnPassword()) {
		    text_oldPassword = formFactory.createTextWithLabelElement(rootElement, Messages.PasswordWizardPage_OLD_PASSWORD, null, SWT.PASSWORD);
		}
		text_password =  formFactory.createTextWithLabelElement(rootElement, Messages.PasswordWizardPage_NEW_PASSWORD, null, SWT.PASSWORD);
		text_passwordRepeat =  formFactory.createTextWithLabelElement(rootElement, Messages.PasswordWizardPage_REPEAT_PASSWORD, null, SWT.PASSWORD);

		((Text)text_passwordRepeat.getMainControl()).addModifyListener(this);

		Color bgColor = getShell().getBackground();
		rootElement.setBackground(bgColor);
		control.setBackground(bgColor);
		setControl(control);
	}

    protected boolean isChangingOwnPassword() {
        return getEntity() != null && getEntity().getUsername().equals(CdmStore.getCurrentAuthentiation().getName());
    }


	@Override
	public AbstractCdmDetailElement<User> createElement(
			ICdmFormElement rootElement) {
		// not used
		return null;
	}

	/** {@inheritDoc} */
	@Override
    public void modifyText(ModifyEvent e) {
		validate();
	}

	private void validate(){
		String message;

		if((message = passwordValidator.passwordsMatch(text_password.getText(), text_passwordRepeat.getText())) != null){
			// pass
		}
		else if((message = passwordValidator.isValid(text_password.getText())) != null){
			// pass
		}

		setErrorMessage(message);
	}

	private class PasswordValidator implements IInputValidator{

		private final int PW_MIN_LENGTH = 5;

		private final String TO_SHORT = String.format(Messages.PasswordWizardPage_PASSWORD_MIN_CHARACTER, PW_MIN_LENGTH);
		private final String NO_MATCH = Messages.PasswordWizardPage_PASSWORDS_DO_NOT_MATCH;

		@Override
        public String isValid(String newText) {
			if(newText.length() < PW_MIN_LENGTH){
				setPageComplete(false);
				return TO_SHORT;
			}

			setPageComplete(true);
			return null;
		}

		public String passwordsMatch(String password1, String password2){

			if(! password1.equals(password2)){
				setPageComplete(false);
				return NO_MATCH;
			}

			setPageComplete(true);
			return null;
		}

	}

	public String getOldPassword() {
		return text_oldPassword.getText();
	}

	public String getNewPassword() {
		return text_password.getText();
	}
}
