/**
 * Copyright (C) 2007 EDIT
 * European Distributed Institute of Taxonomy
 * http://www.e-taxonomy.eu
 *
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */
package eu.etaxonomy.taxeditor.ui.section.agent;

import org.apache.commons.lang.StringUtils;
import org.eclipse.swt.SWT;

import eu.etaxonomy.cdm.model.agent.Person;
import eu.etaxonomy.taxeditor.ui.element.CdmFormFactory;
import eu.etaxonomy.taxeditor.ui.element.CdmPropertyChangeEvent;
import eu.etaxonomy.taxeditor.ui.element.ICdmFormElement;
import eu.etaxonomy.taxeditor.ui.element.LsidWithExceptionLabelElement;
import eu.etaxonomy.taxeditor.ui.element.OrcidWithLabelElement;
import eu.etaxonomy.taxeditor.ui.element.TextWithLabelElement;
import eu.etaxonomy.taxeditor.ui.element.TimePeriodElement;
import eu.etaxonomy.taxeditor.ui.section.AbstractIdentifiableEntityDetailElement;

/**
 * PersonDetailElement class.
 *
 * @author n.hoffmann
 * @created Mar 8, 2010
 */
public class PersonDetailElement extends AbstractIdentifiableEntityDetailElement<Person> {

	private TextWithLabelElement text_nomenclaturalTitle;
//	private TextWithLabelElement text_nomenclaturalTitleCache;
	private TextWithLabelElement text_collectorTitle;
//	private TextWithLabelElement text_collectorTitleCache;
	private TextWithLabelElement text_firstname;
	private TextWithLabelElement text_lastname;
	private TextWithLabelElement text_prefix;
	private TextWithLabelElement text_suffix;
	private TextWithLabelElement text_initials;
	private TimePeriodElement time_lifespan;
	private OrcidWithLabelElement text_orcid;
    private LsidWithExceptionLabelElement text_lsid;

	public PersonDetailElement(CdmFormFactory cdmFormFactory,
			ICdmFormElement formElement, int style) {
		super(cdmFormFactory, formElement);
	}

	@Override
	protected void createControls(ICdmFormElement formElement, Person entity,
			int style) {
		setWarnForReferencingObjects(formElement);
		toggleable_cache = formFactory.createToggleableTextField(formElement,
				"Title Cache", entity.getTitleCache(),
				entity.isProtectedTitleCache(), SWT.NULL);



		text_nomenclaturalTitle = formFactory.createTextWithLabelElement(
                formElement, "Abbrev. Title", entity.getNomenclaturalTitle(),
                SWT.SEARCH);

		text_nomenclaturalTitle.setMessage(entity.getNomenclaturalTitleCache());


		text_collectorTitle = formFactory.createTextWithLabelElement(
                formElement, "Collector Title", entity.getCollectorTitle(),
                SWT.SEARCH);

		text_collectorTitle.setMessage(entity.getCollectorTitleCache());

		text_lastname = formFactory.createTextWithLabelElement(formElement,
                "Familiy Name", entity.getFamilyName(), SWT.NULL);
		text_firstname = formFactory.createTextWithLabelElement(formElement,
				"Other/Given Names", entity.getGivenName(), SWT.NULL);
		text_initials = formFactory.createTextWithLabelElement(formElement,
				"Initials", entity.getInitials(), SWT.NULL);
		text_prefix = formFactory.createTextWithLabelElement(formElement,
				"Prefix", entity.getPrefix(), SWT.NULL);
		time_lifespan = formFactory.createTimePeriodElement(formElement, "Lifespan", entity.getLifespan(), SWT.NULL);
		text_orcid = formFactory.createOrcidWithLabelElement(formElement, "ORCID", entity.getOrcid(), SWT.NULL);
		text_lsid = formFactory.createLsidWithExceptionLabelElement(formElement, "LSID", entity.getLsid(), SWT.NULL);

		getLayoutComposite().layout();

	    //cache relevance
        registerCacheRelevance(text_firstname);
        registerCacheRelevance(text_lastname);
        registerCacheRelevance(text_prefix);
        registerCacheRelevance(text_suffix);
        registerCacheRelevance(text_initials);



	}

	@Override
	protected void updateContent() {
		super.updateContent();
		toggleable_cache.setCacheEnabled(getEntity().isProtectedTitleCache());
		updateCacheRelevance();
	}

	@Override
	public void handleEvent(Object eventSource) {
		if (eventSource == toggleable_cache) {
			handleToggleableCacheField();
		} else if (eventSource == text_nomenclaturalTitle) {
		    if (StringUtils.isBlank(text_nomenclaturalTitle.getText()) && getEntity().getNomenclaturalTitle() != null){
                getEntity().setNomenclaturalTitle(null);

            }else if (StringUtils.isNotBlank(text_nomenclaturalTitle.getText()) ){
                getEntity().setNomenclaturalTitle(text_nomenclaturalTitle.getText());

            }
		} else if (eventSource == text_collectorTitle) {
		    if (StringUtils.isBlank(text_collectorTitle.getText()) && getEntity().getCollectorTitle() != null){
		        getEntity().setCollectorTitle(null);

		    }else if (StringUtils.isNotBlank(text_collectorTitle.getText()) ){
		        getEntity().setCollectorTitle(text_collectorTitle.getText());

		    }
        }else if (eventSource == text_firstname) {
			getEntity().setGivenName(text_firstname.getText());
		}else if (eventSource == text_initials) {
			getEntity().setInitials(text_initials.getText());
		}else if (eventSource == text_lastname) {
			getEntity().setFamilyName(text_lastname.getText());
		} else if (eventSource == text_prefix) {
			getEntity().setPrefix(text_prefix.getText());
		} else if (eventSource == text_suffix) {
			getEntity().setSuffix(text_suffix.getText());
		}else if (eventSource == text_orcid) {
            getEntity().setOrcid(text_orcid.parseText());
        }else if (eventSource == text_lsid) {
            getEntity().setLsid(text_lsid.parseText());
        }else if (eventSource == time_lifespan) {
            getEntity().setLifespan(time_lifespan.getTimePeriod());
        }
		text_nomenclaturalTitle.setMessage(getEntity().getNomenclaturalTitleCache());
		text_collectorTitle.setMessage(getEntity().getCollectorTitleCache());

		if (!getEntity().isProtectedTitleCache()){
			toggleable_cache.setText(getEntity().generateTitle());
		}

		firePropertyChangeEvent(new CdmPropertyChangeEvent(this, null));

	}

	@Override
	protected void handleToggleableCacheField() {
        boolean pushedState = toggleable_cache.getState();
        getEntity().setProtectedTitleCache(pushedState);
        getEntity().getTitleCache();
        if (pushedState){
            getEntity().setTitleCache(toggleable_cache.getText(), pushedState);
        }
        updateCacheRelevance();
        updateToggleableCacheField();
    }
}