/**
 * Copyright (C) 2007 EDIT
 * European Distributed Institute of Taxonomy
 * http://www.e-taxonomy.eu
 *
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */
package eu.etaxonomy.taxeditor.ui.section.agent;

import org.eclipse.swt.events.SelectionListener;

import eu.etaxonomy.cdm.model.agent.Person;
import eu.etaxonomy.cdm.model.agent.Team;
import eu.etaxonomy.taxeditor.ui.element.AbstractFormSection;
import eu.etaxonomy.taxeditor.ui.element.CdmFormFactory;
import eu.etaxonomy.taxeditor.ui.element.CdmPropertyChangeEvent;
import eu.etaxonomy.taxeditor.ui.element.ICdmFormElement;
import eu.etaxonomy.taxeditor.ui.section.AbstractEntityCollectionElement;
import eu.etaxonomy.taxeditor.ui.selection.EntitySelectionElement;

/**
 * @author n.hoffmann
 * @created Apr 30, 2010
 */
public class TeamMemberElement extends AbstractEntityCollectionElement<Person> {

	protected EntitySelectionElement<Person> selection_person;

	public TeamMemberElement(CdmFormFactory cdmFormFactory,
			AbstractFormSection section, Person entity,
			SelectionListener removeListener, int style) {
		super(cdmFormFactory, section, entity, removeListener, null, style);
	}

	@Override
	public void createControls(ICdmFormElement element, int style) {
		selection_person = formFactory
				.createSelectionElement(Person.class,
				        this, "Person", null,
						EntitySelectionElement.EDITABLE | EntitySelectionElement.SELECTABLE,
						style);
		if (entity != null){
			setEntity(entity);
		}
	}

	@Override
	public void setEntity(Person entity) {
		this.entity = entity;
		if (selection_person != null){
			selection_person.setEntity(entity);
		}
	}

    @Override
    public void handleEvent(Object eventSource) {
        if (eventSource == selection_person) {
            if (getParentElement() instanceof TeamMemberSection) {
                TeamMemberSection teamMemberSection = (TeamMemberSection) getParentElement();
                Team team = teamMemberSection.getEntity();

                int index = team.getTeamMembers().indexOf(getEntity());

                if(index==-1){
                    team.addTeamMember(selection_person.getEntity());
                }
                else{
                    //first remove the edited team member and then add it again
                    //because the model allows the same person multiple time in one team
                    team.removeTeamMember(getEntity());
                    team.addTeamMember(selection_person.getEntity(), index);
                }
                firePropertyChangeEvent(new CdmPropertyChangeEvent(teamMemberSection, eventSource));
            }
            entity = selection_person.getEntity();
        }
    }
}