/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.ui.section.agent;

import java.util.Collection;
import java.util.Comparator;
import java.util.List;

import org.eclipse.swt.graphics.Color;

import eu.etaxonomy.cdm.api.conversation.ConversationHolder;
import eu.etaxonomy.cdm.model.agent.Person;
import eu.etaxonomy.cdm.model.agent.Team;
import eu.etaxonomy.taxeditor.ui.dialog.selection.CollectorSelectionDialog;
import eu.etaxonomy.taxeditor.ui.dialog.selection.NomenclaturalPersonAuthorSelectionDialog;
import eu.etaxonomy.taxeditor.ui.dialog.selection.PersonSelectionDialog;
import eu.etaxonomy.taxeditor.ui.element.CacheRelevance;
import eu.etaxonomy.taxeditor.ui.element.CacheRelevanceHelper;
import eu.etaxonomy.taxeditor.ui.element.CdmFormFactory;
import eu.etaxonomy.taxeditor.ui.element.ICacheRelevantFormElement;
import eu.etaxonomy.taxeditor.ui.element.ICdmFormElement;
import eu.etaxonomy.taxeditor.ui.element.ToggleableTextElement;
import eu.etaxonomy.taxeditor.ui.section.AbstractEntityCollectionSection;

/**
 * TeamMemberSection class.
 *
 * @author n.hoffmann
 * @created Apr 30, 2010
 */
public class TeamMemberSection
        extends AbstractEntityCollectionSection<Team, Person>
        implements ICacheRelevantFormElement {

    private boolean isNomenclaturalTeam = false;
    private boolean isCollectorTeam = false;

    private CacheRelevanceHelper cacheRelevanceHelper = new CacheRelevanceHelper();

	/**
	 * <p>Constructor for TeamMemberSection.</p>
	 *
	 * @param cdmFormFactory a {@link eu.etaxonomy.taxeditor.ui.element.CdmFormFactory} object.
	 * @param conversation a {@link eu.etaxonomy.cdm.api.conversation.ConversationHolder} object.
	 * @param parentElement a {@link eu.etaxonomy.taxeditor.ui.element.ICdmFormElement} object.
	 * @param style a int.
	 */
	public TeamMemberSection(CdmFormFactory cdmFormFactory, ConversationHolder conversation, ICdmFormElement parentElement, int style, boolean isNomenclatural, boolean isCollectorTeam) {
		super(cdmFormFactory, conversation, parentElement, null, style);
		isNomenclaturalTeam = isNomenclatural;
		this.isCollectorTeam = isCollectorTeam;
	}

	@Override
	public String getTitleString() {
		return "Team Members";
	}

	@Override
	public void addElement(Person element) {
		getEntity().addTeamMember(element);
	}

	@Override
	public Person createNewElement()  {
	    Person selection = null;
	    if (isNomenclaturalTeam){
	        selection = NomenclaturalPersonAuthorSelectionDialog.select(getShell(), null, true);
	    }else if(isCollectorTeam){
	    	selection = CollectorSelectionDialog.select(getShell(), null, true);
		}else{
	        selection = PersonSelectionDialog.select(getShell(), null);
	    }

	    return selection;
	}

	@Override
	public String getEmptyString() {
		return "No persons yet.";
	}

	@Override
	protected String getTooltipString() {
		return "Add a member to this team";
	}

	@Override
	public void removeElement(Person element) {
		getEntity().removeTeamMember(element);
	}



    @Override
    public Comparator<Person> getComparator() {
        return (p1, p2) -> {
            if (p1 == p2){
                return 0;
            }else if(p1==null){
                return -1;
            }else if(p2==null){
                return 1;
            }

            List<Person> teamMembers = getEntity().getTeamMembers();
            int indexOfP1 = teamMembers.indexOf(p1);
            int indexOfP2 = teamMembers.indexOf(p2);
            if(indexOfP1==-1){
                return 1;
            }
            if(indexOfP2==-1){
                return -1;
            }
            return indexOfP1 - indexOfP2;
        };
    }

    public boolean isNomenclaturalTeam() {
       return isNomenclaturalTeam;
    }

    @Override
    public Person addExisting() {
        return null;
    }

    @Override
    public boolean allowAddExisting() {
        return false;
    }

    @Override
    public void updateCacheRelevance() {
        Color color = cacheRelevance().getColor();
        setBackground(color);
    }

    @Override
    public void addDependsOnCache(ToggleableTextElement toggleElement) {
        cacheRelevanceHelper.addDependsOnCache(toggleElement);
    }

    @Override
    public CacheRelevance cacheRelevance() {
        return cacheRelevanceHelper.cacheRelevance();
    }

    @Override
    public Collection<Person> getCollection(Team entity) {
        return entity.getTeamMembers();

    }

}