/**
* Copyright (C) 2018 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.ui.section.common;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionListener;

import eu.etaxonomy.cdm.model.media.ExternalLink;
import eu.etaxonomy.taxeditor.ui.element.AbstractFormSection;
import eu.etaxonomy.taxeditor.ui.element.CdmFormFactory;
import eu.etaxonomy.taxeditor.ui.element.ICdmFormElement;
import eu.etaxonomy.taxeditor.ui.element.ISelectableElement;
import eu.etaxonomy.taxeditor.ui.element.MultilanguageTextElement;
import eu.etaxonomy.taxeditor.ui.element.SelectionArbitrator;
import eu.etaxonomy.taxeditor.ui.element.UriWithLabelElement;
import eu.etaxonomy.taxeditor.ui.section.AbstractEntityCollectionElement;

/**
 * @author k.luther
 * @since 23.08.2018
 *
 */
public class ExternalLinksElement extends AbstractEntityCollectionElement<ExternalLink> implements ISelectableElement{

    private SelectionArbitrator selectionArbitrator;

    private UriWithLabelElement externalLinkUriText;
    private MultilanguageTextElement description;
    private boolean isShowDescription = true;



    /**
     * @param formFactory
     * @param section
     * @param entity
     * @param removeListener
     * @param style
     */
    public ExternalLinksElement(CdmFormFactory formFactory, AbstractFormSection section, ExternalLink entity,
            SelectionListener removeListener, int style) {
        super(formFactory, section, entity, removeListener, null, style);
        if(formFactory.getSelectionProvider() != null){
            selectionArbitrator = formFactory.createSelectionArbitrator(this);
        }

    }

    /**
     * @param formFactory
     * @param section
     * @param entity
     * @param removeListener
     * @param style
     */
    public ExternalLinksElement(CdmFormFactory formFactory, AbstractFormSection section, ExternalLink entity, boolean isWithTypeAndDesc,
            SelectionListener removeListener, int style) {
        super(formFactory, section, entity, removeListener, null, style);
        if(formFactory.getSelectionProvider() != null){
            selectionArbitrator = formFactory.createSelectionArbitrator(this);
        }
        this.isShowDescription = isWithTypeAndDesc;
        setEntity(entity);


    }

    @Override
    public SelectionArbitrator getSelectionArbitrator() {

        return selectionArbitrator;
    }

    @Override
    public void setEntity(ExternalLink entity) {
    	this.entity = entity;
    	if (externalLinkUriText != null){
	        if (entity.getUri() != null){
	            externalLinkUriText.setText(entity.getUri().toString());
	        }
	
	        if (entity.getDescription() != null && isShowDescription) {
	            if (description == null){
	                description = formFactory.createMultiLanguageTextElement(this, "Description", null, 50, SWT.NULL);
	            }
	            description.setMultilanguageText(entity.getDescription());
	        }
    	}
    }

    @Override
    public void createControls(ICdmFormElement element, int style) {
        externalLinkUriText = formFactory.createUriWithLabelElement(element, "URI", null,  style);
        if(isShowDescription){
            description = formFactory.createMultiLanguageTextElement(element, "Description", null, 50, style);
        }
    }

    @Override
    public void handleEvent(Object eventSource) {
        if (eventSource.equals(externalLinkUriText)){
            getEntity().setUri(externalLinkUriText.parseText());
        }else if (eventSource.equals(description)){
            description.getMultilanguageText().values()
            .forEach(languageString -> getEntity().putDescription(languageString));

        }

    }



}
