/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.ui.section.description;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Comparator;
import java.util.List;

import eu.etaxonomy.cdm.api.conversation.ConversationHolder;
import eu.etaxonomy.cdm.model.description.TaxonDescription;
import eu.etaxonomy.cdm.model.occurrence.DerivedUnit;
import eu.etaxonomy.cdm.model.occurrence.SpecimenOrObservationBase;
import eu.etaxonomy.taxeditor.ui.dialog.selection.DerivedUnitSelectionDialog;
import eu.etaxonomy.taxeditor.ui.element.CdmFormFactory;
import eu.etaxonomy.taxeditor.ui.element.ICdmFormElement;
import eu.etaxonomy.taxeditor.ui.section.AbstractEntityCollectionSection;
import eu.etaxonomy.taxeditor.ui.section.DefaultCdmBaseComparator;

/**
 * <p>DescribedSpecimenSection class.</p>
 *
 * @author n.hoffmann
 * @created Sep 16, 2010
 * @version 1.0
 */
public class DescribedSpecimenSection extends AbstractEntityCollectionSection<TaxonDescription, SpecimenOrObservationBase> {

	/**
	 * <p>Constructor for DescribedSpecimenSection.</p>
	 *
	 * @param formFactory a {@link eu.etaxonomy.taxeditor.ui.element.CdmFormFactory} object.
	 * @param conversation a {@link eu.etaxonomy.cdm.api.conversation.ConversationHolder} object.
	 * @param parentElement a {@link eu.etaxonomy.taxeditor.ui.element.ICdmFormElement} object.
	 * @param style a int.
	 */
	public DescribedSpecimenSection(CdmFormFactory formFactory,
			ConversationHolder conversation, ICdmFormElement parentElement,
			int style) {
		super(formFactory, conversation, parentElement, "Described Specimen", style);
	}

	/** {@inheritDoc} */
	@Override
	public Collection<SpecimenOrObservationBase> getCollection(
			TaxonDescription entity) {
		//FIXME:This class is not really an entity collection section, and should be moved
		// to a single entity section, but one with the possibility of launching a wizard
		List<SpecimenOrObservationBase> collection = new ArrayList<SpecimenOrObservationBase>();
		if(entity.getDescribedSpecimenOrObservation() != null) {
			collection.add(entity.getDescribedSpecimenOrObservation());
		}
		return collection;
	}

    @Override
    public Comparator<SpecimenOrObservationBase> getComparator() {
        return new DefaultCdmBaseComparator<>();
    }

	/** {@inheritDoc} */
	@Override
	public SpecimenOrObservationBase createNewElement() {
		DerivedUnit selection = DerivedUnitSelectionDialog.select(getShell(), //getConversationHolder(),
		        null);
		return selection;
	}

	/** {@inheritDoc} */
	@Override
	public void addElement(SpecimenOrObservationBase element) {
		getEntity().setDescribedSpecimenOrObservation(element);
	}

	/** {@inheritDoc} */
	@Override
	public void removeElement(SpecimenOrObservationBase element) {
		getEntity().setDescribedSpecimenOrObservation(null);
	}

	/** {@inheritDoc} */
	@Override
	public String getEmptyString() {
		return "No described specimen yet.";
	}

	/** {@inheritDoc} */
	@Override
	protected String getTooltipString() {
		return "Add a described specimen.";
	}

    /**
     * {@inheritDoc}
     */
    @Override
    public SpecimenOrObservationBase addExisting() {
        return null;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public boolean allowAddExisting() {
        return false;
    }
}
