/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.ui.section.description;

import java.util.Collection;
import java.util.stream.Collectors;

import org.eclipse.jface.util.PropertyChangeEvent;

import eu.etaxonomy.cdm.model.description.DescriptionBase;
import eu.etaxonomy.cdm.model.description.DescriptionType;
import eu.etaxonomy.taxeditor.ui.element.CdmFormFactory;
import eu.etaxonomy.taxeditor.ui.element.ICdmFormElement;
import eu.etaxonomy.taxeditor.ui.element.IEnableableFormElement;
import eu.etaxonomy.taxeditor.ui.element.TextWithLabelElement;
import eu.etaxonomy.taxeditor.ui.section.AbstractIdentifiableEntityDetailElement;

/**
 * <p>DescriptionDetailElement class.</p>
 *
 * @author n.hoffmann
 * @created Mar 5, 2010
 */
public class DescriptionDetailElement
        extends AbstractIdentifiableEntityDetailElement<DescriptionBase<?>> {

//	private EntitySelectionElement<FeatureTree> selection_featureTree;
    private TextWithLabelElement txtDescriptionTypes;

	public DescriptionDetailElement(CdmFormFactory cdmFormFactory,
			ICdmFormElement formElement, int style) {
		super(cdmFormFactory, formElement);

	}

	@Override
	protected void createControls(ICdmFormElement formElement, DescriptionBase entity, int style) {
        toggleable_cache = formFactory.createToggleableTextField(this, "Label",
                entity.getTitleCache(), entity.isProtectedTitleCache(), style);

		// TODO I initially thought that we can handle the entity's descriptive system with a feature tree
		// Since descriptiveSystem is a Set of Features that are present or allowed for the current description,
		// it is similar to a FeatureTree (you can get this information from a feature tree) but not the same.
		// Please correct this once it is clear what descriptive system will be used for in CDM
//		selection_featureTree = formFactory.createSelectionElement(FeatureTree.class, getConversationHolder(), formElement, "Feature Tree", null, EntitySelectionElement.ALL, style);

        Object typesString = entity.getTypes().stream().filter(type->type instanceof DescriptionType)
        		.map(type->((DescriptionType)type).getLabel())
        		.collect(Collectors.joining(","));
        txtDescriptionTypes = formFactory.createTextWithLabelElement(formElement, "Types", typesString, style);
        txtDescriptionTypes.setEnabled(false);

	}

	@Override
    public void setEnabled(boolean enabled, Collection<Object> except) {
        this.enabled = enabled;
        for (ICdmFormElement element : getElements()) {
            if (element instanceof IEnableableFormElement) {
                IEnableableFormElement enableableElement = (IEnableableFormElement) element;
                enableableElement.setEnabled(enabled);
//                if (enableableElement instanceof ToggleableTextElement){
//                    ToggleableTextElement toggle = (ToggleableTextElement)enableableElement;
//                    toggle.getBtnToggleProtectTitleCache().setEnabled(enabled);
//                }

            }
        }
        txtDescriptionTypes.setEnabled(false);
    }



    @Override
    protected void handleTitleCacheRelevantChange(PropertyChangeEvent event) {
        if(!getEntity().isProtectedTitleCache()){
            String title = getEntity().generateTitle();
            getEntity().setTitleCache(title, false);
        }
        super.updateToggleableCacheField();
    }

	@Override
	public void handleEvent(Object eventSource) {
        if (eventSource == toggleable_cache) {
            if (!this.enabled){
                return;
            }
            getEntity().setTitleCache(toggleable_cache.getText(), toggleable_cache.getState());
            updateCacheRelevance();
        }
	}


}