/**
 * Copyright (C) 2007 EDIT
 * European Distributed Institute of Taxonomy
 * http://www.e-taxonomy.eu
 *
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */

package eu.etaxonomy.taxeditor.ui.section.description;

import org.eclipse.swt.events.SelectionListener;

import eu.etaxonomy.cdm.model.reference.Reference;
import eu.etaxonomy.taxeditor.ui.element.AbstractFormSection;
import eu.etaxonomy.taxeditor.ui.element.CdmFormFactory;
import eu.etaxonomy.taxeditor.ui.element.ICdmFormElement;
import eu.etaxonomy.taxeditor.ui.section.AbstractEntityCollectionElement;
import eu.etaxonomy.taxeditor.ui.selection.EntitySelectionElement;

/**
 * <p>
 * DescriptionSourceElement class.
 * </p>
 *
 * @author n.hoffmann
 * @created Mar 18, 2010
 * @version 1.0
 */
public class DescriptionSourceElement extends
		AbstractEntityCollectionElement<Reference> {

	private EntitySelectionElement<Reference> selection_descriptionSource;

	/**
	 * <p>
	 * Constructor for DescriptionSourceElement.
	 * </p>
	 *
	 * @param cdmFormFactory
	 *            a {@link eu.etaxonomy.taxeditor.ui.element.CdmFormFactory}
	 *            object.
	 * @param formElement
	 *            a {@link eu.etaxonomy.taxeditor.ui.element.AbstractFormSection}
	 *            object.
	 * @param element
	 *            a {@link eu.etaxonomy.cdm.model.reference.ReferenceBase}
	 *            object.
	 * @param removeListener
	 *            a {@link org.eclipse.swt.events.SelectionListener} object.
	 * @param style
	 *            a int.
	 */
	public DescriptionSourceElement(CdmFormFactory cdmFormFactory,
			AbstractFormSection formElement, Reference element,
			SelectionListener removeListener, int style) {
		super(cdmFormFactory, formElement, element, removeListener, null, style);
	}

	/** {@inheritDoc} */
	@Override
	public void createControls(ICdmFormElement element, int style) {
		selection_descriptionSource = formFactory
				.createSelectionElement(Reference.class,
				        element, "Source",
						null, EntitySelectionElement.ALL, style);
		if (entity != null){
			setEntity(entity);
		}
	}

	/** {@inheritDoc} */
	@Override
	public void setEntity(Reference element) {
		this.entity = element;
		if (selection_descriptionSource != null){
			selection_descriptionSource.setEntity(element);
		}
	}

	/** {@inheritDoc} */
	@Override
	public void handleEvent(Object eventSource) {

	}
}
