/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.ui.section.description;

import org.eclipse.swt.events.SelectionListener;

import eu.etaxonomy.cdm.model.term.DefinedTerm;
import eu.etaxonomy.cdm.model.term.TermType;
import eu.etaxonomy.cdm.model.description.PolytomousKey;
import eu.etaxonomy.cdm.model.description.TaxonDescription;
import eu.etaxonomy.taxeditor.ui.combo.term.TermComboElement;
import eu.etaxonomy.taxeditor.ui.element.AbstractFormSection;
import eu.etaxonomy.taxeditor.ui.element.CdmFormFactory;
import eu.etaxonomy.taxeditor.ui.element.ICdmFormElement;
import eu.etaxonomy.taxeditor.ui.section.AbstractEntityCollectionElement;
import eu.etaxonomy.taxeditor.ui.section.key.ScopeRestrictionSection;

/**
 * <p>ScopeElement class.</p>
 *
 * @author n.hoffmann
 * @created Mar 18, 2010
 * @version 1.0
 */
public class ScopeElement extends AbstractEntityCollectionElement<DefinedTerm> {

	private TermComboElement<DefinedTerm> combo_scope;

	/**
	 * <p>Constructor for ScopeElement.</p>
	 *
	 * @param cdmFormFactory a {@link eu.etaxonomy.taxeditor.ui.element.CdmFormFactory} object.
	 * @param section a {@link eu.etaxonomy.taxeditor.ui.element.AbstractFormSection} object.
	 * @param element a {@link eu.etaxonomy.cdm.model.description.Scope} object.
	 * @param removeListener a {@link org.eclipse.swt.events.SelectionListener} object.
	 * @param style a int.
	 */
	public ScopeElement(CdmFormFactory cdmFormFactory,
			AbstractFormSection section, DefinedTerm element,
			SelectionListener removeListener, int style) {
		super(cdmFormFactory, section, element, removeListener, null, style);
	}

	/** {@inheritDoc} */
	@Override
	public void setEntity(DefinedTerm entity) {
		this.entity = entity;
		if(entity!=null && entity.getId()!=0 && combo_scope != null){
		    combo_scope.setSelection(entity);
		    combo_scope.removeEmptyElement();
		}
	}

	/** {@inheritDoc} */
	@Override
	public void createControls(ICdmFormElement element, int style) {
		combo_scope = formFactory.createDefinedTermComboElement(TermType.Scope, element, "Scope", null, style);
		if (entity != null){
			setEntity(entity);
		}
	}

	/** {@inheritDoc} */
	@Override
	public void handleEvent(Object eventSource) {
	    DefinedTerm scopeTerm = combo_scope.getSelection();
	    if(eventSource==combo_scope && scopeTerm!=null){
	        if(getParentElement() instanceof ScopeRestrictionSection){
	            PolytomousKey key = ((ScopeRestrictionSection)getParentElement()).getEntity();
	            key.removeScopeRestriction(entity);
	            key.addScopeRestriction(scopeTerm);
                entity = scopeTerm;
	        }
	        else if(getParentElement() instanceof ScopeSection){
	            TaxonDescription description = ((ScopeSection)getParentElement()).getEntity();
	            description.removeScope(entity);
	            description.addScope(scopeTerm);
	            entity = scopeTerm;

	        }
	    }
	}

}
