/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.ui.section.description;

import java.util.ArrayList;
import java.util.List;
import java.util.Set;

import org.eclipse.swt.events.SelectionListener;

import eu.etaxonomy.cdm.model.common.LanguageString;
import eu.etaxonomy.cdm.model.description.State;
import eu.etaxonomy.cdm.model.description.StateData;
import eu.etaxonomy.cdm.model.metadata.PreferencePredicate;
import eu.etaxonomy.cdm.model.term.TermType;
import eu.etaxonomy.cdm.model.term.TermVocabulary;
import eu.etaxonomy.taxeditor.preference.PreferencesUtil;
import eu.etaxonomy.taxeditor.store.CdmStore;
import eu.etaxonomy.taxeditor.store.StoreUtil;
import eu.etaxonomy.taxeditor.ui.combo.term.TermComboElement;
import eu.etaxonomy.taxeditor.ui.element.AbstractFormSection;
import eu.etaxonomy.taxeditor.ui.element.CdmFormFactory;
import eu.etaxonomy.taxeditor.ui.element.ICdmFormElement;
import eu.etaxonomy.taxeditor.ui.element.LayoutConstants;
import eu.etaxonomy.taxeditor.ui.element.TextWithLabelElement;
import eu.etaxonomy.taxeditor.ui.section.AbstractEntityCollectionElement;

/**
 * @author n.hoffmann
 * @created Sep 15, 2010
 */
public class StateDataElement extends AbstractEntityCollectionElement<StateData> {

	private TermComboElement<State> combo_state;
	private ModifierSection section_modifiers;
	private TextWithLabelElement text_modifyingText;

	public StateDataElement(CdmFormFactory formFactory,
			AbstractFormSection section, StateData entity,
			SelectionListener removeListener, int style) {
		super(formFactory, section, entity, removeListener, null, style);
	}

	@Override
	public void setEntity(StateData entity) {
		this.entity = entity;
		if (combo_state != null){
			
			combo_state.setSelection(entity.getState());
			if (section_modifiers != null){
	    		section_modifiers.setEntity(entity);
	    		section_modifiers.setExpanded(!entity.getModifiers().isEmpty());
			}
			if(getEntity().getModifyingText().get(CdmStore.getDefaultLanguage()) != null && text_modifyingText != null) {
	            text_modifyingText.setText(getEntity().getModifyingText().get(CdmStore.getDefaultLanguage()).getText());
	        }
			if(getEntity().getId()>0){
			    combo_state.removeEmptyElement();
			}
		}
	}

	@Override
	public void createControls(ICdmFormElement element, int style) {
		if (getEntity() != null){
			List<State> stateTerms = new ArrayList<>();
			Set<TermVocabulary<State>> stateVocabularies = getEntity().getCategoricalData().getFeature().getSupportedCategoricalEnumerations();
			for (TermVocabulary<State> termVocabulary : stateVocabularies) {
			    stateTerms.addAll(termVocabulary.getTerms());
	        }
			combo_state = formFactory.createDefinedTermComboElement(stateTerms, element, "State", null, style);
		}else{
			combo_state = formFactory.createDefinedTermComboElement(TermType.State, element, "State", null, style);
		}
//		combo_state.setTermComparator(new DefaultTermComparator<State>());
		if (PreferencesUtil.getBooleanValue(PreferencePredicate.ShowModifier.getKey())){
		    section_modifiers = formFactory.createModifierSection(getConversationHolder(), element, StoreUtil.getSectionStyle(ModifierSection.class, StateData.class.getCanonicalName(), true));
		    section_modifiers.setLayoutData(LayoutConstants.FILL_HORIZONTALLY(2, 1));
		}
		if (PreferencesUtil.getBooleanValue(PreferencePredicate.ShowModifierFreeText.getKey())){
		    text_modifyingText = formFactory.createTextWithLabelElement(element, "Modifying Text", null, style);
		}
		if (entity != null){
			setEntity(entity);
		}

	}

	@Override
	public void handleEvent(Object eventSource) {
		if(eventSource == combo_state){
			getEntity().setState(combo_state.getSelection());
			combo_state.removeEmptyElement();
		}
		if(eventSource == text_modifyingText){
			getEntity().putModifyingText(LanguageString.NewInstance(text_modifyingText.getText(), CdmStore.getDefaultLanguage()));
		}
	}

	@Override
    public void setEnabled(boolean enable){
		if (combo_state != null){
			combo_state.setEnabled(enable);
		}
	    if (section_modifiers != null){
	    	section_modifiers.setEnabled(enable);
	    }
	    if (text_modifyingText != null){
	    	text_modifyingText.setEnabled(enable);
	    }
	    super.setEnabled(enable);
	}
}
