/**
 *
 */
package eu.etaxonomy.taxeditor.ui.section.description.detail;

import eu.etaxonomy.cdm.model.description.CategoricalData;
import eu.etaxonomy.taxeditor.store.StoreUtil;
import eu.etaxonomy.taxeditor.ui.element.CdmFormFactory;
import eu.etaxonomy.taxeditor.ui.element.CheckboxElement;
import eu.etaxonomy.taxeditor.ui.element.ICdmFormElement;
import eu.etaxonomy.taxeditor.ui.element.LayoutConstants;
import eu.etaxonomy.taxeditor.ui.section.description.StateDataSection;

/**
 * <p>CategoricalDataDetailElement class.</p>
 *
 * @author n.hoffmann
 */
public class CategoricalDataDetailElement extends AbstractDetailedDescriptionDetailElement<CategoricalData> {

	private CheckboxElement checkbox_orderRelevant;
	private StateDataSection section_stateData;

	/**
	 * <p>Constructor for CategoricalDataDetailElement.</p>
	 *
	 * @param formFactory a {@link eu.etaxonomy.taxeditor.ui.element.CdmFormFactory} object.
	 * @param formElement a {@link eu.etaxonomy.taxeditor.ui.element.ICdmFormElement} object.
	 * @param entity a {@link eu.etaxonomy.cdm.model.description.CategoricalData} object.
	 * @param style a int.
	 */
	public CategoricalDataDetailElement(CdmFormFactory formFactory,
			ICdmFormElement formElement, CategoricalData entity, boolean enabled, int style) {
		super(formFactory, formElement, entity, enabled, style);
		
	}

	@Override
	protected void createControls(ICdmFormElement formElement,
			CategoricalData entity, int style) {

		checkbox_orderRelevant = formFactory.createCheckbox(formElement, "Order Relevant", entity.getOrderRelevant(), style);
		section_stateData = formFactory.createStateDataSection(getConversationHolder(), formElement, StoreUtil.getSectionStyle(StateDataSection.class, entity.getClass().getCanonicalName(), true));
		section_stateData.setEntity(entity);
		section_stateData.setLayoutData(LayoutConstants.FILL_HORIZONTALLY(2, 1));
		section_stateData.setEnabled(enabled);
		createTimePeriod(formElement, entity, style);
		if (entity.getFeature().getRecommendedModifierEnumeration() != null && !entity.getFeature().getRecommendedModifierEnumeration().isEmpty()){
		    createModifier(formElement, entity, style);
		}else{
		    createFreetextModifier(formElement, entity, style);
		}
	}

	@Override
	public void handleEvent(Object eventSource) {
	    super.handleEvent(eventSource);
		if(eventSource == checkbox_orderRelevant){
			getEntity().setOrderRelevant(checkbox_orderRelevant.getSelection());
		}
	}
}
