/**
* Copyright (C) 2009 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.ui.section.description.operation;

import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.commands.operations.IUndoContext;
import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.widgets.Display;

import eu.etaxonomy.cdm.model.description.CategoricalData;
import eu.etaxonomy.cdm.model.description.CommonTaxonName;
import eu.etaxonomy.cdm.model.description.DescriptionBase;
import eu.etaxonomy.cdm.model.description.DescriptionElementBase;
import eu.etaxonomy.cdm.model.description.Distribution;
import eu.etaxonomy.cdm.model.description.Feature;
import eu.etaxonomy.cdm.model.description.IndividualsAssociation;
import eu.etaxonomy.cdm.model.description.QuantitativeData;
import eu.etaxonomy.cdm.model.description.TaxonInteraction;
import eu.etaxonomy.cdm.model.description.TextData;
import eu.etaxonomy.taxeditor.operation.AbstractPostTaxonOperation;
import eu.etaxonomy.taxeditor.operation.IPostOperationEnabled;
import eu.etaxonomy.taxeditor.view.e4.details.DetailsPartE4;

/**
 * <p>ChangeDescriptionElementType class.</p>
 *
 * @author e.-m.lee
 * @date 25.05.2010
 */
public class ChangeDescriptionElementType extends AbstractPostTaxonOperation {

	private DescriptionElementBase sourceElement;
	private DescriptionElementBase destinationElement;
	private Viewer viewer;

	/**
	 * <p>Constructor for ChangeDescriptionElementType.</p>
	 *
	 * @param label a {@link java.lang.String} object.
	 * @param undoContext a {@link org.eclipse.core.commands.operations.IUndoContext} object.
	 * @param sourceType a {@link eu.etaxonomy.cdm.model.description.DescriptionElementBase} object.
	 * @param postOperation a {@link eu.etaxonomy.taxeditor.operation.IPostOperationEnabled} object.
	 */
	public ChangeDescriptionElementType(String label, DescriptionElementBase sourceType, IPostOperationEnabled postOperation,
			IUndoContext undoContext) {
		super(label, undoContext, postOperation);
		this.sourceElement = sourceType;
		if(postOperation instanceof DetailsPartE4){
		    this.viewer = ((DetailsPartE4) postOperation).getViewer();
		}
	}

	@Override
	public IStatus execute(IProgressMonitor monitor, IAdaptable info)
			throws ExecutionException {
		DescriptionBase<?> inDescription = sourceElement.getInDescription();
		Feature feature = sourceElement.getFeature();

		if(sourceElement instanceof TextData){
			// FIXME I guess there are features that support more that one of these
			// and that case is not handled correctly at the moment
			if(feature.isSupportsDistribution()){
				destinationElement = Distribution.NewInstance();
			}
			else if(feature.isSupportsCommonTaxonName()){
				destinationElement = CommonTaxonName.NewInstance(null, null);
			}
			else if(feature.isSupportsTaxonInteraction()){
				destinationElement = TaxonInteraction.NewInstance();
			}
			else if(feature.isSupportsIndividualAssociation()){
				destinationElement = IndividualsAssociation.NewInstance();
			}
			else if(feature.isSupportsCategoricalData()){
				destinationElement = CategoricalData.NewInstance();
			}
			else if(feature.isSupportsQuantitativeData()){
				destinationElement = QuantitativeData.NewInstance();
			}
		}
		else{
			if(feature.isSupportsTextData()){
				destinationElement = TextData.NewInstance();
			}
		}
		if(destinationElement==null){
			MessageDialog.openInformation(Display.getCurrent().getActiveShell(), "Change not possible", "Feature does not have another type");
			return Status.CANCEL_STATUS;
		}
		destinationElement.setFeature(feature);
		inDescription.addElement(destinationElement);
		inDescription.removeElement(sourceElement);

        viewer.setInput(destinationElement);
		return postExecute(destinationElement);
	}

	@Override
	public IStatus redo(IProgressMonitor monitor, IAdaptable info)
			throws ExecutionException {
		return null;
	}

	@Override
	public IStatus undo(IProgressMonitor monitor, IAdaptable info)
			throws ExecutionException {
		return null;
	}

}
