/**
* Copyright (C) 2019 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.ui.section.feature;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.UUID;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Color;

import eu.etaxonomy.cdm.api.service.ITermService;
import eu.etaxonomy.cdm.model.description.Feature;
import eu.etaxonomy.cdm.model.description.FeatureState;
import eu.etaxonomy.cdm.model.description.State;
import eu.etaxonomy.cdm.model.term.DefinedTerm;
import eu.etaxonomy.cdm.model.term.DefinedTermBase;
import eu.etaxonomy.cdm.model.term.TermType;
import eu.etaxonomy.cdm.persistence.dto.FeatureStateDto;
import eu.etaxonomy.taxeditor.store.CdmStore;
import eu.etaxonomy.taxeditor.store.TermManager;
import eu.etaxonomy.taxeditor.ui.combo.term.TermComboElement;
import eu.etaxonomy.taxeditor.ui.element.AbstractFormSection;
import eu.etaxonomy.taxeditor.ui.element.CdmFormFactory;
import eu.etaxonomy.taxeditor.ui.element.ICdmFormElement;
import eu.etaxonomy.taxeditor.ui.section.AbstractEntityCollectionElement;
import eu.etaxonomy.taxeditor.ui.selection.EntitySelectionElement;

/**
 * @author pplitzner
 * @since Aug 16, 2019
 *
 */
public abstract class AbstractApplicableElementForDto extends AbstractEntityCollectionElement<FeatureStateDto> {

    protected TermComboElement<State> comboState;
    protected EntitySelectionElement<Feature> selectFeature;
    

    public AbstractApplicableElementForDto(CdmFormFactory formFactory, AbstractFormSection section, FeatureStateDto entity,
            SelectionListener removeListener, Color backgroundColor, int style) {
        super(formFactory, section, entity, removeListener, backgroundColor, style);

    }

    @Override
    public void createControls(ICdmFormElement element, int style) {
        selectFeature = formFactory.createSelectionElement(Feature.class, element, "Feature", null, EntitySelectionElement.DELETABLE, SWT.NONE);
        comboState = formFactory.createDefinedTermComboElement(Collections.EMPTY_LIST, element, getComboLabel(), null, style);
        setFeatureState();
    }

	private void setFeatureState() {
		Feature feature = (Feature)CdmStore.getService(ITermService.class).load(entity.getFeature().getUuid());
		selectFeature.setEntity(feature);
		List<UUID> termList = new ArrayList<>();
		termList.add(entity.getState().getUuid());
		List<State> terms =  CdmStore.getTermManager().getTerms(termList, TermType.State);
		if (terms.size() == 1){
			comboState.setSelection(terms.get(0));
		}
        
	}

    protected abstract String getComboLabel();

    @Override
    public void setEntity(FeatureStateDto entity) {
        this.entity = entity;
        if (selectFeature != null){
        	setFeatureState();
        }
    }

    @Override
    public void handleEvent(Object eventSource) {
    }
    
    @Override
    public void setEnabled(boolean enabled){
    	super.setEnabled(enabled);
    	this.comboState.setEnabled(false);
    	this.selectFeature.setEnabled(false);
    }
}
