/**
* Copyright (C) 2009 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.ui.section.feature;

import org.eclipse.jface.util.PropertyChangeEvent;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;

import eu.etaxonomy.cdm.model.common.Language;
import eu.etaxonomy.cdm.model.description.Character;
import eu.etaxonomy.cdm.model.term.DefinedTerm;
import eu.etaxonomy.cdm.model.term.Representation;
import eu.etaxonomy.cdm.model.term.TermType;
import eu.etaxonomy.cdm.persistence.dto.CharacterDto;
import eu.etaxonomy.cdm.persistence.dto.FeatureDto;
import eu.etaxonomy.cdm.persistence.dto.TermDto;
import eu.etaxonomy.taxeditor.editor.definedterm.TermBasePropertyTester;
import eu.etaxonomy.taxeditor.event.EventUtility;
import eu.etaxonomy.taxeditor.event.WorkbenchEventConstants;
import eu.etaxonomy.taxeditor.model.ColorResources;
import eu.etaxonomy.taxeditor.preference.Resources;
import eu.etaxonomy.taxeditor.store.CdmStore;
import eu.etaxonomy.taxeditor.store.StoreUtil;
import eu.etaxonomy.taxeditor.ui.combo.term.TermComboElement;
import eu.etaxonomy.taxeditor.ui.element.CdmFormFactory;
import eu.etaxonomy.taxeditor.ui.element.CheckboxElement;
import eu.etaxonomy.taxeditor.ui.element.ICdmFormElement;
import eu.etaxonomy.taxeditor.ui.element.LabelElement;
import eu.etaxonomy.taxeditor.ui.element.LayoutConstants;
import eu.etaxonomy.taxeditor.ui.element.RemovableTextElement;
import eu.etaxonomy.taxeditor.ui.element.RepresentationElement;
import eu.etaxonomy.taxeditor.ui.section.AbstractCdmDetailElement;
import eu.etaxonomy.taxeditor.ui.section.vocabulary.MeasurementUnitCollectionSection;
import eu.etaxonomy.taxeditor.ui.section.vocabulary.MeasurementUnitDtoCollectionSection;
import eu.etaxonomy.taxeditor.ui.section.vocabulary.RecommendedModifierVocabulariesCollectionSection;
import eu.etaxonomy.taxeditor.ui.section.vocabulary.RecommendedModifierVocabulariesDtoCollectionSection;
import eu.etaxonomy.taxeditor.ui.section.vocabulary.StateVocabularyCollectionSection;
import eu.etaxonomy.taxeditor.ui.section.vocabulary.StateVocabularyDtoCollectionSection;
import eu.etaxonomy.taxeditor.ui.section.vocabulary.StatisticalMeasureCollectionSection;
import eu.etaxonomy.taxeditor.ui.section.vocabulary.StatisticalMeasureDtoCollectionSection;

/**
 *
 * @author pplitzner
 * @since Dec 21, 2017
 *
 */
public class CharacterDtoDetailElement extends AbstractCdmDetailElement<CharacterDto> {

    private ICdmFormElement parentFormElement;

    protected RepresentationElement element_Representation;

    private TermComboElement<DefinedTerm> comboStructuralModifier;

    private CheckboxElement supportsQuantitativeData;

    private CheckboxElement supportsCategoricalData;
    
    private CheckboxElement availableForTaxon;
    private CheckboxElement availableForTaxonName;
    private CheckboxElement availableForSpecimenOrObservation;

    private StateVocabularyDtoCollectionSection sectionStateVocabularies;

    private MeasurementUnitDtoCollectionSection sectionMeasurementUnits;

    private StatisticalMeasureDtoCollectionSection sectionStatisticalMeasures;

    private RecommendedModifierVocabulariesDtoCollectionSection sectionModifierVocabularies;
    private Label labelText_structure;
    private Text text_structure;

    private Label labelText_property;
    private Text text_property;

    private RemovableTextElement ratioTo;

//    private TextWithLabelElement text_structure;
//    private TextWithLabelElement text_property;


	public CharacterDtoDetailElement(CdmFormFactory formFactory,
			ICdmFormElement formElement) {
		super(formFactory, formElement);
	}

	@Override
	protected void createControls(ICdmFormElement formElement, CharacterDto entity, int style) {
	    this.parentFormElement = formElement;

	    element_Representation = formFactory.createTranslatableRepresentationElementDto(formElement,  entity.getPreferredRepresentation(CdmStore.getDefaultLanguage()),entity, 100, style, true);
	    if (labelText_property != null){
	        labelText_structure.dispose();
	    }
	    labelText_structure = new Label(formElement.getLayoutComposite(), SWT.NONE);
	    labelText_structure.setText("Structure");
	    if (text_structure != null){
	        text_structure.dispose();
	    }
	    text_structure = new Text(formElement.getLayoutComposite(), SWT.WRAP);
        text_structure.setText( entity.getStructure().getPath());
        text_structure.setEditable(false);
        text_structure.setLayoutData(LayoutConstants.FILL());
        text_structure.setForeground(ColorResources.getColor(Resources.BLACK));
        if (labelText_property != null){
            labelText_property.dispose();
        }
        labelText_property =  new Label(formElement.getLayoutComposite(), SWT.NONE);
	    labelText_property.setText("Property");
	    if (text_property != null){
	        text_property.dispose();
	    }
	    text_property = new Text(formElement.getLayoutComposite(), SWT.WRAP);
	    text_property.setEditable(false);
	    text_property.setText(entity.getProperty().getPath());
	    text_property.setLayoutData(LayoutConstants.FILL());
	    text_property.setForeground(ColorResources.getColor(Resources.BLACK));

        comboStructuralModifier = formFactory.createDefinedTermComboElementDto(TermType.StructureModifier, formElement, "Structural Modifier", entity.getStructureModifier(), style);

//        if (labelText_ratioTo != null){
//            labelText_ratioTo.dispose();
//        }
//        labelText_ratioTo =  new Label(formElement.getLayoutComposite(), SWT.NONE);
//        labelText_ratioTo.setText("Ratio to");
//        if (text_ratioTo != null){
//            text_ratioTo.dispose();
//        }
//        text_ratioTo = new Text(formElement.getLayoutComposite(), SWT.WRAP);
//        text_ratioTo.setEditable(false);
//        if (entity.getRatioTo() != null){
//            text_ratioTo.setText(entity.getRatioTo().getPath());
//        }
//        text_ratioTo.setLayoutData(LayoutConstants.FILL());
//        text_ratioTo.setForeground(ColorResources.getColor(Resources.BLACK));

        ratioTo = formFactory.createRemovableTextField(formElement, "Ratio to", entity.getRatioTo() != null? entity.getRatioTo().getPath():null, style);

	    supportsQuantitativeData = formFactory.createCheckbox(formElement, "Supports Quantitative Data", ((FeatureDto) entity).isSupportsQuantitativeData(), style);
	    supportsCategoricalData = formFactory.createCheckbox(formElement, "Supports Categorical Data", ((FeatureDto) entity).isSupportsCategoricalData(), style);
	    LabelElement availableLabel = formFactory.createLabel(formElement, "Available for");
        availableLabel.setBold();
	    availableForTaxon = formFactory.createCheckbox(formElement, "Taxon", entity.isAvailableForTaxon(), style);
        availableForTaxonName = formFactory.createCheckbox(formElement, "Taxon Name", entity.isAvailableForTaxonName(), style);
        availableForSpecimenOrObservation = formFactory.createCheckbox(formElement, "Occurrence", entity.isAvailableForOccurrence(), style);
        if (supportsCategoricalData.getSelection()) {
            sectionStateVocabularies = formFactory.createStateVocabulariesDtoSection(getConversationHolder(),
                    parentFormElement, StoreUtil.getSectionStyle(StateVocabularyCollectionSection.class, entity.getClass().getCanonicalName(), true));
            sectionStateVocabularies.setLayoutData(LayoutConstants.FILL_HORIZONTALLY(2, 1));
            sectionStateVocabularies.setEntity((entity));
        }
        if (supportsQuantitativeData.getSelection()) {
            sectionMeasurementUnits = formFactory.createMeasurementUnitDtoCollectionSection(getConversationHolder(),
                    parentFormElement, StoreUtil.getSectionStyle(MeasurementUnitCollectionSection.class, entity.getClass().getCanonicalName(), true));
            sectionMeasurementUnits.setLayoutData(LayoutConstants.FILL_HORIZONTALLY(2, 1));
            sectionMeasurementUnits.setEntity((entity));

            sectionStatisticalMeasures = formFactory.createStatisticalMeasureDtoCollectionSection(getConversationHolder(),
                    parentFormElement, StoreUtil.getSectionStyle(StatisticalMeasureCollectionSection.class, entity.getClass().getCanonicalName(), true));
            sectionStatisticalMeasures.setLayoutData(LayoutConstants.FILL_HORIZONTALLY(2, 1));
            sectionStatisticalMeasures.setEntity((entity));
        }
        if (supportsCategoricalData.getSelection() || supportsQuantitativeData.getSelection()) {
            sectionModifierVocabularies = formFactory.createRecommendedModifierVocabulariesDtoCollectionSection(
                    getConversationHolder(), parentFormElement, StoreUtil.getSectionStyle(RecommendedModifierVocabulariesCollectionSection.class, entity.getClass().getCanonicalName(), true));
            sectionModifierVocabularies.setLayoutData(LayoutConstants.FILL_HORIZONTALLY(2, 1));
            sectionModifierVocabularies.setEntity((entity));
        }

    }

	public String getPropertyText() {
        return getEntity().getProperty()!=null?getEntity().getProperty().getPath():"[no property]";
    }

    public String getStructureText(Character entity) {
        return entity.getStructure()!=null?entity.getStructure().getPath():"[no structure]";
    }
//	public void createText(ICdmFormElement formElement, String label, String path) {
//	    labelText = new Label(formElement.getLayoutComposite(), SWT.NONE);
//        labelText.setText(label);
//        Text text = new Text(formElement.getLayoutComposite(), SWT.WRAP);
//        text.setText(path);
//        text.setEditable(false);
//        text.setLayoutData(LayoutConstants.FILL());
//        text.setForeground(ColorResources.getColor(Resources.BLACK));
//    }

    @Override
    public void setEntity(CharacterDto entity) {
        super.setEntity(entity);
        setEnabled(TermBasePropertyTester.isModifiable(entity));
    }

	@Override
	public void handleEvent(Object eventSource) {
	    if(eventSource == comboStructuralModifier){
	        getEntity().setStructureModifier(TermDto.fromTerm(comboStructuralModifier.getSelection()));
	    }
	    if(eventSource == ratioTo){
	        if (!ratioTo.isState()){
	            getEntity().setRatioTo(null);
	        }
        }

//	    if(eventSource == supportsQuantitativeData || eventSource == supportsCategoricalData){
//	        //if any type is selected disable supportsTextData
//	        getEntity().setSupportsCategoricalData(supportsCategoricalData.getSelection());
//	        getEntity().setSupportsQuantitativeData(supportsQuantitativeData.getSelection());
//	    }
	    if(eventSource == supportsQuantitativeData){
	        getEntity().setSupportsQuantitativeData(supportsQuantitativeData.getSelection());
	        getEntity().setSupportsCategoricalData(supportsCategoricalData.getSelection());
	        if(supportsQuantitativeData.getSelection()){
	            activateQuantitativeWidgets();
	        }
	        else{
	            removeQuantitativeWidgets();
	        }
            StoreUtil.reflowParentScrolledForm(getLayoutComposite(), true);
	    }
	    else if(eventSource == supportsCategoricalData){
	        getEntity().setSupportsCategoricalData(supportsCategoricalData.getSelection());
	        getEntity().setSupportsQuantitativeData(supportsQuantitativeData.getSelection());
            if (supportsCategoricalData.getSelection()) {
                activateCategoricalWidgets();
            }
	        else{
	            removeCategoricalWidgets();
	        }
            StoreUtil.reflowParentScrolledForm(getLayoutComposite(), true);
	    }
	    else if (eventSource == element_Representation) {
            Representation selectedRepresentation = element_Representation.getSelectedRepresentation();
            if(selectedRepresentation!=null){
                Language representationLanguage = selectedRepresentation.getLanguage();
                if(representationLanguage==null){
                    representationLanguage = Language.getDefaultLanguage();
                }
//                TODO:handle representations correctly
//                getEntity().setLabel(selectedRepresentation.getLabel(),  representationLanguage);
//                getEntity().getRepresentation(representationLanguage).setAbbreviatedLabel(selectedRepresentation.getAbbreviatedLabel());
//                getEntity().getRepresentation(representationLanguage).setText(selectedRepresentation.getDescription());
            }
//            getEntity().setTitleCache(null);
        } 
	    else if(eventSource == availableForSpecimenOrObservation){
            getEntity().setAvailableForOccurrence(availableForSpecimenOrObservation.getSelection());
            
        }
	    else if(eventSource == availableForTaxon){
            getEntity().setAvailableForTaxon(availableForTaxon.getSelection());
            
        }
	    else if(eventSource == availableForTaxonName){
            getEntity().setAvailableForTaxonName(availableForTaxonName.getSelection());
            
        }
	}

	private void activateQuantitativeWidgets(){
	    //disable categorical widgets
	    supportsCategoricalData.setSelection(false);
	    getEntity().setSupportsCategoricalData(false);
	    removeCategoricalWidgets();

	    //measurement units
	    sectionMeasurementUnits = formFactory.createMeasurementUnitDtoCollectionSection(getConversationHolder(),
	            parentFormElement, StoreUtil.getSectionStyle(MeasurementUnitCollectionSection.class, getEntity().getClass().getCanonicalName(), true));
	    sectionMeasurementUnits.setLayoutData(LayoutConstants.FILL_HORIZONTALLY(2, 1));
	    sectionMeasurementUnits.setEntity((getEntity()));

	    //statistical measures
	    sectionStatisticalMeasures = formFactory.createStatisticalMeasureDtoCollectionSection(
	            getConversationHolder(), parentFormElement, StoreUtil.getSectionStyle(StatisticalMeasureCollectionSection.class, getEntity().getClass().getCanonicalName(), true));
	    sectionStatisticalMeasures.setLayoutData(LayoutConstants.FILL_HORIZONTALLY(2, 1));
	    sectionStatisticalMeasures.setEntity((getEntity()));

	    //modifiers
	    sectionModifierVocabularies = formFactory.createRecommendedModifierVocabulariesDtoCollectionSection(
	            getConversationHolder(), parentFormElement, StoreUtil.getSectionStyle(RecommendedModifierVocabulariesCollectionSection.class, getEntity().getClass().getCanonicalName(), true));
	    sectionModifierVocabularies.setLayoutData(LayoutConstants.FILL_HORIZONTALLY(2, 1));
	    sectionModifierVocabularies.setEntity((getEntity()));
	}

	private void removeQuantitativeWidgets(){
	    if(sectionMeasurementUnits!=null){
	        removeElementsAndControls(sectionMeasurementUnits);
	        removeElementsAndControls(sectionStatisticalMeasures);
	        removeElementsAndControls(sectionModifierVocabularies);
	    }
	}

	private void activateCategoricalWidgets(){
	    //disable quantitative widgeets
	    supportsQuantitativeData.setSelection(false);
	    getEntity().setSupportsQuantitativeData(false);
	    removeQuantitativeWidgets();

	    //states
	    sectionStateVocabularies = formFactory.createStateVocabulariesDtoSection(getConversationHolder(),
                parentFormElement, StoreUtil.getSectionStyle(StateVocabularyCollectionSection.class, getEntity().getClass().getCanonicalName(), true));
        sectionStateVocabularies.setLayoutData(LayoutConstants.FILL_HORIZONTALLY(2, 1));
        sectionStateVocabularies.setEntity((getEntity()));

        //modifiers
        sectionModifierVocabularies = formFactory.createRecommendedModifierVocabulariesDtoCollectionSection(
                getConversationHolder(), parentFormElement, StoreUtil.getSectionStyle(RecommendedModifierVocabulariesCollectionSection.class, getEntity().getClass().getCanonicalName(), true));
        sectionModifierVocabularies.setLayoutData(LayoutConstants.FILL_HORIZONTALLY(2, 1));
        sectionModifierVocabularies.setEntity((getEntity()));
	}

	private void removeCategoricalWidgets(){
	    if(sectionStateVocabularies!=null){
	        removeElementsAndControls(sectionStateVocabularies);
	        removeElementsAndControls(sectionModifierVocabularies);
	    }
	}

	@Override
    public void propertyChange(PropertyChangeEvent event) {
	    EventUtility.postEvent(WorkbenchEventConstants.ADD_SAVE_CANDIDATE, getEntity());
	    super.propertyChange(event);

	}

}
