/**
* Copyright (C) 2009 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.ui.section.feature;

import org.eclipse.jface.util.PropertyChangeEvent;

import eu.etaxonomy.cdm.model.description.Character;
import eu.etaxonomy.cdm.persistence.dto.CharacterDto;
import eu.etaxonomy.cdm.persistence.dto.TermNodeDto;
import eu.etaxonomy.taxeditor.editor.definedterm.TermBasePropertyTester;
import eu.etaxonomy.taxeditor.event.EventUtility;
import eu.etaxonomy.taxeditor.event.WorkbenchEventConstants;
import eu.etaxonomy.taxeditor.store.StoreUtil;
import eu.etaxonomy.taxeditor.ui.element.CdmFormFactory;
import eu.etaxonomy.taxeditor.ui.element.ICdmFormElement;
import eu.etaxonomy.taxeditor.ui.element.LayoutConstants;
import eu.etaxonomy.taxeditor.ui.section.AbstractCdmDetailElement;

/**
 *
 * @author pplitzner
 * @since Aug 19, 2019
 *
 */
public class CharacterNodeDtoDetailElement extends AbstractCdmDetailElement<TermNodeDto> {
    private CharacterDtoDetailSection characterDetails;

    private ICdmFormElement parentFormElement;

    private InapplicableIfEntityCollectionSection sectionInapplicableIf;
    private OnlyApplicableIfEntityCollectionSection sectionOnlyApplicableIf;

	public CharacterNodeDtoDetailElement(CdmFormFactory formFactory,
			ICdmFormElement formElement) {
		super(formFactory, formElement);
	}

	@Override
	protected void createControls(ICdmFormElement formElement, TermNodeDto entity, int style) {
	    this.parentFormElement = formElement;
//	    characterDetails = formFactory.createCharacterDetailElement(formElement, style);
	    characterDetails = formFactory.createCharacterDtoDetailSection(getConversationHolder(), formElement, null, StoreUtil.getSectionStyle(CharacterDetailSection.class, entity.getClass().getCanonicalName(), true));
	    characterDetails.setLayoutData(LayoutConstants.FILL_HORIZONTALLY(2, 1));
	    addElement(characterDetails);
	    addControl(characterDetails);
	    characterDetails.setEntity((CharacterDto)entity.getTerm());

        // applicability
        sectionInapplicableIf = formFactory.createInapplicableIfEntityCollectionSection(getConversationHolder(),
                formElement, StoreUtil.getSectionStyle(InapplicableIfEntityCollectionSection.class, entity.getTerm().getClass().getCanonicalName()));
        sectionInapplicableIf.setLayoutData(LayoutConstants.FILL_HORIZONTALLY(2, 1));
        sectionInapplicableIf.setEntity(getEntity());
        sectionOnlyApplicableIf = formFactory.createOnlyApplicableIfEntityCollectionSection(getConversationHolder(),
                formElement, StoreUtil.getSectionStyle(OnlyApplicableIfEntityCollectionSection.class, entity.getTerm().getClass().getCanonicalName()));
        sectionOnlyApplicableIf.setLayoutData(LayoutConstants.FILL_HORIZONTALLY(2, 1));
        sectionOnlyApplicableIf.setEntity(entity);
    }



    public static String getPropertyText(Character entity) {
        return entity.getProperty()!=null?StoreUtil.getPath(entity.getProperty()):"[no property]";
    }

    public static String getStructureText(Character entity) {
        return entity.getStructure()!=null?StoreUtil.getPath(entity.getStructure()):"[no structure]";
    }

    @Override
    public void setEntity(TermNodeDto entity) {
        super.setEntity(entity);
        characterDetails.setEntity((CharacterDto)entity.getTerm());
        setEnabled(TermBasePropertyTester.isModifiable(entity));
    }

    @Override
    public void propertyChange(PropertyChangeEvent event) {
        EventUtility.postEvent(WorkbenchEventConstants.ADD_SAVE_CHARACTER, getEntity());
        super.propertyChange(event);

    }

	@Override
	public void handleEvent(Object eventSource) {

//	    if(eventSource == characterDetails){
//	        getEntity().getTerm().setStructureModifier(comboStructuralModifier.getSelection());
//	    }
//	    if(eventSource == supportsQuantitativeData || eventSource == supportsCategoricalData){
//	        //if any type is selected disable supportsTextData
//	        getEntity().getTerm().setSupportsTextData(false);
//	    }
//	    if(eventSource == supportsQuantitativeData){
//	        getEntity().getTerm().setSupportsQuantitativeData(supportsQuantitativeData.getSelection());
//	        if(supportsQuantitativeData.getSelection()){
//	            activateQuantitativeWidgets();
//	        }
//	        else{
//	            removeQuantitativeWidgets();
//	        }
//            StoreUtil.reflowParentScrolledForm(getLayoutComposite(), true);
//	    }
//	    else if(eventSource == supportsCategoricalData){
//	        getEntity().getTerm().setSupportsCategoricalData(supportsCategoricalData.getSelection());
//            if (supportsCategoricalData.getSelection()) {
//                activateCategoricalWidgets();
//            }
//	        else{
//	            removeCategoricalWidgets();
//	        }
//            StoreUtil.reflowParentScrolledForm(getLayoutComposite(), true);
//	    }
//	    else if (eventSource == element_Representation) {
//            Representation selectedRepresentation = element_Representation.getSelectedRepresentation();
//            if(selectedRepresentation!=null){
//                Language representationLanguage = selectedRepresentation.getLanguage();
//                if(representationLanguage==null){
//                    representationLanguage = Language.getDefaultLanguage();
//                }
//                getEntity().getTerm().setLabel(selectedRepresentation.getLabel(),  representationLanguage);
//                getEntity().getTerm().getRepresentation(representationLanguage).setAbbreviatedLabel(selectedRepresentation.getAbbreviatedLabel());
//                getEntity().getTerm().getRepresentation(representationLanguage).setText(selectedRepresentation.getDescription());
//            }
//            getEntity().getTerm().setTitleCache(null);
//        }
	}

//	private void activateQuantitativeWidgets(){
//	    //disable categorical widgets
//	    supportsCategoricalData.setSelection(false);
//	    getEntity().getTerm().setSupportsCategoricalData(false);
//	    removeCategoricalWidgets();
//
//	    //measurement units
//	    sectionMeasurementUnits = formFactory.createMeasurementUnitCollectionSection(getConversationHolder(),
//	            parentFormElement, StoreUtil.getSectionStyle(MeasurementUnitCollectionSection.class, getEntity().getClass().getCanonicalName(), true));
//	    sectionMeasurementUnits.setLayoutData(LayoutConstants.FILL_HORIZONTALLY(2, 1));
//	    sectionMeasurementUnits.setEntity(getEntity().getTerm());
//
//	    //statistical measures
//	    sectionStatisticalMeasures = formFactory.createStatisticalMeasureCollectionSection(
//	            getConversationHolder(), parentFormElement, StoreUtil.getSectionStyle(StatisticalMeasureCollectionSection.class, getEntity().getClass().getCanonicalName(), true));
//	    sectionStatisticalMeasures.setLayoutData(LayoutConstants.FILL_HORIZONTALLY(2, 1));
//	    sectionStatisticalMeasures.setEntity(getEntity().getTerm());
//
//	    //modifiers
//	    sectionModifierVocabularies = formFactory.createRecommendedModifierVocabulariesCollectionSection(
//	            getConversationHolder(), parentFormElement, StoreUtil.getSectionStyle(RecommendedModifierVocabulariesCollectionSection.class, getEntity().getClass().getCanonicalName(), true));
//	    sectionModifierVocabularies.setLayoutData(LayoutConstants.FILL_HORIZONTALLY(2, 1));
//	    sectionModifierVocabularies.setEntity(getEntity().getTerm());
//	}
//
//	private void removeQuantitativeWidgets(){
//	    if(sectionMeasurementUnits!=null){
//	        removeElementsAndControls(sectionMeasurementUnits);
//	        removeElementsAndControls(sectionStatisticalMeasures);
//	        removeElementsAndControls(sectionModifierVocabularies);
//	    }
//	}
//
//	private void activateCategoricalWidgets(){
//	    //disable quantitative widgeets
//	    supportsQuantitativeData.setSelection(false);
//	    getEntity().getTerm().setSupportsQuantitativeData(false);
//	    removeQuantitativeWidgets();
//
//	    //states
//	    sectionStateVocabularies = formFactory.createStateVocabulariesSection(getConversationHolder(),
//                parentFormElement, StoreUtil.getSectionStyle(StateVocabularyCollectionSection.class, getEntity().getClass().getCanonicalName(), true));
//        sectionStateVocabularies.setLayoutData(LayoutConstants.FILL_HORIZONTALLY(2, 1));
//        sectionStateVocabularies.setEntity(getEntity().getTerm());
//
//        //modifiers
//        sectionModifierVocabularies = formFactory.createRecommendedModifierVocabulariesCollectionSection(
//                getConversationHolder(), parentFormElement, StoreUtil.getSectionStyle(RecommendedModifierVocabulariesCollectionSection.class, getEntity().getClass().getCanonicalName(), true));
//        sectionModifierVocabularies.setLayoutData(LayoutConstants.FILL_HORIZONTALLY(2, 1));
//        sectionModifierVocabularies.setEntity(getEntity().getTerm());
//	}
//
//	private void removeCategoricalWidgets(){
//	    if(sectionStateVocabularies!=null){
//	        removeElementsAndControls(sectionStateVocabularies);
//	        removeElementsAndControls(sectionModifierVocabularies);
//	    }
//	}

}
