/**
* Copyright (C) 2009 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.ui.section.feature;

import org.eclipse.swt.widgets.Label;

import eu.etaxonomy.cdm.model.term.TermTree;
import eu.etaxonomy.taxeditor.model.AbstractUtility;
import eu.etaxonomy.taxeditor.model.ColorResources;
import eu.etaxonomy.taxeditor.preference.Resources;
import eu.etaxonomy.taxeditor.termtree.e4.ICharacterEditor;
import eu.etaxonomy.taxeditor.ui.element.CdmFormFactory;
import eu.etaxonomy.taxeditor.ui.element.CheckboxElement;
import eu.etaxonomy.taxeditor.ui.element.ICdmFormElement;
import eu.etaxonomy.taxeditor.ui.element.TextWithLabelElement;
import eu.etaxonomy.taxeditor.ui.section.AbstractCdmDetailElement;

/**
 * @author pplitzner
 * @since Dec 21, 2017
 */
public class TermTreeDetailElementForNode extends AbstractCdmDetailElement<TermTree> {

    private CheckboxElement allowDuplicates;
    private CheckboxElement orderRelevant;
    private CheckboxElement isFlat;

    private TextWithLabelElement titleCacheText;

	public TermTreeDetailElementForNode(CdmFormFactory formFactory,
			ICdmFormElement formElement) {
		super(formFactory, formElement);
	}


	@Override
	protected void createControls(ICdmFormElement formElement, TermTree entity, int style) {

	    Label lblTermType = new Label(formElement.getLayoutComposite(), style);
	    lblTermType.setText("Term Type");
	    Label lblStructureText = new Label(formElement.getLayoutComposite(), style);
	    lblStructureText.setText(entity.getTermType().getLabel());
	    lblStructureText.setForeground(ColorResources.getColor(Resources.BLACK));

	    titleCacheText = formFactory.createTextWithLabelElement(
                formElement, "Title Cache", entity.getTitleCache(), style);


	    allowDuplicates = formFactory.createCheckbox(formElement, "Allow duplicates", entity.isAllowDuplicates(), style);
	    orderRelevant = formFactory.createCheckbox(formElement, "Order relevant", entity.isOrderRelevant(), style);
	    isFlat = formFactory.createCheckbox(formElement, "Is flat", entity.isFlat(), style);
	    if (AbstractUtility.getActivePart() instanceof ICharacterEditor){
	        titleCacheText.setEnabled(false);
	        allowDuplicates.setEnabled(false);
	        orderRelevant.setEnabled(false);
	        isFlat.setEnabled(false);
	    }

    }

	@Override
	protected void updateControlStates() {
	    super.updateControlStates();
//	    toggleable_cache.setEnabled(getEntity().isProtectedTitleCache());
	}

	@Override
	public void handleEvent(Object eventSource) {
	    if (eventSource == titleCacheText) {
	            getEntity().setTitleCache(titleCacheText.getText());;
        }
	    else
	    if (eventSource == allowDuplicates) {
	        getEntity().setAllowDuplicates(allowDuplicates.getSelection());
	    }
	    else if (eventSource == orderRelevant) {
	        getEntity().setOrderRelevant(orderRelevant.getSelection());
	    }
	    else if (eventSource == isFlat) {
	        getEntity().setFlat(isFlat.getSelection());
	    }
	}

}
