/**
 * Copyright (C) 2007 EDIT
 * European Distributed Institute of Taxonomy
 * http://www.e-taxonomy.eu
 *
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */

package eu.etaxonomy.taxeditor.ui.section.grantedAuthority;

import java.util.Collection;
import java.util.Comparator;

import org.springframework.security.core.GrantedAuthority;

import eu.etaxonomy.cdm.api.conversation.ConversationHolder;
import eu.etaxonomy.cdm.model.permission.Group;
import eu.etaxonomy.taxeditor.ui.dialog.selection.GrantedAuthoritySelectionDialog;
import eu.etaxonomy.taxeditor.ui.element.CdmFormFactory;
import eu.etaxonomy.taxeditor.ui.element.ICdmFormElement;
import eu.etaxonomy.taxeditor.ui.section.AbstractEntityCollectionSection;

/**
 * @author n.hoffmann
 * @created Mar 9, 2011
 * @version 1.0
 */
public class GrantedAuthorityDetailSection extends
		AbstractEntityCollectionSection<Group, GrantedAuthority> {

	/**
	 * @param formFactory
	 * @param conversation
	 * @param parentElement
	 * @param title
	 * @param style
	 */
	public GrantedAuthorityDetailSection(CdmFormFactory formFactory,
			ConversationHolder conversation, ICdmFormElement parentElement,
			int style) {
		super(formFactory, conversation, parentElement, "Granted Authorities",
				style);
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see eu.etaxonomy.taxeditor.ui.section.AbstractEntityCollectionSection#
	 * getCollection(java.lang.Object)
	 */
	@Override
	public Collection<GrantedAuthority> getCollection(Group entity) {
		return entity.getGrantedAuthorities();
	}

	@Override
	public Comparator<GrantedAuthority> getComparator() {
        return new Comparator<GrantedAuthority>() {

            @Override
            public int compare(GrantedAuthority o1, GrantedAuthority o2) {
                if(o1==null){
                    return -1;
                }
                if(o2==null){
                    return 1;
                }
                return o1.getAuthority().compareTo(o2.getAuthority());
            }
        };
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see eu.etaxonomy.taxeditor.ui.section.AbstractEntityCollectionSection#
	 * createNewElement()
	 */
	@Override
	public GrantedAuthority createNewElement() {
//		StoreUtil
//				.warningDialog("Not implemented yet", getClass(),
//						"This functionality needs to be implemented in CDM Library first.");

		GrantedAuthority selection = GrantedAuthoritySelectionDialog.select(getLayoutComposite().getShell(), //getConversationHolder(),
		        null);
		return selection;
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see
	 * eu.etaxonomy.taxeditor.ui.section.AbstractEntityCollectionSection#addElement
	 * (java.lang.Object)
	 */
	@Override
	public void addElement(GrantedAuthority element) {
		getEntity().addGrantedAuthority(element);
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see eu.etaxonomy.taxeditor.ui.section.AbstractEntityCollectionSection#
	 * removeElement(java.lang.Object)
	 */
	@Override
	public void removeElement(GrantedAuthority element) {
		getEntity().removeGrantedAuthority(element);
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see eu.etaxonomy.taxeditor.ui.section.AbstractEntityCollectionSection#
	 * getEmptyString()
	 */
	@Override
	public String getEmptyString() {
		return "No Granted Authorities yet.";
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see eu.etaxonomy.taxeditor.ui.section.AbstractEntityCollectionSection#
	 * getTooltipString()
	 */
	@Override
	protected String getTooltipString() {
		return "Add a Granted Authority";
	}

    /**
     * {@inheritDoc}
     */
    @Override
    public GrantedAuthority addExisting() {

        return null;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public boolean allowAddExisting() {
        return false;
    }

}
