package eu.etaxonomy.taxeditor.ui.section.grantedAuthority;

import java.util.UUID;

import org.apache.log4j.Logger;
import org.springframework.security.core.GrantedAuthority;

import eu.etaxonomy.cdm.api.service.IAgentService;
import eu.etaxonomy.cdm.api.service.IClassificationService;
import eu.etaxonomy.cdm.api.service.IDescriptionElementService;
import eu.etaxonomy.cdm.api.service.IDescriptionService;
import eu.etaxonomy.cdm.api.service.INameService;
import eu.etaxonomy.cdm.api.service.IOccurrenceService;
import eu.etaxonomy.cdm.api.service.IReferenceService;
import eu.etaxonomy.cdm.api.service.IRegistrationService;
import eu.etaxonomy.cdm.api.service.ITaxonNodeService;
import eu.etaxonomy.cdm.api.service.ITaxonService;
import eu.etaxonomy.cdm.api.service.ITermService;
import eu.etaxonomy.cdm.api.service.IUserService;
import eu.etaxonomy.cdm.model.taxon.TaxonNode;
import eu.etaxonomy.cdm.persistence.permission.CdmAuthority;
import eu.etaxonomy.taxeditor.store.CdmStore;

public class GrantedAuthorityLabelTextProvider {

	public static final Logger logger = Logger.getLogger(CdmAuthority.class);


	public static String getText(GrantedAuthority grantedAuthority){
		String labelText =  grantedAuthority.getAuthority();
		try {
		    if (!labelText.equals("ROLE_REMOTING")){
    			CdmAuthority cdmAuthority = CdmAuthority.fromGrantedAuthority(grantedAuthority);
    			String targetLabelText = getTargetText(cdmAuthority);
    			if(targetLabelText.length() > 0 && cdmAuthority.hasTargetUuid() ){
    				labelText = grantedAuthority.getAuthority().replace(cdmAuthority.getTargetUUID().toString(), targetLabelText);
    			}else if (targetLabelText.length() >0){
    			    labelText = grantedAuthority.getAuthority()+ "."+ labelText;
    			}
		    }
		} catch (Exception e) {
			/* will in most cases catch a ParsingException
			 * this is ignored
			 */
			if(!e.getClass().getSimpleName().equals("ParsingException")){
				e.printStackTrace();
			}
			// in case it is a Role
			labelText = grantedAuthority.getAuthority();
		}
		return labelText;
	}

 	public static String getTargetText(GrantedAuthority grantedAuthority){
 		if(grantedAuthority instanceof CdmAuthority){
 			return getCdmAuthorityTargetText((CdmAuthority)grantedAuthority);

 		}
 		return "";
 	}

	private static String getCdmAuthorityTargetText(CdmAuthority cdmAuthority) {

		UUID uuid = cdmAuthority.getTargetUUID();
		String targetText = "";
		String propertyString = cdmAuthority.getProperty();

		if(uuid != null){
			targetText = uuid.toString();
			if(cdmAuthority.getPermissionClass() != null){
				try{
					switch(cdmAuthority.getPermissionClass()) {
					case USER:
						targetText = CdmStore.getService(IUserService.class).load(uuid).getUsername();
						break;
					case DESCRIPTIONBASE:
						targetText = CdmStore.getService(IDescriptionService.class).load(uuid).getTitleCache();
						break;
					case DESCRIPTIONELEMENTBASE:
						targetText = CdmStore.getService(IDescriptionElementService.class).load(uuid, null).toString();
						break;
					case TAXONBASE:
						targetText = CdmStore.getService(ITaxonService.class).load(uuid).getTitleCache();
						break;
					case ALL:
						// makes not much sense here
						break;
					case TAXONNODE:
						TaxonNode node = CdmStore.getService(ITaxonNodeService.class).load(uuid);
						if(node.getClassification() != null){
							targetText = node.getClassification().getTitleCache() + " : ";
						}
						if(node.getTaxon() != null){
							targetText += node.getTaxon().getTitleCache();
						} else{
							uuid = node.getClassification().getUuid();
							targetText = CdmStore.getService(IClassificationService.class).load(uuid).getTitleCache();
						}
						break;
					case CLASSIFICATION:
						targetText = CdmStore.getService(IClassificationService.class).load(uuid).getTitleCache();
						break;
					case REFERENCE:
						targetText = CdmStore.getService(IReferenceService.class).load(uuid).getTitleCache();
						break;
					case TAXONNAME:
						targetText = CdmStore.getService(INameService.class).load(uuid).getTitleCache();
						break;
					case TEAMORPERSONBASE:
						targetText = CdmStore.getService(IAgentService.class).load(uuid).getTitleCache();
						break;
                    case REGISTRATION:
                        targetText = CdmStore.getService(IRegistrationService.class).load(uuid).getIdentifier();
                        break;
                    case SPECIMENOROBSERVATIONBASE:
                        targetText = CdmStore.getService(IOccurrenceService.class).load(uuid).getTitleCache();
                        break;
                    case FEATURE:
                        targetText = CdmStore.getService(ITermService.class).load(uuid).getTitleCache();
                        break;
                    default:
                        break;

					}
				} catch (NullPointerException e){
					logger.warn("Either no service found for " + cdmAuthority.getPermissionClass() + " or entitiy not found" , e);
				}
			}
		}else if (propertyString != null){
		    targetText = propertyString;
		}

		return targetText;
	}


}
