/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.ui.section.media;

import org.apache.commons.io.FileUtils;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.core.runtime.jobs.Job;
import org.eclipse.swt.events.SelectionListener;

import eu.etaxonomy.cdm.api.service.media.MediaInfoFileReader;
import eu.etaxonomy.cdm.common.URI;
import eu.etaxonomy.cdm.common.media.CdmImageInfo;
import eu.etaxonomy.cdm.model.media.ImageFile;
import eu.etaxonomy.taxeditor.store.StoreUtil;
import eu.etaxonomy.taxeditor.ui.element.AbstractFormSection;
import eu.etaxonomy.taxeditor.ui.element.CdmFormFactory;
import eu.etaxonomy.taxeditor.ui.element.CdmPropertyChangeEvent;
import eu.etaxonomy.taxeditor.ui.element.ICdmFormElement;
import eu.etaxonomy.taxeditor.ui.element.ImageElement;
import eu.etaxonomy.taxeditor.ui.element.KeyValueViewerElement;
import eu.etaxonomy.taxeditor.ui.element.NumberWithLabelElement;

/**
 * @author n.hoffmann
 * @created Mar 26, 2010
 */
public class ImageFileElement extends MediaRepresentationPartElement<ImageFile> {

    private final class LoadImageJob extends Job{

        private URI uri;
        private boolean updateDimensions;

        public LoadImageJob(URI uri, boolean updateDimensions) {
            super("Load image");
            this.uri = uri;
            this.updateDimensions = updateDimensions;
        }

        @Override
        protected IStatus run(IProgressMonitor monitor) {
            try{
                CdmImageInfo imageInfo = MediaInfoFileReader.legacyFactoryMethod(uri).readBaseInfo().readMetaData().getCdmImageInfo();
                ImageFileElement.this.getLayoutComposite().getDisplay().asyncExec(()->{
                    element_keyValue.setInput(imageInfo.getMetaData());
                    try {
                        disposeImage();
                        element_image = formFactory.createImageElement(parentFormElement, uri, style);
                        element_image.initImageUri(uri);
                    } catch (Exception e) {
                        handleException();
                    }
                    element_image.loadImage();
                    if(uri == null){
                        return;
                    }
                    if(updateDimensions){
                        text_size.setText(FileUtils.byteCountToDisplaySize(imageInfo.getLength()));
                        // KLUDGE this is not save for very large files, because of the int cast.
                        // But then, I don't think we will handle such large files in the near future
                        getEntity().setSize((int) imageInfo.getLength());

                        text_height.setNumber(imageInfo.getHeight());
                        getEntity().setHeight(imageInfo.getHeight());

                        text_width.setNumber(imageInfo.getWidth());
                        getEntity().setWidth(imageInfo.getWidth());
                    }
                    StoreUtil.reflowParentScrolledForm(getLayoutComposite(), true);
                });
            }
            catch (Exception e) {
                handleException();
            }
            return Status.OK_STATUS;
        }
    }

	private NumberWithLabelElement text_height;
	private NumberWithLabelElement text_width;
	private KeyValueViewerElement element_keyValue;
	private ImageElement element_image;
    private ICdmFormElement parentFormElement;
    private int style;

	public ImageFileElement(CdmFormFactory cdmFormFactory,
			AbstractFormSection section, ImageFile element,
			SelectionListener removeListener, int style) {
		super(cdmFormFactory, section, element, removeListener, style);
	}

	@Override
	public void createControls(ICdmFormElement formElement, int style) {
		super.createControls(formElement, style);

        this.parentFormElement = formElement;
        this.style = style;

		text_height = formFactory.createFloatTextWithLabelElement(formElement, "Height", null, style);
		text_height.setEnabled(false);
		text_width = formFactory.createFloatTextWithLabelElement(formElement, "Width", null, style);
		text_width.setEnabled(false);
		element_keyValue = formFactory.createKeyValueViewerElement(formElement, "Key", "Value", null);
		if (entity != null){
			setEntity(entity);
		}
	}

	@Override
	public void setEntity(ImageFile entity) {
		super.setEntity(entity);
		if (text_height != null){
			text_height.setNumber(entity.getHeight());
			text_width.setNumber(entity.getWidth());
			try {
	            new LoadImageJob(getEntity().getUri(), true).schedule();
			} catch (Exception e) {
			    e.printStackTrace();
				handleException();
			}
		}
	}

	@Override
	public void handleEvent(Object eventSource) {
		if(eventSource == text_uri){
			try {
			    URI uri = text_uri.parseText();
				getEntity().setUri(uri);
				if(uri==null){
				    //buffer URI if parsing error occurred
				    MediaDetailElement parentMediaDetailElement = getParentMediaDetailElement();
				    if(parentMediaDetailElement!=null){
				        parentMediaDetailElement.setUriBuffer(text_uri.getText());
				    }
				}
				new LoadImageJob(getEntity().getUri(), true).schedule();
				firePropertyChangeEvent(new CdmPropertyChangeEvent(this, null));
			} catch (Exception e) {
				handleException();
			} finally {
				firePropertyChangeEvent(this);
			}
		}
	}

	protected void handleException() {
	    if(!getLayoutComposite().isDisposed()){
	        getLayoutComposite().getDisplay().asyncExec(()->{
	            disposeImage();
	            StoreUtil.reflowParentScrolledForm(getLayoutComposite(), true);
	        });
	    }
	}

    private void disposeImage(){
        if(element_image!=null){
            element_image.dispose();
            element_image = null;
        }
    }
}
