/**
* Copyright (C) 2014 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.ui.section.media;

import java.io.IOException;
import java.util.List;
import java.util.Set;

import org.apache.commons.imaging.ImageReadException;
import org.apache.http.HttpException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.core.runtime.jobs.Job;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Label;

import eu.etaxonomy.cdm.api.service.media.MediaInfoFileReader;
import eu.etaxonomy.cdm.common.URI;
import eu.etaxonomy.cdm.common.UriUtils;
import eu.etaxonomy.cdm.common.media.CdmImageInfo;
import eu.etaxonomy.cdm.model.common.CdmBase;
import eu.etaxonomy.cdm.model.media.ImageFile;
import eu.etaxonomy.cdm.model.media.Media;
import eu.etaxonomy.cdm.model.media.MediaRepresentation;
import eu.etaxonomy.cdm.model.media.MediaRepresentationPart;
import eu.etaxonomy.cdm.model.media.MediaUtils;
import eu.etaxonomy.taxeditor.l10n.Messages;
import eu.etaxonomy.taxeditor.model.MessagingUtils;
import eu.etaxonomy.taxeditor.preference.IPreferenceKeys;
import eu.etaxonomy.taxeditor.preference.PreferencesUtil;
import eu.etaxonomy.taxeditor.store.StoreUtil;
import eu.etaxonomy.taxeditor.ui.element.CdmFormFactory;
import eu.etaxonomy.taxeditor.ui.element.ICdmFormElement;
import eu.etaxonomy.taxeditor.ui.element.ImageElement;
import eu.etaxonomy.taxeditor.ui.element.LayoutConstants;
import eu.etaxonomy.taxeditor.ui.element.UriWithLabelElement;
import eu.etaxonomy.taxeditor.ui.section.AbstractCdmDetailElement;
import eu.etaxonomy.taxeditor.ui.section.description.MediaDetailsSection;

/**
 * @author pplitzner
 * @date 25.08.2014
 */
public class MediaDetailElement extends AbstractCdmDetailElement<Media>{

    private static final String LOAD_IMAGE = Messages.MediaDetailElement_LOAD_IMAGE;

    private boolean isShowImage = true;

    /**
     * @author pplitzner
     * @since Jul 11, 2019
     */
    private final class LoadImageJob extends Job {
        private URI uri;
        private LoadImageJob(URI uri, String name) {
            super(name);
            this.uri = uri;
        }

        @Override
        protected IStatus run(IProgressMonitor monitor) {
            try {
                //first check if uri refers to an actual (non-image) file

                UriUtils.getInputStream(uri);// will fail with a FileNotFoundException if not

                CdmImageInfo imageInfo = MediaInfoFileReader.legacyFactoryMethod(uri).readBaseInfo().readMetaData().getCdmImageInfo(); //will fail when it is no image file
                MediaDetailElement.this.getLayoutComposite().getDisplay().asyncExec(()->{
                    singleMediaRepresentationPart.setSize((int) imageInfo.getLength());
                    if(singleMediaRepresentationPart.isInstanceOf(ImageFile.class)){
                        ImageFile image = CdmBase.deproxy(singleMediaRepresentationPart, ImageFile.class);
                        image.setHeight(imageInfo.getHeight());
                        image.setWidth(imageInfo.getWidth());
                    }
                    if(singleMediaRepresentationPart.getMediaRepresentation()!=null){
                        singleMediaRepresentationPart.getMediaRepresentation().setMimeType(imageInfo.getMimeType());
                        singleMediaRepresentationPart.getMediaRepresentation().setSuffix(imageInfo.getSuffix());
                    }
                    disposeImage();
                    if (!parentFormElement.getLayoutComposite().isDisposed()) {
                        element_image = formFactory.createImageElement(parentFormElement, uri, style);
//                        StoreUtil.reflowParentScrolledForm(getLayoutComposite(), true);
                        try {
                            element_image.initImageUri(uri);
                        } catch (IOException | HttpException e) {
                            exception(e);
                        }
                        element_image.loadImage();

                        disposeErrorLabel();
                        StoreUtil.reflowParentScrolledForm(getLayoutComposite(), true);


                    }
                });
            } catch (Exception e){
               exception(e);
            }
            return Status.OK_STATUS;
        }
    }

    private UriWithLabelElement textUri;
    private MediaRepresentationSection section_mediaRepresentation;

    private boolean isAdvancedMediaView;
    private MediaRepresentationPart singleMediaRepresentationPart;
    private ICdmFormElement parentFormElement;
    private int style;

    /**
     * Used to store the URI even if it is invalid and thus cannot be stored in CDM
     */
    private String uriBuffer;
    private ImageElement element_image;
    private Label lblNoImage;

    public MediaDetailElement(CdmFormFactory formFactory, ICdmFormElement formElement) {
        super(formFactory, formElement);
    }

    @Override
    protected void createControls(ICdmFormElement formElement, Media entity, int style) {
        this.parentFormElement = formElement;
        this.style = style;
    }

    @Override
    public void setEntity(Media entity) {
        super.setEntity(entity);
        if(singleMediaRepresentationPart==null){
            singleMediaRepresentationPart = MediaUtils.initFirstMediaRepresentationPart(entity, true);
        }
        initIsAdvancedMediaView();
        showAdvancedView();
    }

    public boolean isShowImage() {
        return isShowImage;
    }

    public void setShowImage(boolean isShowImage) {
        this.isShowImage = isShowImage;
    }

    @Override
    public void handleEvent(Object eventSource){
        if(eventSource==textUri){
            textUri.setBackground(getPersistentBackground());
            URI uri = textUri.parseText();
            singleMediaRepresentationPart.setUri(uri);
            if(uri==null){
                uriBuffer=textUri.getText();
            }
            else{
                if (isShowImage){
                    LoadImageJob job = new LoadImageJob(uri, LOAD_IMAGE);
                    job.schedule();
                }
            }
            if (this.getParentElement() != null && this.getParentElement() instanceof MediaMetaElement){
                MediaMetaElement metaElement = (MediaMetaElement)this.getParentElement();
                if (metaElement.getParentElement() != null && metaElement.getParentElement() instanceof MediaDetailsSection){
                    MediaDetailsSection sec = (MediaDetailsSection)metaElement.getParentElement();
//                    sec.setMediaSectionTitle();
                    metaElement.createTitle();
                    sec.setMediaSectionTitle();
                }
            }
        }
    }

    private void exception(Exception e){
        if(!getLayoutComposite().isDisposed()){
            getLayoutComposite().getDisplay().asyncExec(()->{
                String exceptionString;
                if(e.getCause()!=null && e.getCause().getClass().equals(ImageReadException.class)){
                    disposeErrorLabel();
                    exceptionString = Messages.MediaDetailElement_NO_PREVIEW;
                }
                else{
                    disposeErrorLabel();
                    exceptionString = Messages.MediaDetailElement_NO_FILE_FOUND;
                }
                disposeImage();
                if(lblNoImage==null){
                    lblNoImage = formFactory.createLabel(getLayoutComposite(), exceptionString);
                    lblNoImage.setLayoutData(LayoutConstants.FILL_HORIZONTALLY(2, 1));
                    lblNoImage.setAlignment(SWT.CENTER);
                }
                StoreUtil.reflowParentScrolledForm(getLayoutComposite(), true);
            });
        }
    }

    private void disposeImage(){
        if(element_image!=null){
            element_image.dispose();
            element_image = null;
        }
    }

	private void disposeErrorLabel() {
		if(lblNoImage!=null){
		    lblNoImage.dispose();
		}
		lblNoImage = null;
	}

    public void toggleAdvancedMediaView() {
        if (getEntity().getRepresentations() != null
                && (getEntity().getRepresentations().size() > 1 ||
                        (getEntity().getRepresentations().size() == 1
                        && getEntity().getRepresentations().iterator().next().getParts().size() > 1))) {
            MessagingUtils.informationDialog(Messages.MediaDetailElement_TOGGLE_NOT_POSSIBLE_TITLE,
                    Messages.MediaDetailElement_TOGGLE_NOT_POSSIBLE_MESSAGE);
            // toggling is only possible if there are no more than one
            // MediaRepresentation resp. MediaRepresentationParts
            return;
        }
        isAdvancedMediaView = !isAdvancedMediaView;
        showAdvancedView();
        StoreUtil.reflowParentScrolledForm(getLayoutComposite(), true);
    }

    private void showAdvancedView() {

        if(isAdvancedMediaView){
            if(textUri!=null){
                removeElementsAndControls(textUri);
            }
            disposeImage();
            section_mediaRepresentation = formFactory.createMediaRepresentationSection(getConversationHolder(), parentFormElement, StoreUtil.getSectionStyle(MediaRepresentationSection.class, getEntity().getClass().getCanonicalName()));
            section_mediaRepresentation.setLayoutData(LayoutConstants.FILL_HORIZONTALLY(2, 1));
            section_mediaRepresentation.setEntity(getEntity());
            //set buffered uri as text if uri had parsing problems in simple view
            if(uriBuffer!=null){
                section_mediaRepresentation.getLayoutComposite().getChildren();
            }
        }
        else{
            if(section_mediaRepresentation!=null){
                removeElementsAndControls(section_mediaRepresentation);
            }
            textUri = formFactory.createUriWithLabelElement(parentFormElement, Messages.MediaDetailElement_Media_URI, null, style);
            URI uri = singleMediaRepresentationPart.getUri();
            textUri.setParsedText(uri);
            //set buffered uri as text if uri had parsing problems in advanced view
            if(uri==null && uriBuffer!=null){
                textUri.setText(uriBuffer);
                textUri.parseText();
            }
            textUri.getLayoutComposite().layout();
            if (isShowImage){
                LoadImageJob job = new LoadImageJob(singleMediaRepresentationPart.getUri(), LOAD_IMAGE);
                job.schedule();
            } else{
                Button button_showImage = formFactory.createButton(getLayoutComposite(), Messages.MediaDetailElement_SHOW_IMAGE, SWT.PUSH);
                button_showImage.setLayoutData(LayoutConstants.FILL_HORIZONTALLY(2, 1));
                button_showImage.addSelectionListener(new SelectionAdapter() {
                    @Override
                    public void widgetSelected(SelectionEvent e) {
                        LoadImageJob job = new LoadImageJob(singleMediaRepresentationPart.getUri(), Messages.MediaDetailElement_LOAD_IMAGE);
                        job.schedule();
                        button_showImage.setText(Messages.MediaDetailElement_RELOAD_IMAGE);
                    }
                });
            }
        }
    }

    private void initIsAdvancedMediaView(){
        if(PreferencesUtil.getBooleanValue(IPreferenceKeys.SHOW_ADVANCED_MEDIA_SECTION)){
            isAdvancedMediaView = true;
            return;
        }
        Set<MediaRepresentation> representations = getEntity().getRepresentations();
        if(representations.size()>1){
            isAdvancedMediaView = true;
            return;
        }
        if(representations.size()==1){
            List<MediaRepresentationPart> parts = representations.iterator().next().getParts();
            if(parts.size()>1){
                isAdvancedMediaView =  true;
                return;
            }
        }
        isAdvancedMediaView =  false;
    }

    public boolean isAdvancedMediaView() {
        return isAdvancedMediaView;
    }

    public void setUriBuffer(String uriBuffer) {
        this.uriBuffer = uriBuffer;
    }

    public String getUriBuffer() {
        return uriBuffer;
    }

}
