/**
 * Copyright (C) 2007 EDIT
 * European Distributed Institute of Taxonomy
 * http://www.e-taxonomy.eu
 *
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */
package eu.etaxonomy.taxeditor.ui.section.media;

import java.util.Map;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionListener;

import eu.etaxonomy.cdm.hibernate.HibernateProxyHelper;
import eu.etaxonomy.cdm.model.agent.AgentBase;
import eu.etaxonomy.cdm.model.common.Language;
import eu.etaxonomy.cdm.model.common.LanguageString;
import eu.etaxonomy.cdm.model.common.TimePeriod;
import eu.etaxonomy.cdm.model.media.Media;
import eu.etaxonomy.taxeditor.model.AbstractUtility;
import eu.etaxonomy.taxeditor.store.CdmStore;
import eu.etaxonomy.taxeditor.ui.dialog.selection.MediaSelectionDialog;
import eu.etaxonomy.taxeditor.ui.element.AbstractFormSection;
import eu.etaxonomy.taxeditor.ui.element.CdmFormFactory;
import eu.etaxonomy.taxeditor.ui.element.ICdmFormElement;
import eu.etaxonomy.taxeditor.ui.element.ISelectableElement;
import eu.etaxonomy.taxeditor.ui.element.MultilanguageTextElement;
import eu.etaxonomy.taxeditor.ui.element.SelectionArbitrator;
import eu.etaxonomy.taxeditor.ui.element.TimePeriodElement;
import eu.etaxonomy.taxeditor.ui.section.AbstractCdmDetailSection;
import eu.etaxonomy.taxeditor.ui.section.AbstractEntityCollectionElement;
import eu.etaxonomy.taxeditor.ui.selection.EntitySelectionElement;

/**
 * @author n.hoffmann
 * @created Nov 17, 2009
 */
public class MediaMetaElement extends AbstractEntityCollectionElement<Media> implements ISelectableElement, ITogglableMediaElement{

    private EntitySelectionElement<AgentBase> selection_artist;
    private MultilanguageTextElement text_description;
    private MultilanguageTextElement text_title;
    private MediaDetailElement mediaDetailElement;
    private TimePeriodElement mediaCreated;
    private SelectionArbitrator selectionArbitrator;

    public MediaMetaElement(CdmFormFactory cdmFormFactory,
            AbstractFormSection<?> formElement, Media element,
            SelectionListener removeListener, int style) {
        super(cdmFormFactory, formElement, element, removeListener, null, style);
        //make element selectable
        if(cdmFormFactory.getSelectionProvider() != null){
            selectionArbitrator = cdmFormFactory.createSelectionArbitrator(this);
        }
    }

    @Override
    public void createControls(ICdmFormElement element, int style) {
//        final IAction switchAction = MediaViewSwitch.createSwitch(MediaMetaElement.this);
//        Button btnToggleAdvancedMediaView = new Button(getBox(), style);
//        btnToggleAdvancedMediaView.setToolTipText(switchAction.getText());
//        btnToggleAdvancedMediaView.setImage(ImageResources.getImage(ImageResources.SWITCH_VIEW_TYPE));
//        btnToggleAdvancedMediaView.addSelectionListener(new SelectionAdapter() {
//            @Override
//            public void widgetSelected(SelectionEvent e) {
//                switchAction.run();
//            }
//        });
        setWarnForReferencingObjects(element, 2);
        text_title = formFactory.createMultiLanguageTextElement(element, "Title", null, 50, style);
//        text_title = formFactory.createLanguageStringWithLabelElement(element, "Title", null, style);
        selection_artist = formFactory.createSelectionElement(AgentBase.class, //getConversationHolder(),
                element, "Artist", null, EntitySelectionElement.ALL, style);
        addElement(selection_artist);

        mediaCreated = formFactory.createTimePeriodElement(element, "Media Created", getEntity().getMediaCreated(), style);
        addElement(mediaCreated);
        text_description = formFactory.createMultiLanguageTextElement(this, "Description", null, 200, SWT.WRAP);


        mediaDetailElement = formFactory.createMediaDetailElement(element);

        ICdmFormElement grandGrandParent = getParentElement().getParentElement().getParentElement();
        Class<?> clazz = null;
        if (grandGrandParent != null){
            clazz = grandGrandParent.getClass();
        }
        //TODO: find a nicer solution
        if (grandGrandParent != null && clazz != null && clazz.getName().equals("eu.etaxonomy.taxeditor.ui.element.AbstractCdmEntityWizardPage$WizardPageRootElement")){
            mediaDetailElement.setShowImage(false);
        }
        addElement(mediaDetailElement);
        if (entity != null){
        	setEntity(entity);
        }
    }

    @Override
    public void setEntity(Media element) {
        this.entity = element;
        if (mediaDetailElement != null){
	        mediaDetailElement.setEntity(element);
	
	        selection_artist.setEntity(HibernateProxyHelper
	                .deproxy(element.getArtist()));
	
	        text_description.setMultilanguageText(element.getAllDescriptions());
	        createTitle();
        }
    }

    public void createTitle() {
       LanguageString title = this.entity.getTitle(CdmStore.getDefaultLanguage());
       Map<Language, LanguageString> languageStrings = entity.getAllTitles();
       if (title == null || title.getText().contains("empty media")) {
           entity.getAllTitles().remove(title);
           title = LanguageString.NewInstance(entity.getTitleCache(), CdmStore.getDefaultLanguage());
           if (!title.getText().contains("empty media")){
               languageStrings.put(CdmStore.getDefaultLanguage(), title);
           }
       }
       text_title.setMultilanguageText(languageStrings);
    }

    @Override
    protected Media selectFromDialog() {

    	setEntity(MediaSelectionDialog.select(AbstractUtility.getShell(), //null,
    	        null));
        return entity;
    }

    @Override
    public void toggleAdvancedMediaView(){
        mediaDetailElement.toggleAdvancedMediaView();
    }

   @Override
    public void handleEvent(Object eventSource) {
        if (eventSource == selection_artist) {
            getEntity().setArtist(selection_artist.getSelection());
        } else if (eventSource == text_description) {
            text_description.getMultilanguageText().values()
            .forEach(languageString -> getEntity().putDescription(languageString));
        } else if (eventSource == text_title) {
            text_title.getMultilanguageText().values()
            .forEach(languageString -> getEntity().putTitle(languageString));
            if (getParentElement() instanceof AbstractCdmDetailSection) {
                ((AbstractCdmDetailSection<?>) getParentElement()).updateTitle();
            }
        } else if (eventSource == mediaCreated){
        	TimePeriod date = mediaCreated.getTimePeriod();
        	getEntity().setMediaCreated(date);
        }
    }

    @Override
    public SelectionArbitrator getSelectionArbitrator() {
        return selectionArbitrator;
    }

    public MediaDetailElement getMediaDetailElement(){
        return mediaDetailElement;
    }
}