/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.ui.section.media;

import java.io.IOException;
import java.util.Collection;

import org.apache.http.HttpException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.core.runtime.jobs.Job;
import org.eclipse.swt.events.SelectionListener;

import eu.etaxonomy.cdm.api.service.media.MediaInfoFileReader;
import eu.etaxonomy.cdm.common.URI;
import eu.etaxonomy.cdm.common.UriUtils;
import eu.etaxonomy.cdm.common.media.CdmImageInfo;
import eu.etaxonomy.cdm.model.media.ImageFile;
import eu.etaxonomy.cdm.model.media.MediaRepresentation;
import eu.etaxonomy.cdm.model.media.MediaRepresentationPart;
import eu.etaxonomy.taxeditor.store.StoreUtil;
import eu.etaxonomy.taxeditor.ui.element.AbstractFormSection;
import eu.etaxonomy.taxeditor.ui.element.CdmFormFactory;
import eu.etaxonomy.taxeditor.ui.element.ICdmFormElement;
import eu.etaxonomy.taxeditor.ui.element.LayoutConstants;
import eu.etaxonomy.taxeditor.ui.element.TextWithLabelElement;
import eu.etaxonomy.taxeditor.ui.section.AbstractEntityCollectionElement;

/**
 * <p>MediaRepresentationElement class.</p>
 *
 * @author n.hoffmann
 * @created Mar 24, 2010
 */
public class MediaRepresentationElement extends AbstractEntityCollectionElement<MediaRepresentation> {

	private MediaRepresentationPartSection section_mediaRepresentationPart;
	private TextWithLabelElement text_mimeType;
	private TextWithLabelElement text_suffix;

	public MediaRepresentationElement(CdmFormFactory cdmFormFactory,
			AbstractFormSection section,
			MediaRepresentation element, SelectionListener removeListener,
			int style) {
		super(cdmFormFactory, section, element, removeListener, null, style);
	}

	@Override
	public void createControls(ICdmFormElement element, int style) {
		text_mimeType = formFactory.createTextWithLabelElement(element, "Mime Type", null, style);
		text_suffix = formFactory.createTextWithLabelElement(element, "Suffix", null, style);
		section_mediaRepresentationPart = formFactory.createMediaRepresentationPartSection(getConversationHolder(), element, StoreUtil.getSectionStyle(MediaRepresentationPartSection.class, MediaRepresentation.class.getCanonicalName()));
		section_mediaRepresentationPart.setLayoutData(LayoutConstants.FILL_HORIZONTALLY(2, 1));
		if (entity != null){
			setEntity(entity);
		}
	}

	@Override
	public void setEntity(MediaRepresentation element) {
		this.entity = element;
		if (section_mediaRepresentationPart != null){
			section_mediaRepresentationPart.setEntity(element);
			text_mimeType.setText(element.getMimeType());
			text_suffix.setText(element.getSuffix());
		}
	}

	@Override
	public void handleEvent(Object eventSource) {
		if(eventSource == text_mimeType){
			getEntity().setMimeType(text_mimeType.getText());
		}
		else if(eventSource == text_suffix){
			getEntity().setSuffix(text_suffix.getText());
		}
		// FIXME HACK automatically set the mime type to the first mediaRepresentationPart's mimetype
		else if(eventSource == section_mediaRepresentationPart){
			firePropertyChangeEvent(this);
			new Job("Set mime type") {
                @Override
                protected IStatus run(IProgressMonitor monitor) {
                    Collection<MediaRepresentationPart> imageFileElements = section_mediaRepresentationPart.getCollection(section_mediaRepresentationPart.getEntity());
                    if(! imageFileElements.iterator().hasNext()){
                        return Status.CANCEL_STATUS;
                    }
                    MediaRepresentationPart mediaRepresentationPart = imageFileElements.iterator().next();
                    if(mediaRepresentationPart == null || !(mediaRepresentationPart instanceof ImageFile)){
                        return Status.CANCEL_STATUS;
                    }
                    ImageFile imageFile = (ImageFile) mediaRepresentationPart;

                    URI uri = imageFile.getUri();
                    if(!UriUtils.isServiceAvailable(uri)){
                        return Status.CANCEL_STATUS;
                    }
                    try {
                    	CdmImageInfo imageInfo = MediaInfoFileReader.legacyFactoryMethod(uri).readBaseInfo().readMetaData().getCdmImageInfo();
                        String mimeType = imageInfo.getMimeType();
                        getEntity().setMimeType(mimeType);
                        getEntity().setSuffix(imageInfo.getSuffix());
                        MediaRepresentationElement.this.getLayoutComposite().getDisplay().asyncExec(()->{
                            text_mimeType.setText(mimeType);
                            text_suffix.setText(imageInfo.getSuffix());
                        });
                    }
                    //theses exceptions do not need to be logged
                    //especially because this happens with every key stroke
                    catch (IOException|HttpException e) {
                        // ignore
                    }
                    return Status.OK_STATUS;
                }
            }.schedule();
		}
	}
}
