
package eu.etaxonomy.taxeditor.ui.section.media;

import java.util.Collection;
import java.util.Comparator;

import org.eclipse.jface.util.PropertyChangeEvent;

import eu.etaxonomy.cdm.api.conversation.ConversationHolder;
import eu.etaxonomy.cdm.model.media.ImageFile;
import eu.etaxonomy.cdm.model.media.MediaRepresentation;
import eu.etaxonomy.cdm.model.media.MediaRepresentationPart;
import eu.etaxonomy.taxeditor.ui.element.CdmFormFactory;
import eu.etaxonomy.taxeditor.ui.element.CdmPropertyChangeEvent;
import eu.etaxonomy.taxeditor.ui.element.ICdmFormElement;
import eu.etaxonomy.taxeditor.ui.section.AbstractEntityCollectionSection;
import eu.etaxonomy.taxeditor.ui.section.DefaultCdmBaseComparator;

/**
 * <p>MediaRepresentationPartSection class.</p>
 *
 * @author n.hoffmann
 */
public class MediaRepresentationPartSection extends
		AbstractEntityCollectionSection<MediaRepresentation, MediaRepresentationPart> {

	/**
	 * <p>Constructor for MediaRepresentationPartSection.</p>
	 *
	 * @param cdmFormFactory a {@link eu.etaxonomy.taxeditor.ui.element.CdmFormFactory} object.
	 * @param conversation a {@link eu.etaxonomy.cdm.api.conversation.ConversationHolder} object.
	 * @param parentElement a {@link eu.etaxonomy.taxeditor.ui.element.ICdmFormElement} object.
	 * @param style a int.
	 */
	public MediaRepresentationPartSection(CdmFormFactory cdmFormFactory, ConversationHolder conversation,
			ICdmFormElement parentElement, int style) {
		super(cdmFormFactory, conversation, parentElement, "Media Representation Part", style);
		formFactory.addPropertyChangeListener(this);
	}

	@Override
	public void addElement(MediaRepresentationPart element) {
		getEntity().addRepresentationPart(element);
	}
	@Override
	public MediaRepresentationPart createNewElement() {
		// FIXME we are supporting image only at the moment
		return ImageFile.NewInstance(null, null);
	}

	@Override
	public Collection<MediaRepresentationPart> getCollection(
			MediaRepresentation entity) {
		return getEntity().getParts();
	}

	@Override
	public Comparator<MediaRepresentationPart> getComparator() {
        return new DefaultCdmBaseComparator<>();
	}

	@Override
	public String getEmptyString() {
		return "No media representation parts yet.";
	}

	@Override
	protected String getTooltipString() {
		return "Create new media representation part.";
	}

	@Override
	public void removeElement(MediaRepresentationPart element) {
		getEntity().removeRepresentationPart(element);
	}

	@Override
	public void propertyChange(PropertyChangeEvent event) {
		if(event == null){
			return;
		}
		Object eventSource = event.getSource();
		if(eventSource != this && getElements().contains(eventSource)){
			firePropertyChangeEvent(new CdmPropertyChangeEvent(this, null));
			super.propertyChange(event);
		}
	}

    @Override
    public MediaRepresentationPart addExisting() {
        return null;
    }

    @Override
    public boolean allowAddExisting() {
        return false;
    }
}
