/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.ui.section.media;

import java.util.Collection;
import java.util.Comparator;

import eu.etaxonomy.cdm.api.conversation.ConversationHolder;
import eu.etaxonomy.cdm.model.media.Media;
import eu.etaxonomy.cdm.model.media.MediaRepresentation;
import eu.etaxonomy.taxeditor.ui.element.CdmFormFactory;
import eu.etaxonomy.taxeditor.ui.element.ICdmFormElement;
import eu.etaxonomy.taxeditor.ui.section.AbstractEntityCollectionSection;
import eu.etaxonomy.taxeditor.ui.section.DefaultCdmBaseComparator;

/**
 * <p>MediaRepresentationSection class.</p>
 *
 * @author n.hoffmann
 * @created Mar 24, 2010
 * @version 1.0
 */
public class MediaRepresentationSection extends AbstractEntityCollectionSection<Media, MediaRepresentation> {

	/**
	 * <p>Constructor for MediaRepresentationSection.</p>
	 *
	 * @param cdmFormFactory a {@link eu.etaxonomy.taxeditor.ui.element.CdmFormFactory} object.
	 * @param conversation a {@link eu.etaxonomy.cdm.api.conversation.ConversationHolder} object.
	 * @param parentElement a {@link eu.etaxonomy.taxeditor.ui.element.ICdmFormElement} object.
	 * @param style a int.
	 */
	public MediaRepresentationSection(CdmFormFactory cdmFormFactory, ConversationHolder conversation,
			ICdmFormElement parentElement, int style) {
		super(cdmFormFactory, conversation, parentElement, "Media Representation", style);
	}

	/** {@inheritDoc} */
	@Override
	public void addElement(MediaRepresentation element) {
		getEntity().addRepresentation(element);
	}

	/** {@inheritDoc} */
	@Override
	public MediaRepresentation createNewElement() {
		return MediaRepresentation.NewInstance();
	}

	/** {@inheritDoc} */
	@Override
	public Collection<MediaRepresentation> getCollection(
			Media entity) {
		return getEntity().getRepresentations();
	}

	@Override
	public Comparator<MediaRepresentation> getComparator() {
	    return new DefaultCdmBaseComparator<>();
	}

	/** {@inheritDoc} */
	@Override
	public String getEmptyString() {
		return "No media representations yet.";
	}

	/** {@inheritDoc} */
	@Override
	protected String getTooltipString() {
		return "Create a new media representation";
	}

	/** {@inheritDoc} */
	@Override
	public void removeElement(MediaRepresentation element) {
		getEntity().removeRepresentation(element);
	}

    /**
     * {@inheritDoc}
     */
    @Override
    public MediaRepresentation addExisting() {
        return null;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public boolean allowAddExisting() {
        return false;
    }


}
