/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.ui.section.name;

import java.util.ArrayList;
import java.util.Comparator;
import java.util.List;

import org.apache.log4j.Logger;
import org.eclipse.swt.SWT;

import eu.etaxonomy.cdm.model.metadata.PreferencePredicate;
import eu.etaxonomy.cdm.model.name.IBacterialName;
import eu.etaxonomy.cdm.model.name.ICultivarPlantName;
import eu.etaxonomy.cdm.model.name.IFungusName;
import eu.etaxonomy.cdm.model.name.INonViralName;
import eu.etaxonomy.cdm.model.name.IZoologicalName;
import eu.etaxonomy.cdm.model.name.NomenclaturalCode;
import eu.etaxonomy.cdm.model.name.Rank;
import eu.etaxonomy.cdm.model.name.TaxonNameFactory;
import eu.etaxonomy.cdm.model.term.DefinedTermBase;
import eu.etaxonomy.cdm.model.term.TermType;
import eu.etaxonomy.cdm.strategy.parser.ParserProblem;
import eu.etaxonomy.taxeditor.preference.NameDetailsConfigurator;
import eu.etaxonomy.taxeditor.preference.PreferencesUtil;
import eu.etaxonomy.taxeditor.store.CdmStore;
import eu.etaxonomy.taxeditor.ui.combo.term.TermComboElement;
import eu.etaxonomy.taxeditor.ui.element.CdmFormFactory;
import eu.etaxonomy.taxeditor.ui.element.CheckboxElement;
import eu.etaxonomy.taxeditor.ui.element.ICdmFormElement;
import eu.etaxonomy.taxeditor.ui.element.ISelectableElement;
import eu.etaxonomy.taxeditor.ui.element.SelectionArbitrator;
import eu.etaxonomy.taxeditor.ui.element.TextWithLabelElement;
import eu.etaxonomy.taxeditor.ui.section.AbstractIdentifiableEntityDetailElement;

/**
 * <p>NameDetailElement class.</p>
 *
 * @author n.hoffmann
 * @created Feb 26, 2010
 */
public class NameDetailElement
        extends AbstractIdentifiableEntityDetailElement<INonViralName>
        implements ISelectableElement {

    private final Logger logger = Logger.getLogger(NameDetailElement.class);

    private SelectionArbitrator selectionArbitrator;

    private TermComboElement<Rank> combo_rank;
	private TextWithLabelElement text_appendedPhrase;
	private TextWithLabelElement text_uninomial;
	private TextWithLabelElement text_infragenericEpithet;
	private TextWithLabelElement text_specificEpithet;
	private TextWithLabelElement text_infraspecificEpithet;
	private CheckboxElement checkbox_anamorphic;
	private TextWithLabelElement text_cultivarName;
	private TextWithLabelElement text_cultivarGroup;
	private TextWithLabelElement text_subGenusAuthorship;
	private TextWithLabelElement text_breed;

	private NameDetailsConfigurator config;

	/**
	 * <p>Constructor for NameDetailElement.</p>
	 *
	 * @param cdmFormFactory a {@link eu.etaxonomy.taxeditor.ui.element.CdmFormFactory} object.
	 * @param formElement a {@link eu.etaxonomy.taxeditor.ui.element.ICdmFormElement} object.
	 * @param style a int.
	 */
	public NameDetailElement(CdmFormFactory cdmFormFactory, ICdmFormElement formElement,
			int style) {
		super(cdmFormFactory, formElement);
		// make this element selectable
	}

	@Override
	protected void createControls(ICdmFormElement formElement, INonViralName nonViralName, int style) {
		Comparator<Rank> comparator = null;
		config = PreferencesUtil.getPreferredNameDetailsConfiguration();

		if(PreferencesUtil.getSortRanksHierarchichally()){
			comparator = new Comparator<Rank>(){
				@Override
				public int compare(Rank o1, Rank o2) {
				    if (o2 == null){
				        return -1;
				    }
					return -1*o1.compareTo(o2);
				}
			};
		}
		if (nonViralName.getRank() != null && !(nonViralName.getRank().isCultivar() == nonViralName.getNameType().equals(NomenclaturalCode.ICNCP))){
		    nonViralName.setRank(null);
		}
	    if (config == null){
	    	toggleable_cache = formFactory.createToggleableTextField(this, "Name Cache", nonViralName.getNameCache(), nonViralName.isProtectedNameCache(), style);
	    	List<Rank> ranks = getRanks(nonViralName, nonViralName.getNameType());

	    	combo_rank = formFactory.createDefinedTermComboElement(ranks, this, "Rank", nonViralName.getRank(),style, comparator);

	    	createGenusOrUninomialControls(this, nonViralName, style);
            createInfragenerericEpithetControls(this, nonViralName, style);
            createSpecificEpithetControls(this, nonViralName, style);
            createInfraSpecificEpithetControls(this, nonViralName, style);
            createSpecificNameParts(this, nonViralName, style);
            text_appendedPhrase = formFactory.createTextWithLabelElement(formElement, "Appended Phrase", nonViralName.getAppendedPhrase(), style);
	    }else{
	    	if (config.isNameCacheActivated()){
	    		toggleable_cache = formFactory.createToggleableTextField(this, "Name Cache", nonViralName.getNameCache(), nonViralName.isProtectedNameCache(), style);
        	}

    	    if (config.isRankActivated()){
    	        List<Rank> ranks = getRanks(nonViralName, nonViralName.getNameType());
                combo_rank = formFactory.createDefinedTermComboElement(ranks, this, "Rank", nonViralName.getRank(),style, comparator);
    	    }

    		if (config.isAtomisedEpithetsActivated()){
    			createGenusOrUninomialControls(this, nonViralName, style);
    			createInfragenerericEpithetControls(this, nonViralName, style);
    			createSpecificEpithetControls(this, nonViralName, style);
    			createInfraSpecificEpithetControls(this, nonViralName, style);
    			createSpecificNameParts(this, nonViralName, style);
    		}
    		if (config.isAppendedPhraseActivated()){
    				text_appendedPhrase = formFactory.createTextWithLabelElement(formElement, "Appended Phrase", nonViralName.getAppendedPhrase(), style);
           	}
	    }
	    if(this.getFormFactory().getSelectionProvider() != null){
            selectionArbitrator = this.getFormFactory().createSelectionArbitrator(this);
        }
	    for (ICdmFormElement element: this.getElements()){
	        element.setBackground(getPersistentBackground());
	    }
	    ((NameDetailSection)this.getParentElement()).layout();
	    //cache relevance
        registerCacheRelevance(combo_rank);
        //TODO decide if relevant
        registerCacheRelevance(text_appendedPhrase);
        registerCacheRelevance(text_uninomial);
        registerCacheRelevance(text_infragenericEpithet);
        registerCacheRelevance(text_specificEpithet);
        registerCacheRelevance(text_infraspecificEpithet);
        registerCacheRelevance(text_cultivarName);
        registerCacheRelevance(text_subGenusAuthorship);
        registerCacheRelevance(text_breed);

        registerCacheRelevance(toggleable_cache, cacheRelevanceHelper.getDependenciesArray());
	}

    /**
	 * clearCheckRankWarnings
	 */
	protected void clearCheckRankWarnings() {
		if(getEntity().hasProblem(ParserProblem.CheckRank)){
			getEntity().removeParsingProblem(ParserProblem.CheckRank);

			// FIXME this has to be refactored completely. See tickets #1959, #1663, #1957, #1958
//			TaxonNameEditor nameEditor = (TaxonNameEditor) EditorUtil.getActiveEditorPage(Page.NAME);
//			nameEditor.getSelectedContainer().getNameViewer().clearErrors();
		}
	}

	@Override
	public void updateContent() {
		if(getEntity() == null){
			setEntity(TaxonNameFactory.NewNonViralInstance(null));
		}

		super.updateContent();
		if (toggleable_cache != null){
		    toggleable_cache.setCacheEnabled(getEntity().isProtectedNameCache());

    		if (config == null){
    			updateCacheRelevance();
    		}
		}
	}

	private void createSpecificNameParts(ICdmFormElement formElement,
			INonViralName nonViralName, int style) {
		NomenclaturalCode code = nonViralName.getNameType();
		if (code != null){
			switch(code){
			case ICNAFP :
				//nothing do add
			    if (nonViralName.getRank() != null && nonViralName.getRank().isCultivar()){
			        nonViralName.setRank(null);
			    }else{
			        createBotanicalNameParts(formElement, nonViralName, style);
			    }
				break;
			case Fungi :
				if (config == null){
					createFungusNameParts(formElement, nonViralName, style);
				}
				break;
			case ICZN:
				createZoologicalNameParts(formElement, nonViralName, style);
				break;
            case ICVCN:
                //TODO implement
                logger.warn("ICVCN not yet implemented");
                break;
            case ICNP:
            	createBacterialNameParts(formElement, nonViralName, style);
                break;
            case ICNCP:
                if (nonViralName.getRank() != null && !nonViralName.getRank().isCultivar()){
                    nonViralName.setRank(null);
                }
            	createCultivarNameParts(formElement, nonViralName, style);
                break;
            case NonViral:
                //TODO implement
                logger.warn("NonViral not yet implemented");
                break;
            }

		}

	}

	private void createBacterialNameParts(ICdmFormElement formElement, INonViralName nonViralName, int style) {
		IBacterialName bacterialName = (IBacterialName) nonViralName;
		text_subGenusAuthorship = formFactory.createTextWithLabelElement(formElement, "Subgenus Authorship", bacterialName.getSubGenusAuthorship(), style);
//		text_nameApprobation = formFactory.createTextWithLabelElement(formElement, "Name Approbation", bacterialName.getNameApprobation(), style);

	}

	private void createCultivarNameParts(ICdmFormElement formElement, INonViralName nonViralName, int style) {
		ICultivarPlantName cultivarName = (ICultivarPlantName)nonViralName;

		List<Rank> cultivarRanks = getRanks(cultivarName, nonViralName.getNameType());
		combo_rank.setTerms(cultivarRanks);
		combo_rank.setSelection(cultivarName.getRank());

		Rank rank = cultivarName.getRank();
		if (rank != null && (rank.equals(Rank.CULTIVAR()) || rank.equals(Rank.CULTIVARGROUP()) || rank.equals(Rank.GREX_ICNCP()))){
		    text_cultivarGroup = formFactory.createTextWithLabelElement(formElement, "Cultivar Group Epi", cultivarName.getCultivarGroupEpithet(), style);
		}
		if(rank != null && rank.equals(Rank.CULTIVAR())){
		    text_cultivarName = formFactory.createTextWithLabelElement(formElement, "Cultivar Epithet", cultivarName.getCultivarEpithet(), style);
		}

	}

    private List<Rank> getRanks(INonViralName name, NomenclaturalCode code) {
        List<Rank> ranks = new ArrayList<>();
        String availableRanks = PreferencesUtil.getStringValue(PreferencePredicate.AvailableRanks.getKey());
        List<DefinedTermBase> allRanks = CdmStore.getTermManager().createTermListFromString(availableRanks, TermType.Rank);
        for(DefinedTermBase term: allRanks){
		    Rank rank = (Rank)term;
		    if (rank.isCultivar() == code.equals(NomenclaturalCode.ICNCP)){
		        ranks.add(rank);
		    }
		}
		return ranks;
    }

	private void createBotanicalNameParts(ICdmFormElement formElement, INonViralName nonViralName, int style){
		//nothing to do
	}

	private void createFungusNameParts(ICdmFormElement formElement, INonViralName nonViralName, int style){
		IFungusName fungusName= (IFungusName)nonViralName;
		checkbox_anamorphic = formFactory.createCheckbox(formElement, "Anamorphic", fungusName.isAnamorphic(), style);
	}

	private void createZoologicalNameParts(ICdmFormElement formElement, INonViralName nonViralName, int style){
		IZoologicalName zoologicalName = (IZoologicalName) nonViralName;

		if (config == null){
		    text_breed = formFactory.createTextWithLabelElement(formElement, "Breed", zoologicalName.getBreed(), style);
		}
	}

	private void createGenusOrUninomialControls(ICdmFormElement element, INonViralName nonViralName, int style){
		String title = "Genus";
		if(nonViralName.isSupraGeneric()){
			title = "Uninomial";
		}
		text_uninomial = formFactory.createTextWithLabelElement(element, title, nonViralName.getGenusOrUninomial(), style);
	}

	private void createInfragenerericEpithetControls(ICdmFormElement element, INonViralName nonViralName, int style){
	    boolean isIcznOrNull = nonViralName.getNameType() == null || nonViralName.getNameType().equals(NomenclaturalCode.ICZN);
	    if (nonViralName.isInfraGeneric() || nonViralName.isSpecies()
	            || isIcznOrNull && (nonViralName.isInfraSpecific() || nonViralName.isSpeciesAggregate()) //for iczn all subgeneric names may have infrageneric part, but unclear if speciesAggregates exist at all
	            ){
	        text_infragenericEpithet = formFactory.createTextWithLabelElement(element, "Infrageneric Epithet", nonViralName.getInfraGenericEpithet(), style);
	    }
	}

	private void createSpecificEpithetControls(ICdmFormElement element, INonViralName nonViralName, int style){
		if(nonViralName.isSpecies()  || nonViralName.isInfraSpecific() || nonViralName.isSpeciesAggregate()){
			text_specificEpithet = formFactory.createTextWithLabelElement(element, "Specific Epithet", nonViralName.getSpecificEpithet(), SWT.NULL);
		}
	}

	private void createInfraSpecificEpithetControls(ICdmFormElement element, INonViralName nonViralName, int style){
		if(nonViralName.isInfraSpecific()){
			text_infraspecificEpithet = formFactory.createTextWithLabelElement(element, "Infraspecific Epithet", nonViralName.getInfraSpecificEpithet(), SWT.NULL);
		}
	}

	@Override
	public void handleEvent(Object eventSource){
		if (eventSource == toggleable_cache) {
			getEntity().setNameCache(toggleable_cache.getText(),
					toggleable_cache.getState());
                updateCacheRelevance();
		}
		else if(eventSource == combo_rank){
			getEntity().setRank(combo_rank.getSelection());
			clearCheckRankWarnings();
			updateContent();

//			((NameDetailSection)this.getParentElement()).layout(true, true);
            //this is needed for the "new" wizards in order to redraw correctly

		}
		else if(eventSource == text_appendedPhrase){
			getEntity().setAppendedPhrase(text_appendedPhrase.getText());
		}
		else if(eventSource == text_infragenericEpithet){
			getEntity().setInfraGenericEpithet(text_infragenericEpithet.getText());
		}
		else if(eventSource == text_infraspecificEpithet){
			getEntity().setInfraSpecificEpithet(text_infraspecificEpithet.getText());
		}
		else if(eventSource == text_specificEpithet){
			getEntity().setSpecificEpithet(text_specificEpithet.getText());
		}
		else if(eventSource == text_uninomial){
			getEntity().setGenusOrUninomial(text_uninomial.getText());
		}
		else if(eventSource == checkbox_anamorphic){
			((IFungusName)getEntity()).setAnamorphic(checkbox_anamorphic.getSelection());
		}else if(eventSource == text_breed){
            ((IZoologicalName)getEntity()).setBreed(text_breed.getText());
        }else if(eventSource == this.text_cultivarName){
            ((ICultivarPlantName)getEntity()).setCultivarEpithet(text_cultivarName.getText());
        }else if(eventSource == this.text_cultivarGroup){
            ((ICultivarPlantName)getEntity()).setCultivarGroupEpithet(text_cultivarGroup.getText());
        }else if(eventSource == this.text_subGenusAuthorship){
            ((IBacterialName)getEntity()).setSubGenusAuthorship(text_subGenusAuthorship.getText());
        }
	}

	/**
	 * <p>Getter for the field <code>selectionArbitrator</code>.</p>
	 */
	@Override
    public SelectionArbitrator getSelectionArbitrator() {
		return selectionArbitrator;
	}

	@Override
	public void updateToggleableCacheField() {
	    if (toggleable_cache != null){
    		if(! getEntity().isProtectedNameCache()){
    			toggleable_cache.setText(getEntity().getNameCache());
    		}
	    }
	}

	@Override
	protected void handleToggleableCacheField() {
        getEntity().setTitleCache(toggleable_cache.getText(), toggleable_cache.getState());
        updateCacheRelevance();
        updateToggleableCacheField();
    }
}