/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.ui.section.name;

import org.eclipse.jface.wizard.Wizard;

import eu.etaxonomy.cdm.api.conversation.ConversationHolder;
import eu.etaxonomy.cdm.api.conversation.IConversationEnabled;
import eu.etaxonomy.cdm.model.common.CdmBase;
import eu.etaxonomy.cdm.model.name.NameRelationship;
import eu.etaxonomy.cdm.persistence.hibernate.CdmDataChangeMap;
import eu.etaxonomy.taxeditor.session.ICdmEntitySession;

/**
 * <p>NameRelationshipWizard class.</p>
 *
 * @author n.hoffmann
 * @created Jun 1, 2010
 * @version 1.0
 */
public class NameRelationshipWizard extends Wizard implements IConversationEnabled{

	private static NameRelationshipDetailSection callingSection;

	//private ICdmEntitySession cdmEntitySession;
    private ICdmEntitySession previousCdmEntitySession;
    private CdmBase rootElement;

	/**
	 * <p>Constructor for NameRelationshipWizard.</p>
	 *
	 * @param callingSection a {@link eu.etaxonomy.taxeditor.ui.section.name.NameRelationshipDetailSection} object.
	 */
	public NameRelationshipWizard(NameRelationshipDetailSection callingSection) {
		NameRelationshipWizard.callingSection = callingSection;
		rootElement = callingSection.getEntity();
	}

	private NameRelationshipWizardPage page;

	/** {@inheritDoc} */
	@Override
	public void addPages() {
		page = new NameRelationshipWizardPage(callingSection, callingSection.getFormFactory());

		addPage(page);
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.wizard.Wizard#performFinish()
	 */
	/** {@inheritDoc} */
	@Override
	public boolean performFinish() {
		return page.isPageComplete();
	}

	@Override
    public void dispose() {
        super.dispose();

    }

	/**
	 * <p>getNameRelationship</p>
	 *
	 * @return a {@link eu.etaxonomy.cdm.model.name.NameRelationship} object.
	 */
	public NameRelationship getNameRelationship() {
		return page.getNameRelationship();
	}

	/**
	 * <p>getConversationHolder</p>
	 *
	 * @return a {@link eu.etaxonomy.cdm.api.conversation.ConversationHolder} object.
	 */
	@Override
    public ConversationHolder getConversationHolder() {
		return callingSection.getConversationHolder();
	}

	/** {@inheritDoc} */
	@Override
    public void update(CdmDataChangeMap changeEvents) {}


}
