/**
 * Copyright (C) 2007 EDIT
 * European Distributed Institute of Taxonomy
 * http://www.e-taxonomy.eu
 *
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */

package eu.etaxonomy.taxeditor.ui.section.name;

import org.eclipse.jface.util.PropertyChangeEvent;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Composite;

import eu.etaxonomy.cdm.model.name.NameRelationship;
import eu.etaxonomy.cdm.model.name.NameRelationshipType;
import eu.etaxonomy.cdm.model.name.TaxonName;
import eu.etaxonomy.taxeditor.l10n.Messages;
import eu.etaxonomy.taxeditor.ui.AbstractEntityCollectionElementWizardPage;
import eu.etaxonomy.taxeditor.ui.combo.NameRelationshipTypeCombo;
import eu.etaxonomy.taxeditor.ui.element.CdmFormFactory;
import eu.etaxonomy.taxeditor.ui.selection.EntitySelectionElement;

/**
 * <p>
 * NameRelationshipWizardPage class.
 * </p>
 *
 * @author n.hoffmann
 * @created Jun 1, 2010
 * @version 1.0
 */
public class NameRelationshipWizardPage extends AbstractEntityCollectionElementWizardPage {

	private EntitySelectionElement<TaxonName> selection_relatedTo;

	private NameRelationshipTypeCombo combo_relationshipType;

	private final TaxonName entity;

	private NameRelationshipType type;

	private TaxonName relatedName;
	boolean inverse;

	/**
	 * <p>
	 * Constructor for NameRelationshipWizardPage.
	 * </p>
	 *
	 * @param callingSection
	 *            a
	 *            {@link eu.etaxonomy.taxeditor.ui.section.name.NameRelationshipDetailSection}
	 *            object.
	 */
	protected NameRelationshipWizardPage(
			NameRelationshipDetailSection callingSection, CdmFormFactory formFactory) {
		super("NameRelationshipWizardPage", formFactory); //$NON-NLS-1$
		setTitle("New Name Relationship"); //$NON-NLS-1$
		//setDescription(callingSection.getEntity().getTitleCache());
		setDescription(Messages.NameRelationshipWizardPage_description);
		this.entity = callingSection.getEntity();

		formFactory.addPropertyChangeListener(this);
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see
	 * org.eclipse.jface.dialogs.IDialogPage#createControl(org.eclipse.swt.widgets
	 * .Composite)
	 */
	/** {@inheritDoc} */
	@Override
	public void createControl(Composite parent) {
	    super.createControl(parent);

		combo_relationshipType = formFactory.createNameRelationshipTypeCombo(rootElement,
	            entity.getNameCache(), SWT.NULL, false,null);


		selection_relatedTo = formFactory
				.createSelectionElement(TaxonName.class, rootElement,
						"",this.entity, EntitySelectionElement.ALL,
						SWT.NULL, true);

	}

	/**
	 * <p>
	 * getNameRelationship
	 * </p>
	 *
	 * @return a {@link eu.etaxonomy.cdm.model.name.NameRelationship} object.
	 */
	public NameRelationship getNameRelationship() {
	    NameRelationship rel;
	    if (inverse){
	        rel = entity.addRelationshipFromName(relatedName, type, null, null, null, null);
	    }else{
	        rel = entity.addRelationshipToName(relatedName, type, null, null, null, null);
	    }
	    return rel;
	}

	/** {@inheritDoc} */
	@Override
	public void propertyChange(PropertyChangeEvent event) {
		if (event == null) {
			return;
		}
		Object eventSource = event.getSource();
		if (eventSource == combo_relationshipType && combo_relationshipType.getSelection() != null) {
		    type = combo_relationshipType.getSelection().getTerm();
			this.inverse = combo_relationshipType.getSelection().isInverse();
		} else if (eventSource == selection_relatedTo) {
			relatedName = selection_relatedTo.getEntity();

		}

		boolean complete = type != null && relatedName != null;
		setPageComplete(complete);
	}

}
