/**
 * Copyright (C) 2007 EDIT
 * European Distributed Institute of Taxonomy
 * http://www.e-taxonomy.eu
 *
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */

package eu.etaxonomy.taxeditor.ui.section.name;

import org.eclipse.swt.events.SelectionListener;

import eu.etaxonomy.cdm.hibernate.HibernateProxyHelper;
import eu.etaxonomy.cdm.model.name.NameTypeDesignation;
import eu.etaxonomy.cdm.model.name.NameTypeDesignationStatus;
import eu.etaxonomy.cdm.model.name.TaxonName;
import eu.etaxonomy.cdm.model.reference.NamedSource;
import eu.etaxonomy.cdm.model.term.TermType;
import eu.etaxonomy.taxeditor.l10n.Messages;
import eu.etaxonomy.taxeditor.model.MessagingUtils;
import eu.etaxonomy.taxeditor.ui.combo.term.TermComboElement;
import eu.etaxonomy.taxeditor.ui.element.AbstractFormSection;
import eu.etaxonomy.taxeditor.ui.element.CdmFormFactory;
import eu.etaxonomy.taxeditor.ui.element.CheckboxElement;
import eu.etaxonomy.taxeditor.ui.element.ICdmFormElement;
import eu.etaxonomy.taxeditor.ui.selection.EntitySelectionElement;

/**
 *
 * @author n.hoffmann
 * @created May 17, 2010
 * @version 1.0
 */
public class NameTypeDesignationElement extends
		AbstractTypeDesignationElement<NameTypeDesignation> {

	private CheckboxElement checkbox_rejectedType;
	private CheckboxElement checkbox_conservedType;
	private EntitySelectionElement<TaxonName> selection_typeName;
	private TermComboElement<NameTypeDesignationStatus> combo_typeStatus;

	public NameTypeDesignationElement(CdmFormFactory formFactory,
			AbstractFormSection section, NameTypeDesignation entity,
			SelectionListener removeListener, int style) {
		super(formFactory, section, entity, removeListener, style);
	}

	/** {@inheritDoc} */
	@Override
	public void createControls(ICdmFormElement formElement, int style) {

		checkbox_rejectedType = formFactory.createCheckbox(formElement,
				"Rejected Type", false, style); //$NON-NLS-1$
		checkbox_conservedType = formFactory.createCheckbox(formElement,
				"Conserved Type", false, style); //$NON-NLS-1$
		selection_typeName = formFactory
				.createSelectionElement(TaxonName.class,//getConversationHolder(),
				        formElement, "Name", null, //$NON-NLS-1$
						EntitySelectionElement.ALL, style);
		combo_typeStatus = formFactory.createDefinedTermComboElement(TermType.NameTypeDesignationStatus, formElement,
						"Designation Status", null, style); //$NON-NLS-1$

		super.createControls(formElement, style);
		if (entity != null){
			setEntity(entity);
		}
	}

	/** {@inheritDoc} */
	@Override
	public void setEntity(NameTypeDesignation entity) {
		super.setEntity(entity);
		if (checkbox_conservedType != null){
			checkbox_conservedType.setSelection(entity.isConservedType());
			checkbox_rejectedType.setSelection(entity.isRejectedType());
			selection_typeName.setEntity(HibernateProxyHelper.deproxy(entity.getTypeName()));
			combo_typeStatus.setSelection(HibernateProxyHelper.deproxy(entity.getTypeStatus(), NameTypeDesignationStatus.class));
			checkbox_notDesignated.setSelection(entity.isNotDesignated());
		}

	}

	@Override
	public void handleEvent(Object eventSource) {
		if (eventSource == checkbox_conservedType) {
			getEntity().setConservedType(checkbox_conservedType.getSelection());
		} else if (eventSource == checkbox_rejectedType) {
			getEntity().setRejectedType(checkbox_rejectedType.getSelection());
		} else if (eventSource == selection_typeName) {
			getEntity().setTypeName(selection_typeName.getEntity());
		} else if (eventSource == combo_typeStatus) {
		    if ((combo_typeStatus.getSelection() == null && entity.getCitation() != null)){
		        boolean ok = MessagingUtils.confirmDialog(Messages.NameTypeDesignationElement_4, Messages.NameTypeDesignationElement_5);
                if (!ok){
                    combo_typeStatus.setSelection(entity.getTypeStatus());
                    return;
                }
		    }else if ( entity.getTypeStatus() != null && getEntity().getTypeStatus().hasDesignationSource() && !combo_typeStatus.getSelection().hasDesignationSource() && entity.getCitation() != null){
		        boolean ok = MessagingUtils.confirmDialog(Messages.NameTypeDesignationElement_4, Messages.NameTypeDesignationElement_5);
                if (!ok){
                    combo_typeStatus.setSelection(entity.getTypeStatus());
                    return;
                }
            }
			getEntity().setTypeStatus(combo_typeStatus.getSelection());
			if (getEntity().hasDesignationSource()){
			    NamedSource source = NamedSource.NewPrimarySourceInstance(null, null);
                entity.setDesignationSource(source);
			}

			setEntity(entity);
		} else if (eventSource == checkbox_notDesignated) {
			getEntity().setNotDesignated(checkbox_notDesignated.getSelection());
		}
//		else if (eventSource == selection_reference) {
//			getEntity().setCitation(selection_reference.getSelection());
//		} else if (eventSource == text_referenceDetail) {
//			getEntity().setCitationMicroReference(
//					text_referenceDetail.getText());
//		}
	}
}
