/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.ui.section.name;

import java.util.Collection;
import java.util.Comparator;

import eu.etaxonomy.cdm.api.conversation.ConversationHolder;
import eu.etaxonomy.cdm.hibernate.HibernateProxyHelper;
import eu.etaxonomy.cdm.model.name.NomenclaturalStatus;
import eu.etaxonomy.cdm.model.name.TaxonName;
import eu.etaxonomy.cdm.model.taxon.TaxonBase;
import eu.etaxonomy.taxeditor.ui.element.CdmFormFactory;
import eu.etaxonomy.taxeditor.ui.element.ICdmFormElement;
import eu.etaxonomy.taxeditor.ui.section.AbstractEntityCollectionSection;
import eu.etaxonomy.taxeditor.ui.section.DefaultCdmBaseComparator;
import eu.etaxonomy.taxeditor.ui.section.ITaxonBaseDetailSection;

/**
 * <p>NomenclaturalStatusSection class.</p>
 *
 * @author n.hoffmann
 */
public class NomenclaturalStatusSection extends AbstractEntityCollectionSection<TaxonName, NomenclaturalStatus> implements ITaxonBaseDetailSection{

	private TaxonBase taxonBase;

	/**
	 * <p>Constructor for NomenclaturalStatusSection.</p>
	 *
	 * @param parentElement a {@link eu.etaxonomy.taxeditor.ui.element.ICdmFormElement} object.
	 * @param style a int.
	 * @param cdmFormFactory a {@link eu.etaxonomy.taxeditor.ui.element.CdmFormFactory} object.
	 * @param conversation a {@link eu.etaxonomy.cdm.api.conversation.ConversationHolder} object.
	 */
	public NomenclaturalStatusSection(CdmFormFactory cdmFormFactory, ConversationHolder conversation, ICdmFormElement parentElement,
			int style) {
		super(cdmFormFactory, conversation, parentElement, "Nomenclatural Status", style);
	}

	/** {@inheritDoc} */
	@Override
	public Collection<NomenclaturalStatus> getCollection(TaxonName entity) {
		return entity.getStatus();
	}

	@Override
	public Comparator<NomenclaturalStatus> getComparator() {
        return new DefaultCdmBaseComparator<>();
	}

	/** {@inheritDoc} */
	@Override
	public String getEmptyString() {
		return "No nomenclatural status yet.";
	}

	/** {@inheritDoc} */
	@Override
	public NomenclaturalStatus createNewElement() {
		return NomenclaturalStatus.NewInstance(null);
	}

	/** {@inheritDoc} */
	@Override
	protected String getTooltipString() {
		return "Create a new nomenclatural status";
	}

	/** {@inheritDoc} */
	@Override
	public void addElement(NomenclaturalStatus element) {
		getEntity().addStatus(element);
	}

	/** {@inheritDoc} */
	@Override
	public void removeElement(NomenclaturalStatus element) {
		getEntity().removeStatus(element);
	}

	/** {@inheritDoc} */
	@Override
    public void setTaxonBase(TaxonBase entity) {
		this.taxonBase = entity;
		TaxonName nonViralName = HibernateProxyHelper.deproxy(entity.getName());
		setEntity(nonViralName);
	}

	@Override
	public TaxonBase getTaxonBase() {
		return taxonBase;
	}

    /**
     * {@inheritDoc}
     */
    @Override
    public NomenclaturalStatus addExisting() {
        return null;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public boolean allowAddExisting() {
        return false;
    }

}
