/**
* Copyright (C) 2019 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.ui.section.name.operation;

import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.commands.operations.IUndoContext;
import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;

import eu.etaxonomy.cdm.api.service.INameService;
import eu.etaxonomy.cdm.model.name.TaxonName;
import eu.etaxonomy.cdm.model.name.TypeDesignationBase;
import eu.etaxonomy.taxeditor.operation.AbstractPostOperation;
import eu.etaxonomy.taxeditor.operation.IPostOperationEnabled;
import eu.etaxonomy.taxeditor.session.ICdmEntitySessionEnabled;
import eu.etaxonomy.taxeditor.store.CdmStore;

/**
 * @author pplitzner
 * @since Apr 10, 2019
 */
public class DeleteTypeDesignationOperation extends AbstractPostOperation<TaxonName> {

    private final TypeDesignationBase<?> typeDesignation;

    public DeleteTypeDesignationOperation(String label, IUndoContext undoContext, TaxonName taxonName,
            TypeDesignationBase<?> typeDesignation, IPostOperationEnabled postOperationEnabled,
            ICdmEntitySessionEnabled<TaxonName> cdmEntitySessionEnabled) {
        super(label, undoContext, taxonName, postOperationEnabled, cdmEntitySessionEnabled);
        this.typeDesignation = typeDesignation;
    }

    @Override
    public IStatus execute(IProgressMonitor monitor, IAdaptable info) throws ExecutionException {
        monitor.worked(20);
        if (typeDesignation != null && typeDesignation.isPersited()){
            if(element==null){
                CdmStore.getService(INameService.class).deleteTypeDesignation(null, typeDesignation);
            }
            else{
                CdmStore.getService(INameService.class).deleteTypeDesignation(element.getUuid(), typeDesignation.getUuid());
            }
            return postExecute(element);
        }
        return null;
    }

    @Override
    public IStatus redo(IProgressMonitor monitor, IAdaptable info) throws ExecutionException {
        return execute(monitor, info);
    }

    @Override
    public IStatus undo(IProgressMonitor monitor, IAdaptable info) throws ExecutionException {
        element.removeTypeDesignation(typeDesignation);
        return postExecute(element);
    }
}