/**
 * Copyright (C) 2019 EDIT
 * European Distributed Institute of Taxonomy
 * http://www.e-taxonomy.eu
 *
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */
package eu.etaxonomy.taxeditor.ui.section.name.type;

import org.eclipse.jface.resource.JFaceResources;
import org.eclipse.jface.viewers.ComboViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;

import eu.etaxonomy.cdm.common.URI;
import eu.etaxonomy.cdm.model.term.TermType;
import eu.etaxonomy.taxeditor.l10n.Messages;
import eu.etaxonomy.taxeditor.model.ImageResources;
import eu.etaxonomy.taxeditor.ui.combo.term.TermUuidComboViewer;

/**
 * @author pplitzner
 * @since Mar 13, 2019
 *
 */
public class CloneTypeWizardComposite extends Composite implements ModifyListener{
    private Text txtAccNumber;
    private Text txtCollection;
    private Button btnBrowseCollection;
    private TermUuidComboViewer comboTypeStatus;
    private ComboViewer comboViewerBaseType;
    private Text textBarcode;
    private Text textCatalogNumber;
    private Text textStableIdentifier;
    private Label labelException;
    private URI stableIdentifier;

    /**
     * Create the composite.
     * @param parent
     * @param style
     */
    public CloneTypeWizardComposite(Composite parent, int style) {
        super(parent, style);
        setLayout(new GridLayout(3, false));

        Label lblNewLabel_3 = new Label(this, SWT.NONE);
        lblNewLabel_3.setLayoutData(new GridData(SWT.RIGHT, SWT.CENTER, false, false, 1, 1));
        lblNewLabel_3.setText("Base Type");

        comboViewerBaseType = new ComboViewer(this, SWT.READ_ONLY);
        Combo combo = comboViewerBaseType.getCombo();
        combo.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 2, 1));

        Label lblNewLabel_1 = new Label(this, SWT.NONE);
        lblNewLabel_1.setLayoutData(new GridData(SWT.RIGHT, SWT.CENTER, false, false, 1, 1));
        lblNewLabel_1.setText("Type Status");

        comboTypeStatus = new TermUuidComboViewer(this, SWT.NONE);
        comboTypeStatus.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 2, 1));
        comboTypeStatus.setInput(TermType.SpecimenTypeDesignationStatus);


        Label lblNewLabel_2 = new Label(this, SWT.NONE);
        lblNewLabel_2.setLayoutData(new GridData(SWT.RIGHT, SWT.CENTER, false, false, 1, 1));
        lblNewLabel_2.setText("Collection");

        txtCollection = new Text(this, SWT.BORDER);
        txtCollection.setEnabled(false);
        txtCollection.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));

        btnBrowseCollection = new Button(this, SWT.NONE);
        btnBrowseCollection.setImage(ImageResources.getImage(ImageResources.BROWSE_ICON));

        Label lblNewLabel = new Label(this, SWT.NONE);
        lblNewLabel.setLayoutData(new GridData(SWT.RIGHT, SWT.CENTER, false, false, 1, 1));
        lblNewLabel.setText("Accession Number");

        txtAccNumber = new Text(this, SWT.BORDER);
        txtAccNumber.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 2, 1));

        Label lblNewLabel_4 = new Label(this, SWT.NONE);
        lblNewLabel_4.setLayoutData(new GridData(SWT.RIGHT, SWT.CENTER, false, false, 1, 1));
        lblNewLabel_4.setText("Barcode");

        textBarcode = new Text(this, SWT.BORDER);
        textBarcode.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 2, 1));

        Label lblNewLabel_5 = new Label(this, SWT.NONE);
        lblNewLabel_5.setLayoutData(new GridData(SWT.RIGHT, SWT.CENTER, false, false, 1, 1));
        lblNewLabel_5.setText("Catalog Number");

        textCatalogNumber = new Text(this, SWT.BORDER);
        textCatalogNumber.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 2, 1));


        Label lblNewLabel_stableIdentifier = new Label(this, SWT.NONE);
        lblNewLabel_stableIdentifier.setLayoutData(new GridData(SWT.RIGHT, SWT.CENTER, false, false, 1, 1));
        lblNewLabel_stableIdentifier.setText("Stable Identifier");

        textStableIdentifier = new Text(this, SWT.BORDER);
        textStableIdentifier.setEnabled(true);
        textStableIdentifier.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 2, 1));
        textStableIdentifier.addModifyListener(this);


    }

    @Override
    protected void checkSubclass() {
        // Disable the check that prevents subclassing of SWT components
    }

    public Button getBtnBrowseCollection() {
        return btnBrowseCollection;
    }
    public TermUuidComboViewer getComboTypeStatus() {
        return comboTypeStatus;
    }
    public Text getTxtCollection() {
        return txtCollection;
    }
    public ComboViewer getComboViewerBaseType() {
        return comboViewerBaseType;
    }
    public Text getTxtAccNumber() {
        return txtAccNumber;
    }
    public Text getTextBarcode() {
        return textBarcode;
    }
    public Text getTextCatalogNumber() {
        return textCatalogNumber;
    }

    public URI getStableIdentifier() {
        return stableIdentifier;
    }


    @Override
    public void modifyText(ModifyEvent e) {
        if (e.getSource().equals(textStableIdentifier)){
            try{
                stableIdentifier = URI.create(textStableIdentifier.getText());
                if (labelException != null){
                    labelException.setText("");
                }
            }catch(IllegalArgumentException ex){
                if (labelException == null){
                    labelException = new Label(this, SWT.WRAP); //$NON-NLS-1$
                    labelException.setBackground(this.getBackground());
                    labelException.setFont(JFaceResources.getFontRegistry().getBold(JFaceResources.DEFAULT_FONT));
                    labelException.setForeground(Display.getCurrent().getSystemColor(SWT.COLOR_RED));
                    labelException.requestLayout();
                    labelException.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 3, 1));
                }
                labelException.setText(Messages.UriWithLabelElement_URL_NOT_SAVED + ex.getLocalizedMessage());

                stableIdentifier = null;
            }
        }

    }
}
