/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.ui.section.occurrence;

import java.util.Collection;
import java.util.Comparator;

import eu.etaxonomy.cdm.api.conversation.ConversationHolder;
import eu.etaxonomy.cdm.api.facade.DerivedUnitFacade;
import eu.etaxonomy.cdm.model.location.NamedArea;
import eu.etaxonomy.cdm.model.occurrence.FieldUnit;
import eu.etaxonomy.taxeditor.ui.dialog.selection.NamedAreaSelectionDialog;
import eu.etaxonomy.taxeditor.ui.element.CdmFormFactory;
import eu.etaxonomy.taxeditor.ui.element.ICdmFormElement;
import eu.etaxonomy.taxeditor.ui.section.AbstractEntityCollectionSection;
import eu.etaxonomy.taxeditor.ui.section.DefaultCdmBaseComparator;

/**
 * @author n.hoffmann
 * @created Oct 14, 2010
 */
public class CollectingAreasDetailSection extends AbstractEntityCollectionSection<DerivedUnitFacade, NamedArea> {

	/**
	 * <p>Constructor for CollectingAreasDetailSection.</p>
	 *
	 * @param formFactory a {@link eu.etaxonomy.taxeditor.ui.element.CdmFormFactory} object.
	 * @param conversation a {@link eu.etaxonomy.cdm.api.conversation.ConversationHolder} object.
	 * @param parentElement a {@link eu.etaxonomy.taxeditor.ui.element.ICdmFormElement} object.
	 * @param style a int.
	 */
	public CollectingAreasDetailSection(CdmFormFactory formFactory,
			ConversationHolder conversation, ICdmFormElement parentElement, int style) {
		super(formFactory, conversation, parentElement, "Collecting Areas", style);
	}

	@Override
	public Collection<NamedArea> getCollection(DerivedUnitFacade entity) {
		return entity.getCollectingAreas();
	}

	@Override
	public Comparator<NamedArea> getComparator() {
        return new DefaultCdmBaseComparator<>();
	}

	@Override
	public NamedArea createNewElement() {
		NamedArea selection = NamedAreaSelectionDialog.select(getLayoutComposite().getShell(), //getConversationHolder(),
		        null, FieldUnit.class.getCanonicalName());

		return selection;
	}

	@Override
	public void addElement(NamedArea element) {
		getEntity().addCollectingArea(element);
	}

	@Override
	public void removeElement(NamedArea element) {
		getEntity().removeCollectingArea(element);
	}

	@Override
	public String getEmptyString() {
		return "No collecting areas yet.";
	}

	@Override
	protected String getTooltipString() {
		return "Add a collecting area";
	}

    @Override
    public NamedArea addExisting() {
        return null;
    }

    @Override
    public boolean allowAddExisting() {
        return false;
    }
}
