/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.ui.section.occurrence;

import eu.etaxonomy.cdm.api.facade.DerivedUnitFacade;
import eu.etaxonomy.taxeditor.preference.PreferencesUtil;
import eu.etaxonomy.taxeditor.store.CdmStore;
import eu.etaxonomy.taxeditor.ui.element.CdmFormFactory;
import eu.etaxonomy.taxeditor.ui.element.ICdmFormElement;
import eu.etaxonomy.taxeditor.ui.element.TextWithLabelElement;

/**
 * <p>FieldUnitDetailElement class.</p>
 *
 * @author n.hoffmann
 * @created Jun 17, 2010
 * @version 1.0
 */
public class FieldUnitFacadeDetailElement extends AbstractSpecimenOrObservationDetailElement {

	private TextWithLabelElement text_ecology;

	private TextWithLabelElement text_plantDescription;

	private TextWithLabelElement text_fieldObjectDefinition;

	private TextWithLabelElement text_fieldNotes;

	private TextWithLabelElement text_lifeForm;

	private TextWithLabelElement number_individualCount;

//	private TermComboElement<DefinedTerm> combo_kindOfUnit;
//
//	private TermComboElement<DefinedTerm> combo_lifeStage;
//
//	private TermComboElement<DefinedTerm> combo_sex;

	/**
	 * <p>Constructor for FieldUnitDetailElement.</p>
	 *
	 * @param formFactory a {@link eu.etaxonomy.taxeditor.ui.element.CdmFormFactory} object.
	 * @param formElement a {@link eu.etaxonomy.taxeditor.ui.element.ICdmFormElement} object.
	 */
	public FieldUnitFacadeDetailElement(CdmFormFactory formFactory,
			ICdmFormElement formElement) {
		super(formFactory, formElement);
	}

	/** {@inheritDoc} */
	@Override
	protected void createControls(ICdmFormElement formElement,
			DerivedUnitFacade entity, int style) {
		text_ecology = formFactory.createTextWithLabelElement(formElement, "Ecology", entity.getEcology(CdmStore.getDefaultLanguage()), style);
		text_plantDescription = formFactory.createTextWithLabelElement(formElement, "Plant Description", entity.getPlantDescription(CdmStore.getDefaultLanguage()), style);
		// Disable for now
//		text_fieldObjectDefinition = formFactory.createTextWithLabelElement(formElement, "Field Object Definition", entity.getFieldObjectDefinition(CdmStore.getDefaultLanguage()), style);
		text_fieldNotes = formFactory.createTextWithLabelElement(formElement, "Field Notes", entity.getFieldNotes(), style);
		if (PreferencesUtil.isShowLifeForm()){
			text_lifeForm = formFactory.createTextWithLabelElement(formElement, "Life Form", entity.getLifeform(PreferencesUtil.getGlobalLanguage()), style);
		}

		number_individualCount = formFactory.createTextWithLabelElement(formElement, "Individual Count", entity.getIndividualCount(), style);
		super.createControls(formElement, entity, style);
	}

	/** {@inheritDoc} */
	@Override
	public void handleEvent(Object eventSource) {
		super.handleEvent(eventSource);
		if(eventSource == text_ecology){
			getEntity().setEcology(text_ecology.getText());
		}
		else if(eventSource == text_plantDescription){
			getEntity().setPlantDescription(text_plantDescription.getText());
		}
		else if(eventSource == text_fieldObjectDefinition){
			getEntity().addFieldObjectDefinition(text_fieldObjectDefinition.getText(), CdmStore.getDefaultLanguage());
		}
		else if(eventSource == text_fieldNotes){
			getEntity().setFieldNotes(text_fieldNotes.getText());
		}
		else if(eventSource == text_lifeForm){
			getEntity().setLifeform(text_lifeForm.getText());
		}
		else if(eventSource == number_individualCount){
			getEntity().setIndividualCount(number_individualCount.getText());
		}

	}
}
