/**
* Copyright (C) 2013 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.ui.section.occurrence.derivedUnit;

import eu.etaxonomy.cdm.model.occurrence.Collection;
import eu.etaxonomy.cdm.model.occurrence.DerivedUnit;
import eu.etaxonomy.taxeditor.ui.element.CdmFormFactory;
import eu.etaxonomy.taxeditor.ui.element.ICdmFormElement;
import eu.etaxonomy.taxeditor.ui.element.TextWithLabelElement;
import eu.etaxonomy.taxeditor.ui.element.ToggleableTextElement;
import eu.etaxonomy.taxeditor.ui.section.AbstractCdmDetailElement;
import eu.etaxonomy.taxeditor.ui.selection.EntitySelectionElement;

/**
 * @author pplitzner
 * @date 17.12.2013
 *
 */
public class PreservedSpecimenGeneralDetailElement extends AbstractCdmDetailElement<DerivedUnit> {

    private ToggleableTextElement toggleableText_titleCache;
    private EntitySelectionElement<Collection> selection_collection;
    private TextWithLabelElement text_accessionNumber;


    /**
     * @param formFactory
     * @param formElement
     */
    public PreservedSpecimenGeneralDetailElement(CdmFormFactory formFactory,
            ICdmFormElement formElement) {
        super(formFactory, formElement);
    }

    /*
     * (non-Javadoc)
     *
     * @see
     * eu.etaxonomy.taxeditor.section.AbstractCdmDetailElement#createControls
     * (eu.etaxonomy.taxeditor.forms.ICdmFormElement, java.lang.Object, int)
     */
    @Override
    protected void createControls(ICdmFormElement formElement,
            DerivedUnit entity, int style) {
        toggleableText_titleCache = formFactory.createToggleableTextField(
                formElement, "Title Cache", entity.getTitleCache(),
                entity.isProtectedTitleCache(), style);
        selection_collection = formFactory
                .createSelectionElement(Collection.class,//getConversationHolder(),
                        formElement, "Collection",
                        entity.getCollection(),
                        EntitySelectionElement.ALL, style);
        text_accessionNumber = formFactory.createTextWithLabelElement(
                formElement, "Accession Number", entity.getAccessionNumber(),
                style);
    }

    /*
     * (non-Javadoc)
     *
     * @see
     * eu.etaxonomy.taxeditor.section.AbstractCdmDetailElement#handleEvent(java
     * .lang.Object)
     */
    @Override
    public void handleEvent(Object eventSource) {
        if (eventSource == toggleableText_titleCache) {
            getEntity().setTitleCache(toggleableText_titleCache.getText(),
                    toggleableText_titleCache.getState());
        } else if (eventSource == selection_collection) {
            getEntity().setCollection(selection_collection.getSelection());
        } else if (eventSource == text_accessionNumber) {
            getEntity().setAccessionNumber(text_accessionNumber.getText());
        }
        if (eventSource != toggleableText_titleCache) {
            toggleableText_titleCache.setText(getEntity().getTitleCache());
        }
    }

}
