/**
* Copyright (C) 2020 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.ui.section.reference;

import org.eclipse.swt.widgets.Label;

import eu.etaxonomy.cdm.hibernate.HibernateProxyHelper;
import eu.etaxonomy.cdm.model.common.CdmBase;
import eu.etaxonomy.cdm.model.description.DescriptionElementSource;
import eu.etaxonomy.cdm.model.name.NomenclaturalSource;
import eu.etaxonomy.cdm.model.name.TaxonName;
import eu.etaxonomy.cdm.model.reference.Reference;
import eu.etaxonomy.taxeditor.model.AbstractUtility;
import eu.etaxonomy.taxeditor.preference.Resources;
import eu.etaxonomy.taxeditor.store.StoreUtil;
import eu.etaxonomy.taxeditor.ui.element.CdmFormFactory;
import eu.etaxonomy.taxeditor.ui.element.ICdmFormElement;
import eu.etaxonomy.taxeditor.ui.element.LayoutConstants;
import eu.etaxonomy.taxeditor.ui.section.common.ExternalLinksSection;
import eu.etaxonomy.taxeditor.ui.selection.EntitySelectionElement;
import eu.etaxonomy.taxeditor.ui.selection.EntitySelectionElementWithAbbreviatedTitle;

/**
 * @author k.luther
 * @since Nov 11, 2020
 */
public class NomenclaturalSourceElement extends OriginalSourceElement<NomenclaturalSource> {

    private EntitySelectionElementWithAbbreviatedTitle<Reference> selection_NomRef;
    protected EntitySelectionElement<TaxonName> select_nameUsedInSource;
    

    public NomenclaturalSourceElement(CdmFormFactory formFactory, ICdmFormElement formElement, CdmBase cdmEntity,
            String label) {
        super(formFactory, formElement, cdmEntity, label);
    }

    @Override
    protected void createControls(ICdmFormElement formElement, NomenclaturalSource entity,  int style) {

        entity = HibernateProxyHelper.deproxy(entity);

        selection_NomRef = formFactory.createSelectionElementWithAbbreviatedTitle(Reference.class,
                getConversationHolder(), formElement, "Reference",
                null,
                EntitySelectionElement.ALL, style);

        if (entity != null){
            selection_NomRef.setEntity(entity.getCitation());
        }
        selection_NomRef.setBackground(this.getPersistentBackground());
        for (ICdmFormElement element: selection_NomRef.getElements()){
            element.setBackground(getPersistentBackground());
        }

        microReference = formFactory.createTextWithLabelElement(formElement, "Details", entity != null? entity.getCitationMicroReference(): "", style);

        Label sep = formFactory.createLabel(formElement.getLayoutComposite(), "");
        sep.setLayoutData(LayoutConstants.FILL_HORIZONTALLY(2, 1));

        select_nameUsedInSource = formFactory.createSelectionElement(TaxonName.class, formElement, "Original Spelling", entity != null? entity.getNameUsedInSource(): null, EntitySelectionElement.ALL, style);

        externalLinks = formFactory.createExternalLinksSection(getConversationHolder(), formElement, "Protologue/Original Publication", false, StoreUtil.getSectionStyle(ExternalLinksSection.class, DescriptionElementSource.class.getCanonicalName()));
        externalLinks.setEmptySectionString("No protologue yet.");
        externalLinks.setLayoutData(LayoutConstants.FILL_HORIZONTALLY(2, 3));
        externalLinks.setEntity(entity);
        externalLinks.setFont(AbstractUtility.getFont(Resources.COLOR_FONT_DEFAULT));


        addControl(externalLinks);
        addElement(externalLinks);

//        advancedSection = formFactory.createOriginalSourceAdvancedSection(getConversationHolder(), formElement, null, StoreUtil.getSectionStyle(OriginalSourceAdvancedSection.class, INomenclaturalReference.class.getCanonicalName()));
//        TableWrapData layoutData = LayoutConstants.FILL_HORIZONTALLY(2, 1);
//
//        advancedSection.setLayoutData(layoutData);
//        advancedSection.setBackground(this.getPersistentBackground());
//        advancedSection.setEntity(entity);
//        advancedSection.setFont(AbstractUtility.getFont(Resources.COLOR_FONT_DEFAULT));
//        addControl(advancedSection);
//        addElement(advancedSection);
        this.getLayoutComposite().layout();
    }

    @Override
    public EntitySelectionElement<Reference> getReferenceSelectionElement(){
        return selection_NomRef;
    }

    @Override
    public void handleEvent(Object eventSource) {
        if (eventSource.equals(selection_NomRef)){
            getEntity().setCitation(selection_NomRef.getEntity());
        }
        if (eventSource.equals(microReference)){
            getEntity().setCitationMicroReference(microReference.getText());
        }
        if (eventSource.equals(select_nameUsedInSource)){
            (getEntity()).setNameUsedInSource(select_nameUsedInSource.getEntity());
        }
        super.handleEvent(eventSource);
    }
}