/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.ui.section.supplemental;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.widgets.Text;

import eu.etaxonomy.cdm.common.CdmUtils;
import eu.etaxonomy.cdm.model.common.CdmBase;
import eu.etaxonomy.cdm.model.common.IdentifiableEntity;
import eu.etaxonomy.cdm.model.metadata.PreferencePredicate;
import eu.etaxonomy.cdm.model.reference.ICdmTarget;
import eu.etaxonomy.cdm.model.reference.OriginalSourceBase;
import eu.etaxonomy.cdm.model.reference.OriginalSourceType;
import eu.etaxonomy.cdm.model.reference.Reference;
import eu.etaxonomy.taxeditor.preference.PreferencesUtil;
import eu.etaxonomy.taxeditor.ui.combo.EnumComboElement;
import eu.etaxonomy.taxeditor.ui.combo.OriginalSourceTypeComparator;
import eu.etaxonomy.taxeditor.ui.element.AbstractFormSection;
import eu.etaxonomy.taxeditor.ui.element.CdmFormFactory;
import eu.etaxonomy.taxeditor.ui.element.ICdmFormElement;
import eu.etaxonomy.taxeditor.ui.element.SelectionArbitrator;
import eu.etaxonomy.taxeditor.ui.element.TextWithLabelElement;
import eu.etaxonomy.taxeditor.ui.section.AbstractEntityCollectionElement;
import eu.etaxonomy.taxeditor.ui.section.common.ExternalLinksSection;
import eu.etaxonomy.taxeditor.ui.selection.EntitySelectionElement;

/**
 * @author n.hoffmann
 * @created Mar 16, 2010
 */
public abstract class AbstractOriginalSourceElement<T extends OriginalSourceBase> extends AbstractEntityCollectionElement<T> {

    private SelectionArbitrator selectionArbitrator;

    protected boolean isCommonNameReference = false;
    protected EntitySelectionElement<Reference> selection_reference;
    protected TextWithLabelElement text_referenceDetail;
	protected EnumComboElement<OriginalSourceType> combo_origsourcetype;
	protected TextWithLabelElement text_idInSource;
	protected TextWithLabelElement text_idNamespace;
    protected TextWithLabelElement text_originaleNameString;
    protected TextWithLabelElement text_cdmsource;

    protected ExternalLinksSection externalLinks;

	public AbstractOriginalSourceElement(CdmFormFactory formFactory,
			AbstractFormSection section,
			T element, SelectionListener removeListener,
			int style, boolean isCommonNameReference) {
	    super(formFactory, section, element, removeListener, null, style);
		if(formFactory.getSelectionProvider() != null){
            selectionArbitrator = formFactory.createSelectionArbitrator(this);
        }
        this.isCommonNameReference = isCommonNameReference;
	}

	public AbstractOriginalSourceElement(CdmFormFactory formFactory,
            AbstractFormSection section,
            T element, SelectionListener removeListener,
            int style) {
	    this(formFactory, section, element, removeListener, style, false);
    }

	@Override
	public void setEntity(T entity) {
		this.entity = entity;
		if (selection_reference != null){
		    selection_reference.setEntity(entity.getCitation());
	        text_referenceDetail.setText(entity.getCitationMicroReference());
	        text_originaleNameString.setText(entity.getOriginalNameString());
	        ICdmTarget cdmSource = entity.getCdmSource();
	        String source = "";
	        if(cdmSource instanceof IdentifiableEntity) {
	            source = ((IdentifiableEntity) cdmSource).getTitleCache();
	        } else if(cdmSource instanceof CdmBase) {
	            source += CdmUtils.isNotBlank(source)?"\nUUID: ":"";
	            source += ((CdmBase) cdmSource).getUuid().toString();
	        } else if(cdmSource!=null){
	            source = cdmSource.toString();
	        }
	        text_cdmsource.setText(source);
	        externalLinks.setEntity(entity);
		}
	}

	/**
	 * @wbp.parser.entryPoint*/
	@Override
	public void createControls(ICdmFormElement formElement, int style) {
	    if (isCommonNameReference && PreferencesUtil.getFilterCommonNameReferences()){
          selection_reference = formFactory
                    .createCommonNameReferenceSelectionElement(formElement, "Reference",
                            null, EntitySelectionElement.ALL, style);
        }else{
           selection_reference = formFactory
                      .createSelectionElement(Reference.class,//getConversationHolder(),
                               formElement, "Reference",
                              null, EntitySelectionElement.ALL, style);
        }

        text_referenceDetail = formFactory.createTextWithLabelElement(
                  formElement, "Reference Detail", null, SWT.NULL);

		combo_origsourcetype = formFactory
				.createEnumComboElement(OriginalSourceType.class,
						formElement, new OriginalSourceTypeComparator(getEntity()), style, false);

		if (PreferencesUtil.getBooleanValue(PreferencePredicate.ShowIdInSource.getKey())){
		    text_idInSource = formFactory.createTextWithLabelElement(formElement, "ID in Source", null, style);
		}
		if (PreferencesUtil.getBooleanValue(PreferencePredicate.ShowNamespaceInSource.getKey())){
		    text_idNamespace = formFactory.createTextWithLabelElement(formElement, "ID Namespace", null, style);
		}
		text_originaleNameString = formFactory.createTextWithLabelElement(
		        formElement, "Original Information", null, SWT.NULL);
		text_cdmsource = formFactory.createTextWithLabelElement(
		        formElement, "CDM Source", null, SWT.NULL);
		((Text) text_cdmsource.getMainControl()).setEditable(false);
		
		if (entity != null){			
			selection_reference.setEntity(entity.getCitation());
	        text_referenceDetail.setText(entity.getCitationMicroReference());
	        text_originaleNameString.setText(entity.getOriginalNameString());
	        ICdmTarget cdmSource = entity.getCdmSource();
	        String source = "";
	        if(cdmSource instanceof IdentifiableEntity) {
	            source = ((IdentifiableEntity) cdmSource).getTitleCache();
	        } else if(cdmSource instanceof CdmBase) {
	            source += CdmUtils.isNotBlank(source)?"\nUUID: ":"";
	            source += ((CdmBase) cdmSource).getUuid().toString();
	        } else if(cdmSource!=null){
	            source = cdmSource.toString();
	        }
	        text_cdmsource.setText(source);
	        if (externalLinks != null){
	        	externalLinks.setEntity(entity);
	        }
		}
	}
}
