/**
* Copyright (C) 2020 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.ui.section.supplemental;

import eu.etaxonomy.cdm.model.common.SingleSourcedEntityBase;
import eu.etaxonomy.cdm.model.reference.NamedSource;
import eu.etaxonomy.taxeditor.ui.element.CdmFormFactory;
import eu.etaxonomy.taxeditor.ui.element.ICdmFormElement;
import eu.etaxonomy.taxeditor.ui.element.ISelectableElement;
import eu.etaxonomy.taxeditor.ui.section.AbstractCdmDetailElement;
import eu.etaxonomy.taxeditor.ui.section.reference.OriginalSourceElement;

/**
 * @author k.luther
 * @since Aug 18, 2020
 */
public abstract class AbstractSingleSourceElement <T extends SingleSourcedEntityBase>
            extends AbstractCdmDetailElement<T> implements ISelectableElement{

    protected OriginalSourceElement singleSourceSection;
    private String label;

    /**
     * @param formFactory
     * @param section
     * @param entity
     * @param removeListener
     * @param backgroundColor
     * @param style
     */
    public AbstractSingleSourceElement(CdmFormFactory formFactory, ICdmFormElement formElement) {
        super(formFactory, formElement);

    }

    public String getLabel() {
        return label;
    }

    public void setSourceLabel(String label) {
        if (singleSourceSection != null){
            singleSourceSection.getReferenceSelectionElement().setLabel(label);
        }
        this.label = label;
    }

    @Override
    public void createControls(ICdmFormElement formElement, T entity, int style) {
        singleSourceSection = formFactory.createOriginalSourceElement(formElement, entity, label);
        if (entity.getSource() == null){
            NamedSource source = NamedSource.NewPrimarySourceInstance(null, null);
            entity.setSource(source);
            singleSourceSection.setEntity(source);
        }else{
            singleSourceSection.setEntity(entity.getSource());

        }
    }

    public void setIndent(int indent){
        singleSourceSection.setIndent(indent);
    }


}
