/**
* Copyright (C) 2013 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.ui.section.supplemental;

import java.util.Collection;
import java.util.Comparator;

import eu.etaxonomy.cdm.api.conversation.ConversationHolder;
import eu.etaxonomy.cdm.model.common.AnnotatableEntity;
import eu.etaxonomy.cdm.model.common.Annotation;
import eu.etaxonomy.cdm.model.common.AnnotationType;
import eu.etaxonomy.cdm.model.common.Language;
import eu.etaxonomy.taxeditor.store.CdmStore;
import eu.etaxonomy.taxeditor.ui.element.CdmFormFactory;
import eu.etaxonomy.taxeditor.ui.element.ICdmFormElement;
import eu.etaxonomy.taxeditor.ui.section.AbstractEntityCollectionSection;
import eu.etaxonomy.taxeditor.ui.section.DefaultCdmBaseComparator;

/**
 * @author n.hoffmann
 * @created Nov 5, 2009
 */
public class AnnotationSection extends AbstractEntityCollectionSection<AnnotatableEntity, Annotation>{

	/**
	 * <p>Constructor for AnnotationSection.</p>
	 *
	 */
	public AnnotationSection(CdmFormFactory cdmFormFactory, ConversationHolder conversation, ICdmFormElement parentElement, int style) {
		super(cdmFormFactory, conversation, parentElement, "Annotations", style);
	}

	@Override
	public Annotation createNewElement() {
		String text = "";
		AnnotationType type = AnnotationType.EDITORIAL();
		Language language = CdmStore.getDefaultLanguage();

		return Annotation.NewInstance(text, type, language);
	}

	@Override
	public Collection<Annotation> getCollection(AnnotatableEntity entity) {
		return entity.getAnnotations();
	}

	@Override
	public Comparator<Annotation> getComparator() {
        return new DefaultCdmBaseComparator<>();
	}

	@Override
	public String getEmptyString() {
		return "No annotations yet.";
	}

	@Override
	protected String getTooltipString() {
		return "Create a new annotation";
	}

	@Override
	public void addElement(Annotation element) {
		getEntity().addAnnotation(element);
	}

	@Override
	public void removeElement(Annotation element) {
		getEntity().removeAnnotation(element);
	}

    @Override
    public Annotation addExisting() {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public boolean allowAddExisting() {
        // TODO Auto-generated method stub
        return false;
    }
}