/**
 *
 */
package eu.etaxonomy.taxeditor.ui.section.supplemental;

import org.eclipse.swt.events.SelectionListener;

import eu.etaxonomy.cdm.hibernate.HibernateProxyHelper;
import eu.etaxonomy.cdm.model.common.Extension;
import eu.etaxonomy.cdm.model.common.ExtensionType;
import eu.etaxonomy.cdm.model.term.TermType;
import eu.etaxonomy.taxeditor.ui.combo.term.TermComboElement;
import eu.etaxonomy.taxeditor.ui.element.AbstractFormSection;
import eu.etaxonomy.taxeditor.ui.element.CdmFormFactory;
import eu.etaxonomy.taxeditor.ui.element.ICdmFormElement;
import eu.etaxonomy.taxeditor.ui.element.TextWithLabelElement;
import eu.etaxonomy.taxeditor.ui.section.AbstractEntityCollectionElement;

/**
 * <p>ExtensionElement class.</p>
 *
 * @author n.hoffmann
 * @created Nov 17, 2009
 */
public class ExtensionElement extends AbstractEntityCollectionElement<Extension> {

	private TermComboElement<ExtensionType> combo_extensionType;
	private TextWithLabelElement text_extensionValue;

	/**
	 * <p>Constructor for ExtensionElement.</p>
	 *
	 * @param formElement a {@link eu.etaxonomy.taxeditor.ui.element.AbstractFormSection} object.
	 * @param element a {@link eu.etaxonomy.cdm.model.common.Extension} object.
	 * @param removeListener a {@link org.eclipse.swt.events.SelectionListener} object.
	 * @param style a int.
	 * @param cdmFormFactory a {@link eu.etaxonomy.taxeditor.ui.element.CdmFormFactory} object.
	 */
	public ExtensionElement(CdmFormFactory cdmFormFactory, AbstractFormSection formElement, Extension element,
			SelectionListener removeListener, int style) {
		super(cdmFormFactory, formElement, element, removeListener, null, style);
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.forms.entitysections.AbstractEntitySetElementComposite#createControls(org.eclipse.swt.widgets.Composite, int)
	 */
	/** {@inheritDoc} */
	@Override
	public void createControls(ICdmFormElement element, int style) {
		combo_extensionType = formFactory.createDefinedTermComboElement(TermType.ExtensionType, element, "Type", null, style);
		text_extensionValue = formFactory.createTextWithLabelElement(element, "Extension", null, style);
		if (entity != null){
			setEntity(entity);
		}
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.forms.entitysections.AbstractEntitySetElementComposite#setElement(eu.etaxonomy.cdm.model.common.VersionableEntity)
	 */
	/** {@inheritDoc} */
	@Override
	public void setEntity(Extension element) {
		this.entity = element;
		if (combo_extensionType != null){
			ExtensionType extensionType = HibernateProxyHelper.deproxy(element.getType());
			combo_extensionType.setSelection(extensionType);
			text_extensionValue.setText(element.getValue());
		}
	}

	/*
	 * (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.section.AbstractEntityCollectionElement#handleEvent(java.lang.Object)
	 */
	/** {@inheritDoc} */
	@Override
	public void handleEvent(Object eventSource) {
		if(eventSource == combo_extensionType){
			entity.setType(combo_extensionType.getSelection());
		}
		else if(eventSource == text_extensionValue){
			entity.setValue(text_extensionValue.getText());
		}
	}

}
