/**
 *
 */
package eu.etaxonomy.taxeditor.ui.section.supplemental;

import java.util.Collection;
import java.util.Comparator;

import eu.etaxonomy.cdm.api.conversation.ConversationHolder;
import eu.etaxonomy.cdm.model.common.IdentifiableSource;
import eu.etaxonomy.cdm.model.media.Media;
import eu.etaxonomy.cdm.model.reference.ISourceable;
import eu.etaxonomy.cdm.model.reference.OriginalSourceType;
import eu.etaxonomy.taxeditor.ui.element.CdmFormFactory;
import eu.etaxonomy.taxeditor.ui.element.ICdmFormElement;
import eu.etaxonomy.taxeditor.ui.section.AbstractEntityCollectionSection;
import eu.etaxonomy.taxeditor.ui.section.description.SourceComparator;

/**
 * <p>SourceSection class.</p>
 *
 * @author n.hoffmann
 * @created Nov 17, 2009
 */
public class SourceSection extends AbstractEntityCollectionSection<ISourceable, IdentifiableSource> {

	/**
	 * <p>Constructor for SourceSection.</p>
	 *
	 * @param parentElement a {@link eu.etaxonomy.taxeditor.ui.element.ICdmFormElement} object.
	 * @param style a int.
	 * @param cdmFormFactory a {@link eu.etaxonomy.taxeditor.ui.element.CdmFormFactory} object.
	 * @param conversation a {@link eu.etaxonomy.cdm.api.conversation.ConversationHolder} object.
	 */
	public SourceSection(CdmFormFactory cdmFormFactory, ConversationHolder conversation, ICdmFormElement parentElement, int style) {
		super(cdmFormFactory, conversation, parentElement, "Sources", style);
	}

	@Override
	public IdentifiableSource createNewElement() {
	    ISourceable entity = getEntity();
	    OriginalSourceType type;
	    if(entity instanceof Media) {
	        type = OriginalSourceType.PrimaryMediaSource;
        }
	    else{
	        //default
	        type = OriginalSourceType.PrimaryTaxonomicSource;
	    }
	    return IdentifiableSource.NewInstance(type);
	}

	@Override
	public Collection<IdentifiableSource> getCollection(
			ISourceable entity) {
		return entity.getSources();
	}

	@Override
	public Comparator<IdentifiableSource> getComparator() {
        return new SourceComparator();
	}

	@Override
	public String getEmptyString() {
		return "No references yet.";
	}

	@Override
	protected String getTooltipString() {
		return "Create a new reference.";
	}

	@Override
	public void addElement(IdentifiableSource element) {
		getEntity().addSource(element);
	}

	@Override
	public void removeElement(IdentifiableSource element) {
		getEntity().removeSource(element);
	}

    @Override
    public IdentifiableSource addExisting() {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public boolean allowAddExisting() {
        // TODO Auto-generated method stub
        return false;
    }
}