/**
 * Copyright (C) 2007 EDIT
 * European Distributed Institute of Taxonomy
 * http://www.e-taxonomy.eu
 *
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */
package eu.etaxonomy.taxeditor.ui.section.taxon;

import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Display;

import eu.etaxonomy.cdm.common.CdmUtils;
import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.taxeditor.store.CdmStore;
import eu.etaxonomy.taxeditor.ui.element.CdmFormFactory;
import eu.etaxonomy.taxeditor.ui.element.CheckboxElement;
import eu.etaxonomy.taxeditor.ui.element.ICdmFormElement;
import eu.etaxonomy.taxeditor.ui.element.ISelectableElement;
import eu.etaxonomy.taxeditor.ui.element.LayoutConstants;
import eu.etaxonomy.taxeditor.ui.element.SelectionArbitrator;
import eu.etaxonomy.taxeditor.ui.section.AbstractIdentifiableEntityDetailElement;

public class TaxonOfRelationshipElement extends  AbstractIdentifiableEntityDetailElement<Taxon> implements ISelectableElement{

	private SelectionArbitrator selectionArbitrator;

    private CheckboxElement checkbox_doubtful;

	public TaxonOfRelationshipElement(CdmFormFactory formFactory, ICdmFormElement formElement, int style ) {
		super(formFactory, formElement);
	}

    @Override
    protected void handleToggleableCacheField() {
        getEntity().setTitleCache(toggleable_cache.getText(), toggleable_cache.getState());
        updateCacheRelevance();
        updateToggleableCacheField();
    }

    @Override
    public void updateToggleableCacheField() {
        if (toggleable_cache != null){
            if (!getEntity().isProtectedTitleCache()) {
                toggleable_cache.setText(getEntity().generateTitle());
            }
        }
	}

	 @Override
	public void refresh(){
		 updateContent();
		 updateToggleableCacheField();
	}

	@Override
	protected void updateContent() {
		super.updateContent();
		toggleable_cache.setCacheEnabled(getEntity().isProtectedTitleCache());
		updateCacheRelevance();
		firePropertyChangeEvent(getEntity());
	}

	@Override
	protected void createControls(ICdmFormElement formElement, Taxon entity, int style) {
	    this.setWarnForReferencingObjects(formElement, 2);
		toggleable_cache = formFactory.createToggleableTextField(this,
				"Title Cache", getEntity().getTitleCache(),
				getEntity().isProtectedTitleCache(), style);

		checkbox_doubtful = formFactory.createCheckbox(this,
		        "Orginal Taxon Doubtful", getEntity().isDoubtful(), style);

		if(formFactory.getSelectionProvider() != null){
            selectionArbitrator = formFactory.createSelectionArbitrator(this);
        }

	    //cache relevance
        registerCacheRelevance(checkbox_doubtful);
	}

	@Override
	public void handleEvent(Object eventSource) {
		if (eventSource== toggleable_cache) {
			handleToggleableCacheField();
		} else if (eventSource == checkbox_doubtful) {
			getEntity().setDoubtful(checkbox_doubtful.getSelection());
		}
	}

	@Override
    public SelectionArbitrator getSelectionArbitrator() {
        return selectionArbitrator;
    }
	@Override
    public void setWarnForReferencingObjects(ICdmFormElement formElement, int defaultReferencingObjects){
        Taxon taxon = getEntity();
        if (taxon.getId() != 0){
            long referencingObjectsCount = CdmStore.getCommonService().getReferencingObjectsCount(taxon);

            if (referencingObjectsCount > defaultReferencingObjects){
                setWarnForReferencedObjects(formFactory.createLabel(formElement, CdmUtils.Nz("The "+ taxon.getUserFriendlyTypeName()+" is referenced by " + (referencingObjectsCount - defaultReferencingObjects)+ " other object(s), if you change it, it is changed for all these objects")));
                getWarnForReferencedObjects().setBackground(Display.getCurrent().getSystemColor(SWT.COLOR_RED));
                getWarnForReferencedObjects().setLayout(LayoutConstants.FILL(2, 3));
                getWarnForReferencedObjects().setForeground(Display.getCurrent().getSystemColor(SWT.COLOR_RED));
                formFactory.createLabel(formElement, "");
            }
        }
    }
}