/**
 *
 */
package eu.etaxonomy.taxeditor.ui.section.user;

import java.util.ArrayList;
import java.util.Collection;

import eu.etaxonomy.cdm.model.agent.Person;
import eu.etaxonomy.cdm.model.permission.User;
import eu.etaxonomy.taxeditor.store.CdmStore;
import eu.etaxonomy.taxeditor.ui.element.CdmFormFactory;
import eu.etaxonomy.taxeditor.ui.element.CheckboxElement;
import eu.etaxonomy.taxeditor.ui.element.ICdmFormElement;
import eu.etaxonomy.taxeditor.ui.element.IEnableableFormElement;
import eu.etaxonomy.taxeditor.ui.element.TextWithLabelElement;
import eu.etaxonomy.taxeditor.ui.section.AbstractCdmDetailElement;
import eu.etaxonomy.taxeditor.ui.selection.EntitySelectionElement;

/**
 * @author n.hoffmann
 */
public class UserDetailElement extends AbstractCdmDetailElement<User> {

	private TextWithLabelElement text_username;
	private TextWithLabelElement text_emailAdress;
	private EntitySelectionElement<Person> selection_person;
	private CheckboxElement checkbox_credentialsNonExpired;
	private CheckboxElement checkbox_enabled;
	private CheckboxElement checkbox_accountNonLocked;
	private CheckboxElement checkbox_accountNonExpired;

	public UserDetailElement(CdmFormFactory formFactory,
			ICdmFormElement formElement) {
		super(formFactory, formElement);
	}

	@Override
	protected void createControls(ICdmFormElement formElement, User entity,
			int style) {

		text_username = formFactory.createTextWithLabelElement(formElement,
				"Username", entity.getUsername(), style);
		// Disabling the text field since the user entity is immutable
		text_username.setEnabled(false);

		if (userIsAuthenticated() || CdmStore.getLoginManager().isUserManager() || CdmStore.getLoginManager().isAdmin() ) {
			formFactory.createEditPasswordElement(
					formElement, "Change Password", getConversationHolder(),
					entity, style);
		}

		text_emailAdress = formFactory.createTextWithLabelElement(formElement,
				"Email Adress", entity.getEmailAddress(), style);

		selection_person = formFactory
				.createSelectionElement(Person.class,
						//getConversationHolder(),
						formElement,
						"Person",
						entity.getPerson(),
						EntitySelectionElement.EDITABLE | EntitySelectionElement.SELECTABLE,
						style);

		checkbox_enabled = formFactory.createCheckbox(formElement, "Enabled",
				entity.isEnabled(), style);

	}

	@Override
    public void setEnabled(boolean enabled) {
        Collection<Object> except = new ArrayList<Object>();

        for(ICdmFormElement formElement:getElements()){
            if(formElement instanceof IEnableableFormElement && !((IEnableableFormElement) formElement).isEnabled()){
                except.add(formElement);
            }
        }
        setEnabled(enabled, except);
    }
	private boolean userIsAuthenticated() {
		return getEntity().equals(
				CdmStore.getLoginManager().getAuthenticatedUser());
	}

	@Override
	public void handleEvent(Object eventSource) {
		if (eventSource == text_username) {
			getEntity().setUsername(text_username.getText());
		} else if (eventSource == text_emailAdress) {
			getEntity().setEmailAddress(text_emailAdress.getText());
		} else if (eventSource == selection_person) {
			getEntity().setPerson(selection_person.getSelection());
		} else if (eventSource == checkbox_accountNonExpired) {
			getEntity().setAccountNonExpired(
					checkbox_accountNonExpired.getSelection());
		} else if (eventSource == checkbox_accountNonLocked) {
			getEntity().setAccountNonLocked(
					checkbox_accountNonLocked.getSelection());
		} else if (eventSource == checkbox_enabled) {
			getEntity().setEnabled(checkbox_enabled.getSelection());
		} else if (eventSource == checkbox_credentialsNonExpired) {
			getEntity().setCredentialsNonExpired(
					checkbox_credentialsNonExpired.getSelection());
		}
	}
}
