/**
* Copyright (C) 2021 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.ui.section.vocabulary;

import org.apache.commons.lang3.StringUtils;

import eu.etaxonomy.cdm.model.common.Language;
import eu.etaxonomy.cdm.model.term.TermVocabulary;
import eu.etaxonomy.taxeditor.ui.element.CdmFormFactory;
import eu.etaxonomy.taxeditor.ui.element.ICdmFormElement;
import eu.etaxonomy.taxeditor.ui.element.TextWithLabelElement;

/**
 * @author k.luther
 * @since Sep 15, 2021
 */
public class LanguageDetailElement extends AbstractTermBaseDetailElement<Language> {

    protected TextWithLabelElement textIdInVocabulary;

    protected TextWithLabelElement textISO_639_1;

    protected TextWithLabelElement textSymbol;

    protected TextWithLabelElement textSymbol2;


    /**
     * @param formFactory
     * @param formElement
     */
    public LanguageDetailElement(CdmFormFactory formFactory, ICdmFormElement formElement) {
        super(formFactory, formElement);

    }

    @Override
    protected void createControls(ICdmFormElement formElement, Language entity, int style) {
        super.createControls(formElement, entity, style);
        textIdInVocabulary = formFactory.createTextWithLabelElement(formElement, "ID in vocabulary", entity.getIdInVocabulary(), style);
        textISO_639_1 = formFactory.createTextWithLabelElement(formElement, "ID in ISO 639-1", entity.getIdInVocabulary(), style);
        textSymbol = formFactory.createTextWithLabelElement(formElement, "Symbol", entity.getSymbol(), style);
        textSymbol2 = formFactory.createTextWithLabelElement(formElement, "Symbol 2", entity.getSymbol2(), style);
    };

    @Override
    public void handleEvent(Object eventSource) {
        super.handleEvent(eventSource);
        //handleRepresentation(eventSource);
        if(eventSource==textIdInVocabulary){
            if (StringUtils.isBlank(textIdInVocabulary.getText())){
                getEntity().setIdInVocabulary(null);
            }else{
                getEntity().setIdInVocabulary(textIdInVocabulary.getText());
            }

        }
        if(eventSource==textSymbol){
            if (StringUtils.isBlank(textSymbol.getText())){
                getEntity().setSymbol(null);
            }else{
                getEntity().setSymbol(textSymbol.getText());
            }

        }

        if(eventSource==textSymbol2){
            if (StringUtils.isBlank(textSymbol2.getText())){
                getEntity().setSymbol2(null);
            }else{
                getEntity().setSymbol2(textSymbol2.getText());
            }

        }
        if (eventSource == textISO_639_1){
            if (StringUtils.isBlank(textISO_639_1.getText())){
                getEntity().setIso639_1(null);
            }else{
                getEntity().setIso639_1(textISO_639_1.getText());
            }
        }
    }

    @Override
    public TermVocabulary getVocabulary() {
        return getEntity() != null ? getEntity().getVocabulary() : null;
    }


}
