/**
* Copyright (C) 2015 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.ui.section.vocabulary;

import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Color;

import eu.etaxonomy.cdm.model.description.MeasurementUnit;
import eu.etaxonomy.cdm.model.term.TermType;
import eu.etaxonomy.cdm.persistence.dto.FeatureDto;
import eu.etaxonomy.cdm.persistence.dto.TermDto;
import eu.etaxonomy.taxeditor.ui.combo.term.TermComboElement;
import eu.etaxonomy.taxeditor.ui.element.AbstractFormSection;
import eu.etaxonomy.taxeditor.ui.element.CdmFormFactory;
import eu.etaxonomy.taxeditor.ui.element.ICdmFormElement;
import eu.etaxonomy.taxeditor.ui.section.AbstractEntityCollectionElement;

/**
 * @author pplitzner
 * @date Apr 27, 2015
 *
 */
public class MeasurementUnitDtoCollectionElement extends AbstractEntityCollectionElement<TermDto> {


    private TermComboElement<MeasurementUnit> comboMeasurementUnit;

    /**
     * @param formFactory
     * @param section
     * @param entity
     * @param removeListener
     * @param isChoosableEntity
     * @param backgroundColor
     * @param style
     */
    public MeasurementUnitDtoCollectionElement(CdmFormFactory formFactory, AbstractFormSection section, TermDto entity,
            SelectionListener removeListener, Color backgroundColor, int style) {
        super(formFactory, section, entity, removeListener, backgroundColor, style);

    }

    @Override
    public void createControls(ICdmFormElement element, int style) {
        comboMeasurementUnit = formFactory.createDefinedTermComboElementDto(TermType.MeasurementUnit, element, "Measurement unit", null, style);
        if (entity != null){
        	setEntity(entity);
        }
    }

    @Override
    public void setEntity(TermDto entity) {
        this.entity = entity;
        if(comboMeasurementUnit != null){
            comboMeasurementUnit.setSelectionDto(entity);
            comboMeasurementUnit.removeEmptyElement();
        }
    }

    @Override
    public void handleEvent(Object eventSource) {
        if(eventSource==comboMeasurementUnit && comboMeasurementUnit.getSelection()!=null){
            if(getParentElement() instanceof MeasurementUnitDtoCollectionSection){
                MeasurementUnitDtoCollectionSection parentElement = (MeasurementUnitDtoCollectionSection) getParentElement();
                FeatureDto feature = parentElement.getEntity();
                TermDto remove = null;
                for (TermDto unit: feature.getRecommendedMeasurementUnits()){
                    if (unit.getUuid().equals(getEntity().getUuid())){
                        remove =unit;
                        break;
                    }
                }
                feature.getRecommendedMeasurementUnits().remove(remove);
                MeasurementUnit measurementUnit = comboMeasurementUnit.getSelection();
                entity = TermDto.fromTerm(measurementUnit);
                feature.getRecommendedMeasurementUnits().add(entity);

                //update the parent section to re-set the listeners to the persisted measurement unit
//                parentElement.removeElementAndUpdate(null);
            }
        }
    }

}
