/**
* Copyright (C) 2015 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.ui.section.vocabulary;

import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Color;

import eu.etaxonomy.cdm.model.description.State;
import eu.etaxonomy.cdm.model.term.TermType;
import eu.etaxonomy.cdm.model.term.TermVocabulary;
import eu.etaxonomy.cdm.persistence.dto.FeatureDto;
import eu.etaxonomy.cdm.persistence.dto.TermVocabularyDto;
import eu.etaxonomy.taxeditor.ui.combo.VocabularyComboElement;
import eu.etaxonomy.taxeditor.ui.element.AbstractFormSection;
import eu.etaxonomy.taxeditor.ui.element.CdmFormFactory;
import eu.etaxonomy.taxeditor.ui.element.ICdmFormElement;
import eu.etaxonomy.taxeditor.ui.section.AbstractEntityCollectionElement;

/**
 * @author pplitzner
 * @date Apr 27, 2015
 *
 */
public class StateVocabularyDtoCollectionElement extends AbstractEntityCollectionElement<TermVocabularyDto> {


    private VocabularyComboElement<State, TermVocabulary<State>> comboStateVocabulary;

    /**
     * @param formFactory
     * @param section
     * @param entity
     * @param removeListener
     * @param isChoosableEntity
     * @param backgroundColor
     * @param style
     */
    public StateVocabularyDtoCollectionElement(CdmFormFactory formFactory, AbstractFormSection section, TermVocabularyDto entity,
            SelectionListener removeListener, Color backgroundColor, int style) {
        super(formFactory, section, entity, removeListener, backgroundColor, style);

    }

    @Override
    public void createControls(ICdmFormElement element, int style) {
        comboStateVocabulary = formFactory.createVocabularyDtoComboElement(TermType.State, "State vocabulary", null, element, style);
        if (entity != null){
        	setEntity(entity);
        }
    }

    @Override
    public void setEntity(TermVocabularyDto entity) {
        this.entity = entity;
        if(comboStateVocabulary != null){
            comboStateVocabulary.setSelectionDto(entity);
            comboStateVocabulary.removeEmptyElement();
        }
    }

    @Override
    public void handleEvent(Object eventSource) {
        if(eventSource==comboStateVocabulary && comboStateVocabulary.getSelection()!=null){
            if(getParentElement() instanceof StateVocabularyDtoCollectionSection){
                StateVocabularyDtoCollectionSection parentElement = (StateVocabularyDtoCollectionSection) getParentElement();
                FeatureDto feature = parentElement.getEntity();
                TermVocabularyDto remove = null;
                for (TermVocabularyDto voc: feature.getSupportedCategoricalEnumerations()){
                    if (voc.getUuid().equals(getEntity().getUuid())){
                        remove = voc;
                        break;
                    }
                }
                feature.getSupportedCategoricalEnumerations().remove(remove);
                TermVocabulary<State> vocabulary = comboStateVocabulary.getSelection();
                TermVocabularyDto dto = new TermVocabularyDto(vocabulary.getUuid(), vocabulary.getRepresentations(), vocabulary.getTermType(), vocabulary.getTitleCache(), vocabulary.isAllowDuplicates(), vocabulary.isOrderRelevant(), vocabulary.isFlat());
                feature.getSupportedCategoricalEnumerations().add(dto);
                entity = dto;
                //update the parent section to re-set the listeners to the persisted measurement unit
                parentElement.removeElementAndUpdate(null);
            }
        }
    }

}
