/**
* Copyright (C) 2016 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.ui.section.vocabulary;

import org.eclipse.jface.wizard.Wizard;

import eu.etaxonomy.cdm.common.CdmUtils;
import eu.etaxonomy.cdm.model.term.Representation;
import eu.etaxonomy.cdm.model.term.TermBase;
import eu.etaxonomy.taxeditor.ui.translation.TermTranslationWizardPage;

/**
 * @author k.luther
 * @date 22.06.2016
 *
 */
public class TermTranslationWizard extends Wizard {
    private TermTranslationWizardPage page;

    private final TermBase term;

    public TermTranslationWizard(TermBase term) {
        setWindowTitle("Translation Editor");
        this.term = term;
        setNeedsProgressMonitor(true);
    }

    @Override
    public void addPages() {
        page = new TermTranslationWizardPage(term);
        addPage(page);
    }

    @Override
    public boolean performFinish() {
        Representation representation = page.getTargetRepresentation();

        if (representation.getLanguage() == null){
        	return false;
        }
        if (CdmUtils.isNotBlank(representation.getText())
                || CdmUtils.isNotBlank(representation.getLabel())
                || CdmUtils.isNotBlank(representation.getAbbreviatedLabel())
                || CdmUtils.isNotBlank(representation.getPlural())){
            term.addRepresentation(representation);
        }

        return true;
    }

    @Override
    public boolean canFinish() {
        Representation representation = page.getTargetRepresentation();
        if (representation==null || representation.getLanguage() == null){
            return false;
        }
        return CdmUtils.isNotBlank(representation.getText())
                || CdmUtils.isNotBlank(representation.getLabel())
                || CdmUtils.isNotBlank(representation.getAbbreviatedLabel())
                || CdmUtils.isNotBlank(representation.getPlural());
    }
}
