/**
* Copyright (C) 2018 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.ui.selection;

import org.eclipse.swt.events.SelectionEvent;

import eu.etaxonomy.cdm.model.common.Marker;
import eu.etaxonomy.cdm.model.common.MarkerType;
import eu.etaxonomy.cdm.model.reference.Reference;
import eu.etaxonomy.taxeditor.store.StoreUtil;
import eu.etaxonomy.taxeditor.ui.dialog.selection.SelectionDialogFactory;
import eu.etaxonomy.taxeditor.ui.element.CdmFormFactory;
import eu.etaxonomy.taxeditor.ui.element.ICdmFormElement;

/**
 * @author k.luther
 * @since 17.09.2018
 */
public class CommonNameReferenceSelectionElement extends EntitySelectionElement<Reference> {

    public CommonNameReferenceSelectionElement(CdmFormFactory formFactory, ICdmFormElement parentElement,
             String labelString, Reference entity, int mode, int style) {
        super(formFactory, parentElement, Reference.class, labelString, entity, mode, style);

    }

    @Override
    public void widgetSelected(SelectionEvent e) {
        if (e.getSource().equals(button_selection) ){
            Reference selection = SelectionDialogFactory.getSelectionFromDialog(Reference.class, getShell(),
                    getEntity(), getParentElement());
            boolean alreadyMarked = false;
            if (selection != null){
                for (Marker marker: selection.getMarkers()){
                    if (marker.getMarkerType().equals(MarkerType.COMMON_NAME_REFERENCE())){
                        alreadyMarked = true;
                        break;
                    }
                }
                if (!alreadyMarked){
                    selection.addMarker(Marker.NewInstance(MarkerType.COMMON_NAME_REFERENCE(), true));
                }

                setSelectionInternal(selection);
            }
            StoreUtil.reflowParentScrolledForm(getLayoutComposite(), true);
        }else{
            Reference selection = SelectionDialogFactory.getSelectionFromExtDialog(Reference.class, getShell(),
                    getParentElement());
            setSelectionInternal(selection);
        }
    }
}