/**
 * Copyright (C) 2007 EDIT
 * European Distributed Institute of Taxonomy
 * http://www.e-taxonomy.eu
 *
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */
package eu.etaxonomy.taxeditor.ui.selection;

import java.util.Collections;
import java.util.List;
import java.util.Map;

import org.eclipse.core.runtime.IStatus;
import org.eclipse.jface.wizard.Wizard;
import org.eclipse.jface.wizard.WizardDialog;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;

import eu.etaxonomy.cdm.api.facade.DerivedUnitFacade;
import eu.etaxonomy.cdm.api.facade.DerivedUnitFacadeNotSupportedException;
import eu.etaxonomy.cdm.api.service.IService;
import eu.etaxonomy.cdm.hibernate.HibernateProxyHelper;
import eu.etaxonomy.cdm.model.agent.Institution;
import eu.etaxonomy.cdm.model.agent.Person;
import eu.etaxonomy.cdm.model.agent.Team;
import eu.etaxonomy.cdm.model.common.CdmBase;
import eu.etaxonomy.cdm.model.permission.User;
import eu.etaxonomy.cdm.model.description.PolytomousKey;
import eu.etaxonomy.cdm.model.molecular.Amplification;
import eu.etaxonomy.cdm.model.molecular.Primer;
import eu.etaxonomy.cdm.model.name.TaxonName;
import eu.etaxonomy.cdm.model.occurrence.Collection;
import eu.etaxonomy.cdm.model.occurrence.DerivedUnit;
import eu.etaxonomy.cdm.model.occurrence.FieldUnit;
import eu.etaxonomy.cdm.model.occurrence.SpecimenOrObservationBase;
import eu.etaxonomy.cdm.model.occurrence.SpecimenOrObservationType;
import eu.etaxonomy.cdm.model.reference.Reference;
import eu.etaxonomy.cdm.model.taxon.TaxonBase;
import eu.etaxonomy.taxeditor.model.MessagingUtils;
import eu.etaxonomy.taxeditor.model.TextHelper;
import eu.etaxonomy.taxeditor.newWizard.AmplificationGeneralWizardPage;
import eu.etaxonomy.taxeditor.newWizard.PolytomousKeyWizardPage;
import eu.etaxonomy.taxeditor.newWizard.PrimerWizardPage;
import eu.etaxonomy.taxeditor.preference.PreferencesUtil;
import eu.etaxonomy.taxeditor.session.ICdmEntitySession;
import eu.etaxonomy.taxeditor.session.ICdmEntitySessionEnabled;
import eu.etaxonomy.taxeditor.store.CdmStore;
import eu.etaxonomy.taxeditor.ui.element.CdmFormFactory;
import eu.etaxonomy.taxeditor.ui.section.agent.InstitutionWizardPage;
import eu.etaxonomy.taxeditor.ui.section.agent.PersonWizardPage;
import eu.etaxonomy.taxeditor.ui.section.agent.TeamWizardPage;
import eu.etaxonomy.taxeditor.ui.section.name.NonViralNameWizardPage;
import eu.etaxonomy.taxeditor.ui.section.occurrence.CollectionWizardPage;
import eu.etaxonomy.taxeditor.ui.section.occurrence.DerivedUnitBaseWizardPage;
import eu.etaxonomy.taxeditor.ui.section.occurrence.DerivedUnitGeneralWizardPage;
import eu.etaxonomy.taxeditor.ui.section.occurrence.FieldUnitWizardPage;
import eu.etaxonomy.taxeditor.ui.section.occurrence.GatheringEventWizardPage;
import eu.etaxonomy.taxeditor.ui.section.reference.ReferenceWizardPage;
import eu.etaxonomy.taxeditor.ui.section.taxon.TaxonWizardPage;
import eu.etaxonomy.taxeditor.ui.section.user.UserDetailWizardPage;

/**
 * @author n.hoffmann
 * @created Jun 1, 2010
 */
public class EditFromSelectionWizard extends Wizard implements
		SelectionListener, ICdmEntitySessionEnabled<CdmBase> {

    private ICdmEntitySession cdmEntitySession;
    private ICdmEntitySession previousCdmEntitySession;
	private final EntitySelectionElement<CdmBase> selectionElement;
	private CdmBase rootElement;

	public EditFromSelectionWizard(EntitySelectionElement selectionElement) {
        if (CdmStore.isActive()) {
            previousCdmEntitySession = CdmStore.getCurrentSessionManager().getActiveSession();
            cdmEntitySession = CdmStore.getCurrentSessionManager().newSession(this, true);
            cdmEntitySession.bind();
        }
		this.selectionElement = selectionElement;
		this.setWindowTitle(String.format("Edit %s", TextHelper.deproxyClassName(selectionElement.getEntity().getClass())));
	}

	@Override
	public void addPages() {

		CdmFormFactory formFactory = selectionElement.getFormFactory();
		CdmBase entity = HibernateProxyHelper
				.deproxy(selectionElement.getEntity());
		//check if entity has already been persisted
		if(entity.getId()==0){
			rootElement = entity;
		}
		else{
			IService<CdmBase> service = CdmStore.getService(entity);
			rootElement = service.load(entity.getUuid());
		}
		if (rootElement.isInstanceOf(Reference.class)&& selectionElement instanceof EntitySelectionElementWithAbbreviatedTitle<?>) {
			addPage(new ReferenceWizardPage(formFactory,
					selectionElement.getConversationHolder(),
					HibernateProxyHelper.deproxy(rootElement, Reference.class), true));
		} else if (rootElement.isInstanceOf(Reference.class) ) {
			addPage(new ReferenceWizardPage(formFactory,
					selectionElement.getConversationHolder(), HibernateProxyHelper.deproxy(rootElement, Reference.class), false));
		} else if (rootElement.isInstanceOf(Team.class) && (selectionElement instanceof EntitySelectionElementWithAbbreviatedTitle<?> )) {
			addPage(new TeamWizardPage(formFactory,
					selectionElement.getConversationHolder(), HibernateProxyHelper.deproxy(rootElement, Team.class), true, false));
		} else if (rootElement.isInstanceOf(Team.class) && (selectionElement.getClass().equals(CollectorSelectionElement.class)) ){
			addPage(new TeamWizardPage(formFactory,
					selectionElement.getConversationHolder(), HibernateProxyHelper.deproxy(rootElement, Team.class), false, true));
		} else if (rootElement.isInstanceOf(Team.class) && (selectionElement instanceof EntitySelectionElement<?>)) {
            addPage(new TeamWizardPage(formFactory,
                    selectionElement.getConversationHolder(), HibernateProxyHelper.deproxy(rootElement, Team.class), false, false));
        } else if (rootElement.isInstanceOf(Person.class)) {
			addPage(new PersonWizardPage(formFactory,
					selectionElement.getConversationHolder(), HibernateProxyHelper.deproxy(rootElement, Person.class)));
		} else if (rootElement.isInstanceOf(TaxonName.class)) {
			addPage(new NonViralNameWizardPage(formFactory,
					selectionElement.getConversationHolder(),
					HibernateProxyHelper.deproxy(rootElement, TaxonName.class)));
		} else if (rootElement.isInstanceOf(TaxonBase.class)) {
            addPage(new TaxonWizardPage(formFactory,
                    selectionElement.getConversationHolder(),
                    HibernateProxyHelper.deproxy(rootElement, TaxonBase.class)));
        }  else if (rootElement.isInstanceOf(SpecimenOrObservationBase.class)) {

			DerivedUnitFacade facade;
			try {
			    if(rootElement.isInstanceOf(DerivedUnit.class)){
			        facade = DerivedUnitFacade.NewInstance(
			                HibernateProxyHelper.deproxy(rootElement, DerivedUnit.class),
			                PreferencesUtil.getDerivedUnitConfigurator());
			    }
			    else {
                    facade = DerivedUnitFacade.NewInstance(SpecimenOrObservationType.FieldUnit,
                            HibernateProxyHelper.deproxy(rootElement, FieldUnit.class),
                            PreferencesUtil.getDerivedUnitConfigurator());
			    }

			} catch (DerivedUnitFacadeNotSupportedException e) {
				// we should never get here
				throw new IllegalStateException();
			}

			addPage(new DerivedUnitGeneralWizardPage(formFactory,
					selectionElement.getConversationHolder(), facade));
			addPage(new GatheringEventWizardPage(formFactory,
					selectionElement.getConversationHolder(), facade));
			addPage(new FieldUnitWizardPage(formFactory,
					selectionElement.getConversationHolder(), facade));
			if(facade.innerDerivedUnit()!=null){
			    addPage(new DerivedUnitBaseWizardPage(formFactory,
			            selectionElement.getConversationHolder(), facade));
			}
		} else if (rootElement.isInstanceOf(Collection.class)) {
			addPage(new CollectionWizardPage(formFactory,
					selectionElement.getConversationHolder(),
					HibernateProxyHelper.deproxy(rootElement, Collection.class)));
		} else if (rootElement instanceof Institution) {
			addPage(new InstitutionWizardPage(formFactory,
					selectionElement.getConversationHolder(),
					HibernateProxyHelper.deproxy(rootElement, Institution.class)));
		} else if (rootElement instanceof User) {
			addPage(new UserDetailWizardPage(formFactory,
					selectionElement.getConversationHolder(),
					HibernateProxyHelper.deproxy(rootElement, User.class)));
		} else if (rootElement instanceof Primer) {
            addPage(new PrimerWizardPage(formFactory,
                    selectionElement.getConversationHolder(),
                    HibernateProxyHelper.deproxy(rootElement, Primer.class)));
        } else if (rootElement instanceof Amplification) {
            addPage(new AmplificationGeneralWizardPage(formFactory,
                    selectionElement.getConversationHolder(),
                    HibernateProxyHelper.deproxy(rootElement, Amplification.class)));
        } else if (rootElement instanceof PolytomousKey) {
            addPage(new PolytomousKeyWizardPage(formFactory,
                    selectionElement.getConversationHolder(),
                    HibernateProxyHelper.deproxy(rootElement, PolytomousKey.class)));
        } else {
			MessagingUtils.warningDialog("Missing interface", this,
					"No detail element for current selection");
		}
	}

	@Override
	public boolean performFinish() {
	    IService<CdmBase> service = CdmStore.getService(rootElement);
	    if(rootElement.getId()!=0){
	        service.merge(rootElement, true);

	    }else{
	        rootElement = service.save(rootElement);
	    }
	    if(previousCdmEntitySession!=null){
            selectionElement.setEntity(previousCdmEntitySession.load(rootElement, true));
        }

		return true;
	}

	@Override
	public void widgetSelected(SelectionEvent e) {
		WizardDialog dialog = new WizardDialog(selectionElement.getShell(),
				this);
		if (dialog.open() == IStatus.OK) {
			selectionElement.updateFromWizard();
		}
	}

	@Override
	public void widgetDefaultSelected(SelectionEvent e) {
	}

    @Override
    public void dispose() {
        super.dispose();
        if(cdmEntitySession != null) {
            cdmEntitySession.dispose();
            cdmEntitySession = null;
        }
        if(previousCdmEntitySession!=null){
            previousCdmEntitySession.bind();
        }
    }

    @Override
    public ICdmEntitySession getCdmEntitySession() {
        return cdmEntitySession;
    }

    @Override
    public java.util.Collection<CdmBase> getRootEntities() {
        return Collections.singleton(rootElement);
    }

    @Override
    public Map<Object, List<String>> getPropertyPathsMap() {
        return null;
    }
}
