/**
* Copyright (C) 2016 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.ui.translation;

import org.eclipse.jface.util.IPropertyChangeListener;
import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;

import eu.etaxonomy.cdm.model.common.Language;
import eu.etaxonomy.cdm.model.term.Representation;
import eu.etaxonomy.cdm.model.term.TermBase;
import eu.etaxonomy.taxeditor.preference.PreferencesUtil;
import eu.etaxonomy.taxeditor.store.CdmStore;
import eu.etaxonomy.taxeditor.ui.element.CdmFormFactory;
import eu.etaxonomy.taxeditor.ui.element.ICdmFormElement;
import eu.etaxonomy.taxeditor.ui.element.LayoutConstants;
import eu.etaxonomy.taxeditor.ui.element.MultilanguageTextElement;
import eu.etaxonomy.taxeditor.ui.element.RepresentationElement;
import eu.etaxonomy.taxeditor.ui.element.RootElement;

/**
 * @author k.luther
 * @date 22.06.2016
 *
 */
public class TermTranslationWizardPage extends WizardPage implements IPropertyChangeListener{

        private static int TEXT_HEIGHT = 200;

        private final TermBase term;

        private MultilanguageTextElement source;

        private final CdmFormFactory formFactory;

        private RepresentationElement text_source;

        private RepresentationElement text_target;

        public TermTranslationWizardPage(TermBase term) {
            super("TranslationWizardPage");
            //Assert.isNotNull(multilanguageText, "Multilanguage text may not be null");
            setTitle("Add or edit translations.");
            this.term = term;
            formFactory = new CdmFormFactory(Display.getCurrent());
            formFactory.addPropertyChangeListener(this);
            setPageComplete(false);
        }

        @Override
        public void createControl(Composite parent) {
            Composite control = formFactory.createComposite(parent);
            control.setLayout(LayoutConstants.LAYOUT(2, false));

            Composite leftContainer = formFactory.createComposite(control);
            leftContainer.setLayoutData(LayoutConstants.FILL(1, 4));
            leftContainer.setLayout(LayoutConstants.LAYOUT(2, false));
            RootElement leftRoot = new RootElement(formFactory, leftContainer);

            Composite rightContainer = formFactory.createComposite(control);
            rightContainer.setLayoutData(LayoutConstants.FILL(1, 4));
            rightContainer.setLayout(LayoutConstants.LAYOUT(2, false));
            RootElement rightRoot = new RootElement(formFactory, rightContainer);

            createLeftControls(leftRoot);
            createRightControls(rightRoot);

            // initialize with default language
            setSource(CdmStore.getDefaultLanguage());

            setControl(control);
            setPageComplete(true);
        }

        private void setSource(Language language) {
           Representation preferredRepresentation =term.getPreferredRepresentation(language);
            if(preferredRepresentation != null){
               text_source.setSelectedRepresentation(preferredRepresentation);
            }
        }

        private void createLeftControls(ICdmFormElement element){
            Representation preferredRepresentation= term.getPreferredRepresentation(PreferencesUtil.getGlobalLanguage());

            text_source = formFactory.createRepresentationElement(element, preferredRepresentation,term,  TEXT_HEIGHT,  SWT.NULL, true);
            text_source.setTerm(term, true);

        }

        private void createRightControls(ICdmFormElement element){
            text_target = formFactory.createRepresentationElement(element, null, term, TEXT_HEIGHT, SWT.NULL, false);
        }

        @Override
        public void dispose() {
            super.dispose();
        }

        public Representation getTargetRepresentation() {
            return text_target.getSelectedRepresentation();
        }

        @Override
        public void propertyChange(org.eclipse.jface.util.PropertyChangeEvent event) {
            if (event.getSource() == text_target){
                getWizard().getContainer().updateButtons();
            }
        }
}
