// $Id$
/**
 * Copyright (C) 2017 EDIT
 * European Distributed Institute of Taxonomy
 * http://www.e-taxonomy.eu
 *
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */
package eu.etaxonomy.taxeditor.util;

import org.eclipse.jface.viewers.ColumnLabelProvider;
import org.eclipse.jface.viewers.DelegatingStyledCellLabelProvider.IStyledLabelProvider;
import org.eclipse.jface.viewers.StyledString;
import org.eclipse.jface.viewers.StyledString.Styler;
import org.eclipse.swt.graphics.TextStyle;
import org.hibernate.LazyInitializationException;

import eu.etaxonomy.cdm.hibernate.HibernateProxyHelper;
import eu.etaxonomy.cdm.model.common.CdmBase;
import eu.etaxonomy.cdm.model.common.IIdentifiableEntity;
import eu.etaxonomy.cdm.model.taxon.Classification;
import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.cdm.model.taxon.TaxonNode;
import eu.etaxonomy.cdm.persistence.dto.TaxonNodeDto;
import eu.etaxonomy.taxeditor.model.MessagingUtils;
import eu.etaxonomy.taxeditor.preference.Resources;
import eu.etaxonomy.taxeditor.security.RequiredPermissions;
import eu.etaxonomy.taxeditor.store.CdmStore;
import eu.etaxonomy.taxeditor.store.StoreUtil;

/**
 * @author pplitzner
 * @date 05.09.2017
 *
 */
public class TaxonTreeNodeLabelProvider extends ColumnLabelProvider
implements IStyledLabelProvider {

    protected Styler notGrantedStyler = null;

    /**
     * {@inheritDoc}
     */
    @Override
    public String getText(Object element) {
        //classification
        if(element instanceof Classification){
            String text = ((Classification) element).getName().getText();
            return text != null ? text : "Unnamed Taxonomic Tree";
        }
        //taxon node
        else if (element instanceof TaxonNode){
            TaxonNode taxonNode = (TaxonNode) HibernateProxyHelper.deproxy(element);
          try{
                Taxon taxon = HibernateProxyHelper.deproxy(taxonNode.getTaxon());
                if(taxon == null){
                    String text = taxonNode.getClassification().getName() == null ? null : taxonNode.getClassification().getName().getText();
                    if(text==null){
                        text = taxonNode.getClassification().getTitleCache();
                    }
                    return text;
                }else{
                    try{
                        String text = "";
                            if (taxonNode.getStatus() != null){
                                text += taxonNode.getStatus().getSymbol() + " ";
                            }
                        text += taxon.getName() != null ? ((IIdentifiableEntity) HibernateProxyHelper.deproxy(taxon.getName())).getTitleCache() : new String();
                        return text;
                    }catch(Exception e){
                        MessagingUtils.error(getClass(), e);
                    }
                }
            }catch (LazyInitializationException e){
                MessagingUtils.error(getClass(), e);
            }
        }
        else if (element instanceof TaxonNodeDto){
            TaxonNodeDto taxonNode = (TaxonNodeDto) element;
            String text= "";
            if (taxonNode.isUnplaced()){
                text += Character.toString((char)63) + Character.toString((char)63) + " ";
            }
            if (taxonNode.isExcluded()) {
                text +=Character.toString((char)248) + " ";
            }
            if (taxonNode.isDoubtful()) {
                text +=Character.toString((char)63) + " ";
            }
            text += taxonNode.getTitleCache();
            return text;
        }
        return new String();
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public StyledString getStyledText(Object element) {
        //classification
        if(element instanceof Classification){
            return new StyledString(getText(element), StyledString.DECORATIONS_STYLER);
        }
        //taxon node
        else if(element instanceof TaxonNode){
            // determine style base on  user grants
            Styler styler = null;
            if(!CdmStore.currentAuthentiationHasPermission((CdmBase) element, RequiredPermissions.TAXONNODE_EDIT)){
                styler = getNotGrantedStyler();
            }
            return new StyledString(getText(element), styler);
        }
        return new StyledString(getText(element));
    }

    private Styler getNotGrantedStyler() {
        if (notGrantedStyler  == null) {
            notGrantedStyler = new Styler() {
                @Override
                public void applyStyles(TextStyle textStyle) {
                    textStyle.underline = false;
                    textStyle.foreground = StoreUtil.getColor(Resources.COLOR_TEXT_DISABLED);
                }
            };
        }
        return notGrantedStyler;
    }
}
