/**
 * Copyright (C) 2015 EDIT
 * European Distributed Institute of Taxonomy
 * http://www.e-taxonomy.eu
 *
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */
package eu.etaxonomy.taxeditor.view;

import java.util.List;
import java.util.Map;
import java.util.Map.Entry;

import javax.inject.Inject;
import javax.inject.Named;

import org.eclipse.core.commands.Command;
import org.eclipse.core.commands.common.NotDefinedException;
import org.eclipse.e4.core.commands.ECommandService;
import org.eclipse.e4.core.commands.EHandlerService;
import org.eclipse.e4.ui.di.AboutToShow;
import org.eclipse.e4.ui.model.application.commands.MCommand;
import org.eclipse.e4.ui.model.application.commands.MCommandsFactory;
import org.eclipse.e4.ui.model.application.ui.menu.MHandledMenuItem;
import org.eclipse.e4.ui.model.application.ui.menu.MMenu;
import org.eclipse.e4.ui.model.application.ui.menu.MMenuElement;
import org.eclipse.e4.ui.model.application.ui.menu.MMenuFactory;
import org.eclipse.e4.ui.services.IServiceConstants;

import eu.etaxonomy.cdm.model.common.ICdmBase;
import eu.etaxonomy.cdm.persistence.dto.UuidAndTitleCache;
import eu.etaxonomy.taxeditor.l10n.Messages;

/**
 * Generic context menu for opening elements in the taxeditor.
 */
public class CdmViewerContextMenuE4 {

    @Inject
    private EHandlerService handlerService;

    @Inject
    private ECommandService commandService;

    @AboutToShow
    public void aboutToShow(List<MMenuElement> items, @Named(IServiceConstants.ACTIVE_SELECTION) Object selectedObject) {

        if (selectedObject == null){
            return;
        }

        List<UuidAndTitleCache<? extends ICdmBase>> uuidTypes = CdmViewerUtil.transformSelectionToUuidAndTitleCacheList(selectedObject);

        Map<Command, String> enabledCommands = CdmViewerUtil.getAvailableViewers(uuidTypes, commandService, handlerService);

        //check if only one or multiple viewers/commands are available
        if (!enabledCommands.isEmpty()){
            MMenu menu = MMenuFactory.INSTANCE.createMenu();
            menu.setLabel(Messages.CdmViewerContextMenu_OPEN_IN);
            items.add(menu);
            for(Entry<Command, String> entry: enabledCommands.entrySet()){
                String viewerName = entry.getValue();
                Command command = entry.getKey();
                menu.getChildren().add(addCommand(uuidTypes, command, viewerName));
            }
        }
    }


    public MHandledMenuItem addCommand(List<UuidAndTitleCache<? extends ICdmBase>> selectedUuids, Command command, String commandLabel) {
        MHandledMenuItem menuItem = MMenuFactory.INSTANCE.createHandledMenuItem();
        menuItem.setLabel(commandLabel);
        MCommand mCommand = MCommandsFactory.INSTANCE.createCommand();
        mCommand.setElementId(command.getId());
        mCommand.setCommandName(commandLabel);
        try {
            mCommand.setCommandName(command.getName());
        } catch (NotDefinedException e) {
            e.printStackTrace();
        }
        //set params

          menuItem.getTransientData().put(command.getId()+".uuid", selectedUuids);

//        MParameter parameter = MCommandsFactory.INSTANCE.createParameter();
//        parameter.setElementId(command.getId()+".uuid");
//        parameter.setValue(uuid.toString());
//        menuItem.getParameters().add(parameter);

        menuItem.setCommand(mCommand);
        return menuItem;
    }
}