/**
* Copyright (C) 2013 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.view.dataimport.e4;

import java.util.ArrayList;
import java.util.Collection;

import javax.annotation.PostConstruct;
import javax.annotation.PreDestroy;

import org.apache.log4j.Logger;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.e4.ui.di.Focus;
import org.eclipse.jface.viewers.CheckboxTableViewer;
import org.eclipse.jface.wizard.IWizard;
import org.eclipse.jface.wizard.WizardDialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableItem;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.IMemento;
import org.eclipse.ui.forms.widgets.FormToolkit;

import eu.etaxonomy.cdm.api.conversation.ConversationHolder;
import eu.etaxonomy.cdm.api.conversation.IConversationEnabled;
import eu.etaxonomy.cdm.ext.occurrence.OccurenceQuery;
import eu.etaxonomy.cdm.io.specimen.abcd206.in.Abcd206ImportConfigurator;
import eu.etaxonomy.cdm.persistence.hibernate.CdmDataChangeMap;
import eu.etaxonomy.taxeditor.databaseAdmin.wizard.ImportPreferencesWizard;
import eu.etaxonomy.taxeditor.model.IContextListener;
import eu.etaxonomy.taxeditor.model.IPartContentHasDetails;
import eu.etaxonomy.taxeditor.model.IPartContentHasFactualData;
import eu.etaxonomy.taxeditor.model.IPartContentHasSupplementalData;
import eu.etaxonomy.taxeditor.store.CdmStore;
import eu.etaxonomy.taxeditor.workbench.part.IEditorAreaPart;

/**
 * View which shows a list of "data" that can be imported into the CDM
 * @author pplitzner
 * @date 22.08.2017
 *
 * @param <T> the CDM type that will be handled by this view
 */
public abstract class DataImportView<T> implements IPartContentHasFactualData,
IConversationEnabled, IPartContentHasDetails, IPartContentHasSupplementalData, IContextListener, Listener, IEditorAreaPart{

    protected final Logger logger = Logger.getLogger(DataImportView.class);

    protected Collection<T> results = new ArrayList<>();

    protected boolean updated = false;

    protected OccurenceQuery query;

    private static ConversationHolder conversationHolder;

    private SaveImportedSpecimenAction saveImportedSpecimenAction;

//    private Text textClassification;

//    private Classification classification;

    protected Text textReferenceString;

    private Button selectAllButton;
    //private Button selectNoneButton;

    private Button importButton;

    private boolean state;

    private Button openConfigurator;

    private Abcd206ImportConfigurator config;

//    public Classification getClassification() {
//		return classification;
//	}
//
//	public void setClassification(Classification classification) {
//		this.classification = classification;
//	}

	private Button btnBrowseClassification;

	private Button btnClear;

    private final FormToolkit toolkit = new FormToolkit(Display.getCurrent());

    private Table table;
    private Composite composite_3;

    /**
     * Constructs a new DataImportEditor and registers it to listen to context changes
     */
    public DataImportView() {
        CdmStore.getContextManager().addContextListener(this);
    }

    /**
     * Create contents of the view part.
     * @param parent
     */
    @PostConstruct
    public void createPartControl(Composite parent) {
        if(CdmStore.isActive()){
            initConversation();
        }
        else{
            return;
        }
        parent.setLayout(new GridLayout(1, false));
        Composite composite = new Composite(parent, SWT.NONE);
        composite.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1));
//        composite.setBounds(0, 0, 0, 298);
        composite.setLayout(new GridLayout(2, false));

        Composite composite_1 = new Composite(composite, SWT.NONE);
//        gd_composite_1.widthHint = 280;
        composite_1.setLayoutData(new GridData(SWT.FILL, SWT.TOP, false, false, 1, 1));
        composite_1.setLayout(new GridLayout(3, false));

//        Label label = new Label(composite_1, SWT.TOP);
//        label.setText("Classification");
//        label.setLayoutData(new GridData(SWT.LEFT, SWT.CENTER, false, false, 3, 1));
//        textClassification = new Text(composite_1, SWT.BORDER);
//        textClassification.setEnabled(false);
//        GridData gd_textClassification = new GridData(SWT.LEFT, SWT.CENTER, false, false, 1, 1);
////        gd_textClassification.widthHint = 118;
//        textClassification.setLayoutData(new GridData(SWT.FILL, SWT.LEFT, true, true));
//        btnBrowseClassification = new Button(composite_1, SWT.NONE);
//        btnBrowseClassification.setImage(ResourceManager.getPluginImage("eu.etaxonomy.taxeditor.store", "icons/open.gif"));
//        btnBrowseClassification.addListener(SWT.Selection, this);
//        btnClear = new Button(composite_1, SWT.NONE);
//        btnClear.setImage(ResourceManager.getPluginImage("eu.etaxonomy.taxeditor.store", "icons/trash.gif"));
//        btnClear.addListener(SWT.Selection, this);
        //source reference

//        Label labelRef = new Label(composite_1, SWT.NONE);
//        labelRef.setText("Import souce reference");
//        labelRef.setLayoutData(new GridData(SWT.LEFT, SWT.CENTER, false, false, 3, 1));
//        textReferenceString = new Text(composite_1, SWT.NONE);
//        textReferenceString.setEnabled(true);
//        GridData gd_textReferenceString = new GridData(SWT.LEFT, SWT.CENTER, true, true, 3, 1);
//        gd_textReferenceString.widthHint = 229;
//        textReferenceString.setLayoutData(gd_textReferenceString);
//        DateTimeFormatter dtf = DateTimeFormatter.ofPattern("yyyy/MM/dd");
//        LocalDate localDate = LocalDate.now();
//
//        textReferenceString.setText("Specimen Import " + localDate.toString());
        //open configuration
//
//        PreferencesUtil.getLocalAbcdImportConfigurator(false);
//
//        openConfigurator = new Button(composite_1, SWT.PUSH);
//        openConfigurator.setText("Configuration");
//        new Label(composite_1, SWT.NONE);
//        new Label(composite_1, SWT.NONE);
//
//        openConfigurator.addSelectionListener(new SelectionAdapter(){
//            @Override
//            public void widgetSelected(SelectionEvent e) {
//
//                IWizard wizard = new ImportPreferencesWizard();
//                WizardDialog dialog = new WizardDialog(e.display.getActiveShell(), wizard);
//                dialog.open();
//
//            }
//        });

        //checkbox table for result
        Composite composite_2 = new Composite(composite, SWT.NONE);
        composite_2.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1));
        composite_2.setLayout(new GridLayout(2, false));


        CheckboxTableViewer checkboxTableViewer = CheckboxTableViewer.newCheckList(composite_2, SWT.BORDER | SWT.FULL_SELECTION);


        table = checkboxTableViewer.getTable();
        table.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1));
        table.addListener(SWT.Selection, this);
        toolkit.paintBordersFor(table);

        composite_3 = new Composite(composite_2, SWT.NONE);
//        gd_composite_3.heightHint = 48;
        composite_3.setLayoutData(new GridData(SWT.FILL, SWT.FILL, false, false, 1, 1));
        toolkit.paintBordersFor(composite_3);
        composite_3.setLayout(new GridLayout(1, false));
        //toggle button
        selectAllButton = new Button(composite_3, SWT.PUSH);
        selectAllButton.setText("Select all/none");
        new Label(composite_2, SWT.NONE);

//        Composite composite_4 = new Composite(composite_2, SWT.NONE);
//        composite_4.setLayoutData(new GridData(SWT.FILL, SWT.FILL, false, false, 1, 1));
//        composite_4.setBackground(composite.getBackground());
//        toolkit.adapt(composite_4);
//        toolkit.paintBordersFor(composite_4);

        importButton = new Button(composite_2, SWT.PUSH);
        importButton.setSize(48, 25);
        importButton.setText("Import");
        importButton.addSelectionListener(new SelectionAdapter(){
            @Override
            public void widgetSelected(SelectionEvent e) {
                IWizard wizard = new ImportPreferencesWizard();
                WizardDialog dialog = new WizardDialog(e.display.getActiveShell(), wizard);
                int status = dialog.open();
                if (status == 0){
                    saveImportedSpecimenAction.run();
                }
            }
        });
//        new Label(composite_2, SWT.NONE);
//        new Label(composite_2, SWT.NONE);
//        new Label(composite_2, SWT.NONE);
//        new Label(composite_2, SWT.NONE);
//        new Label(composite_2, SWT.NONE);
//        new Label(composite_2, SWT.NONE);
        selectAllButton.addSelectionListener(new SelectionAdapter(){
            @Override
            public void widgetSelected(SelectionEvent e) {
                state = state ? false : true;
                for (TableItem item: getTable().getItems()){
                    item.setChecked(state);
                }

            }
        });
        //import button

        saveImportedSpecimenAction = new SaveImportedSpecimenAction(this);
        initializeToolBar();
    }

    /**
     * Initialize the toolbar.
     */
    private void initializeToolBar() {
        //FIXME E4 migrate
//        IToolBarManager tbm = getViewSite().getActionBars().getToolBarManager();
//        tbm.add(saveImportedSpecimenAction);
    }

    public Table getTable() {
        return table;
    }

    public void setQuery(OccurenceQuery query) {
        this.query = query;
    }

    public void setResults(Collection<T> results) {
        this.results = results;
        updated=false;

    }

    @PreDestroy
    public void dispose() {
        CdmStore.getContextManager().removeContextListener(this);
    }

    @Override
    public void contextRefresh(IProgressMonitor monitor) {
        refresh();
    }

    protected void refresh(){
        if (!updated){
            if (getTable() != null){
                getTable().removeAll();
            }
            if(results!=null){
                for(T item:results){
                    TableItem tableItem = new TableItem(getTable(), SWT.NONE);
                    tableItem.setText(getTextForTableItem(item));
                    tableItem.setData(item);
                }
            }
            updated = true;
        }
    }

    public abstract void query();

    @Override
    public void update(CdmDataChangeMap changeEvents) {
        //nothing
    }

    /**
     * Returns the text label of the given item.
     * @param item the item for which the label should be returns
     * @return the label of the item
     */
    protected abstract String getTextForTableItem(T item);

    @Focus
    public void setFocus() {
        if(getTable()!=null){
            getTable().setFocus();
        }
        //make sure to bind again if maybe in another view the conversation was unbound
        if(getConversationHolder()!=null && !getConversationHolder().isBound()){
            getConversationHolder().bind();
        }
    }

    @Override
    public void contextAboutToStop(IMemento memento, IProgressMonitor monitor) {
    }

    @Override
    public void contextStop(IMemento memento, IProgressMonitor monitor) {
    }

    @Override
    public void contextStart(IMemento memento, IProgressMonitor monitor) {
        initConversation();
    }

    private void initConversation(){
        if(conversationHolder==null){
            conversationHolder = CdmStore.createConversation();
        }
    }

    @Override
    public void workbenchShutdown(IMemento memento, IProgressMonitor monitor) {
        if(getConversationHolder()!=null && getConversationHolder().isBound() && !getConversationHolder().isClosed()) {
            getConversationHolder().close();
        }
    }

    /**
     * @return the conversationHolder
     */
    @Override
    public ConversationHolder getConversationHolder() {
        if(CdmStore.isActive() && conversationHolder==null){
            initConversation();
        }
        return conversationHolder;
    }

    @Override
	public void handleEvent(Event event) {
//	    if(event.widget==btnBrowseClassification){
//	        classification = SelectionDialogFactory.getSelectionFromDialog(Classification.class, event.widget.getDisplay().getActiveShell(), null, null);
//	        if(classification!=null){
//	            textClassification.setText(classification.getTitleCache());
//	        }
//	    }
//	    else if(event.widget==btnClear){
//	        classification = null;
//	        textClassification.setText("");
//	    }

//	    else {
	        if(event.widget == table){
	            TableItem[] item = table.getSelection();
	            if (item.length == 1){
	                Object o = item[0].getData();
	            }
	        }
//	    }
	}

    public String getReferenceString() {
        return textReferenceString.getText();
    }
}
