package eu.etaxonomy.taxeditor.view.dataimport.e4;

import java.time.LocalDate;
import java.time.format.DateTimeFormatter;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import org.eclipse.core.runtime.jobs.Job;
import org.eclipse.jface.action.Action;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableItem;

import eu.etaxonomy.cdm.common.URI;
import eu.etaxonomy.cdm.api.conversation.ConversationHolder;
import eu.etaxonomy.cdm.api.service.ITermService;
import eu.etaxonomy.cdm.ext.occurrence.OccurenceQuery;
import eu.etaxonomy.cdm.ext.occurrence.bioCase.BioCaseResponse;
import eu.etaxonomy.cdm.ext.occurrence.gbif.DataSetResponse;
import eu.etaxonomy.cdm.ext.occurrence.gbif.GbifDataSetProtocol;
import eu.etaxonomy.cdm.ext.occurrence.gbif.GbifResponse;
import eu.etaxonomy.cdm.io.specimen.abcd206.in.Abcd206ImportConfigurator;
import eu.etaxonomy.cdm.io.specimen.gbif.in.GbifImportConfigurator;
import eu.etaxonomy.cdm.model.term.DefinedTermBase;
import eu.etaxonomy.taxeditor.preference.PreferencesUtil;
import eu.etaxonomy.taxeditor.store.CdmStore;
import eu.etaxonomy.taxeditor.view.dataimport.GbifResponseComparator;

/**
 * @author pplitzner
 * @since Oct 4, 2017
 */
public class SaveImportedSpecimenAction extends Action {

    private Comparator<GbifResponse> comparator;

    private DataImportView<?> dataImportView;

    public SaveImportedSpecimenAction(DataImportView<?> dataImportView) {
        super();
        this.dataImportView = dataImportView;
    }

    @Override
    public void run() {
        Table table = dataImportView.getTable();
        Collection<GbifResponse> checkedResults = new HashSet<>();
        Collection<BioCaseResponse> checkedAbcdResults = new HashSet<>();
        Object data;
        Set<String[]> unitIdsGbif = new HashSet<>();
        Set<String[]> unitIdsBioCase = new HashSet<>();
        List<GbifResponse> biocaseResponses = new ArrayList<>();

        for(TableItem item:table.getItems()){
            if(item.getChecked()){
                //Save Specimen
                data = item.getData();
                if(data instanceof GbifResponse){
                    checkedResults.add((GbifResponse)item.getData());
                }
                if(data instanceof BioCaseResponse){
                    checkedAbcdResults.add((BioCaseResponse)data);
                }
            }
        }
        String[] tripleId ;
        for (GbifResponse response:checkedResults){
            if (response != null){
                tripleId = response.getTripleID();

                if (response.getDataSetProtocol()!=null && response.getDataSetProtocol().equals(GbifDataSetProtocol.BIOCASE)){
                    biocaseResponses.add(response);

                }else{
                    unitIdsGbif.add(tripleId);
                }
            }
        }
        Collections.sort(biocaseResponses, getComparator());

        Abcd206ImportConfigurator configuratorAbcd;
        List<Abcd206ImportConfigurator> abcdConfigurators = new ArrayList<>();
        DataSetResponse dataSetResponse;
        OccurenceQuery bioCaseOccurrenceQuery = null;
        URI dataSetUri = null;
        for (BioCaseResponse response:checkedAbcdResults){

            if (!response.getDataSetUri().equals(dataSetUri)){
                dataSetUri = response.getDataSetUri();
                //configuratorAbcd = Abcd206ImportConfigurator.NewInstance(null, null);
                configuratorAbcd = PreferencesUtil.getLastUsedAbcdImportConfigurator();
                if (configuratorAbcd != null){

                    if (configuratorAbcd.getNomenclaturalCode() == null){
                        configuratorAbcd.setNomenclaturalCode(PreferencesUtil.getPreferredNomenclaturalCode());
                    }
                }else{
                    configuratorAbcd = Abcd206ImportConfigurator.NewInstance(null, null);

                    if (configuratorAbcd.getNomenclaturalCode() == null){
                        configuratorAbcd.setNomenclaturalCode(PreferencesUtil.getPreferredNomenclaturalCode());
                    }
                }
                configuratorAbcd.setSourceUri(dataSetUri);
                if (configuratorAbcd.getSourceReference() == null && configuratorAbcd.getSourceRefUuid() == null){
                    DateTimeFormatter dtf = DateTimeFormatter.ofPattern("yyyy/MM/dd");
                    LocalDate localDate = LocalDate.now();

                    String sourceRef ="Specimen Import " + localDate.toString();
                    configuratorAbcd.setSourceReferenceTitle(sourceRef);
                }

                unitIdsBioCase = new HashSet<>();
                bioCaseOccurrenceQuery = new OccurenceQuery(unitIdsBioCase);
                configuratorAbcd.setOccurenceQuery(bioCaseOccurrenceQuery);

//                if (dataImportView.getClassification() != null){
//                    configuratorAbcd.setClassificationUuid(dataImportView.getClassification().getUuid());
//                }
//                if (dataImportView.getReferenceString() != null){
//                    configuratorAbcd.setSourceReferenceTitle(dataImportView.getReferenceString());
//                }
                abcdConfigurators.add(configuratorAbcd);


            }
            tripleId = response.getTripleID();
            unitIdsBioCase.add(tripleId);

        }


        //                if (!unitIdsBioCase.isEmpty()){
        //                    configuratorAbcd = Abcd206ImportConfigurator.NewInstance(dataSetUri, null, false);
        //                    configuratorAbcd.addMediaAsMediaSpecimen(true);
        //                    configuratorAbcd.setIgnoreAuthorship(true);
        //                    configuratorAbcd.setMoveNewTaxaToDefaultClassification(false);
        //                    if (configuratorAbcd != null){
        //                        abcdConfigurators.add(configuratorAbcd);
        //                    }
        ////                    unitIdsBioCase = new HashSet<String[]>();
        //                    bioCaseOccurrenceQuery = new OccurenceQuery(unitIdsBioCase);
        //                    configuratorAbcd.setOccurenceQuery(bioCaseOccurrenceQuery);
        //                    configuratorAbcd.setGetSiblings(true);
        //                    if (dataImportView.getClassification() != null){
        //                        configuratorAbcd.setClassificationUuid(dataImportView.getClassification().getUuid());
        //                    }
        //                }

        //                for (GbifResponse response: biocaseResponses){
        //                	try {
        //						dataSetResponse = new GbifQueryServiceWrapper().queryOriginalDataSet(response);
        //						dataSetUri = dataSetResponse.getEndpoint();
        //					} catch (ClientProtocolException e) {
        //						// TODO Auto-generated catch block
        //						e.printStackTrace();
        //					} catch (IOException e) {
        //						// TODO Auto-generated catch block
        //						e.printStackTrace();
        //					}
        //                	if (!response.getDataSetUri().equals(dataSetUri)){
        //                		configuratorAbcd = Abcd206ImportConfigurator.NewInstance(dataSetUri, null, false);
        //                		configuratorAbcd.addMediaAsMediaSpecimen(true);
        //                		configuratorAbcd.setIgnoreAuthorship(true);
        //                		configuratorAbcd.setMoveNewTaxaToDefaultClassification(false);
        //                		if (configuratorAbcd != null){
        //                			abcdConfigurators.add(configuratorAbcd);
        //                		}
        //                		unitIdsBioCase = new HashSet<String[]>();
        //                		bioCaseOccurrenceQuery = new OccurenceQuery(unitIdsBioCase);
        //                		configuratorAbcd.setOccurenceQuery(bioCaseOccurrenceQuery);
        //                		configuratorAbcd.setGetSiblings(true);
        //                		if (dataImportView.getClassification() != null){
        //                			configuratorAbcd.setClassificationUuid(dataImportView.getClassification().getUuid());
        //                		}
        //
        //                	}
        //                	tripleId = response.getTripleID();
        //                	unitIdsBioCase.add(tripleId);
        //                }


        //  configurator.setQuery(query);
        if (!checkedAbcdResults.isEmpty()){
            Job bioCaseJob = CdmStore.getImportManager().createIOServiceJob(abcdConfigurators);
            CdmStore.getImportManager().run(bioCaseJob);
        }

        if (!unitIdsGbif.isEmpty()){
            OccurenceQuery query = new OccurenceQuery(unitIdsGbif);
            GbifImportConfigurator configurator = GbifImportConfigurator.newInstance(query);
            configurator.setSourceReferenceTitle("Import Gbif data");

            Job gbifJob = CdmStore.getImportManager().createIOServiceJob(configurator);
            CdmStore.getImportManager().run(gbifJob);
        }
    }

    private void persistTerm(DefinedTermBase<?> term, ITermService termService, ConversationHolder conversation){
        if(term!=null){
            //if the term does not exist in the DB save it
            if(termService.find(term.getUuid())==null){
                termService.saveOrUpdate(term);
            }
        }
    }

    @Override
    public String getText() {
        return "Import";
    }

    private Comparator<GbifResponse> getComparator(){
        if (comparator == null){
            comparator = new GbifResponseComparator();
        }
        return comparator;
    }
}