/**
* Copyright (C) 2013 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.view.dataimport.e4;

import java.io.IOException;
import java.io.InputStream;

import javax.inject.Inject;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.core.runtime.jobs.Job;
import org.eclipse.e4.core.contexts.ContextInjectionFactory;
import org.eclipse.e4.core.contexts.IEclipseContext;
import org.eclipse.e4.ui.model.application.MApplication;
import org.eclipse.e4.ui.model.application.ui.basic.MPart;
import org.eclipse.e4.ui.model.application.ui.basic.MPartStack;
import org.eclipse.e4.ui.workbench.modeling.EModelService;
import org.eclipse.e4.ui.workbench.modeling.EPartService;
import org.eclipse.e4.ui.workbench.modeling.EPartService.PartState;
import org.eclipse.jface.wizard.Wizard;
import org.eclipse.swt.widgets.Display;

import eu.etaxonomy.cdm.database.DbSchemaValidation;
import eu.etaxonomy.cdm.ext.common.ServiceWrapperBase;
import eu.etaxonomy.cdm.ext.occurrence.OccurenceQuery;
import eu.etaxonomy.cdm.ext.occurrence.bioCase.BioCaseQueryServiceWrapper;
import eu.etaxonomy.cdm.io.specimen.abcd206.in.Abcd206ImportConfigurator;
import eu.etaxonomy.cdm.io.specimen.abcd206.in.AbcdParseUtility;
import eu.etaxonomy.cdm.io.specimen.abcd206.in.SpecimenImportReport;
import eu.etaxonomy.cdm.io.specimen.abcd206.in.UnitAssociationWrapper;
import eu.etaxonomy.taxeditor.store.AppModelId;
import eu.etaxonomy.taxeditor.store.CdmStore;
import eu.etaxonomy.taxeditor.view.dataimport.QueryType;
import eu.etaxonomy.taxeditor.workbench.WorkbenchUtility;
import eu.etaxonomy.taxeditor.workbench.part.IEditorAreaPart;

/**
 * Wizard for querying specimen provider.
 * @author pplitzner
 * @date 11.09.2013
 */
public class SpecimenSearchWizard extends Wizard {

    private OccurenceQuery query = null;
    private SpecimenProviderSelectionWizardPage providerSelectionPage;
    private SpecimenSearchWizardPage searchPage;

    @Inject
    private EPartService partService;

    @Inject
    private MApplication application;

    @Inject
    private EModelService modelService;

    private IEclipseContext context;

    /**
     * Creates a new SpecimenSearchWizard
     */
    @Inject
    public SpecimenSearchWizard(IEclipseContext context) {
        this.context = context;
        //default constructor needed for RCP extension points

        //check if connected to a data source. If not this will open an error dialog
        CdmStore.getCurrentApplicationConfiguration();
        this.setWindowTitle("Search Specimens");
    }

    @Override
    public boolean performFinish() {
        searchPage.getController().saveLastSate();
        providerSelectionPage.getController().saveLastState();
        ServiceWrapperBase serviceWrapper;
        DataImportView<?> dataImportView = null;
        boolean biocase = providerSelectionPage.getQueryType().equals(QueryType.BIOCASE);
        query = searchPage.getQuery(biocase);

        //query for specific dataset...<DataSets><DataSet><Metadata><Description><Representation language="EN"><Title>...
        Abcd206ImportConfigurator configurator =  CdmStore.getImportManager().AbcdConfigurator();
        configurator.setOccurenceQuery(query);
        configurator.setSourceUri(providerSelectionPage.getEndPoint());
        configurator.setAddMediaAsMediaSpecimen(true);
        configurator.setDbSchemaValidation(DbSchemaValidation.CREATE);
        configurator.setIgnoreAuthorship(true);
        configurator.setMoveNewTaxaToDefaultClassification(false);

        try {
            serviceWrapper = providerSelectionPage.getQueryServiceWrapper();
            InputStream stream = null;
            MPart part = null;
            if (providerSelectionPage.getQueryType().equals(QueryType.BIOCASE)){
                stream =((BioCaseQueryServiceWrapper) serviceWrapper).query(query, configurator.getSourceUri());
                SpecimenImportReport report = new SpecimenImportReport();
                UnitAssociationWrapper unitNodesList = AbcdParseUtility.parseUnitsNodeList(stream, report);

                part = partService.createPart(AppModelId.PARTDESCRIPTOR_EU_ETAXONOMY_TAXEDITOR_VIEW_DATAIMPORT_E4_SPECIMENIMPORTVIEWE4);
                part = partService.showPart(part, PartState.ACTIVATE);
                SpecimenImportView specimenImportView = (SpecimenImportView) part.getObject();
                specimenImportView.setEndpoint(providerSelectionPage.getEndPoint());
                specimenImportView.setQuery(query);

                dataImportView = specimenImportView;

            } else{
                part = partService.createPart(AppModelId.PARTDESCRIPTOR_EU_ETAXONOMY_TAXEDITOR_VIEW_DATAIMPORT_E4_GBIFRESPONSEIMPORTVIEWE4);
                part = partService.showPart(part, PartState.ACTIVATE);
                GbifResponseImportView gbifImportView = (GbifResponseImportView) part.getObject();
                gbifImportView.setQuery(query);

                dataImportView = gbifImportView;

            }
            if(part.getObject() instanceof IEditorAreaPart){
                MPartStack editorAreaPartStack = WorkbenchUtility.getEditorAreaPartStack(application, modelService);
                if(editorAreaPartStack!=null){
                    editorAreaPartStack.getChildren().add(part);
                }
                part = partService.showPart(part, PartState.VISIBLE);
            }
        } catch (IOException e) {
            e.printStackTrace();
        }
       // CdmStore.getImportManager().run(configurator);

//        try {
//            switch (providerSelectionPage.getQueryType()) {
//            case BIOCASE:
//                SpecimenImportView specimenImportView = (SpecimenImportView) TaxeditorEditorPlugin.getDefault().getWorkbench().getActiveWorkbenchWindow().getActivePage()
//                        .showView(SpecimenImportView.ID, "query_"+query.toString().replaceAll(",", ""),IWorkbenchPage.VIEW_ACTIVATE);
//                specimenImportView.setEndpoint(providerSelectionPage.getEndPoint());
//                dataImportView = specimenImportView;
//                break;
//            case GBIF:
//                dataImportView = (GbifResponseImportView) TaxeditorEditorPlugin.getDefault().getWorkbench().getActiveWorkbenchWindow().getActivePage()
//                .showView(GbifResponseImportView.ID, "query_"+query.toString().replaceAll(",", ""),IWorkbenchPage.VIEW_ACTIVATE);
//                break;
//            }
//        } catch ( PartInitException e ) {
//            logger.error("Could not open import view for query: " + query.toString(), e);
//        }
//        if(dataImportView!=null){
//            dataImportView.setQuery(query);
////            dataImportView.query();
////            Display.getDefault().asyncExec(new Runnable() {
////
////                @Override
////                public void run() {
////                    CdmStore.getContextManager().notifyContextRefresh();
////                }
////            });
            Job queryJob = new QueryJob("Query specimen provider", dataImportView);
           queryJob.schedule();

        return true;
    }

    @Override
    public void addPages() {
        addPage(providerSelectionPage);
        addPage(searchPage);
    }

    public void init() {
        providerSelectionPage = ContextInjectionFactory.make(SpecimenProviderSelectionWizardPage.class, context);
//      new SpecimenProviderSelectionWizardPage("Select specimen provider");
//      searchPage = new SpecimenSearchWizardPage("Specimen Search");
        searchPage = ContextInjectionFactory.make(SpecimenSearchWizardPage.class, context);
    }

    private class QueryJob extends Job{

        private final DataImportView<?> view;

        public QueryJob(String name, DataImportView<?> view) {
            super(name);
            this.view = view;
        }

        @Override
        protected IStatus run(IProgressMonitor monitor) {
            view.getConversationHolder().bind();
            view.query();
            Display.getDefault().asyncExec(new Runnable() {

                @Override
                public void run() {
                    CdmStore.getContextManager().notifyContextRefresh();
                }
            });
            return Status.OK_STATUS;
        }

    }
}
