/**
* Copyright (C) 2013 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.view.dataimport.e4;

import java.util.Calendar;

import javax.inject.Inject;

import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.widgets.Composite;

import eu.etaxonomy.cdm.ext.occurrence.OccurenceQuery;
import eu.etaxonomy.taxeditor.view.search.specimen.SpecimenSearchController;


/**
 * Wizard page for entering search parameters
 * @author pplitzner
 * @date 12.09.2013
 *
 */
public class SpecimenSearchWizardPage extends WizardPage{

    private SpecimenSearchController specimenSearchController;

    @Inject
    public SpecimenSearchWizardPage() {
        super("Specimen Search");
        setDescription("Specify search parameters for external search query.");
        setTitle("Search Specimens");
    }

    @Override
    public void createControl(Composite parent) {
        specimenSearchController = SpecimenSearchController.getInstance(parent);
        setControl(specimenSearchController.getComposite());
    }

    /**
     * Returns an {@link OccurenceQuery} filled with the parameters defined in this wizard page
     * @return
     */
    public OccurenceQuery getQuery(boolean biocase) {
        String accessionNumber = specimenSearchController.getAccessionNumber();
        String collector = specimenSearchController.getCollector();
        String collectorsNumber = specimenSearchController.getCollectorNumber();
        String country = specimenSearchController.getCountry(biocase);
        Calendar dateFrom = specimenSearchController.getDateFrom();
        Calendar dateTo = specimenSearchController.getDateTo();
        String herbarium = specimenSearchController.getHerbarium();
        String locality = specimenSearchController.getLocality();
        String taxonName = specimenSearchController.getTaxonName();
        OccurenceQuery query = new OccurenceQuery(taxonName, collector, collectorsNumber, accessionNumber, herbarium, country, locality, dateFrom, dateTo, false);
        query.higherTaxon =specimenSearchController.getHigherTaxon();
        return query;
    }

    /**
     * @return the specimenSearchController
     */
    public SpecimenSearchController getController() {
        return specimenSearchController;
    }



}
