/**
 * Copyright (C) 2011 EDIT
 * European Distributed Institute of Taxonomy
 * http://www.e-taxonomy.eu
 *
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */
package eu.etaxonomy.taxeditor.view.e4.details;

import java.util.Collection;
import java.util.EnumSet;
import java.util.HashSet;
import java.util.Set;

import javax.inject.Inject;

import org.apache.lucene.search.BulkScorer;
import org.eclipse.e4.core.di.annotations.Optional;
import org.eclipse.e4.ui.di.UIEventTopic;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.TreeNode;
import org.eclipse.swt.SWT;
import org.springframework.security.core.GrantedAuthority;

import eu.etaxonomy.cdm.api.facade.DerivedUnitFacade;
import eu.etaxonomy.cdm.api.facade.DerivedUnitFacadeNotSupportedException;
import eu.etaxonomy.cdm.ext.occurrence.gbif.GbifResponse;
import eu.etaxonomy.cdm.model.agent.Person;
import eu.etaxonomy.cdm.model.agent.Team;
import eu.etaxonomy.cdm.model.agent.TeamOrPersonBase;
import eu.etaxonomy.cdm.model.common.CdmBase;
import eu.etaxonomy.cdm.model.common.Marker;
import eu.etaxonomy.cdm.model.description.CategoricalData;
import eu.etaxonomy.cdm.model.description.Character;
import eu.etaxonomy.cdm.model.description.DescriptionBase;
import eu.etaxonomy.cdm.model.description.DescriptionElementBase;
import eu.etaxonomy.cdm.model.description.Distribution;
import eu.etaxonomy.cdm.model.description.PolytomousKeyNode;
import eu.etaxonomy.cdm.model.description.SpecimenDescription;
import eu.etaxonomy.cdm.model.description.TaxonDescription;
import eu.etaxonomy.cdm.model.media.Media;
import eu.etaxonomy.cdm.model.molecular.DnaSample;
import eu.etaxonomy.cdm.model.molecular.Sequence;
import eu.etaxonomy.cdm.model.molecular.SingleRead;
import eu.etaxonomy.cdm.model.name.TaxonName;
import eu.etaxonomy.cdm.model.occurrence.DerivedUnit;
import eu.etaxonomy.cdm.model.occurrence.FieldUnit;
import eu.etaxonomy.cdm.model.occurrence.MediaSpecimen;
import eu.etaxonomy.cdm.model.occurrence.SpecimenOrObservationType;
import eu.etaxonomy.cdm.model.permission.CRUD;
import eu.etaxonomy.cdm.model.permission.Group;
import eu.etaxonomy.cdm.model.permission.User;
import eu.etaxonomy.cdm.model.reference.Reference;
import eu.etaxonomy.cdm.model.taxon.SecundumSource;
import eu.etaxonomy.cdm.model.taxon.Synonym;
import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.cdm.model.taxon.TaxonBase;
import eu.etaxonomy.cdm.model.taxon.TaxonNode;
import eu.etaxonomy.cdm.model.taxon.TaxonRelationship;
import eu.etaxonomy.cdm.model.term.DefinedTermBase;
import eu.etaxonomy.cdm.model.term.TermNode;
import eu.etaxonomy.cdm.model.term.TermTree;
import eu.etaxonomy.cdm.model.term.TermType;
import eu.etaxonomy.cdm.model.term.TermVocabulary;
import eu.etaxonomy.cdm.persistence.dto.TermNodeDto;
import eu.etaxonomy.cdm.persistence.dto.TermTreeDto;
import eu.etaxonomy.cdm.persistence.hibernate.CdmDataChangeMap;
import eu.etaxonomy.taxeditor.editor.IBulkEditor;
import eu.etaxonomy.taxeditor.editor.IDistributionEditorPart;
import eu.etaxonomy.taxeditor.event.WorkbenchEventConstants;
import eu.etaxonomy.taxeditor.l10n.Messages;
import eu.etaxonomy.taxeditor.model.AbstractUtility;
import eu.etaxonomy.taxeditor.model.FeatureNodeContainer;
import eu.etaxonomy.taxeditor.model.IDerivedUnitFacadePart;
import eu.etaxonomy.taxeditor.model.MessagingUtils;
import eu.etaxonomy.taxeditor.model.PolytomousKeyRelationship;
import eu.etaxonomy.taxeditor.preference.NameDetailsConfigurator;
import eu.etaxonomy.taxeditor.preference.PreferencesUtil;
import eu.etaxonomy.taxeditor.store.CdmStore;
import eu.etaxonomy.taxeditor.store.StoreUtil;
import eu.etaxonomy.taxeditor.store.UsageTermCollection;
import eu.etaxonomy.taxeditor.termtree.e4.ICharacterEditor;
import eu.etaxonomy.taxeditor.ui.element.ICdmFormElement;
import eu.etaxonomy.taxeditor.ui.element.IEnableableFormElement;
import eu.etaxonomy.taxeditor.ui.element.RootElement;
import eu.etaxonomy.taxeditor.ui.section.agent.PersonDetailSection;
import eu.etaxonomy.taxeditor.ui.section.agent.TeamDetailSection;
import eu.etaxonomy.taxeditor.ui.section.agent.TeamOrPersonBaseDetailSection;
import eu.etaxonomy.taxeditor.ui.section.common.ReferencedEntityDetailSection;
import eu.etaxonomy.taxeditor.ui.section.description.DescriptionDetailSection;
import eu.etaxonomy.taxeditor.ui.section.description.DescriptionElementDetailElement;
import eu.etaxonomy.taxeditor.ui.section.description.DescriptionElementDetailSection;
import eu.etaxonomy.taxeditor.ui.section.description.DescriptionElementMediaSection;
import eu.etaxonomy.taxeditor.ui.section.description.DescriptionElementSourceSection;
import eu.etaxonomy.taxeditor.ui.section.description.DescriptionSourceSection;
import eu.etaxonomy.taxeditor.ui.section.description.MediaDetailsSection;
import eu.etaxonomy.taxeditor.ui.section.description.NaturalLanguageSection;
import eu.etaxonomy.taxeditor.ui.section.description.ScopeSection;
import eu.etaxonomy.taxeditor.ui.section.description.detail.DistributionDetailElement;
import eu.etaxonomy.taxeditor.ui.section.feature.CharacterDetailSection;
import eu.etaxonomy.taxeditor.ui.section.feature.CharacterNodeDetailSection;
import eu.etaxonomy.taxeditor.ui.section.feature.CharacterNodeDtoDetailSection;
import eu.etaxonomy.taxeditor.ui.section.feature.FeatureDistributionDetailSection;
import eu.etaxonomy.taxeditor.ui.section.feature.TermNodeDetailSection;
import eu.etaxonomy.taxeditor.ui.section.feature.TermTreeDetailSection;
import eu.etaxonomy.taxeditor.ui.section.feature.TermTreeDetailSectionForNode;
import eu.etaxonomy.taxeditor.ui.section.grantedAuthority.GrantedAuthorityDetailSection;
import eu.etaxonomy.taxeditor.ui.section.group.GroupDetailSection;
import eu.etaxonomy.taxeditor.ui.section.group.MemberDetailSection;
import eu.etaxonomy.taxeditor.ui.section.key.PolytomousKeyDetailSection;
import eu.etaxonomy.taxeditor.ui.section.key.PolytomousKeyNodeDetailSection;
import eu.etaxonomy.taxeditor.ui.section.name.NameRelationshipDetailSection;
import eu.etaxonomy.taxeditor.ui.section.name.NomenclaturalStatusSection;
import eu.etaxonomy.taxeditor.ui.section.name.NonViralNameDetailSection;
import eu.etaxonomy.taxeditor.ui.section.name.TypeDesignationSection;
import eu.etaxonomy.taxeditor.ui.section.occurrence.DerivedUnitBaseDetailSection;
import eu.etaxonomy.taxeditor.ui.section.occurrence.DerivedUnitGeneralDetailSection;
import eu.etaxonomy.taxeditor.ui.section.occurrence.DeterminationDetailSection;
import eu.etaxonomy.taxeditor.ui.section.occurrence.FieldUnitFacadeDetailSection;
import eu.etaxonomy.taxeditor.ui.section.occurrence.FieldUnitFacadeGeneralDetailSection;
import eu.etaxonomy.taxeditor.ui.section.occurrence.GatheringEventDetailSection;
import eu.etaxonomy.taxeditor.ui.section.occurrence.association.DerivedUnitTypeDesignationSection;
import eu.etaxonomy.taxeditor.ui.section.occurrence.association.TaxonAssociationDetailSection;
import eu.etaxonomy.taxeditor.ui.section.occurrence.dna.DnaQualityDetailSection;
import eu.etaxonomy.taxeditor.ui.section.occurrence.dna.DnaSampleGeneralDetailSection;
import eu.etaxonomy.taxeditor.ui.section.occurrence.dna.DnaSamplePreparationPreservationSection;
import eu.etaxonomy.taxeditor.ui.section.occurrence.dna.SampleDesignationDetailSection;
import eu.etaxonomy.taxeditor.ui.section.occurrence.dna.SequenceContigFileCollectionDetailSection;
import eu.etaxonomy.taxeditor.ui.section.occurrence.dna.SequenceGeneralDetailSection;
import eu.etaxonomy.taxeditor.ui.section.occurrence.dna.SequenceReferenceCollectionDetailSection;
import eu.etaxonomy.taxeditor.ui.section.occurrence.dna.SingleReadGeneralDetailSection;
import eu.etaxonomy.taxeditor.ui.section.occurrence.dna.SingleReadPherogramCollectionDetailSection;
import eu.etaxonomy.taxeditor.ui.section.occurrence.dna.TissueSampleGeneralDetailSection;
import eu.etaxonomy.taxeditor.ui.section.occurrence.media.MediaSpecimenGeneralDetailSection;
import eu.etaxonomy.taxeditor.ui.section.reference.NomenclaturalSourceDetailSection;
import eu.etaxonomy.taxeditor.ui.section.reference.ReferenceDetailSection;
import eu.etaxonomy.taxeditor.ui.section.reference.SecundumSourceSection;
import eu.etaxonomy.taxeditor.ui.section.supplemental.RightsSection;
import eu.etaxonomy.taxeditor.ui.section.supplemental.identifier.DerivedUnitFacadeIdentifierSection;
import eu.etaxonomy.taxeditor.ui.section.taxon.ParsingMessagesSection;
import eu.etaxonomy.taxeditor.ui.section.taxon.TaxonBaseDetailSection;
import eu.etaxonomy.taxeditor.ui.section.taxon.TaxonRelationshipDetailSection;
import eu.etaxonomy.taxeditor.ui.section.user.GroupsByUserDetailSection;
import eu.etaxonomy.taxeditor.ui.section.user.UserDetailSection;
import eu.etaxonomy.taxeditor.ui.section.userecords.UseRecordDetailSection;
import eu.etaxonomy.taxeditor.ui.section.vocabulary.DefinedTermDetailSection;
import eu.etaxonomy.taxeditor.ui.section.vocabulary.TermMediaSection;
import eu.etaxonomy.taxeditor.ui.section.vocabulary.TermVocabularyDetailSection;
import eu.etaxonomy.taxeditor.view.detail.CdmSectionPart;
import eu.etaxonomy.taxeditor.view.e4.AbstractCdmDataViewer;

/**
 * The DetailsViewer handles the content of the details view (
 * {@link DetailsPartE4}).<br>
 * Depending on the type of the selected element the section of the details view
 * are created.
 *
 * @author pplitzner
 * @date 18.07.2017
 */
public class DetailsViewerE4 extends AbstractCdmDataViewer {

    private ISelection selection;
    private boolean detailsEnabled = true;
    private Object activePart;

    public boolean isDetailsEnabled() {
		return detailsEnabled;
	}

	public void setDetailsEnabled(boolean detailsEnabled) {
		this.detailsEnabled = detailsEnabled;
	}

	public void setInput(Object input, Object activePart) {
	    this.activePart = activePart;
        setInput(input);

    }
    // START HACK TO MAKE THE DERIVED UNIT FACADE WORK
    // since we are getting implementations of DerivedUnitBase from the bulk
    // editor
    // and not derived unit facade objects,

    @Override
    public void setInput(Object input) {

        if(input instanceof TreeNode){
            input = ((TreeNode) input).getValue();
        }
        if(input instanceof GbifResponse){
            input = ((GbifResponse) input).getDerivedUnitFacade();
        }
        if (input instanceof DerivedUnit) {
            DerivedUnit derivedUnit = (DerivedUnit)input;
            if ((derivedUnit instanceof MediaSpecimen
                    ||  (derivedUnit instanceof DnaSample && derivedUnit.getRecordBasis() == SpecimenOrObservationType.DnaSample ))
                && !(this.activePart instanceof IBulkEditor)){
                    //do nothing
            }else{
                try {
                    input = DerivedUnitFacade.NewInstance(derivedUnit,
                            PreferencesUtil.getDerivedUnitConfigurator());
                } catch (DerivedUnitFacadeNotSupportedException e) {
                    MessagingUtils.error(getClass(), e); //nothing should happen, later on the Details View will show an according information that it is not possible to open this data
                }
            }
        }
        else if(input instanceof FieldUnit){
            input = DerivedUnitFacade.NewInstance(SpecimenOrObservationType.FieldUnit, (FieldUnit) input);
        }
        super.setInput(input);
        
        
    }

    @Override
    protected void markViewPartDirty() {
        if (getInput() instanceof DerivedUnitFacade) {
            if(part instanceof IDerivedUnitFacadePart){
                part.changed(getInput());
            }
            else{
                DerivedUnitFacade facade = (DerivedUnitFacade) getInput();
                if(facade.innerDerivedUnit()!=null){
                    part.changed(facade.innerDerivedUnit());
                }
                else if(facade.innerFieldUnit()!=null){
                    part.changed(facade.innerFieldUnit());
                }
            }
        }
        else{
            super.markViewPartDirty();
        }
    }

    @Override
    protected void showParts() {
        // FIXME (CM) : Need to clean out this code.
        // Too much type checking to decide which detail view to display.
        // Need to build in a mechanism where navigators / editors are 'aware'
        // of the corresponding detail viewer.

        Object input = getInput();
        if (input == null){
            return;
        }
        destroySections();
        if (input instanceof TaxonBase) {
            if(((TaxonBase<?>) input).getName()==null){
                createEmptySection(Messages.DetailsViewerE4_TAXON_HAS_NO_NAME, rootElement);
            }
            else{
                createTaxonSections(rootElement);
            }
        } else if (input instanceof TaxonName) {
            createNameSections(rootElement);
        } else if (input instanceof Reference) {
            createReferenceSections(rootElement);

        } else if (input instanceof Team) {
            createTeamDetailSection(rootElement);

        } else if (input instanceof Person) {
            createPersonDetailSection(rootElement);

        } else if (input instanceof TeamOrPersonBase) {
            createTeamOrPersonBaseDetailSection(rootElement);

        } else if (input instanceof DescriptionBase) {
            if ((((DescriptionBase)input).isComputed() || ((DescriptionBase)input).isCloneForSource())&& PreferencesUtil.isComputedDesciptionHandlingDisabled()){
                this.setDetailsEnabled(false);
            }
            if(input instanceof SpecimenDescription){
                //TODO: add more sections to DetailsView for SpecimenDescription
                createSpecimenDescriptionSection(rootElement);
            }
            else{
                Set<Marker> descriptionMarkers = ((DescriptionBase<?>) input).getMarkers();

                Boolean isUseDescription = false;
                for (Marker marker : descriptionMarkers) {
                    if (marker.getMarkerType()!= null && marker.getMarkerType().getUuid().equals(UsageTermCollection.uuidUseMarkerType)) {
                        isUseDescription = true;
                    }
                }
                if (((DescriptionBase<?>) input).isImageGallery()) {
                    createImageGallerySection(rootElement);

                } else if (isUseDescription) {
                    createUseDescriptionSection(rootElement);
                } else if (input instanceof TaxonDescription){
                    createTaxonDescriptionSection(rootElement, !(((DescriptionBase) input).isComputed() && PreferencesUtil.isComputedDesciptionHandlingDisabled()));

                } else{
                    createDescriptionSection(rootElement, false, !(((DescriptionBase) input).isComputed() && PreferencesUtil.isComputedDesciptionHandlingDisabled()));
                }
            }
        } else if (input instanceof DescriptionElementBase){
            Set<Marker> descriptionMarkers = new HashSet<>();
            if(((DescriptionElementBase) input).getInDescription()!=null) {
                descriptionMarkers = ((DescriptionElementBase) input).getInDescription().getMarkers();
            }

            Boolean isUseDescription = false;
            for (Marker marker : descriptionMarkers) {
                if (marker.getMarkerType()!= null && marker.getMarkerType().getUuid().equals(UsageTermCollection.uuidUseMarkerType)) {
                    isUseDescription = true;
                }
            }
            if (((DescriptionElementBase)input).getInDescription() != null && ((DescriptionElementBase)input).getInDescription().isComputed() && PreferencesUtil.isComputedDesciptionHandlingDisabled()){
                this.setDetailsEnabled(false);
            }
            if (isUseDescription == true && input instanceof CategoricalData) {
                createUseRecordSection(rootElement);
            } else {
                createDescriptionElementSection(rootElement, activePart);
            }

        } else if (input instanceof Media) {
            createMediaElementSection(rootElement);
        }
        //FIXME hack to show more informative message to the user when
        //DerivedUnit has more than one FieldUnit #4363
        else if (input instanceof DerivedUnit && !(input instanceof MediaSpecimen) && !(input instanceof DnaSample)) {
            DerivedUnit derivedUnit = (DerivedUnit)input;
            try {
                input = DerivedUnitFacade.NewInstance(derivedUnit,
                        PreferencesUtil.getDerivedUnitConfigurator());
            } catch (DerivedUnitFacadeNotSupportedException e) {
                createEmptySection(e.getMessage(), rootElement);
            }
        } else if (input instanceof DerivedUnitFacade) {
            DerivedUnitFacade facade = (DerivedUnitFacade) input;
            if(facade.getType() == SpecimenOrObservationType.FieldUnit){
                if(facade.innerFieldUnit() == null){
                    MessagingUtils.error(DetailsViewerE4.class, "FieldUnit of facade is null", null); //$NON-NLS-1$
                }
                else{
                    createFieldUnitFacadeSection(rootElement);
                }
            }
            else if(facade.getType() == SpecimenOrObservationType.PreservedSpecimen){
                if(facade.innerDerivedUnit() == null){
                    MessagingUtils.error(DetailsViewerE4.class, "DerivedUnit of facade is null", null); //$NON-NLS-1$
                }
                else{
                    createDerivedUnitBaseElementSection(rootElement);
                }
            }
            else if(facade.getType() == SpecimenOrObservationType.TissueSample){
                //TissueSample should only be created by using it's own class
                //in future using only one class with different SpecimenOrObservationTypes is desired
                createTissueSampleSection(rootElement);
            }else if (facade.innerDerivedUnit() instanceof DnaSample){
                createEmptySection("DNA sample not supported by specimen list editor", rootElement);
            }else if (facade.innerDerivedUnit() instanceof MediaSpecimen){
                createEmptySection("Image specimen not supported by specimen list editor", rootElement);
            }else if(facade.getType()==SpecimenOrObservationType.DnaSample){

                if(facade.innerDerivedUnit().getRecordBasis()==SpecimenOrObservationType.TissueSample){
                    //this case seems to be very unlikely as innerDerivedUnit in real cases should more or less always have the same type
                    createTissueSampleSection(rootElement);
                }
                else if(facade.innerDerivedUnit().getRecordBasis()==SpecimenOrObservationType.DnaSample){
                    createDnaSampleSection(rootElement);
                }
            }else{
                createDerivedUnitBaseElementSection(rootElement);
            }
        } else if (input instanceof DnaSample){
            DnaSample dnaSample = (DnaSample)input;
            if(dnaSample.getRecordBasis()==SpecimenOrObservationType.TissueSample){
                createTissueSampleSection(rootElement);
            }
            else if(dnaSample.getRecordBasis()==SpecimenOrObservationType.DnaSample){
                createDnaSampleSection(rootElement);
            }
        } else if (input instanceof MediaSpecimen){
            createMediaSpecimenSection(rootElement);
        }
        else if(input instanceof Sequence){
            createSequenceSection(rootElement);
        }
        else if(input instanceof SingleRead){
            createSingleReadSection(rootElement);
        }
        else if (input instanceof FeatureNodeContainer) {
            createFeatureDistributionSection(rootElement);

        } else if (input instanceof TermNode) {
            TermType type= ((TermNode<?>) input).getTermType();
            if(type.equals(TermType.Character) ){
                createCharacterNodeSection(rootElement);
            }
            else if(type.equals(TermType.Feature)){
                createFeatureNodeSection(rootElement);
            }

        } else if (input instanceof TermTree) {
            createFeatureTreeSection(rootElement);
        }  else if (input instanceof TermNodeDto) {
            TermType type= ((TermNodeDto) input).getTerm().getTermType();
            if(type.equals(TermType.Character) && activePart instanceof ICharacterEditor){
                createCharacterNodeDtoSection(rootElement);
            }
            else if(type.equals(TermType.Feature) || type.equals(TermType.Character)){
                createFeatureNodeSection(rootElement);
            }

        } else if (input instanceof TermTreeDto) {
            createFeatureTreeSectionForDto(rootElement);

        } else if (input instanceof Character) {
            createCharacterSection(rootElement);

        }else if ((input instanceof PolytomousKeyNode) || (input instanceof PolytomousKeyRelationship)) {
            createPolytomousKeyNodeSection(rootElement);
        } else if (input instanceof User) {
            createUserSection(rootElement);
        } else if (input instanceof Group) {
            createGroupSection(rootElement);
        } else if (input instanceof TaxonRelationship && ((TaxonRelationship)input).getType()!=null
                && (((TaxonRelationship)input).getType().isAnyMisappliedName() || ((TaxonRelationship)input).getType().isAnySynonym() || ((TaxonRelationship)input).getType().isMisappliedName())){
            createMisappliedNameSection(rootElement);
        }else if (input instanceof TaxonRelationship ) {
            createTaxonRelationshipSection(rootElement);
        } else if (input instanceof TermVocabulary) {
            createTermVocabularySection(rootElement);
        } else if (input instanceof DefinedTermBase) {
            createDefinedTermSection(rootElement);

        } else if (input instanceof SelectionChangedEvent ) {
            IStructuredSelection sel =(IStructuredSelection) ((SelectionChangedEvent)input).getSelection();
            if (sel.getFirstElement()instanceof Distribution){
                createDescriptionElementSection(rootElement);
            }

//            createDescriptionElementSection(rootElement);

        }else {
            createEmptySection(null, rootElement);
        }
        layout();
    }

    private void createGroupSection(RootElement parent) {
        GroupDetailSection groupDetailSection = formFactory.createGroupDetailSection(getConversationHolder(), parent, this, StoreUtil.getSectionStyle(GroupDetailSection.class, getInput().getClass().getCanonicalName()));
        MemberDetailSection memberDetailSection = formFactory.createMemberDetailSection(getConversationHolder(), parent, StoreUtil.getSectionStyle(MemberDetailSection.class, getInput().getClass().getCanonicalName()));
        GrantedAuthorityDetailSection grantedAuthorityDetailSection = formFactory.createGrantedAuthorityDetailSection(getConversationHolder(), parent,StoreUtil.getSectionStyle(GrantedAuthorityDetailSection.class, getInput().getClass().getCanonicalName()));

        addPart(groupDetailSection);
        addPart(memberDetailSection);
        addPart(grantedAuthorityDetailSection);
    }

    @Override
    public ISelection getSelection() {
        return selection;
    }

    @Override
    public void setSelection(ISelection selection, boolean reveal) {

        this.selection = selection;
        if(this.selection!=null){
            SelectionChangedEvent selectionChangedEvent = new SelectionChangedEvent(this, selection);
            fireSelectionChanged(selectionChangedEvent);
        }
    }

    private void createTaxonSections(RootElement parent) {
        NameDetailsConfigurator config = PreferencesUtil.getPreferredNameDetailsConfiguration();
        EnumSet<CRUD> requiredCrud;
        if(getInput() == null || StoreUtil.getCdmEntity(getInput()).getId() == 0) {
            // new entity, not yet saved
            requiredCrud = EnumSet.of(CRUD.CREATE);
        } else {
            requiredCrud = EnumSet.of(CRUD.UPDATE);
        }
        boolean hasPermission = CdmStore.currentAuthentiationHasPermission(StoreUtil.getCdmEntity(getInput()), requiredCrud);
        TaxonBase<?> taxonBase = (TaxonBase<?>)getInput();
        if (hasPermission){
            // check for subtree permissions as well.
            
            Taxon acceptedTaxon = null;
            if (taxonBase instanceof Taxon){
                acceptedTaxon = (Taxon)taxonBase;
            }else{
                acceptedTaxon = ((Synonym)taxonBase).getAcceptedTaxon();
            }
            if (acceptedTaxon != null){ //needs discussion if synonyms without accepted taxon should never be filtered by subtree permission(current implementation). See also #8849
                Set<TaxonNode> nodes = acceptedTaxon.getTaxonNodes();
                for (TaxonNode node: nodes){
                    hasPermission &= CdmStore.currentAuthentiationHasPermission(node, requiredCrud);
                    if (!hasPermission){
                        //check whether there are explicit TaxonNode rights
                        boolean taxonnodePermissionExists = false;
                        Collection<? extends GrantedAuthority> authorities = CdmStore.getCurrentAuthentiation().getAuthorities();
                        for (GrantedAuthority grantedAuthority: authorities){
                            if (grantedAuthority.getAuthority().startsWith("TAXONNODE")){
                                taxonnodePermissionExists = true;
                            }
                        }
                        if (!taxonnodePermissionExists){
                            hasPermission = true;
                        }
                    }
                }
            }
        }
        detailsEnabled = hasPermission;
        ParsingMessagesSection parsingMessagesSection = formFactory.createParsingMessagesSection(getConversationHolder(), parent, this,
                StoreUtil.getSectionStyle(ParsingMessagesSection.class, getInput().getClass().getCanonicalName(), true));
        if (config == null){
            TaxonBaseDetailSection taxonBaseDetailSection = formFactory.createTaxonBaseDetailSection(getConversationHolder(), parent, this,
                    StoreUtil.getSectionStyle(TaxonBaseDetailSection.class, getInput().getClass().getCanonicalName(), true));
            formFactory.createHorizontalSeparator(parent, SWT.BORDER);
          //  boolean hasPermission = CdmStore.currentAuthentiationHasPermission(getInput(), Operation.UPDATE);
            taxonBaseDetailSection.setEnabled(detailsEnabled);
            addPart(taxonBaseDetailSection);
        }else{
            if (config.isTaxonSectionActivated()){
                TaxonBaseDetailSection taxonBaseDetailSection = formFactory.createTaxonBaseDetailSection(getConversationHolder(), parent, this,
                        StoreUtil.getSectionStyle(TaxonBaseDetailSection.class, getInput().getClass().getCanonicalName(), true));
                taxonBaseDetailSection.setEnabled(detailsEnabled);
                formFactory.createHorizontalSeparator(parent, SWT.BORDER);
                addPart(taxonBaseDetailSection);
            }
        }
        //TODO: test code for secundum source section
//        SecundumSourceSection sourceSection = formFactory.createSecundumDetailSection(getConversationHolder(), parent, this, StoreUtil.getSectionStyle(SecundumSourceSection.class, getInput().getClass().getCanonicalName(), true));
//        formFactory.createHorizontalSeparator(parent, SWT.BORDER);
//        addPart(sourceSection);
//        if (taxonBase.getSecSource() == null){
//            SecundumSource source = SecundumSource.NewSecundumInstance(taxonBase);
//            taxonBase.setSecSource(source);
//            sourceSection.setEntity(taxonBase);
//        }else{
//        	sourceSection.setEntity(taxonBase);
//        }
//
//        if (config != null && !config.isSecEnabled()){
//        	sourceSection.setEnabled(false);
//        }
        NonViralNameDetailSection nonViralNameSection = formFactory
                .createNonViralNameDetailSection(getConversationHolder(), parent, this, true,
                        StoreUtil.getSectionStyle(NonViralNameDetailSection.class, getInput().getClass().getCanonicalName(), true));
        formFactory.createHorizontalSeparator(parent, SWT.BORDER);
        nonViralNameSection.setEnabled(detailsEnabled);
        if (config == null || config.isNomenclaturalReferenceSectionActivated()){
            NomenclaturalSourceDetailSection referenceDetailSection = formFactory.createNomenclaturalReferenceDetailSection(getConversationHolder(), parent, this,
                    StoreUtil.getSectionStyle(NomenclaturalSourceDetailSection.class, getInput().getClass().getCanonicalName(), true));
            referenceDetailSection.setEnabled(detailsEnabled);
            formFactory.createHorizontalSeparator(parent, SWT.BORDER);
            addPart(referenceDetailSection);

        }
        if ( config == null || config.isNomenclaturalStatusSectionActivated()){
            NomenclaturalStatusSection nomenclaturalStatusSection = formFactory.createNomenclaturalStatusSection(getConversationHolder(), parent,
                    StoreUtil.getSectionStyle(NomenclaturalStatusSection.class, getInput().getClass().getCanonicalName(), true));
            nomenclaturalStatusSection.setEnabled(detailsEnabled);
            formFactory.createHorizontalSeparator(parent, SWT.BORDER);
            addPart(nomenclaturalStatusSection);
        }

//        if ( config == null || config.isProtologueActivated()){
//            ProtologueSection protologSection = formFactory.createProtologueSection(getConversationHolder(), parent, StoreUtil.getSectionStyle(ProtologueSection.class, getInput().getClass().getCanonicalName()));
//            protologSection.setEnabled(detailsEnabled);
//            formFactory.createHorizontalSeparator(parent, SWT.BORDER);
//            addPart(protologSection);
//        }

        if (config == null || config.isTypeDesignationSectionActivated()){
            TypeDesignationSection typeDesignationSection = formFactory.createTypeDesignationSection(getConversationHolder(), parent, StoreUtil.getSectionStyle(TypeDesignationSection.class, getInput().getClass().getCanonicalName()));
            typeDesignationSection.setEnabled(detailsEnabled);
            formFactory.createHorizontalSeparator(parent, SWT.BORDER);
            addPart(typeDesignationSection);
        }

        if (config == null || config.isNameRelationsSectionActivated()){
            NameRelationshipDetailSection nameRelationshipSection = formFactory.createNameRelationshipDetailSection(getConversationHolder(), parent, StoreUtil.getSectionStyle(NameRelationshipDetailSection.class, getInput().getClass().getCanonicalName()));
            nameRelationshipSection.setEnabled(detailsEnabled);
            formFactory.createHorizontalSeparator(parent, SWT.BORDER);
            addPart(nameRelationshipSection);
        }

        addPart(parsingMessagesSection);
        addPart(nonViralNameSection);
    }

    private void createNameSections(RootElement parent) {
        NonViralNameDetailSection nonViralNameSection = formFactory.createNonViralNameDetailSection(
                getConversationHolder(), parent, this, false, StoreUtil.getSectionStyle(NonViralNameDetailSection.class, getInput().getClass().getCanonicalName(), true));

        formFactory.createHorizontalSeparator(parent, SWT.BORDER);

        NomenclaturalSourceDetailSection referenceDetailSection = formFactory.createNomenclaturalReferenceDetailSection(getConversationHolder(), parent, this,
                StoreUtil.getSectionStyle(NomenclaturalSourceDetailSection.class, getInput().getClass().getCanonicalName(), true));
        formFactory.createHorizontalSeparator(parent, SWT.BORDER);

        NomenclaturalStatusSection nomenclaturalStatusSection = formFactory.createNomenclaturalStatusSection(getConversationHolder(), parent,
                StoreUtil.getSectionStyle(NomenclaturalStatusSection.class, getInput().getClass().getCanonicalName(), true));
        formFactory.createHorizontalSeparator(parent, SWT.BORDER);

        addPart(nonViralNameSection);
        addPart(nomenclaturalStatusSection);
        addPart(referenceDetailSection);

//        ProtologueSection protologSection = formFactory.createProtologueSection(getConversationHolder(), parent, StoreUtil.getSectionStyle(ProtologueSection.class, getInput().getClass().getCanonicalName()));
//        formFactory.createHorizontalSeparator(parent, SWT.BORDER);

        TypeDesignationSection typeDesignationSection = formFactory.createTypeDesignationSection(getConversationHolder(), parent, StoreUtil.getSectionStyle(TypeDesignationSection.class, getInput().getClass().getCanonicalName()));
        formFactory.createHorizontalSeparator(parent, SWT.BORDER);

        NameRelationshipDetailSection nameRelationshipSection = formFactory.createNameRelationshipDetailSection(getConversationHolder(), parent, StoreUtil.getSectionStyle(NameRelationshipDetailSection.class, getInput().getClass().getCanonicalName()));
        formFactory.createHorizontalSeparator(parent, SWT.BORDER);
//        addPart(protologSection);
        addPart(typeDesignationSection);
        addPart(nameRelationshipSection);
    }

    private void createReferenceSections(RootElement parent) {
        ReferenceDetailSection referenceDetailSection = formFactory.createReferenceDetailSection(getConversationHolder(), parent, this,
                StoreUtil.getSectionStyle(ReferenceDetailSection.class, getInput().getClass().getCanonicalName(), true));

        addPart(referenceDetailSection);
    }

    private void createTeamOrPersonBaseDetailSection(RootElement parent) {
        TeamOrPersonBaseDetailSection teamOrPersonBaseDetailSection = formFactory.createTeamOrPersonBaseDetailSection(getConversationHolder(), parent, this, StoreUtil.getSectionStyle(TeamOrPersonBaseDetailSection.class, getInput().getClass().getCanonicalName(), true));

        addPart(teamOrPersonBaseDetailSection);
    }

    private void createTeamDetailSection(RootElement parent) {
        TeamDetailSection teamDetailSection = formFactory.createTeamDetailSection(getConversationHolder(), parent, StoreUtil.getSectionStyle(TeamDetailSection.class, getInput().getClass().getCanonicalName(), true));

        addPart(teamDetailSection);
    }

    private void createPersonDetailSection(RootElement parent) {
        PersonDetailSection personDetailSection = formFactory.createPersonDetailSection(getConversationHolder(), parent, StoreUtil.getSectionStyle(PersonDetailSection.class, getInput().getClass().getCanonicalName(), true));
        addPart(personDetailSection);
    }

    private void createDescriptionElementSection(RootElement parent) {
        createDescriptionElementSection(parent, null);
    }

    private void createDescriptionElementSection(RootElement parent, Object activePart) {
        DescriptionElementDetailSection descriptionElementDetailSection = formFactory.createDescriptionElementDetailSection(getConversationHolder(), parent, this, StoreUtil.getSectionStyle(DescriptionElementDetailSection.class, getInput().getClass().getCanonicalName(), true));


        //  IntextReferenceSection intextReferenceSection = formFactory.createIntextReferenceSection(getConversationHolder(), parent,  getSectionStyle(.class, getInput().getClass().getCanonicalName()));

        formFactory.createHorizontalSeparator(parent, SWT.BORDER);
        DescriptionElementSourceSection descriptionElementSourceSection;
        if (activePart != null && activePart instanceof IDistributionEditorPart){
            Reference defaultSource =((IDistributionEditorPart)activePart).getEditor().getDefaultSource();
            descriptionElementSourceSection = formFactory.createDescriptionElementSourceSection(getConversationHolder(), parent, defaultSource, StoreUtil.getSectionStyle(DescriptionElementSourceSection.class, getInput().getClass().getCanonicalName()));

        }else{
            descriptionElementSourceSection = formFactory.createDescriptionElementSourceSection(getConversationHolder(), parent, StoreUtil.getSectionStyle(DescriptionElementSourceSection.class, getInput().getClass().getCanonicalName()) );
            descriptionElementSourceSection.setEnabled(this.detailsEnabled);

        }


        formFactory.createHorizontalSeparator(parent, SWT.BORDER);

        DescriptionElementMediaSection descriptionElementMediaSection = formFactory.createDescriptionElementMediaSection(getConversationHolder(), parent, StoreUtil.getSectionStyle(DescriptionElementMediaSection.class, getInput().getClass().getCanonicalName()));

        formFactory.createHorizontalSeparator(parent, SWT.BORDER);

        addPart(descriptionElementDetailSection);
        addPart(descriptionElementSourceSection);
        addPart(descriptionElementMediaSection);
    }

    private void createDescriptionSection(RootElement parent, boolean isTaxonDescription, boolean isEnabled) {
        DescriptionDetailSection descriptionDetailSection = formFactory.createDescriptionDetailSection(getConversationHolder(), parent, this, StoreUtil.getSectionStyle(DescriptionDetailSection.class, getInput().getClass().getCanonicalName(), true));
        descriptionDetailSection.setEnabled(isEnabled);
        addPart(descriptionDetailSection);

        formFactory.createHorizontalSeparator(parent, SWT.BORDER);

        if(isTaxonDescription) {
            NaturalLanguageSection naturalLanguageSection = formFactory.createNaturalLanguageSection(
                    getConversationHolder(), parent, this, StoreUtil.getSectionStyle(NaturalLanguageSection.class, getInput().getClass().getCanonicalName(), true));
            formFactory.createHorizontalSeparator(parent, SWT.BORDER);
            naturalLanguageSection.setEnabled(isEnabled);
            addPart(naturalLanguageSection);
        }
        DescriptionSourceSection descriptionSourceSection = formFactory.createDescriptionSourceSection(
                getConversationHolder(), parent, StoreUtil.getSectionStyle(DescriptionSourceSection.class, getInput().getClass().getCanonicalName(), true));
        descriptionSourceSection.setEnabled(isEnabled);
        addPart(descriptionSourceSection);

        formFactory.createHorizontalSeparator(parent, SWT.BORDER);

        if(isTaxonDescription) {
            ScopeSection scopeSection = formFactory.createScopeSection(getConversationHolder(), parent, StoreUtil.getSectionStyle(ScopeSection.class, getInput().getClass().getCanonicalName()));
            scopeSection.setEnabled(isEnabled);
            addPart(scopeSection);
        }
    }

    private void createTaxonDescriptionSection(RootElement parent, boolean isEnabled) {
        createDescriptionSection(parent, true, isEnabled);
    }

    private void createSpecimenDescriptionSection(RootElement parent) {
        createDescriptionSection(parent, false, true);
    }

    private void createUseDescriptionSection(RootElement parent) {
        DescriptionDetailSection descriptionDetailSection = formFactory.createDescriptionDetailSection(getConversationHolder(), parent, this, StoreUtil.getSectionStyle(DescriptionDetailSection.class, getInput().getClass().getCanonicalName()));

        formFactory.createHorizontalSeparator(parent, SWT.BORDER);

        DescriptionSourceSection descriptionSourceSection = formFactory.createDescriptionSourceSection(getConversationHolder(), parent, StoreUtil.getSectionStyle(DescriptionSourceSection.class, getInput().getClass().getCanonicalName()));

        formFactory.createHorizontalSeparator(parent, SWT.BORDER);

        addPart(descriptionDetailSection);
        addPart(descriptionSourceSection);
    }

    private void createImageGallerySection(RootElement parent) {
        DescriptionDetailSection descriptionDetailSection = formFactory.createDescriptionDetailSection(getConversationHolder(), parent, this, StoreUtil.getSectionStyle(DescriptionDetailSection.class, getInput().getClass().getCanonicalName()));

        addPart(descriptionDetailSection);
    }

    private void createMediaElementSection(RootElement parent) {
        MediaDetailsSection mediaDetailSection = formFactory.createMediaDetailsSection(getConversationHolder(), parent, this, StoreUtil.getSectionStyle(MediaDetailsSection.class, getInput().getClass().getCanonicalName(), true));

        addPart(mediaDetailSection);
    }

    private void createDerivedUnitBaseElementSection(RootElement parent) {
        DerivedUnitGeneralDetailSection derivedUnitGeneralDetailSection = formFactory.createDerivedUnitGeneralDetailSection(getConversationHolder(), parent, this, StoreUtil.getSectionStyle(DerivedUnitGeneralDetailSection.class, getInput().getClass().getCanonicalName(), true));
        //TODO for DerivateEditor do not use facade anymore to avoid this special case handling #4539
        if(!(AbstractUtility.getActivePart() instanceof IDerivedUnitFacadePart)){
            derivedUnitGeneralDetailSection.setShowOnlyDerivedUnitData(true);
        }
        addPart(derivedUnitGeneralDetailSection);

        formFactory.createHorizontalSeparator(parent, SWT.BORDER);

        DerivedUnitBaseDetailSection derivedUnitBaseDetailSection = formFactory.createDerivedUnitBaseDetailSection(getConversationHolder(), parent, this, StoreUtil.getSectionStyle(DerivedUnitBaseDetailSection.class, getInput().getClass().getCanonicalName(), true));
        addPart(derivedUnitBaseDetailSection);

        //for editors working with facades
        if(AbstractUtility.getActivePart() instanceof IDerivedUnitFacadePart){
            formFactory.createHorizontalSeparator(parent, SWT.BORDER);
            GatheringEventDetailSection gatheringEventDetailSection = formFactory.createGatheringEventDetailSection(getConversationHolder(), parent, this, StoreUtil.getSectionStyle(GatheringEventDetailSection.class, getInput().getClass().getCanonicalName()));
            addPart(gatheringEventDetailSection);

            formFactory.createHorizontalSeparator(parent, SWT.BORDER);
            FieldUnitFacadeDetailSection fieldUnitDetailSection = formFactory.createFieldUnitFacadeDetailSection(getConversationHolder(), parent, this, StoreUtil.getSectionStyle(FieldUnitFacadeDetailSection.class, getInput().getClass().getCanonicalName()));
            addPart(fieldUnitDetailSection);
        }
        else{

            if(PreferencesUtil.isShowTaxonAssociations()){
                formFactory.createHorizontalSeparator(parent, SWT.BORDER);
                TaxonAssociationDetailSection taxonAssociationDetailSection = formFactory.createTaxonAssociationDetailSection(getConversationHolder(), parent, this, StoreUtil.getSectionStyle(TaxonAssociationDetailSection.class, getInput().getClass().getCanonicalName(), true));
                addPart(taxonAssociationDetailSection);
            }
            formFactory.createHorizontalSeparator(parent, SWT.BORDER);
            DerivedUnitFacadeIdentifierSection identifierDetailSection = formFactory.createDerivedUnitFacadeIdentifierSection(getConversationHolder(), parent, StoreUtil.getSectionStyle(DerivedUnitFacadeIdentifierSection.class, getInput().getClass().getCanonicalName()));
            addPart(identifierDetailSection);
        }
        if(!PreferencesUtil.isDeterminationOnlyForFieldUnits()){
            formFactory.createHorizontalSeparator(parent, SWT.BORDER);
            DeterminationDetailSection determinationDetailSection = formFactory.createDeterminationDetailSection(getConversationHolder(), parent, this, StoreUtil.getSectionStyle(DeterminationDetailSection.class, getInput().getClass().getCanonicalName()));
            addPart(determinationDetailSection);
        }

        formFactory.createHorizontalSeparator(parent, SWT.BORDER);
        DerivedUnitTypeDesignationSection derivedUnitTypeDesignationSection = formFactory.createDerivedUnitTypeDesignationSection(getConversationHolder(), parent, this, StoreUtil.getSectionStyle(DerivedUnitTypeDesignationSection.class, getInput().getClass().getCanonicalName()));
        addPart(derivedUnitTypeDesignationSection);
    }

    private void createFieldUnitFacadeSection(RootElement parent) {
        FieldUnitFacadeGeneralDetailSection fielUnitGeneralDetailSection = formFactory.createFieldUnitFacadeGeneralDetailSection(getConversationHolder(), parent, this, StoreUtil.getSectionStyle(FieldUnitFacadeGeneralDetailSection.class, getInput().getClass().getCanonicalName(), true));

        formFactory.createHorizontalSeparator(parent, SWT.BORDER);

        GatheringEventDetailSection gatheringEventDetailSection = formFactory.createGatheringEventDetailSection(getConversationHolder(), parent, this, StoreUtil.getSectionStyle(GatheringEventDetailSection.class, getInput().getClass().getCanonicalName()));

        formFactory.createHorizontalSeparator(parent, SWT.BORDER);

        FieldUnitFacadeDetailSection fieldUnitDetailSection = formFactory.createFieldUnitFacadeDetailSection(getConversationHolder(), parent, this, StoreUtil.getSectionStyle(FieldUnitFacadeDetailSection.class, getInput().getClass().getCanonicalName()));

        addPart(fielUnitGeneralDetailSection);
        addPart(gatheringEventDetailSection);
        addPart(fieldUnitDetailSection);

        formFactory.createHorizontalSeparator(parent, SWT.BORDER);
        DeterminationDetailSection determinationDetailSection = formFactory.createDeterminationDetailSection(getConversationHolder(), parent, this, StoreUtil.getSectionStyle(DeterminationDetailSection.class, getInput().getClass().getCanonicalName()));
        addPart(determinationDetailSection);
    }

    private void createTissueSampleSection(RootElement parent) {
        TissueSampleGeneralDetailSection section = formFactory.createTissueSampleGeneralDetailSection(getConversationHolder(), parent, this, StoreUtil.getSectionStyle(TissueSampleGeneralDetailSection.class, getInput().getClass().getCanonicalName(), true));

        formFactory.createHorizontalSeparator(parent, SWT.BORDER);

        SampleDesignationDetailSection sampleDesignationDetailSection = formFactory.createSampleDesignationDetailSection(getConversationHolder(), parent, this, StoreUtil.getSectionStyle(SampleDesignationDetailSection.class, getInput().getClass().getCanonicalName(), true));

        addPart(section);
        addPart(sampleDesignationDetailSection);
    }

    private void createDnaSampleSection(RootElement parent) {
        DnaSampleGeneralDetailSection section = formFactory.createDnaSampleGeneralDetailSection(getConversationHolder(), parent, this, StoreUtil.getSectionStyle(DnaSampleGeneralDetailSection.class, getInput().getClass().getCanonicalName(), true));

        formFactory.createHorizontalSeparator(parent, SWT.BORDER);

        DnaSamplePreparationPreservationSection preparationPreservationSection = formFactory.createDnaSamplePreparationPreservationSection(getConversationHolder(), parent, this, StoreUtil.getSectionStyle(DnaSamplePreparationPreservationSection.class, getInput().getClass().getCanonicalName(), true));

        formFactory.createHorizontalSeparator(parent, SWT.BORDER);

        DnaQualityDetailSection qualitySection = formFactory.createDnaQualityDetailSection(getConversationHolder(), parent, this, StoreUtil.getSectionStyle(DnaQualityDetailSection.class, getInput().getClass().getCanonicalName(), true));

        formFactory.createHorizontalSeparator(parent, SWT.BORDER);

        SampleDesignationDetailSection sampleDesignationDetailSection = formFactory.createSampleDesignationDetailSection(getConversationHolder(), parent, this, StoreUtil.getSectionStyle(SampleDesignationDetailSection.class, getInput().getClass().getCanonicalName(), true));

        formFactory.createHorizontalSeparator(parent, SWT.BORDER);
        DeterminationDetailSection determinationDetailSection = null;
        if(!PreferencesUtil.isDeterminationOnlyForFieldUnits()){
            determinationDetailSection = formFactory.createDeterminationDetailSection(getConversationHolder(), parent, this, StoreUtil.getSectionStyle(DeterminationDetailSection.class, getInput().getClass().getCanonicalName()));
        }
        addPart(section);
        addPart(preparationPreservationSection);
        addPart(qualitySection);
        addPart(sampleDesignationDetailSection);
        if(!PreferencesUtil.isDeterminationOnlyForFieldUnits()){
            addPart(determinationDetailSection);
        }
    }

    private void createSequenceSection(RootElement parent) {
        SequenceGeneralDetailSection section = formFactory.createSequenceGeneralDetailSection(getConversationHolder(), parent, this, StoreUtil.getSectionStyle(SequenceGeneralDetailSection.class, getInput().getClass().getCanonicalName(), true));

        formFactory.createHorizontalSeparator(parent, SWT.BORDER);

        SequenceReferenceCollectionDetailSection referenceSection = formFactory.createSequenceReferenceCollectionDetailSection(getConversationHolder(), parent, StoreUtil.getSectionStyle(SequenceReferenceCollectionDetailSection.class, getInput().getClass().getCanonicalName(), true));

        formFactory.createHorizontalSeparator(parent, SWT.BORDER);

        SequenceContigFileCollectionDetailSection contigFileSection = formFactory.createSequenceContigFileCollectionDetailSection(getConversationHolder(), parent, StoreUtil.getSectionStyle(SequenceContigFileCollectionDetailSection.class, getInput().getClass().getCanonicalName(), true));

        addPart(section);
        addPart(referenceSection);
        addPart(contigFileSection);
    }

    private void createSingleReadSection(RootElement parent) {
        SingleReadGeneralDetailSection section = formFactory.createSingleReadGeneralDetailSection(getConversationHolder(), parent, this, StoreUtil.getSectionStyle(SingleReadGeneralDetailSection.class, getInput().getClass().getCanonicalName(), true));
        addPart(section);

        formFactory.createHorizontalSeparator(parent, SWT.BORDER);

        SingleReadPherogramCollectionDetailSection pherogramSection = formFactory.createSingleReadPherogramCollectionDetailSection(getConversationHolder(), parent, StoreUtil.getSectionStyle(SingleReadPherogramCollectionDetailSection.class, getInput().getClass().getCanonicalName(), true));
        addPart(pherogramSection);
    }

    private void createMediaSpecimenSection(RootElement parent) {
        MediaSpecimenGeneralDetailSection generalSection = formFactory.createMediaSpecimenGeneralDetailSection(getConversationHolder(), parent, this, StoreUtil.getSectionStyle(MediaSpecimenGeneralDetailSection.class, getInput().getClass().getCanonicalName(), true));
        addPart(generalSection);

        formFactory.createHorizontalSeparator(parent, SWT.BORDER);

        RightsSection rightsSection = formFactory.createRightsSection(getConversationHolder(), parent, StoreUtil.getSectionStyle(RightsSection.class, getInput().getClass().getCanonicalName()));
//        rightsSection.setEntity(generalSection.getEntity().innerDerivedUnit());
        addPart(rightsSection);
    }

    private void createFeatureTreeSectionForDto(RootElement parent) {
        TermTreeDetailSection section = formFactory.createFeatureTreeDetailSection(getConversationHolder(), parent, this, StoreUtil.getSectionStyle(TermTreeDetailSection.class, getInput().getClass().getCanonicalName(), true));

        addPart(section);
    }
    private void createFeatureTreeSection(RootElement parent) {
        TermTreeDetailSectionForNode section = formFactory.createFeatureTreeDetailSectionForTree(getConversationHolder(), parent, this, StoreUtil.getSectionStyle(TermTreeDetailSection.class, getInput().getClass().getCanonicalName(), true));
        addPart(section);
    }

    private void createFeatureNodeSection(RootElement parent) {
        TermNodeDetailSection featureDistributionSection = formFactory.createFeatureNodeDetailSection(getConversationHolder(), parent, this, StoreUtil.getSectionStyle(TermNodeDetailSection.class, getInput().getClass().getCanonicalName(), true));
        addPart(featureDistributionSection);
    }

    private void createFeatureDistributionSection(RootElement parent) {
        FeatureDistributionDetailSection featureDistributionSection = formFactory.createFeatureDistributionDetailSection(getConversationHolder(), parent, this, StoreUtil.getSectionStyle(FeatureDistributionDetailSection.class, getInput().getClass().getCanonicalName(), true));
        addPart(featureDistributionSection);
    }

    private void createCharacterNodeSection(RootElement parent) {
        CharacterNodeDetailSection featureDistributionSection = formFactory.createCharacterNodeDetailSection(getConversationHolder(), parent, this, StoreUtil.getSectionStyle(CharacterDetailSection.class, getInput().getClass().getCanonicalName(), true));
        addPart(featureDistributionSection);
    }

    private void createCharacterNodeDtoSection(RootElement parent) {
        CharacterNodeDtoDetailSection featureDistributionSection = formFactory.createCharacterNodeDtoDetailSection(getConversationHolder(), parent, this, StoreUtil.getSectionStyle(CharacterDetailSection.class, getInput().getClass().getCanonicalName(), true));
        addPart(featureDistributionSection);
    }

    private void createCharacterSection(RootElement parent) {
        CharacterDetailSection featureDistributionSection = formFactory.createCharacterDetailSection(getConversationHolder(), parent, this, StoreUtil.getSectionStyle(CharacterDetailSection.class, getInput().getClass().getCanonicalName(), true));
        addPart(featureDistributionSection);
    }

    private void createPolytomousKeyNodeSection(RootElement parent) {
        PolytomousKeyDetailSection polytomousKeyDetailSection = formFactory.createPolytomousKeyDetailSection(getConversationHolder(), parent, this, StoreUtil.getSectionStyle(PolytomousKeyDetailSection.class, getInput().getClass().getCanonicalName()));

        formFactory.createHorizontalSeparator(parent, SWT.BORDER);

        PolytomousKeyNodeDetailSection polytomousKeyNodeDetailSection = formFactory.createPolytomousKeyNodeDetailSection(getConversationHolder(), parent, this, StoreUtil.getSectionStyle(PolytomousKeyNodeDetailSection.class, getInput().getClass().getCanonicalName(), true));

        addPart(polytomousKeyDetailSection);
        addPart(polytomousKeyNodeDetailSection);

    }

    private void createUseRecordSection(RootElement parent) {
        UseRecordDetailSection descriptionUseRecordSection = formFactory.createUseRecordDetailSection(getConversationHolder(), parent, this,StoreUtil.getSectionStyle(UseRecordDetailSection.class, getInput().getClass().getCanonicalName(), true));
        formFactory.createHorizontalSeparator(parent, SWT.BORDER);
        addPart(descriptionUseRecordSection);
    }

    private void createUserSection(RootElement parent) {
        UserDetailSection userDetailSection = formFactory.createUserDetailSection(getConversationHolder(), parent, this, StoreUtil.getSectionStyle(UserDetailSection.class, getInput().getClass().getCanonicalName(), true));

        GroupsByUserDetailSection groupByUserDetailSection = formFactory.createGroupsByUserDetailSection(getConversationHolder(), parent, StoreUtil.getSectionStyle(GroupsByUserDetailSection.class, getInput().getClass().getCanonicalName(), true));

        addPart(userDetailSection);
        addPart(groupByUserDetailSection);
    }

    private void createMisappliedNameSection(RootElement parent) {
        TaxonRelationshipDetailSection taxonRelationshipDetailSection = formFactory.createTaxonRelationshipDetailSection(getConversationHolder(), parent, this,
                StoreUtil.getSectionStyle(TaxonRelationshipDetailSection.class, getInput().getClass().getCanonicalName(), true));
        addPart(taxonRelationshipDetailSection);

        formFactory.createHorizontalSeparator(parent, SWT.BORDER);

        NonViralNameDetailSection nonViralNameSection = formFactory
                .createNonViralNameDetailSection(getConversationHolder(), parent, this, true,
                        StoreUtil.getSectionStyle(NonViralNameDetailSection.class, getInput().getClass().getCanonicalName(), true));
        addPart(nonViralNameSection);

        formFactory.createHorizontalSeparator(parent, SWT.BORDER);

        NomenclaturalSourceDetailSection referenceDetailSection = formFactory.createNomenclaturalReferenceDetailSection(getConversationHolder(), parent, this,
                StoreUtil.getSectionStyle(NomenclaturalSourceDetailSection.class, getInput().getClass().getCanonicalName(), true));
        formFactory.createHorizontalSeparator(parent, SWT.BORDER);
        addPart(referenceDetailSection);

        NomenclaturalStatusSection nomenclaturalStatusSection = formFactory.createNomenclaturalStatusSection(getConversationHolder(), parent,
            		StoreUtil.getSectionStyle(NomenclaturalStatusSection.class, getInput().getClass().getCanonicalName(), true));
        formFactory.createHorizontalSeparator(parent, SWT.BORDER);
        addPart(nomenclaturalStatusSection);

//		ProtologueSection protologSection = formFactory.createProtologueSection(getConversationHolder(), parent, StoreUtil.getSectionStyle(ProtologueSection.class, getInput().getClass().getCanonicalName()));
//		formFactory.createHorizontalSeparator(parent, SWT.BORDER);
//		addPart(protologSection);

		TypeDesignationSection typeDesignationSection = formFactory.createTypeDesignationSection(getConversationHolder(), parent, StoreUtil.getSectionStyle(TypeDesignationSection.class, getInput().getClass().getCanonicalName()));
		formFactory.createHorizontalSeparator(parent, SWT.BORDER);
		addPart(typeDesignationSection);

		NameRelationshipDetailSection nameRelationshipSection = formFactory.createNameRelationshipDetailSection(getConversationHolder(), parent, StoreUtil.getSectionStyle(NameRelationshipDetailSection.class, getInput().getClass().getCanonicalName()));
		formFactory.createHorizontalSeparator(parent, SWT.BORDER);
		addPart(nameRelationshipSection);
    }

    private void createTaxonRelationshipSection(RootElement parent) {
        TaxonRelationshipDetailSection taxonRelationshipDetailSection = formFactory.createTaxonRelationshipDetailSection(getConversationHolder(), parent, this, StoreUtil.getSectionStyle(TaxonRelationshipDetailSection.class, getInput().getClass().getCanonicalName(), true));

        ReferencedEntityDetailSection referencedEntityBaseDetailSection = formFactory.createReferencedEntityDetailSection(getConversationHolder(), parent, this, StoreUtil.getSectionStyle(ReferencedEntityDetailSection.class, getInput().getClass().getCanonicalName(), true));
        addPart(taxonRelationshipDetailSection);
        addPart(referencedEntityBaseDetailSection);
    }
    private void createTermVocabularySection(RootElement parent) {
        TermVocabularyDetailSection termVocabularyDetailSection = formFactory.createTermVocabularyDetailSection(getConversationHolder(), parent, this, StoreUtil.getSectionStyle(TermVocabularyDetailSection.class, getInput().getClass().getCanonicalName(), true));
        addPart(termVocabularyDetailSection);
    }

    private void createDefinedTermSection(RootElement parent) {
        DefinedTermDetailSection<?> definedTermDetailSection = formFactory.createDefinedTermDetailSection(getInput().getClass(), getConversationHolder(), parent, this, StoreUtil.getSectionStyle(DefinedTermDetailSection.class, getInput().getClass().getCanonicalName(), true));
        addPart(definedTermDetailSection);
        TermMediaSection termMediaSection = formFactory.createTermMediaSection(getConversationHolder(), parent, StoreUtil.getSectionStyle(DefinedTermDetailSection.class, getInput().getClass().getCanonicalName(), true));
        addPart(termMediaSection);
    }

    @Override
    public void update(CdmDataChangeMap arg0) {

    }
    @Inject
    @Optional
    private void updateView(@UIEventTopic(WorkbenchEventConstants.REFRESH_DETAILS)boolean refresh){
        if(refresh){
            refresh();
        }
    }

    @Inject
    @Optional
    private void updateView(@UIEventTopic(WorkbenchEventConstants.REFRESH_DETAILS_DISTRIBUTION)String nameCache){
        if(nameCache != null && getInput() instanceof Distribution && this.activePart instanceof IDistributionEditorPart){
           if (part instanceof CdmSectionPart && ((CdmSectionPart)part).getSection() instanceof DescriptionElementDetailSection){
               for (ICdmFormElement element: ((DescriptionElementDetailSection)((CdmSectionPart)part).getSection()).getElements()){
                   if (element instanceof DescriptionElementDetailElement){
                       for(ICdmFormElement descriptionElement:((DescriptionElementDetailElement)element).getElements()){
                           if (descriptionElement instanceof DistributionDetailElement){
                               ((DistributionDetailElement) descriptionElement).setName(nameCache);
                           }
                       }
                   }
               }
           }
        }
    }
}