/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.view.e4.supplementaldata;

import java.util.Set;

import javax.annotation.PostConstruct;
import javax.inject.Inject;

import org.eclipse.e4.core.contexts.ContextInjectionFactory;
import org.eclipse.e4.core.contexts.IEclipseContext;
import org.eclipse.e4.core.di.annotations.Optional;
import org.eclipse.e4.ui.di.UIEventTopic;
import org.eclipse.e4.ui.model.application.ui.basic.MPart;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.TreeNode;
import org.eclipse.swt.SWTException;
import org.eclipse.swt.widgets.Composite;

import eu.etaxonomy.cdm.api.facade.DerivedUnitFacade;
import eu.etaxonomy.cdm.ext.occurrence.gbif.GbifResponse;
import eu.etaxonomy.cdm.model.common.ICdmBase;
import eu.etaxonomy.cdm.model.description.DescriptionBase;
import eu.etaxonomy.cdm.model.description.DescriptionElementBase;
import eu.etaxonomy.cdm.model.description.PolytomousKeyNode;
import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.cdm.model.taxon.TaxonRelationship;
import eu.etaxonomy.cdm.persistence.dto.TermDto;
import eu.etaxonomy.cdm.persistence.dto.TermNodeDto;
import eu.etaxonomy.cdm.persistence.dto.TermVocabularyDto;
import eu.etaxonomy.taxeditor.editor.ITaxonEditor;
import eu.etaxonomy.taxeditor.event.WorkbenchEventConstants;
import eu.etaxonomy.taxeditor.l10n.Messages;
import eu.etaxonomy.taxeditor.model.FeatureNodeContainer;
import eu.etaxonomy.taxeditor.model.IPartContentHasSupplementalData;
import eu.etaxonomy.taxeditor.model.MessagingUtils;
import eu.etaxonomy.taxeditor.preference.PreferencesUtil;
import eu.etaxonomy.taxeditor.view.e4.AbstractCdmEditorPartE4;

/**
 * @author pplitzner
 * @since Aug 10, 2017
 */
public class SupplementalDataPartE4 extends AbstractCdmEditorPartE4 {

	@Inject
    public SupplementalDataPartE4() {}

	@PostConstruct
	public void create(Composite parent, MPart thisPart, IEclipseContext context) {

	    this.thisPart = thisPart;

	    viewer = ContextInjectionFactory.make(SupplementalDataViewerE4.class, context);
	    ((SupplementalDataViewerE4)viewer).init(parent, this);
	}

    @Override
    public void selectionChanged_internal(Object selection, MPart activePart, MPart thisPart) {
	    if(activePart==thisPart){
	        return;
	    }
//	    setEnabled(true);
        Object partObject = createPartObject(activePart);

	    if(partObject instanceof IPartContentHasSupplementalData) {
	    	setEnabled(true);

	        IStructuredSelection structuredSelection = createSelection(selection);
	        if(structuredSelection==null || structuredSelection.isEmpty()){
	            showEmptyPage();
	            return;
	        }
	        if(structuredSelection.size() != 1){
	            showEmptyPage();
	            return;
	        }
	        if (partObject instanceof ITaxonEditor && structuredSelection.getFirstElement() instanceof Taxon ){
	            if (((ITaxonEditor)partObject).getTaxon() != structuredSelection.getFirstElement() && ((Taxon)structuredSelection.getFirstElement()).isMisapplication()){
	                Set<TaxonRelationship> rels =((Taxon)structuredSelection.getFirstElement()).getTaxonRelations(((ITaxonEditor)partObject).getTaxon());
	                if (rels != null && rels.size() == 1){
	                    structuredSelection = new StructuredSelection(rels.iterator().next());
	                }else{
	                    showEmptyPage();
	                    return;
	                }

	            }
	        }
	        // do not show supplemental data for feature nodes
	        if(structuredSelection.getFirstElement() instanceof FeatureNodeContainer){
	            showEmptyPage();
	            return;
	        }

	        else if(structuredSelection.getFirstElement() instanceof DerivedUnitFacade){
	            return;
	        }
	        else if(structuredSelection.getFirstElement() instanceof PolytomousKeyNode){
	            structuredSelection = new StructuredSelection(((PolytomousKeyNode)structuredSelection.getFirstElement()).getKey());
	        }
	        else if(structuredSelection.getFirstElement() instanceof TreeNode){
	            structuredSelection = new StructuredSelection(((TreeNode)structuredSelection.getFirstElement()).getValue());
	        }
	        else if(structuredSelection.getFirstElement() instanceof GbifResponse){
	            structuredSelection = new StructuredSelection(((GbifResponse)structuredSelection.getFirstElement()).getDerivedUnitFacade().innerDerivedUnit());
	        }
	        else if(structuredSelection.getFirstElement() instanceof DescriptionElementBase ){
	            structuredSelection = new StructuredSelection((structuredSelection.getFirstElement()));

	            if (((DescriptionElementBase)structuredSelection.getFirstElement()).getInDescription() != null && (((DescriptionElementBase)structuredSelection.getFirstElement()).getInDescription().isComputed() || ((DescriptionElementBase)structuredSelection.getFirstElement()).getInDescription().isCloneForSource())){
                    if (PreferencesUtil.isComputedDesciptionHandlingDisabled()){
                        setEnabled(false);
                    }else{
                        setEnabled(true);
                    }
                }
            }
	        else if(structuredSelection.getFirstElement() instanceof DescriptionBase ){

                if (((DescriptionBase)structuredSelection.getFirstElement()).isComputed() || ((DescriptionBase)structuredSelection.getFirstElement()).isCloneForSource()){
                    if (PreferencesUtil.isComputedDesciptionHandlingDisabled()){
                        setEnabled(false);
                    }else{
                        setEnabled(true);
                    }
                }
            }
	        else if(structuredSelection.getFirstElement() instanceof TermDto
	                || structuredSelection.getFirstElement() instanceof TermVocabularyDto
	                || structuredSelection.getFirstElement() instanceof TermNodeDto){
	            showViewer(structuredSelection, activePart, viewer);
	            return;
	        }


	        if(!(structuredSelection.getFirstElement() instanceof ICdmBase)){
	            showEmptyPage();
	            return;
	        }
	        showViewer(structuredSelection, activePart, viewer);
	        ((SupplementalDataViewerE4)viewer).setEnabled(isEnabled());

	        return;
	    }
        else{
	        showEmptyPage();
	        return;
	    }
	}

    @Override
    protected void showEmptyPage() {
        super.showEmptyPage();
        if(viewer!=null && viewer.getControl()!=null && !viewer.getControl().isDisposed()){
        	try{
        		((SupplementalDataViewerE4)viewer).destroySections();
        	}catch (SWTException e){
        		if (PreferencesUtil.isShowUpWidgetIsDisposedMessages() && e.getMessage().equals("Widget is disposed")){
                    MessagingUtils.errorDialog("Widget is disposed",
                            null,
                            MessagingUtils.WIDGET_IS_DISPOSED_MESSAGE,
                            null,
                            e,
                            true);

                }
        	}
        }
    }

	@Override
    protected String getViewName() {
		return Messages.SupplementalDataViewPart_VIEWER_NAME;
	}

	@Inject
    @Optional
    private void updateView(@UIEventTopic(WorkbenchEventConstants.REFRESH_SUPPLEMENTAL)boolean refresh){
        if(refresh){
            viewer.refresh();
        }
    }
}
