/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.view.e4.supplementaldata;

import javax.inject.Inject;

import org.eclipse.e4.core.di.annotations.Optional;
import org.eclipse.e4.ui.di.UIEventTopic;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.swt.SWT;
import org.eclipse.ui.forms.IFormPart;
import org.eclipse.ui.forms.widgets.ExpandableComposite;

import eu.etaxonomy.cdm.model.common.AnnotatableEntity;
import eu.etaxonomy.cdm.model.common.IdentifiableEntity;
import eu.etaxonomy.cdm.model.common.VersionableEntity;
import eu.etaxonomy.cdm.model.description.DescriptionElementBase;
import eu.etaxonomy.cdm.model.media.IdentifiableMediaEntity;
import eu.etaxonomy.cdm.model.metadata.PreferencePredicate;
import eu.etaxonomy.cdm.model.name.TypeDesignationBase;
import eu.etaxonomy.cdm.model.reference.ISourceable;
import eu.etaxonomy.cdm.persistence.hibernate.CdmDataChangeMap;
import eu.etaxonomy.taxeditor.event.WorkbenchEventConstants;
import eu.etaxonomy.taxeditor.model.IElementHasDetails;
import eu.etaxonomy.taxeditor.preference.PreferencesUtil;
import eu.etaxonomy.taxeditor.store.StoreUtil;
import eu.etaxonomy.taxeditor.ui.element.RootElement;
import eu.etaxonomy.taxeditor.ui.section.media.MediaSection;
import eu.etaxonomy.taxeditor.ui.section.supplemental.AnnotationSection;
import eu.etaxonomy.taxeditor.ui.section.supplemental.CdmBaseSection;
import eu.etaxonomy.taxeditor.ui.section.supplemental.CreditSection;
import eu.etaxonomy.taxeditor.ui.section.supplemental.ExtensionSection;
import eu.etaxonomy.taxeditor.ui.section.supplemental.HeadlineSection;
import eu.etaxonomy.taxeditor.ui.section.supplemental.MarkerSection;
import eu.etaxonomy.taxeditor.ui.section.supplemental.RightsSection;
import eu.etaxonomy.taxeditor.ui.section.supplemental.SourceSection;
import eu.etaxonomy.taxeditor.ui.section.supplemental.VersionSection;
import eu.etaxonomy.taxeditor.ui.section.supplemental.identifier.IdentifierSection;
import eu.etaxonomy.taxeditor.view.detail.CdmSectionPart;
import eu.etaxonomy.taxeditor.view.e4.AbstractCdmDataViewer;

/**
 * @author pplitzner
 * @since Aug 10, 2017
 */
public class SupplementalDataViewerE4 extends AbstractCdmDataViewer {

	private boolean showDebug;

	@Override
	public ISelection getSelection() {
		return null;
	}

	@Override
	public void setSelection(ISelection selection, boolean reveal) {
	}

	@Override
	protected void showParts() {

		showDebug = PreferencesUtil.getBooleanValue(PreferencePredicate.ShowUuidAndIdInSupplementalDataView.getKey(), true);

		Object input;

		if(getInput() instanceof IElementHasDetails){
			input = ((IElementHasDetails) getInput()).getData();
		}else{
			input = getInput();
		}

		destroySections();

		//1. Headline
		createHeadlineSection(rootElement);
		//2. Annotatable entitiy
		if(input instanceof AnnotatableEntity){
				createAnnotationSections(rootElement);
		}
		//3. Identifiable entity
		if(input instanceof IdentifiableEntity){
			createIdentifiableSections(rootElement);
		}
		if(input instanceof ISourceable
		        && !(input instanceof IdentifiableEntity)
		        && !(input instanceof DescriptionElementBase)
		        && !(input instanceof TypeDesignationBase)
		        ){
            createSourceSection(rootElement);
        }
		//4. Identifiable media entity
		if(input instanceof IdentifiableMediaEntity){
			createIdentifiableMediaSections(rootElement);
		}
		//5. Versionable
		if(input instanceof VersionableEntity){
				createVersionSection(rootElement);
		}
		//6. CdmBase
		if(showDebug){
			createCdmBaseSection(rootElement);
		}

		layout();
	}

	private void createHeadlineSection(RootElement parent){
		HeadlineSection headlineSection = formFactory.createHeadlineSection(parent);
		addPart(headlineSection);
	}

	private void createSourceSection(RootElement parent){
	    int expandStyle = StoreUtil.getSectionStyle(SourceSection.class, getInput().getClass().getCanonicalName(), false);
        SourceSection sourceSection =  formFactory.createSourceSection(getConversationHolder(), parent, expandStyle);
        formFactory.createHorizontalSeparator(parent, SWT.BORDER);
        addPart(sourceSection);
    }

    private void createAnnotationSections(RootElement parent) {
        int expandStyle = StoreUtil.getSectionStyle(AnnotationSection.class, getInput().getClass().getCanonicalName(), false);
        AnnotationSection annotationSection = formFactory.createAnnotationSection(getConversationHolder(), parent, expandStyle);

        formFactory.createHorizontalSeparator(parent, SWT.BORDER);
        expandStyle = StoreUtil.getSectionStyle(MarkerSection.class, getInput().getClass().getCanonicalName(), false);
        MarkerSection markerSection = formFactory.createMarkerSection(getConversationHolder(), parent, expandStyle);

        formFactory.createHorizontalSeparator(parent, SWT.BORDER);

        addPart(annotationSection);
        addPart(markerSection);
        
    }

    private void createIdentifiableSections(RootElement parent) {
        int expandStyle = StoreUtil.getSectionStyle(SourceSection.class, getInput().getClass().getCanonicalName(), false);
        SourceSection sourceSection = formFactory.createSourceSection(getConversationHolder(), parent, expandStyle);

        formFactory.createHorizontalSeparator(parent, SWT.BORDER);
        expandStyle =  StoreUtil.getSectionStyle(IdentifierSection.class, getInput().getClass().getCanonicalName(), false);
        IdentifierSection identifierSection = formFactory.createIdentifierDetailSection(getConversationHolder(), parent, expandStyle);

        formFactory.createHorizontalSeparator(parent, SWT.BORDER);
        expandStyle =  StoreUtil.getSectionStyle(ExtensionSection.class, getInput().getClass().getCanonicalName(), false);
        ExtensionSection extensionSection = formFactory.createExtensionSection(getConversationHolder(), parent, expandStyle);

        formFactory.createHorizontalSeparator(parent, SWT.BORDER);
        expandStyle =  StoreUtil.getSectionStyle(CreditSection.class, getInput().getClass().getCanonicalName(), false);
        CreditSection creditSection = formFactory.createCreditSection(getConversationHolder(), parent, expandStyle);

        formFactory.createHorizontalSeparator(parent, SWT.BORDER);
        expandStyle =  StoreUtil.getSectionStyle(RightsSection.class, getInput().getClass().getCanonicalName(), false);
        RightsSection rightsSection = formFactory.createRightsSection(getConversationHolder(), parent, expandStyle);

        formFactory.createHorizontalSeparator(parent, SWT.BORDER);

        addPart(sourceSection);
        addPart(identifierSection);
        addPart(extensionSection);
        addPart(creditSection);
        addPart(rightsSection);

	}

    private void createIdentifiableMediaSections(RootElement parent) {
        int expandStyle = StoreUtil.getSectionStyle(MediaSection.class, getInput().getClass().getCanonicalName(), false);
        MediaSection mediaSection = formFactory.createMediaSection(getConversationHolder(), parent, expandStyle);

        formFactory.createHorizontalSeparator(parent, SWT.BORDER);

        addPart(mediaSection);
    }

	private void createVersionSection(RootElement parent){
		VersionSection versionSection = formFactory.createVersionSection(parent, ExpandableComposite.NO_TITLE);
		addPart(versionSection);
	}

	private void createCdmBaseSection(RootElement parent) {
		CdmBaseSection cdmBaseSection = formFactory.createCdmBaseSection(parent, ExpandableComposite.NO_TITLE);
		addPart(cdmBaseSection);
	}

	@Override
	public void update(CdmDataChangeMap arg0) {
	}

	@Inject
    @Optional
    private void updateView(@UIEventTopic(WorkbenchEventConstants.REFRESH_SUPPLEMENTAL)boolean refresh){
        if(refresh){
            this.refresh();
        }
    }

	public void setEnabled(boolean doEnable){
	    for (IFormPart section: this.managedForm.getParts()){
	        if (section instanceof CdmSectionPart && !((CdmSectionPart)section).getSection().isDisposed()){
	            ((CdmSectionPart)section).getSection().setEnabled(doEnable);
	        }
	    }

	}

}
