// $Id$
/**
* Copyright (C) 2019 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.view.search.facet;

import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.SashForm;
import org.eclipse.swt.custom.ScrolledComposite;
import org.eclipse.swt.events.TraverseEvent;
import org.eclipse.swt.events.TraverseListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Text;

import eu.etaxonomy.taxeditor.model.ImageResources;

/**
 * @author pplitzner
 * @date 22.01.2019
 *
 */
public class SearchComposite extends SashForm {
    private Text txtSearch;
    private ScrolledComposite filterScrolledComposite;
    private Composite filterListComposite;
    private Composite searchResultComposite;
    private ScrolledComposite resultScrolledComposite;
    private Button btnSearchButton;

    /**
     * Create the composite.
     * @param parent
     * @param style
     */
    public SearchComposite(Composite parent, int style) {
        super(parent, style);
        setLayout(new GridLayout(2, false));

        Composite filterComposite = new Composite(this, SWT.NONE);
        filterComposite.setLayout(new GridLayout(1, false));
        GridData gd_filterComposite = new GridData(SWT.FILL, SWT.FILL, false, false, 1, 1);
        filterComposite.setLayoutData(gd_filterComposite);

        Composite composite = new Composite(filterComposite, SWT.NONE);
        composite.setLayoutData(new GridData(SWT.FILL, SWT.FILL, false, false, 1, 1));
        composite.setLayout(new GridLayout(2, false));

        txtSearch = new Text(composite, SWT.BORDER);
        txtSearch.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, false, 1, 1));
        txtSearch.addTraverseListener(new TraverseListener() {

            @Override
            public void keyTraversed(TraverseEvent e) {
                if (e.detail == SWT.TRAVERSE_RETURN){
                    btnSearchButton.setFocus();
                }

            }
        });

        btnSearchButton = new Button(composite, SWT.NONE);
        btnSearchButton.setImage(ImageResources.getImage(ImageResources.SEARCH_ICON));

        filterScrolledComposite = new ScrolledComposite(filterComposite, SWT.BORDER | SWT.H_SCROLL | SWT.V_SCROLL);
        filterScrolledComposite.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1));
        filterScrolledComposite.setExpandHorizontal(true);
        filterScrolledComposite.setExpandVertical(true);

        initFilterList();

        Composite composite_1 = new Composite(this, SWT.NONE);
        composite_1.setLayout(new GridLayout(1, false));
        composite_1.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1));

        resultScrolledComposite = new ScrolledComposite(composite_1, SWT.BORDER | SWT.H_SCROLL | SWT.V_SCROLL);
        resultScrolledComposite.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1));
        resultScrolledComposite.setExpandHorizontal(true);
        resultScrolledComposite.setExpandVertical(true);
        setWeights(new int[] {40, 60});

        initSearchResultList();


    }

    private void initSearchResultList() {
        searchResultComposite = new Composite(resultScrolledComposite, SWT.NONE);
        searchResultComposite.setLayout(new GridLayout(1, false));
        resultScrolledComposite.setContent(searchResultComposite);
        resultScrolledComposite.setMinSize(searchResultComposite.computeSize(SWT.DEFAULT, SWT.DEFAULT));
    }

    private void initFilterList() {
        filterListComposite = new Composite(filterScrolledComposite, SWT.NONE);
        GridLayout gl_filterListComposite = new GridLayout(1, false);
        gl_filterListComposite.marginHeight = 0;
        gl_filterListComposite.verticalSpacing = 0;
        filterListComposite.setLayout(gl_filterListComposite);
        filterScrolledComposite.setContent(filterListComposite);
        filterScrolledComposite.setMinSize(filterListComposite.computeSize(SWT.DEFAULT, SWT.DEFAULT));
    }

    @Override
    protected void checkSubclass() {
        // Disable the check that prevents subclassing of SWT components
    }
    public Text getTxtSearch() {
        return txtSearch;
    }

    public Composite getFilterListComposite() {
        return filterListComposite;
    }

    public void clear(){
        filterListComposite.dispose();
        initFilterList();

        clearResults();
    }

    public void clearResults() {
        searchResultComposite.dispose();
        initSearchResultList();
    }

    public void refresh(){
        filterScrolledComposite.layout(true, true);
        filterScrolledComposite.setMinSize(filterListComposite.computeSize(SWT.DEFAULT, SWT.DEFAULT));
        resultScrolledComposite.layout(true, true);
        resultScrolledComposite.setMinSize(searchResultComposite.computeSize(SWT.DEFAULT, SWT.DEFAULT));
    }
    public Composite getSearchResultComposite() {
        return searchResultComposite;
    }
    public Button getBtnSearchButton() {
        return btnSearchButton;
    }

}
