/**
* Copyright (C) 2019 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.view.search.facet;

import java.util.HashSet;
import java.util.Set;

/**
 * @author pplitzner
 * @since Jan 23, 2019
 *
 * @param <T> The type of the search result
 */
public abstract class SearchResult<T> {

    private Set<Facet> facets = new HashSet<>();
    private T content;

    public SearchResult(T content) {
        super();
        this.content = content;
        facets = initFacets(content);
    }

    protected abstract Set<Facet> initFacets(T content);

    public boolean hasFacet(Facet facet){
        return facets.contains(facet);
    }

    public boolean hasAnyFacet(Set<Facet> facets){
        return facets.stream().anyMatch(facet->this.facets.contains(facet));
    }

    public T getContent() {
        return content;
    }

    public Set<Facet> getFacets() {
        return facets;
    }

}
