/**
* Copyright (C) 2019 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.view.search.facet.occurrence;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Comparator;
import java.util.List;

import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Composite;

import eu.etaxonomy.cdm.api.service.IOccurrenceService;
import eu.etaxonomy.cdm.api.service.config.FindOccurrencesConfigurator;
import eu.etaxonomy.cdm.api.service.dto.DerivedUnitDTO;
import eu.etaxonomy.cdm.persistence.query.MatchMode;
import eu.etaxonomy.taxeditor.store.CdmStore;
import eu.etaxonomy.taxeditor.store.StoreUtil;
import eu.etaxonomy.taxeditor.view.search.facet.SearchComposite;
import eu.etaxonomy.taxeditor.view.search.facet.SearchController;

/**
 * @author pplitzner
 * @since Jan 22, 2019
 *
 */
public class OccurrenceSearchController extends SearchController<OccurrenceSearchResult, DerivedUnitDTO> {

    public OccurrenceSearchController(SearchComposite composite) {
        super(composite);
    }

    @Override
    protected Comparator<OccurrenceSearchResult> getResultComparator(String searchString) {
        return new Comparator<OccurrenceSearchResult>() {
            @Override
            public int compare(OccurrenceSearchResult o1, OccurrenceSearchResult o2) {
                String label1 = o1.getContent().getLabel();
                String label2 = o2.getContent().getLabel();
                return StoreUtil.compareBySearchString(searchString, label1, label2);
            }

        };
    }

    @Override
    protected List<OccurrenceSearchResult> searchResults(String searchString){
        List<OccurrenceSearchResult> searchResults = new ArrayList<>();
        FindOccurrencesConfigurator config = new FindOccurrencesConfigurator();
        config.setTitleSearchString(searchString);
        config.setMatchMode(MatchMode.ANYWHERE);
        Collection<DerivedUnitDTO> dtos = CdmStore.getService(IOccurrenceService.class).findByTitleDerivedUnitDTO(config);
        dtos.stream().forEach(dto->searchResults.add(new OccurrenceSearchResult(dto)));
        return searchResults;
    }

    @Override
    protected OccurrenceSearchResultComposite createResultComposite(OccurrenceSearchResult result, Composite parent) {
        return new OccurrenceSearchResultComposite(result, parent, SWT.NONE);
    }

    @Override
    protected List<OccurrenceSearchResult> getSelectedResults() {
        return null;
    }

}
