/**
* Copyright (C) 2019 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.view.search.facet.term;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Comparator;
import java.util.List;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;

import eu.etaxonomy.cdm.api.service.ITermService;
import eu.etaxonomy.cdm.api.service.IVocabularyService;
import eu.etaxonomy.cdm.common.CdmUtils;
import eu.etaxonomy.cdm.model.term.TermType;
import eu.etaxonomy.cdm.persistence.dto.AbstractTermDto;
import eu.etaxonomy.cdm.persistence.dto.TermDto;
import eu.etaxonomy.cdm.persistence.dto.TermVocabularyDto;
import eu.etaxonomy.taxeditor.store.CdmStore;
import eu.etaxonomy.taxeditor.store.StoreUtil;
import eu.etaxonomy.taxeditor.termtree.TermChooseWizardPage;
import eu.etaxonomy.taxeditor.view.search.facet.CheckBoxSearchResultComposite;
import eu.etaxonomy.taxeditor.view.search.facet.SearchComposite;
import eu.etaxonomy.taxeditor.view.search.facet.SearchController;

/**
 * @author pplitzner
 * @since Jan 22, 2019
 */
public class TermSearchController extends SearchController<TermSearchResult, AbstractTermDto> {

    private TermDto term;
    private TermChooseWizardPage page = null;

    public TermSearchController(SearchComposite composite, TermDto term) {
        super(composite);
        this.term = term;
    }

    public TermSearchController(SearchComposite composite, TermDto term, TermChooseWizardPage page) {
        super(composite);
        this.term = term;
        this.page = page;
    }

    @Override
    protected Comparator<TermSearchResult> getResultComparator(String searchString) {
        return new Comparator<TermSearchResult>() {
            @Override
            public int compare(TermSearchResult o1, TermSearchResult o2) {
                String label1 = o1.getContent().getRepresentation_L10n();
                String label2 = o2.getContent().getRepresentation_L10n();
                return StoreUtil.compareBySearchString(searchString, label1, label2);
            }
        };
    }

    @Override
    protected List<TermSearchResult> searchResults(String searchString){
        List<TermSearchResult> searchResults = new ArrayList<>();
        if(CdmUtils.isNotBlank(searchString)){
            TermType type = term!= null ? term.getTermType():null;
            Collection<TermDto> dtos = CdmStore.getService(ITermService.class).findByTitleAsDtoWithVocDto(searchString, type);
            dtos.remove(term);
            dtos.stream().forEach(dto->searchResults.add(new TermSearchResult(dto)));
            if (page != null){
                Collection<TermVocabularyDto> vocDtos = CdmStore.getService(IVocabularyService.class).findVocabularyDtoByTermTypeAndPattern(searchString, type);
                vocDtos.stream().forEach(vocDto->searchResults.add(new TermSearchResult(vocDto)));
            }
        }
        return searchResults;
    }

    @Override
    protected TermSearchResultComposite createResultComposite(TermSearchResult result, Composite parent) {
        TermSearchResultComposite comp = new TermSearchResultComposite(result, parent, SWT.NONE);
        comp.getBtnCheck().addSelectionListener(new SelectionListener() {

            @Override
            public void widgetSelected(SelectionEvent e) {
                Button selectedComp = (Button) e.widget;
                boolean selected = selectedComp.getSelection();
                for (CheckBoxSearchResultComposite<?,?> comp: getSearchResultComposites()){
                    comp.getBtnCheck().setSelection(false);
                }
                selectedComp.setSelection(selected);
                if (page != null){
                    page.setPageComplete(selected);
                }
            }

            @Override
            public void widgetDefaultSelected(SelectionEvent e) {
                // TODO Auto-generated method stub
            }
        });
        return comp;
    }

    @Override
    protected List<TermSearchResult> getSelectedResults() {
        return null;
    }

    @Override
    protected void search(){
        super.search();

    }
}
