/**
* Copyright (C) 2014 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.view.search.specimen;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.HttpURLConnection;
import java.net.URL;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.Set;

import org.eclipse.equinox.internal.p2.ui.misc.StringMatcher;
import org.eclipse.jface.fieldassist.ComboContentAdapter;
import org.eclipse.jface.fieldassist.ContentProposalAdapter;
import org.eclipse.jface.fieldassist.IContentProposal;
import org.eclipse.jface.fieldassist.IContentProposalProvider;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.DisposeEvent;
import org.eclipse.swt.events.DisposeListener;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Label;
import org.eclipse.ui.forms.widgets.FormToolkit;
import org.eclipse.ui.forms.widgets.TableWrapData;
import org.eclipse.ui.forms.widgets.TableWrapLayout;
import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import eu.etaxonomy.cdm.model.metadata.PreferencePredicate;
import eu.etaxonomy.taxeditor.preference.PreferencesUtil;

/**
 * @author pplitzner
 * @date 22.05.2014
 *
 */
public class SpecimenProviderSelectionComposite extends Composite {

    private final FormToolkit toolkit = new FormToolkit(Display.getCurrent());
    private final Combo txtAccessPoint;
    private final Button btnGbif;
    private final Button btnBioCaseProvider;
//    private final Label lblAccessPointUrl;

    private String[] items;

    /**
     * Create the composite.
     * @param parent
     * @param style
     */
    public SpecimenProviderSelectionComposite(Composite parent, int style) {
        super(parent, style);
        addDisposeListener(new DisposeListener() {
            @Override
            public void widgetDisposed(DisposeEvent e) {
                toolkit.dispose();
            }
        });
        toolkit.adapt(this);
        toolkit.paintBordersFor(this);
        {
            TableWrapLayout tableWrapLayout = new TableWrapLayout();
            tableWrapLayout.numColumns = 2;
            setLayout(tableWrapLayout);
        }
        btnBioCaseProvider = new Button(this, SWT.RADIO);
        toolkit.adapt(btnBioCaseProvider, true, true);
        btnBioCaseProvider.setText("BioCASE Provider");
        btnBioCaseProvider.setLayoutData(new TableWrapData(TableWrapData.LEFT, TableWrapData.MIDDLE, 1, 2));
//        txtAccessPoint = new Combo(this, SWT.BORDER);
//
        Label lblAccessPointUrl = new Label(this, SWT.NONE);
        lblAccessPointUrl.setLayoutData(new TableWrapData(TableWrapData.LEFT, TableWrapData.MIDDLE, 1, 1));
        toolkit.adapt(lblAccessPointUrl, true, true);
        lblAccessPointUrl.setText("Access Point URL");
        txtAccessPoint = new Combo(this, SWT.BORDER);
        txtAccessPoint.setLayoutData(new TableWrapData(TableWrapData.FILL_GRAB, TableWrapData.TOP, 1, 1));
        toolkit.adapt(txtAccessPoint, true, true);
//        new Label(this, SWT.NONE);
        btnGbif = new Button(this, SWT.RADIO);
        toolkit.adapt(btnGbif, true, true);
        btnGbif.setText("GBIF");
        btnGbif.setLayoutData(new TableWrapData(TableWrapData.LEFT, TableWrapData.MIDDLE, 1, 2));
        Set<String> itemList = new HashSet<>();
        URL url;
        String response = null;
        try {
            url = new URL("https://www.biocase.org/whats_biocase/gbif_biocase.cgi");

            HttpURLConnection conn = (HttpURLConnection) url.openConnection();
            conn.setRequestProperty("Accept", "application/json");
    //        conn.setConnectTimeout(TIMEOUT*9);
    //        conn.setReadTimeout(TIMEOUT*9);

            if (conn.getResponseCode() != 200) {
             //TODO
            }
            BufferedReader br = new BufferedReader(new InputStreamReader((conn.getInputStream())));

            StringBuilder  builder = new StringBuilder();
            String line;
            while ((line = br.readLine()) != null) {
                builder.append(line);
            }
            conn.disconnect();
            response = builder.toString();
        } catch (IOException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }

        JSONArray jsonResponseArray;
        try {
            jsonResponseArray =new JSONArray(response);

            int i = 0;
            String service_url = null;
            JSONArray dataSets = null;
//            items = new String[jsonResponseArray.length()];
            while (i<jsonResponseArray.length()){
                JSONObject object = jsonResponseArray.getJSONObject(i);
                service_url = (String) object.get("service_url");
                if (service_url != null){
                    if (itemList.add(service_url)){
                        dataSets = (JSONArray) object.get("datasets");
                        if (service_url != null){
                            int j = 0;
                            while ( j < dataSets.length()){
                                String title = ((JSONObject)dataSets.get(j)).getString("title");
                                itemList.add(title+" - "+ service_url);
                                txtAccessPoint.add(title+" - "+ service_url);
                                //create object with title and service url to add as data
                                txtAccessPoint.setData(title+" - "+ service_url, title);
                                j++;
                            }

                        }



//                        txtAccessPoint.add(service_url);
                    }
//                    items[i] = service_url;
                }
                i++;
            }

        } catch (JSONException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
        String biocaseProvider = PreferencesUtil.getStringValue(PreferencePredicate.BioCaseProvider.getKey());
        if (biocaseProvider != null){
            String items[] = biocaseProvider.split(";");
            for (String item: items){
                if (itemList.add(item)){
//                    how to handle biocase provider + different datasets???
                    txtAccessPoint.add(item);
                }
            }
        }
        items = new String[itemList.size()];
        items = itemList.toArray(items);

//        GridData dataProviderUri = new GridData();
//        dataProviderUri.horizontalAlignment = GridData.FILL;
//        dataProviderUri.horizontalSpan = 3;
//        txtAccessPoint.setLayoutData(dataProviderUri);
      //  providerURI.setVisible(false);

        ContentProposalAdapter adapter = new ContentProposalAdapter(txtAccessPoint, new ComboContentAdapter(), getProposalProvider(), null, null);
        adapter.setPropagateKeys(true);
        adapter.setProposalAcceptanceStyle(ContentProposalAdapter.PROPOSAL_REPLACE);
//        String[] proposals = new String[providerURI.getItems().length];



//


    }

    public Button getBtnGbif() {
        return btnGbif;
    }
    public Button getBtnBioCaseProvider() {
        return btnBioCaseProvider;
    }
    public Combo getTxtAccessPoint() {
        return txtAccessPoint;
    }
//    public Label getLblAccessPointUrl() {
//        return lblAccessPointUrl;
//    }

    /**
     * @return
     */
    IContentProposalProvider getProposalProvider() {
        return new IContentProposalProvider() {
            @Override
            public IContentProposal[] getProposals(String contents, int position) {
                //String[] items = getStringItems();
                if (contents.length() == 0 || items.length == 0) {
                    return new IContentProposal[0];
                }
                StringMatcher matcher = new StringMatcher("*" + contents + "*", true, false); //$NON-NLS-1$ //$NON-NLS-2$
                ArrayList<String> matches = new ArrayList<String>();
                for (int i = 0; i < items.length; i++) {
                    if (matcher.match(items[i])) {
                        matches.add(items[i]);
                    }
                }

                // We don't want to autoactivate if the only proposal exactly matches
                // what is in the combo.  This prevents the popup from
                // opening when the user is merely scrolling through the combo values or
                // has accepted a combo value.
                if (matches.size() == 1 && matches.get(0).equals(txtAccessPoint.getText())) {
                    return new IContentProposal[0];
                }

                if (matches.isEmpty()) {
                    return new IContentProposal[0];
                }

                // Make the proposals
                IContentProposal[] proposals = new IContentProposal[matches.size()];
                for (int i = 0; i < matches.size(); i++) {
                    final String proposal = matches.get(i);
                    proposals[i] = new IContentProposal() {

                        @Override
                        public String getContent() {
                            return proposal;
                        }

                        @Override
                        public int getCursorPosition() {
                            return proposal.length();
                        }

                        @Override
                        public String getDescription() {
                            return null;
                        }

                        @Override
                        public String getLabel() {
                            return null;
                        }
                    };
                }
                return proposals;
            }
        };
    }
}
