/**
* Copyright (C) 2014 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.view.search.specimen;

import org.eclipse.jface.wizard.IWizard;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Listener;

import eu.etaxonomy.taxeditor.preference.PreferencesUtil;

/**
 * Controller class for handling {@link SpecimenProviderSelectionComposite}
 * @author pplitzner
 * @date 22.05.2014
 *
 */
public class SpecimenProviderSelectionController implements Listener{

    private SpecimenProviderSelectionComposite composite;
    private IWizard wizard;
    private String lastAccessPoint;

    private static SpecimenProviderSelectionController instance;

    public static SpecimenProviderSelectionController getInstance(Composite parent, IWizard wizard){
        if(instance==null){
            instance = new SpecimenProviderSelectionController(parent, wizard);
            return instance;
        }
        instance.init(parent, wizard);
        return instance;
    }


    /**
     * Constructs a new controller which will itself construct the composite
     * @param parent the parent {@link Composite} for the one handles by this controller
     */
    private SpecimenProviderSelectionController(Composite parent, IWizard wizard) {
        init(parent, wizard);
    }

    /**
     * @param parent
     * @param wizard
     */
    private void init(Composite parent, IWizard wizard) {
        this.wizard = wizard;
        composite = new SpecimenProviderSelectionComposite(parent, SWT.NONE);
        composite.getBtnBioCaseProvider().addListener(SWT.Selection, this);
        composite.getBtnGbif().addListener(SWT.Selection, this);
        composite.getTxtAccessPoint().addListener(SWT.Modify, this);
        composite.getBtnGbif().setSelection(true);
        composite.getTxtAccessPoint().setEnabled(false);
        loadLastState();
    }

    private void loadLastState() {
        if (lastAccessPoint == null){
            String lastSelected = PreferencesUtil.getStringValue(PreferencesUtil.LAST_USED_BIOCASE_PROVIDER, true);
            lastAccessPoint = lastSelected;
        }
        if(lastAccessPoint!=null){
            Combo text = composite.getTxtAccessPoint();
            Listener[] listeners = text.getListeners(SWT.Modify);
            for (int i = 0; i < listeners.length; i++) {
                text.removeListener(SWT.Modify, listeners[i]);
            }
            text.setText(lastAccessPoint);
            for (int i = 0; i < listeners.length; i++) {
                text.addListener(SWT.Modify, listeners[i]);
            }
            composite.getBtnGbif().setSelection(false);
            composite.getTxtAccessPoint().setEnabled(true);


        }
    }


    /* (non-Javadoc)
     * @see org.eclipse.swt.widgets.Listener#handleEvent(org.eclipse.swt.widgets.Event)
     */
    @Override
    public void handleEvent(Event event) {
        if(event.widget==composite.getBtnBioCaseProvider()){
            composite.getTxtAccessPoint().setEnabled(true);
        }
        else if(event.widget==composite.getBtnGbif()){
            composite.getTxtAccessPoint().setEnabled(false);
        }
        wizard.getContainer().updateButtons();
    }

    /**
     * @return the composite
     */
    public SpecimenProviderSelectionComposite getComposite() {
        return composite;
    }


    public void saveLastState() {
        lastAccessPoint = composite.getTxtAccessPoint().getText();
        PreferencesUtil.setLastSelectedBiocaseProvider(lastAccessPoint);
    }

}
