/**
 * Copyright (C) 2018 EDIT
 * European Distributed Institute of Taxonomy
 * http://www.e-taxonomy.eu
 *
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */
package eu.etaxonomy.taxeditor.view.webimport.termimport;

import org.eclipse.jface.viewers.ListViewer;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.List;
import org.eclipse.swt.widgets.Text;
import org.eclipse.swt.widgets.Tree;
import org.eclipse.wb.swt.SWTResourceManager;

import eu.etaxonomy.cdm.api.service.IVocabularyService;
import eu.etaxonomy.cdm.model.term.TermVocabulary;
import eu.etaxonomy.taxeditor.model.ImageResources;
import eu.etaxonomy.taxeditor.store.CdmStore;
import eu.etaxonomy.taxeditor.ui.combo.termvocabulary.TermVocabularyComboViewer;

/**
 * @author pplitzner
 * @since Apr 23, 2018
 *
 */
public class GfBioTerminologyImportComposite extends Composite{
    private TreeViewer treeTermHierarchy;
    private Text txtSearch;
    private Button btnSearch;
    private Composite composite;
    private Composite composite_1;
    private Composite composite_2;
    private Label lblNewLabel;
    private Label lblNewLabel_1;
    private Label lblNewLabel_2;
    private Text txtLabel;
    private Text txtUri;
    private Text txtDescription;
    private Label lblNewLabel_3;
    private Composite composite_3;
    private Label lblNewLabel_4;
    private Button btnRemoveVocabulary;
    private TermVocabularyComboViewer termVocabularyComboViewer;
    private List list;
    private ListViewer listSynonyms;
    private Composite composite_4;
    private Label lblNewLabel_5;
    private Composite composite_5;
    private Label lblResultCount;
    private Label lblNewLabel_6;
    private Text textOntologies;
    private Button btnChooseOntology;

    public GfBioTerminologyImportComposite(Composite parent, int style) {
        super(parent, style);
        setLayout(new GridLayout(1, false));

        composite = new Composite(this, SWT.NONE);
        composite.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, false, 1, 1));
        composite.setLayout(new GridLayout(2, false));

        txtSearch = new Text(composite, SWT.BORDER);
        txtSearch.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));

        btnSearch = new Button(composite, SWT.NONE);
        btnSearch.setText("Search");

        composite_3 = new Composite(this, SWT.NONE);
        composite_3.setLayout(new GridLayout(6, false));
        composite_3.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, false, 1, 1));

        lblNewLabel_6 = new Label(composite_3, SWT.NONE);
        lblNewLabel_6.setLayoutData(new GridData(SWT.RIGHT, SWT.CENTER, false, false, 1, 1));
        lblNewLabel_6.setText("Ontology");

        textOntologies = new Text(composite_3, SWT.BORDER);
        textOntologies.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));

        btnChooseOntology = new Button(composite_3, SWT.NONE);
        btnChooseOntology.setImage(ImageResources.getImage(ImageResources.BROWSE_ICON));

        lblNewLabel_4 = new Label(composite_3, SWT.NONE);
        lblNewLabel_4.setLayoutData(new GridData(SWT.RIGHT, SWT.CENTER, false, false, 1, 1));
        lblNewLabel_4.setText("Import Vocabulary");

        termVocabularyComboViewer = new TermVocabularyComboViewer(composite_3, SWT.NONE);
        termVocabularyComboViewer.setLayoutData(new GridData(SWT.RIGHT, SWT.FILL, false, false, 1, 1));
        termVocabularyComboViewer.setInput(CdmStore.getService(IVocabularyService.class).list(TermVocabulary.class, null, null, null, null));

        btnRemoveVocabulary = new Button(composite_3, SWT.NONE);
        btnRemoveVocabulary.setLayoutData(new GridData(SWT.RIGHT, SWT.CENTER, false, false, 1, 1));
        btnRemoveVocabulary.setImage(ImageResources.getImage(ImageResources.TRASH_ICON));

        composite_2 = new Composite(this, SWT.NONE);
        composite_2.setLayout(new GridLayout(2, false));
        composite_2.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1));

        composite_4 = new Composite(composite_2, SWT.NONE);
        composite_4.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1));
        composite_4.setLayout(new GridLayout(1, false));

        treeTermHierarchy = new TreeViewer(composite_4, SWT.BORDER);
        Tree tree = treeTermHierarchy.getTree();
        tree.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1));

        composite_5 = new Composite(composite_4, SWT.NONE);
        composite_5.setLayoutData(new GridData(SWT.FILL, SWT.FILL, false, false, 1, 1));
        composite_5.setLayout(new GridLayout(2, false));

        lblNewLabel_5 = new Label(composite_5, SWT.NONE);
        lblNewLabel_5.setText("Results found:");

        lblResultCount = new Label(composite_5, SWT.NONE);
        lblResultCount.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));

        composite_1 = new Composite(composite_2, SWT.NONE);
        composite_1.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1));
        composite_1.setLayout(new GridLayout(2, false));

        lblNewLabel = new Label(composite_1, SWT.NONE);
        lblNewLabel.setLayoutData(new GridData(SWT.RIGHT, SWT.CENTER, false, false, 1, 1));
        lblNewLabel.setText("Label");

        txtLabel = new Text(composite_1, SWT.NONE);
        txtLabel.setBackground(SWTResourceManager.getColor(SWT.COLOR_WIDGET_BACKGROUND));
        txtLabel.setEditable(false);
        txtLabel.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));

        lblNewLabel_1 = new Label(composite_1, SWT.NONE);
        lblNewLabel_1.setLayoutData(new GridData(SWT.RIGHT, SWT.CENTER, false, false, 1, 1));
        lblNewLabel_1.setText("URI");

        txtUri = new Text(composite_1, SWT.NONE);
        txtUri.setBackground(SWTResourceManager.getColor(SWT.COLOR_WIDGET_BACKGROUND));
        txtUri.setEditable(false);
        txtUri.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));

        lblNewLabel_2 = new Label(composite_1, SWT.NONE);
        lblNewLabel_2.setLayoutData(new GridData(SWT.RIGHT, SWT.TOP, false, false, 1, 1));
        lblNewLabel_2.setText("Description");

        txtDescription = new Text(composite_1, SWT.BORDER | SWT.WRAP | SWT.V_SCROLL | SWT.MULTI);
        txtDescription.setBackground(SWTResourceManager.getColor(SWT.COLOR_WIDGET_BACKGROUND));
        txtDescription.setEditable(false);
        txtDescription.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1));

        lblNewLabel_3 = new Label(composite_1, SWT.NONE);
        lblNewLabel_3.setLayoutData(new GridData(SWT.LEFT, SWT.TOP, false, false, 1, 1));
        lblNewLabel_3.setText("Synonyms");

        listSynonyms = new ListViewer(composite_1, SWT.BORDER | SWT.H_SCROLL | SWT.V_SCROLL);
        list = listSynonyms.getList();
        list.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1));
        list.setBackground(SWTResourceManager.getColor(SWT.COLOR_WIDGET_BACKGROUND));
    }
    public TreeViewer getTreeTermHierarchy() {
        return treeTermHierarchy;
    }
    public Button getBtnSearch() {
        return btnSearch;
    }
    public Text getTxtSearch() {
        return txtSearch;
    }
    public Text getLblDescription() {
        return txtDescription;
    }
    public Text getLblUri() {
        return txtUri;
    }
    public Text getLblLabel() {
        return txtLabel;
    }
    public ListViewer getListSynonyms() {
        return listSynonyms;
    }
    public TermVocabularyComboViewer getTermVocabularyComboViewer() {
        return termVocabularyComboViewer;
    }
    public Button getBtnRemoveVocabulary() {
        return btnRemoveVocabulary;
    }
    public Label getLblResultCount() {
        return lblResultCount;
    }
    public Text getTextOntologies() {
        return textOntologies;
    }
    public Button getBtnChooseOntology() {
        return btnChooseOntology;
    }
}
