/**
* Copyright (C) 2018 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.view.webimport.termimport;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;

import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.viewers.ArrayContentProvider;
import org.eclipse.jface.viewers.CheckboxTableViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Table;

import eu.etaxonomy.taxeditor.view.webimport.termimport.wrapper.TerminologyWrapper;

/**
 * Dialog to choose specimens for the character matrix.<br>
 * Filters specimens that have already been added.
 * @author pplitzner
 * @since Jan 3, 2018
 *
 */
public class OntologySelectionDialog extends Dialog {

    private CheckboxTableViewer list;

    private Collection<TerminologyWrapper> selectedOntologies = new ArrayList<>();

    private Collection<TerminologyWrapper> initiallySelectedOntologies;

    private Collection<TerminologyWrapper> availableOntologies;


    protected OntologySelectionDialog(Shell parentShell, Collection<TerminologyWrapper> initiallySelectedOntologies, Collection<TerminologyWrapper> availableOntologies) {
        super(parentShell);
        this.initiallySelectedOntologies = initiallySelectedOntologies;
        this.availableOntologies = availableOntologies;
    }

    @Override
    protected Control createDialogArea(Composite parent) {
        Table table = new Table(parent, SWT.CHECK | SWT.MULTI);
        list = new CheckboxTableViewer(table);
        list.setLabelProvider(new TerminologyLabelProvider());
        list.setContentProvider(new ArrayContentProvider());
        list.setInput(availableOntologies);
        if(initiallySelectedOntologies!=null){
            initiallySelectedOntologies.stream().forEach(wrapper->list.setChecked(wrapper, true));
        }
        return table;
    }

    @Override
    protected void configureShell(Shell newShell) {
        super.configureShell(newShell);
        newShell.setText("Choose ontologies");
    }

    @Override
    protected void okPressed() {
        selectedOntologies.clear();
        selectedOntologies = new ArrayList(Arrays.asList(list.getCheckedElements()));
        super.okPressed();
    }

    @Override
    protected boolean isResizable() {
        return true;
    }

    public Collection<TerminologyWrapper> getSelectedOntologies(){
        return selectedOntologies;
    }

}
