/**
* Copyright (C) 2018 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.view.webimport.termimport.parser;

import java.util.ArrayList;
import java.util.List;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import eu.etaxonomy.taxeditor.view.webimport.termimport.requests.RequestTerm;
import eu.etaxonomy.taxeditor.view.webimport.termimport.wrapper.OntologyTermWrapper;

/**
 * @author pplitzner
 * @since May 31, 2018
 *
 */
public class ParserUtil {

    public static JSONArray parseResults(JSONObject jsonObject) throws JSONException{
        return jsonObject.getJSONArray("results");
    }

    public static String parseUri(JSONObject jsonObject) throws JSONException{
        return jsonObject.getString("uri");
    }

    public static String parseLabel(JSONObject jsonObject) throws JSONException{
        return jsonObject.getString("label");
    }

    public static String parseDescription(JSONObject jsonObject) throws JSONException{
        String descriptionKey = "description";
        if(jsonObject.has(descriptionKey)){
            return jsonObject.getString(descriptionKey);
        }
        return null;
    }

    public static List<String> parseSynonyms(JSONObject jsonObject) throws JSONException{
        List<String> synonyms = new ArrayList<>();
        String synomymKey = "synonyms";
        if(jsonObject.has(synomymKey)){
             JSONArray jsonArray = jsonObject.getJSONArray(synomymKey);
             for(int i=0;i<jsonArray.length();i++){
                 synonyms.add(jsonArray.getString(i));
             }
        }
        return synonyms;
    }

    public static String parseTerminology(JSONObject jsonObject) throws JSONException {
        return jsonObject.getString("sourceTerminology");
    }

    /**
     * Loads and sets detail information for the given wrapper
     * @param wrapper the wrapper for which the detailed information will be loaded and set
     */
    public static void loadDetails(OntologyTermWrapper wrapper) {
        String request = new RequestTerm(wrapper.getUri(), wrapper.getSourceTerminology()).request();
        OntologyTermWrapper detailTerm = TermParser.parseSingleTerm(request);
        wrapper.setDescription(detailTerm.getDescription());
        wrapper.setSynonyms(detailTerm.getSynonyms());
    }
}
