/**
* Copyright (C) 2018 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.view.webimport.termimport.parser;

import java.util.ArrayList;
import java.util.List;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import eu.etaxonomy.taxeditor.view.webimport.termimport.wrapper.OntologyTermWrapper;

/**
 * @author pplitzner
 * @since May 9, 2018
 *
 */
public class TermParser {
    public static OntologyTermWrapper parseSingleTerm(String response){
        try {
            JSONObject jsonResponse = new JSONObject(response);
            JSONArray responseArray = ParserUtil.parseResults(jsonResponse);
            for(int i=0;i<responseArray.length();i++){
                JSONObject jsonObject = responseArray.getJSONObject(i);
                return createWrapper(jsonObject);
            }
        } catch (JSONException e) {
            e.printStackTrace();
        }
        return null;
    }

    public static List<OntologyTermWrapper> parse(String response){
        List<OntologyTermWrapper> wrapperList = new ArrayList<>();
        try {
            JSONObject jsonResponse = new JSONObject(response);
            JSONArray responseArray = ParserUtil.parseResults(jsonResponse);
            for(int i=0;i<responseArray.length();i++){
                JSONObject jsonObject = responseArray.getJSONObject(i);
                wrapperList.add(createWrapper(jsonObject));
            }
        } catch (JSONException e) {
            e.printStackTrace();
        }
        return wrapperList;
    }

    private static OntologyTermWrapper createWrapper(JSONObject jsonObject) throws JSONException{
        String description = ParserUtil.parseDescription(jsonObject);
        String label = ParserUtil.parseLabel(jsonObject);
        String uri= ParserUtil.parseUri(jsonObject);
        String terminology= ParserUtil.parseTerminology(jsonObject);
        List<String> synonyms = ParserUtil.parseSynonyms(jsonObject);
        OntologyTermWrapper wrapper = new OntologyTermWrapper(uri, label, terminology);
        wrapper.setSynonyms(synonyms);
        wrapper.setDescription(description);
        return wrapper;
    }

}
