/**
* Copyright (C) 2018 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.view.webimport.termimport.requests;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.HttpURLConnection;
import java.net.URI;
import java.net.URISyntaxException;
import java.net.URL;
import java.util.List;

/**
 * @author pplitzner
 * @since Apr 23, 2018
 */
public abstract class AbstractTerminologyServiceRequest {

    @SuppressWarnings("unused")
    private static final String TERM_SERVICE_DEV = "dev-gfbio.bgbm.org";
    private static final String TERM_SERVICE = "terminologies.gfbio.org";
    private static final int TIMEOUT = 10000;

    public String request() {

        try {
            String queryString = "format=json&";
            String pathString = "/api/terminologies/"
                    + (getTerminologyId()!=null?getTerminologyId()+"/":"")
                    + (getServiceUri()!=null?getServiceUri():"");

            List<RequestParameter> serviceParameters = getServiceParameters();
            if(serviceParameters!=null){
                StringBuilder params = new StringBuilder();
                serviceParameters.forEach(param->params.append(param.key+"="+param.value+"&"));
                queryString += params.toString();
            }
            URI uri = new URI("https", TERM_SERVICE, pathString, queryString, null);
            URL url = uri.toURL();
            HttpURLConnection conn = (HttpURLConnection) url.openConnection();
            conn.setRequestProperty("Accept", "application/json");
            conn.setConnectTimeout(TIMEOUT*9);
            conn.setReadTimeout(TIMEOUT*9);

            if (conn.getResponseCode() != 200) {
                throw new RuntimeException("Failed : HTTP error code : " + conn.getResponseCode());
            }
            BufferedReader br = new BufferedReader(new InputStreamReader((conn.getInputStream())));

            StringBuilder  builder = new StringBuilder();
            String line;
            while ((line = br.readLine()) != null) {
                builder.append(line);
            }
            conn.disconnect();
            return builder.toString();

        } catch (IOException e) {
            e.printStackTrace();
        } catch (URISyntaxException e) {
            e.printStackTrace();
        }
        return null;
    }

    protected abstract String getServiceUri();
    protected abstract String getTerminologyId();
    protected abstract List<RequestParameter> getServiceParameters();

    protected class RequestParameter{
        String key;
        String value;
        public RequestParameter(String key, String value) {
            super();
            this.key = key;
            this.value = value;
        }
        public String getKey() {
            return key;
        }
        public String getValue() {
            return value;
        }
    }
}