/**
* Copyright (C) 2018 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.view.webimport.termimport.requests;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.stream.Collectors;

import eu.etaxonomy.taxeditor.view.webimport.termimport.wrapper.TerminologyWrapper;

/**
 * @author pplitzner
 * @since Apr 23, 2018
 *
 */
public class RequestSearch extends AbstractTerminologyServiceRequest{

    private static final String REGEX_WILDCARD = ".*";
    private static final String REGEX_BEGINNING = "^";
    private static final String REGEX_END = "$";
    private static final String ASTERISK = "*";
    private static final String SEARCH = "search";
    private static final String TRUE = "true";
    private static final String INTERNAL_ONLY = "internal_only";
    private static final String INCLUDED = "included";
    private static final String REGEX = "regex";
    private static final String EXACT = "exact";
    private static final String MATCH_TYPE = "match_type";
    private String searchString;
    private Collection<TerminologyWrapper> selectedOntologies;

    public RequestSearch(String searchString, Collection<TerminologyWrapper> selectedOntologies) {
        super();
        this.searchString = searchString;
        this.selectedOntologies = selectedOntologies;
    }

    @Override
    protected String getServiceUri() {
        return SEARCH;
    }

    @Override
    protected String getTerminologyId() {
        return null;
    }

    @Override
    protected List<RequestParameter> getServiceParameters() {
        List<RequestParameter> params = new ArrayList<>();
        if(searchString.contains(ASTERISK)){
            params.add(new RequestParameter(MATCH_TYPE, REGEX));
            if(!searchString.startsWith(ASTERISK)){
                searchString = REGEX_BEGINNING + searchString;
            }
            if(!searchString.endsWith(ASTERISK)){
                searchString += REGEX_END;
            }
            searchString = searchString.replace(ASTERISK, REGEX_WILDCARD);
        }
        else{
            params.add(new RequestParameter(MATCH_TYPE, EXACT));
        }
        params.add(new RequestParameter(TerminologyServiceConstants.ATTR_QUERY, searchString));
        params.add(new RequestParameter(INTERNAL_ONLY, TRUE));
        params.add(new RequestParameter(TerminologyServiceConstants.ATTR_TERMINOLOGIES,
                selectedOntologies.stream()
                .map(wrapper->wrapper.getAcronym())
                .collect(Collectors.joining(","))));
        return params;
    }

}
