/**
* Copyright (C) 2013 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.ui.section.occurrence.dna;

import eu.etaxonomy.cdm.model.agent.AgentBase;
import eu.etaxonomy.cdm.model.agent.Institution;
import eu.etaxonomy.cdm.model.molecular.Amplification;
import eu.etaxonomy.cdm.model.molecular.Primer;
import eu.etaxonomy.cdm.model.term.DefinedTerm;
import eu.etaxonomy.cdm.model.term.TermType;
import eu.etaxonomy.taxeditor.store.StoreUtil;
import eu.etaxonomy.taxeditor.ui.combo.term.TermComboElement;
import eu.etaxonomy.taxeditor.ui.element.CdmFormFactory;
import eu.etaxonomy.taxeditor.ui.element.ICdmFormElement;
import eu.etaxonomy.taxeditor.ui.element.LayoutConstants;
import eu.etaxonomy.taxeditor.ui.element.TextWithLabelElement;
import eu.etaxonomy.taxeditor.ui.element.TimePeriodElement;
import eu.etaxonomy.taxeditor.ui.section.AbstractCdmDetailElement;
import eu.etaxonomy.taxeditor.ui.section.supplemental.AnnotationSection;
import eu.etaxonomy.taxeditor.ui.selection.EntitySelectionElement;

/**
 * @author pplitzner
 * @date 19.12.2013
 *
 */
public class AmplificationGeneralDetailElement extends AbstractCdmDetailElement<Amplification> {

    private TextWithLabelElement textDesignation;
    private TextWithLabelElement textNotes;
    private TermComboElement<DefinedTerm> comboMarker;
    private TimePeriodElement dateAmplification;
    private EntitySelectionElement<Institution> selectionInstitution;
    private EntitySelectionElement<AgentBase> selectionAmplificationStaff;
    private EntitySelectionElement<Primer> selectionPrimerFW;
    private EntitySelectionElement<Primer> selectionPrimerRW;

    /**
     * @param formFactory
     * @param formElement
     */
    public AmplificationGeneralDetailElement(CdmFormFactory formFactory, ICdmFormElement formElement) {
        super(formFactory, formElement);
    }

    /*
     * (non-Javadoc)
     *
     * @see
     * eu.etaxonomy.taxeditor.section.AbstractCdmDetailElement#createControls
     * (eu.etaxonomy.taxeditor.forms.ICdmFormElement, java.lang.Object, int)
     */
    @Override
    protected void createControls(ICdmFormElement formElement, Amplification entity, int style) {
        textDesignation = formFactory.createTextWithLabelElement(formElement, "Designation", entity.getLabelCache(), style);
        textDesignation.setEnabled(false);
        textDesignation.setBackground(getPersistentBackground());
        selectionInstitution = formFactory.createSelectionElement(Institution.class, //getConversationHolder(),
                formElement, "Institution", entity.getInstitution(), EntitySelectionElement.ALL, style);
        selectionAmplificationStaff = formFactory.createSelectionElement(AgentBase.class, //getConversationHolder(),
                formElement, "Amplification staff", entity.getActor(), EntitySelectionElement.ALL, style);
        dateAmplification = formFactory.createTimePeriodElement(formElement, "Date", entity.getTimeperiod(), style);
        comboMarker = formFactory.createDefinedTermComboElement(TermType.DnaMarker, formElement, "DNA Marker", entity.getDnaMarker(), style);
        selectionPrimerFW = formFactory.createSelectionElement(Primer.class,// getConversationHolder(),
                formElement, "Primer FW", entity.getForwardPrimer(), EntitySelectionElement.ALL, style);
        selectionPrimerRW = formFactory.createSelectionElement(Primer.class, //getConversationHolder(),
                formElement, "Primer RW", entity.getReversePrimer(), EntitySelectionElement.ALL, style);

        AnnotationSection annotationSection = formFactory.createAnnotationSection(getConversationHolder(), formElement, StoreUtil.getSectionStyle(AnnotationSection.class, entity.getClass().getCanonicalName()));
        annotationSection.setLayoutData(LayoutConstants.FILL_HORIZONTALLY(2, 1));
        annotationSection.setEntity(entity);
    }

    /*
     * (non-Javadoc)
     *
     * @see
     * eu.etaxonomy.taxeditor.section.AbstractCdmDetailElement#handleEvent(java
     * .lang.Object)
     */
    @Override
    public void handleEvent(Object eventSource) {
        if(eventSource==textNotes){
            getEntity().setDescription(textNotes.getText());
        }
        else if(eventSource==comboMarker){
            getEntity().setDnaMarker(comboMarker.getSelection());
        }
        else if(eventSource==dateAmplification){
            getEntity().setTimeperiod(dateAmplification.getTimePeriod());
        }
        else if(eventSource==selectionAmplificationStaff){
            getEntity().setActor(selectionAmplificationStaff.getEntity());
        }
        else if(eventSource==selectionInstitution){
            getEntity().setInstitution(selectionInstitution.getSelection());
        }
        else if(eventSource==selectionPrimerFW){
            getEntity().setForwardPrimer(selectionPrimerFW.getSelection());
        }
        else if(eventSource==selectionPrimerRW){
            getEntity().setReversePrimer(selectionPrimerRW.getSelection());
        }
        getEntity().updateCache();
        textDesignation.setText(getEntity().getLabelCache());
    }
}
