/**
 * Copyright (C) 2013 EDIT
 * European Distributed Institute of Taxonomy
 * http://www.e-taxonomy.eu
 *
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */
package eu.etaxonomy.taxeditor.ui.section.occurrence.dna;

import eu.etaxonomy.cdm.model.agent.AgentBase;
import eu.etaxonomy.cdm.model.agent.Institution;
import eu.etaxonomy.cdm.model.common.TimePeriod;
import eu.etaxonomy.cdm.model.molecular.DnaSample;
import eu.etaxonomy.cdm.model.occurrence.PreservationMethod;
import eu.etaxonomy.taxeditor.ui.element.CdmFormFactory;
import eu.etaxonomy.taxeditor.ui.element.ICdmFormElement;
import eu.etaxonomy.taxeditor.ui.element.NumberWithLabelElement;
import eu.etaxonomy.taxeditor.ui.element.TextWithLabelElement;
import eu.etaxonomy.taxeditor.ui.element.TimePeriodElement;
import eu.etaxonomy.taxeditor.ui.section.AbstractCdmDetailElement;
import eu.etaxonomy.taxeditor.ui.selection.EntitySelectionElement;

/**
 * @author pplitzner
 * @date 16.12.2013
 */
public class DnaSamplePreparationPreservationDetailElement extends AbstractCdmDetailElement<DnaSample> {

    private EntitySelectionElement<Institution> selectionInstitution;
    private EntitySelectionElement<AgentBase> selectionStaff;
    private TimePeriodElement datePreparationDate;
    private NumberWithLabelElement numberPreservationTemperature;
    private TextWithLabelElement txtMaterialOrMethods;

    public DnaSamplePreparationPreservationDetailElement(CdmFormFactory formFactory, ICdmFormElement formElement) {
        super(formFactory, formElement);
    }

    @Override
    protected void createControls(ICdmFormElement formElement, DnaSample entity, int style) {
        Institution institution = null;
        AgentBase<?> staff = null;
        TimePeriod preparationDate = null;

        if(entity.getDerivedFrom()!=null){
            institution = entity.getDerivedFrom().getInstitution();
            staff = entity.getDerivedFrom().getActor();
            preparationDate = entity.getDerivedFrom().getTimeperiod();
        }

        Double temperature = null;
        String materialOrMethodText = null;
        if(entity.getPreservation()!=null){
            temperature = entity.getPreservation().getTemperature();
        }
        if(entity.getDerivedFrom()!=null){
            materialOrMethodText = entity.getDerivedFrom().getDescription();
        }
        selectionInstitution = formFactory.createSelectionElement(Institution.class, //getConversationHolder(),
                formElement, "Institution", institution, EntitySelectionElement.ALL, style);
        selectionStaff = formFactory.createSelectionElement(AgentBase.class, //getConversationHolder(),
                formElement, "Staff", staff, EntitySelectionElement.ALL, style);
        datePreparationDate = formFactory.createTimePeriodElement(formElement, "Preparation date", preparationDate, style);
        numberPreservationTemperature = formFactory.createFloatTextWithLabelElement(formElement, "Preservation Temp. [C°]", temperature, style);
        txtMaterialOrMethods = formFactory.createTextWithLabelElement(formElement, "Materials && Methods", materialOrMethodText, style);

    }

    @Override
    public void handleEvent(Object eventSource) {
        DnaSample entity = getEntity();
        if(eventSource==datePreparationDate){
            if(entity.getDerivedFrom()!=null){
                entity.getDerivedFrom().setTimeperiod(datePreparationDate.getTimePeriod());
            }
        }
        else if(eventSource==selectionInstitution){
            if(entity.getDerivedFrom()!=null){
                entity.getDerivedFrom().setInstitution(selectionInstitution.getSelection());
            }
        }
        else if(eventSource==selectionStaff){
            if(entity.getDerivedFrom()!=null){
                entity.getDerivedFrom().setActor(selectionStaff.getSelection());
            }
        }
        else if(eventSource==numberPreservationTemperature && numberPreservationTemperature.getText()!=null){
            addPreservationMethod().setTemperature(numberPreservationTemperature.getDouble());
        }
        else if(eventSource==txtMaterialOrMethods && entity.getDerivedFrom()!=null){
            entity.getDerivedFrom().setDescription(txtMaterialOrMethods.getText());
        }
    }

    private PreservationMethod addPreservationMethod(){
        DnaSample entity = getEntity();
        PreservationMethod preservationMethod = entity.getPreservation();
        if(preservationMethod==null){
            preservationMethod = PreservationMethod.NewInstance();
            entity.setPreservation(preservationMethod);
        }
        return preservationMethod;
    }
}
