/**
* Copyright (C) 2014 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.ui.section.occurrence.dna;

import java.util.Collection;
import java.util.Collections;
import java.util.Comparator;

import eu.etaxonomy.cdm.api.conversation.ConversationHolder;
import eu.etaxonomy.cdm.api.service.IMediaService;
import eu.etaxonomy.cdm.model.media.Media;
import eu.etaxonomy.cdm.model.molecular.Sequence;
import eu.etaxonomy.taxeditor.store.CdmStore;
import eu.etaxonomy.taxeditor.ui.element.CdmFormFactory;
import eu.etaxonomy.taxeditor.ui.element.ICdmFormElement;
import eu.etaxonomy.taxeditor.ui.section.AbstractEntityCollectionSection;
import eu.etaxonomy.taxeditor.ui.section.DefaultCdmBaseComparator;

/**
 * @author pplitzner
 * @date 03.03.2014
 *
 */
public class SequenceContigFileCollectionDetailSection extends AbstractEntityCollectionSection<Sequence, Media> {

    /**
     * @param formFactory
     * @param conversation
     * @param parentElement
     * @param title
     * @param style
     */
    public SequenceContigFileCollectionDetailSection(CdmFormFactory formFactory, ConversationHolder conversation,
            ICdmFormElement parentElement, int style) {
        super(formFactory, conversation, parentElement, "Contig File", style);
    }



    /* (non-Javadoc)
     * @see eu.etaxonomy.taxeditor.ui.section.AbstractEntityCollectionSection#getCollection(java.lang.Object)
     */
    @Override
    public Collection<Media> getCollection(Sequence entity) {
        if(entity.getContigFile()!=null){
            return Collections.singleton(entity.getContigFile());
        }
        return Collections.EMPTY_LIST;
    }

    @Override
    public Comparator<Media> getComparator() {
        return new DefaultCdmBaseComparator<>();
    }

    /* (non-Javadoc)
     * @see eu.etaxonomy.taxeditor.ui.section.AbstractEntityCollectionSection#createNewElement()
     */
    @Override
    public Media createNewElement() {
        if(getEntity().getContigFile()==null){
            return Media.NewInstance();
        }
        return null;
    }

    /* (non-Javadoc)
     * @see eu.etaxonomy.taxeditor.ui.section.AbstractEntityCollectionSection#addElement(java.lang.Object)
     */
    @Override
    public void addElement(Media element) {
        if(getEntity().getContigFile()==null){
            getEntity().setContigFile(element);
			/*
			 * merge to avoid duplicate key exception
			 * the exception seems to be caused by the two links to the Media object:
			 *  - Sequence->contigFile (used to be not in the cache after saving)
			 *  - MediaRepresentationPart->media
			 *  Invoking merge() solves this problem
			 */
            CdmStore.getService(IMediaService.class).merge(element);
        }
    }

    /* (non-Javadoc)
     * @see eu.etaxonomy.taxeditor.ui.section.AbstractEntityCollectionSection#removeElement(java.lang.Object)
     */
    @Override
    public void removeElement(Media element) {
        getEntity().setContigFile(null);
    }

    /* (non-Javadoc)
     * @see eu.etaxonomy.taxeditor.ui.section.AbstractEntityCollectionSection#getEmptyString()
     */
    @Override
    public String getEmptyString() {
        return "No contig file yet";
    }

    /* (non-Javadoc)
     * @see eu.etaxonomy.taxeditor.ui.section.AbstractEntityCollectionSection#getTooltipString()
     */
    @Override
    protected String getTooltipString() {
        return "Add a contig file";
    }



    /**
     * {@inheritDoc}
     */
    @Override
    public Media addExisting() {
        return null;
    }



    /**
     * {@inheritDoc}
     */
    @Override
    public boolean allowAddExisting() {
        return false;
    }
}
