/**
* Copyright (C) 2013 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.ui.section.occurrence.dna;

import org.apache.log4j.Logger;

import eu.etaxonomy.cdm.api.facade.DerivedUnitFacade;
import eu.etaxonomy.cdm.api.service.IVocabularyService;
import eu.etaxonomy.cdm.model.agent.AgentBase;
import eu.etaxonomy.cdm.model.agent.Institution;
import eu.etaxonomy.cdm.model.common.TimePeriod;
import eu.etaxonomy.cdm.model.occurrence.Collection;
import eu.etaxonomy.cdm.model.occurrence.DerivedUnit;
import eu.etaxonomy.cdm.model.occurrence.PreservationMethod;
import eu.etaxonomy.cdm.model.term.DefinedTerm;
import eu.etaxonomy.cdm.model.term.TermType;
import eu.etaxonomy.cdm.model.term.TermVocabulary;
import eu.etaxonomy.cdm.model.term.VocabularyEnum;
import eu.etaxonomy.taxeditor.store.CdmStore;
import eu.etaxonomy.taxeditor.ui.combo.term.TermComboElement;
import eu.etaxonomy.taxeditor.ui.element.CdmFormFactory;
import eu.etaxonomy.taxeditor.ui.element.CheckboxElement;
import eu.etaxonomy.taxeditor.ui.element.ICdmFormElement;
import eu.etaxonomy.taxeditor.ui.element.TextWithLabelElement;
import eu.etaxonomy.taxeditor.ui.element.TimePeriodElement;
import eu.etaxonomy.taxeditor.ui.section.AbstractCdmDetailElement;
import eu.etaxonomy.taxeditor.ui.selection.EntitySelectionElement;

/**
 * @author pplitzner
 * @date 16.12.2013
 */
public class TissueSampleGeneralDetailElement extends AbstractCdmDetailElement<DerivedUnitFacade> {

    @SuppressWarnings("unused")
    static private final Logger logger = Logger.getLogger(TissueSampleGeneralDetailElement.class);

    private final TermVocabulary<?> plantKindOfUnitVocabulary = CdmStore.getService(IVocabularyService.class).find(VocabularyEnum.PlantKindOfUnit.getUuid());

    private TermComboElement<DefinedTerm> comboKindOfTissue;
    private EntitySelectionElement<Institution> selectionInstitution;
    private EntitySelectionElement<AgentBase> selectionStaff;
    private TimePeriodElement datePreparation;
    private TermComboElement<DefinedTerm> comboPreservationMedium;
    private EntitySelectionElement<Collection> selectionCollection;
    private TextWithLabelElement textAccessionNumber;
    private TextWithLabelElement textBarcode;
    private CheckboxElement checkIsPublish;

    public TissueSampleGeneralDetailElement(CdmFormFactory formFactory, ICdmFormElement formElement) {
        super(formFactory, formElement);
    }

    @Override
    protected void createControls(ICdmFormElement formElement, DerivedUnitFacade facade, int style) {
        DerivedUnit entity = getEntity().innerDerivedUnit();

        comboKindOfTissue = formFactory.createDefinedTermComboElement(plantKindOfUnitVocabulary, formElement, "Kind Of Tissue", entity.getKindOfUnit(), style);
        TimePeriod preparationDate = null;
        DefinedTerm preservationMedium = null;
        AgentBase<?> staff = null;
        Institution institution = null;
        if(entity.getDerivedFrom()!=null){
            staff = entity.getDerivedFrom().getActor();
            preparationDate = entity.getDerivedFrom().getTimeperiod();
            institution = entity.getDerivedFrom().getInstitution();
        }
        if(entity.getPreservation()!=null){
            preservationMedium = entity.getPreservation().getMedium();
        }
        selectionInstitution = formFactory.createSelectionElement(Institution.class,
                formElement, "Institution", institution, EntitySelectionElement.ALL, style);
        selectionStaff = formFactory.createSelectionElement(AgentBase.class,
                formElement, "Staff", staff, EntitySelectionElement.ALL,style);
        datePreparation = formFactory.createTimePeriodElement(formElement, "Preparation date", preparationDate, style);
        comboPreservationMedium = formFactory.createDefinedTermComboElement(TermType.MaterialOrMethod, formElement, "Preservation medium", preservationMedium, style);
        selectionCollection = formFactory.createSelectionElement(Collection.class,
                formElement, "Collection", entity.getCollection(), EntitySelectionElement.ALL, style);
        textAccessionNumber = formFactory.createTextWithLabelElement(formElement, "Accession Number", entity.getAccessionNumber(), style);
        textBarcode = formFactory.createTextWithLabelElement(formElement, "Barcode", entity.getBarcode(), style);
        checkIsPublish = formFactory.createCheckbox(formElement, "Publish", entity.isPublish(), style);

    }

    @Override
    public void handleEvent(Object eventSource) {
        DerivedUnit entity = getEntity().innerDerivedUnit();
        if(eventSource==comboKindOfTissue) {
            entity.setKindOfUnit(comboKindOfTissue.getSelection());
        }
        else if(eventSource==comboPreservationMedium) {
            addPreservationMethod().setMedium(comboPreservationMedium.getSelection());
        }
        else if(eventSource==selectionCollection){
            getEntity().setCollection(selectionCollection.getSelection());
        }
        else if(eventSource==textAccessionNumber){
            getEntity().setAccessionNumber(textAccessionNumber.getText());
        }
        else if(eventSource==textBarcode){
            getEntity().setBarcode(textBarcode.getText());
        }
        else if(eventSource==datePreparation){
            if(entity.getDerivedFrom()!=null){
                entity.getDerivedFrom().setTimeperiod(datePreparation.getTimePeriod());
            }
        }
        else if(eventSource==selectionInstitution){
            if(entity.getDerivedFrom()!=null){
                entity.getDerivedFrom().setInstitution(selectionInstitution.getSelection());
            }
        }
        else if(eventSource==selectionStaff){
            if(entity.getDerivedFrom()!=null){
                entity.getDerivedFrom().setActor(selectionStaff.getSelection());
            }
        }
        else if(eventSource==checkIsPublish){
            entity.setPublish(checkIsPublish.getSelection());
        }
    }

    private PreservationMethod addPreservationMethod(){
        DerivedUnit entity = getEntity().innerDerivedUnit();
        PreservationMethod preservationMethod = null;
        preservationMethod = entity.getPreservation();
        if(entity.getPreservation()==null){
            preservationMethod = PreservationMethod.NewInstance();
            entity.setPreservation(preservationMethod);
        }
        return preservationMethod;
    }
}